/*-========================================================================-_
 |                    - NATURAL USER INTERFACE: AUDIO -                     |
 |        Copyright (c) Microsoft Corporation.  All rights reserved.        |
 |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
 |PROJECT: NuiAudio                     MODEL:   Unmanaged Kernel/User-mode |
 |VERSION: 1.0                          EXCEPT:  No Exceptions              |
 |CLASS:   N / A                        MINREQ:  Xbox360                    |
 |BASE:    N / A                        DIALECT: MSC++ 14.00                |
 |>------------------------------------------------------------------------<|
 | DUTY: Microphone array pipeline interface                                |
 ^~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~^
  NOTES:
    1.  Use the debug build to enable parameter validation via asserts.

    2.  Definition of terms:
            Frame: A block of samples, one per channel,
                   to be played simultaneously.
                   E.g. a mono stream has one sample per frame.

    3.  To minimize glitches, the realtime audio processing thread must
        not block.  Methods called by the realtime thread are commented
        as non-blocking and therefore should not use blocking synchronization,
        allocate memory, access the disk, etc.

    4.  NuiAudio API functions may block and cannot be called from
        NuiAudio callbacks, except NuiAudioSetMicArrayBeamAngle.            */

#pragma once
//--------------<D-E-F-I-N-I-T-I-O-N-S>-------------------------------------//
#include <windef.h> // general windows types

// size of handle to NuiAudio singleton in bytes
#define NUIAUDIO_HANDLE_BYTESIZE 4

// maximum value of NUIAUDIO_RESULTS.Duration
#define NUIAUDIO_MAX_DURATION 256

// Range of micarray beam angle, in radians.
// Angle is measured clockwise from the micarray's point of view,
// in the plane parallel to the ground.
#define NUIAUDIO_MICARRAY_BEAM_ANGLE_MIN -0.873f // (-50 degrees)
#define NUIAUDIO_MICARRAY_BEAM_ANGLE_MAX  0.873f // ( 50 degrees)

// Specify to allow micarray beam angle to be computed automatically,
// focusing on the loudest sound detected in the playspace.
#define NUIAUDIO_MICARRAY_BEAM_ANGLE_AUTO FLT_MAX

// Specify the speech or chat pipeline.
#define NUIAUDIO_SPEECH_PIPELINE 0x00000001
#define NUIAUDIO_CHAT_PIPELINE   0x00000002

// default Xbox hardware thread to run the micarray pipeline
#define NUIAUDIO_DEFAULT_PROCESSOR 5


//--------------<D-A-T-A---T-Y-P-E-S>---------------------------------------//
#pragma pack(push, 1) // set packing alignment to ensure consistency across arbitrary build environments
#ifdef __cplusplus
extern "C" {
#endif


// handle to NuiAudio singleton
typedef BYTE NUIAUDIO_HANDLE[NUIAUDIO_HANDLE_BYTESIZE];


// NuiAudio processing results:
// Receives output of a micarray pipeline processing pass.
typedef struct _NUIAUDIO_RESULTS
{
    INT16* __restrict pOutputMic; // [out] NuiAudio output buffer (Xbox), big-endian 16kHz 16-bit integer mono, 16-byte aligned
    UINT32 Duration;              // [out] size of buffer in frames (currently always NUIAUDIO_MAX_DURATION)
    FLOAT BeamAngle;              // [out] most recent micarray beam angle in radians, within [NUIAUDIO_MICARRAY_BEAM_ANGLE_MIN, NUIAUDIO_MICARRAY_BEAM_ANGLE_MAX]
    FLOAT Confidence;             // [out] most recent micarray beam angle confidence probability, within [0.0f, 1.0f]
} NUIAUDIO_RESULTS;


// Callback signatures:
typedef void (*PNUIAUDIO_CALLBACK)       (NUIAUDIO_RESULTS* pNuiAudioResults); // this method should not block as it is called from the realtime processing thread
typedef void (*PNUIAUDIO_ERROR_CALLBACK) (HRESULT hr); // called on a critical system error such as the camera being unplugged



//--------------<F-U-N-C-T-I-O-N-S>-----------------------------------------//

  ////
  // DESCRIPTION:
  //  Initializes NuiAudio, starts streaming data through the micarray pipeline.
  //  Use the returned handle with subsequent NuiAudio calls.
  //
  // REMARKS:
  //  NuiAudio must be initialized before callbacks can be registered.
  //  If already initialized, calling this method will have no effect until
  //  NuiAudio is released, other than registering the error callback.
  //
  // PARAMETERS:
  //  uHardwareThreadRequested - [in]      Xbox hardware thread to run the micarray pipeline, must be within [0, 5]. Default: NUIAUDIO_DEFAULT_PROCESSOR
  //                                       If already initialized, this parameter has no effect.
  //  pfnErrorCallback         - [in]      Pointer to function called on a critical system error which requires
  //                                       NuiAudio to be released and reinitialized, must not be NULL.
  //  Flags                    - [in]      Specifies which pipeline(s) to initialize. Can be a combination of the following:
  //                                       NUIAUDIO_SPEECH_PIPELINE - speech pipeline
  //                                       NUIAUDIO_CHAT_PIPELINE - chat pipeline
  //  Handle                   - [out]     Handle to NuiAudio singleton.
  //  pHardwareThreadUsed      - [out opt] Xbox hardware thread running the micarray pipeline, specify NULL if not needed.
  //
  // RETURN VALUE:
  //  COM error code
  ////
HRESULT NuiAudioCreate (
    __in UINT32 uHardwareThreadRequested,
    __in const PNUIAUDIO_ERROR_CALLBACK pfnErrorCallback,
    __in DWORD Flags,
    __out NUIAUDIO_HANDLE Handle,
    __out_opt UINT32* pHardwareThreadUsed = NULL
);



  ////
  // DESCRIPTION:
  //  Releases handle to NuiAudio, unregistering the error callback.
  //  When the last outstanding error callback is unregistered,
  //  the micarray pipeline stops streaming, deallocates all its internal resources,
  //  and reverts to the uninitialized state.
  //
  // REMARKS:
  //  The caller is responsible for ensuring all processing callbacks for the
  //  given handle are unregistered before calling this function.
  //
  //  The caller can assume no more calls will occur to unregistered
  //  callbacks after this function returns.
  //
  //  If already uninitialized, calling this method will have no effect.
  //
  // PARAMETERS:
  //  Handle - [in] Handle to NuiAudio singleton.
  //
  // RETURN VALUE:
  //  Asserts on error.
  ////
void NuiAudioRelease (
    __in const NUIAUDIO_HANDLE Handle
);



  ////
  // DESCRIPTION:
  //  Registers processing callbacks with NuiAudio.
  //
  // PARAMETERS:
  //  Handle                - [in] Handle to NuiAudio singleton.
  //  Flags                 - [in] Specifies which pipeline to register a callback for:
  //                               NUIAUDIO_SPEECH_PIPELINE - register callback for the speech pipeline
  //                               NUIAUDIO_CHAT_PIPELINE - register callback for the chat pipeline
  //  pfnProcessingCallback - [in] pointer to function to receive NuiAudio results (speech pipeline), must not be NULL
  //
  // RETURN VALUE:
  //  Asserts on error.
  ////
void NuiAudioRegisterCallbacks (
    __in const NUIAUDIO_HANDLE Handle,
    __in DWORD Flags,
    __in const PNUIAUDIO_CALLBACK pfnProcessingCallback
);



  ////
  // DESCRIPTION:
  //  Unregisters processing callbacks with NuiAudio.
  //
  // REMARKS:
  //  The caller can assume no more callbacks will occur to unregistered
  //  callbacks after this function returns.
  //
  // PARAMETERS:
  //  Handle                - [in] Handle to NuiAudio singleton.
  //  pfnProcessingCallback - [in] pointer to function to receive NuiAudio results (speech pipeline), must not be NULL
  //
  // RETURN VALUE:
  //  Asserts on error.
  ////
void NuiAudioUnregisterCallbacks (
    __in const NUIAUDIO_HANDLE Handle,
    __in const PNUIAUDIO_CALLBACK pfnProcessingCallback
);



  ////
  // DESCRIPTION:
  //  Overrides the micarray beam angle for the speech pipeline.
  //  NUIAUDIO_MICARRAY_BEAM_ANGLE_AUTO is the default setting.
  //
  // REMARKS:
  //  This function may be called from a NuiAudio callback.
  //  Note only the most recent call to this function has effect,
  //  as NuiAudio is a singleton.
  //
  // PARAMETERS:
  //  Handle    - [in] Handle to NuiAudio singleton.
  //  Flags     - [in] Specifies which pipeline to lock or release.
  //                   Currently only NUIAUDIO_SPEECH_PIPELINE is supported
  //  BeamAngle - [in] Micarray beam angle in radians.
  //                   An angle of zero is straight in front of the camera,
  //                   positive values are to the left of the camera (from the camera's perspective)
  //                   and negative values are to the right of the camera.
  //
  //                   Must be within [NUIAUDIO_MICARRAY_BEAM_ANGLE_MIN, NUIAUDIO_MICARRAY_BEAM_ANGLE_MAX].
  //                   Or use NUIAUDIO_MICARRAY_BEAM_ANGLE_AUTO to allow the
  //                   micarray beam angle to be computed automatically.
  //
  // RETURN VALUE:
  //  Asserts on error.
  ////
void NuiAudioSetMicArrayBeamAngle (
    __in const NUIAUDIO_HANDLE Handle,
    __in DWORD Flags,
    __in FLOAT BeamAngle
);


  ////
  // DESCRIPTION:
  //  Checks the validity & presence of calibration data.
  //
  // PARAMETERS:
  //  None
  //
  // RETURN VALUE:
  //  TRUE if calibration data is present on the system and is valid.
  //  FALSE if calibration data is not present or is invalid.
  ////
BOOL NuiAudioIsCalibrationValid(void);



#ifdef __cplusplus
} //close extern "C"
#endif
#pragma pack(pop) // revert packing alignment
//---------------------------------<-EOF->----------------------------------//

