/************************************************************************
*                                                                       *
*   NuiFitness.h                                                        *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once



#ifndef NUIAPI
#error "You must include nuiapi.h rather than including NuiFitness.h directly"
#endif


#ifdef __cplusplus
extern "C" {
#endif

//
// Maximum number of players that we can track for fitness calculation.
//
#define NUI_FITNESS_MAX_TRACKED_COUNT 8

//
// MET value range.
//
#define NUI_FITNESS_MIN_MET          0.5f
#define NUI_FITNESS_MAX_MET          20.0f

//
// Auto mode means, we automatic calculate and track calorie value based on 
// the input skeleton; manual mode means, titles need to calculate their own 
// MET value.
//
typedef enum _NUI_FITNESS_TRACKING_MODE { 
    NUI_FITNESS_TRACKING_AUTO = 0,
    NUI_FITNESS_TRACKING_MANUAL
} NUI_FITNESS_TRACKING_MODE;

//
// Gender used in BodyProfile.
//
typedef enum _NUI_GENDER_VALUE { 
    NUI_GENDER_MALE, 
    NUI_GENDER_FEMALE, 
    NUI_GENDER_UNKNOWN 
} NUI_GENDER_VALUE; 

//
// Our HWAG (Height, Weight, Age, and Gender) structure, which is
// needed to convert MET value into Calorie burned.
//
typedef struct _NUI_FITNESS_BODY_PROFILE_RECORD {
    FLOAT Height;            // Height in meter. 
    FLOAT Weight;            // Weight in kilogram. 
    USHORT YearOfBirth;      // Year of birth must be greater than 1900. 
    NUI_GENDER_VALUE Gender; // Male, Female or Unknown.
} NUI_FITNESS_BODY_PROFILE_RECORD;

//
// The returned record during calorie calculation.
//
typedef struct _NUI_FITNESS_DATA {
    DWORD DurationInMS; // Duration in Milliseconds.
    FLOAT AverageMET;   // Average MET value during the above duration.
    DWORD Joules;       // The accumulated energy burned for the above duration in Joules.
} NUI_FITNESS_DATA;


//
// NuiFitnessStartTracking
//
// DESCRIPTION:
//  Start Fitness tracking for a particular skeleton.
//
// REMARKS:
//  Titles are responsible in associating TrackingID to UserIndex for storage and accurate HWAG values. 
//  Also, Titles are responsible in making sure that manual trakcingID doesn't conflict with automatic 
//  trackingID.
//
// PARAMETERS:
//  TrackingMode - [in]      NUI_FITNESS_TRACKING_AUTO or NUI_FITNESS_TRACKING_MANUAL. For NUI_FITNESS_TRACKING_AUTO
//                              TrackingID must be a valid skeleton trackingID.
//  TrackingID   - [in]      Specify the TrackingID for the skeleton that you want to track its fitness in 
//                              NUI_FITNESS_TRACKING_AUTO mode. Or specify a unique value to identify a particular
//                              player in manual mode (NUI_FITNESS_TRACKING_MANUAL).
//  UserIndex    - [in]      Specify the UserIndex of the logged in user in which you want to store the tracked
//                              Fitness data into his or her profile. Use XUSER_INDEX_NONE for in memory only
//                              Fitness tracking.
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT and not XUSER_INDEX_NONE).
//      TrackingID is invalid (0).
//  E_NUI_LOST_TRACKING:
//      In automatic tracking mode, the provided tracking ID doesn't exist.
//  E_NUI_TRACKING_ID_IN_USE:
//      We are already tracking the provided TrackingID.
//  E_NUI_TRACKED_PLAYER_LIMIT_EXCEEDED:
//      We are already tracking 8 players.
//
NUIAPI
HRESULT 
NuiFitnessStartTracking(_In_ NUI_FITNESS_TRACKING_MODE TrackingMode,
                        _In_ DWORD TrackingID,
                        _In_ DWORD UserIndex);

//
// NuiFitnessPauseTracking
//
// DESCRIPTION:
//  Pause current tracking session.
//
// REMARKS:
//  This function is valid only in automatic tracking mode.
//
// PARAMETERS:
//  TrackingID   - [in]      Specify the TrackingID for the skeleton that you want to pause tracking its fitness in 
//                              NUI_FITNESS_TRACKING_AUTO mode.
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//  E_UNEXPECTED:
//      TrackingID specified is used in manual mode.
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT 
NuiFitnessPauseTracking(_In_ DWORD TrackingID);

//
// NuiFitnessResumeTracking
//
// DESCRIPTION:
//  Resume currently paused tracking session.
//
// REMARKS:
//  This function is valid only in automatic tracking mode.
//
// PARAMETERS:
//  OldTrackingID   - [in]   Specify the old TrackingID for the skeleton that was used before pausing fitness tracking.
//  NewTrackingID   - [in]   Specify the new TrackingID for the skeleton that you want to resume tracking its fitness in 
//                              NUI_FITNESS_TRACKING_AUTO mode. If TrackingID didn't change, then specify the same TrackingID
//                              for both OldTrackingID and NewTrackingID.
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      OldTrackingID is invalid (0).
//      NewTrackingID is invalid (0).
//   E_UNEXPECTED:
//      TrackingID specified is used in manual mode.
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT 
NuiFitnessResumeTracking(_In_ DWORD OldTrackingID, 
                         _In_ DWORD NewTrackingID);

//
// NuiFitnessStopTracking
//
// DESCRIPTION:
//  Stop currently active fitness session.
//
// REMARKS:
//  If a valid UserIndex was specified in NuiFitnessStartTracking, then this function will store
//  the calculated fitness data to the profile of the specified UserIndex.
//
// PARAMETERS:
//  TrackingID      - [in]      Specify the TrackingID for the tracked player that you want to stop his or her fitness tracking.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT
NuiFitnessStopTracking(_In_ DWORD TrackingID);

//
// NuiFitnessRecordMETValue
//
// DESCRIPTION:
//  For manual tracking mode, set average MET value across the specified duration.
//
// REMARKS:
//  This function is only valid in manual mode.
//
// PARAMETERS:
//  TrackingID      - [in]    Specify the TrackingID for the tracked player that you want to set his or her MET value.
//  MET             - [in]    Title calculated average MET value for the duration DurationInMS.
//  DurationInMS    - [in]    The duration in which the above MET value is calculated in milliseconds.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//      The specified MET values is less than .5 or higher than 20.
//  E_UNEXPECTED:
//      TrackingID specified is used in automactic tracking mode..
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT 
NuiFitnessRecordMETValue(_In_ DWORD TrackingID,
                         _In_ FLOAT MET,
                         _In_ DWORD DurationInMS);


//
// NuiFitnessRecordCalorieValue
//
// DESCRIPTION:
//  For manual tracking mode, set current Calorie value for the specified duration.
//
// REMARKS:
//  This function is only valid in manual mode.
//
// PARAMETERS:
//  TrackingID      - [in]    Specify the TrackingID for the tracked player that you want to set his or her MET value.
//  CalorieInJoules - [in]    Title calculated calorie value for the duration DurationInMS.
//  DurationInMS    - [in]    The duration in which the above MET value is calculated in milliseconds.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//  E_UNEXPECTED:
//      TrackingID specified is used in automactic tracking mode..
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT
NuiFitnessRecordCalorieValue(_In_ DWORD TrackingID,
                             _In_ FLOAT CalorieInJoules,
                             _In_ DWORD DurationInMS);

//
// NuiFitnessGetCurrentFitnessData
//
// DESCRIPTION:
//  Use this function to get calculated fitness data in real-time during a fitness tracking
//  session.
//
// REMARKS:
//  Must be called between NuiFitnessStartTracking and NuiFitnessStopTracking only.
//
// PARAMETERS:
//  TrackingID      - [in]    Specify the TrackingID for the tracked player that you want to get his or her 
//                              fitness data.
//  pFitnessData    - [out]   The currently calculated fitness data for the active session.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//      pFitnessData is nullptr or NULL or 0.
//  E_NUI_NOT_FOUND:
//      The provided TrackingID is not currently being tracked.
//
NUIAPI
HRESULT
NuiFitnessGetCurrentFitnessData(_In_ DWORD TrackingID,
                                _Out_ NUI_FITNESS_DATA *pFitnessData);

//
// NuiFitnessSetBodyProfileRecord
//
// DESCRIPTION:
//  Use this function to set height, weight, age and gender for this UserIndex.
//
// REMARKS:
//  UserIndex must be valid and HWAG values must be within the documented range.
//
// PARAMETERS:
//  UserIndex         - [in]   Specify the UserIndex for the user that you want to update his or her 
//                                 HWAG values.
//  BodyProfileRecord - [in]   BodyProfileRecord structure than contains UserIndex HWAG values.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_ACCESSDENIED
//      Privilege bit is not set.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//      HWAG values are out of the documented range.
NUIAPI
HRESULT
NuiFitnessSetBodyProfileRecord(_In_ DWORD UserIndex,
                               _In_ NUI_FITNESS_BODY_PROFILE_RECORD BodyProfileRecord);

//
// NuiFitnessGetBodyProfileRecord
//
// DESCRIPTION:
//  Use this function to get current height, weight, age and gender for this UserIndex.
//
// REMARKS:
//  UserIndex must be a valid index.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to get his or her 
//                                  HWAG values.
//  pBodyProfileRecord - [in]   pBodyProfileRecord structure, title allocated, that will contain this UserIndex 
//                                  HWAG values.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_ACCESSDENIED
//      Privilege bit is not set.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//      pBodyProfileRecord is nullptr or NULL or 0.
//  E_NUI_NOT_FOUND:
//      Not record found.
NUIAPI
HRESULT
NuiFitnessGetBodyProfileRecord(_In_ DWORD UserIndex,
                               _Out_ NUI_FITNESS_BODY_PROFILE_RECORD *pBodyProfileRecord);

//
// XShowNuiFitnessBodyProfileUI
//
// DESCRIPTION:
//  Pop up a NUI UI for the player to enter his or her HWAG 
//  (Height, Weight, Age, and Gender) values.
//
// REMARKS:
//  UserIndex and TrackingID must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to update his or her 
//                                  HWAG values.
//  TrackingID         - [in]   Specify the TrackingID of the skeleton that should be in control of the UI.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
NUIAPI
HRESULT
XShowNuiFitnessBodyProfileUI(_In_ DWORD TrackingID, _In_ DWORD UserIndex);

//
// XShowFitnessBodyProfileUI
//
// DESCRIPTION:
//  Pop up a controller UI for the player to enter his or her HWAG 
//  (Height, Weight, Age, and Gender) values.
//
// REMARKS:
//  UserIndex must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to update his or her 
//                                  HWAG values.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
NUIAPI
HRESULT
XShowFitnessBodyProfileUI(_In_ DWORD UserIndex);

//
// Granularity of the Query.
//
typedef enum _XFITNESS_QUERY_ROLLUP_VALUES
{
    XFITNESS_QUERY_ROLLUP_PER_RECORD = 1,   // Raw data, not valid for online search.
    XFITNESS_QUERY_ROLLUP_PER_DAY    = 2,   // Accumulate data per day
    XFITNESS_QUERY_ROLLUP_PER_MONTH  = 3,   // Accumulate data per month
    XFITNESS_QUERY_ROLLUP_PER_YEAR   = 4,   // Accumulate data per year   
    XFITNESS_QUERY_ROLLUP_TOTAL      = 5    // Accumulate all data
} XFITNESS_QUERY_ROLLUP_VALUES;

//
// Date range for the Query, start and end date can be empty. In this case,
// we will do as follow:
//     1. If StartDate is Empty, then set StartDate to the date of the first 
//        record stored locally.
//     2. If EndDate is Empty, then set EndDate to current date.
// UserIndexToQuery and OnlineUserIDToQuery are exlusive, UserIndexToQuery to 
// search against locally logged-in user and OnlineUserIDToQuery to search against 
//  a specific user in Xbox live.
//
typedef struct _XFITNESS_QUERY_PARAMETERS
{
    DWORD UserIndexToQuery;
    XUID OnlineUserIDToQuery;
    SYSTEMTIME StartDate; // Start date of the search, can be empty.
    SYSTEMTIME EndDate;   // End date of the search, can be empty.
    XFITNESS_QUERY_ROLLUP_VALUES QueryRollupValues;
} XFITNESS_QUERY_PARAMETERS;

//
// Fitness record returned from Query API.
//
typedef struct _XFITNESS_RECORD
{
    SYSTEMTIME TimeStamp;    // TimeStamp for the start of the record
    DWORD      Duration;     // Duration in second
    DWORD      TitleID;      // TitleID for this record.
    FLOAT      AverageMET;   // AverageMET for the duration of this record
    DWORD      Joules;       // Energy burned in joule.
    USHORT     Reserved;     // Reserved for future use.
} XFITNESS_RECORD;

//
// Structure to hold the accumulated fitness data.
//
typedef struct _NUI_FITNESS_TITLE_SUMMARY {
    SYSTEMTIME LatestTimeStamp; // 
    DWORD   TotalDuration;      // In seconds.
    DWORD   TotalJoules;        // Total energy burned, in Joule, for the above duration.
    FLOAT   AverageMET;         // Weighted average MET for the above duration
} NUI_FITNESS_TITLE_SUMMARY;

//
// XFitnessQueryCreateEnumerator
//
// DESCRIPTION:
//  Create an enumerator handle based on XFITNESS_QUERY_PARAMETERS.
//
// REMARKS:
//  UserIndex and TrackingID must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]      Specify the UserIndex for the user that you want to query his or her 
//                                     fitness records.
//  pQueryParams       - [in]      Specify the search criteria for the query.
//  cItems             - [in]      Number of content items to be retrieved.
//  pcbBuffer          - [out opt] Size, in bytes, of the buffer needed to contain the enumeration results.
//  phEnum             - [out]     On successful return, a handle that can be passed to XEnumerate to iterate 
//                                     through Fitness Records. This handle should be closed using XCloseHandle 
//                                     after enumeration is complete.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//      pQueryParams is nullptr or NULL or 0.
//      phEnum is nullptr or NULL or 0.
//      Date range is not a valid range.
NUIAPI
HRESULT
XFitnessQueryCreateEnumerator(_In_ DWORD UserIndex,
                              _In_ CONST XFITNESS_QUERY_PARAMETERS* pQueryParams,
                              _In_ DWORD cItems,
                              _Out_opt_ PDWORD pcbBuffer,
                              _Out_ PHANDLE phEnum);

//
// NuiFitnessGetCurrentTitleSummary
//
// DESCRIPTION:
//  Get current title accumulated burned energy, avergae MET, and total duration.
//
// REMARKS:
//  UserIndex and TrackingID must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]      Specify the UserIndex for the user that you want to get his or her 
//                                     title summary.
//  pSummary           - [out]     Title allocated structure that will be filled with title summary data.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//      pSummary is nullptr or NULL or 0.
//  E_NUI_NOT_FOUND:
//      No record found.
NUIAPI
HRESULT
NuiFitnessGetCurrentTitleSummary(_In_ DWORD UserIndex,
                                 _Out_ NUI_FITNESS_TITLE_SUMMARY* pSummary);
//
// NuiFitnessGetOverallSummary
//
// DESCRIPTION:
//  Get total accumulated burned energy, avergae MET, and total duration across all titles.
//
// REMARKS:
//  UserIndex and TrackingID must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]      Specify the UserIndex for the user that you want to get his or her 
//                                     fitness summary.
//  pSummary           - [out]     Title allocated structure that will be filled with fitness summary data.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//      pSummary is nullptr or NULL or 0.
//  E_NUI_NOT_FOUND:
//      No record found.
NUIAPI
HRESULT 
NuiFitnessGetOverallSummary(_In_ DWORD UserIndex,
                            _Out_ NUI_FITNESS_TITLE_SUMMARY* pSummary);

//
/////////////////// Privacy settings APIs /////////////////////////////////////
//

//
// Supported privacy settings.
//
typedef enum _NUI_FITNESS_PRIVACY_SETTINGS {
    NUI_FITNESS_PRIVACY_SETTINGS_UNKNOWN = 0,
    NUI_FITNESS_PRIVACY_SETTINGS_NOTSET,
    NUI_FITNESS_PRIVACY_SETTINGS_LOCALONLY,
    NUI_FITNESS_PRIVACY_SETTINGS_BLOCKED,
    NUI_FITNESS_PRIVACY_SETTINGS_FRIENDS,
    NUI_FITNESS_PRIVACY_SETTINGS_EVERYONE,
} NUI_FITNESS_PRIVACY_SETTINGS;

//
// NuiFitnessGetPrivacySettings
//
// DESCRIPTION:
//  Return current privacy settings.
//
// REMARKS:
//  UserIndex must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to get his or her 
//                              privacy settings.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
//  E_NUI_NOT_LIVE_USER:
//      This user doesn't have a live account.
//  E_NUI_LIVE_USER_NOT_LOGGED_IN:
//      This user is not logged in.
HRESULT NuiFitnessGetPrivacySettings(_In_ DWORD UserIndex,
                                     _Out_ NUI_FITNESS_PRIVACY_SETTINGS* pPrivacySettings);

//
// XShowNuiFitnessPrivacySettingsUI
//
// DESCRIPTION:
//  Pop up a NUI UI for the player to change his or her privacy settings.
//
// REMARKS:
//  UserIndex and TrackingID must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to update his or her 
//                              privacy settings.
//  TrackingID         - [in]   Specify the TrackingID of the skeleton that should be in control of the UI.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      TrackingID is invalid (0).
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
NUIAPI
HRESULT
XShowNuiFitnessPrivacySettingsUI(_In_ DWORD TrackingID, _In_ DWORD UserIndex);

//
// XShowFitnessPrivacySettingsUI
//
// DESCRIPTION:
//  Pop up a controller UI for the player to change his or her privacy settings.
//
// REMARKS:
//  UserIndex must be valid.
//
// PARAMETERS:
//  UserIndex          - [in]   Specify the UserIndex for the user that you want to update his or her 
//                              privacy settings.
//
// RETURN VALUE:
//  E_NUI_FEATURE_NOT_INITIALIZED:
//      NuiInitialize didn't get called.
//      NUI_INITIALIZE_FLAG_USES_FITNESS wasn't specified in NuiInitialize.
//  E_INVALIDARG:
//      UserIndex is Invalid (greater than XUSER_MAX_COUNT).
NUIAPI
HRESULT
XShowFitnessPrivacySettingsUI(_In_ DWORD UserIndex);

#ifdef __cplusplus
} //close extern "C"
#endif
