////////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) Microsoft Corporation. All rights reserved.
//
// AsfWriterApi.h
//
// Public APIs (XDK) for Asf Writer.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(__cplusplus)
#error C++ compiler required.
#endif

#pragma once


/////////////////////////////////////////////////////////////////////////////
//
//  Asf Writer definitions
//
/////////////////////////////////////////////////////////////////////////////

typedef struct tAsfWriterBitmapInfoHeader
{
    DWORD      biSize;
    LONG       biWidth;
    LONG       biHeight;
    WORD       biPlanes;
    WORD       biBitCount;
    DWORD      biCompression;
    DWORD      biSizeImage;
    LONG       biXPelsPerMeter;
    LONG       biYPelsPerMeter;
    DWORD      biClrUsed;
    DWORD      biClrImportant;
} AsfWriterBitmapInfoHeader;

//
// For uncompressed RGB format, only RGB32 is supported.
#ifndef BI_RGB
#define BI_RGB        0L
#endif


// FOURCC code specified as a DWORD always in little-endian order.
#ifndef MAKEFOURCC
#define MAKEFOURCC(ch0, ch1, ch2, ch3)                              \
            ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) |       \
            ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 ))
#endif // defined(MAKEFOURCC)

// FOURCC code for VC1
#define FOURCC_WVC1     MAKEFOURCC('W','V','C','1')

// Standard type: WAVEFORMATEX
#ifndef _WAVEFORMATEX_
#define _WAVEFORMATEX_
#pragma pack(push, 1) // WAVEFORMATEX is packed to 1-byte boundaries
typedef struct tWAVEFORMATEX
{
    WORD wFormatTag;        // Integer identifier of the format
    WORD nChannels;         // Number of audio channels
    DWORD nSamplesPerSec;   // Audio sample rate
    DWORD nAvgBytesPerSec;  // Bytes per second (possibly approximate)
    WORD nBlockAlign;       // Size in bytes of a sample block (all channels)
    WORD wBitsPerSample;    // Size in bits of a single per-channel sample
    WORD cbSize;            // Bytes of extra data appended to this struct
} WAVEFORMATEX;
#pragma pack(pop)
#endif // _WAVEFORMATEX_

//
// WAVE Format Tags for input uncompressed audio stream
//

#ifndef WAVE_FORMAT_PCM
#define WAVE_FORMAT_PCM                 0x0001  // PCM format
#endif

#ifndef WAVE_FORMAT_IEEE_FLOAT
#define WAVE_FORMAT_IEEE_FLOAT          0x0003  // 32-bit floating-point
#endif

#ifndef WAVE_FORMAT_EXTENSIBLE
#define WAVE_FORMAT_EXTENSIBLE          0xFFFE  // All WAVEFORMATEXTENSIBLE formats
#endif

//
// WAVE Format Tags for output compressed audio stream
//

#ifndef WAVE_FORMAT_WMAUDIO2
#define WAVE_FORMAT_WMAUDIO2            0x0161  // WMA Standard
#endif


/////////////////////////////////////////////////////////////////////////////
//
// IAsfWriterVideoStream interface
//
// This interface is called by the encoder to pull the next video frame
// to encode.
//
/////////////////////////////////////////////////////////////////////////////
interface IAsfWriterVideoStream 
{
    // Return S_OK with zero length when at the end of stream.
    virtual HRESULT GetNextFrame( __out_ecount(*puLength) BYTE *pFrame,
                                 __inout UINT* puLength,
                                 __out LONGLONG* phnsSampleTime ) = 0;
};

/////////////////////////////////////////////////////////////////////////////
//
// IAsfWriterAudioStream interface
//
// This interface is called by the encoder to pull the set of audio samples
// to encode.
//
/////////////////////////////////////////////////////////////////////////////
interface IAsfWriterAudioStream
{
    // Return S_OK with zero length when at the end of stream.
    virtual HRESULT GetNextSample( __out_ecount(*puLength) BYTE *pSample,
                                  __inout UINT* puLength) = 0;
};

// Video encoding mode
#define ASFWRITER_VIDEO_ONEPASS_CBR                 0

// Audio encoding mode
#define ASFWRITER_AUDIO_ONEPASS_CBR                 0

// Default key frame distance 4000 milliseconds
#define ASFWRITER_VIDEO_KEYFRAME_DISTANCE_DEFAULT   4000

// Default buffer delay window 3000 milliseconds
#define ASFWRITER_VIDEO_BUFFER_DELAY_DEFAULT        3000

// Default quality 90 per cent
#define ASFWRITER_VIDEO_QUALITY_PERCENT_DEFAULT     90

typedef struct tAsfWriterVideoEncInfo
{
    WORD    cbSize;             // Size of the structure including extended fields if any.
    WORD    wMode;              // Video encoding mode.
    DWORD   dwFormat;           // FOURCC code as a DWORD in little-endian order.
    DWORD   dwBitRate;          // Average bitrate in Kbps.
    DWORD   dwMaxBitRate;       // Maximum bitrate in Kbps.
    DWORD   dwQuality;          // Percentage(1-100) for quality based encoding.
    DWORD   dwKeyFrameDistance; // Key frame distance in millisecond, default 4000.
    DWORD   dwBufferDelay;      // Buffer delay in millisecond, default 3000.
    DOUBLE  frameRate;          // Average frames per second.
} AsfWriterVideoEncInfo;


typedef struct tAsfWriterAudioEncInfo
{
    WORD    cbSize;             // Size of the structure including extended fields if any.
    WORD    wMode;              // Audio encoding mode.
    WORD    wFormatTag;         // Format tag defined by WAVEFORMATEX.
    DWORD   dwBitRate;          // Average bitrate in Kbps.
    DWORD   dwMaxBitRate;       // Maximum bitrate in Kbps.
} AsfWriterAudioEncInfo;

#define ASFWRITER_UNSET_PROCESSOR                   -1

#define ASFWRITER_AUDIO_BUFFER_MS_DEFAULT           500
#define ASFWRITER_VIDEO_BUFFER_MS_DEFAULT           250

typedef enum tAsfWriterPropertyKey
{
    AsfWriterMuxAudioOffsetInMS = 0,                // INT32
    AsfWriterThreadProcessorAudio,                  // UINT32
    AsfWriterThreadProcessorVideo,                  // UINT32
    AsfWriterThreadProcessorMux,                    // UINT32
    AsfWriterAudioBufferInMS,                       // UINT32
    AsfWriterVideoBufferInMS,                       // UINT32
} AsfWriterPropertyKey;

////////////////////////////////////////////////////////////////////////////////
//
// AsfWriterPropertyValue structure
//
// Structure that describes a generic property, which has a type and value
//
////////////////////////////////////////////////////////////////////////////////

class AsfWriterPropertyValue
{
public:
    AsfWriterPropertyValue();
    VOID Clear();

    enum PropertyType
    {
        NotUsedProperty = 0,        // invalid property type, value not set
        UINT32Property,             // unsigned 32 byte value
        UINT64Property,             // unsigned 64 byte value
        INT32Property,              // signed 32 byte value
        INT64Property,              // signed 64 byte value
        DOUBLEProperty,             // double float value
        BOOLProperty,               // BOOL value
    };

    // a union of values of different types
    union Value
    {
        UINT32          uInt32Value;
        UINT64          uInt64Value;
        INT32           iInt32Value;
        INT64           iInt64Value;
        DOUBLE          flValue;
        BOOL            fValue;
        BYTE            reserved[16];
    };

    PropertyType        m_Type;     // property type
    Value               m_Value;    // property value
};

/////////////////////////////////////////////////////////////////////////////
//
// IASFWriter interface
//
// This interface is called by title to encode and generate Asf file.
//
/////////////////////////////////////////////////////////////////////////////
interface IAsfWriter
{
    // Sets the video stream input as the specified filename.
    virtual HRESULT SetVideoInputFile(__in CONST CHAR* pszAsfFilename) = 0;

    // Sets the video stream input as the specified video stream.
    virtual HRESULT SetVideoInput(__in IAsfWriterVideoStream* pVideoStream,
                                  __in CONST AsfWriterBitmapInfoHeader *pbmiHeader,
                                  __in CONST AsfWriterVideoEncInfo *pVideoEncInfo) = 0;

    // Sets the audio stream input as the specified filename.
    virtual HRESULT SetAudioInputFile(__in CONST CHAR* pszAsfFilename) = 0;

    // Sets the audio stream input as the specified audio stream.
    virtual HRESULT SetAudioInput(__in IAsfWriterAudioStream* pAudioStream,
                                  __in CONST WAVEFORMATEX *pwfx,
                                  __in CONST AsfWriterAudioEncInfo *pAudioEncInfo) = 0;

    // Sets property to configure components inside Asf Writer.
    virtual HRESULT SetProperty(__in AsfWriterPropertyKey propKey,
                                __in CONST AsfWriterPropertyValue* pPropValue ) = 0;

    // Sets the metadata: dwNameLength and dwValueLength are the number of wide char excluding NULL.
    virtual HRESULT SetMetadata(__in_ecount_z(dwNameLength) CONST WCHAR* pwszName,
                                __in DWORD dwNameLength, 
                                __in_ecount_z(dwValueLength) CONST WCHAR* pwszValue,
                                __in DWORD dwValueLength) = 0;

    // Starts the encoding and muxing process. At least one audio or video
    // input must be specified before encoding/muxing can begin.
    //
    // When called asynchronously, the function returns E_PENDING immediately,
    // indicating that the operation will be completed asynchronously.
    // If the operation cannot be performed, it returns a different
    // error code instead. The final status of the operation is stored
    // in the provided XOVERLAPPED structure and can be retrieved
    // using the XGetOverlappedResult function. Use pdwResult in
    // XGetOverlappedResult to retrieve failed HRESULT.
    // Only hEvent in XOVERLAPPED is supported. pCompletionRoutine
    // will be ignored.
    virtual HRESULT Start(__in PXOVERLAPPED pOverlapped) = 0;

    // Stops the encoding/muxing before reaching the end of audio/video stream.
    // It cannot be re-started once it is stopped.
    virtual HRESULT Stop() = 0;
};

/////////////////////////////////////////////////////////////////////////////
//
// CreateAsfWriter
//
// Creates and returns a new instance of Asf Writer, which implements
// IASFWriter interface.
//
// Arguments:
//      ppAsfWriter:    pointer to where the Asf writer is returned.
//      pszASfFilename: pointer to output file name
//
// Return:
//      S_OK on success, error otherwise.
//
/////////////////////////////////////////////////////////////////////////////

XBOXAPI
HRESULT
WINAPI
CreateAsfWriter(__in CONST CHAR* pszAsfFilename,
                __out IAsfWriter** ppAsfWriter);

/////////////////////////////////////////////////////////////////////////////
//
// DestroyAsfWriter
//
// Destroys the instance of Asf Writer, which implements
// IASFWriter interface.
//
// Arguments:
//      pAsfWriter:    pointer to Asf writer instance.
//
// Return:
//      S_OK on success, error otherwise.
//
/////////////////////////////////////////////////////////////////////////////

XBOXAPI
HRESULT
WINAPI
DestroyAsfWriter(__in IAsfWriter* pAsfWriter);
