/************************************************************************
*                                                                       *
*   NuiSpeech.h -- This module defines the APIs for the Natural         *
*                  User Interface(NUI) speech recognition services.     *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once

#ifndef NUIAPI
#error "You must include nuiapi.h rather than including nuispeech.h directly"
#endif

#ifdef __cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------------
//  NuiSpeech definitions
//------------------------------------------------------------------------------

//
// Fired when the speech engine recieves a new stream input with audio data  This will be sent when a NuiSpeechStartRecognition is called.
//
#define NUI_SPEECH_EVENT_START_STREAM        0x00000001
//
// Fired when the input stream is finished because there is no more audio bytes being supplied by the
// microphone.  This will be sent after a NuiSpeechStopRecognition is called.
//
#define NUI_SPEECH_EVENT_END_STREAM          0x00000002
//
// Fired when the speech engine encounters the beginning of a sound
//
#define NUI_SPEECH_EVENT_SOUND_START         0x00000004
//
// Fired when audible sound is no longer available through the input stream or the sound stream has been inactive.
//
#define NUI_SPEECH_EVENT_SOUND_END           0x00000008
//
// Fired when the engine begins recognizing a phrase.  This event must be followed by a NUI_SPEECH_FALSE_RECOGNITION or
// NUI_SPEECH_RECOGNITION event.
//
#define NUI_SPEECH_EVENT_PHRASE_START        0x00000010
//
// Fired when the engine has done a partial recognition of the input stream.  The hypothesis is the engine's best guess of the phrase 
// uttered up to that point in the stream.
//
#define NUI_SPEECH_EVENT_HYPOTHESIS          0x00000020
//
// Fired when the engine has recognized the full phrase of the audio data.
//
#define NUI_SPEECH_EVENT_RECOGNITION         0x00000040
//
// Fired when the engine has no valid recognition.
//
#define NUI_SPEECH_EVENT_FALSE_RECOGNITION   0x00000080
//
// Fired when the soundstream has a hindrance that prevents successful recognition.
//
#define NUI_SPEECH_EVENT_INTERFERENCE        0x00000100
//
// Fired when the engine reached the stream position specified by the client's bookmark.  
//
#define NUI_SPEECH_EVENT_BOOKMARK            0x00000200
//
// Returns the audio data that was sent to the recognizer.  This event will only be fired if NuiSpeechSetProperty is called with NUI_SPEECH_RETAIN_AUDIO.
// set to TRUE.
//
#define NUI_SPEECH_EVENT_RETAINEDAUDIO       0x00000400
//
// Helpers to specify all recognition events or all events
//
#define NUI_SPEECH_ALL_RECOGNITION_EVENTS   (NUI_SPEECH_EVENT_HYPOTHESIS | NUI_SPEECH_EVENT_RECOGNITION | NUI_SPEECH_EVENT_FALSE_RECOGNITION)
#define NUI_SPEECH_ALL_EVENTS 0xFFFFFFFF

//
// This define is used for dynamic grammars with the NuiSpeechAddRuleTransition method 
//
#define NUI_SPEECH_RULE_TRANSITION_WILDCARD (NUI_SPEECH_STATEHANDLE)-2

typedef enum _NUI_SPEECH_GRAMMARSTATE{
  //
  // Enables all top-level rules in the grammar for recognition. 
  //
  NUI_SPEECH_GRAMMARSTATE_DISABLED = 0,
  //
  // Disables all top-level rules in this grammar for recognition
  //
  NUI_SPEECH_GRAMMARSTATE_ENABLED = 1,
  //
  // Disables all top-level rules that are not part of this grammar.  Setting this grammar state
  // effectively disables all other grammers besides this one
  //
  NUI_SPEECH_GRAMMARSTATE_EXCLUSIVE = 3
} NUI_SPEECH_GRAMMARSTATE;

typedef enum _NUI_SPEECH_RULESTATE{
    //
    // disable this rule for recognition
    //
    NUI_SPEECH_RULESTATE_INACTIVE   = 0,
    //
    // enable this rule for recognition
    //
    NUI_SPEECH_RULESTATE_ACTIVE = 1,
}NUI_SPEECH_RULESTATE;

typedef enum _NUI_SPEECH_LOADOPTIONS{
  //
  // grammar is loaded statically and cannot be modified at runtime
  //
  NUI_SPEECH_LOADOPTIONS_STATIC = 0,
  //
  // grammar is loaded dynamically which means that rules may be modified and committed at runtime
  //
  NUI_SPEECH_LOADOPTIONS_DYNAMIC = 1
} NUI_SPEECH_LOADOPTIONS;

typedef enum _NUI_SPEECH_BOOKMARKOPTIONS
{
    NUI_SPEECH_BOOKMARKOPTIONS_NONE = 0x0,
    //
    // if specified, the value passed to the bookmark method is in 100 nanosecond units
    // if absent, the value passed to the bookmark method is in bytes
    //
    NUI_SPEECH_BOOKMARKOPTIONS_TIME_UNITS = 0x1,
    //
    // specifies that the value passed to the bookmark method is relative and ahead
    // the current position
    //
    NUI_SPEECH_BOOKMARKOPTIONS_AHEAD = 0x2,

} NUI_SPEECH_BOOKMARKOPTIONS;

typedef enum _NUI_SPEECH_CONFIDENCE
{
    //
    // Separates the low and rejected confidence range.  If not specified, the default value is 20.
    //
    NUI_SPEECH_CONFIDENCE_LOW,
    //
    // Separates the normal and low threshold values.  If not specified, the default value is 50.
    //
    NUI_SPEECH_CONFIDENCE_NORMAL,
    //
    // Value separates the high and normal confidence range.  If not specified, the default value is 80.
    //
    NUI_SPEECH_CONFIDENCE_HIGH,
} NUI_SPEECH_CONFIDENCE;

typedef enum _NUI_SPEECH_PROPERTY
{
   //
   // Specifies the maximum number of alternate phrases returned in the recognition result.  Default is 5.
   //
   NUI_SPEECH_PROPERTY_MAXIMUM_ALTERNATES,
   //
   // see NUI_SPEECH_CONFIDENCE enum values explanation above
   //
   NUI_SPEECH_PROPERTY_HIGH_CONFIDENCE_THRESHOLD,
   NUI_SPEECH_PROPERTY_NORMAL_CONFIDENCE_THRESHOLD,
   NUI_SPEECH_PROPERTY_LOW_CONFIDENCE_THRESHOLD,
   //
   // Indicates the amount of silence the speech engine looks for before completing
   // a recognition.  This attribute is used when the recognition is not ambiguous.  For example
   // in the case when a grammar has two sentences 1) new game please and 2) new game, a 
   // non-ambiguos recognition would be "new game please".  The valid range for this property is
   // 0 - 10000 milliseconds.  The default is 150 milliseconds.
   // 
   NUI_SPEECH_PROPERTY_RESPONSE_SPEED,
   //
   // Indicates the amount of silence the speech engine will look for before completing
   // an ambiguous recognition.  In the case when the grammar has two sentences 1) new game please
   // and 2) new game, an ambigous recognition would be "new game".  This value must be greater
   // than NUI_SPEECH_PROPERTY_RESPONSE_SPEED.  The valid range for this property is 
   // 0 - 10000 millisconds.  The default is 500 milliseconds.
   //
   NUI_SPEECH_PROPERTY_COMPLEX_RESPONSE_SPEED,
   //
   // Indicates the engine should the buffer the audio used in recognition.  Valid values are TRUE or FALSE.  The
   // default value is FALSE.
   //
   NUI_SPEECH_PROPERTY_RETAIN_AUDIO,
   //
   // Indicates the engine not send hypothesis events.
   // The default value is TRUE.
   //
   NUI_SPEECH_PROPERTY_DISABLE_HYPOTHESES,
   //
   // The engine will only send recognition events for recognitions with confidence
   // values greater than this number.
   // Default value is 0.
   //
   NUI_SPEECH_PROPERTY_CFGCONFIDENCE_REJECTION_THRESHOLD,
} NUI_SPEECH_PROPERTY;

typedef enum _NUI_SPEECH_INTERFERENCE
{
    //
    // The sound received is interpreted by the engine as noise.  The event is generated if there is a SOUND_START followed by a
    // SOUND_END without an intervening PHRASE_START.
    //
    NUI_SPEECH_INTERFERENCE_NOISE = 1,
    //
    // Indicates that the sound recieved is of constant intesity.  This may also indicate that the microphone has been unplugged
    // or muted
    //
    NUI_SPEECH_INTERFERENCE_NOSIGNAL,
    //
    // A sound had been recieved but it is too high for discrete recognition
    //
    NUI_SPEECH_INTERFERENCE_TOOLOUD,
    //
    // A sound has been received but it is too low for discrete recognition
    //
    NUI_SPEECH_INTERFERENCE_TOOQUIET,
    //
    // The words are spoken too quickly for discrete recognition
    //
    NUI_SPEECH_INTERFERENCE_TOOFAST,
    //
    // The words are spoken too slowly and indicates excessive time between words
    //
    NUI_SPEECH_INTERFERENCE_TOOSLOW
} NUI_SPEECH_INTERFERENCE;

typedef enum _NUI_SPEECH_LANGUAGE
{
    //
    // United States English
    //
    NUI_SPEECH_LANGUAGE_EN_US,
    //
    // French Canadian
    //
    NUI_SPEECH_LANGUAGE_FR_CA,
    //
    // Great Britain English
    //
    NUI_SPEECH_LANGUAGE_EN_GB,
    //
    // Mexican Spanish
    //
    NUI_SPEECH_LANGUAGE_ES_MX,
    //
    // Japanese
    //
    NUI_SPEECH_LANGUAGE_JA_JP,
    //
    // European French
    //
    NUI_SPEECH_LANGUAGE_FR_FR,
    //
    // European Spanish
    //
    NUI_SPEECH_LANGUAGE_ES_ES,
    //
    // German
    //    
    NUI_SPEECH_LANGUAGE_DE_DE,
    //
    // Italian
    //
    NUI_SPEECH_LANGUAGE_IT_IT,
    //
    // Australian English
    //
    NUI_SPEECH_LANGUAGE_EN_AU,
    //
    // Brazilian Portuguese
    //
    NUI_SPEECH_LANGUAGE_PT_BR,
    //
    // Austrian German
    //
    NUI_SPEECH_LANGUAGE_DE_AT,
    //
    // Swiss German
    //
    NUI_SPEECH_LANGUAGE_DE_CH,
    //
    // Swiss French
    //
    NUI_SPEECH_LANGUAGE_FR_CH,
} NUI_SPEECH_LANGUAGE;

typedef enum NUI_SPEECH_MICTYPE{
    NUI_SPEECH_KINECT,
    NUI_SPEECH_WAVE,
} NUI_SPEECH_MICTYPE;

typedef enum _NUI_SPEECH_WORDTYPE{
    //
    // Indicates the word is in lexical form and can be used to access the lexicon
    //
    NUI_SPEECH_WORDTYPE_LEXICAL = 1,
    //
    // Switches off handling of special characters in the grammar.  For example "*" normally means dictation but this 
    // flag means that "*" means asterisk.
    //
    NUI_SPEECH_WORDTYPE_LEXICAL_NO_SPECIAL_CHARS = 3
}NUI_SPEECH_WORDTYPE;

typedef enum _NUI_SPEECH_RULEOPTIONS {
    //
    // Indicates this rule is a top-level rule which means it is an entrypoint into the grammar.
    //
    NUI_SPEECH_RULEOPTIONS_TOPLEVEL = 0x1,
    //
    // Specifies whether this is a top-level rule that is active by default.  
    //
    NUI_SPEECH_RULEOPTIONS_ACTIVE = 0x2,
    //
    // This rule is imported from another grammar and therefore not defined in this grammar.
    //
    NUI_SPEECH_RULEOPTIONS_IMPORT = 0x8,
    //
    // This rule may be modifed or has been created dynamically.  Note that the grammar must have been loaded
    // the NUI_SPEECH_LOADOPTIONS dynamic to enable changes at runtime.
    //
    NUI_SPEECH_RULEOPTIONS_DYNAMIC = 0x20,
    //
    // The root rule for the grammar.
    //
    NUI_SPEECH_RULEOPTIONS_ROOT = 0x40
}NUI_SPEECH_RULEOPTIONS;

typedef enum _NUI_SPEECH_LEXICON_TYPE
{
    NUI_SPEECH_LEXICON_TYPE_USER,
    NUI_SPEECH_LEXICON_TYPE_VENDOR,
    NUI_SPEECH_LEXICON_TYPE_LETTERTOSOUND
} NUI_SPEECH_LEXICON_TYPE;

typedef enum _NUI_SPEECH_EMULATEOPTIONS
{
    NUI_SPEECH_EMULATEOPTIONS_NONE = 0x00000000,
    NUI_SPEECH_EMULATEOPTIONS_IGNORECASE = 0x00000001,
    NUI_SPEECH_EMULATEOPTIONS_GRAMMAR_CONFUSION = 0x10000000,
    NUI_SPEECH_EMULATEOPTIONS_SIMULATE_ONCE = 0x20000000
} NUI_SPEECH_EMULATEOPTIONS;

//------------------------------------------------------------------------------
// NuiSpeech Structures
//------------------------------------------------------------------------------


typedef LPVOID NUI_SPEECH_HANDLE;
typedef NUI_SPEECH_HANDLE NUI_SPEECH_STATEHANDLE;

typedef struct _NUI_SPEECH_INIT_PROPERTIES
{
    NUI_SPEECH_LANGUAGE         Language;
    NUI_SPEECH_MICTYPE          MicrophoneType;
} NUI_SPEECH_INIT_PROPERTIES;


typedef struct _NUI_SPEECH_GRAMMAR
{
    //
    // Grammar id assigned by the title at LoadGrammar time
    //
    ULONG           ulId;
    //
    // reserved
    //
    NUI_SPEECH_HANDLE   hGrammar;
}NUI_SPEECH_GRAMMAR;
    

typedef struct _NUI_SPEECH_RETAINEDAUDIO
{
    //
    // count of bytes of the retained audio in pAudioData
    // 
    ULONG   ulAudioSize;
    //
    // buffer containg the PCM audio retained.  This audio is in 16khz, 16 bit PCM format.
    //
    BYTE *  pAudioData;
} NUI_SPEECH_RETAINEDAUDIO;

typedef struct _NUI_SPEECH_SEMANTIC
{
    //
    //  null-terminated string for the value information of this property
    //
    LPCWSTR             pcwszValue;
} NUI_SPEECH_SEMANTIC;

typedef struct _NUI_SPEECH_RULERESULT
{
    //
    // zero-terminated name of the rule
    //
    LPCWSTR             pcwszName;
    //
    // unique ID of this rule
    //
    ULONG               ulId;
    //
    // The index of the first spoken word of this rule.  Note that the actual NUI_SPEECH_ELEMENT array
    // is found in NUI_SPEECH_PHRASE.pElements 
    //
    ULONG               ulFirstElement;
    //
    // The number of spoken words spanned by this rule
    //
    ULONG               ulCountOfElements;
    //
    // Pointer to the next sibling in the parse tree
    //
    struct _NUI_SPEECH_RULERESULT   *pNextSibling;
    //
    // Pointer to the first child node in the parse tree
    //
    struct _NUI_SPEECH_RULERESULT   *pFirstChild;
    //
    // A floating point value between 0 and 1 for the confidence for this rule computed by the engine.  Note
    // that this rule confidence is the overall confidence of the entire phrase.
    //
    FLOAT               fSREngineConfidence;
    //
    // A bucketized view of the floating point value based on the confidence property threshold
    // values set by the user.  Note that the rule confidence is the overall confidence for the entire phrase.
    //
    NUI_SPEECH_CONFIDENCE eConfidence;
} NUI_SPEECH_RULERESULT;

typedef struct _NUI_SPEECH_SEMANTICRESULT
{
    //
    //null-terminated value of the semantic property
    //
    LPCWSTR                     pcwszValue;
    //
    // the first spoken word spanned by this property
    //
    ULONG                       ulFirstElement;
    //
    // the number of spoken words spanned by this property
    //
    ULONG                       ulCountOfElements;
    //
    // pointer to the first child of this semantic property
    //
    struct _NUI_SPEECH_SEMANTICRESULT   *pFirstChild;
    //
    // pointer to the next sibling in the property tree
    //
    struct _NUI_SPEECH_SEMANTICRESULT   *pNextSibling;
    //
    // Confidence value for this semantic property computed by the speech engine.  If the semantic property is defined
    // on a rule or rule reference then this confidence value is the confidence for all the words in a rule. if the semantic property
    // is defined on a word transition, then this confidence value is the word (or element) confidence.
    //
    FLOAT                       fSREngineConfidence;
    //
    // Bucketized confidence score based on the confidence threshold values set by the user,   If the semantic property is 
    // defined on a rule or rule reference then this confidence value is the confidence for all the words in a rule. if the 
    // semantic property is defined on a word transition, then this confidence value is the word (or element) confidence.
    //
    NUI_SPEECH_CONFIDENCE       eConfidence;
}NUI_SPEECH_SEMANTICRESULT;

typedef struct _NUI_SPEECH_ELEMENT
{
    //
    // The starting offset of the element in 100-nanosecond units of time relative to the start of the phrase
    //
    ULONG         ulAudioTimeOffset;
    //
    // The length of the element in 100-nanosecond units of time
    //
    ULONG         ulAudioSizeTime;
    //
    // The starting offset of the element in bytes relative to the start of the phrase in the input stream
    //
    ULONG         ulAudioStreamOffset;
    //
    // The size of the element in bytes in the input stream
    //
    ULONG         ulAudioSizeBytes;
    //
    // The starting offset of the element in bytes relative to the start of the phrase in the retained
    // audio stream
    //
    ULONG         ulRetainedStreamOffset;
    //
    // The size of the audio in bytes in the retained audio stream
    //
    ULONG         ulRetainedSizeBytes;
    //
    // The null-terminated display text for the element.  For example ","
    //
    LPCWSTR       pcwszDisplayText;
    // 
    // The null-terminated lexical form of this element. For example "comma"
    //
    LPCWSTR       pcwszLexicalForm;
    //
    // The pronunciation of this element as a null-terminated array of phone IDs where a phone ID is a WCHAR
    //
    LPCWSTR       pcwszPronunciation;
    //
    // The required confidence for this element: low, normal, or high.  If a word is prefixed with a '-' (minus) the required
    // confidence is LOW_CONFIDENCE.  If a word is prefixed with a '+' (plus) the required confidence is HIGH_CONFIDENCE.  For example:
    // "This -is -a +test".
    //
    NUI_SPEECH_CONFIDENCE eRequiredConfidence;
    //
    // The actual confidence for this element (low, normal, or high).  This is always at least the required confidence.
    //
    NUI_SPEECH_CONFIDENCE  eActualConfidence;
    //
    // a floating point value between 0 and 1 that is calcuated by the speech engine.
    //
    FLOAT             fSREngineConfidence;
} NUI_SPEECH_ELEMENT;

typedef struct _NUI_SPEECH_PHRASE
{
    //
    // the size of this structure in bytes
    //
    ULONG                       cbSize;
    //
    // language ID of the phrase elements
    //
    NUI_SPEECH_LANGUAGE         eLanguage;
    //
    // user-defined ID of the grammar that contains the top-level rule used to recognize this phrase
    //
    ULONG                       ulGrammarID;
    //
    // Absolute time for the start of phrase as a 64-bit value.  If an application uses wav file input then the stream position
    // and start time are set to zero
    //
    ULONGLONG                   ullStartTime;
    //
    // The starting offset of the phrase in bytes relative to the start of the audio stream.  If downsampling an audio stream the
    // position will be the byte position of the original audio stream
    //
    ULONGLONG                   ullAudioStreamPosition;
    //
    // size of the audio data in byte for this phrase
    //
    ULONG                       ulAudioSizeBytes;
    //
    // length of the phrase audio in 100-nanosecond time units
    //
    ULONG                       ulAudioSizeTime;
    //
    // size, in bytes of the retained audio data
    //
    ULONG                       ulRetainedSizeBytes;
    //
    // Top level rule used to recognize this phrase.  This is also the root of the rule hierarchy.
    //
    NUI_SPEECH_RULERESULT       Rule;
    //
    // number of elements that comprise the recognized phrase
    //
    ULONG                       ulCountOfElements;
    //
    // Array of phrase elements in this phrase.  Each element includes position and text information
    //
    NUI_SPEECH_ELEMENT *        pElements;
    //
    // pointer to the root of the semantic tag property tree
    //
    NUI_SPEECH_SEMANTICRESULT * pSemanticProperties;
} NUI_SPEECH_PHRASE;


typedef struct _NUI_SPEECH_RECORESULT
{
    //
    // The recognized phrase
    //
    NUI_SPEECH_PHRASE           Phrase;
    //
    // Count of alternate phrases that the engine recognized.  Alternate phrases are ordered in terms of decreasing rule confidence values.
    //
    ULONG                       cAlternates;
    //
    // Pointer to an array of alternate phrases
    //
    NUI_SPEECH_PHRASE *         pAlternates;
    //
    // Pointer to the retained audio used to generate this recognition result if available. A valid pointer is returned only if the
    // NUI_SPEECH_PROPERTY_RETAIN_AUDIO property has been specified.
    //
    NUI_SPEECH_RETAINEDAUDIO *  pAudio;
} NUI_SPEECH_RECORESULT;

typedef struct _NUI_SPEECH_EVENT
{
    //
    // This is the type of event fired, see NUI_SPEECH_EVENT_* for further information about event types.  
    // The contents of this structure will vary by event type in terms of which data is valid in the below union.
    //
    ULONG        eventId;
    //
    // The stream number associated with the event.  This number is incremented every time an audio stream is opened.
    //
    ULONG        ulStreamNum;
    //
    // The byte offset in the audio stream associated with the event at which the event was fired.  This indicates the
    // position in the input audio stream.
    //
    ULONGLONG    ullAudioStreamOffset;
    //
    // The fields of the union depend on the event type.  See the comments for each member to determine for which event
    // the member is valid.
    //
    union 
    {
        //
        // applies to NUI_SPEECH_EVENT_HYPOTHESIS, NUI_SPEECH_EVENT_RECOGNITION, and NUI_SPEECH_EVENT_FALSE_RECOGNITON
        // events. 
        //
        NUI_SPEECH_RECORESULT * pResult;
        //
        // applies to NUI_SPEECH_EVENT_BOOKMARK events.  The pvUserContext is passed in by clients when they call 
        // NuiSpeechSetRecogntionBookmark.
        //
        LPVOID  pvUserContext;
        //
        // applies to NUI_SPEECH_EVENT_INTERFERENCE events.
        //
        NUI_SPEECH_INTERFERENCE eInterference;
        //
        // applies to NUI_SPEECH_EVENT_RETAINEDAUDIO
        //
        NUI_SPEECH_RETAINEDAUDIO Audio;
    };
} NUI_SPEECH_EVENT;



typedef struct _NUI_SPEECH_WORD_PRONUNCIATION
{
    LPCWSTR pcwszPronunciation;
    NUI_SPEECH_LANGUAGE eLanguage;
    NUI_SPEECH_LEXICON_TYPE eLexiconType;
} NUI_SPEECH_WORD_PRONUNCIATION;

typedef struct _NUI_SPEECH_WORD_PRONUNCIATION_LIST
{
    ULONG ulCountOfPronunciations;
    NUI_SPEECH_WORD_PRONUNCIATION *pWordPronunciations;    
} NUI_SPEECH_WORD_PRONUNCIATION_LIST;

//------------------------------------------------------------------------------
// NuiSpeech Functions
//------------------------------------------------------------------------------
HRESULT
NuiSpeechEnable(
    _In_ NUI_SPEECH_INIT_PROPERTIES *pInitProperties,
    _In_ DWORD dwHardwareThread
    );

HRESULT
NuiSpeechDisable();

HRESULT
NuiSpeechStartRecognition();

HRESULT 
NuiSpeechStopRecognition();

HRESULT
NuiSpeechGetProperty(
  _In_  NUI_SPEECH_PROPERTY eSpeechProperty,
  _Out_ PLONG plValue
  );

HRESULT
NuiSpeechGetSerializedState(
    _Deref_out_ BYTE **ppbData, 
    _Out_ ULONG *pulSize
    );

HRESULT
NuiSpeechSetSerializedState(
    _In_bytecount_(ulSize) const BYTE *pbData, 
    ULONG ulSize
    );

HRESULT
NuiSpeechDestroySerializedState(
    _In_ BYTE *pbData
    );


HRESULT
NuiSpeechSetProperty(
  _In_  NUI_SPEECH_PROPERTY eSpeechProperty,
  _In_  LONG lNewValue
  );


HRESULT
NuiSpeechLoadGrammar(
    _In_ LPCWSTR pcwszGrammarName,
    _In_  ULONG   ulGrammarId,
    _In_  NUI_SPEECH_LOADOPTIONS Options,
    _Out_ NUI_SPEECH_GRAMMAR * pGrammar
    );

HRESULT
NuiSpeechLoadGrammarFromMemory(
    _In_bytecount_(ulGrammarSize) const BYTE * pcbGrammar,
    _In_ ULONG        ulGrammarSize,
    _In_  ULONG   ulGrammarId,
    _In_  NUI_SPEECH_LOADOPTIONS Options,
    _Out_ NUI_SPEECH_GRAMMAR *pGrammar
    );

HRESULT
NuiSpeechUnloadGrammar(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar
    );

HRESULT
NuiSpeechSetGrammarState(
    _In_ NUI_SPEECH_GRAMMAR *       pGrammar,
    _In_ NUI_SPEECH_GRAMMARSTATE    eState
    );

HRESULT
NuiSpeechGetGrammarState(
    _In_  NUI_SPEECH_GRAMMAR * pGrammar,
    _Out_ NUI_SPEECH_GRAMMARSTATE * peState
    );

HRESULT
NuiSpeechSetRecognitionBookmark(
    _In_ ULONG                      Options,
    _In_ ULONGLONG                  ullStreamPosition,
    _In_opt_ PVOID                  pvUserContext
    );

HRESULT
NuiSpeechSetEventInterest(
    _In_ ULONG ulEvents
    );

HRESULT
NuiSpeechGetEvents(
    _In_               ULONG ulCount,
    _Out_cap_(ulCount) NUI_SPEECH_EVENT *pEventArray,
    _Out_              ULONG *pulFetched
    );

HRESULT
NuiSpeechDestroyEvent(
    _In_ NUI_SPEECH_EVENT* pEvent
    );


HRESULT
NuiSpeechSetRuleWeight(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ LPCWSTR pcwszRuleName,
    _In_ FLOAT fWeight
    );

HRESULT
NuiSpeechSetRulePriority(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ LPCWSTR pcwszRuleName,
    _In_ int nPriority
    );


HRESULT
NuiSpeechSetRuleState(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ LPCWSTR pcwszRuleName,
    _In_ NUI_SPEECH_RULESTATE eState
    );


HRESULT
NuiSpeechCreateGrammar(
    _In_  ULONG   ulGrammarId,
    _Out_ NUI_SPEECH_GRAMMAR * pGrammar
    );

HRESULT
NuiSpeechCreateRule(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ LPCWSTR pwszRuleName,
    _In_ DWORD dwOptions,
    _In_ BOOL fCreateIfNotExist,
    _Out_ NUI_SPEECH_STATEHANDLE * phInitialState
    );


HRESULT
NuiSpeechClearRule(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ NUI_SPEECH_STATEHANDLE hRuleState
    );

HRESULT
NuiSpeechCreateState(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ NUI_SPEECH_STATEHANDLE hRule,
    _Out_ NUI_SPEECH_STATEHANDLE * phNewState
    );

HRESULT
NuiSpeechAddWordTransition(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ NUI_SPEECH_STATEHANDLE hFromState, 
    _In_opt_ NUI_SPEECH_STATEHANDLE hToState,
    _In_opt_ LPCWSTR pcwszWords,
    _In_opt_ LPCWSTR pcwszSeparators,
    _In_ NUI_SPEECH_WORDTYPE eWordType,
    _In_ FLOAT fWeight,
    _In_opt_ NUI_SPEECH_SEMANTIC * pSemantic
    );

HRESULT
NuiSpeechAddRuleTransition(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar,
    _In_ NUI_SPEECH_STATEHANDLE hFromState, 
    _In_opt_ NUI_SPEECH_STATEHANDLE hToState,
    _In_ NUI_SPEECH_STATEHANDLE hRuleState,
    _In_ FLOAT fWeight,
    _In_opt_ NUI_SPEECH_SEMANTIC * pSemantic
    );

HRESULT
NuiSpeechCommitGrammar(
    _In_ NUI_SPEECH_GRAMMAR * pGrammar
    );

HRESULT 
NuiSpeechProcessWaveInput(
    LPCWSTR pcwszWaveName
    );

HRESULT 
NuiSpeechProcessWaveInputFromMemory(
    _In_bytecount_(ulWaveSize) const BYTE * pcbWave,
    _In_ ULONG        ulWaveSize
    );

HRESULT
NuiSpeechGetDefaultPronunciations(
    _In_ LPCWSTR pcwszText,
    _Out_ NUI_SPEECH_WORD_PRONUNCIATION_LIST *pWordPronunciationList
);

HRESULT
NuiSpeechDestroyPronunciationList(
    _In_ NUI_SPEECH_WORD_PRONUNCIATION_LIST *pWordPronunciationList
    );

HRESULT
NuiSpeechEmulateRecognition(
    _In_ LPCWSTR pcwszText,
    _In_ ULONG Options
    );


#define FACILITY_NUI_SPEECH    FACILITY_ITF
#define NUI_SPEECH_ERROR_BASE  0x5000

#define MAKE_NUI_SPEECH_HRESULT(sev, err)    MAKE_HRESULT(sev, FACILITY_NUI_SPEECH, err)
#define MAKE_NUI_SPEECH_ERROR(err)           MAKE_NUI_SPEECH_HRESULT(SEVERITY_ERROR, err + NUI_SPEECH_ERROR_BASE)
#define MAKE_NUI_SPEECH_SCODE(scode)         MAKE_NUI_SPEECH_HRESULT(SEVERITY_SUCCESS, scode + NUI_SPEECH_ERROR_BASE)

//
// E_NUI_SPEECH_UNINITIALIZED                                   0x80045001    -2147201023
// The object has not been properly initialized.
//
#define E_NUI_SPEECH_UNINITIALIZED                              MAKE_NUI_SPEECH_ERROR(0x001)

//
// E_NUI_SPEECH_ALREADY_INITIALIZED                             0x80045002    -2147201022
// The object has already been initialized.
//
#define E_NUI_SPEECH_ALREADY_INITIALIZED                        MAKE_NUI_SPEECH_ERROR(0x002)

//
// E_NUI_SPEECH_UNSUPPORTED_FORMAT                              0x80045003    -2147201021
// The caller has specified an unsupported format.
//
#define E_NUI_SPEECH_UNSUPPORTED_FORMAT                         MAKE_NUI_SPEECH_ERROR(0x003)

//
// E_NUI_SPEECH_INVALID_FLAGS                                   0x80045004    -2147201020
// The caller has specified invalid flags for this operation.
//
#define E_NUI_SPEECH_INVALID_FLAGS                              MAKE_NUI_SPEECH_ERROR(0x004)

//
// S_NUI_SPEECH_END_OF_STREAM                                   0x00045005    282629
// The operation has reached the end of stream.
//
#define S_NUI_SPEECH_END_OF_STREAM                              MAKE_NUI_SPEECH_SCODE(0x005)

//
// E_NUI_SPEECH_DEVICE_BUSY                                     0x80045006    -2147201018
// The wave device is busy.
//
#define E_NUI_SPEECH_DEVICE_BUSY                                MAKE_NUI_SPEECH_ERROR(0x006)

//
// E_NUI_SPEECH_DEVICE_NOT_SUPPORTED                            0x80045007    -2147201017
// The wave device is not supported.
//
#define E_NUI_SPEECH_DEVICE_NOT_SUPPORTED                       MAKE_NUI_SPEECH_ERROR(0x007)

// E_NUI_SPEECH_DEVICE_NOT_ENABLED                              0x80045008    -2147201016
// The wave device is not enabled.
//
#define E_NUI_SPEECH_DEVICE_NOT_ENABLED                         MAKE_NUI_SPEECH_ERROR(0x008)

//
// E_NUI_SPEECH_NO_DRIVER                                       0x80045009    -2147201015
// There is no wave driver installed.
//
#define E_NUI_SPEECH_NO_DRIVER                                  MAKE_NUI_SPEECH_ERROR(0x009)

// E_NUI_SPEECH_FILEMUSTBEUNICODE                               0x8004500a    -2147201014
// The file must be Unicode.
//
#define E_NUI_SPEECH_FILE_MUST_BE_UNICODE                       MAKE_NUI_SPEECH_ERROR(0x00a)

//
// S_NUI_SPEECH_INSUFFICIENTDATA                                0x0004500b    282635
//
#define S_NUI_SPEECH_INSUFFICIENT_DATA                          MAKE_NUI_SPEECH_SCODE(0x00b)

//
// E_NUI_SPEECH_INVALID_PHRASE_ID                               0x8004500c    -2147201012
// The phrase ID specified does not exist or is out of range.
//
#define E_NUI_SPEECH_INVALID_PHRASE_ID                          MAKE_NUI_SPEECH_ERROR(0x00c)

//
// E_NUI_SPEECH_BUFFER_TOO_SMALL                                0x8004500d    -2147201011
// The caller provided a buffer too small to return a result.
//
#define E_NUI_SPEECH_BUFFER_TOO_SMALL                           MAKE_NUI_SPEECH_ERROR(0x00d)

//
// E_NUI_SPEECH_FORMAT_NOT_SPECIFIED                            0x8004500e    -2147201010
// Caller did not specify a format prior to opening a stream.
//
#define E_NUI_SPEECH_FORMAT_NOT_SPECIFIED                       MAKE_NUI_SPEECH_ERROR(0x00e)

//
// S_NUI_SPEECH_AUDIO_PAUSED                                    0x00045010    282640
// This will be returned only on input (read) streams when the stream is paused.  Reads on
// paused streams will not block, and this return code indicates that all of the data has been
// removed from the stream.
//
#define S_NUI_SPEECH_AUDIO_PAUSED                               MAKE_NUI_SPEECH_SCODE(0x010)

//
// E_NUI_SPEECH_RULE_NOT_FOUND                                  0x80045011    -2147201007
// Invalid rule name passed to ActivateGrammar.
//
#define E_NUI_SPEECH_RULE_NOT_FOUND                             MAKE_NUI_SPEECH_ERROR(0x011)

//
// E_NUI_SPEECH_ENGINE_BUSY                                     0x80045014    -2147201004
// In speech recognition, the current method can not be performed while
// a grammar rule is active.
//
#define E_NUI_SPEECH_ENGINE_BUSY                                MAKE_NUI_SPEECH_ERROR(0x014)

//
// S_NUI_SPEECH_NO_HYPOTHESIS_AVAILABLE                         0x00045016    282646
// There is currently no hypothesis recognition available.
//
#define S_NUI_SPEECH_NO_HYPOTHESIS_AVAILABLE                    MAKE_NUI_SPEECH_SCODE(0x016)

//
// E_NUI_SPEECH_CANT_CREATE                                     0x80045017    -2147201001
// Can not create a new object instance for the specified object category.
//
#define E_NUI_SPEECH_CANT_CREATE                                MAKE_NUI_SPEECH_ERROR(0x017)

//
// S_NUI_SPEECH_ALREADY_IN_LEX                                  0x00045018    282648
// The word, pronunciation, or POS pair being added is already in lexicon.
//
#define S_NUI_SPEECH_ALREADY_IN_LEX                             MAKE_NUI_SPEECH_SCODE(0x018)

//
// E_NUI_SPEECH_NOT_IN_LEX                                      0x80045019    -2147200999
// The word does not exist in the lexicon.
//
#define E_NUI_SPEECH_NOT_IN_LEX                                 MAKE_NUI_SPEECH_ERROR(0x019)

// S_NUI_SPEECH_LEX_NOTHING_TO_SYNC                             0x0004501a    282650
// The client is currently synced with the lexicon.
//
#define S_NUI_SPEECH_LEX_NOTHING_TO_SYNC                        MAKE_NUI_SPEECH_SCODE(0x01a)

//
// E_NUI_SPEECH_LEX_VERY_OUT_OF_SYNC                            0x8004501b    -2147200997
// The client is excessively out of sync with the lexicon. Mismatches may not be incrementally sync'd.
//
#define E_NUI_SPEECH_LEX_VERY_OUT_OF_SYNC                       MAKE_NUI_SPEECH_ERROR(0x01b)

//
// E_NUI_SPEECH_UNDEFINED_FORWARD_RULE_REF                      0x8004501c    -2147200996
// A rule reference in a grammar was made to a named rule that was never defined.
//
#define E_NUI_SPEECH_UNDEFINED_FORWARD_RULE_REF                 MAKE_NUI_SPEECH_ERROR(0x01c)

//
// E_NUI_SPEECH_EMPTY_RULE                                      0x8004501d    -2147200995
// A non-dynamic grammar rule that has no body.
//
#define E_NUI_SPEECH_EMPTY_RULE                                 MAKE_NUI_SPEECH_ERROR(0x01d)

//
// E_NUI_SPEECH_GRAMMAR_COMPILER_INTERNAL_ERROR                 0x8004501e    -2147200994
// The grammar compiler failed due to an internal state error.
//
#define E_NUI_SPEECH_GRAMMAR_COMPILER_INTERNAL_ERROR            MAKE_NUI_SPEECH_ERROR(0x01e)

//
// E_NUI_SPEECH_RULE_NOT_DYNAMIC                                0x8004501f    -2147200993
// An attempt was made to modify a non-dynamic rule.
//
#define E_NUI_SPEECH_RULE_NOT_DYNAMIC                           MAKE_NUI_SPEECH_ERROR(0x01f)

//
// E_NUI_SPEECH_DUPLICATE_RULE_NAME                             0x80045020    -2147200992
// A rule name was duplicated.
//
#define E_NUI_SPEECH_DUPLICATE_RULE_NAME                        MAKE_NUI_SPEECH_ERROR(0x020)

//
// E_NUI_SPEECH_DUPLICATE_RESOURCE_NAME                         0x80045021    -2147200991
// A resource name was duplicated for a given rule.
//
#define E_NUI_SPEECH_DUPLICATE_RESOURCE_NAME                    MAKE_NUI_SPEECH_ERROR(0x021)

//
// E_NUI_SPEECH_TOO_MANY_GRAMMARS                               0x80045022    -2147200990
// Too many grammars have been loaded.
//
#define E_NUI_SPEECH_TOO_MANY_GRAMMARS                          MAKE_NUI_SPEECH_ERROR(0x022)

//
// E_NUI_SPEECH_CIRCULAR_REFERENCE                              0x80045023    -2147200989
// Circular reference in import rules of grammars.
//
#define E_NUI_SPEECH_CIRCULAR_REFERENCE                         MAKE_NUI_SPEECH_ERROR(0x023)

//
// E_NUI_SPEECH_INVALID_IMPORT                                  0x80045024    -2147200988
// A rule reference to an imported grammar that could not be resolved.
//
#define E_NUI_SPEECH_INVALID_IMPORT                             MAKE_NUI_SPEECH_ERROR(0x024)

//
// E_NUI_SPEECH_INVALID_WAV_FILE                                0x80045025    -2147200987
// The format of the WAV file is not supported.
//
#define E_NUI_SPEECH_INVALID_WAV_FILE                           MAKE_NUI_SPEECH_ERROR(0x025)

//
// S_NUI_SPEECH_REQUEST_PENDING                                 0x00045026    282662
// This success code indicates that an SR method called with the SPRIF_ASYNC flag is
// being processed.  When it has finished processing, an SPFEI_ASYNC_COMPLETED event will be generated.
//
#define S_NUI_SPEECH_REQUEST_PENDING                            MAKE_NUI_SPEECH_SCODE(0x026)

//
// E_NUI_SPEECH_ALL_WORDS_OPTIONAL                              0x80045027    -2147200985
// A grammar rule was defined with a null path through the rule.  That is, it is possible
// to satisfy the rule conditions with no words.
//
#define E_NUI_SPEECH_ALL_WORDS_OPTIONAL                         MAKE_NUI_SPEECH_ERROR(0x027)

//
// E_NUI_SPEECH_INSTANCE_CHANGE_INVALID                         0x80045028    -2147200984
// It is not possible to change the current engine or input.  This occurs in the
// following cases:
//
//     1) SelectEngine called while a recognition context exists, or
//     2) SetInput called in the shared instance case.
//
#define E_NUI_SPEECH_INSTANCE_CHANGE_INVALID                    MAKE_NUI_SPEECH_ERROR(0x028)

//
// E_NUI_SPEECH_RULE_NAME_ID_CONFLICT                           0x80045029    -2147200983
// A rule exists with matching IDs (names) but different names (IDs).  
//
#define E_NUI_SPEECH_RULE_NAME_ID_CONFLICT                      MAKE_NUI_SPEECH_ERROR(0x029)

//
// E_NUI_SPEECH_NO_RULES                                        0x8004502a    -2147200982
// A grammar contains no top-level, dynamic, or exported rules.  There is no possible
// way to activate or otherwise use any rule in this grammar.
//
#define E_NUI_SPEECH_NO_RULES                                   MAKE_NUI_SPEECH_ERROR(0x02a)

//
// E_NUI_SPEECH_CIRCULAR_RULE_REF                               0x8004502b    -2147200981
// Rule 'A' refers to a second rule 'B' which, in turn, refers to rule 'A'. 
//
#define E_NUI_SPEECH_CIRCULAR_RULE_REF                          MAKE_NUI_SPEECH_ERROR(0x02b)

//
// S_NUI_SPEECH_NO_PARSE_FOUND                                  0x0004502c    282668
// Parse path cannot be parsed given the currently active rules.
//
#define S_NUI_SPEECH_NO_PARSE_FOUND                             MAKE_NUI_SPEECH_SCODE(0x02c)

//
// E_NUI_SPEECH_NO_PARSE_FOUND                                  0x8004502d    -2147200979
//
#define E_NUI_SPEECH_INVALID_HANDLE                             MAKE_NUI_SPEECH_ERROR(0x02d)

//
// E_NUI_SPEECH_REMOTE_CALL_TIMED_OUT                           0x8004502e    -2147200978
// A marshaled remote call failed to respond.
//
#define E_NUI_SPEECH_REMOTE_CALL_TIMED_OUT                      MAKE_NUI_SPEECH_ERROR(0x02e)

//
// E_NUI_SPEECH_AUDIO_BUFFER_OVERFLOW                           0x8004502f    -2147200977
// This will only be returned on input (read) streams when the stream is paused because
// the SR driver has not retrieved data recently.
//
#define E_NUI_SPEECH_AUDIO_BUFFER_OVERFLOW                      MAKE_NUI_SPEECH_ERROR(0x02f)

//
// E_NUI_SPEECH_NO_AUDIO_DATA                                   0x80045030    -2147200976
// The result does not contain any audio, nor does the portion of the element chain of the result
// contain any audio.
//
#define E_NUI_SPEECH_NO_AUDIO_DATA                              MAKE_NUI_SPEECH_ERROR(0x030)

//
// E_NUI_SPEECH_DEAD_ALTERNATE                                  0x80045031    -2147200975
// This alternate is no longer a valid alternate to the result it was obtained from.
// Returned from ISpPhraseAlt methods.
//
#define E_NUI_SPEECH_DEAD_ALTERNATE                             MAKE_NUI_SPEECH_ERROR(0x031)

//
// E_NUI_SPEECH_HIGH_LOW_CONFIDENCE                             0x80045032    -2147200974
// The result does not contain any audio, nor does the portion of the element chain of the result
// contain any audio.  Returned from ISpResult::GetAudio and ISpResult::SpeakAudio.
//
#define E_NUI_SPEECH_HIGH_LOW_CONFIDENCE                        MAKE_NUI_SPEECH_ERROR(0x032)

//
// E_NUI_SPEECH_INVALID_FORMAT_STRING                           0x80045033    -2147200973
// The XML format string for this RULEREF is invalid, e.g. not a GUID or REFCLSID.
//
#define E_NUI_SPEECH_INVALID_FORMAT_STRING                      MAKE_NUI_SPEECH_ERROR(0x033)

//
// S_NUI_SPEECH_UNSUPPORTED_ON_STREAM_INPUT                     0x00045034    282676
// The operation is not supported for stream input.
//
#define S_NUI_SPEECH_UNSUPPORTED_ON_STREAM_INPUT                MAKE_NUI_SPEECH_SCODE(0x034)

//
// E_NUI_SPEECH_APPLEX_READ_ONLY                                0x80045035    -2147200971
// The operation is invalid for all but newly created application lexicons.
//
#define E_NUI_SPEECH_APPLEX_READ_ONLY                           MAKE_NUI_SPEECH_ERROR(0x035)

//
// E_NUI_SPEECH_NO_TERMINATING_RULE_PATH                        0x80045036    -2147200970
//

#define E_NUI_SPEECH_NO_TERMINATING_RULE_PATH                   MAKE_NUI_SPEECH_ERROR(0x036)

//
// S_NUI_SPEECH_WORD_EXISTS_WITHOUT_PRONUNCIATION               0x00045037    282679
// The word exists but without pronunciation.
//
#define S_NUI_SPEECH_WORD_EXISTS_WITHOUT_PRONUNCIATION          MAKE_NUI_SPEECH_SCODE(0x037)

//
// E_NUI_SPEECH_STREAM_CLOSED                                   0x80045038    -2147200968
// An operation was attempted on a stream object that has been closed.
//
#define E_NUI_SPEECH_STREAM_CLOSED                              MAKE_NUI_SPEECH_ERROR(0x038)

//
// E_NUI_SPEECH_NO_MORE_ITEMS                                   0x80045039    -2147200967
// When enumerating items, the requested index is greater than the count of items.
//
#define E_NUI_SPEECH_NO_MORE_ITEMS                              MAKE_NUI_SPEECH_ERROR(0x039)

//
// E_NUI_SPEECH_NOT_FOUND                                       0x8004503a    -2147200966
// The requested data item (data key, value, etc.) was not found.
//
#define E_NUI_SPEECH_NOT_FOUND                                  MAKE_NUI_SPEECH_ERROR(0x03a)

//
// E_NUI_SPEECH_INVALID_AUDIO_STATE                             0x8004503b    -2147200965
// Audio state passed to SetState() is invalid.
//
#define E_NUI_SPEECH_INVALID_AUDIO_STATE                        MAKE_NUI_SPEECH_ERROR(0x03b)

//
// E_NUI_SPEECH_GENERIC_MMSYS_ERROR                             0x8004503c    -2147200964
// A generic MMSYS error not caught by _MMRESULT_TO_HRESULT.
//
#define E_NUI_SPEECH_GENERIC_MMSYS_ERROR                        MAKE_NUI_SPEECH_ERROR(0x03c)

//
// E_NUI_SPEECH_NOT_DYNAMIC_GRAMMAR                             0x8004503e    -2147200962
// Attempt was made to manipulate a non-dynamic grammar.
//
#define E_NUI_SPEECH_NOT_DYNAMIC_GRAMMAR                        MAKE_NUI_SPEECH_ERROR(0x03e)

//
// E_NUI_SPEECH_AMBIGUOUS_PROPERTY                              0x8004503f    -2147200961
// Cannot add ambiguous property.
//
#define E_NUI_SPEECH_AMBIGUOUS_PROPERTY                         MAKE_NUI_SPEECH_ERROR(0x03f)

//
// E_NUI_SPEECH_INVALID_TOKEN_ID                                0x80045041    -2147200959
// The token specified is invalid.
//
#define E_NUI_SPEECH_INVALID_TOKEN_ID                           MAKE_NUI_SPEECH_ERROR(0x041)

//
// E_NUI_SPEECH_TOKEN_IN_USE                                    0x80045044    -2147200956
// Attempted to remove registry data from a token that is already in use elsewhere.
//
#define E_NUI_SPEECH_TOKEN_IN_USE                               MAKE_NUI_SPEECH_ERROR(0x044)

//
// E_NUI_SPEECH_TOKEN_DELETED                                   0x80045045    -2147200955
// Attempted to perform an action on an object token that has had associated registry key deleted.
//
#define E_NUI_SPEECH_TOKEN_DELETED                              MAKE_NUI_SPEECH_ERROR(0x045)

//
// E_NUI_SPEECH_EXPORT_DYNAMIC_RULE                             0x80045047    -2147200953
// Exported rules cannot refer directly or indirectly to a dynamic rule.
//
#define E_NUI_SPEECH_EXPORT_DYNAMIC_RULE                        MAKE_NUI_SPEECH_ERROR(0x047)

//
// E_NUI_SPEECH_WORDFORMAT_ERROR                                0x80045049    -2147200951
// Incorrect word format, probably due to incorrect pronunciation string.
//
#define E_NUI_SPEECH_WORDFORMAT_ERROR                           MAKE_NUI_SPEECH_ERROR(0x049)

//
// E_NUI_SPEECH_STREAM_NOT_ACTIVE                               0x8004504a    -2147200950
// Methods associated with active audio stream cannot be called unless stream is active.
//
#define E_NUI_SPEECH_STREAM_NOT_ACTIVE                          MAKE_NUI_SPEECH_ERROR(0x04a)

//
// E_NUI_SPEECH_ENGINE_RESPONSE_INVALID                         0x8004504b    -2147200949
// Arguments or data supplied by the engine are in an invalid format or are inconsistent.
//
#define E_NUI_SPEECH_ENGINE_RESPONSE_INVALID                    MAKE_NUI_SPEECH_ERROR(0x04b)

//
// E_NUI_SPEECH_SR_ENGINE_EXCEPTION                             0x8004504c    -2147200948
// An exception was raised during a call to the current SR engine.
//
#define E_NUI_SPEECH_SR_ENGINE_EXCEPTION                        MAKE_NUI_SPEECH_ERROR(0x04c)

//
// E_NUI_SPEECH_STREAM_POS_INVALID                              0x8004504d    -2147200947
// Stream position information supplied from engine is inconsistent.
//
#define E_NUI_SPEECH_STREAM_POS_INVALID                         MAKE_NUI_SPEECH_ERROR(0x04d)

//
// S_NUI_SPEECH_RECOGNIZER_INACTIVE                             0x0004504e    282702
// Operation could not be completed because the recognizer is inactive. It is inactive either
// because the recognition state is currently inactive or because no rules are active .
//
#define S_NUI_SPEECH_RECOGNIZER_INACTIVE                        MAKE_NUI_SPEECH_SCODE(0x04e)

//
// E_NUI_SPEECH_LANGID_MISMATCH                                 0x80045052    -2147200942
// An attempt to load a CFG grammar with a LANGID different than other loaded grammars.
//
#define E_NUI_SPEECH_LANGID_MISMATCH                            MAKE_NUI_SPEECH_ERROR(0x052)

//
// S_NUI_SPEECH_PARTIAL_PARSE_FOUND                             0x00045053    282707
// A grammar-ending parse has been found that does not use all available words.
//
#define S_NUI_SPEECH_PARTIAL_PARSE_FOUND                        MAKE_NUI_SPEECH_SCODE(0x053)

//
// E_NUI_SPEECH_NOT_TOPLEVEL_RULE                               0x80045054    -2147200940
// An attempt to deactivate or activate a non-toplevel rule.
//
#define E_NUI_SPEECH_NOT_TOPLEVEL_RULE                          MAKE_NUI_SPEECH_ERROR(0x054)

//
// S_NUI_SPEECH_NO_RULE_ACTIVE                                  0x00045055    282709
// An attempt to parse when no rule was active.
//
#define S_NUI_SPEECH_NO_RULE_ACTIVE                             MAKE_NUI_SPEECH_SCODE(0x055)

// E_NUI_SPEECH_LEX_REQUIRES_COOKIE                             0x80045056    -2147200938
// An attempt to ask a container lexicon for all words at once.
//
#define E_NUI_SPEECH_LEX_REQUIRES_COOKIE                        MAKE_NUI_SPEECH_ERROR(0x056)

//
// S_NUI_SPEECH_STREAM_UNINITIALIZED                            0x00045057    282711
// An attempt to activate a rule/dictation/etc without calling SetInput 
// first in the inproc case.
//
#define S_NUI_SPEECH_STREAM_UNINITIALIZED                       MAKE_NUI_SPEECH_SCODE(0x057)

//
// E_NUI_SPEECH_UNSUPPORTED_LANG                                0x80045059    -2147200935
// The requested language is not supported.
//
#define E_NUI_SPEECH_UNSUPPORTED_LANG                           MAKE_NUI_SPEECH_ERROR(0x059)

//
// E_NUI_SPEECH_VOICE_PAUSED                                    0x8004505a    -2147200934
// The operation cannot be performed because the voice is currently paused.
//
#define E_NUI_SPEECH_VOICE_PAUSED                               MAKE_NUI_SPEECH_ERROR(0x05a)

//
// E_NUI_SPEECH_AUDIO_BUFFER_UNDERFLOW                          0x8004505b    -2147200933
// This will only be returned on input (read) streams when the real time audio device
// stops returning data for a long period of time.
//
#define E_NUI_SPEECH_AUDIO_BUFFER_UNDERFLOW                     MAKE_NUI_SPEECH_ERROR(0x05b)

//
// E_NUI_SPEECH_AUDIO_STOPPED_UNEXPECTEDLY                      0x8004505c    -2147200932
// An audio device stopped returning data from the Read() method even though it was in
// the run state.  This error is only returned in the END_SR_STREAM event.
//
#define E_NUI_SPEECH_AUDIO_STOPPED_UNEXPECTEDLY                 MAKE_NUI_SPEECH_ERROR(0x05c)

//
// E_NUI_SPEECH_NO_WORD_PRONUNCIATION                           0x8004505d    -2147200931
// The SR engine is unable to add this word to a grammar. The application may need to supply 
// an explicit pronunciation for this word.
//
#define E_NUI_SPEECH_NO_WORD_PRONUNCIATION                      MAKE_NUI_SPEECH_ERROR(0x05d)

//
// E_NUI_SPEECH_ALTERNATES_WOULD_BE_INCONSISTENT                0x8004505e    -2147200930
// An attempt to call ScaleAudio on a recognition result having previously
// called GetAlternates. Allowing the call to succeed would result in
// the previously created alternates located in incorrect audio stream positions.
//
#define E_NUI_SPEECH_ALTERNATES_WOULD_BE_INCONSISTENT           MAKE_NUI_SPEECH_ERROR(0x05e)

//
// E_NUI_SPEECH_NOT_SUPPORTED_FOR_SHARED_RECOGNIZER             0x8004505f    -2147200929
// The method called is not supported for the shared recognizer.
// For example, ISpRecognizer::GetInputStream().
//
#define E_NUI_SPEECH_NOT_SUPPORTED_FOR_SHARED_RECOGNIZER        MAKE_NUI_SPEECH_ERROR(0x05f)

//
// E_NUI_SPEECH_TIMEOUT                                         0x80045060    -2147200928
// A task could not complete because the SR engine had timed out.
//
#define E_NUI_SPEECH_TIMEOUT                                    MAKE_NUI_SPEECH_ERROR(0x060)

//
// E_NUI_SPEECH_REENTER_SYNCHRONIZE                             0x80045061    -2147200927
// A SR engine called synchronize while inside of a synchronize call.
//
#define E_NUI_SPEECH_REENTER_SYNCHRONIZE                        MAKE_NUI_SPEECH_ERROR(0x061)

//
// E_NUI_SPEECH_STATE_WITH_NO_ARCS                              0x80045062    -2147200926
// The grammar contains a node no arcs.
//
#define E_NUI_SPEECH_STATE_WITH_NO_ARCS                         MAKE_NUI_SPEECH_ERROR(0x062)

//
// E_NUI_SPEECH_NOT_ACTIVE_SESSION                              0x80045063    -2147200925
// Neither audio output and input is supported for non-active console sessions.
//
#define E_NUI_SPEECH_NOT_ACTIVE_SESSION                         MAKE_NUI_SPEECH_ERROR(0x063)

//
// E_NUI_SPEECH_ALREADY_DELETED                                 0x80045064    -2147200924
// The object is a stale reference and is invalid to use.
// For example having a ISpeechGrammarRule object reference and then calling 
// ISpeechRecoGrammar::Reset() will cause the rule object to be invalidated.
// Calling any methods after this will result in this error.
//
#define E_NUI_SPEECH_ALREADY_DELETED                            MAKE_NUI_SPEECH_ERROR(0x064)

//
// S_NUI_SPEECH_AUDIO_STOPPED                                   0x00045065    282725
// This can be returned from Read or Write calls audio streams when the stream is stopped.
//
#define S_NUI_SPEECH_AUDIO_STOPPED                              MAKE_NUI_SPEECH_SCODE(0x065)

//
// E_NUI_SPEECH_NOT_PROMPT_VOICE                                0x80045068   -2147200920
// The current voice is not a prompt voice, so the ISpPromptVoice
// functions don't work.
//
#define E_NUI_SPEECH_NOT_PROMPT_VOICE                           MAKE_NUI_SPEECH_ERROR(0x068)

//
// E_NUI_SPEECH_ROOTRULE_ALREADY_DEFINED                        0x80045069   -2147200919
// There is already a root rule for this grammar
// Defining another root rule will fail.
//
#define E_NUI_SPEECH_ROOTRULE_ALREADY_DEFINED                   MAKE_NUI_SPEECH_ERROR(0x069)

//
// E_NUI_SPEECH_SCRIPT_DISALLOWED                               0x80045070   -2147200912
// Support for embedded script not supported because browser security settings have disabled it
//
#define E_NUI_SPEECH_SCRIPT_DISALLOWED                          MAKE_NUI_SPEECH_ERROR(0x070)


//
// E_NUI_SPEECH_FAILED_TO_DELETE_FILE                           0x80045075    -2147200907
// Tried and failed to delete an existing file.
//
#define E_NUI_SPEECH_FAILED_TO_DELETE_FILE                      MAKE_NUI_SPEECH_ERROR(0x075)

//
// E_NUI_SPEECH_RECOGNIZER_NOT_FOUND                            0x80045077    -2147200905
// No recognizer is installed.    
//
#define E_NUI_SPEECH_RECOGNIZER_NOT_FOUND                       MAKE_NUI_SPEECH_ERROR(0x077)

//
// E_NUI_SPEECH_AUDIO_NOT_FOUND                                 0x80045078    -2147200904
// No audio device is installed.    
//
#define E_NUI_SPEECH_AUDIO_NOT_FOUND                            MAKE_NUI_SPEECH_ERROR(0x078)

//
// E_NUI_SPEECH_NO_VOWEL                                        0x80045079    -2147200903
// No Vowel in a word 
//
#define E_NUI_SPEECH_NO_VOWEL                                   MAKE_NUI_SPEECH_ERROR(0x079)

//
// E_NUI_SPEECH_UNSUPPORTED_PHONEME                             0x8004507A    -2147200902
// Unknown phoneme
//
#define E_NUI_SPEECH_UNSUPPORTED_PHONEME                        MAKE_NUI_SPEECH_ERROR(0x07A)

//
// S_NUI_SPEECH_NO_RULES_TO_ACTIVATE                            0x0004507B    282747
// The grammar does not have any root or top-level active rules to activate.
//
#define S_NUI_SPEECH_NO_RULES_TO_ACTIVATE                       MAKE_NUI_SPEECH_SCODE(0x07B)

//
// S_NUI_SPEECH_NO_WORDENTRY_NOTIFICATION                       0x0004507C    282748
// The engine does not need SAPI word entry handles for this grammar
//
#define S_NUI_SPEECH_NO_WORDENTRY_NOTIFICATION                  MAKE_NUI_SPEECH_SCODE(0x07C)

//
// E_NUI_SPEECH_WORD_NEEDS_NORMALIZATION                        0x8004507D    -2147200899
// The word passed to the GetPronunciations interface needs normalizing first
//
#define E_NUI_SPEECH_WORD_NEEDS_NORMALIZATION                   MAKE_NUI_SPEECH_ERROR(0x07D)

//
// E_NUI_SPEECH_CANNOT_NORMALIZE                                0x8004507E    -2147200898
// The word passed to the normalize interface cannot be normalized
//
#define E_NUI_SPEECH_CANNOT_NORMALIZE                           MAKE_NUI_SPEECH_ERROR(0x07E)

//
// S_NUI_SPEECH_LIMIT_REACHED                                   0x0004507F    282751
// The word being normalized has generated more than the maximum number of allowed normalized results
// Indicates that returned list is not exhaustive, but contains as many alternatives as the engine is willing to provide.
//
#define S_NUI_SPEECH_LIMIT_REACHED                              MAKE_NUI_SPEECH_SCODE(0x07F)

//
// S_NUI_SPEECH_NOTSUPPORTED                                    0x00045080    282752
// We currently don't support this combination of function call + input
//
#define S_NUI_SPEECH_NOTSUPPORTED                               MAKE_NUI_SPEECH_SCODE(0x080)

//
// E_NUI_SPEECH_PHONEME_CONVERSION                              0x80045082    -2147200894
// Cannot convert the phonemes to the specified phonetic alphabet.
//
#define E_NUI_SPEECH_PHONEME_CONVERSION                         MAKE_NUI_SPEECH_ERROR(0x082)

//
// E_NUI_SPEECH_OVERLOAD                                        0x80045084         -2147200892
// The operation cannot be carried out due to overload and should be attempted again.
//
#define E_NUI_SPEECH_OVERLOAD                                   MAKE_NUI_SPEECH_ERROR(0x084)

//
// E_NUI_SPEECH_LEX_INVALID_DATA                                0x80045085         -2147200891
// The lexicon data is invalid or corrupted.
//
#define E_NUI_SPEECH_LEX_INVALID_DATA                           MAKE_NUI_SPEECH_ERROR(0x085)

//
// E_NUI_SPEECH_CFG_INVALID_DATA                                0x80045086         -2147200890
// The data in the CFG grammar is invalid or corrupted
//
#define E_NUI_SPEECH_CFG_INVALID_DATA                           MAKE_NUI_SPEECH_ERROR(0x086)

//
// E_NUI_SPEECH_LEX_UNEXPECTED_FORMAT                           0x80045087       -2147200889
// The lexicon is not in the expected format.
//
#define E_NUI_SPEECH_LEX_UNEXPECTED_FORMAT                      MAKE_NUI_SPEECH_ERROR(0x087)

//
// E_NUI_SPEECH_STRING_TOO_LONG                                 0x80045088         -2147200888
// The string is too long.
//
#define E_NUI_SPEECH_STRING_TOO_LONG                            MAKE_NUI_SPEECH_ERROR(0x088)

//
// E_NUI_SPEECH_STRING_EMPTY                                    0x80045089         -2147200887
// The string cannot be empty.
//
#define E_NUI_SPEECH_STRING_EMPTY                               MAKE_NUI_SPEECH_ERROR(0x089)

//
// E_NUI_SPEECH_NON_WORD_TRANSITION                             0x80045090         -2147200886
// One of the parse transitions is not a word transition.
//
#define E_NUI_SPEECH_NON_WORD_TRANSITION                        MAKE_NUI_SPEECH_ERROR(0x090)

//
// E_NUI_SPEECH_SISR_ATTRIBUTES_NOT_ALLOWED                     0x80045091   -2147200885
// Attributes are not allowed at the top level.
//
#define E_NUI_SPEECH_SISR_ATTRIBUTES_NOT_ALLOWED                MAKE_NUI_SPEECH_ERROR(0x091)

//
// E_NUI_SPEECH_SISR_MIXED_NOT_ALLOWED                          0x80045092   -2147200884
// Mixed content is not allowed at the top level.
//
#define E_NUI_SPEECH_SISR_MIXED_NOT_ALLOWED                     MAKE_NUI_SPEECH_ERROR(0x092)

#ifdef __cplusplus
} //close extern "C"
#endif

