////////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) Microsoft Corporation. All rights reserved.
//
// XAV.h
//
// Public APIs (XDK) for XAV Player and related components.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(__cplusplus)
#error C++ compiler required.
#endif

#pragma once
#include "d3d9.h"

#if defined(_PREFAST_)
#pragma push_macro("nullptr")
#undef nullptr
#define nullptr NULL
#endif // _PREFAST_

#pragma warning( push )
// disable warning 4201, unnamed struct/union fields
#pragma warning( disable : 4201)


interface IXAudio2;
interface IXAudio2Voice;

////////////////////////////////////////////////////////////////////////////////
//
// XAV global methods and structures
//
// Structures and methods used for global init/uninit of Xbox A/V subsystem, as
// well as to provide the video players the ability to render each frame of video
// playback.
//
////////////////////////////////////////////////////////////////////////////////

// XAV threads, used to assign processor affinity for the global threads and
// defaults for the per-player threads. See documentation for default values.
enum XAVThreadType
{
    // Global media work thread(s)
    XAVGlobalMediaWorkThread,

    // The default per-player work thread(s) affinity
    XAVPlayerWorkThread,

    // The default per-player video work thread(s) affinity
    XAVVideoWorkThread,

    // Last value, not a thread type
    XAVMaxThreadTypes
};

static const DWORD c_dwUseDefaultAffinity = 0xffffffff;

//
// Initialize the XAV and associated rendering system. Application should call
// this method before using any other XAV API.
//
// The initialization structure must provide an existing D3D device and the
// presentation parameters with which the device is set up, as well as a BOOL
// flag fGameMode.
//
// If fGameMode is TRUE, XAV sacrifices video color accuracy to acheive the best game 
// picture quality if color conversion and gamma/degamma are invovled. Otherwise,
// XAV sacrifices UI color accuracy to achieve the best video quality.
//
// The application must provide an array of thread affinities that will be used
// by XAV components that create threads. This array is indexed by
// XAVThreadType values, and each entry in the array is the affinity of the
// corresponding thread. Use value of c_dwUseDefaultAffinity to allow XAV to use
// the default affinity for the corresponding thread.
//
// Once initialized, in order for any playback content to be rendered, the
// application must follow the following contract:
//
// XAVPreRender() must be called right after Swap as next frame's pre-render.
//
// XAVPostRender() must be called right after applications render, and before
// Swap.
//
struct XAVInit
{
    IDirect3DDevice9*               pDevice;
    const D3DPRESENT_PARAMETERS*    pPresentParam;
    BOOL                            fGameMode;
};

XBOXAPI
HRESULT
WINAPI
XAVInitialize(__in const XAVInit* pInit,
              __in DWORD ThreadAffinities[XAVMaxThreadTypes]);

//
// Uninitialize the XAV and associated rendering system. Application should not
// call any other XAV API without re-initializing (calling XAVInitialize).
// Calls to XAVShutdown must match the calls to XAVInitialize, with the
// non-matching calls failing with E_UNEXPECTED.
//
XBOXAPI
HRESULT
WINAPI
XAVShutdown();

//
// Application must call this method right after Swap as next frame's
// pre-render.
//
XBOXAPI
HRESULT
WINAPI
XAVPreRender();

//
// Application must call this method right after its render and before Swap.
//
XBOXAPI
HRESULT
WINAPI
XAVPostRender();


////////////////////////////////////////////////////////////////////////////////
//
// XAVPropertyValue structure
//
// Structure that describes a generic property, which has a type and value
//
////////////////////////////////////////////////////////////////////////////////

struct XAVPropertyValue
{
    XAVPropertyValue() { Clear(); }

    void Clear()
    {
        m_Type = NotUsedProperty;
        m_guidPassthrough = GUID_NULL;
        m_Value.guidValue = GUID_NULL;
    }

    enum PropertyType
    {
        NotUsedProperty = 0,        // invalid property type, value not set
        UINT32Property,             // unsigned 32 byte value
        UINT64Property,             // unsigned 64 byte value
        INT32Property,              // signed 32 byte value
        INT64Property,              // signed 64 byte value
        DOUBLEProperty,             // double float value
        BOOLProperty,               // BOOL value
        CharProperty,               // char value
        WCharProperty,              // wide char value
        GUIDProperty,               // a GUID structure
        IUnknownProperty,           // pointer to IUnknown, with reference count incremented
        StringProperty,             // NULL terminated string (char *)
        WStringProperty,            // NULL terminated string (wchar *)
        PVoidProperty,              // a generic point (void *)
    };

    // a union of values of different types
    union Value
    {
        UINT32          uInt32Value;
        UINT64          uInt64Value;
        INT32           iInt32Value;
        INT64           iInt64Value;
        DOUBLE          flValue;
        BOOL            fValue;
        CHAR            cValue;
        WCHAR           wcValue;
        GUID            guidValue;
        IUnknown*       pIUnkValue;
        struct
        {
            // length in bytes, including the NULL terminator
            DWORD       dwBufferLength;
            union
            {
                PCSTR   pszValue;
                PCWSTR  pwszValue;
                void*   pValue;
            };
        };
    };

    PropertyType        m_Type;         // property type
    Value               m_Value;        // property value
    GUID                m_guidPassthrough; // GUID used for pass-through properties
};


// XAVPropertyKeyRanges enum is used to define property key ranges. These values
// are never used by the application.
enum XAVPropertyKeyRanges
{
    // ranges
    XAVOpenParamRangeStart = 1,
    XAVOpenParamRangeEnd = XAVOpenParamRangeStart + 999,
    XAVGenericPlayerRangeStart,
    XAVGenericPlayerRangeEnd = XAVGenericPlayerRangeStart + 999,
    XAVStatsRangeStart,
    XAVStatsRangeEnd = XAVStatsRangeStart + 999,
    // may be used to start a ranges of extended properties
    XAVFirstAvailable = XAVStatsRangeEnd + 10000,
};

////////////////////////////////////////////////////////////////////////////////
//
// XAVPropertyKey enum
//
// Enum that describes a property Key used for any property bag related to the
// XAV. The names are prefixed with the content they are intended for, such as
// OpenParam for properties used by the player to open content, Stats to access
// the player statistics, Player for properties used by the player after opening
// content.
//
////////////////////////////////////////////////////////////////////////////////

enum XAVPropertyKey
{
    // marks an invalid key
    XAVInvalidPropertyKey = 0,

    // XAV Player open parameters, must be set before IXAVPlayer::Open() call.
    XAVOpenParamAutoPlay = XAVOpenParamRangeStart,          // BOOL

    // If true (default), the player will automatically call the OS to bring up
    // the UI to download the Optional Media Update (codec pack), else, if
    // Optional Media Update is not downloaded, Open will fail
    XAVOpenParamAutoDownloadCodecPack,                      // BOOL
    // Set affinity of video stream thread
    XAVOpenParamVideoWorkThreadAffinity,                    // UINT32

    // Number of decoding threads used by decoder and their processor affinity
    // mask. For example, to use 2 decoder threads that use hardware threads 2
    // and 3 use the mask ((1<<2) | (1<<3)). Number of set bits in the mask
    // specifies the number of decoder threads. The default is to use 3 decoder
    // threads with a mask of 7 (hardware threads 0, 1, 2).
    XAVOpenParamDecoderWorkerThreadAffinityMask,            // UINT32

    // If true, PlayReady functionality will be enabled.
    // If false (default), PlayReady functionality will be disabled.
    XAVOpenParamUsePlayReady,                               // BOOL

    // general player properties (GetPlayerPropertyStore for
    // IXAVPropertyStore).
    XAVPlayerUniqueID = XAVGenericPlayerRangeStart,         // UINT32
    XAVPlayerWorkThreadAffinity,                            // UINT32
    XAVPlayerResetPositionOnStop,                           // BOOL
    XAVPlayerLoopPlayback,                                  // BOOL

    // player stats, see IXAVPlayerStats
    XAVStatsQuality = XAVStatsRangeStart,                   // INT32
    XAVStatsBufferSec,                                      // DOUBLE
    XAVStatsMaxBufferSec,                                   // DOUBLE
    XAVStatsBandwidth,                                      // UINT32
    XAVStatsBitrate,                                        // UINT32
    XAVStatsMaxBitrate,                                     // UINT32
    XAVStatsDownloadBitrate,                                // UINT32
    XAVStatsFrameRate,                                      // DOUBLE
    XAVStatsActualFramesPerSecond,                          // DOUBLE
    XAVStatsDroppedFramesPerSecond,                         // DOUBLE
    XAVStatsTotalDroppedFrames,                             // UINT32
    XAVStatsReceiveRate,                                    // UINT32
    XAVStatsSampleTime,                                     // UINT64
    XAVStatsSessionID,                                      // UINT32
};


////////////////////////////////////////////////////////////////////////////////
//
// IXAVPropertyStore interface
//
// Interface used to get or set a property. This interface is used in several
// places like stats, initialization parameters, etc.
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{B56C6AA7-A00A-4FD2-9728-70CAF7725FE5}"))
IXAVPropertyStore :
    public IUnknown
{
    // Returns the count of properties in the property store
    virtual DWORD GetCount() = 0;

    // Returns the property key at the specified index
    virtual HRESULT GetAt(__in DWORD dwPropIndex,
                          __out DWORD* pdwKey) = 0;

    // Set the property value for the specified key.
    //
    // Returns S_OK on success, E_POINTER if pProperty is NULL, E_HANDLE if
    // property key is unknown, E_OUTOFMEMORY if no memory, E_INVALIDARG if
    // property value type does not match the key, property type is not known,
    // or buffer-based properties have a NULL buffer or 0 size length.
    virtual HRESULT SetValue(__in DWORD dwKey,
                             __in const XAVPropertyValue* pProperty) = 0;

    // Returns the property value of a property for the specified key. If the
    // property value requires a buffer (blob and string properties), the caller
    // must provide a buffer in pProperty->m_Value.pValue and its size in
    // pProperty->m_Value.dwBufferLength in bytes, not characters. If the buffer
    // specified is NULL or buffer size is not sufficient, the required buffer
    // size is returned in pProperty->m_Value.dwBufferLength.
    //
    // Returns S_OK on success, S_FALSE if the specified key is not in the
    // property store, E_POINTER if pProperty is NULL, E_HANDLE if property key
    // is unknown, E_OUTOFMEMORY if no buffer is specified or buffer is too
    // small.
    virtual HRESULT GetValue(__in DWORD dwKey,
                             __out XAVPropertyValue* pProperty) = 0;
};

////////////////////////////////////////////////////////////////////////////////
//
// XAV PlayReady Error Messages
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_HRESULT_FACILITY                  60

////////////////////////////////////////////////////////////////////////////////
//
// Description: Failed to find a valid license.
//
// Value: 0x803C0008
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_NO_LICENSE_AVAILABLE               MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 8)

////////////////////////////////////////////////////////////////////////////////
//
// Description: This is primarily used when the PlayReady.xex binary could not be found at Game:\.
//
// Value: 0x803C000E
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_FILE_NOT_FOUND                     MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 14)

////////////////////////////////////////////////////////////////////////////////
//
// Description: PlayReady could not find enough free disk space to create the LicenseStore.
//
// Value: 0x803C000F
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_DISKSPACE_ERROR                    MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 15)

////////////////////////////////////////////////////////////////////////////////
//
// Description: The Root License was not found.
//
// Value: 0x803C0010
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_UPLINKLICENSENOTFOUND              MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 16)

////////////////////////////////////////////////////////////////////////////////
//
// Description: The License Response returned an error specific to the service. Check the response CustomData for the error.
//
// Value: 0x803C0011
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_SERVER_SERVICE_SPECIFIC            MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 17)

////////////////////////////////////////////////////////////////////////////////
//
// Description: PlayReady Content Header is invalid.
//
// Value: 0x803C0012
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_CH_INVALID_HEADER                  MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 18)

////////////////////////////////////////////////////////////////////////////////
//
// Description: PlayReady Content Key is invalid.
//
// Value: 0x803C0013
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_CH_BAD_KEY                         MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 19)

////////////////////////////////////////////////////////////////////////////////
//
// Description: There was an error in the PlayReady Protocol
//
// Value: 0x803C0015
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_DRM_PROTOCOL_ERROR                 MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 21)

////////////////////////////////////////////////////////////////////////////////
//
// Desciption: Generic PlayReady Failure
//
// Value: 0x803C0064
//
////////////////////////////////////////////////////////////////////////////////
#define XAV_PLAYREADY_E_FAIL                               MAKE_HRESULT(SEVERITY_ERROR, XAV_PLAYREADY_HRESULT_FACILITY, 100)

enum XAVContentKeyType
{
    Aes128Ctr     = 1,
    Cocktail      = 4,
};

////////////////////////////////////////////////////////////////////////////////
//
// IXAVLicenseAcquisition interface
//
// Interface used to perform PlayReady license acquisition
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{1302BCFB-35C5-4BB0-8244-E0B2E05569FC}"))
IXAVLicenseAcquisition :
    public IUnknown
{
    // If cbHeader is 0, then E_INVALIDARG is returned unless this
    // interface was created using the CreateXAVLicenseAcquisition
    // version that takes in the pUnkDetail member of XAVPlayerAlert.
    // If *pcbChallenge = 0 on output, then no license acquisition is required.
    // This will be the case for key-rotation content when this
    // interface was created using the CreateXAVLicenseAcquisition
    // version that takes in the pUnkDetail member of XAVPlayerAlert.
    // This should not be performed on the UI thread.
    // If the application is dealing with a missing root license, it needs
    // to know that and do root license acquisition (i.e. know the root KID).
    // Memory allocated in *ppbChallenge and *ppszUrl should be freed using
    // XMemFree( ptr, MAKE_XALLOC_ATTRIBUTES( 1, TRUE, FALSE, FALSE,
    //  eXALLOCAllocatorId_XAVPipeline, XALLOC_ALIGNMENT_DEFAULT,
    //  XALLOC_MEMPROTECT_READWRITE, FALSE, XALLOC_MEMTYPE_HEAP ) );
    virtual HRESULT GenerateLicenseAcquisitionChallenge(
        __in_opt BYTE   *pbHeader,
        __in     DWORD   cbHeader,
        __in_opt char   *pszChallengeCustomData,
        __out    DWORD  *pcbChallenge,
        __out    BYTE  **ppbChallenge,
        __out    char  **ppszUrl ) = 0;

    // Memory allocated in *ppbChallenge should be freed using
    // XMemFree( ptr, MAKE_XALLOC_ATTRIBUTES( 1, TRUE, FALSE, FALSE,
    //  eXALLOCAllocatorId_XAVPipeline, XALLOC_ALIGNMENT_DEFAULT,
    //  XALLOC_MEMPROTECT_READWRITE, FALSE, XALLOC_MEMTYPE_HEAP ) );
    virtual HRESULT GenerateLicenseAcquisitionChallenge(
        __in     GUID              *pKID,
        __in     XAVContentKeyType  eKeyType,
        __in_opt char              *pszChallengeCustomData,
        __out    DWORD             *pcbChallenge,
        __out    BYTE             **ppbChallenge ) = 0;

    // The caller can specify hrResponse != S_OK to cause playback to fail
    // with the specified error code when this
    // interface was created using the CreateXAVLicenseAcquisition
    // version that takes in the pUnkDetail member of XAVPlayerAlert.
    // In this case, pbResponse can be NULL.
    // Otherwise, the caller should specify S_OK for hrResponse.
    // If pbResponse is a SOAP exception, ProcessLicenseAcquisitionResponse
    // will parse the error appropriately.
    // If that SOAP exception is a protocol redirect, this function will
    // return S_FALSE and populate ppszRedirectUrl.
    // Memory allocated in *ppszRedirectUrl and *ppszResponseCustomData should be freed using
    // XMemFree( ptr, MAKE_XALLOC_ATTRIBUTES( 1, TRUE, FALSE, FALSE,
    //  eXALLOCAllocatorId_XAVPipeline, XALLOC_ALIGNMENT_DEFAULT,
    //  XALLOC_MEMPROTECT_READWRITE, FALSE, XALLOC_MEMTYPE_HEAP ) );
    virtual HRESULT ProcessLicenseAcquisitionResponse(
        __in      HRESULT  hrResponse,
        __in_opt  BYTE    *pbResponse,
        __in      DWORD    cbResponse,
        __out     char   **ppszRedirectUrl,
        __out_opt char   **ppszResponseCustomData ) = 0;

    // Helper method - synchronously sends license acquisition challenge
    // to server and receives response.
    // Application is free to implement their own HTTP communication
    // on top of the XDK instead.
    // This should not be performed on the UI thread.
    // Memory allocated in *ppbResponse should be freed using
    // XMemFree( ptr, MAKE_XALLOC_ATTRIBUTES( 1, TRUE, FALSE, FALSE,
    //  eXALLOCAllocatorId_XAVPipeline, XALLOC_ALIGNMENT_DEFAULT,
    //  XALLOC_MEMPROTECT_READWRITE, FALSE, XALLOC_MEMTYPE_HEAP ) );
    virtual HRESULT SendLicenseAcquisitionChallenge(
        __in      char   *pszUrl,
        __in      BYTE   *pbChallenge,
        __in      DWORD   cbChallenge,
        __out     DWORD  *pcbResponse,
        __out     BYTE  **ppbResponse ) = 0;

    // This should not be performed on the UI thread.
    virtual HRESULT CleanLicenseStore() = 0;
};



////////////////////////////////////////////////////////////////////////////////
//
// XAVPlayerAlert structure
//
//
// Player Alerts structure used by the player to pass various alerts and
// notifications.
//
////////////////////////////////////////////////////////////////////////////////
struct XAVPlayerAlert
{
    XAVPlayerAlert() 
        : Level( NoAlert )
        , fChanged( FALSE )
        , pUnkDetail( nullptr )
        { 
        }

    // Alert Level
    enum AlertLevel
    {
        NoAlert = 0,           // Alert structure is not used
        FatalError,            // Player state going to XAVPlayerStateError, fatal error
        CommandError,          // Individual command failed, player still working
        Warning,               // Unusual occurrence, player still working
        Info,                  // Information only, player still working
    };

    // Component issuing the Alert
    enum AlertComponent
    {
        Load,                   // loading content
        Playback,               // playback of content
        Closing,                // closing player/content
        Buffering,              // Buffering
    };

    // Playback sub types
    enum PlaybackSubType
    {
        PlaybackDefault = 0,        // Playback default sub type
        PlaybackPlayPastEnd,        // playback is past the end of the downloaded content
        PlaybackSeekPastEnd,        // seeked past the end of the downloaded content
        PlaybackInvalidState,       // playback command is issued while player state is not valid
        PlaybackSessionStart,       // error during initial playback startup
        PlaybackNoLicense,          // License is not available or expired
    };

    // Buffering sub types
    enum BufferingSubType
    {
        BufferingStarted = 0,   // buffering started
        BufferingStopped,       // buffering stopped
    };

    // Closing sub types
    enum ClosingSubType
    {
        WhileClosing = 0,       // alert during closing
        WhileClosed = 1,        // alert after closed
        WhileError = 2,         // alert while in XAVPlayerStateError state
    };

    // eLoad sub types
    enum LoadSubType
    {
        LoadDefault = 0,        // Load default sub type
        LoadUnexpectedState,    // Load failed because player is in unexpected state
        LoadInvalidURL,         // Load failed because of an invalid URL
        LoadLocalContentOpen,   // Load of local content failed to open
        LoadCreateTopology,     // error while creating topology
    };

    AlertLevel      Level;      // alert level
    AlertComponent  Component;  // component or area sending the alert
    DWORD           dwType;     // the type of the alert, specific to component

    union {
        INT32       iDetail;    // 32-bit signed alert detail, alert type specific
        HRESULT     hrAlert;    // HRESULT alert detail, alert type specific
        FLOAT       flDetail;   // FLOAT alert detail, alert type specific
    };

    // IUnknown*,  Optional. XAVPlayerAlert does not hold reference, if application want to
    // use it outside alert callback, application needs to hold the reference by itself
    IUnknown*   pUnkDetail; 
    BOOL        fChanged;       // internal use only
};


////////////////////////////////////////////////////////////////////////////////
//
// XAVPlayerState
//
// Player state.
//
////////////////////////////////////////////////////////////////////////////////
enum XAVPlayerState
{
    // Player state after creation; no content is loaded
    XAVPlayerStateClosed,

    // Content is being opened after a call to Open()
    XAVPlayerStateOpening,
    // Content is opened, but underlying infrastructure is not ready to play
    XAVPlayerStateOpened,
    // Underlying infrastructure is being set up
    XAVPlayerStateBuilding,

    // Content is loaded, ready to play, and is currently stopped (black screen
    // is displayed, position is not changing)
    XAVPlayerStateStopped,
    // Content is loaded and about to start playing
    XAVPlayerStateStarting,
    // Content is loaded and playing
    XAVPlayerStatePlaying,
    // Content is loaded and is paused (current frame is displayed,
    // position is not changing)
    XAVPlayerStatePaused,
    // After calling Shutdown(), but before Shutdown() completes. This state is
    // not reported, but can be returned if the state is queried directly
    XAVPlayerStateClosing,
    // there was a fatal error
    XAVPlayerStateError,
    // Shutdown was called, player is no longer usable
    XAVPlayerStateShutdown,
};



// This should be specified when opening the player content if there is no
// bookmark (specific position to start from). For live content, this will start
// playback at the maximum seekable position (as close to live as the stream
// allows), for non-live content, the playback will start from the beginning.
static const ULONGLONG c_ullDefaultStartPosition = 0xffffffffffffffff;

// This value is returned by GetDuration() if the duration is not known.
static const ULONGLONG c_ullDurationUnknown = 0;


////////////////////////////////////////////////////////////////////////////////
//
// IXAVPlayerStatus interface
//
// Interface used to query the player status. Once an object implementing this
// interface is acquired, it holds a snapshot of the player state. The snapshot
// can be refreshed by calling Refresh().
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{AEAA51E0-4910-4607-B99B-9E3613286ADB}"))
IXAVPlayerStatus :
    public IUnknown
{
    // Refreshes the status snapshot.
    virtual void Refresh() = 0;

    // Get current state of the player
    virtual XAVPlayerState GetState() const = 0;

    // Get the current playback positions in hundred nano-second units. Before
    // content is opened, this value is what is specified in the Open() call in
    // the ullStartTime argument.
    virtual ULONGLONG GetPosition() const = 0;

    // Get the timestamp of the last video frame rendered in hundred
    // nano-second units, 0 for audio-only.
    virtual ULONGLONG GetLastFrame() const = 0;

    // Get the duration of the current content in hundred nano-second units.
    // Before content is opened and for some live content for which duration
    // is never known, this field is set to c_ullDurationUnknown.
    virtual ULONGLONG GetDuration() const = 0;

    // Get the height and width of content native resolution.
    virtual DWORD GetContentResolutionHeight() const = 0;
    virtual DWORD GetContentResolutionWidth() const = 0;

    // Returns the currently loaded URL into the provided buffer. If the pointer
    // is NULL, the return value can be used to determine the string buffer size
    // required to hold the whole current URL, including the NULL terminator.
    // The string written is always NULL terminated, but may not contain the
    // whole URL string if the buffer provided is too small. No more than the
    // specified number of bytes are written into the string buffer.
    //
    // The return value is 0 if no URL is loaded, and the length of the URL
    // string buffer in bytes required to hold the whole URL, including the NULL
    // terminator. If the buffer provided was large enough, the return value is
    // one larger than the string length of the URL.
    virtual DWORD GetCurrentUrl(__out_z_opt char* pszUrl, __in DWORD dwBufSize) const = 0;

    // Returns the previous status, allowing the caller to determine the last
    // status change.
    // Note that the caller is given a reference to the returned object, and
    // must call Release() to release this reference.
    virtual HRESULT GetPreviousStatus(__out IXAVPlayerStatus** ppPrevStatus) const = 0;

    // Current playback rate. 1.0 is normal playback, positive rate above 1.0
    // is fast forward playback rate multiple, negative rate is fast reverse
    // playback rate multiple, 0.0 for "scrubbing" mode.
    virtual FLOAT GetRate() const = 0;

    // Returns TRUE if trick play (non-1.0 rate) is "thinning", or not playing
    // every frame, just keyframes, FALSE otherwise
    virtual BOOL IsThinning() const = 0;

    // Returns true for content that can be seeked, FALSE otherwise.
    virtual BOOL IsSeekAllowed() const = 0;

    // Returns TRUE for audio-only content, FALSE for video content.
    virtual BOOL IsAudioOnly() const = 0;

    // Returns TRUE for video-only content, FALSE for audio content.
    virtual BOOL IsVideoOnly() const = 0;

    // Returns TRUE if content reached the end (or start during rewind)
    virtual BOOL IsEndOfMedia() const = 0;

    // Returns TRUE if the sound was muted by the user
    virtual BOOL IsMute() const = 0;

    // Returns true if any transport operation (play, stop, seek, etc.) is
    // pending. This value is meaningless in the previous state.
    virtual BOOL IsOperationPending() const = 0;

    // Returns true if the content is buffering.
    virtual BOOL IsBuffering() const = 0;

    // Returns the fraction of content buffered (1.0 when not buffering).
    virtual FLOAT GetFractionBufferingCompleted() const = 0;

    // If the state is XAVPlayerStateError, the structure returned into *pAlert
    // is the first Alert that caused the transition to the XAVPlayerStateError
    // state. If the state is not XAVPlayerStateError, Alert level is "NoAlert".
    virtual void GetFirstAlert(__out XAVPlayerAlert* pAlert) const = 0;
};


////////////////////////////////////////////////////////////////////////////////
//
// IXAVPlayerStats interface
//
// XAV Player statistics interface returned by the player. It derives from the
// IXAVPropertyStore interface, keeping statistics as a property bag. A snapshot
// of statistics is returned by the player's GetStats() call. The snapshot can
// be updated by calling Refresh. Statistics property keys are selected by
// XAVPropertyKey (XAVStatsXXX).
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{9FA7B2AD-0F42-42ae-B3A0-70C7A78E1220}"))
IXAVPlayerStats :
    public IXAVPropertyStore
{
    virtual HRESULT Refresh() = 0;
};

////////////////////////////////////////////////////////////////////////////////
//
// IXAVPlayerEvents interface
//
// Callbacks called by the XAV Player to notify of player events.
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{9FA7B2AD-0F42-42ae-B3A0-70C7A78E1220}"))
IXAVPlayerEvents :
    public IUnknown
{
    // Called to notify the application of a change in player status
    virtual void StatusChanged(__in const IXAVPlayerStatus* pStatus) = 0;

    // Called often to notify the application of a playback position change
    virtual void PositionUpdate(__in const IXAVPlayerStatus* pStatus) = 0;

    // Called to notify the application of an Alert (see XAVPlayerAlert)
    virtual void Alert(__inout XAVPlayerAlert* pAlert,
                       __in const IXAVPlayerStatus* pStatus) = 0;
};


//
// This structure is used by IXAVPlayerGetTexture::GetTexture to describe the
// texture details.
//
struct XAVRenderResultInfo
{
    RECT            resultRect;
    DWORD           dwPixelAspectRatioNumerator;
    DWORD           dwPixelAspectRatioDenominator;
    D3DCOLORSPACE   colorSpace;
    LONGLONG        llTimeStamp;
};

////////////////////////////////////////////////////////////////////////////////
//
// IXAVPlayerGetTexture interface
//
// Interface implemented by the player to get the current video frame and its
// info. Call GetPlayerFrameInfo() on IXAVPlayer to get this object.
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{B3CC6A2A-F376-4574-B7FA-31AB0E087939}"))
IXAVPlayerGetTexture :
    public IUnknown
{
    //
    // Get the texture for the current video frame. Fails if texture is not
    // available for this player.
    //
    // The parameter fCopyTexture should be set to false unless the caller
    // intends to keep this texture for longer than one render loop cycle.
    // Setting fCopyTexture to true forces a new texture to be created for the
    // next frame (as opposed to reusing the same texture), which incurs a
    // performance penalty. The caller must release the texture returned by
    // GetTexture by calling the Release method.
    //
    virtual HRESULT GetTexture(__in BOOL fCopyTexture,
                               __out IDirect3DTexture9** ppTexture,
                               __out XAVRenderResultInfo* pResultInfo) = 0;
    //
    // Returns true if current texture is "dirty". If a new texture has come in
    // and it hasn't been requested by GetTexture, returns true. If the
    // texture has been requested by GetTexture and no new texture came in,
    // then return false.
    //
    virtual BOOL IsTextureDirty() const = 0;
};

////////////////////////////////////////////////////////////////////////////////
//
// IXAVPlayer interface
//
// Player interface that uses D3D Texture to render the video.
//
////////////////////////////////////////////////////////////////////////////////
interface __declspec(uuid("{636907E3-9C92-48C4-8674-0228A01ED1F3}"))
IXAVPlayer :
    public IUnknown
{
    // This method must be called to allow the player to release circular
    // references to the calling application. This method is asynchronous. Not
    // calling this method may result in leaking the player due to the player
    // keeping a reference to the IXAVPlayerEvents.
    virtual void Shutdown() = 0;

    // Open a URL for playback at the specified starting time. All OpenParamXXX
    // properties must be set to the desired values before calling this method.
    // The starting time (ullStartTime) is in hundred nano-second units.
    virtual HRESULT Open(__in const char* pszUrl,
                         __in ULONGLONG ullStartTime = c_ullDefaultStartPosition) = 0;

    // Start or resume playback of the player
    virtual HRESULT Play() = 0;

    // Pause the player
    virtual HRESULT Pause() = 0;

    // Stop playback. Current position is maintained by default or reset to 0
    // if XAVPlayerResetPositionOnStop property is set.
    virtual HRESULT Stop() = 0;

    // Sets playback rate, negative is reverse (rewind) rate.
    virtual HRESULT SetRate(__in FLOAT flRate, __in BOOL fThinning = FALSE) = 0;

    // Seeks to an absolute position, in hundred nano-second units
    virtual HRESULT SeekAbsolute(__in ULONGLONG ullPosition) = 0;

    // Mutes or un-mutes audio based on fMute.
    virtual HRESULT Mute(__in BOOL fMute) = 0;

    // Returns the current player status into the provided pointer. The previous
    // status can be retrieved from the returned status so that the application
    // can determine the change in status.
    // Note that the caller is given a reference to the returned object, and
    // must call Release() to release this reference.
    virtual HRESULT GetStatus(__out IXAVPlayerStatus** ppStatus) const = 0;

    // Returns the current player state. This state is part of the status,
    // provided directly for expediency.
    virtual XAVPlayerState GetPlayerState() const = 0;

    // Get player statistics.
    // Note that the caller is given a reference to the returned object, and
    // must call Release() to release this reference.
    virtual HRESULT GetStats(__out IXAVPlayerStats** ppStats) = 0;

    // Get player property store interface. Used to set/get XAVPlayerXXX and
    // XAVOpenParamXXX properties.
    // Note that the caller is given a reference to the returned object, and
    // must call Release() to release this reference.
    virtual HRESULT GetPlayerPropertyStore(__out IXAVPropertyStore** ppProperties) = 0;

    // Get player frame information interface. Use this interface to get details
    // about each frame rendered by the player.
    // Note that the caller is given a reference to the returned object, and
    // must call Release() to release this reference.
    virtual HRESULT GetPlayerFrameInfo(__out IXAVPlayerGetTexture** ppFrameInfo) = 0;

    // Provide an optional XAudio2 destination voice to use for audio portion of
    // playback.
    virtual HRESULT SetAudioVoice(__in_opt IXAudio2Voice* pOutputVoice) = 0;
};


////////////////////////////////////////////////////////////////////////////////
//
// CreateXAVPlayer
//
// Creates and returns a new instance of XAV player, which implements IXAVPlayer
// interface.
//
// Arguments:
//      ppPlayer:   pointer to where the player is returned.
//      pEvents     pointer to callbacks for events
//      pAudio2     pointer to XAudio2 engine the player should use for audio
//
// Return:
//      S_OK on success, error otherwise.
//
////////////////////////////////////////////////////////////////////////////////

XBOXAPI
HRESULT
WINAPI
CreateXAVPlayer(__out IXAVPlayer** ppPlayer,
                __in IXAVPlayerEvents* pEvents,
                __in_opt IXAudio2* pAudio2 = nullptr);


////////////////////////////////////////////////////////////////////////////////
//
// CreateXAVLicenseAcquisition
//
// Creates and returns a new instance of XAV License Acquisition,
// which implements IXAVLicenseAcquisition interface.
//
// Arguments:
//      ppLicenseAcquisition:   pointer to where the License Acquisition is returned.
//
// Return:
//      S_OK on success, error otherwise.
//
// Remarks:
//      The caller must call Release on the returned interface
//      when they are done using it.
//
////////////////////////////////////////////////////////////////////////////////

XBOXAPI
HRESULT
WINAPI
CreateXAVLicenseAcquisition(__out IXAVLicenseAcquisition **ppLicenseAcquisition);

////////////////////////////////////////////////////////////////////////////////
//
// CreateXAVLicenseAcquisition
//
// Obtains an instance of XAV License Acquisition,
// which implements IXAVLicenseAcquisition interface,
// from the IUnknown *pUnkDetail member of XAVPlayerAlert.
//
// Arguments:
//      pUnkDetail              pointer to IUnknown *pUnkDetail member of XAVPlayerAlert.
//      ppLicenseAcquisition:   pointer to where the License Acquisition is returned.
//
// Return:
//      S_OK on success, error otherwise.
//
// Remarks:
//      The caller must call Release on the returned interface
//      when they are done using it.
//
////////////////////////////////////////////////////////////////////////////////

XBOXAPI
HRESULT
WINAPI
CreateXAVLicenseAcquisition(__in  IUnknown                *pUnkDetail,
                            __out IXAVLicenseAcquisition **ppLicenseAcquisition);

#pragma warning( pop )

#if defined(_PREFAST_)
#pragma pop_macro("nullptr")
#endif // _PREFAST_
