
/***************************************************************************
 *
 *  XAvatar.h
 *
 *  This module defines the Xbox Avatar interface
 *
 *  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 *
 ***************************************************************************/


#pragma once

#ifndef _XAVATAR_H_
#define _XAVATAR_H_

#include <sal.h>
#include <xboxmath.h>
#include <xavatartypes.h>


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// XAvatar Type Definitions
//-----------------------------------------------------------------------------

//
// Avatar interface definitions
//

typedef struct IXAvatarRenderer*            LPXAVATARRENDERER;
typedef struct IXAvatarAnimation*           LPXAVATARANIMATION;
typedef struct IXAvatarNuiMapper*           LPXAVATARNUIMAPPER;

//
// Vertex structure, including all vertex data.
//

typedef struct _XAVATAR_VERTEX
{
    XMFLOAT3                                Position;
    XMHENDN3                                Normal;
    XMUBYTEN4                               Weights;
    XMUBYTE4                                Bindings;
    XMCOLOR                                 Color;
    XMHALF2                                 Uv0;

} XAVATAR_VERTEX, *PXAVATAR_VERTEX;

//
// Texture flags.
//

typedef DWORD XAVATAR_TEXTURE_FLAGS;

#define XAVATAR_TEXTURE_FLAGS_NONE          0x00000000
#define XAVATAR_TEXTURE_FLAGS_WRAP_U        0x00000001
#define XAVATAR_TEXTURE_FLAGS_WRAP_V        0x00000002


// 
// A constant defining an invalid UV index. This is used in the
// uv-index field of the texture instance structure below when a
// texture has no UVs associated with it for this instance.
//

static CONST WORD XAVATAR_INVALID_UV_INDEX = 0xffff;

//
// Texture instance description.
//

typedef struct _XAVATAR_TEXTURE_INSTANCE
{
    WORD                                    Index;
    WORD                                    UvIndex;
    XAVATAR_TEXTURE_FLAGS                   Flags;

} XAVATAR_TEXTURE_INSTANCE, *PXAVATAR_TEXTURE_INSTANCE;

//
// Shader constant value.
//

typedef struct _XAVATAR_SHADER_CONSTANT_VALUE
{
    FLOAT                                   Value[4];

} XAVATAR_SHADER_CONSTANT_VALUE, *PXAVATAR_SHADER_CONSTANT_VALUE;

//
// Shader parameter data (constant/texture).
//

typedef union _XAVATAR_SHADER_PARAM_DATA
{
    XAVATAR_SHADER_CONSTANT_VALUE           Constant;
    XAVATAR_TEXTURE_INSTANCE                Texture;

} XAVATAR_SHADER_PARAM_DATA;

//
// Shader parameter type.
//

typedef enum _XAVATAR_SHADER_PARAM_TYPE
{
    XAVATAR_SHADER_PARAM_TYPE_NONE            = 0,
    XAVATAR_SHADER_PARAM_TYPE_TEXTURE         = 1,
    XAVATAR_SHADER_PARAM_TYPE_VERTEX_CONSTANT = 2,
    XAVATAR_SHADER_PARAM_TYPE_PIXEL_CONSTANT  = 3,
    XAVATAR_SHADER_PARAM_TYPE_FORCE_DWORD     = 0x7fffffff

} XAVATAR_SHADER_PARAM_TYPE, *PXAVATAR_SHADER_PARAM_TYPE;


//
// Shader parameter usage identifiers.
//

typedef enum _XAVATAR_SHADER_PARAM_USAGE
{
    XAVATAR_SHADER_PARAM_USAGE_NONE                                 =  0,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_COLOR                        =  1,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_INTENSITY                    =  2,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_DECAL                        =  3,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_REFLECTION                   =  4,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_SKIN_FEATURES                =  5,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_FACIAL_HAIR                  =  6,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_EYEBROW_LEFT                 =  7,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_EYEBROW_RIGHT                =  8,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_EYE_LEFT                     =  9,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_EYE_RIGHT                    = 10,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_EYE_SHADOW                   = 11,
    XAVATAR_SHADER_PARAM_USAGE_TEXTURE_MOUTH                        = 12,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_SKIN            = 13,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_HAIR            = 14,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_MOUTH           = 15,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_IRIS            = 16,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_EYEBROW         = 17,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_EYE_SHADOW      = 18,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_FACIAL_HAIR     = 19,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_SKIN_FEATURE_1  = 20,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_SKIN_FEATURE_2  = 21,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_CUSTOM_0        = 22,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_CUSTOM_1        = 23,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_CUSTOM_2        = 24,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_REFLECTIVITY          = 25,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_TRANSPARENCY          = 26,
    XAVATAR_SHADER_PARAM_USAGE_PIXEL_CONSTANT_COLOR_RIMLIGHT        = 27,
    XAVATAR_SHADER_PARAM_USAGE_COUNT                                = 28,
    XAVATAR_SHADER_PARAM_USAGE_FORCE_DWORD                          = 0x7fffff

} XAVATAR_SHADER_PARAM_USAGE, *PXAVATAR_SHADER_PARAM_USAGE;

//
// Shader parameter, including type and data.
//

typedef struct _XAVATAR_SHADER_PARAM
{
    XAVATAR_SHADER_PARAM_TYPE               Type;
    XAVATAR_SHADER_PARAM_USAGE              Usage;
    XAVATAR_SHADER_PARAM_DATA               Data;

} XAVATAR_SHADER_PARAM, *PXAVATAR_SHADER_PARAM;

// 
// Shader enumeration.
//

typedef enum _XAVATAR_SHADER
{
    XAVATAR_SHADER_BODY_OPAQUE              = 0,
    XAVATAR_SHADER_BODY_TRANSPARENT         = 1,
    XAVATAR_SHADER_BODY_SHINY_OPAQUE        = 2,
    XAVATAR_SHADER_BODY_SHINY_TRANSPARENT   = 3,
    XAVATAR_SHADER_HEAD_OPAQUE              = 4,
    XAVATAR_SHADER_COUNT                    = 5,
    XAVATAR_SHADER_FORCE_DWORD              = 0x7fffffff

} XAVATAR_SHADER, *PXAVATAR_SHADER;

//
// The maximum number of parameters associated with an Avatar
// shader instance.
//

#define XAVATAR_SHADER_INSTANCE_MAX_PARAMS  20

//
// Shader instance, including shader enum and parameters.
//

typedef struct _XAVATAR_SHADER_INSTANCE
{
    XAVATAR_SHADER                          Shader;
    XAVATAR_SHADER_PARAM                    Params[ XAVATAR_SHADER_INSTANCE_MAX_PARAMS ];

} XAVATAR_SHADER_INSTANCE, *PXAVATAR_SHADER_INSTANCE;

//
// Triangle batch.
//

typedef struct _XAVATAR_TRIANGLE_BATCH
{
    XAVATAR_SHADER_INSTANCE                 ShaderInstance;
    DWORD                                   TriangleCount;
    DWORD                                   VertexCount;
    DWORD                                   VertexUvCount;
    DWORD                                   VertexStride;
    DWORD                                   IndexStride;
    BYTE*                                   pVertices;
    BYTE*                                   pIndices;

} XAVATAR_TRIANGLE_BATCH, *PXAVATAR_TRIANGLE_BATCH;

//
// Texture data.
//

typedef struct _XAVATAR_TEXTURE
{
    D3DFORMAT                               Format;
    DWORD                                   Width;
    DWORD                                   Height;
    DWORD                                   TotalBaseSize;
    DWORD                                   TotalMipSize;
    DWORD                                   BaseSize;
    DWORD                                   MipSize;
    DWORD                                   MipLevels;
    DWORD                                   LayerCount;
    BYTE*                                   pBaseData;
    BYTE*                                   pMipData;

} XAVATAR_TEXTURE, *PXAVATAR_TEXTURE;


//
// Component count.
//

#define XAVATAR_COMPONENT_COUNT             13


//
// Component masks.
//

typedef WORD XAVATAR_COMPONENT_MASK;

#define XAVATAR_COMPONENT_MASK_NONE                 0x0000
#define XAVATAR_COMPONENT_MASK_HEAD                 0x0001
#define XAVATAR_COMPONENT_MASK_BODY                 0x0002
#define XAVATAR_COMPONENT_MASK_HAIR                 0x0004
#define XAVATAR_COMPONENT_MASK_SHIRT                0x0008
#define XAVATAR_COMPONENT_MASK_TROUSERS             0x0010
#define XAVATAR_COMPONENT_MASK_SHOES                0x0020
#define XAVATAR_COMPONENT_MASK_HAT                  0x0040
#define XAVATAR_COMPONENT_MASK_GLOVES               0x0080
#define XAVATAR_COMPONENT_MASK_GLASSES              0x0100
#define XAVATAR_COMPONENT_MASK_WRISTWEAR            0x0200
#define XAVATAR_COMPONENT_MASK_EARRINGS             0x0400
#define XAVATAR_COMPONENT_MASK_RING                 0x0800
#define XAVATAR_COMPONENT_MASK_CARRYABLE            0x1000
#define XAVATAR_COMPONENT_MASK_ALL                  0x1FFF

//
// Component information.
//

typedef struct _XAVATAR_COMPONENT_INFO
{
    XAVATAR_ASSET_ID                        ModelAssetId;
    XAVATAR_COMPONENT_MASK                  ComponentMask;
    DWORD                                   Reserved[3];

} XAVATAR_COMPONENT_INFO, *PXAVATAR_COMPONENT_INFO;


//
// Body type masks.
//

typedef BYTE XAVATAR_BODY_TYPE;
typedef BYTE XAVATAR_BODY_TYPE_MASK;

#define XAVATAR_BODY_TYPE_UNKNOWN           0x00
#define XAVATAR_BODY_TYPE_MALE              0x01
#define XAVATAR_BODY_TYPE_FEMALE            0x02
#define XAVATAR_BODY_TYPE_ALL               0x03


//
// Model instance.
//

typedef struct _XAVATAR_MODEL
{
    DWORD                                   CpuMemoryBufferSize;
    DWORD                                   GpuMemoryBufferSize;
    DWORD                                   TotalTextureDataSize;
    DWORD                                   GlobalVertexBufferSize;
    DWORD                                   GlobalIndexBufferSize;
    DWORD                                   BatchCount;
    DWORD                                   TextureCount;

    BYTE*                                   pCpuMemoryBuffer;
    BYTE*                                   pGpuMemoryBuffer;
    BYTE*                                   pGlobalVertexBuffer;
    BYTE*                                   pGlobalIndexBuffer;
    XAVATAR_TRIANGLE_BATCH*                 pBatches;
    XAVATAR_TEXTURE*                        pTextures;

} XAVATAR_MODEL, *PXAVATAR_MODEL;

//
// Coordinate system.
//

typedef enum _XAVATAR_COORDINATE_SYSTEM
{
    XAVATAR_COORDINATE_SYSTEM_LEFT_HANDED    = 0,
    XAVATAR_COORDINATE_SYSTEM_RIGHT_HANDED   = 1,
    XAVATAR_COORDINATE_SYSTEM_FORCE_DWORD    = 0x7fffffff

} XAVATAR_COORDINATE_SYSTEM, *PXAVATAR_COORDINATE_SYSTEM;

//
// Defines the position, rotation, and scale of a joint relative 
// to the parent of that joint in a pose (eg, an animated keyframe).
//

typedef struct _XAVATAR_SKELETON_POSE_JOINT
{
    XMVECTOR                                Position;   // Local position (XYZ).
    XMVECTOR                                Rotation;   // Local rotation (XYZW).
    XMVECTOR                                Scale;      // Local scale    (XYZ).

} XAVATAR_SKELETON_POSE_JOINT, *PXAVATAR_SKELETON_POSE_JOINT;

//
// Holds the bind pose position for a skeleton joint, including the
// world position and rotation, and the local transform of the joint
// relative to its parent joint.
//

typedef struct _XAVATAR_SKELETON_BINDPOSE_JOINT
{
    XMVECTOR                                Position;   // World position (XYZ).
    XMVECTOR                                Rotation;   // World rotation (XYZW).
    XAVATAR_SKELETON_POSE_JOINT             Local;      // Local transform.

} XAVATAR_SKELETON_BINDPOSE_JOINT, *PXAVATAR_SKELETON_BINDPOSE_JOINT;

//
// Holds hierarchy information for a skeleton joint.
//

typedef struct _XAVATAR_SKELETON_HIERARCHY_JOINT
{
    DWORD                                   Parent;     // Parent joint index.
    DWORD                                   Child;      // First child joint index.
    DWORD                                   Sibling;    // Next sibling joint index.

} XAVATAR_SKELETON_HIERARCHY_JOINT, *PXAVATAR_SKELETON_HIERARCHY_JOINT;

//
// Holds hierarchy and bind pose information for a joint in a skeleton.
//

typedef struct _XAVATAR_SKELETON_JOINT
{
    XAVATAR_SKELETON_HIERARCHY_JOINT        Hierarchy;  // Parent/child/sibling.
    XAVATAR_SKELETON_BINDPOSE_JOINT         BindPose;   // BindPose transforms.

} XAVATAR_SKELETON_JOINT, *PXAVATAR_SKELETON_JOINT;

//
// Skeleton definition. The skeleton consists of an array of joint data,
// each holding hierarchy (ie, relationship) data, and world and local
// transforms for the joint in bind pose.
//

typedef struct _XAVATAR_SKELETON
{
    DWORD                                   Count;
    XAVATAR_SKELETON_JOINT*                 pJoints;

} XAVATAR_SKELETON, *PXAVATAR_SKELETON;

//
// Standard joint names for the avatar skeleton (NOTE: These joint
// names only apply to the avatar itself.  Carryables have their own
// skeleton)
//

typedef enum _XAVATAR_JOINT_INDEX
{
    BASE__Skeleton = 0,
    BACKA__Skeleton,
    LF_H__Skeleton,
    RT_H__Skeleton,
    SC_BASE__Skeleton,
    BACKB__Skeleton,
    LF_K__Skeleton,
    LF_SC_H__Skeleton,
    RT_K__Skeleton,
    RT_SC_H__Skeleton,
    SC_BACKA__Skeleton,
    LF_A__Skeleton,
    LF_C__Skeleton,
    LF_SC_K__Skeleton,
    NECK__Skeleton,
    RT_A__Skeleton,
    RT_C__Skeleton,
    RT_SC_K__Skeleton,
    SC_BACKB__Skeleton,
    HEAD__Skeleton,
    LF_S__Skeleton,
    LF_T__Skeleton,
    RT_S__Skeleton,
    RT_T__Skeleton,
    SC_NECK__Skeleton,
    LF_E__Skeleton,
    LF_SC_S__Skeleton,
    LF_SC_TWIST_S__Skeleton,
    RT_E__Skeleton,
    RT_SC_S__Skeleton,
    RT_SC_TWIST_S__Skeleton,
    LF_E_TWIST__Skeleton,
    LF_SC_E__Skeleton,
    LF_W__Skeleton,
    RT_E_TWIST__Skeleton,
    RT_SC_E__Skeleton,
    RT_W__Skeleton,
    LF_FINGA__Skeleton,
    LF_FINGB__Skeleton,
    LF_FINGC__Skeleton,
    LF_FINGD__Skeleton,
    LF_PROP__Skeleton,
    LF_SPECIAL__Skeleton,
    LF_THUMB__Skeleton,
    RT_FINGA__Skeleton,
    RT_FINGB__Skeleton,
    RT_FINGC__Skeleton,
    RT_FINGD__Skeleton,
    RT_PROP__Skeleton,
    RT_SPECIAL__Skeleton,
    RT_THUMB__Skeleton,
    LF_FINGA1__Skeleton,
    LF_FINGB1__Skeleton,
    LF_FINGC1__Skeleton,
    LF_FINGD1__Skeleton,
    LF_THUMB1__Skeleton,
    RT_FINGA1__Skeleton,
    RT_FINGB1__Skeleton,
    RT_FINGC1__Skeleton,
    RT_FINGD1__Skeleton,
    RT_THUMB1__Skeleton,
    LF_FINGA2__Skeleton,
    LF_FINGB2__Skeleton,
    LF_FINGC2__Skeleton,
    LF_FINGD2__Skeleton,
    LF_THUMB2__Skeleton,
    RT_FINGA2__Skeleton,
    RT_FINGB2__Skeleton,
    RT_FINGC2__Skeleton,
    RT_FINGD2__Skeleton,
    RT_THUMB2__Skeleton,

} XAVATAR_JOINT_INDEX, *PXAVATAR_JOINT_INDEX;


//
// Waiting effect instance. A waiting effect can be rendered when
// an avatar, or its metadata, is still being loaded.
//

typedef struct _XAVATAR_WAITING_EFFECT
{
    DWORD                                   Reserved[4];    // Reserved data members
    
} XAVATAR_WAITING_EFFECT, *PXAVATAR_WAITING_EFFECT;

//
// Defines the various play modes for an animation.
//

typedef enum _XAVATAR_ANIMATION_PLAYMODE
{
    XAVATAR_ANIMATION_PLAYMODE_ONCE         = 0,
    XAVATAR_ANIMATION_PLAYMODE_LOOP         = 1,
    XAVATAR_ANIMATION_PLAYMODE_BOUNCE       = 2,
    XAVATAR_ANIMATION_PLAYMODE_FORCE_DWORD  = 0x7fffffff

} XAVATAR_ANIMATION_PLAYMODE, *PXAVATAR_ANIMATION_PLAYMODE;

//
// Defines the various flags for IXAvatarRenderer::PlayAnimations().
//

typedef DWORD XAVATAR_PLAYANIMATIONS_FLAGS;

#define XAVATAR_PLAYANIMATIONS_FLAGS_DEFAULT                0x00000000
#define XAVATAR_PLAYANIMATIONS_FLAGS_WAIT_FOR_CURRENT_ANIM  0x00000001
#define XAVATAR_PLAYANIMATIONS_FLAGS_LOOP_LAST_ANIM         0x00000002
#define XAVATAR_PLAYANIMATIONS_FLAGS_SNAP_FIRST_ANIM        0x00000004
#define XAVATAR_PLAYANIMATIONS_FLAGS_NO_CARRYABLE_IDLES     0x00000008
#define XAVATAR_PLAYANIMATIONS_FLAGS_CARRYABLE_IDLES_ONLY   0x00000010


//
// Animated texture channel indices.
//

typedef enum _XAVATAR_ANIMATED_TEXTURE
{
    XAVATAR_ANIMATED_TEXTURE_MOUTH          = 0,
    XAVATAR_ANIMATED_TEXTURE_EYEBROW_LEFT   = 1,
    XAVATAR_ANIMATED_TEXTURE_EYEBROW_RIGHT  = 2,
    XAVATAR_ANIMATED_TEXTURE_EYE_LEFT       = 3,
    XAVATAR_ANIMATED_TEXTURE_EYE_RIGHT      = 4,
    XAVATAR_ANIMATED_TEXTURE_COUNT          = 5,
    XAVATAR_ANIMATED_TEXTURE_FORCE_DWORD    = 0x7fffffff
    
} XAVATAR_ANIMATED_TEXTURE, *PXAVATAR_ANIMATED_TEXTURE;

//
// Valid values for the mouth animated texture channel.
//

typedef enum _XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH
{
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_NEUTRAL        =  0,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_SAD            =  1,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_ANGRY          =  2,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_CONFUSED       =  3,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_LAUGHING       =  4,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_SHOCKED        =  5,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_HAPPY          =  6,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_O     =  7,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_AI    =  8,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_EE    =  9,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_FV    = 10,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_W     = 11,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_L     = 12,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_PHONETIC_DTH   = 13,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_COUNT          = 14,
    XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH_FORCE_DWORD    = 0x7fffffff
    
} XAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH, *PXAVATAR_ANIMATED_TEXTURE_LAYER_MOUTH;

//
// Valid values for an eyebrow animated texture channel.
//

typedef enum _XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW
{
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_NEUTRAL      = 0,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_SAD          = 1,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_ANGRY        = 2,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_CONFUSED     = 3,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_RAISED       = 4,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_COUNT        = 5,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW_FORCE_DWORD  = 0x7fffffff
    
} XAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW, *PXAVATAR_ANIMATED_TEXTURE_LAYER_EYEBROW;

//
// Valid values for an eye animated texture channel.
//

typedef enum _XAVATAR_ANIMATED_TEXTURE_LAYER_EYE
{
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_NEUTRAL          =  0,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_SAD              =  1,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_ANGRY            =  2,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_CONFUSED         =  3,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_LAUGHING         =  4,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_SHOCKED          =  5,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_HAPPY            =  6,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_YAWNING          =  7,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_SLEEPING         =  8,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_LOOK_UP          =  9,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_LOOK_DOWN        = 10,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_LOOK_OUTER       = 11,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_LOOK_INNER       = 12,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_BLINK            = 13,
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_COUNT            = 14,    
    XAVATAR_ANIMATED_TEXTURE_LAYER_EYE_FORCE_DWORD      = 0x7fffffff
    
} XAVATAR_ANIMATED_TEXTURE_LAYER_EYE, *PXAVATAR_ANIMATED_TEXTURE_LAYER_EYE;

// 
// Structure that holds animated motion data. Motion data
// consists of a position and rotation for the motion.
//

typedef struct _XAVATAR_MOTION_DATA
{
    XMVECTOR    Position;
    XMVECTOR    Rotation;

} XAVATAR_MOTION_DATA, *PXAVATAR_MOTION_DATA;

//
// Valid animated motion context indices.
//

typedef enum _XAVATAR_MOTION_CONTEXT
{
    XAVATAR_MOTION_CONTEXT_BASE                     = 0,
    XAVATAR_MOTION_CONTEXT_FOOT_LEFT                = 1,
    XAVATAR_MOTION_CONTEXT_FOOT_RIGHT               = 2,
    XAVATAR_MOTION_CONTEXT_COUNT                    = 3,
    XAVATAR_MOTION_CONTEXT_FORCE_DWORD              = 0x7fffffff

} XAVATAR_MOTION_CONTEXT, *PXAVATAR_MOTION_CONTEXT;

//
// Valid animated motion modes.
//

typedef enum _XAVATAR_MOTION_MEASUREMODE
{
    XAVATAR_MOTION_MEASUREMODE_ABSOLUTE             = 0,
    XAVATAR_MOTION_MEASUREMODE_RELATIVE             = 1,
    XAVATAR_MOTION_MEASUREMODE_COUNT                = 2,
    XAVATAR_MOTION_MEASUREMODE_FORCE_DWORD          = 0x7fffffff
    
} XAVATAR_MOTION_MEASUREMODE, *PXAVATAR_MOTION_MEASUREMODE;

//
// Defines the various views that can be supplied to
// IXAvatarRenderer::GetMatrices() in order to have the renderer
// supply useful model/view matrices for use in Render calls.
//

typedef enum _XAVATAR_RENDER_VIEW
{
    XAVATAR_RENDER_VIEW_FULL_BODY           = 0,
    XAVATAR_RENDER_VIEW_HEAD                = 1,
    XAVATAR_RENDER_VIEW_HEAD_AND_SHOULDERS  = 2,
    XAVATAR_RENDER_VIEW_COUNT               = 3,
    XAVATAR_RENDER_VIEW_FORCE_DWORD         = 0x7fffffff

} XAVATAR_RENDER_VIEW, *PXAVATAR_RENDER_VIEW;

//
// Defines a few preset level-of-detail settings for renderers. 
//

#define XAVATAR_RENDER_LOD_MINIMUM           0
#define XAVATAR_RENDER_LOD_LOW              19
#define XAVATAR_RENDER_LOD_MEDIUM           37
#define XAVATAR_RENDER_LOD_MAXIMUM          XAVATAR_MAX_SKELETON_JOINTS

//
// Defines the maximum number of joints in an avatar skeleton and
// a special "invalid joint" index.
//

static CONST DWORD XAVATAR_MAX_SKELETON_JOINTS    =    72;
static CONST DWORD XAVATAR_MAX_TEXTURES_PER_MODEL =    18;
static CONST DWORD XAVATAR_MAX_LAYERS_PER_TEXTURE =    16;
static CONST DWORD XAVATAR_MAX_TEXTURE_SIZE       = 65536;
static CONST DWORD XAVATAR_INVALID_JOINT_INDEX    =   255;
static CONST DWORD XAVATAR_MAX_JOINT_BUFFERS      =     4;

//
// Defines the valid sizes for a renderer shadow map.
//

typedef enum _XAVATAR_SHADOW_SIZE
{
    XAVATAR_SHADOW_SIZE_NONE                = 0,
    XAVATAR_SHADOW_SIZE_SMALL               = 1,
    XAVATAR_SHADOW_SIZE_LARGE               = 2,
    XAVATAR_SHADOW_SIZE_FORCE_DWORD         = 0x7fffffff

} XAVATAR_SHADOW_SIZE, *PXAVATAR_SHADOW_SIZE;

//
// Defines the animation cursor.  The Time, Speed, PlayMode and MotionMode 
// members are set by the IXAvatarAnimation::InitializeCursor and 
// IncrementCursor methods and read by the calling renderer.  The four 
// Reserved members may store any context required by the animation and will 
// not be referenced by the renderer.
//
// During InitializeCursor, an animation would normally just copy the Speed,
// PlayMode and MotionMode arguments directly to the cursor structure and
// set Time to 0, but it may elect to modify any of them.
//
// During IncrementCursor, an animation would typically increment Time by
// adding the result of the cursor's Speed member multiplied by the Seconds
// argument passed to IncremeentCursor, but again, it may elect to calculate
// Time differently.
//
// A pointer to the cursor structure should not be kept beyond the duration
// of the IXAvatarAnimation method that received it.
//

typedef struct _XAVATAR_ANIMATION_CURSOR
{
    FLOAT                       Time;
    FLOAT                       Speed;
    XAVATAR_ANIMATION_PLAYMODE  PlayMode;
    XAVATAR_MOTION_MEASUREMODE  MotionMode;
    DWORD                       Reserved0;
    DWORD                       Reserved1;
    DWORD                       Reserved2;
    DWORD                       Reserved3;

} XAVATAR_ANIMATION_CURSOR, *PXAVATAR_ANIMATION_CURSOR;


// 
// Defines the maximum axis-aligned bounding box for an avatar. This
// is enforced in the asset converters to ensure that no public model
// or animation will ever exceed these model-space bounds. Because the
// bounds are symmetric around the x- and z-planes, they are valid for
// both left- and right-handed coordinate systems.
//

#define XAVATAR_MAXIMUM_BOUNDING_BOX_MIN    ( XMVectorSet( -1.1f, -0.2f, -1.1f, 1.0f ) )
#define XAVATAR_MAXIMUM_BOUNDING_BOX_MAX    ( XMVectorSet( +1.1f, +2.2f, +1.1f, 1.0f ) )

//
// AvatarNuiMapper enumerations
//

typedef enum _XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE
{
    XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE_CONFIDENT               = 0,
    XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE_LOW_CONFIDENCE_SKELETON = 1,
    XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE_LOW_CONFIDENCE_JOINT    = 2,
    XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE_NOT_RETARGETED          = 3,
    XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE_FORCE_DWORD             = 0x7fffffff
} XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE, *PXAVATAR_NUI_MAPPER_JOINT_CONFIDENCE;

typedef enum _XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS
{
    XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS_OFF                = 0,
    XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS_MINOR              = 1,
    XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS_MAJOR              = 2,
    XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS_FORCE_DWORD        = 0x7fffffff
} XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS, *PXAVATAR_NUI_MAPPER_FIX_RAISED_ARMS;

//
// AvatarNuiMapper structures
//

typedef struct _XAVATAR_NUI_MAPPER_GENERAL_OPTIONS
{
    BOOL                               MirrorSkeleton;
    BOOL                               PreventSelfIntersection;
    BOOL                               RescaleToUserProportions;
    BOOL                               SkeletonDataIsFiltered;
    BOOL                               DisableJumping;
} XAVATAR_NUI_MAPPER_GENERAL_OPTIONS;

typedef struct _XAVATAR_NUI_MAPPER_FIXUP_OPTIONS
{
    BOOL                               IdleOnJitteryData;
    BOOL                               IdleOnSidePoses;
    BOOL                               IdleOnLeaningToSide;
    BOOL                               PreventFeetSliding;
    XAVATAR_NUI_MAPPER_FIX_RAISED_ARMS ProceduralRaisedArms;
    float                              ProceduralCrouchingThreshold;
    float                              ProceduralRaisedLegsThreshold;
} XAVATAR_NUI_MAPPER_FIXUP_OPTIONS;

typedef struct _XAVATAR_NUI_MAPPER_OPTIONS
{
    XAVATAR_NUI_MAPPER_GENERAL_OPTIONS GeneralOptions;
    XAVATAR_NUI_MAPPER_FIXUP_OPTIONS   FixupOptions;
} XAVATAR_NUI_MAPPER_OPTIONS;

typedef struct _XAVATAR_NUI_MAPPER_TRANSFORM_INFO
{
    XMVECTOR                        Position;
    FLOAT                           YawInRadians;
    FLOAT                           ScaleRelativeToPlayer;
} XAVATAR_NUI_MAPPER_TRANSFORM_INFO;


//
// Animation asset opaque blob.
//

typedef void  XAVATAR_ANIMATION_ASSET;
typedef void* PXAVATAR_ANIMATION_ASSET;


//
// Avatar carryable asset data. 
//

typedef struct _XAVATAR_CARRYABLE
{
    XAVATAR_SKELETON*                       pSkeleton;
    XAVATAR_COMPONENT_INFO                  ComponentInfo;
    XAVATAR_MODEL                           ComponentModel;
    XAVATAR_ANIMATION_ASSET*                pAnimationAsset;

} XAVATAR_CARRYABLE, *PXAVATAR_CARRYABLE;

//
// Avatar assets. Holds all of the information about a single avatar.
//

typedef struct _XAVATAR_ASSETS
{
    XAVATAR_SKELETON*                       pSkeleton;
    XAVATAR_CARRYABLE*                      pCarryable;
    DWORD                                   ComponentCount;
    XAVATAR_COMPONENT_INFO*                 pComponentInfo;
    XAVATAR_MODEL*                          pComponentModels;

} XAVATAR_ASSETS, *PXAVATAR_ASSETS;


//
// The maximum number of assets allowed per avatar.
//
static CONST DWORD XAVATAR_MAX_ASSETS_PER_AVATAR        = 30;

//
// The maximum number of animations that can be supplied to a call to IXAvatarRenderer::PlayAnimations().
//
static CONST DWORD XAVATAR_RENDER_MAX_ANIMATIONS        = 8;


//
// Flags for XAvatarInitialize()
//

typedef DWORD XAVATAR_INITIALIZE_FLAGS;

#define XAVATAR_INITIALIZE_FLAGS_ENABLERENDERER          0x00000001
#define XAVATAR_INITIALIZE_FLAGS_ENABLEPLACEHOLDERASSETS 0x00000002


#define XAVATAR_INITIALIZE_DEFAULT_ASSET_LOAD_THREAD     5

//
// Flags that control how avatar assets are loaded.
//

typedef DWORD XAVATAR_ASSET_LOAD_FLAGS;

#define XAVATAR_ASSET_LOAD_BACKGROUND_TASK      0x00000001


//-----------------------------------------------------------------------------
// XAvatar Specific Error Codes
//-----------------------------------------------------------------------------

#define FACILITY_XAVATAR                        0x99

//
// MessageId: XAVATAR_E_USER_HAS_NO_AVATAR
//
// MessageText:
//
// The user has not yet created an avatar for their profile
//

#define XAVATAR_E_USER_HAS_NO_AVATAR            MAKE_HRESULT(SEVERITY_ERROR,    FACILITY_XAVATAR, 1)

//
// MessageId: XAVATAR_E_IDLE_ANIMATION_PLAYING
//
// MessageText:
//
// The idle animation is currently playing
//

#define XAVATAR_E_IDLE_ANIMATION_PLAYING        MAKE_HRESULT(SEVERITY_ERROR,    FACILITY_XAVATAR, 2)

//
// MessageId: XAVATAR_E_CARRYABLE_IDLE_ANIMATION_PLAYING
//
// MessageText:
//
// The carryable animation is currently playing
//

#define XAVATAR_E_CARRYABLE_IDLE_ANIMATION_PLAYING   MAKE_HRESULT(SEVERITY_ERROR,    FACILITY_XAVATAR, 3)

//-----------------------------------------------------------------------------
// XAvatar Core APIs
//-----------------------------------------------------------------------------

//
// Initializes the avatar system.
//

_Check_return_
XBOXAPI
HRESULT
WINAPI
XAvatarInitialize(
    _In_    XAVATAR_COORDINATE_SYSTEM           CoordinateSystem,
    _In_    XAVATAR_INITIALIZE_FLAGS            Flags,
    _In_    DWORD                               AssetLoadHardwareThread,
    _In_    DWORD                               JointsBufferCount,
    _In_opt_ LPDIRECT3DDEVICE9                  pDevice
    );

//
// Shuts down the avatar system.
//

XBOXAPI
VOID 
WINAPI
XAvatarShutdown(
    VOID
    );

//-----------------------------------------------------------------------------
// XAvatar Metadata Retrieval APIs
//-----------------------------------------------------------------------------

XBOXAPI
DWORD
WINAPI
XAvatarGetMetadataLocalUser(
    _In_    DWORD                               UserIndex,
    _Out_   XAVATAR_METADATA*                   pAvatarMetadata,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );


XBOXAPI
DWORD
WINAPI
XAvatarGetMetadataRandom(
    _In_    XAVATAR_BODY_TYPE_MASK              BodyTypeMask,
    _In_    DWORD                               AvatarCount,
    _Out_cap_(AvatarCount) XAVATAR_METADATA*    pAvatarMetadata,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    ); 

XBOXAPI
DWORD
WINAPI
XAvatarGetMetadataSignedOutProfileCount();

XBOXAPI
DWORD
WINAPI
XAvatarGetMetadataSignedOutProfile(
    _In_    DWORD                               ProfileIndex,
    _Out_   XAVATAR_METADATA*                   pAvatarMetadata,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );

XBOXAPI
XAVATAR_BODY_TYPE
WINAPI
XAvatarMetadataGetBodyType(
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata
    );

XBOXAPI
FLOAT
WINAPI
XAvatarMetadataGetHeight(
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata
    );

XBOXAPI
BOOL
WINAPI
XAvatarMetadataCompare(
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadataLhs,
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadataRhs
    );

XBOXAPI
DWORD
WINAPI
XAvatarMetadataGetComponentInfo( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
    _In_    BOOL                                ExactMatch,
    _In_    DWORD                               MaxItems,
    _Out_   XAVATAR_COMPONENT_INFO*             pComponents 
    );

XBOXAPI
HRESULT
WINAPI
XAvatarMetadataGetColors( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _Out_   XMCOLOR*                            pSkinColor,
    _Out_   XMCOLOR*                            pHairColor,
    _Out_   XMCOLOR*                            pMouthColor,
    _Out_   XMCOLOR*                            pIrisColor,
    _Out_   XMCOLOR*                            pEyebrowColor,
    _Out_   XMCOLOR*                            pEyeShadowColor,
    _Out_   XMCOLOR*                            pFacialHairColor,
    _Out_   XMCOLOR*                            pSkinFeature1Color,
    _Out_   XMCOLOR*                            pSkinFeature2Color 
    );

XBOXAPI
HRESULT
WINAPI
XAvatarMetadataGetBlendShapeIDs( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _Out_   XAVATAR_ASSET_ID*                   pChinBlendShapeID,
    _Out_   XAVATAR_ASSET_ID*                   pEarBlendShapeID,
    _Out_   XAVATAR_ASSET_ID*                   pNoseBlendShapeID 
    );

XBOXAPI
HRESULT
WINAPI
XAvatarMetadataGetDynamicTextureAssetIDs( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _Out_   XAVATAR_ASSET_ID*                   pMouthTextureID,
    _Out_   XAVATAR_ASSET_ID*                   pEyeTextureID,
    _Out_   XAVATAR_ASSET_ID*                   pEyeBrowTextureID,
    _Out_   XAVATAR_ASSET_ID*                   pFacialHairTextureID,
    _Out_   XAVATAR_ASSET_ID*                   pEyeShadowTextureID,
    _Out_   XAVATAR_ASSET_ID*                   pSkinFeatureTextureID
    );

XBOXAPI
BOOL
WINAPI
XAvatarMetadataIsAssetEquipped( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _In_    CONST XAVATAR_ASSET_ID*             pAssetId 
    );

XBOXAPI
FLOAT
WINAPI
XAvatarMetadataGetWeightFactor( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata 
    );

XBOXAPI
FLOAT
WINAPI
XAvatarMetadataGetHeightFactor( 
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata 
    );

//-----------------------------------------------------------------------------
// XAvatar Asset Retrieval APIs
//-----------------------------------------------------------------------------


//
// Returns the needed size of the buffers to supply to XAvatarGetAssets.
//

XBOXAPI
HRESULT
WINAPI
XAvatarGetAssetsResultSize(
    _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
    _Out_   DWORD*                              pResultBufferSize,
    _Out_   DWORD*                              pGpuResourceBufferSize
    );

//
// Retrieves avatar model data for an avatar as described by the
// supplied XAVATAR_METADATA
//

XBOXAPI
DWORD
WINAPI
XAvatarGetAssets(
    _In_    CONST XAVATAR_METADATA*             pAvatarMetadata,
    _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
    _In_    XAVATAR_ASSET_LOAD_FLAGS            AssetLoadFlags,
    _In_    DWORD                               ResultBufferSize,
    _Out_bytecap_(ResultBufferSize) XAVATAR_ASSETS* pResultBuffer,
    _In_    DWORD                               GpuResourceBufferSize,
    _Out_bytecap_(GpuResourceBufferSize) VOID*  pGpuResourceBuffer,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );

//
// Retrives placeholder model data for an avatar as described by
// the optionally supplied XAVATAR_METADATA. If no metadata is
// supplied then a generic placeholder is returned. If a custom
// color is specified, it is incoporated into the asset. If not
// then a standard color is used.
//

#define XAVATAR_PLACEHOLDER_ASSETS_SIZE ( 8 * 1024 )

XBOXAPI
DWORD
WINAPI
XAvatarGetPlaceholderAssets( 
    _In_opt_    CONST XAVATAR_METADATA*         pAvatarMetadata,
    _In_opt_    CONST XMCOLOR*                  pCustomColor,
    _In_        DWORD                           ResultBufferSize,
    _Out_bytecap_(ResultBufferSize) XAVATAR_ASSETS* pResultBuffer 
    );

//
// Builds mip maps for all of the textures in an avatar.
//

#define XAVATAR_MIPMAP_RECOMMENDED_BUFFER_SIZE ( 400 * 1024 )

XBOXAPI
DWORD
WINAPI
XAvatarGenerateMipMaps(
    _Inout_ XAVATAR_ASSETS*                     pAvatarAssets,
    _In_    XAVATAR_ASSET_LOAD_FLAGS            AssetLoadFlags,
    _In_    DWORD                               BufferSize,
    _Out_bytecap_(BufferSize) VOID*             pMipMapBuffer,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );


//-----------------------------------------------------------------------------
// XAvatar Animation APIs
//-----------------------------------------------------------------------------

//
// Stock animation assetIDs for use in XAvatarLoadAnimation().
//

DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_0,          0x00400000, 0x0003, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_1,          0x00400000, 0x0004, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_2,          0x00400000, 0x0005, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_3,          0x00400000, 0x0027, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_4,          0x00400000, 0x0028, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_5,          0x00400000, 0x0029, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_6,          0x00400000, 0x002A, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_STAND_7,          0x00400000, 0x002B, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_CLAP,             0x00400000, 0x0006, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_WAVE,             0x00400000, 0x0007, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_GENERIC_CELEBRATE,        0x00400000, 0x0008, 0x0003, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);

DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_IDLE_CHECK_NAILS,  0x00400000, 0x0009, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_IDLE_LOOK_AROUND,  0x00400000, 0x000A, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_IDLE_SHIFT_WEIGHT, 0x00400000, 0x000B, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_IDLE_FIXES_SHOE,   0x00400000, 0x000C, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_ANGRY,             0x00400000, 0x000D, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_CONFUSED,          0x00400000, 0x000E, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_LAUGH,             0x00400000, 0x000F, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_SAD_CRY,           0x00400000, 0x0010, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_SHOCKED_SURPRISED, 0x00400000, 0x0011, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_FEMALE_YAWN,              0x00400000, 0x0012, 0x0002, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);

DEFINE_GUID(XAVATAR_ANIMATION_MALE_IDLE_LOOK_AROUND,    0x00400000, 0x0013, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_IDLE_STRETCH,        0x00400000, 0x0014, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_IDLE_SHIFTS_WEIGHT,  0x00400000, 0x0015, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_IDLE_CHECKS_HAND,    0x00400000, 0x0026, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_ANGRY,               0x00400000, 0x0016, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_CONFUSED,            0x00400000, 0x0017, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_LAUGH,               0x00400000, 0x0018, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_SAD_CRY,             0x00400000, 0x0019, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_SHOCKED_SURPRISED,   0x00400000, 0x001A, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);
DEFINE_GUID(XAVATAR_ANIMATION_MALE_YAWN,                0x00400000, 0x001B, 0x0001, 0xC1, 0xC8, 0xF1, 0x09, 0xA1, 0x9C, 0xB2, 0xE0);


// 
// Load the specified animation. Always allocates a fixed amount of memory
// for the animation, regardless of the animation that is loaded.
//

XBOXAPI
HRESULT
WINAPI
XAvatarLoadAnimation(
    _In_    CONST XAVATAR_ASSET_ID *            pAssetID,
    _In_    XAVATAR_ASSET_LOAD_FLAGS            AssetLoadFlags,
    _Deref_out_ LPXAVATARANIMATION*             ppAnimation
    );

// 
// Load the specified animation. Always allocates precisely the amount of
// memory that is required for the animation in the buffer.
//

XBOXAPI
HRESULT
WINAPI
XAvatarLoadAnimationFromBuffer(
    _In_    DWORD                               FromBufferSize,
    _In_bytecount_(FromBufferSize) CONST VOID * pFromBuffer,
    _Deref_out_ LPXAVATARANIMATION*             ppAnimation
    );


//
// Use this function with the XAVATAR_ANIMATION_ASSET structure returned
// by XAvatarGetAssets(). If you are not using XAvatarGetAssets(), use the
// IXAvatarAnimation interface returned by XAvatarLoadAnimation() instead.
// Initializes an animation cursor. The cursor now points to the first 
// frame of the animation.
//
XBOXAPI
HRESULT
WINAPI
XAvatarAnimationAssetInitializeCursor(
    _In_    CONST XAVATAR_ANIMATION_ASSET*      pAsset,
    _In_    FLOAT                               Speed,
    _In_    XAVATAR_ANIMATION_PLAYMODE          PlayMode,
    _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
    _Out_   XAVATAR_ANIMATION_CURSOR*           pCursor 
    );

//
// Use this function with the XAVATAR_ANIMATION_ASSET structure returned
// by XAvatarGetAssets(). If you are not using XAvatarGetAssets(), use the
// IXAvatarAnimation interface returned by XAvatarLoadAnimation() instead.
// Increments an animation cursor. The cursor is incremented by the 
// specified time in seconds. The cursor must have been previously 
// initialized using XAvatarAnimationAssetInitializeCursor().
//
XBOXAPI
HRESULT
WINAPI
XAvatarAnimationAssetIncrementCursor(
    _In_    CONST XAVATAR_ANIMATION_ASSET*      pAsset,
    _In_    FLOAT                               Seconds,
    _Inout_ XAVATAR_ANIMATION_CURSOR*           pCursor
    );

//
// Use this function with the XAVATAR_ANIMATION_ASSET structure returned
// by XAvatarGetAssets(). If you are not using XAvatarGetAssets(), use the
// IXAvatarAnimation interface returned by XAvatarLoadAnimation() instead.
// Returns the length of the animation in seconds, and the number of joints,
// motion data contexts, and textures used by this animation. These can be
// used to size the buffers to be passed to XAvatarAnimationAssetGetPose(). 
// Alternatively, you can use the constant maximum sizes:
//  - XAVATAR_MAX_SKELETON_JOINTS       .. for joint buffers,
//  - XAVATAR_MOTION_CONTEXT_COUNT      .. for motion data,
//  - XAVATAR_ANIMATED_TEXTURE_COUNT    .. for texture layers.
//
XBOXAPI
HRESULT
WINAPI
XAvatarAnimationAssetGetAttributes(
    _In_    CONST XAVATAR_ANIMATION_ASSET*      pAsset,
    _Out_opt_ FLOAT*                            pLengthInSeconds,
    _Out_opt_ DWORD*                            pAvatarJointCount,
    _Out_opt_ DWORD*                            pCarryableJointCount,
    _Out_opt_ DWORD*                            pMotionDataCount,
    _Out_opt_ DWORD*                            pTextureCount
    );

//
// Use this function with the XAVATAR_ANIMATION_ASSET structure returned
// by XAvatarGetAssets(). If you are not using XAvatarGetAssets(), use the
// IXAvatarAnimation interface returned by XAvatarLoadAnimation() instead.
// Retrieves the current animation frame. The primary and secondary poses
// are returned as an array of local joint transforms, each relative to 
// the parent of that joint.
//
XBOXAPI
HRESULT
WINAPI
XAvatarAnimationAssetGetPose(
    _In_    CONST XAVATAR_ANIMATION_ASSET*      pAsset,
    _In_    CONST XAVATAR_ANIMATION_CURSOR*     pCursor,
    _In_    FLOAT                               BlendWeight,
    _In_    DWORD                               AvatarJointCount,
    _Inout_opt_cap_(AvatarJointCount) XAVATAR_SKELETON_POSE_JOINT* pAvatarJointPoseBuffer,
    _In_    DWORD                               CarryableJointCount,
    _Inout_opt_cap_(CarryableJointCount) XAVATAR_SKELETON_POSE_JOINT* pCarryableJointPoseBuffer,
    _In_    DWORD                               MotionDataCount,
    _Inout_opt_cap_(MotionDataCount) XAVATAR_MOTION_DATA* pMotionDataBuffer,
    _In_    DWORD                               TextureCount,
    _Inout_opt_cap_(TextureCount) DWORD*        pTextureLayerBuffer
    );

//
// Use this function with the XAVATAR_ANIMATION_ASSET structure returned
// by XAvatarGetAssets(). If you are not using XAvatarGetAssets(), use the
// IXAvatarAnimation interface returned by XAvatarLoadAnimation() instead.
// Calculates the bounding box for this animation if used on an avatar
// with the specified component mask. This allows us to give the minimal
// bounding box based on the components actually in use. For example, if
// the avatar has only a head then the bounding box will be significantly
// smaller than if it has both a head and body.
//
XBOXAPI
HRESULT
WINAPI
XAvatarAnimationAssetCalculateBoundingBox(
    _In_    CONST XAVATAR_ANIMATION_ASSET*      pAsset,
    _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
    _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
    _Out_   XMVECTOR*                           pBoundingBoxMin,
    _Out_   XMVECTOR*                           pBoundingBoxMax
    );

#undef  INTERFACE
#define INTERFACE   IXAvatarAnimation

DECLARE_INTERFACE(IXAvatarAnimation)
{
    STDMETHOD_(ULONG, AddRef)( THIS ) PURE;
    STDMETHOD_(ULONG, Release)( THIS ) PURE;

    //
    // Returns the status of the animation
    //
    STDMETHOD(GetStatus)( THIS ) PURE;

    //
    // Initializes an animation cursor. The cursor now points to
    // the first frame of the animation.
    //
    STDMETHOD(InitializeCursor)( 
        THIS_
        _In_    FLOAT                               Speed,
        _In_    XAVATAR_ANIMATION_PLAYMODE          PlayMode,
        _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
        _Out_   XAVATAR_ANIMATION_CURSOR*           pCursor
        ) PURE;

    //
    // Increments an animation cursor. The cursor is incremented
    // by the specified time in seconds. The cursor must have been
    // previously initialized using InitializeCursor().
    //
    STDMETHOD(IncrementCursor)(
        THIS_
        _In_    FLOAT                               Seconds,
        _Inout_ XAVATAR_ANIMATION_CURSOR*           pCursor
        ) PURE;

    //
    // Returns the length of the animation in seconds, and the number of
    // joints, motion data contexts, and textures used by this animation. 
    // These can be used to size the buffers to be passed to GetPose(). 
    // Alternatively, you can use the constant maximum sizes:
    //  - XAVATAR_MAX_SKELETON_JOINTS       .. for joint buffers,
    //  - XAVATAR_MOTION_CONTEXT_COUNT      .. for motion data,
    //  - XAVATAR_ANIMATED_TEXTURE_COUNT    .. for texture layers.
    //
    STDMETHOD(GetAttributes)(
        THIS_
        _Out_opt_   FLOAT*                          pLengthInSeconds,
        _Out_opt_   DWORD*                          pAvatarJointCount,
        _Out_opt_   DWORD*                          pCarryableJointCount,
        _Out_opt_   DWORD*                          pMotionDataCount,
        _Out_opt_   DWORD*                          pTextureCount
        ) PURE;

    //
    // Retrieves the current animation frame. The primary and secondary
    // poses are returned as an array of local joint transforms, each 
    // relative to the parent of that joint. 
    //
    STDMETHOD(GetPose)( 
        THIS_
        _In_    CONST XAVATAR_ANIMATION_CURSOR*     pCursor,
        _In_    FLOAT                               BlendWeight,
        _In_    DWORD                               AvatarJointCount,
        _Inout_opt_cap_(AvatarJointCount) XAVATAR_SKELETON_POSE_JOINT* pAvatarJointPoseBuffer,
        _In_    DWORD                               CarryableJointCount,
        _Inout_opt_cap_(CarryableJointCount) XAVATAR_SKELETON_POSE_JOINT* pCarryableJointPoseBuffer,
        _In_    DWORD                               MotionDataCount,
        _Inout_opt_cap_(MotionDataCount) XAVATAR_MOTION_DATA* pMotionDataBuffer,
        _In_    DWORD                               TextureCount,
        _Inout_opt_cap_(TextureCount) DWORD*        pTextureLayerBuffer
        ) PURE;
        
    //
    // Calculates the bounding box for this animation if used on an avatar
    // with the specified component mask. This allows us to give the minimal
    // bounding box based on the components actually in use. For example, if
    // the avatar has only a head then the bounding box will be significantly
    // smaller than if it has both a head and body.
    //
    STDMETHOD(CalculateBoundingBox)(
        THIS_
        _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
        _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
        _Out_   XMVECTOR*                           pBoundingBoxMin,
        _Out_   XMVECTOR*                           pBoundingBoxMax
        ) PURE;
};

//-----------------------------------------------------------------------------
// XAvatar Renderer APIs
//-----------------------------------------------------------------------------


//
// Starts a waiting effect. Must be called once on a waiting effect before used in 
// XAvatarRenderWaitingEffect.
//

XBOXAPI
HRESULT
WINAPI
XAvatarWaitingEffectStart(
    _Out_ XAVATAR_WAITING_EFFECT*                   pEffect
    );
    
//
// Stops a waiting effect. Should be called once on a waiting effect when an 
// avatar has loaded, after which the effect should still be rendered until
// XAvatarWaitingEffectRender() returns S_OK.
//

XBOXAPI
HRESULT
WINAPI
XAvatarWaitingEffectStop(
    _Inout_ XAVATAR_WAITING_EFFECT*                     pEffect
    );

//
// Renders an "avatar loading" visual effect. Can be used by titles while they 
// are waiting for avatar metadata or asset data to download. Returns S_OK if 
// the effect is stable, or E_PENDING if the effect is still starting or stopping.
//

XBOXAPI
HRESULT
WINAPI
XAvatarWaitingEffectRender(
    _Inout_ XAVATAR_WAITING_EFFECT*                     pEffect,
    _In_    LPDIRECT3DDEVICE9                           pDevice,
    _In_    XMMATRIX                                    ModelMatrix,
    _In_    XMMATRIX                                    ViewMatrix,
    _In_    XMMATRIX                                    ProjectionMatrix
    );

//
// Creates a renderer by requesting avatar data and converting it to renderable form. 
// Renderer can be used immediately, but attempts to render will produce nothing until 
// data has fully arrived.
//

XBOXAPI
HRESULT 
WINAPI
XAvatarCreateRenderer(
    _In_    CONST XAVATAR_METADATA*                     pAvatarMetadata,
    _In_    XAVATAR_COMPONENT_MASK                      ComponentMask,
    _In_    XAVATAR_SHADOW_SIZE                         ShadowSize,
    _In_    DWORD                                       MipBufferSize,
    _In_    XAVATAR_ASSET_LOAD_FLAGS                    AssetLoadFlags,
    _Deref_out_ LPXAVATARRENDERER*                      ppRenderer
    );


//
// Sets a custom asset (provided in a memory buffer) into the specified avatar
// metadata, allowing custom title provided assets to override those specified
// in the metadata. This will result in a pointer to the provided memory buffer
// being stored within the avatar metadata, which means the buffer cannot be
// freed until this metadata structure is no longer in use.  If the avatar
// metadata is used in a call to XAvatarGetAssets() or XAvatarCreateRenderer(),
// the asset memory buffer must persist until those calls have finished loading
// the avatar assets.
//

XBOXAPI
HRESULT 
WINAPI
XAvatarSetCustomAsset(
    _In_    DWORD                                       dwBufferSize,
    _In_bytecount_(dwBufferSize) CONST VOID*            pAssetBuffer,
    _In_    DWORD                                       dwCustomColorCount,
    _In_count_(dwCustomColorCount) CONST XMCOLOR*       pCustomColors,
    _Inout_ XAVATAR_METADATA*                           pAvatarMetadata
    );

#undef  INTERFACE
#define INTERFACE   IXAvatarRenderer

DECLARE_INTERFACE(IXAvatarRenderer)
{
    STDMETHOD_(ULONG, AddRef)( THIS ) PURE;
    STDMETHOD_(ULONG, Release)( THIS ) PURE;

    //
    // Returns the status of the renderer.
    //
    STDMETHOD(GetStatus)( THIS ) PURE;
    
    // 
    // Update the renderer.
    // Rebuilds joint matrices (including updating any animations being used by the renderer)
    // and prepares the renderer for rendering. This should be invoked once per frame for each
    // renderer, prior to invoking any other renderer functions (eg, RenderShadow()).
    //
    STDMETHOD(Update)(
        THIS_
        _In_    FLOAT                           FrameTime
        ) PURE;

    //
    // Queries the renderer for an appropriate set of matrices that can be supplied to the Render
    // APIs based on one of the three standard views: head; head-and-shoulders; or full-body.
    //
    STDMETHOD(GetMatrices)(
        THIS_
        _In_    XAVATAR_RENDER_VIEW             View,
        _In_    FLOAT                           AspectRatio,
        _Out_   XMMATRIX*                       pModelMatrix,
        _Out_   XMMATRIX*                       pViewMatrix,
        _Out_   XMMATRIX*                       pProjectionMatrix
        ) PURE;

    //
    // Renders to the currently set render target/depth buffer. Returns S_OK if something was
    // rendered; E_FAIL if renderable wasn't ready to render.
    //
    STDMETHOD(Render)(
        THIS_
        _In_    LPDIRECT3DDEVICE9               pDevice,
        _In_    XMMATRIX                        ModelMatrix,
        _In_    XMMATRIX                        ViewMatrix,
        _In_    XMMATRIX                        ProjectionMatrix
        ) PURE;

    //
    // Generates the avatar's self-shadowing shadow map. Returns S_OK if something was rendered;
    // E_FAIL if renderable wasn't ready to render.
    //
    STDMETHOD(RenderShadow)(
        THIS_
        _In_    LPDIRECT3DDEVICE9               pDevice,
        _In_    XMMATRIX                        ModelMatrix,
        _In_    XMMATRIX                        ViewMatrix,
        _In_    XMMATRIX                        ProjectionMatrix
        ) PURE;

    //
    // Causes the specified animations to be played in sequence by this renderer.  A NULL in 
    // this sequence indicates a random idle should be chosen.  The dwFlags parameter specifies 
    // how the animations should be played.  See the documentation on XAVATAR_PLAYANIMATION_FLAGS
    // for more information on the various flags you can pass.
    //
    STDMETHOD(PlayAnimations)(
        THIS_
        _In_    DWORD                           AnimationCount,
        _In_count_(AnimationCount) CONST LPXAVATARANIMATION* pAnimations,
        _In_    XAVATAR_PLAYANIMATIONS_FLAGS    Flags
        ) PURE;
    
    //
    // Obtains a pointer to the currently playing animation.  If a carryable idle animation
    // is playing, this returns XAVATAR_E_CARRYABLE_IDLE_ANIMATION_PLAYING and sets ppAnimation to NULL.
    // If a different idle animation is playing this returns XAVATAR_E_IDLE_ANIMATION_PLAYING 
    // and sets ppAnimation to NULL.
    // If successful, the interface is AddRef'd and the caller is responsible for Release.
    //
    STDMETHOD(GetCurrentAnimation)(
        THIS_
        _Deref_out_ LPXAVATARANIMATION*         ppAnimation
        ) PURE;

    //
    // Sets the level-of-detail (LOD) for the renderable, where zero is the lowest LOD. The value
    // specified indicates the number of joints to be animated during the Update() call, thus
    // setting this value to zero is the lowest possible LOD, and will prevent any animation from
    // occuring. The highest possible LOD value is thus XAVATAR_MAX_SKELETON_JOINTS, which is the
    // default value. Joints are culled in order of influence, with the least significant joints 
    // (such as the tips of the fingers) being the first to be culled. Culled joints are blended
    // into a default pose until such time as the LOD is restored. As an alternative to specifying
    //  a LOD-value yourself, consider using the predefined XAVATAR_RENDER_LOD_MINIMUM through 
    // XAVATAR_RENDER_LOD_MAXIMUM values.
    //
    STDMETHOD(SetLevelOfDetail)( 
        THIS_
        _In_    DWORD                           LevelOfDetail
        ) PURE;
        
    //
    // Returns the bounding box for this renderable if left to play the default 
    // idles. Playing an animation on the renderable by invoking PlayAnimations()
    // may lead to the avatar exceeding the default bounding box, but this can
    // be checked by calling CalculateBoundingBox() on the IXAvatarAnimation to
    // determine the bounding box for that particular animation.
    //
    STDMETHOD(GetDefaultBoundingBox)(
        THIS_
        _Out_   XMVECTOR*                       pBoundingBoxMin,
        _Out_   XMVECTOR*                       pBoundingBoxMax
        ) PURE;

    //
    // Retrieves a pointer to the avatar skeleton. This is only valid for the
    // lifetime of the renderer. It cannot be modified.
    //
    STDMETHOD(GetAvatarSkeleton)(
        THIS_
        _Out_   CONST XAVATAR_SKELETON**        ppSkeleton
        ) PURE;

    STDMETHOD(GetCarryableSkeleton)(
        THIS_
        _Out_   CONST XAVATAR_SKELETON**        ppSkeleton
        ) PURE;

    //
    // Set a joint to a specified pose. This pose is always offset from the
    // bind-pose, and you should take this into account when generating any
    // pose information you plan to pass to this function.
    //
    STDMETHOD(SetAvatarJointPose)(
        THIS_
        _In_    DWORD                               JointIndex,
        _In_    CONST XAVATAR_SKELETON_POSE_JOINT*  pJointPose
        ) PURE;

    STDMETHOD(SetCarryableJointPose)(
        THIS_
        _In_    DWORD                               JointIndex,
        _In_    CONST XAVATAR_SKELETON_POSE_JOINT*  pJointPose
        ) PURE;
        
    //
    // Get the pose of a specified joint. This pose is always offset from 
    // the bind-pose, and you should take this into account when interpreting 
    // the pose data returned from this function. If the joint index specified
    // is out of range then the E_INVALIDARG is returned.
    //
    STDMETHOD(GetAvatarJointPose)(
        THIS_
        _In_    DWORD                               JointIndex,
        _Out_   XAVATAR_SKELETON_POSE_JOINT*        pJointPose
        ) PURE;

    STDMETHOD(GetCarryableJointPose)(
        THIS_
        _In_    DWORD                               JointIndex,
        _Out_   XAVATAR_SKELETON_POSE_JOINT*        pJointPose
        ) PURE;

    //
    // Set a texture to a specified layer. If you specify an index that is
    // out of range then it will be ignored.
    //
    STDMETHOD(SetTextureLayer)(
        THIS_
        _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
        _In_    DWORD                               LayerIndex
        ) PURE;
    
    //
    // Set a texture to a specified custom D3D texture.
    //
    STDMETHOD(SetTextureLayerCustom)(
        THIS_
        _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
        _In_    LPDIRECT3DTEXTURE9                  pCustomTexture
        ) PURE;

    //
    // Get the current layer for a specified texture. If the texture index
    // specified is out of range then E_INVALIDARG is returned.
    //
    STDMETHOD(GetTextureLayer)(
        THIS_
        _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
        _Out_   DWORD*                              pLayerIndex
        ) PURE;

};

//-----------------------------------------------------------------------------
// XAvatar NUI Mapper APIs
//-----------------------------------------------------------------------------

XBOXAPI
HRESULT
WINAPI
XAvatarCreateNuiMapper(
    _In_        CONST XAVATAR_SKELETON*         pSkeleton,
    _In_        XAVATAR_BODY_TYPE               BodyType,
    _Deref_out_ LPXAVATARNUIMAPPER*             ppAvatarNuiMapper
    );

#undef  INTERFACE
#define INTERFACE   IXAvatarNuiMapper

DECLARE_INTERFACE(IXAvatarNuiMapper)
{
    STDMETHOD_(ULONG, AddRef)( THIS ) PURE;
    STDMETHOD_(ULONG, Release)( THIS ) PURE;

    STDMETHOD(SetNuiSkeletonData)(
        THIS_
        _In_    CONST NUI_SKELETON_FRAME*       pUnfilteredSkeletonFrameData,
        _In_    DWORD                           TrackingID
        ) PURE;

    STDMETHOD_(VOID, Update)(
        THIS_
        _In_    FLOAT                           DeltaTimeSinceLastRenderedFrame
        ) PURE;

    STDMETHOD(GetOptions)(
        THIS_
        _Out_   XAVATAR_NUI_MAPPER_OPTIONS*     pOptions
        ) PURE;

    STDMETHOD(SetOptions)(
        THIS_
        _In_    CONST XAVATAR_NUI_MAPPER_OPTIONS* pOptions
        ) PURE;

    //
    // Retrieve the position, rotation and scale of the Avatar relative to the camera position.
    // Position is scaled by the scaling applied to get the player to fit the Avatar skeleton
    // so that the feet don't slide. Divide the position by ScaleRelativeToPlayer if this is
    // not required.
    //
    STDMETHOD(GetAvatarTransformInfo)(
        THIS_
        _Out_   XAVATAR_NUI_MAPPER_TRANSFORM_INFO* pTransformInfo
        ) PURE;

    STDMETHOD(GetJointPose)(
        THIS_
        _In_      DWORD                                JointIdx,
        _Out_     XAVATAR_SKELETON_POSE_JOINT*         pJointPose,
        _Out_opt_ XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE* pJointConfidence
        ) PURE;
};

#pragma push_macro("VTBL")
#pragma push_macro("THIS")
#pragma push_macro("THIS_")

#undef  VTBL
#undef  THIS
#undef  THIS_

#if defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) (p)
#define THIS
#define THIS_
#else // defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) ((p)->lpVtbl)
#define THIS    pThis
#define THIS_   pThis,
#endif // defined(__cplusplus) && !defined(CINTERFACE)

FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarRenderer_AddRef(
    _Inout_ LPXAVATARRENDERER                   pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarRenderer_Release(
    _Inout_ LPXAVATARRENDERER                   pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetStatus(
    _In_    LPXAVATARRENDERER                   pThis
    )
{
    return VTBL(pThis)->GetStatus(THIS);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_Update(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    FLOAT                               FrameTime
    )
{
    return VTBL(pThis)->Update(THIS_ FrameTime);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetMatrices(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    XAVATAR_RENDER_VIEW                 View,
    _In_    FLOAT                               AspectRatio,
    _Out_   XMMATRIX*                           pModelMatrix,
    _Out_   XMMATRIX*                           pViewMatrix,
    _Out_   XMMATRIX*                           pProjectionMatrix
    )
{
    return VTBL(pThis)->GetMatrices(THIS_ View, AspectRatio, pModelMatrix, pViewMatrix, pProjectionMatrix);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_Render(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    LPDIRECT3DDEVICE9                   pDevice,
    _In_    XMMATRIX                            ModelMatrix,
    _In_    XMMATRIX                            ViewMatrix,
    _In_    XMMATRIX                            ProjectionMatrix
    )
{
    return VTBL(pThis)->Render(THIS_ pDevice, ModelMatrix, ViewMatrix, ProjectionMatrix);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_RenderShadow(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    LPDIRECT3DDEVICE9                   pDevice,
    _In_    XMMATRIX                            ModelMatrix,
    _In_    XMMATRIX                            ViewMatrix,
    _In_    XMMATRIX                            ProjectionMatrix
    )
{
    return VTBL(pThis)->RenderShadow(THIS_ pDevice, ModelMatrix, ViewMatrix, ProjectionMatrix);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_PlayAnimations(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               AnimationCount,
    _In_count_(AnimationCount) CONST LPXAVATARANIMATION* pAnimations,
    _In_    XAVATAR_PLAYANIMATIONS_FLAGS        Flags
    )
{
    return VTBL(pThis)->PlayAnimations(THIS_ AnimationCount, pAnimations, Flags);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetCurrentAnimation(
    _In_    LPXAVATARRENDERER                   pThis,
    _Deref_out_ LPXAVATARANIMATION*             ppAnimation
    )
{
    return VTBL(pThis)->GetCurrentAnimation(THIS_ ppAnimation);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_SetLevelOfDetail(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               LevelOfDetail
    )
{
    return VTBL(pThis)->SetLevelOfDetail(THIS_ LevelOfDetail);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetDefaultBoundingBox(
    _In_    LPXAVATARRENDERER                   pThis,
    _Out_   XMVECTOR*                           pBoundingBoxMin,
    _Out_   XMVECTOR*                           pBoundingBoxMax
    )
{
    return VTBL(pThis)->GetDefaultBoundingBox(THIS_ pBoundingBoxMin, pBoundingBoxMax);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetAvatarSkeleton(
    _In_    LPXAVATARRENDERER                   pThis,
    _Out_   CONST XAVATAR_SKELETON**            ppSkeleton
    )
{
    return VTBL(pThis)->GetAvatarSkeleton(THIS_ ppSkeleton);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetCarryableSkeleton(
    _In_    LPXAVATARRENDERER                   pThis,
    _Out_   CONST XAVATAR_SKELETON**            ppSkeleton
    )
{
    return VTBL(pThis)->GetCarryableSkeleton(THIS_ ppSkeleton);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_SetAvatarJointPose(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               JointIndex,
    _In_    CONST XAVATAR_SKELETON_POSE_JOINT*  pJointPose
    )
{
    return VTBL(pThis)->SetAvatarJointPose(THIS_ JointIndex, pJointPose);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_SetCarryableJointPose(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               JointIndex,
    _In_    CONST XAVATAR_SKELETON_POSE_JOINT*  pJointPose
    )
{
    return VTBL(pThis)->SetCarryableJointPose(THIS_ JointIndex, pJointPose);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetAvatarJointPose(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               JointIndex,
    _Out_   XAVATAR_SKELETON_POSE_JOINT*        pJointPose
    )
{
    return VTBL(pThis)->GetAvatarJointPose(THIS_ JointIndex, pJointPose);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetCarryableJointPose(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    DWORD                               JointIndex,
    _Out_   XAVATAR_SKELETON_POSE_JOINT*        pJointPose
    )
{
    return VTBL(pThis)->GetCarryableJointPose(THIS_ JointIndex, pJointPose);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_SetTextureLayer(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
    _In_    DWORD                               LayerIndex
    )
{
    return VTBL(pThis)->SetTextureLayer(THIS_ Texture, LayerIndex);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_SetTextureLayerCustom(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
    _In_    LPDIRECT3DTEXTURE9                  pCustomTexture
    )
{
    return VTBL(pThis)->SetTextureLayerCustom(THIS_ Texture, pCustomTexture);
}

FORCEINLINE
STDMETHODIMP
IXAvatarRenderer_GetTextureLayer(
    _In_    LPXAVATARRENDERER                   pThis,
    _In_    XAVATAR_ANIMATED_TEXTURE            Texture,
    _Out_   DWORD*                              pLayerIndex
    )
{
    return VTBL(pThis)->GetTextureLayer(THIS_ Texture, pLayerIndex);
}


FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarAnimation_AddRef(
    _Inout_ LPXAVATARANIMATION                  pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarAnimation_Release(
    _Inout_ LPXAVATARANIMATION                  pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_GetStatus(
    _In_    LPXAVATARANIMATION                  pThis
    )
{
    return VTBL(pThis)->GetStatus(THIS);
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_InitializeCursor(
    _In_    LPXAVATARANIMATION                  pThis,
    _In_    FLOAT                               Speed,
    _In_    XAVATAR_ANIMATION_PLAYMODE          PlayMode,
    _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
    _Out_   XAVATAR_ANIMATION_CURSOR*           pCursor
    )
{
    return VTBL(pThis)->InitializeCursor(THIS_ Speed, PlayMode, MotionMode, pCursor);
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_IncrementCursor(
    _In_    LPXAVATARANIMATION                  pThis,
    _In_    FLOAT                               Seconds,
    _Inout_ XAVATAR_ANIMATION_CURSOR*           pCursor
    )
{
    return VTBL(pThis)->IncrementCursor(THIS_ Seconds, pCursor);
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_GetAttributes(
    _In_    LPXAVATARANIMATION                  pThis,
    _Out_   FLOAT*                              pLengthInSeconds,
    _Out_   DWORD*                              pAvatarJointCount,
    _Out_   DWORD*                              pCarryableJointCount,
    _Out_   DWORD*                              pMotionDataCount,
    _Out_   DWORD*                              pTextureCount
    )
{
    return VTBL(pThis)->GetAttributes(THIS_ pLengthInSeconds, pAvatarJointCount, pCarryableJointCount, pMotionDataCount, pTextureCount);
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_GetPose( 
    _In_    LPXAVATARANIMATION                  pThis,
    _In_    CONST XAVATAR_ANIMATION_CURSOR*     pCursor,
    _In_    FLOAT                               BlendWeight,
    _In_    DWORD                               AvatarJointCount,
    _Inout_opt_cap_(AvatarJointCount) XAVATAR_SKELETON_POSE_JOINT* pAvatarJointPoseBuffer,
    _In_    DWORD                               CarryableJointCount,
    _Inout_opt_cap_(CarryableJointCount) XAVATAR_SKELETON_POSE_JOINT* pCarryableJointPoseBuffer,
    _In_    DWORD                               MotionDataCount,
    _Inout_opt_cap_(MotionDataCount) XAVATAR_MOTION_DATA* pMotionDataBuffer,
    _In_    DWORD                               TextureCount,
    _Inout_opt_cap_(TextureCount) DWORD*        pTextureLayerBuffer
    )
{
    return VTBL(pThis)->GetPose(
        THIS_
        pCursor,
        BlendWeight,
        AvatarJointCount,
        pAvatarJointPoseBuffer,
        CarryableJointCount,
        pCarryableJointPoseBuffer,
        MotionDataCount,
        pMotionDataBuffer,
        TextureCount,
        pTextureLayerBuffer );
}

FORCEINLINE
STDMETHODIMP
IXAvatarAnimation_CalculateBoundingBox(
    _In_    LPXAVATARANIMATION                  pThis,
    _In_    XAVATAR_COMPONENT_MASK              ComponentMask,
    _In_    XAVATAR_MOTION_MEASUREMODE          MotionMode,
    _Out_   XMVECTOR*                           pBoundingBoxMin,
    _Out_   XMVECTOR*                           pBoundingBoxMax
    )
{
    return VTBL(pThis)->CalculateBoundingBox(THIS_ ComponentMask, MotionMode, pBoundingBoxMin, pBoundingBoxMax);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarNuiMapper_AddRef(
    _Inout_ LPXAVATARNUIMAPPER                  pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXAvatarNuiMapper_Release(
    _Inout_ LPXAVATARNUIMAPPER                  pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP
IXAvatarNuiMapper_SetNuiSkeletonData(
    _Inout_ LPXAVATARNUIMAPPER                  pThis,
    _In_    CONST NUI_SKELETON_FRAME*           pUnfilteredSkeletonFrameData,
    _In_    DWORD                               TrackingID
    )
{
    return VTBL(pThis)->SetNuiSkeletonData(THIS_ pUnfilteredSkeletonFrameData, TrackingID);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXAvatarNuiMapper_Update(
    _Inout_ LPXAVATARNUIMAPPER                  pThis,
    _In_    FLOAT                               DeltaTimeSinceLastRenderedFrame
    )
{
    VTBL(pThis)->Update(THIS_ DeltaTimeSinceLastRenderedFrame);
}

FORCEINLINE
STDMETHODIMP
IXAvatarNuiMapper_GetOptions(
    _In_    LPXAVATARNUIMAPPER                  pThis,
    _Out_   XAVATAR_NUI_MAPPER_OPTIONS*         pOptions
    )
{
    return VTBL(pThis)->GetOptions(THIS_ pOptions);
}

FORCEINLINE
STDMETHODIMP
IXAvatarNuiMapper_SetOptions(
    _Inout_ LPXAVATARNUIMAPPER                  pThis,
    _In_    CONST XAVATAR_NUI_MAPPER_OPTIONS*   pOptions
    )
{
    return VTBL(pThis)->SetOptions(THIS_ pOptions);
}

FORCEINLINE
STDMETHODIMP
IXAvatarNuiMapper_GetAvatarTransformInfo(
    _In_    CONST LPXAVATARNUIMAPPER            pThis,
    _Out_   XAVATAR_NUI_MAPPER_TRANSFORM_INFO*  pTransformInfo
    )
{
    return VTBL(pThis)->GetAvatarTransformInfo(THIS_ pTransformInfo);
}

FORCEINLINE
STDMETHODIMP
IXAvatarNuiMapper_GetJointPose(
    _In_      CONST LPXAVATARNUIMAPPER             pThis,
    _In_      DWORD                                JointIdx,
    _Out_     XAVATAR_SKELETON_POSE_JOINT*         pJointPose,
    _Out_opt_ XAVATAR_NUI_MAPPER_JOINT_CONFIDENCE* pJointConfidence
    )
{
    return VTBL(pThis)->GetJointPose(THIS_ JointIdx, pJointPose, pJointConfidence);
}

#pragma pop_macro("VTBL")
#pragma pop_macro("THIS")
#pragma pop_macro("THIS_")


#ifdef __cplusplus
}
#endif

#endif  // _AVATAR_H_

