
/*++

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    xhttp.h

Abstract:

    Contains manifests, macros, types and prototypes for Xbox HTTP Services

--*/

#if !defined(_XHTTPX_)
#define _XHTTPX_

#if !defined(XBOXAPI)
#define XBOXAPI
#endif

//
// Set up Structure Packing for all xhttp structures
//

#if defined(_WIN64)
#include <pshpack8.h>
#else
#include <pshpack4.h>
#endif
#if defined(__cplusplus)
extern "C" {
#endif


//
// types
//

typedef PVOID HINTERNET;
typedef HINTERNET * PHINTERNET;

typedef WORD INTERNET_PORT;
typedef INTERNET_PORT * PINTERNET_PORT;


//
// XHTTP_ASYNC_RESULT - this structure is returned to the application via
// the callback with XHTTP_CALLBACK_STATUS_REQUEST_COMPLETE. It is not sufficient to
// just return the result of the async operation. If the API failed then the
// app cannot call GetLastError() because the thread context will be incorrect.
// Both the value returned by the async API and any resultant error code are
// made available. The app need not check dwError if dwResult indicates that
// the API succeeded (in this case dwError will be ERROR_SUCCESS)
//

typedef struct
{
    DWORD_PTR dwResult;  // indicates which async API has encountered an error
    DWORD dwError;       // the error code if the API failed
}
XHTTP_ASYNC_RESULT, * PXHTTP_ASYNC_RESULT;


//
// manifests
//

#define INTERNET_DEFAULT_PORT           0           // use the protocol-specific default
#define INTERNET_DEFAULT_HTTP_PORT      80
#define INTERNET_DEFAULT_HTTPS_PORT     443

//
// INTERNET_SCHEME - URL scheme type
//

typedef int INTERNET_SCHEME, * PINTERNET_SCHEME;

#define INTERNET_SCHEME_HTTP        (1)
#define INTERNET_SCHEME_HTTPS       (2)

//
// URL_COMPONENTS - the constituent parts of an URL. Used in XHttpCrackUrl()
// and XHttpCreateUrl()
//
// For XHttpCrackUrl(), if a pointer field and its corresponding length field
// are both 0 then that component is not returned. If the pointer field is NULL
// but the length field is not zero, then both the pointer and length fields are
// returned. If both pointer and corresponding length fields are non-zero then
// the pointer field points to a buffer where the component is copied. The
// component may be un-escaped, depending on dwFlags
//
// For XHttpCreateUrl(), the pointer fields should be NULL if the component
// is not required. If the corresponding length field is zero then the pointer
// field is the address of a zero-terminated string. If the length field is not
// zero then it is the string length of the corresponding pointer field
//

#pragma warning( disable : 4121 )   // disable alignment warning

typedef struct
{
    DWORD   dwStructSize;       // size of this structure. Used in version check
    LPSTR   lpszScheme;         // pointer to scheme name
    DWORD   dwSchemeLength;     // length of scheme name
    INTERNET_SCHEME nScheme;    // enumerated scheme type (if known)
    LPSTR   lpszHostName;       // pointer to host name
    DWORD   dwHostNameLength;   // length of host name
    INTERNET_PORT nPort;        // converted port number
    LPSTR   lpszUserName;       // pointer to user name
    DWORD   dwUserNameLength;   // length of user name
    LPSTR   lpszPassword;       // pointer to password
    DWORD   dwPasswordLength;   // length of password
    LPSTR   lpszUrlPath;        // pointer to URL-path
    DWORD   dwUrlPathLength;    // length of URL-path
    LPSTR   lpszExtraInfo;      // pointer to extra information (e.g. ?foo or #foo)
    DWORD   dwExtraInfoLength;  // length of extra information
}
URL_COMPONENTS, * PURL_COMPONENTS;

#pragma warning( default : 4121 )   // restore alignment warning


//
// Flags for XHttpCrackUrl() and XHttpCreateUrl():
//

#define ICU_DECODE              0x10000000 // Convert %XX escape sequences to characters
#define ICU_ESCAPE              0x80000000 // (un)escape URL characters


//
// flags for XHttpOpen():
//

#define XHTTP_FLAG_ASYNC                0x10000000  // this session is asynchronous (where supported)

//
// XHttpOpen dwAccessType values
//

#define XHTTP_ACCESS_TYPE_DEFAULT_PROXY               0
#define XHTTP_ACCESS_TYPE_NO_PROXY                    1
#define XHTTP_ACCESS_TYPE_NAMED_PROXY                 3

//
// XHttpOpen pseudonyms for optional parameters, to improve readability
//

#define XHTTP_NO_PROXY_NAME     NULL
#define XHTTP_NO_PROXY_BYPASS   NULL


//
// flags for XHttpConnect():
//

#define XHTTP_FLAG_SECURE               0x00800000  // use SSL if applicable (HTTPS)


//
// callback function for XHttpSetStatusCallback
//

typedef
VOID
(CALLBACK * XHTTP_STATUS_CALLBACK)(
    __in HINTERNET hInternet,
    __in DWORD_PTR dwContext,
    __in DWORD dwInternetStatus,
    __in_bcount_opt(dwStatusInformationLength) PVOID lpvStatusInformation,
    __in DWORD dwStatusInformationLength
    );

typedef XHTTP_STATUS_CALLBACK * PXHTTP_STATUS_CALLBACK;



//
// options manifests for XHttp{Query|Set}Option
//

#define XHTTP_OPTION_CALLBACK                       1
#define XHTTP_OPTION_RECEIVE_TIMEOUT                6
#define XHTTP_OPTION_READ_BUFFER_SIZE               12
#define XHTTP_OPTION_KEEP_CONNECTION                22
#define XHTTP_OPTION_REQUEST_FLAGS                  23
#define XHTTP_OPTION_DISABLE_FEATURE                63
#define XHTTP_OPTION_REDIRECT_POLICY                88
#define XHTTP_OPTION_MAX_HTTP_AUTOMATIC_REDIRECTS   89
#define XHTTP_OPTION_MAX_HTTP_STATUS_CONTINUE       90
#define XHTTP_OPTION_CONNECTION_DEST_ADDRESS        100
#define XHTTP_OPTION_SECURITY_FEATURE               101


//
// values for XHTTP_OPTION_SECURITY_FEATURE
//

#define SECURITY_FLAG_IGNORE_LIVE_VALIDATION_CACHE      0x00010000 // ignore any currently cached LIVE validation results
#define SECURITY_FLAG_ALLOW_LIVE_REJECTED_CERT          0x00020000 // use cert even if LIVE rejects it, for debugging only


//
// values for XHTTP_OPTION_REDIRECT_POLICY
//

#define XHTTP_OPTION_REDIRECT_POLICY_NEVER                      0
#define XHTTP_OPTION_REDIRECT_POLICY_DISALLOW_HTTPS_TO_HTTP     1
#define XHTTP_OPTION_REDIRECT_POLICY_ALWAYS                     2
#define XHTTP_OPTION_REDIRECT_POLICY_NO_SCHEME_CHANGES          3

#define XHTTP_OPTION_REDIRECT_POLICY_LAST                       XHTTP_OPTION_REDIRECT_POLICY_NO_SCHEME_CHANGES
#define XHTTP_OPTION_REDIRECT_POLICY_DEFAULT                    XHTTP_OPTION_REDIRECT_POLICY_NO_SCHEME_CHANGES


//
// values for XHTTP_OPTION_KEEP_CONNECTION
//

#define XHTTP_KEEP_ALIVE_UNKNOWN     0xFFFFFFFF
#define XHTTP_KEEP_ALIVE_ENABLED     1
#define XHTTP_KEEP_ALIVE_DISABLED    0


//
// values for XHTTP_OPTION_DISABLE_FEATURE
//

#define XHTTP_DISABLE_KEEP_ALIVE                0x00000008
#define XHTTP_DISABLE_HOST_HEADER               0x00000010


//
// status manifests for XHttp status callback
//

#define XHTTP_CALLBACK_STATUS_HANDLE_CLOSING            0x00000800
#define XHTTP_CALLBACK_STATUS_REDIRECT                  0x00004000
#define XHTTP_CALLBACK_STATUS_INTERMEDIATE_RESPONSE     0x00008000
#define XHTTP_CALLBACK_STATUS_HEADERS_AVAILABLE         0x00020000
#define XHTTP_CALLBACK_STATUS_READ_COMPLETE             0x00080000
#define XHTTP_CALLBACK_STATUS_WRITE_COMPLETE            0x00100000
#define XHTTP_CALLBACK_STATUS_REQUEST_ERROR             0x00200000
#define XHTTP_CALLBACK_STATUS_SENDREQUEST_COMPLETE      0x00400000

//
// API Enums for XHTTP_CALLBACK_STATUS_REQUEST_ERROR:
//

#define API_RECEIVE_RESPONSE        (1)
#define API_QUERY_DATA_AVAILABLE    (2)
#define API_READ_DATA               (3)
#define API_WRITE_DATA              (4)
#define API_SEND_REQUEST            (5)


#define XHTTP_CALLBACK_FLAG_HANDLES                 XHTTP_CALLBACK_STATUS_HANDLE_CLOSING
#define XHTTP_CALLBACK_FLAG_REDIRECT                XHTTP_CALLBACK_STATUS_REDIRECT
#define XHTTP_CALLBACK_FLAG_INTERMEDIATE_RESPONSE   XHTTP_CALLBACK_STATUS_INTERMEDIATE_RESPONSE
#define XHTTP_CALLBACK_FLAG_HEADERS_AVAILABLE       XHTTP_CALLBACK_STATUS_HEADERS_AVAILABLE
#define XHTTP_CALLBACK_FLAG_READ_COMPLETE           XHTTP_CALLBACK_STATUS_READ_COMPLETE
#define XHTTP_CALLBACK_FLAG_WRITE_COMPLETE          XHTTP_CALLBACK_STATUS_WRITE_COMPLETE
#define XHTTP_CALLBACK_FLAG_REQUEST_ERROR           XHTTP_CALLBACK_STATUS_REQUEST_ERROR
#define XHTTP_CALLBACK_FLAG_SENDREQUEST_COMPLETE    XHTTP_CALLBACK_STATUS_SENDREQUEST_COMPLETE


#define XHTTP_CALLBACK_FLAG_ALL_COMPLETIONS         (XHTTP_CALLBACK_FLAG_HEADERS_AVAILABLE          \
                                                        | XHTTP_CALLBACK_FLAG_READ_COMPLETE         \
                                                        | XHTTP_CALLBACK_FLAG_WRITE_COMPLETE        \
                                                        | XHTTP_CALLBACK_FLAG_REQUEST_ERROR         \
                                                        | XHTTP_CALLBACK_FLAG_SENDREQUEST_COMPLETE)
#define XHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS       0xffffffff


//
// XHttpQueryHeaders info levels. Generally, there is one info level
// for each potential RFC822/HTTP/MIME header that an HTTP server
// may send as part of a request response.
//

#define XHTTP_QUERY_ACCEPT                      0
#define XHTTP_QUERY_ACCEPT_CHARSET              1
#define XHTTP_QUERY_ACCEPT_ENCODING             2
#define XHTTP_QUERY_ACCEPT_LANGUAGE             3
#define XHTTP_QUERY_ACCEPT_RANGES               4
#define XHTTP_QUERY_ALLOW                       5
#define XHTTP_QUERY_CACHE_CONTROL               6
#define XHTTP_QUERY_CONNECTION                  7
#define XHTTP_QUERY_CONTENT_LANGUAGE            8
#define XHTTP_QUERY_CONTENT_LENGTH              9
#define XHTTP_QUERY_CONTENT_RANGE               42
#define XHTTP_QUERY_CONTENT_TRANSFER_ENCODING   10
#define XHTTP_QUERY_CONTENT_TYPE                11
#define XHTTP_QUERY_DATE                        12
#define XHTTP_QUERY_EXPIRES                     13
#define XHTTP_QUERY_EXT                         14
#define XHTTP_QUERY_HOST                        15
#define XHTTP_QUERY_IF_MATCH                    16
#define XHTTP_QUERY_IF_MODIFIED_SINCE           17
#define XHTTP_QUERY_IF_NONE_MATCH               18
#define XHTTP_QUERY_IF_RANGE                    19
#define XHTTP_QUERY_IF_UNMODIFIED_SINCE         20
#define XHTTP_QUERY_LAST_MODIFIED               21
#define XHTTP_QUERY_MAN                         23
#define XHTTP_QUERY_MIME_VERSION                24
#define XHTTP_QUERY_MX                          25
#define XHTTP_QUERY_NT                          26
#define XHTTP_QUERY_NTS                         27
#define XHTTP_QUERY_RANGE                       28
#define XHTTP_QUERY_RAW_HEADERS_CRLF            22 // special: all headers
#define XHTTP_QUERY_REFERRER                    29
#define XHTTP_QUERY_SERVER                      30
#define XHTTP_QUERY_SEQ                         31
#define XHTTP_QUERY_SID                         32
#define XHTTP_QUERY_ST                          33
#define XHTTP_QUERY_TIMEOUT                     34
#define XHTTP_QUERY_TRANSFER_ENCODING           35
#define XHTTP_QUERY_UNLESS_MODIFIED_SINCE       36
#define XHTTP_QUERY_USER_AGENT                  37
#define XHTTP_QUERY_USN                         38
#define XHTTP_QUERY_X_DELAY                     39
#define XHTTP_QUERY_X_DELAYFLAGS                40
#define XHTTP_QUERY_X_ERR                       41

#define XHTTP_QUERY_MAX                         42


#define XHTTP_QUERY_STATUS_CODE                 65534  // special: part of status line


//
// XHTTP_QUERY_CUSTOM - if this special value is supplied as the dwInfoLevel
// parameter of XHttpQueryHeaders() then the pBuffer parameter contains the name
// of the header we are to query
//

#define XHTTP_QUERY_CUSTOM                      65535

//
// XHTTP_QUERY_FLAG_REQUEST_HEADERS - if this bit is set in the dwInfoLevel
// parameter of XHttpQueryHeaders() then the request headers will be queried for the
// request information
//

#define XHTTP_QUERY_FLAG_REQUEST_HEADERS        0x80000000

//
// XHTTP_QUERY_FLAG_SYSTEMTIME - if this bit is set in the dwInfoLevel parameter
// of XHttpQueryHeaders() AND the header being queried contains date information,
// e.g. the "Expires:" header then pBuffer will contain a SYSTEMTIME structure
// containing the date and time information converted from the header string
//

#define XHTTP_QUERY_FLAG_SYSTEMTIME             0x40000000

//
// XHTTP_QUERY_FLAG_NUMBER - if this bit is set in the dwInfoLevel parameter of
// HttpQueryHeader(), then the value of the header will be converted to a number
// before being returned to the caller, if applicable
//

#define XHTTP_QUERY_FLAG_NUMBER                 0x20000000

//
// XHTTP_QUERY_FLAG_FILETIME - if this bit is set in the dwInfoLevel parameter
// of XHttpQueryHeaders() AND the header being queried contains date information,
// e.g. the "Expires:" header then pBuffer will contain a FILETIME structure
// containing the date and time information converted from the header string
//

#define XHTTP_QUERY_FLAG_FILETIME               0x10000000

//
// HTTP Response Status Codes:
//

#define HTTP_STATUS_CONTINUE            100 // OK to continue with request
#define HTTP_STATUS_SWITCH_PROTOCOLS    101 // server has switched protocols in upgrade header

#define HTTP_STATUS_OK                  200 // request completed
#define HTTP_STATUS_CREATED             201 // object created, reason = new URI
#define HTTP_STATUS_ACCEPTED            202 // async completion (TBS)
#define HTTP_STATUS_PARTIAL             203 // partial completion
#define HTTP_STATUS_NO_CONTENT          204 // no info to return
#define HTTP_STATUS_RESET_CONTENT       205 // request completed, but clear form
#define HTTP_STATUS_PARTIAL_CONTENT     206 // partial GET fulfilled
#define HTTP_STATUS_WEBDAV_MULTI_STATUS 207 // WebDAV Multi-Status

#define HTTP_STATUS_AMBIGUOUS           300 // server couldn't decide what to return
#define HTTP_STATUS_MOVED               301 // object permanently moved
#define HTTP_STATUS_REDIRECT            302 // object temporarily moved
#define HTTP_STATUS_REDIRECT_METHOD     303 // redirection w/ new access method
#define HTTP_STATUS_NOT_MODIFIED        304 // if-modified-since was not modified
#define HTTP_STATUS_USE_PROXY           305 // redirection to proxy, location header specifies proxy to use
#define HTTP_STATUS_REDIRECT_KEEP_VERB  307 // HTTP/1.1: keep same verb

#define HTTP_STATUS_BAD_REQUEST         400 // invalid syntax
#define HTTP_STATUS_DENIED              401 // access denied
#define HTTP_STATUS_PAYMENT_REQ         402 // payment required
#define HTTP_STATUS_FORBIDDEN           403 // request forbidden
#define HTTP_STATUS_NOT_FOUND           404 // object not found
#define HTTP_STATUS_BAD_METHOD          405 // method is not allowed
#define HTTP_STATUS_NONE_ACCEPTABLE     406 // no response acceptable to client found
#define HTTP_STATUS_PROXY_AUTH_REQ      407 // proxy authentication required
#define HTTP_STATUS_REQUEST_TIMEOUT     408 // server timed out waiting for request
#define HTTP_STATUS_CONFLICT            409 // user should resubmit with more info
#define HTTP_STATUS_GONE                410 // the resource is no longer available
#define HTTP_STATUS_LENGTH_REQUIRED     411 // the server refused to accept request w/o a length
#define HTTP_STATUS_PRECOND_FAILED      412 // precondition given in request failed
#define HTTP_STATUS_REQUEST_TOO_LARGE   413 // request entity was too large
#define HTTP_STATUS_URI_TOO_LONG        414 // request URI too long
#define HTTP_STATUS_UNSUPPORTED_MEDIA   415 // unsupported media type
#define HTTP_STATUS_RETRY_WITH          449 // retry after doing the appropriate action.

#define HTTP_STATUS_SERVER_ERROR        500 // internal server error
#define HTTP_STATUS_NOT_SUPPORTED       501 // required not supported
#define HTTP_STATUS_BAD_GATEWAY         502 // error response received from gateway
#define HTTP_STATUS_SERVICE_UNAVAIL     503 // temporarily overloaded
#define HTTP_STATUS_GATEWAY_TIMEOUT     504 // timed out waiting for gateway
#define HTTP_STATUS_VERSION_NOT_SUP     505 // HTTP version not supported

#define HTTP_STATUS_FIRST               HTTP_STATUS_CONTINUE
#define HTTP_STATUS_LAST                HTTP_STATUS_VERSION_NOT_SUP


//
// XHttpOpenRequest psuedonyms for optional parameters
//

#define XHTTP_NO_REFERRER                   NULL
#define XHTTP_DEFAULT_ACCEPT_TYPES          NULL


//
// XHttpSendRequest pseudonyms for optional parameters, to improve readability
//

#define XHTTP_NO_ADDITIONAL_HEADERS          NULL
#define XHTTP_NO_REQUEST_DATA                NULL
#define XHTTP_IGNORE_REQUEST_TOTAL_LENGTH    0


//
// XHttpQueryHeaders pseudonyms for optional parameters, to improve readability
//

#define XHTTP_HEADER_NAME_BY_INDEX           NULL
#define XHTTP_NO_OUTPUT_BUFFER               NULL
#define XHTTP_NO_HEADER_INDEX                NULL

//
// XHttp API error returns
//

#define XHTTP_ERROR_BASE                                    12000

#define ERROR_XHTTP_TIMEOUT                                 (XHTTP_ERROR_BASE + 2)
#define ERROR_XHTTP_INTERNAL_ERROR                          (XHTTP_ERROR_BASE + 4)
#define ERROR_XHTTP_UNRECOGNIZED_SCHEME                     (XHTTP_ERROR_BASE + 6)
#define ERROR_XHTTP_NAME_NOT_RESOLVED                       (XHTTP_ERROR_BASE + 7)
#define ERROR_XHTTP_INVALID_OPTION                          (XHTTP_ERROR_BASE + 9)
#define ERROR_XHTTP_OPTION_NOT_SETTABLE                     (XHTTP_ERROR_BASE + 11)
#define ERROR_XHTTP_INCORRECT_HANDLE_TYPE                   (XHTTP_ERROR_BASE + 18)
#define ERROR_XHTTP_INCORRECT_HANDLE_STATE                  (XHTTP_ERROR_BASE + 19)
#define ERROR_XHTTP_CONNECTION_ERROR                        (XHTTP_ERROR_BASE + 30)
#define ERROR_XHTTP_HEADER_NOT_FOUND                        (XHTTP_ERROR_BASE + 150)
#define ERROR_XHTTP_INVALID_SERVER_RESPONSE                 (XHTTP_ERROR_BASE + 152)
#define ERROR_XHTTP_REDIRECT_FAILED                         (XHTTP_ERROR_BASE + 156)
#define ERROR_XHTTP_NOT_INITIALIZED                         (XHTTP_ERROR_BASE + 172)
#define ERROR_XHTTP_SECURE_FAILURE                          (XHTTP_ERROR_BASE + 175)

//
// XHTTP API Signatures
//

XBOXAPI
BOOL
WINAPI
XHttpStartup(
    __in DWORD dwReserved,
    __in_opt VOID * pvReserved
    );



XBOXAPI
BOOL
WINAPI
XHttpCrackUrl(
    __in_ecount(dwUrlLength) CONST CHAR * pcszUrl,
    __in DWORD dwUrlLength,
    __in DWORD dwFlags,
    __inout PURL_COMPONENTS pUrlComponents
    );


XBOXAPI
BOOL
WINAPI
XHttpCreateUrl(
    __in CONST URL_COMPONENTS * pUrlComponents,
    __in DWORD dwFlags,
    __out_ecount_z_opt(*pdwUrlLength) PSTR pszUrl,
    __inout DWORD * pdwUrlLength
    );



XBOXAPI
HINTERNET
WINAPI
XHttpOpen(
    __in_z CONST CHAR * pcszUserAgent,
    __in DWORD dwAccessType,
    __in_z_opt CONST CHAR * pcszProxyName,
    __in_z_opt CONST CHAR * pcszProxyBypass,
    __in DWORD  dwFlags
    );


XBOXAPI
HINTERNET
WINAPI
XHttpConnect(
    __in HINTERNET hSession,
    __in_z CONST CHAR * pcszServerName,
    __in INTERNET_PORT nServerPort,
    __in DWORD dwFlags
    );


XBOXAPI
HINTERNET
WINAPI
XHttpOpenRequest(
    __in HINTERNET hConnect,
    __in_z CONST CHAR * pcszVerb,
    __in_z CONST CHAR * pcszObjectName,
    __in_z_opt CONST CHAR * pcszVersion,
    __in_z_opt CONST CHAR * pcszReferrer,
    __in_opt CONST CHAR ** ppReserved,
    __in DWORD dwFlags
    );



XBOXAPI
BOOL
WINAPI
XHttpSetOption(
    __in HINTERNET hInternet,
    __in DWORD dwOption,
    __in_bcount(dwBufferLength) CONST VOID * pvBuffer,
    __in DWORD dwBufferLength
    );


XBOXAPI
BOOL
WINAPI
XHttpQueryOption(
    __in HINTERNET hInternet,
    __in DWORD dwOption,
    __out_bcount_opt(*pdwBufferLength) VOID * pvBuffer,
    __inout DWORD * pdwBufferLength
    );



XBOXAPI
XHTTP_STATUS_CALLBACK
WINAPI
XHttpSetStatusCallback(
    __in HINTERNET hInternet,
    __in_opt XHTTP_STATUS_CALLBACK pfnInternetCallback,
    __in DWORD dwNotificationFlags,
    __in DWORD_PTR dwReserved
    );



XBOXAPI
BOOL
WINAPI
XHttpSendRequest(
    __in HINTERNET hRequest,
    __in_ecount_opt(dwHeadersLength) CONST CHAR * pcszHeaders,
    __in DWORD dwHeadersLength,
    __in_bcount_opt(dwOptionalLength) CONST VOID * lpOptional,
    __in DWORD dwOptionalLength,
    __in DWORD dwTotalLength,
    __in DWORD_PTR dwContext
    );


XBOXAPI
BOOL
WINAPI
XHttpDoWork(
    __in HINTERNET hSession,
    __in DWORD dwNumMs
    );


XBOXAPI
BOOL
WINAPI
XHttpWriteData(
    __in HINTERNET hRequest,
    __in_bcount(dwNumberOfBytesToWrite) CONST VOID * pBuffer,
    __in DWORD dwNumberOfBytesToWrite,
    __out_opt DWORD * pdwNumberOfBytesWritten
    );


XBOXAPI
BOOL
WINAPI
XHttpReceiveResponse(
    __in HINTERNET hRequest,
    __in_opt VOID * pvReserved
    );


XBOXAPI
BOOL
WINAPI
XHttpQueryHeaders(
    __in HINTERNET hRequest,
    __in DWORD dwInfoLevel,
    __in_z_opt CONST CHAR * pcszName, 
    __out_bcount_opt(*pdwBufferLength) PVOID pBuffer,
    __inout DWORD * pdwBufferLength,
    __inout_opt DWORD * pdwIndex
    );


XBOXAPI
BOOL
WINAPI
XHttpReadData(
    __in HINTERNET hRequest,
    __out_bcount(dwNumberOfBytesToRead) VOID * pBuffer,
    __in DWORD dwNumberOfBytesToRead,
    __out_opt DWORD * pdwNumberOfBytesRead
    );


XBOXAPI
BOOL
WINAPI
XHttpCloseHandle(
    __in HINTERNET hInternet
    );


XBOXAPI
VOID
WINAPI
XHttpShutdown(
    VOID
    );



#if defined(__cplusplus)
}
#endif


//
// Return packing to whatever it was before we
// entered this file
//
#include <poppack.h>



#endif // !defined(_XHTTPX_)
