/***************************************************************************
 *
 *  xhv2.h -- This module defines the Xbox High-Level Voice APIs
 *
 *  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 *
 ***************************************************************************/

#ifndef __XHV2_H__

#define __XHV2_H__

#include <xaudio2.h>
#include <float.h>


#include <xonline.h>


#ifdef __cplusplus
extern "C" {
#endif


// Constants
#define XHVINLINE                               FORCEINLINE

//
// The XHV worker thread will initially run on this processor
//

#define XHV_WORKER_THREAD_PROCESSOR             (1)
#define XHV_STACK_SIZE                          (0x10000)   // 64KB

#define XHV_MAX_REMOTE_TALKERS                  (30)
#define XHV_MAX_LOCAL_TALKERS                   (4)

//
// At the moment, you may only have up to this many enabled processing modes
// for local and remote talkers (separately).
//

#define XHV_MAX_PROCESSING_MODES                (2)

//
// When setting playback priorities, talker pairs between
// XHV_PLAYBACK_PRIORITY_MAX and XHV_PLAYBACK_PRIORITY_MIN, inclusive, will be
// heard, while any other value will result in muting that pair.
//

#define XHV_PLAYBACK_PRIORITY_MAX               (0)
#define XHV_PLAYBACK_PRIORITY_MIN               (0xFFFF)
#define XHV_PLAYBACK_PRIORITY_NEVER             (0xFFFFFFFF)

//
// Each packet reported by voice chat mode is the following size (including the
// XHV_CODEC_HEADER) for low-quality audio.
//

#define XHV_VOICECHAT_MODE_PACKET_SIZE          (42)

//
// When supplying a buffer to GetLocalChatData, you won't have to supply a
// buffer any larger than the following number of packets (or
// XHV_MAX_VOICECHAT_PACKETS * XHV_VOICECHAT_MODE_PACKET_SIZE bytes)
//

#define XHV_MAX_VOICECHAT_PACKETS               (10)

//
// When supplying a buffer of NUI_TALKER_POSITION structures to
// NuiGetLocalChatData, you won't have to supply a buffer any larger than the
// following number of structures.
//

#define XHV_MAX_VOICECHAT_POSITIONS             (1)

//
// XHV's final output submix voice consists of one channel per local talker.
// Each channel contains the mixed voice for its user index.  The data can be
// mapped to speakers or headsets.
//

#define XHV_OUTPUT_CHANNEL_COUNT                (XHV_MAX_LOCAL_TALKERS)

//
// Channel counts for pfxDefaultRemoteTalkerFX and pfxDefaultTalkerPairFX
// effect chains respectively, used with XHV_INIT_PARAMS.
//
#define XHV_REMOTE_TALKER_CHANNEL_COUNT         (1)
#define XHV_TALKER_PAIR_CHANNEL_COUNT           (XHV_OUTPUT_CHANNEL_COUNT)

//
// XHV uses four stages of submix voices.  Hence, XAudio2's maximum stage count
// must be set to at least this value.
//

#define XHV_SUBMIX_STAGE_COUNT (4)

//
// The microphone callback is given PCM data in this format.  Note that
// XAudio2 effect chains will process the data in the native XAudio2 format
// (see XAudio2 for details).
//

#define XHV_PCM_BYTES_PER_SAMPLE                (2)
#define XHV_PCM_SAMPLE_RATE                     (16000)


//
// Data Ready Flags.  These flags are set when there is local data waiting to be
// consumed (e.g. through GetLocalChatData).  GetLocalDataFlags() allows you to
// get the current state of these flags without entering XHV's critical section.
// Each mask is 4 bits, one for each local talker.  The least significant bit in
// each section indicates data is available for user index 0, while the most
// significant bit indicates user index 3.
//

#define XHV_VOICECHAT_DATA_READY_MASK           (0xF)
#define XHV_VOICECHAT_DATA_READY_OFFSET         (0)


//
// Interfaces
//

typedef struct IXHV2Engine                     *LPIXHV2ENGINE, *PIXHV2ENGINE;

//
// Typedefs, Enums and Structures
//

typedef CONST LPVOID                            XHV_PROCESSING_MODE, *PXHV_PROCESSING_MODE;
typedef DWORD                                   XHV_PLAYBACK_PRIORITY;
typedef VOID(*PFNMICRAWDATAREADY)(
    __in DWORD                                  dwUserIndex,
    __in_bcount(dwSize) PVOID                   pvData,
    __in DWORD                                  dwSize,
    __inout PBOOL                               pVoiceDetected
    );

typedef DWORD                                   XHV_LOCK_TYPE;

#define XHV_LOCK_TYPE_LOCK                      0
#define XHV_LOCK_TYPE_TRYLOCK                   1
#define XHV_LOCK_TYPE_UNLOCK                    2
#define XHV_LOCK_TYPE_COUNT                     3

//
// Supported processing modes
//

extern  CONST LPVOID                            _xhv_loopback_mode;
extern  CONST LPVOID                            _xhv_voicechat_mode;

#define XHV_LOOPBACK_MODE                       _xhv_loopback_mode
#define XHV_VOICECHAT_MODE                      _xhv_voicechat_mode

//
// You must specify the following initialization parameters at creation time
// Note the internal XHV "source feeder effect" XAPO is added to the end of
// the pfxOutputFX effect chain.  You must pass in an initialized XAudio2
// instance as pXAudio2.
//

typedef struct XHV_INIT_PARAMS
{
    DWORD                                       dwMaxRemoteTalkers;
    DWORD                                       dwMaxLocalTalkers;
    PXHV_PROCESSING_MODE                        localTalkerEnabledModes;
    DWORD                                       dwNumLocalTalkerEnabledModes;
    PXHV_PROCESSING_MODE                        remoteTalkerEnabledModes;
    DWORD                                       dwNumRemoteTalkerEnabledModes;
    BOOL                                        bCustomVADProvided;
    BOOL                                        bRelaxPrivileges;
    PFNMICRAWDATAREADY                          pfnMicrophoneRawDataReady;
    XAUDIO2_EFFECT_CHAIN**                      ppfxDefaultRemoteTalkerFX;
    XAUDIO2_EFFECT_CHAIN**                      ppfxDefaultTalkerPairFX;
    XAUDIO2_EFFECT_CHAIN*                       pfxOutputFX;
    IXAudio2*                                   pXAudio2;
} XHV_INIT_PARAMS, *PXHV_INIT_PARAMS;

//
// This header appears at the beginning of each blob of data reported by voice
// chat mode
//

#pragma pack(push, 1)
typedef struct XHV_CODEC_HEADER
{
    WORD                                        bMsgNo :  4;
    WORD                                        wSeqNo : 11;
    WORD                                        bFriendsOnly : 1;
} XHV_CODEC_HEADER, *PXHV_CODEC_HEADER;
#pragma pack (pop)

//
// The NUI_TALKER_POSITION structure describes the angle, in radians, to the
// the talkers from the camera and a confidence value. Valid range for an angle
// is from -pi to +pi. An angle of zero is straight in front of the camera,
// positive values are to the left of the camera (from the camera's perspective)
// and negative values are to the right of the camera. The confidence values
// range from 0.0 to 1.0 with 0.0 as a non-localized sound and 1.0 as a perfectly
// localized sound.
//

typedef struct NUI_TALKER_POSITION
{
    FLOAT                                       fDirection;
    FLOAT                                       fConfidence;
} NUI_TALKER_POSITION, *PNUI_TALKER_POSITION;

//
//  IXHV2Engine interface
//

#undef  INTERFACE
#define INTERFACE IXHV2Engine
DECLARE_INTERFACE(IXHV2Engine)
{
    STDMETHOD_(ULONG, AddRef)(
                                                THIS
        )   PURE;

    STDMETHOD_(ULONG, Release)(
                                                THIS
        )   PURE;

    //
    // Locking
    //

    STDMETHOD(Lock)(
                                                THIS_
        __in XHV_LOCK_TYPE                      lockType
        );

    //
    // Processing mode management
    //

    STDMETHOD(StartLocalProcessingModes)(
                                                                        THIS_
        __in DWORD                                                      dwUserIndex,
        __in_ecount(dwNumProcessingModes)   CONST PXHV_PROCESSING_MODE  processingModes,
        __in DWORD                                                      dwNumProcessingModes
        )   PURE;

    STDMETHOD(StopLocalProcessingModes)(
                                                                        THIS_
        __in DWORD                                                      dwUserIndex,
        __in_ecount(dwNumProcessingModes)   CONST PXHV_PROCESSING_MODE  processingModes,
        __in DWORD                                                      dwNumProcessingModes
        )   PURE;

    STDMETHOD(StartRemoteProcessingModes)(
                                                                        THIS_
        __in XUID                                                       xuidRemoteTalker,
        __in_ecount(dwNumProcessingModes)   CONST PXHV_PROCESSING_MODE  processingModes,
        __in DWORD                                                      dwNumProcessingModes
        )   PURE;

    STDMETHOD(StopRemoteProcessingModes)(
                                                                        THIS_
        __in XUID                                                       xuidRemoteTalker,
        __in_ecount(dwNumProcessingModes)   CONST PXHV_PROCESSING_MODE  processingModes,
        __in DWORD                                                      dwNumProcessingModes
        )   PURE;

    STDMETHOD(SetMaxDecodePackets)(
                                                THIS_
        __in DWORD                              dwMaxDecodePackets
        )   PURE;

    //
    // Manage talkers
    //

    STDMETHOD(RegisterLocalTalker)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD(UnregisterLocalTalker)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD(RegisterRemoteTalker)(
                                                THIS_
        __in XUID                               xuidRemoteTalker,
        __in_opt XAUDIO2_EFFECT_CHAIN*          pfxRemoteTalkerFX,
        __in_opt XAUDIO2_EFFECT_CHAIN*          pfxTalkerPairFX,
        __in_opt IXAudio2SubmixVoice*           pOutputVoice
        )   PURE;

    STDMETHOD(UnregisterRemoteTalker)(
                                                THIS_
        __in XUID                               xuidRemoteTalker
        )   PURE;

    STDMETHOD(GetRemoteTalkers)(
                                                THIS_
        __out PDWORD                            pdwRemoteTalkersCount,
        __out PXUID                             pxuidRemoteTalkers
        )   PURE;

    STDMETHOD_(BOOL, IsHeadsetPresent)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD_(BOOL, IsLocalTalking)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD_(BOOL, IsRemoteTalking)(
                                                THIS_
        __in XUID                               xuidRemoteTalker
        )   PURE;

    STDMETHOD(SetRemoteTalkerOutputVoice)(
                                                THIS_
        __in XUID                               xuidRemoteTalker,
        __in_opt IXAudio2SubmixVoice*           pOutputVoice
        )   PURE;

    STDMETHOD(SetRemoteTalkerEffectParam)(
                                                            THIS_
        __in XUID                                           xuidRemoteTalker,
        __in BOOL                                           fPairEffect,
        __in UINT32                                         uEffectIndex,
        __in_bcount_opt(uParametersByteSize) const void*    pParameters,
        __in UINT32                                         uParametersByteSize
        )   PURE;

    STDMETHOD_(DWORD, GetDataReadyFlags)(
                                                THIS
        )   PURE;

    //
    // Voice chat mode functions
    //

    STDMETHOD(GetLocalChatData)(
                                                THIS_
        __in                    DWORD           dwUserIndex,
        __out_bcount(*pdwSize)  PBYTE           pbData,
        __inout                 PDWORD          pdwSize,
        __out_opt               PDWORD          pdwPackets
        )   PURE;

    STDMETHOD(NuiGetLocalChatData)(
                                                THIS_
        __out_bcount_opt(*pdwSize) PBYTE        pbData,
        __inout                 PDWORD          pdwSize,
        __out_opt               PDWORD          pdwPackets,
        __out_opt               PNUI_TALKER_POSITION pPositions,
        __inout                 PDWORD          pNumPositions
        )   PURE;

    STDMETHOD(SetPlaybackPriority)(
                                                THIS_
        __in XUID                               xuidRemoteTalker,
        __in DWORD                              dwUserIndex,
        __in XHV_PLAYBACK_PRIORITY              playbackPriority
        )   PURE;

    STDMETHOD(SubmitIncomingChatData)(
                                                THIS_
        __in XUID                               xuidRemoteTalker,
        __in_bcount(*pdwSize) CONST BYTE*       pbData,
        __inout PDWORD                          pdwSize
        )   PURE;

    STDMETHOD_(BOOL, IsSharedMicPresent)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD(GetVoiceVolume)(
                                                THIS_
        __in DWORD                              dwUserIndex,
        __out PFLOAT                            pfVoiceVolume
        )   PURE;

    STDMETHOD(GetGameVolume)(
                                                THIS_
        __out PFLOAT                            pfVoiceVolume
        )   PURE;

    STDMETHOD_(BOOL, IsVoiceOverSpeakers)(
                                                THIS_
        __in DWORD                              dwUserIndex
        )   PURE;

    STDMETHOD_(VOID, EnableDucking)(
                                                THIS_
        __in BOOL                               fEnable
        )   PURE;

    STDMETHOD_(BOOL, IsDuckingEnabled)(
                                                THIS
        )   PURE;
};

#pragma push_macro("VTBL")
#pragma push_macro("THIS")
#pragma push_macro("THIS_")

#undef  VTBL
#undef  THIS
#undef  THIS_

#if defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) (p)
#define THIS
#define THIS_
#else // defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) ((p)->lpVtbl)
#define THIS    pThis
#define THIS_   pThis,
#endif // defined(__cplusplus) && !defined(CINTERFACE)

XHVINLINE
STDMETHODIMP_(ULONG)
IXHV2Engine_AddRef(
    __in PIXHV2ENGINE                           pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

XHVINLINE
STDMETHODIMP_(ULONG)
IXHV2Engine_Release(
    __in PIXHV2ENGINE                           pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_Lock(
    __in PIXHV2ENGINE                           pThis,
    __in XHV_LOCK_TYPE                          lockType
    )
{
    return VTBL(pThis)->Lock(THIS_ lockType);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_StartLocalProcessingModes(
    __in PIXHV2ENGINE                                               pThis,
    __in DWORD                                                      dwUserIndex,
    __in_ecount(dwNumProcessingModes) CONST PXHV_PROCESSING_MODE    processingModes,
    __in DWORD                                                      dwNumProcessingModes
    )
{
    return VTBL(pThis)->StartLocalProcessingModes(THIS_ dwUserIndex, processingModes, dwNumProcessingModes);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_StopLocalProcessingModes(
    __in PIXHV2ENGINE                                               pThis,
    __in DWORD                                                      dwUserIndex,
    __in_ecount(dwNumProcessingModes) CONST PXHV_PROCESSING_MODE    processingModes,
    __in DWORD                                                      dwNumProcessingModes
    )
{
    return VTBL(pThis)->StopLocalProcessingModes(THIS_ dwUserIndex, processingModes, dwNumProcessingModes);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_StartRemoteProcessingModes(
    __in PIXHV2ENGINE                                               pThis,
    __in XUID                                                       Xuid,
    __in_ecount(dwNumProcessingModes) CONST PXHV_PROCESSING_MODE    processingModes,
    __in DWORD                                                      dwNumProcessingModes
    )
{
    return VTBL(pThis)->StartRemoteProcessingModes(THIS_ Xuid, processingModes, dwNumProcessingModes);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_StopRemoteProcessingModes(
    __in PIXHV2ENGINE                                               pThis,
    __in XUID                                                       Xuid,
    __in_ecount(dwNumProcessingModes) CONST PXHV_PROCESSING_MODE    processingModes,
    __in DWORD                                                      dwNumProcessingModes
    )
{
    return VTBL(pThis)->StopRemoteProcessingModes(THIS_ Xuid, processingModes, dwNumProcessingModes);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_SetMaxDecodePackets(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwMaxDecodePackets
    )
{
    return VTBL(pThis)->SetMaxDecodePackets(THIS_ dwMaxDecodePackets);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_RegisterLocalTalker(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->RegisterLocalTalker(THIS_ dwUserIndex);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_UnregisterLocalTalker(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->UnregisterLocalTalker(THIS_ dwUserIndex);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_RegisterRemoteTalker(
    __in PIXHV2ENGINE                           pThis,
    __in XUID                                   xuidRemoteTalker,
    __in_opt XAUDIO2_EFFECT_CHAIN*              pfxRemoteTalkerFX,
    __in_opt XAUDIO2_EFFECT_CHAIN*              pfxTalkerPairFX,
    __in_opt IXAudio2SubmixVoice*               pOutputVoice
    )
{
    return VTBL(pThis)->RegisterRemoteTalker(THIS_ xuidRemoteTalker, pfxRemoteTalkerFX, pfxTalkerPairFX, pOutputVoice);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_UnregisterRemoteTalker(
    __in PIXHV2ENGINE                           pThis,
    __in XUID                                   xuidRemoteTalker
    )
{
    return VTBL(pThis)->UnregisterRemoteTalker(THIS_ xuidRemoteTalker);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_GetRemoteTalkers(
    __in PIXHV2ENGINE                           pThis,
    __inout PDWORD                              pdwRemoteTalkersCount,
    __out_ecount(*pdwRemoteTalkersCount) PXUID  pxuidRemoteTalkers
    )
{
    return VTBL(pThis)->GetRemoteTalkers(THIS_ pdwRemoteTalkersCount, pxuidRemoteTalkers);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXHV2Engine_IsHeadsetPresent(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->IsHeadsetPresent(THIS_ dwUserIndex);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXHV2Engine_IsSharedMicPresent(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->IsSharedMicPresent(THIS_ dwUserIndex);
}

XHVINLINE
STDMETHODIMP_(VOID)
IXHV2Engine_EnableDucking(
    __in PIXHV2ENGINE                           pThis,
    __in BOOL                                   fEnable
    )
{
    VTBL(pThis)->EnableDucking(THIS_ fEnable);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXH2VEngine_IsDuckingEnabled(
    __in PIXHV2ENGINE                           pThis
    )
{
    return VTBL(pThis)->IsDuckingEnabled(THIS);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXHV2Engine_IsLocalTalking(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->IsLocalTalking(THIS_ dwUserIndex);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXHV2Engine_IsRemoteTalking(
    __in PIXHV2ENGINE                           pThis,
    __in XUID                                   xuidRemoteTalker
    )
{
    return VTBL(pThis)->IsRemoteTalking(THIS_ xuidRemoteTalker);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_SetRemoteTalkerOutputVoice(
    __in PIXHV2ENGINE                           pThis,
    __in XUID                                   xuidRemoteTalker,
    __in_opt IXAudio2SubmixVoice*               pOutputVoice
    )
{
    return VTBL(pThis)->SetRemoteTalkerOutputVoice(THIS_ xuidRemoteTalker, pOutputVoice);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_SetRemoteTalkerEffectParam(
    __in PIXHV2ENGINE                                   pThis,
    __in XUID                                           xuidRemoteTalker,
    __in BOOL                                           fPairEffect,
    __in UINT32                                         uEffectIndex,
    __in_bcount_opt(uParametersByteSize) const void*    pParameters,
    __in UINT32                                         uParametersByteSize
    )
{
    return VTBL(pThis)->SetRemoteTalkerEffectParam(THIS_ xuidRemoteTalker, fPairEffect, uEffectIndex, pParameters, uParametersByteSize);
}

XHVINLINE
STDMETHODIMP_(DWORD)
IXHV2Engine_GetDataReadyFlags(
    __in PIXHV2ENGINE                           pThis
    )
{
    return VTBL(pThis)->GetDataReadyFlags(THIS);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_GetLocalChatData(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex,
    __out_bcount(*pdwSize) PBYTE                pbData,
    __inout PDWORD                              pdwSize,
    __out_opt PDWORD                            pdwPackets
    )
{
    return VTBL(pThis)->GetLocalChatData(THIS_ dwUserIndex, pbData, pdwSize, pdwPackets);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_NuiGetLocalChatData(
    __in PIXHV2ENGINE                           pThis,
    __out_bcount_opt(*pdwSize) PBYTE            pbData,
    __inout PDWORD                              pdwSize,
    __out_opt PDWORD                            pdwPackets,
    __out_opt PNUI_TALKER_POSITION              pPositions,
    __inout PDWORD                              pNumPositions
    )
{
    return VTBL(pThis)->NuiGetLocalChatData(THIS_ pbData, pdwSize, pdwPackets, pPositions, pNumPositions);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_SetPlaybackPriority(
    __in PIXHV2ENGINE                           pThis,
    __in XUID                                   xuidRemoteTalker,
    __in DWORD                                  dwUserIndex,
    __in XHV_PLAYBACK_PRIORITY                  playbackPriority
    )
{
    return VTBL(pThis)->SetPlaybackPriority(THIS_ xuidRemoteTalker, dwUserIndex, playbackPriority);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_SubmitIncomingChatData(
    __in PIXHV2ENGINE                       pThis,
    __in XUID                               xuidRemoteTalker,
    __in_bcount(*pdwSize) CONST BYTE*       pbData,
    __inout PDWORD                          pdwSize
    )
{
    return VTBL(pThis)->SubmitIncomingChatData(THIS_ xuidRemoteTalker, pbData, pdwSize);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_GetVoiceVolume(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex,
    __out PFLOAT                                pfVoiceVolume
    )
{
    return VTBL(pThis)->GetVoiceVolume(THIS_ dwUserIndex, pfVoiceVolume);
}

XHVINLINE
STDMETHODIMP
IXHV2Engine_GetGameVolume(
    __in PIXHV2ENGINE                           pThis,
    __out PFLOAT                                pfVoiceVolume
    )
{
    return VTBL(pThis)->GetGameVolume(THIS_ pfVoiceVolume);
}

XHVINLINE
STDMETHODIMP_(BOOL)
IXHV2Engine_IsVoiceOverSpeakers(
    __in PIXHV2ENGINE                           pThis,
    __in DWORD                                  dwUserIndex
    )
{
    return VTBL(pThis)->IsVoiceOverSpeakers(THIS_ dwUserIndex);
}

#pragma pop_macro("VTBL")
#pragma pop_macro("THIS")
#pragma pop_macro("THIS_")

//
// Use the following function to create an instance of the XHV2 engine.
// Note only one engine instance may exist at a time.
//

XBOXAPI
HRESULT
WINAPI
XHV2CreateEngine(
    __in PXHV_INIT_PARAMS                   pParams,
    __out_opt PHANDLE                       phWorkerThread,
    __out PIXHV2ENGINE*                     ppEngine
    );



#ifdef __cplusplus
}
#endif



#endif // __XHV2_H__

