/**************************************************************************
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 *
 * Module Name:    xinput2.h
 * 
 * Abstract:
 * 
 *     XInput2 API
 *
 **************************************************************************/

#ifndef __XINPUT2_H__
#define __XINPUT2_H__

#include <sal.h>

typedef PVOID XINPUT2CONTEXT;

typedef struct _XINPUTID {
    unsigned __int64 Data1;
    unsigned __int64 Data2;

#ifdef __cplusplus
    int operator == (const _XINPUTID& other) const
    {
        return ! memcmp(this, &other, sizeof(*this));
    }

    int operator != (const _XINPUTID& other) const
    {
        return memcmp(this, &other, sizeof(*this));
    }
#endif // __cplusplus

} XINPUTID;


#ifdef __cplusplus
#endif // __cplusplus


#define IMPLEMENT_XINPUTID(NAME, ORDINAL, L1, L2) \
    const XINPUTID NAME \
    = { L1, L2 };

#define DEFINE_XINPUTID(NAME) \
    extern const XINPUTID NAME;

#ifdef __cplusplus

// Faster (but makes code fatter) inline version...use sparingly
__inline int InlineXInput2EqualId(const XINPUTID& id1, const XINPUTID& id2)
{
    return (id1.Data1 == id2.Data1) && (id1.Data2 == id2.Data2);
}

//
//  inline XINPUTID comparisons
//

__inline int XInput2EqualId(const XINPUTID& id1, const XINPUTID& id2)
{
    return !memcmp(&id1, &id2, sizeof(XINPUTID));
}

#else   // ! __cplusplus

#define InlineXInput2EqualId(id1, id2)  \
    (((unsigned __int64 *) &id1)[0] == ((unsigned __int64 *) &id2)[0] &&   \
     ((unsigned __int64 *) &id1)[1] == ((unsigned __int64 *) &id2)[1] )

#define XInput2EqualId(id1, id2) \
    (!memcmp(&id1, &id2, sizeof(XINPUTID)))

#endif  // __cplusplus


//
//  types returned by XInput2GetType
//

#define XINPUT2_TYPE_UNKNOWN  0
#define XINPUT2_TYPE_BOOL     1
#define XINPUT2_TYPE_DWORD    2
#define XINPUT2_TYPE_FLOAT    3
#define XINPUT2_TYPE_POINT    4
#define XINPUT2_TYPE_VECTOR  5
#define XINPUT2_TYPE_RESERVED 6
#define XINPUT2_TYPE_VECTOR4  7

//
//  flags returned by XInput2GetType
//

#define XINPUT2_CONTROL_IN      0x00000001
#define XINPUT2_CONTROL_OUT     0x00000002


#ifdef __cplusplus
extern "C" {
#endif

//
//  XInput2-specific types
//

#ifndef D3DVECTOR_DEFINED
typedef struct _D3DVECTOR {
    float x;
    float y;
    float z;
} D3DVECTOR;
#define D3DVECTOR_DEFINED
#endif

typedef D3DVECTOR XINPUT2_VECTOR;
typedef __vector4 XINPUT2_VECTOR4;

typedef struct _XINPUT2_POINT
{
	FLOAT	x;
	FLOAT	y;
} XINPUT2_POINT, *PXINPUT2_POINT;


//
//  Sample the device.  Returns FALSE if there is no current sample
//  available.  The context must be passed to subsequent 'getter'
//  and 'setter' functions.
//
XBOXAPI
BOOL
WINAPI
XInput2Sample(
    __in      DWORD                       dwDeviceIndex,
    __out     XINPUT2CONTEXT*             pContext,
    __out     PBOOL                       pChanged
    );

XBOXAPI
BOOL
WINAPI
XInput2GetDeviceId(
    __in      XINPUT2CONTEXT              context,
    __out     XINPUTID*                   pId
    );

XBOXAPI
BOOL
WINAPI
XInput2HasControl(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    controlId
    );

XBOXAPI
BOOL
WINAPI
XInput2GetControl(
    __in      XINPUT2CONTEXT              context,
    __in      DWORD                       dwIndex,
    __out     XINPUTID*                   pId
    );

XBOXAPI
BOOL
WINAPI
XInput2HasProperty(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    controlId
    );

XBOXAPI
BOOL
WINAPI
XInput2GetProperty(
    __in      XINPUT2CONTEXT              context,
    __in      DWORD                       dwIndex,
    __out     XINPUTID*                   pId
    );

XBOXAPI
BOOL
WINAPI
XInput2GetType(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     PDWORD                      pType,
    __out     PDWORD                      pFlags
    );

XBOXAPI
BOOL
WINAPI
XInput2GetBool(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     PBOOL                       pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2GetDWord(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     PDWORD                      pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2GetFloat(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     float*                      pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2GetPoint(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     XINPUT2_POINT*              pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2GetVector(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     XINPUT2_VECTOR*            pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2GetVector4(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __out     XINPUT2_VECTOR4*            pValue
    );

XBOXAPI
BOOL
WINAPI
XInput2BeginUpdate(
    __in      XINPUT2CONTEXT              context
    );

XBOXAPI
BOOL
WINAPI
XInput2EndUpdate(
    __in         XINPUT2CONTEXT           context,
    __inout_opt  XOVERLAPPED*             pOverlapped
    );

XBOXAPI
BOOL
WINAPI
XInput2SetBool(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      BOOL                        value
    );

XBOXAPI
BOOL
WINAPI
XInput2SetDWord(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      DWORD                       value
    );

XBOXAPI
BOOL
WINAPI
XInput2SetFloat(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      float                       value
    );

XBOXAPI
BOOL
WINAPI
XInput2SetPoint(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      XINPUT2_POINT               value
    );

XBOXAPI
BOOL
WINAPI
XInput2SetVector(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      XINPUT2_VECTOR              value
    );

XBOXAPI
BOOL
WINAPI
XInput2SetVector4(
    __in      XINPUT2CONTEXT              context,
    __in      XINPUTID                    inputId,
    __in      XINPUT2_VECTOR4             value
    );

//
//  Device category support.
//
//  The default devices are controllers; this provides access to
//  non-controller devices (e.g., microphones) that may also have input
//  controls
//

#define XINPUT2_USERINDEX_MASK			0x000000ff
#define XINPUT2_CATEGORY_MASK			0xff000000
#define XINPUT2_CATEGORY_SHIFT			24

#define XINPUT2_CATEGORY_INPUT          0x00000000
#define XINPUT2_CATEGORY_VOICE          0x01000000
#define XINPUT2_CATEGORY_TEXT           0x02000000
#define XINPUT2_CATEGORY_BIGBUTTON      0x03000000
#define XINPUT2_CATEGORY_MIC            0x04000000
#define XINPUT2_CATEGORY_IR             0x05000000
#define XINPUT2_CATEGORY_MAX			0x06000000


#ifdef __cplusplus
__inline DWORD XInput2InputDevice(DWORD dwUserIndex)
{
    return dwUserIndex | XINPUT2_CATEGORY_INPUT;
}

__inline DWORD XInput2VoiceDevice(DWORD dwUserIndex)
{
    return dwUserIndex | XINPUT2_CATEGORY_VOICE;
}

__inline DWORD XInput2TextDevice(DWORD dwUserIndex)
{
    return dwUserIndex | XINPUT2_CATEGORY_TEXT;
}

__inline DWORD XInput2MicDevice(DWORD dwUserIndex)
{
    return dwUserIndex | XINPUT2_CATEGORY_MIC;
}

__inline DWORD XInput2BigButtonDevice(DWORD dwUserIndex)
{
    return dwUserIndex | XINPUT2_CATEGORY_BIGBUTTON;
}

#else

#define XInput2InputDevice(USERINDEX) ((USERINDEX) | XINPUT2_CATEGORY_INPUT)
#define XInput2VoiceDevice(USERINDEX) ((USERINDEX) | XINPUT2_CATEGORY_VOICE)
#define XInput2TextDevice(USERINDEX) ((USERINDEX) | XINPUT2_CATEGORY_TEXT)
#define XInput2MicDevice(USERINDEX) ((USERINDEX) | XINPUT2_CATEGORY_MIC)

#endif // __cplusplus

DEFINE_XINPUTID(XINPUTID_DPAD_UP);
DEFINE_XINPUTID(XINPUTID_DPAD_DOWN);
DEFINE_XINPUTID(XINPUTID_DPAD_LEFT);
DEFINE_XINPUTID(XINPUTID_DPAD_RIGHT);
DEFINE_XINPUTID(XINPUTID_START);
DEFINE_XINPUTID(XINPUTID_BACK);
DEFINE_XINPUTID(XINPUTID_LEFT_THUMB);
DEFINE_XINPUTID(XINPUTID_RIGHT_THUMB);
DEFINE_XINPUTID(XINPUTID_LEFT_SHOULDER);
DEFINE_XINPUTID(XINPUTID_RIGHT_SHOULDER);
DEFINE_XINPUTID(XINPUTID_BIGBUTTON);
DEFINE_XINPUTID(XINPUTID_A);
DEFINE_XINPUTID(XINPUTID_B);
DEFINE_XINPUTID(XINPUTID_X);
DEFINE_XINPUTID(XINPUTID_Y);
DEFINE_XINPUTID(XINPUTID_BUTTONS);
DEFINE_XINPUTID(XINPUTID_DIRECTION);
DEFINE_XINPUTID(XINPUTID_DIRECTION8);
DEFINE_XINPUTID(XINPUTID_LEFT_TRIGGER);
DEFINE_XINPUTID(XINPUTID_RIGHT_TRIGGER);
DEFINE_XINPUTID(XINPUTID_PRIMARY_2D);
DEFINE_XINPUTID(XINPUTID_LEFT_THUMBSTICK);
DEFINE_XINPUTID(XINPUTID_SECONDARY_2D);
DEFINE_XINPUTID(XINPUTID_RIGHT_THUMBSTICK);
DEFINE_XINPUTID(XINPUTID_ACCELERATION);
DEFINE_XINPUTID(XINPUTID_RESERVED0026);
DEFINE_XINPUTID(XINPUTID_RESERVED0027);
DEFINE_XINPUTID(XINPUTID_RESERVED0028);
DEFINE_XINPUTID(XINPUTID_VIBRATION);
DEFINE_XINPUTID(XINPUTID_LEDS);
DEFINE_XINPUTID(XINPUTID_LEFT_THUMBSTICK_DEADZONE);
DEFINE_XINPUTID(XINPUTID_RIGHT_THUMBSTICK_DEADZONE);
DEFINE_XINPUTID(XINPUTID_PRIMARY_2D_DEADZONE);
DEFINE_XINPUTID(XINPUTID_SECONDARY_2D_DEADZONE);
DEFINE_XINPUTID(XINPUTID_TRIGGER_THRESHOLD);
DEFINE_XINPUTID(XINPUTID_GAMEPAD);
DEFINE_XINPUTID(XINPUTID_GUITAR);
DEFINE_XINPUTID(XINPUTID_DRUM_KIT);
DEFINE_XINPUTID(XINPUTID_WHEEL);
DEFINE_XINPUTID(XINPUTID_ARCADE_STICK);
DEFINE_XINPUTID(XINPUTID_FLIGHT_STICK);
DEFINE_XINPUTID(XINPUTID_DANCEPAD);
DEFINE_XINPUTID(XINPUTID_RESERVED0043);
DEFINE_XINPUTID(XINPUTID_VOICE);
DEFINE_XINPUTID(XINPUTID_RESERVED0045);
DEFINE_XINPUTID(XINPUTID_RESERVED0046);
DEFINE_XINPUTID(XINPUTID_RESERVED0047);
DEFINE_XINPUTID(XINPUTID_RESERVED0048);
DEFINE_XINPUTID(XINPUTID_RESERVED0049);
DEFINE_XINPUTID(XINPUTID_ACCELERATOR);
DEFINE_XINPUTID(XINPUTID_BRAKE);
DEFINE_XINPUTID(XINPUTID_CLUTCH);
DEFINE_XINPUTID(XINPUTID_DOWN);
DEFINE_XINPUTID(XINPUTID_DRUM_BLUE);
DEFINE_XINPUTID(XINPUTID_DRUM_EXTRA_1);
DEFINE_XINPUTID(XINPUTID_DRUM_EXTRA_2);
DEFINE_XINPUTID(XINPUTID_DRUM_EXTRA_3);
DEFINE_XINPUTID(XINPUTID_DRUM_GREEN);
DEFINE_XINPUTID(XINPUTID_DRUM_PEDAL);
DEFINE_XINPUTID(XINPUTID_DRUM_RED);
DEFINE_XINPUTID(XINPUTID_DRUM_YELLOW);
DEFINE_XINPUTID(XINPUTID_GEAR_DOWN);
DEFINE_XINPUTID(XINPUTID_GEAR_UP);
DEFINE_XINPUTID(XINPUTID_GUITAR_LEFT_EFFECT);
DEFINE_XINPUTID(XINPUTID_GUITAR_RIGHT_EFFECT);
DEFINE_XINPUTID(XINPUTID_H_SHIFT);
DEFINE_XINPUTID(XINPUTID_H_SHIFT_FORWARD_GEARS);
DEFINE_XINPUTID(XINPUTID_H_SHIFT_REVERSE_GEARS);
DEFINE_XINPUTID(XINPUTID_H_SHIFTER);
DEFINE_XINPUTID(XINPUTID_HANDBRAKE);
DEFINE_XINPUTID(XINPUTID_JOYSTICK_POSITION);
DEFINE_XINPUTID(XINPUTID_JOYSTICK_TRIGGER);
DEFINE_XINPUTID(XINPUTID_LEFT);
DEFINE_XINPUTID(XINPUTID_GUITAR_EFFECT3);
DEFINE_XINPUTID(XINPUTID_NECK_BLUE);
DEFINE_XINPUTID(XINPUTID_NECK_GREEN);
DEFINE_XINPUTID(XINPUTID_NECK_ORANGE);
DEFINE_XINPUTID(XINPUTID_NECK_ORIENTATION);
DEFINE_XINPUTID(XINPUTID_NECK_RED);
DEFINE_XINPUTID(XINPUTID_NECK_YELLOW);
DEFINE_XINPUTID(XINPUTID_POV_POSITION);
DEFINE_XINPUTID(XINPUTID_RIGHT);
DEFINE_XINPUTID(XINPUTID_RESERVED0083);
DEFINE_XINPUTID(XINPUTID_RIGHT_STICK);
DEFINE_XINPUTID(XINPUTID_RUDDER_POSITION);
DEFINE_XINPUTID(XINPUTID_STRUM_DOWN);
DEFINE_XINPUTID(XINPUTID_STRUM_UP);
DEFINE_XINPUTID(XINPUTID_THROTTLE_POSITION);
DEFINE_XINPUTID(XINPUTID_TRIGGER);
DEFINE_XINPUTID(XINPUTID_UP);
DEFINE_XINPUTID(XINPUTID_WHAMMY_BAR);
DEFINE_XINPUTID(XINPUTID_WHEEL_FREEDOM);
DEFINE_XINPUTID(XINPUTID_TIMESTAMP);
DEFINE_XINPUTID(XINPUTID_MIC);
DEFINE_XINPUTID(XINPUTID_RESERVED0095);
DEFINE_XINPUTID(XINPUTID_HAT_UP);
DEFINE_XINPUTID(XINPUTID_HAT_DOWN);
DEFINE_XINPUTID(XINPUTID_HAT_LEFT);
DEFINE_XINPUTID(XINPUTID_HAT_RIGHT);
DEFINE_XINPUTID(XINPUTID_LEFT_STICK);
DEFINE_XINPUTID(XINPUTID_NORTHEAST);
DEFINE_XINPUTID(XINPUTID_NORTHWEST);
DEFINE_XINPUTID(XINPUTID_SOUTHEAST);
DEFINE_XINPUTID(XINPUTID_SOUTHWEST);
DEFINE_XINPUTID(XINPUTID_ESTABLISH_DEVICE);
DEFINE_XINPUTID(XINPUTID_SAMPLE);
DEFINE_XINPUTID(XINPUTID_RESERVED0107);
DEFINE_XINPUTID(XINPUTID_WHEEL_MULTITURN);
DEFINE_XINPUTID(XINPUTID_CLUTCH_PRESENT);
DEFINE_XINPUTID(XINPUTID_HANDBRAKE_PRESENT);
DEFINE_XINPUTID(XINPUTID_DRUM_ORANGE);
DEFINE_XINPUTID(XINPUTID_MIDI0);
DEFINE_XINPUTID(XINPUTID_MIDI1);
DEFINE_XINPUTID(XINPUTID_PACKET_NUMBER);
DEFINE_XINPUTID(XINPUTID_1D_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_1E_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_UNSPECIFIED_DWORD_0);
DEFINE_XINPUTID(XINPUTID_UNSPECIFIED_DWORD_1);
DEFINE_XINPUTID(XINPUTID_UNSPECIFIED_DWORD_2);
DEFINE_XINPUTID(XINPUTID_UNSPECIFIED_DWORD_3);
DEFINE_XINPUTID(XINPUTID_1C_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_OUT_UNSPECIFIED_DWORD_0);
DEFINE_XINPUTID(XINPUTID_OUT_UNSPECIFIED_DWORD_1);
DEFINE_XINPUTID(XINPUTID_MODEL_FLAG);
DEFINE_XINPUTID(XINPUTID_IS_WIRELESS);
DEFINE_XINPUTID(XINPUTID_SUBTYPE);
DEFINE_XINPUTID(XINPUTID_0F_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_19_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_DEVICE_VID);
DEFINE_XINPUTID(XINPUTID_DEVICE_PID);
DEFINE_XINPUTID(XINPUTID_DEVICE_REVISION);
DEFINE_XINPUTID(XINPUTID_DEVICE_XID);
DEFINE_XINPUTID(XINPUTID_VENDOR_CONTROLLER);
DEFINE_XINPUTID(XINPUTID_PITCH);
DEFINE_XINPUTID(XINPUTID_0C_CONTROLLER);

#ifdef __cplusplus
}
#endif

#endif // __XINPUT2_H__
