/************************************************************************
*                                                                       *
* xnamathconvert.inl -- SIMD C++ Math library for Windows and Xbox 360  *
*                       Conversion, loading, and storing functions      *
*                                                                       *
* Copyright (c) Microsoft Corp. All rights reserved.                    *
*                                                                       *
************************************************************************/

#if defined(_MSC_VER) && (_MSC_VER > 1000)
#pragma once
#endif

#ifndef __XNAMATHCONVERT_INL__
#define __XNAMATHCONVERT_INL__

#define XM_PACK_FACTOR                  (FLOAT)(1 << 22)
#define XM_UNPACK_FACTOR_UNSIGNED       (FLOAT)(1 << 23)
#define XM_UNPACK_FACTOR_SIGNED         XM_PACK_FACTOR

#define XM_UNPACK_UNSIGNEDN_OFFSET(BitsX, BitsY, BitsZ, BitsW) \
                                        {-XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsX)) - 1), \
                                         -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsY)) - 1), \
                                         -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsZ)) - 1), \
                                         -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsW)) - 1)}

#define XM_UNPACK_UNSIGNEDN_SCALE(BitsX, BitsY, BitsZ, BitsW) \
                                        {XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsX)) - 1), \
                                         XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsY)) - 1), \
                                         XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsZ)) - 1), \
                                         XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << (BitsW)) - 1)}

#define XM_UNPACK_SIGNEDN_SCALE(BitsX, BitsY, BitsZ, BitsW) \
                                        {-XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsX) - 1)) - 1), \
                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsY) - 1)) - 1), \
                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsZ) - 1)) - 1), \
                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsW) - 1)) - 1)}

//#define XM_UNPACK_SIGNEDN_OFFSET(BitsX, BitsY, BitsZ, BitsW) \
//                                        {-XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsX) - 1)) - 1) * 3.0f, \
//                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsY) - 1)) - 1) * 3.0f, \
//                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsZ) - 1)) - 1) * 3.0f, \
//                                         -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << ((BitsW) - 1)) - 1) * 3.0f}

#define XM_PACK_UNSIGNEDN_SCALE(BitsX, BitsY, BitsZ, BitsW) \
                                        {-(FLOAT)((1 << (BitsX)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << (BitsY)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << (BitsZ)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << (BitsW)) - 1) / XM_PACK_FACTOR}

#define XM_PACK_SIGNEDN_SCALE(BitsX, BitsY, BitsZ, BitsW) \
                                        {-(FLOAT)((1 << ((BitsX) - 1)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << ((BitsY) - 1)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << ((BitsZ) - 1)) - 1) / XM_PACK_FACTOR, \
                                         -(FLOAT)((1 << ((BitsW) - 1)) - 1) / XM_PACK_FACTOR}

#define XM_PACK_OFFSET                  XMVectorSplatConstant(3, 0)
//#define XM_UNPACK_OFFSET                XM_PACK_OFFSET

/****************************************************************************
 *
 * Data conversion
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE FLOAT XMConvertHalfToFloat
(
    HALF Value
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)

    UINT Mantissa;
    UINT Exponent;
    UINT Result;

    Mantissa = (UINT)(Value & 0x03FF);

    if ((Value & 0x7C00) != 0)  // The value is normalized
    {
        Exponent = (UINT)((Value >> 10) & 0x1F);
    }
    else if (Mantissa != 0)     // The value is denormalized
    {
        // Normalize the value in the resulting float
        Exponent = 1;

        do
        {
            Exponent--;
            Mantissa <<= 1;
        } while ((Mantissa & 0x0400) == 0);

        Mantissa &= 0x03FF;
    }
    else                        // The value is zero
    {
        Exponent = (UINT)-112;
    }

    Result = ((Value & 0x8000) << 16) | // Sign
             ((Exponent + 112) << 23) | // Exponent
             (Mantissa << 13);          // Mantissa

    return *(FLOAT*)&Result;

#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR ValueV;
    XMVECTOR Permute;
    FLOAT Result;

    XMASSERT(((UINT_PTR)&Value & 1) == 0);

    Permute = __lvsl(&Value, 0);
    ValueV = __lvx(&Value, 0);
    Permute = __vsldoi(Permute, Permute, 1 << 2);
    ValueV = __vperm(ValueV, ValueV, Permute);
    ValueV = __vupkd3d(ValueV, VPACK_FLOAT16_2);
    ValueV = __vspltw(ValueV, 0);

    __stvewx(ValueV, &Result, 0);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR ValueV;
    FLOAT Result;

    XMASSERT(((UINT_PTR)&Value & 1) == 0);

    ValueV = __lvlx(&Value, 0);
    ValueV = __vsplth(ValueV, 0);
    ValueV = __vupkd3d(ValueV, VPACK_FLOAT16_4);

    __stvewx(ValueV, &Result, 0);

    return Result;
#endif
}

//------------------------------------------------------------------------------

XMINLINE FLOAT* XMConvertHalfToFloatStream
(
    FLOAT*      pOutputStream, 
    size_t      OutputStride, 
    CONST HALF* pInputStream, 
    size_t      InputStride, 
    size_t      HalfCount
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)

    size_t  i;
    CONST BYTE* pHalf = (CONST BYTE*)pInputStream;
    BYTE* pFloat = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < HalfCount; i++)
    {
        *(FLOAT*)pFloat = XMConvertHalfToFloat(*(const HALF*)pHalf);
        pHalf += InputStride;
        pFloat += OutputStride; 
    }

    return pOutputStream;

#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)

    UINT  i;
    BYTE* pHalf = (BYTE*)pInputStream;
    BYTE* pFloat = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < HalfCount; i++)
    {
        *(FLOAT*)pFloat = XMConvertHalfToFloat(*(HALF*)pHalf);
        pHalf += InputStride;
        pFloat += OutputStride; 
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR V0[8], V1[8], V2[8], V3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 1) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    if (InputStride == sizeof(HALF))
    {
        if (OutputStride == sizeof(FLOAT))
        {
            UINT GroupIndex;
            CONST UINT VGroupSize = 2 * sizeof(XMVECTOR);
            UINT_PTR OutputStreamEnd = OutputVector + HalfCount * sizeof(FLOAT);
            UINT_PTR OutputStreamCGroupBase = XMMin(OutputVector + (XM_CACHE_LINE_SIZE - 1), OutputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
            UINT_PTR OutputStreamCGroupEnd = OutputStreamCGroupBase + ((OutputStreamEnd - OutputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
            UINT_PTR OutputStreamVGroupBase = XMMin(OutputVector + (VGroupSize - 1), OutputStreamEnd) & ~(VGroupSize - 1);
            UINT_PTR OutputStreamVGroupEnd = OutputStreamEnd & ~(VGroupSize - 1);

            __dcbt(0,                      (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)OutputStreamCGroupBase);

            while (OutputVector < OutputStreamVGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vsplth(V0[0], 0);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvewx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(HALF); 
                OutputVector += sizeof(FLOAT);
            }

            while (OutputVector < OutputStreamCGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V1[0] = __lvrx((VOID*)InputVector, 16);
                V0[0] = __vor(V0[0], V1[0]);
                InputVector += 8 * sizeof(HALF); 

                V1[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);
                V0[0] = __vsldoi(V0[0], V0[0], 2 << 2);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }

            while (OutputVector < OutputStreamCGroupEnd)
            {
                __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)OutputVector);

                for (GroupIndex = 0; GroupIndex < 2; GroupIndex++)
                {
                    V0[0] = __lvlx((VOID*)InputVector, 0);
                    V1[0] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[1] = __lvlx((VOID*)InputVector, 0);
                    V1[1] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[2] = __lvlx((VOID*)InputVector, 0);
                    V1[2] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[3] = __lvlx((VOID*)InputVector, 0);
                    V1[3] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[4] = __lvlx((VOID*)InputVector, 0);
                    V1[4] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[5] = __lvlx((VOID*)InputVector, 0);
                    V1[5] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[6] = __lvlx((VOID*)InputVector, 0);
                    V1[6] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 
                    V0[7] = __lvlx((VOID*)InputVector, 0);
                    V1[7] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 8 * sizeof(HALF); 

                    V0[0] = __vor(V0[0], V1[0]);
                    V0[1] = __vor(V0[1], V1[1]);
                    V0[2] = __vor(V0[2], V1[2]);
                    V0[3] = __vor(V0[3], V1[3]);
                    V0[4] = __vor(V0[4], V1[4]);
                    V0[5] = __vor(V0[5], V1[5]);
                    V0[6] = __vor(V0[6], V1[6]);
                    V0[7] = __vor(V0[7], V1[7]);

                    V1[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);
                    V1[1] = __vupkd3d(V0[1], VPACK_FLOAT16_4);
                    V1[2] = __vupkd3d(V0[2], VPACK_FLOAT16_4);
                    V1[3] = __vupkd3d(V0[3], VPACK_FLOAT16_4);
                    V1[4] = __vupkd3d(V0[4], VPACK_FLOAT16_4);
                    V1[5] = __vupkd3d(V0[5], VPACK_FLOAT16_4);
                    V1[6] = __vupkd3d(V0[6], VPACK_FLOAT16_4);
                    V1[7] = __vupkd3d(V0[7], VPACK_FLOAT16_4);

                    V0[0] = __vsldoi(V0[0], V0[0], 2 << 2);
                    V0[1] = __vsldoi(V0[1], V0[1], 2 << 2);
                    V0[2] = __vsldoi(V0[2], V0[2], 2 << 2);
                    V0[3] = __vsldoi(V0[3], V0[3], 2 << 2);
                    V0[4] = __vsldoi(V0[4], V0[4], 2 << 2);
                    V0[5] = __vsldoi(V0[5], V0[5], 2 << 2);
                    V0[6] = __vsldoi(V0[6], V0[6], 2 << 2);
                    V0[7] = __vsldoi(V0[7], V0[7], 2 << 2);

                    V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);
                    V0[1] = __vupkd3d(V0[1], VPACK_FLOAT16_4);
                    V0[2] = __vupkd3d(V0[2], VPACK_FLOAT16_4);
                    V0[3] = __vupkd3d(V0[3], VPACK_FLOAT16_4);
                    V0[4] = __vupkd3d(V0[4], VPACK_FLOAT16_4);
                    V0[5] = __vupkd3d(V0[5], VPACK_FLOAT16_4);
                    V0[6] = __vupkd3d(V0[6], VPACK_FLOAT16_4);
                    V0[7] = __vupkd3d(V0[7], VPACK_FLOAT16_4);

                    __stvx(V0[0], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[0], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[1], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[1], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[2], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[2], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[3], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[3], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[4], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[4], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[5], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[5], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[6], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[6], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);

                    __stvx(V0[7], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V1[7], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                }
            }

            while (OutputVector < OutputStreamVGroupEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V1[0] = __lvrx((VOID*)InputVector, 16);
                V0[0] = __vor(V0[0], V1[0]);
                InputVector += 8 * sizeof(HALF); 

                V1[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);
                V0[0] = __vsldoi(V0[0], V0[0], 2 << 2);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }

            while (OutputVector < OutputStreamEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vsplth(V0[0], 0);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvewx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(HALF); 
                OutputVector += sizeof(FLOAT);
            }
        }
        else
        {
            UINT GroupIndex;
            CONST UINT VGroupSize = sizeof(XMVECTOR);
            XMVECTOR V4[8], V5[8], V6[8], V7[8];
            UINT_PTR InputStreamEnd = InputVector + HalfCount * sizeof(HALF);
            UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
            UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
            UINT_PTR InputStreamVGroupBase = XMMin(InputVector + (VGroupSize - 1), InputStreamEnd) & ~(VGroupSize - 1);
            UINT_PTR InputStreamVGroupEnd = InputStreamEnd & ~(VGroupSize - 1);

            __dcbt(0,                      (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

            while (InputVector < InputStreamVGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vsplth(V0[0], 0);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvewx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(HALF); 
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamCGroupBase)
            {
                V3[0] = __lvx((VOID*)InputVector, 0);
                InputVector += 8 * sizeof(HALF); 

                V7[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
                V3[0] = __vsldoi(V3[0], V3[0], 2 << 2);
                V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);

                V4[0] = __vspltw(V7[0], 0);
                V5[0] = __vspltw(V7[0], 1);
                V6[0] = __vspltw(V7[0], 2);
                V7[0] = __vspltw(V7[0], 3);

                V0[0] = __vspltw(V3[0], 0);
                V1[0] = __vspltw(V3[0], 1);
                V2[0] = __vspltw(V3[0], 2);
                V3[0] = __vspltw(V3[0], 3);

                __stvewx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V2[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;

                __stvewx(V4[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V5[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V6[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V7[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamCGroupEnd)
            {
                __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

                for (GroupIndex = 0; GroupIndex < 2; GroupIndex++)
                {
                    V3[0] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[1] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[2] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[3] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[4] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[5] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[6] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 
                    V3[7] = __lvx((VOID*)InputVector, 0);
                    InputVector += sizeof(XMVECTOR); 

                    V7[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
                    V7[1] = __vupkd3d(V3[1], VPACK_FLOAT16_4);
                    V7[2] = __vupkd3d(V3[2], VPACK_FLOAT16_4);
                    V7[3] = __vupkd3d(V3[3], VPACK_FLOAT16_4);
                    V7[4] = __vupkd3d(V3[4], VPACK_FLOAT16_4);
                    V7[5] = __vupkd3d(V3[5], VPACK_FLOAT16_4);
                    V7[6] = __vupkd3d(V3[6], VPACK_FLOAT16_4);
                    V7[7] = __vupkd3d(V3[7], VPACK_FLOAT16_4);

                    V3[0] = __vsldoi(V3[0], V3[0], 2 << 2);
                    V3[1] = __vsldoi(V3[1], V3[1], 2 << 2);
                    V3[2] = __vsldoi(V3[2], V3[2], 2 << 2);
                    V3[3] = __vsldoi(V3[3], V3[3], 2 << 2);
                    V3[4] = __vsldoi(V3[4], V3[4], 2 << 2);
                    V3[5] = __vsldoi(V3[5], V3[5], 2 << 2);
                    V3[6] = __vsldoi(V3[6], V3[6], 2 << 2);
                    V3[7] = __vsldoi(V3[7], V3[7], 2 << 2);

                    V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
                    V3[1] = __vupkd3d(V3[1], VPACK_FLOAT16_4);
                    V3[2] = __vupkd3d(V3[2], VPACK_FLOAT16_4);
                    V3[3] = __vupkd3d(V3[3], VPACK_FLOAT16_4);
                    V3[4] = __vupkd3d(V3[4], VPACK_FLOAT16_4);
                    V3[5] = __vupkd3d(V3[5], VPACK_FLOAT16_4);
                    V3[6] = __vupkd3d(V3[6], VPACK_FLOAT16_4);
                    V3[7] = __vupkd3d(V3[7], VPACK_FLOAT16_4);

                    V4[0] = __vspltw(V7[0], 0);
                    V5[0] = __vspltw(V7[0], 1);
                    V6[0] = __vspltw(V7[0], 2);
                    V7[0] = __vspltw(V7[0], 3);

                    V0[0] = __vspltw(V3[0], 0);
                    V1[0] = __vspltw(V3[0], 1);
                    V2[0] = __vspltw(V3[0], 2);
                    V3[0] = __vspltw(V3[0], 3);

                    V4[1] = __vspltw(V7[1], 0);
                    V5[1] = __vspltw(V7[1], 1);
                    V6[1] = __vspltw(V7[1], 2);
                    V7[1] = __vspltw(V7[1], 3);

                    V0[1] = __vspltw(V3[1], 0);
                    V1[1] = __vspltw(V3[1], 1);
                    V2[1] = __vspltw(V3[1], 2);
                    V3[1] = __vspltw(V3[1], 3);

                    V4[2] = __vspltw(V7[2], 0);
                    V5[2] = __vspltw(V7[2], 1);
                    V6[2] = __vspltw(V7[2], 2);
                    V7[2] = __vspltw(V7[2], 3);

                    V0[2] = __vspltw(V3[2], 0);
                    V1[2] = __vspltw(V3[2], 1);
                    V2[2] = __vspltw(V3[2], 2);
                    V3[2] = __vspltw(V3[2], 3);

                    V4[3] = __vspltw(V7[3], 0);
                    V5[3] = __vspltw(V7[3], 1);
                    V6[3] = __vspltw(V7[3], 2);
                    V7[3] = __vspltw(V7[3], 3);
                                        
                    V0[3] = __vspltw(V3[3], 0);
                    V1[3] = __vspltw(V3[3], 1);
                    V2[3] = __vspltw(V3[3], 2);
                    V3[3] = __vspltw(V3[3], 3);

                    V4[4] = __vspltw(V7[4], 0);
                    V5[4] = __vspltw(V7[4], 1);
                    V6[4] = __vspltw(V7[4], 2);
                    V7[4] = __vspltw(V7[4], 3);
                                        
                    V0[4] = __vspltw(V3[4], 0);
                    V1[4] = __vspltw(V3[4], 1);
                    V2[4] = __vspltw(V3[4], 2);
                    V3[4] = __vspltw(V3[4], 3);

                    V4[5] = __vspltw(V7[5], 0);
                    V5[5] = __vspltw(V7[5], 1);
                    V6[5] = __vspltw(V7[5], 2);
                    V7[5] = __vspltw(V7[5], 3);
                                        
                    V0[5] = __vspltw(V3[5], 0);
                    V1[5] = __vspltw(V3[5], 1);
                    V2[5] = __vspltw(V3[5], 2);
                    V3[5] = __vspltw(V3[5], 3);

                    V4[6] = __vspltw(V7[6], 0);
                    V5[6] = __vspltw(V7[6], 1);
                    V6[6] = __vspltw(V7[6], 2);
                    V7[6] = __vspltw(V7[6], 3);
                                        
                    V0[6] = __vspltw(V3[6], 0);
                    V1[6] = __vspltw(V3[6], 1);
                    V2[6] = __vspltw(V3[6], 2);
                    V3[6] = __vspltw(V3[6], 3);

                    V4[7] = __vspltw(V7[7], 0);
                    V5[7] = __vspltw(V7[7], 1);
                    V6[7] = __vspltw(V7[7], 2);
                    V7[7] = __vspltw(V7[7], 3);

                    V0[7] = __vspltw(V3[7], 0);
                    V1[7] = __vspltw(V3[7], 1);
                    V2[7] = __vspltw(V3[7], 2);
                    V3[7] = __vspltw(V3[7], 3);

                    __stvewx(V0[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V0[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V1[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V2[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V3[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvewx(V4[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V5[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V6[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvewx(V7[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                }
            }

            while (InputVector < InputStreamVGroupEnd)
            {
                V3[0] = __lvx((VOID*)InputVector, 0);
                InputVector += 8 * sizeof(HALF); 

                V7[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
                V3[0] = __vsldoi(V3[0], V3[0], 2 << 2);
                V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);

                V4[0] = __vspltw(V7[0], 0);
                V5[0] = __vspltw(V7[0], 1);
                V6[0] = __vspltw(V7[0], 2);
                V7[0] = __vspltw(V7[0], 3);

                V0[0] = __vspltw(V3[0], 0);
                V1[0] = __vspltw(V3[0], 1);
                V2[0] = __vspltw(V3[0], 2);
                V3[0] = __vspltw(V3[0], 3);

                __stvewx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V2[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;

                __stvewx(V4[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V5[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V6[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvewx(V7[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vsplth(V0[0], 0);
                V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

                __stvewx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(HALF); 
                OutputVector += OutputStride;
            }
        }
    }
    else if (OutputStride == sizeof(FLOAT))
    {
        UINT GroupIndex;
        CONST UINT VGroupSize = 2 * sizeof(XMVECTOR);
        XMVECTOR V4[8], V5[8], V6[8], V7[8];
        UINT_PTR OutputStreamEnd = OutputVector + HalfCount * sizeof(FLOAT);
        UINT_PTR OutputStreamCGroupBase = XMMin(OutputVector + (XM_CACHE_LINE_SIZE - 1), OutputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR OutputStreamCGroupEnd = OutputStreamCGroupBase + ((OutputStreamEnd - OutputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
        UINT_PTR OutputStreamVGroupBase = XMMin(OutputVector + (VGroupSize - 1), OutputStreamEnd) & ~(VGroupSize - 1);
        UINT_PTR OutputStreamVGroupEnd = OutputStreamEnd & ~(VGroupSize - 1);

        __dcbt(0,                      (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)OutputStreamCGroupBase);

        while (OutputVector < OutputStreamVGroupBase)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vsplth(V0[0], 0);
            V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

            __stvewx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += sizeof(FLOAT);
        }

        while (OutputVector < OutputStreamCGroupBase)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V1[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V2[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V3[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V4[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V5[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V6[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V7[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            V0[0] = __vmrghh(V0[0], V1[0]);
            V2[0] = __vmrghh(V2[0], V3[0]);
            V4[0] = __vmrghh(V4[0], V5[0]);
            V6[0] = __vmrghh(V6[0], V7[0]);

            V0[0] = __vrlimi(V0[0], V2[0], 0x4, 3);
            V4[0] = __vrlimi(V4[0], V6[0], 0x4, 3);

            V3[0] = __vsldoi(V0[0], V0[0], 2 << 2);
            V7[0] = __vsldoi(V4[0], V4[0], 2 << 2);

            V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
            V7[0] = __vupkd3d(V7[0], VPACK_FLOAT16_4);

            __stvx(V3[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
            __stvx(V7[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
        }

        while (OutputVector < OutputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)OutputVector);

            for (GroupIndex = 0; GroupIndex < 2; GroupIndex++)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[0] = __vmrghh(V0[0], V1[0]);
                V2[0] = __vmrghh(V2[0], V3[0]);
                V4[0] = __vmrghh(V4[0], V5[0]);
                V6[0] = __vmrghh(V6[0], V7[0]);
                V0[1] = __vmrghh(V0[1], V1[1]);
                V2[1] = __vmrghh(V2[1], V3[1]);
                V4[1] = __vmrghh(V4[1], V5[1]);
                V6[1] = __vmrghh(V6[1], V7[1]);
                V0[2] = __vmrghh(V0[2], V1[2]);
                V2[2] = __vmrghh(V2[2], V3[2]);
                V4[2] = __vmrghh(V4[2], V5[2]);
                V6[2] = __vmrghh(V6[2], V7[2]);
                V0[3] = __vmrghh(V0[3], V1[3]);
                V2[3] = __vmrghh(V2[3], V3[3]);
                V4[3] = __vmrghh(V4[3], V5[3]);
                V6[3] = __vmrghh(V6[3], V7[3]);
                V0[4] = __vmrghh(V0[4], V1[4]);
                V2[4] = __vmrghh(V2[4], V3[4]);
                V4[4] = __vmrghh(V4[4], V5[4]);
                V6[4] = __vmrghh(V6[4], V7[4]);
                V0[5] = __vmrghh(V0[5], V1[5]);
                V2[5] = __vmrghh(V2[5], V3[5]);
                V4[5] = __vmrghh(V4[5], V5[5]);
                V6[5] = __vmrghh(V6[5], V7[5]);
                V0[6] = __vmrghh(V0[6], V1[6]);
                V2[6] = __vmrghh(V2[6], V3[6]);
                V4[6] = __vmrghh(V4[6], V5[6]);
                V6[6] = __vmrghh(V6[6], V7[6]);
                V0[7] = __vmrghh(V0[7], V1[7]);
                V2[7] = __vmrghh(V2[7], V3[7]);
                V4[7] = __vmrghh(V4[7], V5[7]);
                V6[7] = __vmrghh(V6[7], V7[7]);

                V0[0] = __vrlimi(V0[0], V2[0], 0x4, 3);
                V4[0] = __vrlimi(V4[0], V6[0], 0x4, 3);
                V0[1] = __vrlimi(V0[1], V2[1], 0x4, 3);
                V4[1] = __vrlimi(V4[1], V6[1], 0x4, 3);
                V0[2] = __vrlimi(V0[2], V2[2], 0x4, 3);
                V4[2] = __vrlimi(V4[2], V6[2], 0x4, 3);
                V0[3] = __vrlimi(V0[3], V2[3], 0x4, 3);
                V4[3] = __vrlimi(V4[3], V6[3], 0x4, 3);
                V0[4] = __vrlimi(V0[4], V2[4], 0x4, 3);
                V4[4] = __vrlimi(V4[4], V6[4], 0x4, 3);
                V0[5] = __vrlimi(V0[5], V2[5], 0x4, 3);
                V4[5] = __vrlimi(V4[5], V6[5], 0x4, 3);
                V0[6] = __vrlimi(V0[6], V2[6], 0x4, 3);
                V4[6] = __vrlimi(V4[6], V6[6], 0x4, 3);
                V0[7] = __vrlimi(V0[7], V2[7], 0x4, 3);
                V4[7] = __vrlimi(V4[7], V6[7], 0x4, 3);

                V3[0] = __vsldoi(V0[0], V0[0], 2 << 2);
                V7[0] = __vsldoi(V4[0], V4[0], 2 << 2);
                V3[1] = __vsldoi(V0[1], V0[1], 2 << 2);
                V7[1] = __vsldoi(V4[1], V4[1], 2 << 2);
                V3[2] = __vsldoi(V0[2], V0[2], 2 << 2);
                V7[2] = __vsldoi(V4[2], V4[2], 2 << 2);
                V3[3] = __vsldoi(V0[3], V0[3], 2 << 2);
                V7[3] = __vsldoi(V4[3], V4[3], 2 << 2);
                V3[4] = __vsldoi(V0[4], V0[4], 2 << 2);
                V7[4] = __vsldoi(V4[4], V4[4], 2 << 2);
                V3[5] = __vsldoi(V0[5], V0[5], 2 << 2);
                V7[5] = __vsldoi(V4[5], V4[5], 2 << 2);
                V3[6] = __vsldoi(V0[6], V0[6], 2 << 2);
                V7[6] = __vsldoi(V4[6], V4[6], 2 << 2);
                V3[7] = __vsldoi(V0[7], V0[7], 2 << 2);
                V7[7] = __vsldoi(V4[7], V4[7], 2 << 2);

                V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
                V7[0] = __vupkd3d(V7[0], VPACK_FLOAT16_4);
                V3[1] = __vupkd3d(V3[1], VPACK_FLOAT16_4);
                V7[1] = __vupkd3d(V7[1], VPACK_FLOAT16_4);
                V3[2] = __vupkd3d(V3[2], VPACK_FLOAT16_4);
                V7[2] = __vupkd3d(V7[2], VPACK_FLOAT16_4);
                V3[3] = __vupkd3d(V3[3], VPACK_FLOAT16_4);
                V7[3] = __vupkd3d(V7[3], VPACK_FLOAT16_4);
                V3[4] = __vupkd3d(V3[4], VPACK_FLOAT16_4);
                V7[4] = __vupkd3d(V7[4], VPACK_FLOAT16_4);
                V3[5] = __vupkd3d(V3[5], VPACK_FLOAT16_4);
                V7[5] = __vupkd3d(V7[5], VPACK_FLOAT16_4);
                V3[6] = __vupkd3d(V3[6], VPACK_FLOAT16_4);
                V7[6] = __vupkd3d(V7[6], VPACK_FLOAT16_4);
                V3[7] = __vupkd3d(V3[7], VPACK_FLOAT16_4);
                V7[7] = __vupkd3d(V7[7], VPACK_FLOAT16_4);

                __stvx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[1], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[1], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[2], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[2], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[3], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[3], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[4], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[4], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[5], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[5], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[6], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[6], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[7], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[7], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }
        }

        while (OutputVector < OutputStreamVGroupEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V1[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V2[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V3[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V4[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V5[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V6[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V7[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            V0[0] = __vmrghh(V0[0], V1[0]);
            V2[0] = __vmrghh(V2[0], V3[0]);
            V4[0] = __vmrghh(V4[0], V5[0]);
            V6[0] = __vmrghh(V6[0], V7[0]);

            V0[0] = __vrlimi(V0[0], V2[0], 0x4, 3);
            V4[0] = __vrlimi(V4[0], V6[0], 0x4, 3);

            V3[0] = __vsldoi(V0[0], V0[0], 2 << 2);
            V7[0] = __vsldoi(V4[0], V4[0], 2 << 2);

            V3[0] = __vupkd3d(V3[0], VPACK_FLOAT16_4);
            V7[0] = __vupkd3d(V7[0], VPACK_FLOAT16_4);

            __stvx(V3[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
            __stvx(V7[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
        }

        while (OutputVector < OutputStreamEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vsplth(V0[0], 0);
            V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

            __stvewx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += sizeof(FLOAT);
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + HalfCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (HalfCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 
            V0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride; 

            V0[0] = __vsplth(V0[0], 0);
            V0[1] = __vsplth(V0[1], 0);
            V0[2] = __vsplth(V0[2], 0);
            V0[3] = __vsplth(V0[3], 0);
            V0[4] = __vsplth(V0[4], 0);
            V0[5] = __vsplth(V0[5], 0);
            V0[6] = __vsplth(V0[6], 0);
            V0[7] = __vsplth(V0[7], 0);

            V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);
            V0[1] = __vupkd3d(V0[1], VPACK_FLOAT16_4);
            V0[2] = __vupkd3d(V0[2], VPACK_FLOAT16_4);
            V0[3] = __vupkd3d(V0[3], VPACK_FLOAT16_4);
            V0[4] = __vupkd3d(V0[4], VPACK_FLOAT16_4);
            V0[5] = __vupkd3d(V0[5], VPACK_FLOAT16_4);
            V0[6] = __vupkd3d(V0[6], VPACK_FLOAT16_4);
            V0[7] = __vupkd3d(V0[7], VPACK_FLOAT16_4);

            __stvewx(V0[0], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[1], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[2], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[3], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[4], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[5], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[6], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvewx(V0[7], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vsplth(V0[0], 0);
            V0[0] = __vupkd3d(V0[0], VPACK_FLOAT16_4);

            __stvewx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE HALF XMConvertFloatToHalf
(
    FLOAT Value
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)
    UINT Result;

    UINT IValue = ((UINT *)(&Value))[0];
    UINT Sign = (IValue & 0x80000000U) >> 16U;
    IValue = IValue & 0x7FFFFFFFU;      // Hack off the sign

    if (IValue > 0x47FFEFFFU)
    {
        // The number is too large to be represented as a half.  Saturate to infinity.
        Result = 0x7FFFU;
    }
    else
    {
        if (IValue < 0x38800000U)
        {
            // The number is too small to be represented as a normalized half.
            // Convert it to a denormalized value.
            UINT Shift = 113U - (IValue >> 23U);
            IValue = (0x800000U | (IValue & 0x7FFFFFU)) >> Shift;
        }
        else
        {
            // Rebias the exponent to represent the value as a normalized half.
            IValue += 0xC8000000U;
        }

        Result = ((IValue + 0x0FFFU + ((IValue >> 13U) & 1U)) >> 13U)&0x7FFFU; 
    }
    return (HALF)(Result|Sign);

#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR ValueV;
    XMVECTOR Permute;
    HALF Result;

    XMASSERT(((UINT_PTR)&Value & 3) == 0);

    ValueV = __lvx(&Value, 0);
    Permute = __lvsl(&Value, 0);
    ValueV = __vperm(ValueV, ValueV, Permute);
    ValueV = __vpkd3d(ValueV, ValueV, VPACK_FLOAT16_4, VPACK_64LO, 2);
    ValueV = __vsplth(ValueV, 0);

    __stvehx(ValueV, &Result, 0);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR ValueV;
    HALF Result;

    XMASSERT(((UINT_PTR)&Value & 3) == 0);

    ValueV = __lvlx(&Value, 0);
    ValueV = __vpkd3d(ValueV, ValueV, VPACK_FLOAT16_4, VPACK_64LO, 2);
    ValueV = __vsplth(ValueV, 0);

    __stvehx(ValueV, &Result, 0);

    return Result;
#endif
}

//------------------------------------------------------------------------------

XMINLINE HALF* XMConvertFloatToHalfStream
(
    HALF*        pOutputStream, 
    size_t       OutputStride, 
    CONST FLOAT* pInputStream, 
    size_t       InputStride, 
    size_t       FloatCount
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)

    size_t  i;
    BYTE* pFloat = (BYTE*)pInputStream;
    BYTE* pHalf = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < FloatCount; i++)
    {
        *(HALF*)pHalf = XMConvertFloatToHalf(*(FLOAT*)pFloat);
        pFloat += InputStride; 
        pHalf += OutputStride;
    }
    return pOutputStream;
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)

    UINT  i;
    BYTE* pFloat = (BYTE*)pInputStream;
    BYTE* pHalf = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < FloatCount; i++)
    {
        *(HALF*)pHalf = XMConvertFloatToHalf(*(FLOAT*)pFloat);
        pFloat += InputStride; 
        pHalf += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR V0[8], V1[8], V2[8], V3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 1) == 0);

    if (InputStride == sizeof(FLOAT))
    {
        if (OutputStride == sizeof(HALF))
        {
            UINT GroupIndex;
            CONST UINT VGroupSize = sizeof(XMVECTOR);
            UINT_PTR OutputStreamEnd = OutputVector + FloatCount * sizeof(HALF);
            UINT_PTR OutputStreamCGroupBase = XMMin(OutputVector + (XM_CACHE_LINE_SIZE - 1), OutputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
            UINT_PTR OutputStreamCGroupEnd = OutputStreamCGroupBase + ((OutputStreamEnd - OutputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
            UINT_PTR OutputStreamVGroupBase = XMMin(OutputVector + (VGroupSize - 1), OutputStreamEnd) & ~(VGroupSize - 1);
            UINT_PTR OutputStreamVGroupEnd = OutputStreamEnd & ~(VGroupSize - 1);

            __dcbt(0,                      (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)OutputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)OutputStreamCGroupBase);

            while (OutputVector < OutputStreamVGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vsplth(V0[0], 0);

                __stvehx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(FLOAT); 
                OutputVector += sizeof(HALF);
            }

            while (OutputVector < OutputStreamCGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V1[0] = __lvrx((VOID*)InputVector, 16);
                InputVector += 4 * sizeof(FLOAT); 
                V2[0] = __lvlx((VOID*)InputVector, 0);
                V3[0] = __lvrx((VOID*)InputVector, 16);
                InputVector += 4 * sizeof(FLOAT); 

                V0[0] = __vor(V0[0], V1[0]);
                V1[0] = __vor(V2[0], V3[0]);

                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vpkd3d(V0[0], V1[0], VPACK_FLOAT16_4, VPACK_64HI, 0);

                __stvx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }

            while (OutputVector < OutputStreamCGroupEnd)
            {
                __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)OutputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)OutputVector);

                for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
                {
                    V0[0] = __lvlx((VOID*)InputVector, 0);
                    V1[0] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[0] = __lvlx((VOID*)InputVector, 0);
                    V3[0] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[1] = __lvlx((VOID*)InputVector, 0);
                    V1[1] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[1] = __lvlx((VOID*)InputVector, 0);
                    V3[1] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[2] = __lvlx((VOID*)InputVector, 0);
                    V1[2] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[2] = __lvlx((VOID*)InputVector, 0);
                    V3[2] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[3] = __lvlx((VOID*)InputVector, 0);
                    V1[3] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[3] = __lvlx((VOID*)InputVector, 0);
                    V3[3] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[4] = __lvlx((VOID*)InputVector, 0);
                    V1[4] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[4] = __lvlx((VOID*)InputVector, 0);
                    V3[4] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[5] = __lvlx((VOID*)InputVector, 0);
                    V1[5] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[5] = __lvlx((VOID*)InputVector, 0);
                    V3[5] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[6] = __lvlx((VOID*)InputVector, 0);
                    V1[6] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[6] = __lvlx((VOID*)InputVector, 0);
                    V3[6] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V0[7] = __lvlx((VOID*)InputVector, 0);
                    V1[7] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 
                    V2[7] = __lvlx((VOID*)InputVector, 0);
                    V3[7] = __lvrx((VOID*)InputVector, 16);
                    InputVector += 4 * sizeof(FLOAT); 

                    V0[0] = __vor(V0[0], V1[0]);
                    V0[1] = __vor(V0[1], V1[1]);
                    V0[2] = __vor(V0[2], V1[2]);
                    V0[3] = __vor(V0[3], V1[3]);
                    V0[4] = __vor(V0[4], V1[4]);
                    V0[5] = __vor(V0[5], V1[5]);
                    V0[6] = __vor(V0[6], V1[6]);
                    V0[7] = __vor(V0[7], V1[7]);

                    V1[0] = __vor(V2[0], V3[0]);
                    V1[1] = __vor(V2[1], V3[1]);
                    V1[2] = __vor(V2[2], V3[2]);
                    V1[3] = __vor(V2[3], V3[3]);
                    V1[4] = __vor(V2[4], V3[4]);
                    V1[5] = __vor(V2[5], V3[5]);
                    V1[6] = __vor(V2[6], V3[6]);
                    V1[7] = __vor(V2[7], V3[7]);

                    V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[0] = __vpkd3d(V0[0], V1[0], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[1] = __vpkd3d(V0[1], V0[1], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[1] = __vpkd3d(V0[1], V1[1], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[2] = __vpkd3d(V0[2], V0[2], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[2] = __vpkd3d(V0[2], V1[2], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[3] = __vpkd3d(V0[3], V0[3], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[3] = __vpkd3d(V0[3], V1[3], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[4] = __vpkd3d(V0[4], V0[4], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[4] = __vpkd3d(V0[4], V1[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[5] = __vpkd3d(V0[5], V0[5], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[5] = __vpkd3d(V0[5], V1[5], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[6] = __vpkd3d(V0[6], V0[6], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[6] = __vpkd3d(V0[6], V1[6], VPACK_FLOAT16_4, VPACK_64HI, 0);
                    V0[7] = __vpkd3d(V0[7], V0[7], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V0[7] = __vpkd3d(V0[7], V1[7], VPACK_FLOAT16_4, VPACK_64HI, 0);

                    __stvx(V0[0], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[1], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[2], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[3], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[4], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[5], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[6], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                    __stvx(V0[7], (VOID*)OutputVector, 0);
                    OutputVector += 4 * sizeof(FLOAT);
                }
            }

            while (OutputVector < OutputStreamVGroupEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V1[0] = __lvrx((VOID*)InputVector, 16);
                InputVector += 4 * sizeof(FLOAT); 
                V2[0] = __lvlx((VOID*)InputVector, 0);
                V3[0] = __lvrx((VOID*)InputVector, 16);
                InputVector += 4 * sizeof(FLOAT); 

                V0[0] = __vor(V0[0], V1[0]);
                V1[0] = __vor(V2[0], V3[0]);

                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vpkd3d(V0[0], V1[0], VPACK_FLOAT16_4, VPACK_64HI, 0);

                __stvx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }

            while (OutputVector < OutputStreamEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vsplth(V0[0], 0);

                __stvehx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(FLOAT); 
                OutputVector += sizeof(HALF);
            }
        }
        else
        {
            UINT GroupIndex;
            CONST UINT VGroupSize = sizeof(XMVECTOR);
            UINT_PTR InputStreamEnd = InputVector + FloatCount * sizeof(FLOAT);
            UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
            UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
            UINT_PTR InputStreamVGroupBase = XMMin(InputVector + (VGroupSize - 1), InputStreamEnd) & ~(VGroupSize - 1);
            UINT_PTR InputStreamVGroupEnd = InputStreamEnd & ~(VGroupSize - 1);

            __dcbt(0,                      (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

            while (InputVector < InputStreamVGroupBase)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vsplth(V0[0], 0);

                __stvehx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(FLOAT); 
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamCGroupBase)
            {
                V3[0] = __lvx((VOID*)InputVector, 0);
                InputVector += 4 * sizeof(FLOAT); 

                V3[0] = __vpkd3d(V3[0], V3[0], VPACK_FLOAT16_4, VPACK_64LO, 2);

                V0[0] = __vsplth(V3[0], 0);
                V1[0] = __vsplth(V3[0], 1);
                V2[0] = __vsplth(V3[0], 2);
                V3[0] = __vsplth(V3[0], 3);

                __stvehx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V2[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamCGroupEnd)
            {
                __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
                __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

                for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
                {
                    V3[0] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[1] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[2] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[3] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[4] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[5] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[6] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 
                    V3[7] = __lvx((VOID*)InputVector, 0);
                    InputVector += 4 * sizeof(FLOAT); 

                    V3[0] = __vpkd3d(V3[0], V3[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[1] = __vpkd3d(V3[1], V3[1], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[2] = __vpkd3d(V3[2], V3[2], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[3] = __vpkd3d(V3[3], V3[3], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[4] = __vpkd3d(V3[4], V3[4], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[5] = __vpkd3d(V3[5], V3[5], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[6] = __vpkd3d(V3[6], V3[6], VPACK_FLOAT16_4, VPACK_64LO, 2);
                    V3[7] = __vpkd3d(V3[7], V3[7], VPACK_FLOAT16_4, VPACK_64LO, 2);

                    V0[0] = __vsplth(V3[0], 0);
                    V1[0] = __vsplth(V3[0], 1);
                    V2[0] = __vsplth(V3[0], 2);
                    V3[0] = __vsplth(V3[0], 3);
                                        
                    V0[1] = __vsplth(V3[1], 0);
                    V1[1] = __vsplth(V3[1], 1);
                    V2[1] = __vsplth(V3[1], 2);
                    V3[1] = __vsplth(V3[1], 3);
                                        
                    V0[2] = __vsplth(V3[2], 0);
                    V1[2] = __vsplth(V3[2], 1);
                    V2[2] = __vsplth(V3[2], 2);
                    V3[2] = __vsplth(V3[2], 3);
                                        
                    V0[3] = __vsplth(V3[3], 0);
                    V1[3] = __vsplth(V3[3], 1);
                    V2[3] = __vsplth(V3[3], 2);
                    V3[3] = __vsplth(V3[3], 3);
                                        
                    V0[4] = __vsplth(V3[4], 0);
                    V1[4] = __vsplth(V3[4], 1);
                    V2[4] = __vsplth(V3[4], 2);
                    V3[4] = __vsplth(V3[4], 3);
                                        
                    V0[5] = __vsplth(V3[5], 0);
                    V1[5] = __vsplth(V3[5], 1);
                    V2[5] = __vsplth(V3[5], 2);
                    V3[5] = __vsplth(V3[5], 3);
                                        
                    V0[6] = __vsplth(V3[6], 0);
                    V1[6] = __vsplth(V3[6], 1);
                    V2[6] = __vsplth(V3[6], 2);
                    V3[6] = __vsplth(V3[6], 3);
                                        
                    V0[7] = __vsplth(V3[7], 0);
                    V1[7] = __vsplth(V3[7], 1);
                    V2[7] = __vsplth(V3[7], 2);
                    V3[7] = __vsplth(V3[7], 3);

                    __stvehx(V0[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[0], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[1], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[2], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[3], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[4], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[5], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[6], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;

                    __stvehx(V0[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V1[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V2[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                    __stvehx(V3[7], (VOID*)OutputVector, 0);
                    OutputVector += OutputStride;
                }
            }

            while (InputVector < InputStreamVGroupEnd)
            {
                V3[0] = __lvx((VOID*)InputVector, 0);
                InputVector += 4 * sizeof(FLOAT); 

                V3[0] = __vpkd3d(V3[0], V3[0], VPACK_FLOAT16_4, VPACK_64LO, 2);

                V0[0] = __vsplth(V3[0], 0);
                V1[0] = __vsplth(V3[0], 1);
                V2[0] = __vsplth(V3[0], 2);
                V3[0] = __vsplth(V3[0], 3);

                __stvehx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V2[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
                __stvehx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += OutputStride;
            }

            while (InputVector < InputStreamEnd)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vsplth(V0[0], 0);

                __stvehx(V0[0], (VOID*)OutputVector, 0);

                InputVector += sizeof(FLOAT); 
                OutputVector += OutputStride;
            }
        }
    }
    else if (OutputStride == sizeof(HALF))
    {
        UINT GroupIndex;
        CONST UINT VGroupSize = sizeof(XMVECTOR);
        XMVECTOR V4[8], V5[8], V6[8], V7[8];
        UINT_PTR OutputStreamEnd = OutputVector + FloatCount * sizeof(HALF);
        UINT_PTR OutputStreamCGroupBase = XMMin(OutputVector + (XM_CACHE_LINE_SIZE - 1), OutputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR OutputStreamCGroupEnd = OutputStreamCGroupBase + ((OutputStreamEnd - OutputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));
        UINT_PTR OutputStreamVGroupBase = XMMin(OutputVector + (VGroupSize - 1), OutputStreamEnd) & ~(VGroupSize - 1);
        UINT_PTR OutputStreamVGroupEnd = OutputStreamEnd & ~(VGroupSize - 1);

        __dcbt(0,                      (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)OutputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)OutputStreamCGroupBase);

        while (OutputVector < OutputStreamVGroupBase)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[0] = __vsplth(V0[0], 0);

            __stvehx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += sizeof(HALF);
        }

        while (OutputVector < OutputStreamCGroupBase)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            V0[0] = __vrlimi(V0[0], V0[1], 0x4, 3);
            V0[2] = __vrlimi(V0[2], V0[3], 0x4, 3);
            V0[4] = __vrlimi(V0[4], V0[5], 0x4, 3);
            V0[6] = __vrlimi(V0[6], V0[7], 0x4, 3);
                                      
            V0[0] = __vrlimi(V0[0], V0[2], 0x3, 2);
            V0[4] = __vrlimi(V0[4], V0[6], 0x3, 2);
                                      
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[0] = __vpkd3d(V0[0], V0[4], VPACK_FLOAT16_4, VPACK_64HI, 0);

            __stvx(V0[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
        }

        while (OutputVector < OutputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)OutputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)OutputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                V0[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V0[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V1[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V1[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V2[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V2[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V3[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V3[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V4[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V4[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V5[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V5[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V6[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V6[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V7[0] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[1] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[2] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[3] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[4] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[5] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[6] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;
                V7[7] = __lvlx((VOID*)InputVector, 0);
                InputVector += InputStride;

                V0[0] = __vrlimi(V0[0], V0[1], 0x4, 3);
                V0[2] = __vrlimi(V0[2], V0[3], 0x4, 3);
                V0[4] = __vrlimi(V0[4], V0[5], 0x4, 3);
                V0[6] = __vrlimi(V0[6], V0[7], 0x4, 3);
                                         
                V1[0] = __vrlimi(V1[0], V1[1], 0x4, 3);
                V1[2] = __vrlimi(V1[2], V1[3], 0x4, 3);
                V1[4] = __vrlimi(V1[4], V1[5], 0x4, 3);
                V1[6] = __vrlimi(V1[6], V1[7], 0x4, 3);
                                         
                V2[0] = __vrlimi(V2[0], V2[1], 0x4, 3);
                V2[2] = __vrlimi(V2[2], V2[3], 0x4, 3);
                V2[4] = __vrlimi(V2[4], V2[5], 0x4, 3);
                V2[6] = __vrlimi(V2[6], V2[7], 0x4, 3);
                                         
                V3[0] = __vrlimi(V3[0], V3[1], 0x4, 3);
                V3[2] = __vrlimi(V3[2], V3[3], 0x4, 3);
                V3[4] = __vrlimi(V3[4], V3[5], 0x4, 3);
                V3[6] = __vrlimi(V3[6], V3[7], 0x4, 3);
                                         
                V4[0] = __vrlimi(V4[0], V4[1], 0x4, 3);
                V4[2] = __vrlimi(V4[2], V4[3], 0x4, 3);
                V4[4] = __vrlimi(V4[4], V4[5], 0x4, 3);
                V4[6] = __vrlimi(V4[6], V4[7], 0x4, 3);
                                         
                V5[0] = __vrlimi(V5[0], V5[1], 0x4, 3);
                V5[2] = __vrlimi(V5[2], V5[3], 0x4, 3);
                V5[4] = __vrlimi(V5[4], V5[5], 0x4, 3);
                V5[6] = __vrlimi(V5[6], V5[7], 0x4, 3);
                                         
                V6[0] = __vrlimi(V6[0], V6[1], 0x4, 3);
                V6[2] = __vrlimi(V6[2], V6[3], 0x4, 3);
                V6[4] = __vrlimi(V6[4], V6[5], 0x4, 3);
                V6[6] = __vrlimi(V6[6], V6[7], 0x4, 3);
                                         
                V7[0] = __vrlimi(V7[0], V7[1], 0x4, 3);
                V7[2] = __vrlimi(V7[2], V7[3], 0x4, 3);
                V7[4] = __vrlimi(V7[4], V7[5], 0x4, 3);
                V7[6] = __vrlimi(V7[6], V7[7], 0x4, 3);

                V0[0] = __vrlimi(V0[0], V0[2], 0x3, 2);
                V1[0] = __vrlimi(V1[0], V1[2], 0x3, 2);
                V2[0] = __vrlimi(V2[0], V2[2], 0x3, 2);
                V3[0] = __vrlimi(V3[0], V3[2], 0x3, 2);
                V4[0] = __vrlimi(V4[0], V4[2], 0x3, 2);
                V5[0] = __vrlimi(V5[0], V5[2], 0x3, 2);
                V6[0] = __vrlimi(V6[0], V6[2], 0x3, 2);
                V7[0] = __vrlimi(V7[0], V7[2], 0x3, 2);

                V0[4] = __vrlimi(V0[4], V0[6], 0x3, 2);
                V1[4] = __vrlimi(V1[4], V1[6], 0x3, 2);
                V2[4] = __vrlimi(V2[4], V2[6], 0x3, 2);
                V3[4] = __vrlimi(V3[4], V3[6], 0x3, 2);
                V4[4] = __vrlimi(V4[4], V4[6], 0x3, 2);
                V5[4] = __vrlimi(V5[4], V5[6], 0x3, 2);
                V6[4] = __vrlimi(V6[4], V6[6], 0x3, 2);
                V7[4] = __vrlimi(V7[4], V7[6], 0x3, 2);

                V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V0[0] = __vpkd3d(V0[0], V0[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V1[0] = __vpkd3d(V1[0], V1[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V1[0] = __vpkd3d(V1[0], V1[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V2[0] = __vpkd3d(V2[0], V2[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V2[0] = __vpkd3d(V2[0], V2[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V3[0] = __vpkd3d(V3[0], V3[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V3[0] = __vpkd3d(V3[0], V3[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V4[0] = __vpkd3d(V4[0], V4[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V4[0] = __vpkd3d(V4[0], V4[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V5[0] = __vpkd3d(V5[0], V5[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V5[0] = __vpkd3d(V5[0], V5[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V6[0] = __vpkd3d(V6[0], V6[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V6[0] = __vpkd3d(V6[0], V6[4], VPACK_FLOAT16_4, VPACK_64HI, 0);
                V7[0] = __vpkd3d(V7[0], V7[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
                V7[0] = __vpkd3d(V7[0], V7[4], VPACK_FLOAT16_4, VPACK_64HI, 0);

                __stvx(V0[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V1[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V2[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V3[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V4[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V5[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V6[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
                __stvx(V7[0], (VOID*)OutputVector, 0);
                OutputVector += 4 * sizeof(FLOAT);
            }
        }

        while (OutputVector < OutputStreamVGroupEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            V0[0] = __vrlimi(V0[0], V0[1], 0x4, 3);
            V0[2] = __vrlimi(V0[2], V0[3], 0x4, 3);
            V0[4] = __vrlimi(V0[4], V0[5], 0x4, 3);
            V0[6] = __vrlimi(V0[6], V0[7], 0x4, 3);
                                      
            V0[0] = __vrlimi(V0[0], V0[2], 0x3, 2);
            V0[4] = __vrlimi(V0[4], V0[6], 0x3, 2);
                                      
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[0] = __vpkd3d(V0[0], V0[4], VPACK_FLOAT16_4, VPACK_64HI, 0);

            __stvx(V0[0], (VOID*)OutputVector, 0);
            OutputVector += 4 * sizeof(FLOAT);
        }

        while (OutputVector < OutputStreamEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[0] = __vsplth(V0[0], 0);

            __stvehx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += sizeof(HALF);
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + FloatCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (FloatCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            V0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            V0[0] = __vrlimi(V0[0], V0[1], 0x4, 3);
            V0[2] = __vrlimi(V0[2], V0[3], 0x4, 3);
            V0[4] = __vrlimi(V0[4], V0[5], 0x4, 3);
            V0[6] = __vrlimi(V0[6], V0[7], 0x4, 3);
                                      
            V0[0] = __vrlimi(V0[0], V0[2], 0x3, 2);
            V0[4] = __vrlimi(V0[4], V0[6], 0x3, 2);
                                      
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[4] = __vpkd3d(V0[4], V0[4], VPACK_FLOAT16_4, VPACK_64LO, 2);

            V0[3] = __vsplth(V0[0], 3);
            V0[2] = __vsplth(V0[0], 2);
            V0[1] = __vsplth(V0[0], 1);
            V0[0] = __vsplth(V0[0], 0);

            V0[7] = __vsplth(V0[4], 3);
            V0[6] = __vsplth(V0[4], 2);
            V0[5] = __vsplth(V0[4], 1);
            V0[4] = __vsplth(V0[4], 0);

            __stvehx(V0[0], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[1], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[2], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[3], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[4], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[5], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[6], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
            __stvehx(V0[7], (VOID*)OutputVector, 0);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            V0[0] = __lvlx((VOID*)InputVector, 0);
            V0[0] = __vpkd3d(V0[0], V0[0], VPACK_FLOAT16_4, VPACK_64LO, 2);
            V0[0] = __vsplth(V0[0], 0);

            __stvehx(V0[0], (VOID*)OutputVector, 0);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)
// For VMX128, these routines are all defines in the main header

#pragma warning(push)
#pragma warning(disable:4701) // Prevent warnings about 'Result' potentially being used without having been initialized

XMINLINE XMVECTOR XMConvertVectorIntToFloat
(
    FXMVECTOR VInt,
    UINT     DivExponent
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ElementIndex;
    FLOAT fScale;
    XMVECTOR Result;
    XMASSERT(DivExponent<32);
    fScale = 1.0f / (FLOAT)(1U << DivExponent);
    ElementIndex = 0;
    do {
        INT iTemp = (INT)VInt.vector4_u32[ElementIndex];
        Result.vector4_f32[ElementIndex] = ((FLOAT)iTemp) * fScale;
    } while (++ElementIndex<4);
    return Result;
#else // _XM_SSE_INTRINSICS_
#endif
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMConvertVectorFloatToInt
(
    FXMVECTOR VFloat,
    UINT     MulExponent
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ElementIndex;
    XMVECTOR Result;
    FLOAT fScale;
    XMASSERT(MulExponent<32);
    // Get the scalar factor.
    fScale = (FLOAT)(1U << MulExponent);
    ElementIndex = 0;
    do {
        INT iResult;
        FLOAT fTemp = VFloat.vector4_f32[ElementIndex]*fScale;
        if (fTemp <= -(65536.0f*32768.0f)) {
            iResult = (-0x7FFFFFFF)-1;
        } else if (fTemp > (65536.0f*32768.0f)-128.0f) {
            iResult = 0x7FFFFFFF;
        } else {
            iResult = (INT)fTemp;
        }
        Result.vector4_u32[ElementIndex] = (UINT)iResult;
    } while (++ElementIndex<4);
    return Result;
#else // _XM_SSE_INTRINSICS_
#endif
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMConvertVectorUIntToFloat
(
    FXMVECTOR VUInt,
    UINT      DivExponent
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ElementIndex;
    FLOAT fScale;
    XMVECTOR Result;
    XMASSERT(DivExponent<32);
    fScale = 1.0f / (FLOAT)(1U << DivExponent);
    ElementIndex = 0;
    do {
        Result.vector4_f32[ElementIndex] = (FLOAT)VUInt.vector4_u32[ElementIndex] * fScale;
    } while (++ElementIndex<4);
    return Result;
#else // _XM_SSE_INTRINSICS_
#endif
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMConvertVectorFloatToUInt
(
    FXMVECTOR VFloat,
    UINT      MulExponent
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ElementIndex;
    XMVECTOR Result;
    FLOAT fScale;
    XMASSERT(MulExponent<32);
    // Get the scalar factor.
    fScale = (FLOAT)(1U << MulExponent);
    ElementIndex = 0;
    do {
        UINT uResult;
        FLOAT fTemp = VFloat.vector4_f32[ElementIndex]*fScale;
        if (fTemp <= 0.0f) {
            uResult = 0;
        } else if (fTemp >= (65536.0f*65536.0f)) {
            uResult = 0xFFFFFFFFU;
        } else {
            uResult = (UINT)fTemp;
        }
        Result.vector4_u32[ElementIndex] = uResult;
    } while (++ElementIndex<4);
    return Result;
#else // _XM_SSE_INTRINSICS_
#endif
}

#pragma warning(pop)

#endif // _XM_NO_INTRINSICS_ || _XM_SSE_INTRINSICS_

/****************************************************************************
 *
 * Vector and matrix load operations
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt(CONST UINT* pSource)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V.vector4_u32[0] = *pSource;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR Permute;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V = __vperm(V, V, Permute);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    return __lvlx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat(CONST FLOAT* pSource)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V.vector4_f32[0] = *pSource;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR Permute;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V = __vperm(V, V, Permute);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    return __lvlx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt2
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];

    return V;
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 7);                            // Load the 16 bytes starting at address ((BYTE*)pSource + 7) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadSInt2
(
    CONST XMINT2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 7);                            // Load the 16 bytes starting at address ((BYTE*)pSource + 7) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfsx(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    V = __vor(VL, VR);
    V = __vcfsx(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUInt2
(
    CONST XMUINT2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 7);                            // Load the 16 bytes starting at address ((BYTE*)pSource + 7) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfux(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    V = __vor(VL, VR);
    V = __vcfux(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt2A
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat2
(
    CONST XMFLOAT2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    ((UINT *)(&V.vector4_f32[0]))[0] = ((const UINT *)(&pSource->x))[0];
    ((UINT *)(&V.vector4_f32[1]))[0] = ((const UINT *)(&pSource->y))[0];
    return V;
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 7);                            // Load the 16 bytes starting at address ((BYTE*)pSource + 7) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat2A
(
    CONST XMFLOAT2A* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_f32[0] = pSource->x;
    V.vector4_f32[1] = pSource->y;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}
    
//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadHalf2
(
    CONST XMHALF2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        XMConvertHalfToFloat(pSource->x),
        XMConvertHalfToFloat(pSource->y),
        0.0f,
        0.0f
    };
    return vResult;
    }
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V, V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);

    Permute = __lvsl(pSource, 0);
    V0 = __lvx(pSource, 0);
    Permute = __vsldoi(Permute, Permute, 1 << 2);
    V1 = __lvx(pSource, 3);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_FLOAT16_2);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);

    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 4);
    VL = __vsldoi(VL, VL, 1 << 2);
    V = __vor(VL, VR);
    V = __vupkd3d(V, VPACK_FLOAT16_2);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadShortN2
(
    CONST XMSHORTN2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (pSource->x == -32768) ? -1.f : ((FLOAT)pSource->x * (1.0f/32767.0f)),
        (pSource->y == -32768) ? -1.f : ((FLOAT)pSource->y * (1.0f/32767.0f)),
        0.0f,
        0.0f
    };
    return vResult;
    }

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V0, V1;
    XMVECTOR                V;
    XMVECTOR                Permute;
    XMVECTOR                Result;
    XMVECTOR                UnpackOffset, NegativeOne, Control;
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pSource);

    UnpackOffset = __vspltisw(3);
    UnpackOffset = __vcfsx(UnpackOffset, 0);

    Permute = __lvsl(pSource, 0);
    V0 = __lvx(pSource, 0);
    Permute = __vsldoi(Permute, Permute, 1 << 2);
    V1 = __lvx(pSource, 3);
    Result = __vmulfp(UnpackScale, UnpackOffset);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_NORMSHORT2);
    V = __vnmsubfp(V, UnpackScale, Result);

    Control = __vcmpeqfp(V, V);
    Control = __vnor(Control, Control);
    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    Result = __vsel( V, NegativeOne, Control );

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackOffset, NegativeOne, Control;
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pSource);

    UnpackOffset = __vspltisw(3);
    UnpackOffset = __vcfsx(UnpackOffset, 0);
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 4);
    Result = __vmulfp(UnpackScale, UnpackOffset);
    VL = __vsldoi(VL, VL, 1 << 2);
    V = __vor(VL, VR);
    V = __vupkd3d(V, VPACK_NORMSHORT2);
    V = __vnmsubfp(V, UnpackScale, Result);
   
    Control = __vcmpeqfp(V, V);
    Control = __vnor(Control, Control);
    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    Result = __vsel( V, NegativeOne, Control );

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadShort2
(
    CONST XMSHORT2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 short values into the most significant 32 bits
    V = __vupkhsh(V);                                                 // Unpack the 2 short values
    V = __vcfsx(V, 0);                                                // Convert to floating point

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;

    XMASSERT(pSource);

    VR = __lvrx(pSource, 4);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 3 << 2);
    V = __vor(VL, VR);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUShortN2
(
    CONST XMUSHORTN2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x / 65535.0f;
    V.vector4_f32[1] = (FLOAT)pSource->y / 65535.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Zero;
    XMVECTOR                Permute;
    static CONST XMVECTOR   UnpackScale = {1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1)};

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    Zero = __vspltisw(0);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 ushort values into the most significant 32 bits
    V = __vmrghh(Zero, V);                                            // Unpack the 2 ushort values
    V = __vcfsx(V, 0);                                                // Convert to floating point
    V = __vmulfp(V, UnpackScale);      // Scale to the correct range

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Zero;
    static CONST XMVECTOR   UnpackScale = {1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1)};

    XMASSERT(pSource);

    VR = __lvrx(pSource, 4);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 12);
    Zero = __vspltisw(0);
    V = __vor(VL, VR);
    V = __vmrghh(Zero, V);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, UnpackScale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUShort2
(
    CONST XMUSHORT2* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Zero;
    XMVECTOR                Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    Zero = __vspltisw(0);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 ushort values into the most significant 32 bits
    V = __vmrghh(Zero, V);                                            // Unpack the 2 ushort values
    V = __vcfsx(V, 0);                                                // Convert to floating point

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Zero;

    XMASSERT(pSource);

    VR = __lvrx(pSource, 4);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 12);
    Zero = __vspltisw(0);
    V = __vor(VL, VR);
    V = __vmrghh(Zero, V);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadByteN2
(
    CONST XMBYTEN2* pSource
)
{
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (pSource->x == -128) ? -1.f : ((FLOAT)pSource->x * (1.0f/127.0f)),
        (pSource->y == -128) ? -1.f : ((FLOAT)pSource->y * (1.0f/127.0f)),
        0.0f,
        0.0f
    };
    return vResult;
    }
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadByte2
(
    CONST XMBYTE2* pSource
)
{
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (FLOAT)pSource->x,
        (FLOAT)pSource->y,
        0.0f,
        0.0f
    };
    return vResult;
    }
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUByteN2
(
    CONST XMUBYTEN2* pSource
)
{
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (FLOAT)pSource->x * (1.0f/255.0f),
        (FLOAT)pSource->y * (1.0f/255.0f),
        0.0f,
        0.0f
    };
    return vResult;
    }
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUByte2
(
    CONST XMUBYTE2* pSource
)
{
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (FLOAT)pSource->x,
        (FLOAT)pSource->y,
        0.0f,
        0.0f
    };
    return vResult;
    }
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt3
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];
    V.vector4_u32[2] = pSource[2];

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 11);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 11) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 12);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 1 << 2);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadSInt3
(
    CONST XMINT3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

#ifdef _XBOX_VER
    V = XMLoadInt3( (const UINT*)pSource );
    return XMConvertVectorIntToFloat( V, 0 );
#else
    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    V.vector4_f32[2] = (float)pSource->z;
    return V;
#endif

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 11);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 11) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfsx(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 12);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 1 << 2);
    V = __vor(VL, VR);
    V = __vcfsx(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUInt3
(
    CONST XMUINT3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    V.vector4_f32[2] = (float)pSource->z;
    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 11);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 11) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfux(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 12);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 1 << 2);
    V = __vor(VL, VR);
    V = __vcfux(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt3A
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];
    V.vector4_u32[2] = pSource[2];

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat3
(
    CONST XMFLOAT3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    ((UINT *)(&V.vector4_f32[0]))[0] = ((const UINT *)(&pSource->x))[0];
    ((UINT *)(&V.vector4_f32[1]))[0] = ((const UINT *)(&pSource->y))[0];
    ((UINT *)(&V.vector4_f32[2]))[0] = ((const UINT *)(&pSource->z))[0];
    return V;
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 11);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 11) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VR = __lvrx(pSource, 12);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 1 << 2);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat3A
(
    CONST XMFLOAT3A* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_f32[0] = pSource->x;
    V.vector4_f32[1] = pSource->y;
    V.vector4_f32[2] = pSource->z;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUHenDN3
(
    CONST XMUHENDN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x7FF;
    V.vector4_f32[0] = (FLOAT)Element / 2047.0f;
    Element = (pSource->v >> 11) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)Element / 2047.0f;
    Element = (pSource->v >> 22) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)Element / 1023.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << 11) - 1), 1.0f / (FLOAT)((1 << 11) - 1), 1.0f / (FLOAT)((1 << 10) - 1), 0.0f};

    XMASSERT(pSource);

    I[0] = pSource->v & 0x7FF;
    I[1] = (pSource->v >> 11) & 0x7FF;
    I[2] = (pSource->v >> 22) & 0x3FF;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Scale;
    XMVECTOR               Mask;
    static CONST __vector4i C0 = {0x3A001002, 0x3A802008, 0x7FF, 0x3FF}; // {1.0f / (FLOAT)((1 << 11) - 1), 1.0f / (FLOAT)((1 << 10) - 1), *(FLOAT*)&0x7FF, *(FLOAT*)&0x3FF};
    static CONST __vector4i Shift = {0, 11, 22, 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    V = __vspltw(V, 0);
    Mask = __vpermwi(*(XMVECTOR*)&C0, 0xAF);
    V = __vsrw(V, *(XMVECTOR*)&Shift);
    V = __vand(V, Mask);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0x5);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUHenD3
(
    CONST XMUHEND3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x7FF;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 11) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 22) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)Element;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;

    XMASSERT(pSource);

    I[0] = pSource->v & 0x7FF;
    I[1] = (pSource->v >> 11) & 0x7FF;
    I[2] = (pSource->v >> 22) & 0x3FF;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Shift;
    XMVECTOR               Mask;
    static CONST __vector4i ShiftMask = {0x7FF0000, 0x7FF000B, 0x3FF0016, 0x0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    Shift = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    Mask = __vsrw(*(XMVECTOR*)&ShiftMask, Shift);
    V = __vsrw(V, *(XMVECTOR*)&ShiftMask);
    V = __vand(V, Mask);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadHenDN3
(
    CONST XMHENDN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtendXY[] = {0x00000000, 0xFFFFF800};
    static CONST UINT SignExtendZ[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 11) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 22) & 0x3FF) != 0x200);

    Element = pSource->v & 0x7FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtendXY[Element >> 10]) / 1023.0f;
    Element = (pSource->v >> 11) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtendXY[Element >> 10]) / 1023.0f;
    Element = (pSource->v >> 22) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtendZ[Element >> 9]) / 511.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtendXY[] = {0x00000000, 0xFFFFF800};
    static CONST UINT      SignExtendZ[] = {0x00000000, 0xFFFFFC00};
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << (11 - 1)) - 1), 1.0f / (FLOAT)((1 << (11 - 1)) - 1), 1.0f / (FLOAT)((1 << (10 - 1)) - 1), 1.0f};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 11) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 22) & 0x3FF) != 0x200);

    Element = pSource->v & 0x7FF;
    I[0] = Element | SignExtendXY[Element >> 10];
    Element = (pSource->v >> 11) & 0x7FF;
    I[1] = Element | SignExtendXY[Element >> 10];
    Element = (pSource->v >> 22) & 0x3FF;
    I[2] = Element | SignExtendZ[Element >> 9];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Scale = {0x3A802008, 0x3A802008, 0x3B004020, 0}; // {1.0f / (FLOAT)((1 << (11 - 1)) - 1), ..., 1.0f / (FLOAT)((1 << (10 - 1)) - 1)};
    static CONST __vector4i Shift = {((32 - 11) << 16) | (32 - 11), ((32 - 11) << 16) | (32 - 22), ((32 - 10) << 16) | (32 - 32), 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, *(XMVECTOR*)&Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadHenD3
(
    CONST XMHEND3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtendXY[] = {0x00000000, 0xFFFFF800};
    static CONST UINT SignExtendZ[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 11) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 22) & 0x3FF) != 0x200);

    Element = pSource->v & 0x7FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtendXY[Element >> 10]);
    Element = (pSource->v >> 11) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtendXY[Element >> 10]);
    Element = (pSource->v >> 22) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtendZ[Element >> 9]);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtendXY[] = {0x00000000, 0xFFFFF800};
    static CONST UINT      SignExtendZ[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 11) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 22) & 0x3FF) != 0x200);

    Element = pSource->v & 0x7FF;
    I[0] = Element | SignExtendXY[Element >> 10];
    Element = (pSource->v >> 11) & 0x7FF;
    I[1] = Element | SignExtendXY[Element >> 10];
    Element = (pSource->v >> 22) & 0x3FF;
    I[2] = Element | SignExtendZ[Element >> 9];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Shift = {((32 - 11) << 16) | (32 - 11), ((32 - 11) << 16) | (32 - 22), ((32 - 10) << 16) | (32 - 32), 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUDHenN3
(
    CONST XMUDHENN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)Element / 1023.0f;
    Element = (pSource->v >> 10) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)Element / 2047.0f;
    Element = (pSource->v >> 21) & 0x7FF;
    V.vector4_f32[2] = (FLOAT)Element / 2047.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    static CONST XMVECTOR  Scale = {1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 11) - 1), 1.0f / (FLOAT)((1 << 11) - 1), 0.0f};

    XMASSERT(pSource);

    I[0] = pSource->v & 0x3FF;
    I[1] = (pSource->v >> 10) & 0x7FF;
    I[2] = (pSource->v >> 21) & 0x7FF;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Scale;
    XMVECTOR               Mask;
    static CONST __vector4i C0 = {0x3A802008, 0x3A001002, 0x3FF, 0x7FF}; // {1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 11) - 1), *(FLOAT*)&0x3FF, *(FLOAT*)&0x7FF};
    static CONST __vector4i Shift = {0, 10, 21, 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    V = __vspltw(V, 0);
    Mask = __vpermwi(*(XMVECTOR*)&C0, 0xBC);
    V = __vsrw(V, *(XMVECTOR*)&Shift);
    V = __vand(V, Mask);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0x14);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUDHen3
(
    CONST XMUDHEN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 10) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 21) & 0x7FF;
    V.vector4_f32[2] = (FLOAT)Element;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;

    XMASSERT(pSource);

    I[0] = pSource->v & 0x3FF;
    I[1] = (pSource->v >> 10) & 0x7FF;
    I[2] = (pSource->v >> 21) & 0x7FF;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Shift;
    XMVECTOR               Mask;
    static CONST __vector4i ShiftMask = {0x3FF0000, 0x7FF000A, 0x7FF0015, 0x0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    Shift = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    Mask = __vsrw(*(XMVECTOR*)&ShiftMask, Shift);
    V = __vsrw(V, *(XMVECTOR*)&ShiftMask);
    V = __vand(V, Mask);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadDHenN3
(
    CONST XMDHENN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtendX[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT SignExtendYZ[] = {0x00000000, 0xFFFFF800};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 21) & 0x7FF) != 0x400);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtendX[Element >> 9]) / 511.0f;
    Element = (pSource->v >> 10) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtendYZ[Element >> 10]) / 1023.0f;
    Element = (pSource->v >> 21) & 0x7FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtendYZ[Element >> 10]) / 1023.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtendX[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT      SignExtendYZ[] = {0x00000000, 0xFFFFF800};
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << (10 - 1)) - 1), 1.0f / (FLOAT)((1 << (11 - 1)) - 1), 1.0f / (FLOAT)((1 << (11 - 1)) - 1), 1.0f};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 21) & 0x7FF) != 0x400);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtendX[Element >> 9];
    Element = (pSource->v >> 10) & 0x7FF;
    I[1] = Element | SignExtendYZ[Element >> 10];
    Element = (pSource->v >> 21) & 0x7FF;
    I[2] = Element | SignExtendYZ[Element >> 10];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Scale = {0x3B004020, 0x3A802008, 0x3A802008, 0}; // {1.0f / (FLOAT)((1 << (10 - 1)) - 1), ..., 1.0f / (FLOAT)((1 << (11 - 1)) - 1)};
    static CONST __vector4i Shift = {((32 - 10) << 16) | (32 - 10), ((32 - 11) << 16) | (32 - 21), ((32 - 11) << 16) | (32 - 32), 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, *(XMVECTOR*)&Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadDHen3
(
    CONST XMDHEN3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtendX[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT SignExtendYZ[] = {0x00000000, 0xFFFFF800};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 21) & 0x7FF) != 0x400);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtendX[Element >> 9]);
    Element = (pSource->v >> 10) & 0x7FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtendYZ[Element >> 10]);
    Element = (pSource->v >> 21) & 0x7FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtendYZ[Element >> 10]);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtendX[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT      SignExtendYZ[] = {0x00000000, 0xFFFFF800};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x7FF) != 0x400);
    XMASSERT(((pSource->v >> 21) & 0x7FF) != 0x400);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtendX[Element >> 9];
    Element = (pSource->v >> 10) & 0x7FF;
    I[1] = Element | SignExtendYZ[Element >> 10];
    Element = (pSource->v >> 21) & 0x7FF;
    I[2] = Element | SignExtendYZ[Element >> 10];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Shift = {((32 - 10) << 16) | (32 - 10), ((32 - 11) << 16) | (32 - 21), ((32 - 11) << 16) | (32 - 32), 0};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadU565
(
    CONST XMU565* pSource
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x1F;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 5) & 0x3F;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 11) & 0x1F;
    V.vector4_f32[2] = (FLOAT)Element;

    return V;
#endif // !_XM_SSE_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat3PK
(
    CONST XMFLOAT3PK* pSource
)
{
    _DECLSPEC_ALIGN_16_ UINT Result[4];
    UINT Mantissa;
    UINT Exponent;

    XMASSERT(pSource);

    // X Channel (6-bit mantissa)
    Mantissa = pSource->xm;

    if ( pSource->xe == 0x1f ) // INF or NAN
    {
        Result[0] = 0x7f800000 | (pSource->xm << 17);
    }
    else
    {
        if ( pSource->xe != 0 ) // The value is normalized
        {
            Exponent = pSource->xe;
        }
        else if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;
    
            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x40) == 0);
    
            Mantissa &= 0x3F;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }
    
        Result[0] = ((Exponent + 112) << 23) | (Mantissa << 17);
    }

    // Y Channel (6-bit mantissa)
    Mantissa = pSource->ym;

    if ( pSource->ye == 0x1f ) // INF or NAN
    {
        Result[1] = 0x7f800000 | (pSource->ym << 17);
    }
    else
    {
        if ( pSource->ye != 0 ) // The value is normalized
        {
            Exponent = pSource->ye;
        }
        else if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;
    
            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x40) == 0);
    
            Mantissa &= 0x3F;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }
    
        Result[1] = ((Exponent + 112) << 23) | (Mantissa << 17);
    }

    // Z Channel (5-bit mantissa)
    Mantissa = pSource->zm;

    if ( pSource->ze == 0x1f ) // INF or NAN
    {
        Result[2] = 0x7f800000 | (pSource->zm << 17);
    }
    else
    {
        if ( pSource->ze != 0 ) // The value is normalized
        {
            Exponent = pSource->ze;
        }
        else if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;
    
            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x20) == 0);
    
            Mantissa &= 0x1F;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }

        Result[2] = ((Exponent + 112) << 23) | (Mantissa << 18);
    }

    return XMLoadFloat3A( (const XMFLOAT3A*)&Result );
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat3SE
(
    CONST XMFLOAT3SE* pSource
)
{
    _DECLSPEC_ALIGN_16_ UINT Result[4];
    UINT Mantissa;
    UINT Exponent, ExpBits;

    XMASSERT(pSource);

    if ( pSource->e == 0x1f ) // INF or NAN
    {
        Result[0] = 0x7f800000 | (pSource->xm << 14);
        Result[1] = 0x7f800000 | (pSource->ym << 14);
        Result[2] = 0x7f800000 | (pSource->zm << 14);
    }
    else if ( pSource->e != 0 ) // The values are all normalized
    {
        Exponent = pSource->e;

        ExpBits = (Exponent + 112) << 23;

        Mantissa = pSource->xm;
        Result[0] = ExpBits | (Mantissa << 14);

        Mantissa = pSource->ym;
        Result[1] = ExpBits | (Mantissa << 14);

        Mantissa = pSource->zm;
        Result[2] = ExpBits | (Mantissa << 14);
    }
    else
    {
        // X Channel
        Mantissa = pSource->xm;

        if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;

            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x200) == 0);

            Mantissa &= 0x1FF;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }

        Result[0] = ((Exponent + 112) << 23) | (Mantissa << 14);

        // Y Channel
        Mantissa = pSource->ym;

        if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;

            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x200) == 0);

            Mantissa &= 0x1FF;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }

        Result[1] = ((Exponent + 112) << 23) | (Mantissa << 14);

        // Z Channel
        Mantissa = pSource->zm;

        if (Mantissa != 0) // The value is denormalized
        {
            // Normalize the value in the resulting float
            Exponent = 1;

            do
            {
                Exponent--;
                Mantissa <<= 1;
            } while ((Mantissa & 0x200) == 0);

            Mantissa &= 0x1FF;
        }
        else // The value is zero
        {
            Exponent = (UINT)-112;
        }

        Result[2] = ((Exponent + 112) << 23) | (Mantissa << 14);
    }

    return XMLoadFloat3A( (const XMFLOAT3A*)&Result );
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt4
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];
    V.vector4_u32[2] = pSource[2];
    V.vector4_u32[3] = pSource[3];

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 15);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 15) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 16);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadSInt4
(
    CONST XMINT4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

#ifdef _XBOX_VER
    V = XMLoadInt4( (const UINT*)pSource );
    return XMConvertVectorIntToFloat( V, 0 );
#else
    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    V.vector4_f32[2] = (float)pSource->z;
    V.vector4_f32[3] = (float)pSource->w;
    return V;
#endif

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 15);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 15) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfsx(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 16);
    V = __vor(VL, VR);
    V = __vcfsx(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUInt4
(
    CONST XMUINT4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    V.vector4_f32[0] = (float)pSource->x;
    V.vector4_f32[1] = (float)pSource->y;
    V.vector4_f32[2] = (float)pSource->z;
    V.vector4_f32[3] = (float)pSource->w;
    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 15);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 15) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector
    V = __vcfux(V, 0);
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 16);
    V = __vor(VL, VR);
    V = __vcfux(V, 0);
    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadInt4A
(
    CONST UINT* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_u32[0] = pSource[0];
    V.vector4_u32[1] = pSource[1];
    V.vector4_u32[2] = pSource[2];
    V.vector4_u32[3] = pSource[3];

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat4
(
    CONST XMFLOAT4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    XMASSERT(pSource);

    ((UINT *)(&V.vector4_f32[0]))[0] = ((const UINT *)(&pSource->x))[0];
    ((UINT *)(&V.vector4_f32[1]))[0] = ((const UINT *)(&pSource->y))[0];
    ((UINT *)(&V.vector4_f32[2]))[0] = ((const UINT *)(&pSource->z))[0];
    ((UINT *)(&V.vector4_f32[3]))[0] = ((const UINT *)(&pSource->w))[0];
    return V;
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);
    
    V0 = __lvx(pSource, 0);                            // Load the 16 bytes starting at address pSource & ~0xF
    Permute = __lvsl(pSource, 0);                      // Compute the permute control vector for shift left
    V1 = __lvx(pSource, 15);                           // Load the 16 bytes starting at address ((BYTE*)pSource + 15) & ~0xF
    V = __vperm(V0, V1, Permute);                      // Align the vector

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);
    
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 16);
    V = __vor(VL, VR);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadFloat4A
(
    CONST XMFLOAT4A* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    V.vector4_f32[0] = pSource->x;
    V.vector4_f32[1] = pSource->y;
    V.vector4_f32[2] = pSource->z;
    V.vector4_f32[3] = pSource->w;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    return __lvx(pSource, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadHalf4
(
    CONST XMHALF4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        XMConvertHalfToFloat(pSource->x),
        XMConvertHalfToFloat(pSource->y),
        XMConvertHalfToFloat(pSource->z),
        XMConvertHalfToFloat(pSource->w)
    };
    return vResult;
    }
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V, V0, V1;
    XMVECTOR Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    Permute = __vsldoi(Permute, Permute, 2 << 2);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_FLOAT16_4);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR VL;
    XMVECTOR VR;
    XMVECTOR V;

    XMASSERT(pSource);

    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 8);
    VL = __vsldoi(VL, VL, 2 << 2);
    V = __vor(VL, VR);
    V = __vupkd3d(V, VPACK_FLOAT16_4);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadShortN4
(
    CONST XMSHORTN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMASSERT(pSource);
    {
    XMVECTOR vResult = {
        (pSource->x == -32768) ? -1.f : ((FLOAT)pSource->x * (1.0f/32767.0f)),
        (pSource->y == -32768) ? -1.f : ((FLOAT)pSource->y * (1.0f/32767.0f)),
        (pSource->z == -32768) ? -1.f : ((FLOAT)pSource->z * (1.0f/32767.0f)),
        (pSource->w == -32768) ? -1.f : ((FLOAT)pSource->w * (1.0f/32767.0f))
    };
    return vResult;
    }
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;
    XMVECTOR                Result;
    XMVECTOR                UnpackOffset, NegativeOne, Control;
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pSource);

    UnpackOffset = __vspltisw(3);
    UnpackOffset = __vcfsx(UnpackOffset, 0);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    Permute = __vsldoi(Permute, Permute, 2 << 2);
    Result = __vmulfp(UnpackScale, UnpackOffset);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_NORMSHORT4);
    V = __vnmsubfp(V, UnpackScale, Result);

    Control = __vcmpeqfp(V, V);
    Control = __vnor(Control, Control);
    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    Result = __vsel( V, NegativeOne, Control );

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackOffset, NegativeOne, Control;
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pSource);

    UnpackOffset = __vspltisw(3);
    UnpackOffset = __vcfsx(UnpackOffset, 0);
    VL = __lvlx(pSource, 0);
    VR = __lvrx(pSource, 8);
    Result = __vmulfp(UnpackScale, UnpackOffset);
    VL = __vsldoi(VL, VL, 2 << 2);
    V = __vor(VL, VR);
    V = __vupkd3d(V, VPACK_NORMSHORT4);
    V = __vnmsubfp(V, UnpackScale, Result);

    Control = __vcmpeqfp(V, V);
    Control = __vnor(Control, Control);
    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    Result = __vsel( V, NegativeOne, Control );

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadShort4
(
    CONST XMSHORT4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;
    V.vector4_f32[2] = (FLOAT)pSource->z;
    V.vector4_f32[3] = (FLOAT)pSource->w;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 short values into the most significant 32 bits
    V = __vupkhsh(V);                                                 // Unpack the 2 short values
    V = __vcfsx(V, 0);                                                // Convert to floating point

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;

    XMASSERT(pSource);

    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    V = __vor(VL, VR);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUShortN4
(
    CONST XMUSHORTN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x / 65535.0f;
    V.vector4_f32[1] = (FLOAT)pSource->y / 65535.0f;
    V.vector4_f32[2] = (FLOAT)pSource->z / 65535.0f;
    V.vector4_f32[3] = (FLOAT)pSource->w / 65535.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Zero;
    XMVECTOR                Permute;
    static CONST XMVECTOR   UnpackScale = {1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1)};

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    Zero = __vspltisw(0);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 ushort values into the most significant 32 bits
    V = __vmrghh(Zero, V);                                            // Unpack the 2 ushort values
    V = __vcfsx(V, 0);                                                // Convert to floating point
    V = __vmulfp(V, UnpackScale);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Zero;
    static CONST XMVECTOR   UnpackScale = {1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1), 1.0f / (FLOAT)((1 << 16) - 1)};

    XMASSERT(pSource);

    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    Zero = __vspltisw(0);
    V = __vor(VL, VR);
    V = __vmrghh(Zero, V);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, UnpackScale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUShort4
(
    CONST XMUSHORT4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;
    V.vector4_f32[2] = (FLOAT)pSource->z;
    V.vector4_f32[3] = (FLOAT)pSource->w;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Zero;
    XMVECTOR                Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    Zero = __vspltisw(0);
    V = __vperm(V0, V1, Permute);                                     // Shift the 2 ushort values into the most significant 32 bits
    V = __vmrghh(Zero, V);                                            // Unpack the 2 ushort values
    V = __vcfsx(V, 0);                                                // Convert to floating point

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                VL;
    XMVECTOR                VR;
    XMVECTOR                V;
    XMVECTOR                Zero;

    XMASSERT(pSource);

    VR = __lvrx(pSource, 8);
    VL = __lvlx(pSource, 0);
    VR = __vsldoi(VR, VR, 2 << 2);
    Zero = __vspltisw(0);
    V = __vor(VL, VR);
    V = __vmrghh(Zero, V);
    V = __vcfux(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadXIcoN4
(
    CONST XMXICON4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFF00000};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    Element = (UINT)(pSource->v & 0xFFFFF);
    V.vector4_f32[0] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    Element = (UINT)((pSource->v >> 20) & 0xFFFFF);
    V.vector4_f32[1] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    Element = (UINT)((pSource->v >> 40) & 0xFFFFF);
    V.vector4_f32[2] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 60) / 15.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFF00000};
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << 4) - 1)};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    Element = (UINT)pSource->v & 0xFFFFF;
    I[0] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[1] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[2] = Element | SignExtend[Element >> 19];
    I[3] = (UINT)(pSource->v >> 60);

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);
      
    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackScale;
    static CONST XMVECTOR   UnpackScaleOffset = {-XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << (20 - 1)) - 1), // Signed scale
                                                 -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << 4) - 1), // Unsigned scale
                                                 -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << (20 - 1)) - 1) * 3.0f, // Signed offset
                                                 -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << 4) - 1)}; // Unsigned offset

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Result = __vpermwi(UnpackScaleOffset, 0xAB); // UnpackOffset
    UnpackScale = __vpermwi(UnpackScaleOffset, 0x1);
    V = __vsldoi(V, V, 2 << 2);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Result = __vnmsubfp(V, UnpackScale, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadXIco4
(
    CONST XMXICO4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFF00000};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    Element = (UINT)(pSource->v & 0xFFFFF);
    V.vector4_f32[0] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    Element = (UINT)((pSource->v >> 20) & 0xFFFFF);
    V.vector4_f32[1] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    Element = (UINT)((pSource->v >> 40) & 0xFFFFF);
    V.vector4_f32[2] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 60);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFF00000};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    Element = (UINT)pSource->v & 0xFFFFF;
    I[0] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[1] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[2] = Element | SignExtend[Element >> 19];
    I[3] = (UINT)(pSource->v >> 60);

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackScale;
    static CONST XMVECTOR   UnpackScaleOffset = {-XM_UNPACK_FACTOR_SIGNED, // Signed scale
                                                 -XM_UNPACK_FACTOR_UNSIGNED, // Unsigned scale
                                                 -XM_UNPACK_FACTOR_SIGNED * 3.0f, // Signed offset
                                                 -XM_UNPACK_FACTOR_UNSIGNED}; // Unsigned offset

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Result = __vpermwi(UnpackScaleOffset, 0xAB); // UnpackOffset
    UnpackScale = __vpermwi(UnpackScaleOffset, 0x1);
    V = __vsldoi(V, V, 2 << 2);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Result = __vnmsubfp(V, UnpackScale, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUIcoN4
(
    CONST XMUICON4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)(pSource->v & 0xFFFFF) / 1048575.0f;
    V.vector4_f32[1] = (FLOAT)((pSource->v >> 20) & 0xFFFFF) / 1048575.0f;
    V.vector4_f32[2] = (FLOAT)((pSource->v >> 40) & 0xFFFFF) / 1048575.0f;
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 60) / 15.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    static CONST XMVECTORF32 Scale = {1.0f / (FLOAT)((1 << 20) - 1), 1.0f / (FLOAT)((1 << 20) - 1), 1.0f / (FLOAT)((1 << 20) - 1), 1.0f / (FLOAT)((1 << 4) - 1)};

    XMASSERT(pSource);

    I[0] = (UINT)pSource->v & 0xFFFFF;
    I[1] = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[2] = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[3] = (UINT)(pSource->v >> 60);

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    #define XM_SignedScale          (-XM_UNPACK_FACTOR_SIGNED)
    #define XM_UnsignedScaleOffset  (-XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << 4) - 1))
    #define XM_SignedOffset         (-XM_UNPACK_FACTOR_SIGNED * 3.0f)
    #define XM_URange               ((FLOAT)(1 << 20))
    #define XM_UMax                 ((FLOAT)((1 << 20) - 1))

    XMVECTOR V;
    XMVECTOR Scale1, Scale2, Mod;
    XMVECTOR Convert;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_SignedScale / XM_URange,
                                XM_SignedScale / XM_UMax,
                                XM_SignedOffset / XM_URange + 1.0f,
                                XM_UnsignedScaleOffset};
    static CONST XMVECTOR C1 = {XM_URange / XM_UMax,
                                0.0f,
                                (XM_SignedOffset + XM_URange) / XM_UMax,
                                XM_UnsignedScaleOffset};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Scale1 = __vpermwi(C0, 0x3);
    V = __vsldoi(V, V, 2 << 2);
    Convert = __vpermwi(C0, 0xAB);
    Scale2 = __vpermwi(C0, 0x57);
    Result = __vpermwi(C1, 0xAB);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Convert = __vnmsubfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vpermwi(C1, 0x1);
    Convert = __vrfiz(Convert);
    Result = __vnmsubfp(Convert, Mod, Result);

    return Result;

    #undef XM_SignedScale
    #undef XM_UnsignedScaleOffset
    #undef XM_SignedOffset
    #undef XM_URange
    #undef XM_UMax
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUIco4
(
    CONST XMUICO4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)(pSource->v & 0xFFFFF);
    V.vector4_f32[1] = (FLOAT)((pSource->v >> 20) & 0xFFFFF);
    V.vector4_f32[2] = (FLOAT)((pSource->v >> 40) & 0xFFFFF);
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 60);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;

    XMASSERT(pSource);

    I[0] = (UINT)pSource->v & 0xFFFFF;
    I[1] = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[2] = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[3] = (UINT)(pSource->v >> 60);

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    #define XM_SignedScale          (-XM_UNPACK_FACTOR_SIGNED)
    #define XM_UnsignedScaleOffset  (-XM_UNPACK_FACTOR_UNSIGNED)
    #define XM_SignedOffset         (-XM_UNPACK_FACTOR_SIGNED * 3.0f)
    #define XM_URange               ((FLOAT)(1 << 20))
    #define XM_UMax                 ((FLOAT)((1 << 20) - 1))

    XMVECTOR V;
    XMVECTOR Scale1, Scale2, Mod;
    XMVECTOR Convert;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_SignedScale / XM_URange,
                                XM_SignedScale,
                                XM_SignedOffset / XM_URange + 1.0f,
                                XM_UnsignedScaleOffset};
    static CONST XMVECTOR C1 = {XM_URange,
                                0.0f,
                                XM_SignedOffset + XM_URange,
                                XM_UnsignedScaleOffset};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Scale1 = __vpermwi(C0, 0x3);
    V = __vsldoi(V, V, 2 << 2);
    Convert = __vpermwi(C0, 0xAB);
    Scale2 = __vpermwi(C0, 0x57);
    Result = __vpermwi(C1, 0xAB);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Convert = __vnmsubfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vpermwi(C1, 0x1);
    Convert = __vrfiz(Convert);
    Result = __vnmsubfp(Convert, Mod, Result);

    return Result;

    #undef XM_SignedScale
    #undef XM_UnsignedScaleOffset
    #undef XM_SignedOffset
    #undef XM_URange
    #undef XM_UMax
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadIcoN4
(
    CONST XMICON4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFF00000};
    static CONST UINT SignExtendW[] = {0x00000000, 0xFFFFFFF0};

    XMASSERT(pSource);

    Element = (UINT)(pSource->v & 0xFFFFF);
    V.vector4_f32[0] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    Element = (UINT)((pSource->v >> 20) & 0xFFFFF);
    V.vector4_f32[1] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    Element = (UINT)((pSource->v >> 40) & 0xFFFFF);
    V.vector4_f32[2] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]) / 524287.0f;
    Element = (UINT)(pSource->v >> 60);
    V.vector4_f32[3] = (FLOAT)(INT)(Element | SignExtendW[Element >> 3]) / 7.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFF00000};
    static CONST UINT      SignExtendW[] = {0x00000000, 0xFFFFFFF0};
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << (20 - 1)) - 1), 1.0f / (FLOAT)((1 << (4 - 1)) - 1)};

    XMASSERT(pSource);

    Element = (UINT)pSource->v & 0xFFFFF;
    I[0] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[1] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[2] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 60);
    I[3] = Element | SignExtendW[Element >> 3];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    #define XM_SignedScale          (-XM_UNPACK_FACTOR_SIGNED)
    #define XM_UnsignedScaleOffset  (-XM_UNPACK_FACTOR_UNSIGNED)
    #define XM_SignedOffset         (-XM_UNPACK_FACTOR_SIGNED * 3.0f)
    #define XM_URange               ((FLOAT)(1 << 4))
    #define XM_URangeDiv2           ((FLOAT)(1 << 3))

    XMVECTOR V;
    XMVECTOR Scale1, Scale2, Mod;
    XMVECTOR Convert;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_SignedScale / (FLOAT)((1 << (20 - 1)) - 1),
                                XM_SignedOffset / (FLOAT)((1 << (20 - 1)) - 1),
                                XM_URange / (FLOAT)((1 << (4 - 1)) - 1),
                                XM_UnsignedScaleOffset / (FLOAT)((1 << (4 - 1)) - 1)};
    static CONST XMVECTOR C1 = {XM_SignedScale,
                                XM_UnsignedScaleOffset / XM_URange,
                                XM_SignedOffset,
                                (XM_UnsignedScaleOffset + XM_URangeDiv2) / XM_URange};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Scale1 = __vpermwi(C1, 0x1);
    V = __vsldoi(V, V, 2 << 2);
    Convert = __vpermwi(C1, 0xAB);
    Scale2 = __vpermwi(C0, 0x3);
    Result = __vpermwi(C0, 0x57);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Mod = __vspltisw(0);
    Convert = __vnmsubfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vrlimi(Mod, C0, 0x1, 3);
    Convert = __vrfiz(Convert);
    Result = __vnmsubfp(Convert, Mod, Result);

    return Result;

    #undef XM_SignedScale
    #undef XM_UnsignedScaleOffset
    #undef XM_SignedOffset
    #undef XM_URange
    #undef XM_URangeDiv2
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadIco4
(
    CONST XMICO4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFF00000};
    static CONST UINT SignExtendW[] = {0x00000000, 0xFFFFFFF0};

    XMASSERT(pSource);

    Element = (UINT)(pSource->v & 0xFFFFF);
    V.vector4_f32[0] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    Element = (UINT)((pSource->v >> 20) & 0xFFFFF);
    V.vector4_f32[1] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    Element = (UINT)((pSource->v >> 40) & 0xFFFFF);
    V.vector4_f32[2] = (FLOAT)(INT)(Element | SignExtend[Element >> 19]);
    Element = (UINT)(pSource->v >> 60);
    V.vector4_f32[3] = (FLOAT)(INT)(Element | SignExtendW[Element >> 3]);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFF00000};
    static CONST UINT      SignExtendW[] = {0x00000000, 0xFFFFFFF0};

    XMASSERT(pSource);

    Element = (UINT)pSource->v & 0xFFFFF;
    I[0] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 20) & 0xFFFFF;
    I[1] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 40) & 0xFFFFF;
    I[2] = Element | SignExtend[Element >> 19];
    Element = (UINT)(pSource->v >> 60);
    I[3] = Element | SignExtendW[Element >> 3];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    #define XM_SignedScale          (-XM_UNPACK_FACTOR_SIGNED)
    #define XM_UnsignedScaleOffset  (-XM_UNPACK_FACTOR_UNSIGNED)
    #define XM_SignedOffset         (-XM_UNPACK_FACTOR_SIGNED * 3.0f)
    #define XM_URange               ((FLOAT)(1 << 4))
    #define XM_URangeDiv2           ((FLOAT)(1 << 3))

    XMVECTOR V;
    XMVECTOR Scale1, Scale2, Mod;
    XMVECTOR Convert;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_SignedScale,
                                XM_SignedOffset,
                                XM_UnsignedScaleOffset / XM_URange,
                                XM_UnsignedScaleOffset};
    static CONST XMVECTOR C1 = {0.0f,
                                XM_URange,
                                XM_SignedOffset,
                                (XM_UnsignedScaleOffset + XM_URangeDiv2) / XM_URange};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 7) == 0);
    XMASSERT((pSource->v & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 20) & 0xFFFFFull) != 0x80000ull);
    XMASSERT(((pSource->v >> 40) & 0xFFFFFull) != 0x80000ull);

    V = __lvlx(pSource, 0);
    Scale1 = __vpermwi(C0, 0x2);
    V = __vsldoi(V, V, 2 << 2);
    Convert = __vpermwi(C1, 0xAB);
    Scale2 = __vpermwi(C0, 0x3);
    Result = __vpermwi(C0, 0x57);
    V = __vupkd3d(V, VPACK_NORMPACKED64);
    Convert = __vnmsubfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vpermwi(C1, 0x1);
    Convert = __vrfiz(Convert);
    Result = __vnmsubfp(Convert, Mod, Result);

    return Result;

    #undef XM_SignedScale
    #undef XM_UnsignedScaleOffset
    #undef XM_SignedOffset
    #undef XM_URange
    #undef XM_URangeDiv2
#endif // _XM_VMX128_INTRINSICS_
}


//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadXDecN4
(
    CONST XMXDECN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR V;
    UINT Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 30) / 3.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFFFFC00};
    static CONST XMVECTORF32  Scale = {1.956947162e-3f, 1.956947162e-3f, 1.956947162e-3f, 3.333333333e-1f};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 10) & 0x3FF;
    I[1] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 20) & 0x3FF;
    I[2] = Element | SignExtend[Element >> 9];
    I[3] = pSource->v >> 30;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackScale;
    static CONST XMVECTOR   UnpackScaleOffset = {-XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << (10 - 1)) - 1), // Signed scale
                                                 -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << 2) - 1), // Unsigned scale
                                                 -XM_UNPACK_FACTOR_SIGNED / (FLOAT)((1 << (10 - 1)) - 1) * 3.0f, // Signed offset
                                                 -XM_UNPACK_FACTOR_UNSIGNED / (FLOAT)((1 << 2) - 1)}; // Unsigned offset

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    V = __lvlx(pSource, 0);
    Result = __vpermwi(UnpackScaleOffset, 0xAB); // UnpackOffset
    UnpackScale = __vpermwi(UnpackScaleOffset, 0x1);
    V = __vsldoi(V, V, 1 << 2);
    V = __vupkd3d(V, VPACK_NORMPACKED32);
    Result = __vnmsubfp(V, UnpackScale, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadXDec4
(
    CONST XMXDEC4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 30);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFFFFC00};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 10) & 0x3FF;
    I[1] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 20) & 0x3FF;
    I[2] = Element | SignExtend[Element >> 9];
    I[3] = pSource->v >> 30;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                Result;
    XMVECTOR                UnpackScale;
    static CONST XMVECTOR   UnpackScaleOffset = {-XM_UNPACK_FACTOR_SIGNED, // Signed scale
                                                 -XM_UNPACK_FACTOR_UNSIGNED, // Unsigned scale
                                                 -XM_UNPACK_FACTOR_SIGNED * 3.0f, // Signed offset
                                                 -XM_UNPACK_FACTOR_UNSIGNED}; // Unsigned offset

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);

    V = __lvlx(pSource, 0);
    Result = __vpermwi(UnpackScaleOffset, 0xAB); // UnpackOffset
    UnpackScale = __vpermwi(UnpackScaleOffset, 0x1);
    V = __vsldoi(V, V, 1 << 2);
    V = __vupkd3d(V, VPACK_NORMPACKED32);
    Result = __vnmsubfp(V, UnpackScale, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUDecN4
(
    CONST XMUDECN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)Element / 1023.0f;
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)Element / 1023.0f;
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)Element / 1023.0f;
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 30) / 3.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    static CONST XMVECTORF32  Scale = {1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 2) - 1)};

    XMASSERT(pSource);

    I[0] = pSource->v & 0x3FF;
    I[1] = (pSource->v >> 10) & 0x3FF;
    I[2] = (pSource->v >> 20) & 0x3FF;
    I[3] = pSource->v >> 30;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Scale;
    XMVECTOR               Mask;
    static CONST __vector4i C0 = {0x3A802008, 0x3EAAAAAB, 0x3FF, 0}; // {1.0f / (FLOAT)((1 << 10) - 1), 1.0f / (FLOAT)((1 << 2) - 1), *(FLOAT*)&0x3FF, 0.0f};
    static CONST __vector4i Shift = {0, 10, 20, 30};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    V = __vspltw(V, 0);
    Mask = __vspltw(*(XMVECTOR*)&C0, 2);
    V = __vsrw(V, *(XMVECTOR*)&Shift);
    V = __vand(V, Mask);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0x1);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUDec4
(
    CONST XMUDEC4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)Element;
    V.vector4_f32[3] = (FLOAT)(pSource->v >> 30);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;

    XMASSERT(pSource);

    I[0] = pSource->v & 0x3FF;
    I[1] = (pSource->v >> 10) & 0x3FF;
    I[2] = (pSource->v >> 20) & 0x3FF;
    I[3] = pSource->v >> 30;

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               Shift;
    XMVECTOR               Mask;
    static CONST __vector4i ShiftMask = {0x3FF0000, 0x3FF000A, 0x3FF0014, 0x3FF001E};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    Shift = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    Mask = __vsrw(*(XMVECTOR*)&ShiftMask, Shift);
    V = __vsrw(V, *(XMVECTOR*)&ShiftMask);
    V = __vand(V, Mask);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadDecN4
(
    CONST XMDECN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT SignExtendW[] = {0x00000000, 0xFFFFFFFC};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 30) & 0x3) != 0x2);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]) / 511.0f;
    Element = pSource->v >> 30;
    V.vector4_f32[3] = (FLOAT)(SHORT)(Element | SignExtendW[Element >> 1]);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT      SignExtendW[] = {0x00000000, 0xFFFFFFFC};
    static CONST XMVECTORF32  Scale = {1.956947162e-3f, 1.956947162e-3f, 1.956947162e-3f, 1.0f};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 30) & 0x3) != 0x2);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 10) & 0x3FF;
    I[1] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 20) & 0x3FF;
    I[2] = Element | SignExtend[Element >> 9];
    Element = pSource->v >> 30;
    I[3] = Element | SignExtendW[Element >> 1];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Scale = {0x3B004020, 0x3B004020, 0x3B004020, 0x3F800000}; // {1.0f / (FLOAT)((1 << (10 - 1)) - 1), ..., 1.0f / (FLOAT)((1 << (2 - 1)) - 1)};
    static CONST __vector4i Shift = {0x00160016, 0x0016000C, 0x00160002, 0x001E0000};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, *(XMVECTOR*)&Scale);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadDec4
(
    CONST XMDEC4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR          V;
    UINT              Element;
    static CONST UINT SignExtend[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT SignExtendW[] = {0x00000000, 0xFFFFFFFC};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 30) & 0x3) != 0x2);

    Element = pSource->v & 0x3FF;
    V.vector4_f32[0] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    Element = (pSource->v >> 10) & 0x3FF;
    V.vector4_f32[1] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    Element = (pSource->v >> 20) & 0x3FF;
    V.vector4_f32[2] = (FLOAT)(SHORT)(Element | SignExtend[Element >> 9]);
    Element = pSource->v >> 30;
    V.vector4_f32[3] = (FLOAT)(SHORT)(Element | SignExtendW[Element >> 1]);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR               V;
    __vector4i              I;
    UINT                   Element;
    static CONST UINT      SignExtend[] = {0x00000000, 0xFFFFFC00};
    static CONST UINT      SignExtendW[] = {0x00000000, 0xFFFFFFFC};

    XMASSERT(pSource);
    XMASSERT((pSource->v & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 10) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 20) & 0x3FF) != 0x200);
    XMASSERT(((pSource->v >> 30) & 0x3) != 0x2);

    Element = pSource->v & 0x3FF;
    I[0] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 10) & 0x3FF;
    I[1] = Element | SignExtend[Element >> 9];
    Element = (pSource->v >> 20) & 0x3FF;
    I[2] = Element | SignExtend[Element >> 9];
    Element = pSource->v >> 30;
    I[3] = Element | SignExtendW[Element >> 1];

    V = __lvx(I, 0);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR               V;
    XMVECTOR               ShiftR;
    static CONST __vector4i Shift = {0x00160016, 0x0016000C, 0x00160002, 0x001E0000};

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    ShiftR = __vspltisw(-16); // Shift 16
    V = __vspltw(V, 0);
    ShiftR = __vsrw(*(XMVECTOR*)&Shift, ShiftR);
    V = __vslw(V, *(XMVECTOR*)&Shift);
    V = __vsraw(V, ShiftR);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUByteN4
(
    CONST XMUBYTEN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x / 255.0f;
    V.vector4_f32[1] = (FLOAT)pSource->y / 255.0f;
    V.vector4_f32[2] = (FLOAT)pSource->z / 255.0f;
    V.vector4_f32[3] = (FLOAT)pSource->w / 255.0f;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;
    static CONST XMVECTOR   UnpackOffset = XM_UNPACK_UNSIGNEDN_OFFSET(8, 8, 8, 8);
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    Permute = __vsldoi(Permute, Permute, 1 << 2);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_D3DCOLOR);
    V = __vmaddcfp(UnpackScale, V, UnpackOffset);
    V = __vpermwi(V, 0xC6); // 3, 0, 1, 2

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    static CONST XMVECTOR   UnpackOffset = XM_UNPACK_UNSIGNEDN_OFFSET(8, 8, 8, 8);
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    V = __vsldoi(V, V, 1 << 2);
    V = __vupkd3d(V, VPACK_D3DCOLOR);
    V = __vmaddcfp(UnpackScale, V, UnpackOffset);
    V = __vpermwi(V, 0xC6); // 3, 0, 1, 2

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUByte4
(
    CONST XMUBYTE4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;
    V.vector4_f32[2] = (FLOAT)pSource->z;
    V.vector4_f32[3] = (FLOAT)pSource->w;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Zero;
    XMVECTOR                Permute;
//    static CONST __vector4i  SwizzleWZYX = {XM_PERMUTE_0W, XM_PERMUTE_0Z, XM_PERMUTE_0Y, XM_PERMUTE_0X};

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 7);
    Zero = __vspltisw(0);
    V = __vperm(V0, V1, Permute);
    V = __vmrghb(Zero, V);
    V = __vmrghh(Zero, V);
    V = __vcfux(V, 0);
//    V = __vperm(V, V, *(XMVECTOR*)&SwizzleWZYX);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                Zero;

    XMASSERT(pSource);

    Zero = __vspltisw(0);

    V = __lvlx(pSource, 0);
    V = __vmrghb(Zero, V);
    V = __vmrghh(Zero, V);
    V = __vcfux(V, 0);
//    V = __vpermwi(V, 0xE4); // 3, 2, 1, 0

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadByteN4
(
    CONST XMBYTEN4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (pSource->x == -128) ? -1.f : ((FLOAT)pSource->x / 127.0f);
    V.vector4_f32[1] = (pSource->y == -128) ? -1.f : ((FLOAT)pSource->y / 127.0f);
    V.vector4_f32[2] = (pSource->z == -128) ? -1.f : ((FLOAT)pSource->z / 127.0f);
    V.vector4_f32[3] = (pSource->w == -128) ? -1.f : ((FLOAT)pSource->w / 127.0f);

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute, NegativeOne;
    static CONST XMVECTORF32   Scale = {1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1)};

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    V = __vperm(V0, V1, Permute);
    V = __vupkhsb(V);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    V = __vmaxfp( V, NegativeOne );

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    XMVECTOR                NegativeOne;
    static CONST XMVECTORF32   Scale = {1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1), 
                                     1.0f / (FLOAT)((1 << (8 - 1)) - 1)};

    XMASSERT(pSource);

    V = __lvlx(pSource, 0);
    V = __vupkhsb(V);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);
    V = __vmulfp(V, Scale.v);

    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);
    V = __vmaxfp( V, NegativeOne );

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadByte4
(
    CONST XMBYTE4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;

    XMASSERT(pSource);

    V.vector4_f32[0] = (FLOAT)pSource->x;
    V.vector4_f32[1] = (FLOAT)pSource->y;
    V.vector4_f32[2] = (FLOAT)pSource->z;
    V.vector4_f32[3] = (FLOAT)pSource->w;

    return V;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    V = __vperm(V0, V1, Permute);
    V = __vupkhsb(V);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR V;

    XMASSERT(pSource);

    V = __lvlx(pSource, 0);
    V = __vupkhsb(V);
    V = __vupkhsh(V);
    V = __vcfsx(V, 0);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadUNibble4
(
     CONST XMUNIBBLE4* pSource
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0xF;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 4) & 0xF;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 8) & 0xF;
    V.vector4_f32[2] = (FLOAT)Element;
    Element = (pSource->v >> 12) & 0xF;
    V.vector4_f32[3] = (FLOAT)Element;

    return V;
#endif // !_XM_SSE_INTRISICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadU555
(
     CONST XMU555* pSource
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR          V;
    UINT              Element;

    XMASSERT(pSource);

    Element = pSource->v & 0x1F;
    V.vector4_f32[0] = (FLOAT)Element;
    Element = (pSource->v >> 5) & 0x1F;
    V.vector4_f32[1] = (FLOAT)Element;
    Element = (pSource->v >> 10) & 0x1F;
    V.vector4_f32[2] = (FLOAT)Element;
    Element = (pSource->v >> 15) & 0x1;
    V.vector4_f32[3] = (FLOAT)Element;

    return V;
#endif // !_XM_SSE_INTRISICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMLoadColor
(
    CONST XMCOLOR* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMASSERT(pSource);
    {
    // INT -> Float conversions are done in one instruction.
    // UINT -> Float calls a runtime function. Keep in INT
    INT iColor = (INT)(pSource->c);
    XMVECTOR vColor = {
        (FLOAT)((iColor >> 16) & 0xFF) * (1.0f/255.0f),
        (FLOAT)((iColor >> 8) & 0xFF) * (1.0f/255.0f),
        (FLOAT)(iColor & 0xFF) * (1.0f/255.0f),
        (FLOAT)((iColor >> 24) & 0xFF) * (1.0f/255.0f)
    };
    return vColor;
    }
#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR                V;
    XMVECTOR                V0, V1;
    XMVECTOR                Permute;
    static CONST XMVECTOR   UnpackOffset = XM_UNPACK_UNSIGNEDN_OFFSET(8, 8, 8, 8);
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 3);
    Permute = __vsldoi(Permute, Permute, 1 << 2);
    V = __vperm(V0, V1, Permute);
    V = __vupkd3d(V, VPACK_D3DCOLOR);
    V = __vmaddcfp(UnpackScale, V, UnpackOffset);

    return V;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR                V;
    static CONST XMVECTOR   UnpackOffset = XM_UNPACK_UNSIGNEDN_OFFSET(8, 8, 8, 8);
    static CONST XMVECTOR   UnpackScale = XM_UNPACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    V = __lvlx(pSource, 0);
    V = __vsldoi(V, V, 1 << 2);
    V = __vupkd3d(V, VPACK_D3DCOLOR);
    V = __vmaddcfp(UnpackScale, V, UnpackOffset);

    return V;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMMATRIX XMLoadFloat3x3
(
    CONST XMFLOAT3X3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX M;

    XMASSERT(pSource);

    M.r[0].vector4_f32[0] = pSource->m[0][0];
    M.r[0].vector4_f32[1] = pSource->m[0][1];
    M.r[0].vector4_f32[2] = pSource->m[0][2];
    M.r[0].vector4_f32[3] = 0.0f;

    M.r[1].vector4_f32[0] = pSource->m[1][0];
    M.r[1].vector4_f32[1] = pSource->m[1][1];
    M.r[1].vector4_f32[2] = pSource->m[1][2];
    M.r[1].vector4_f32[3] = 0.0f;

    M.r[2].vector4_f32[0] = pSource->m[2][0];
    M.r[2].vector4_f32[1] = pSource->m[2][1];
    M.r[2].vector4_f32[2] = pSource->m[2][2];
    M.r[2].vector4_f32[3] = 0.0f;

    M.r[3].vector4_f32[0] = 0.0f;
    M.r[3].vector4_f32[1] = 0.0f;
    M.r[3].vector4_f32[2] = 0.0f;
    M.r[3].vector4_f32[3] = 1.0f;

    return M;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V0, V1, V2, V3;
    XMVECTOR P0, P1, P2;
    XMVECTOR ZO;
    XMMATRIX M;

    XMASSERT(pSource);

    XMDUMMY_INITIALIZE_VECTOR(ZO);

    V0 = __lvx(pSource, 0);
    V1 = __lvx(pSource, 12);
    V2 = __lvx(pSource, 24);
    V3 = __lvx(pSource, 35);

    P0 = __lvsl(pSource, 0);
    P1 = __lvsl(pSource, 12);
    P2 = __lvsl(pSource, 24);

    ZO = __vupkd3d(ZO, VPACK_NORMSHORT2);

    M.r[0] = __vperm(V0, V1, P0);
    M.r[1] = __vperm(V1, V2, P1);
    M.r[2] = __vperm(V2, V3, P2);
    M.r[3] = __vpermwi(ZO, 0xAB);

    M.r[0] = __vrlimi(M.r[0], ZO, 0x1, 3);
    M.r[1] = __vrlimi(M.r[1], ZO, 0x1, 3);
    M.r[2] = __vrlimi(M.r[2], ZO, 0x1, 3);

    return M;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR V0L, V0R;
    XMVECTOR V1L, V1R, V1;
    XMVECTOR V2;
    XMVECTOR ZO;
    XMMATRIX M;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(ZO);

    V0L = __lvlx(pSource, 0);
    V0R = __lvrx(pSource, 16);
    V1L = __lvlx(pSource, 16);
    V1R = __lvrx(pSource, 32);
    V2  = __lvlx(pSource, 32);

    ZO = __vupkd3d(ZO, VPACK_NORMSHORT2);

    M.r[0] = __vor(V0L, V0R);
    V1 = __vor(V1L, V1R);
    V2 = __vrlimi(V2, ZO, 0x4, 1);

    M.r[1] = __vsldoi(M.r[0], V1, 3 << 2);

    M.r[3] = __vpermwi(ZO, 0xAB);
    M.r[0] = __vrlimi(M.r[0], ZO, 0x1, 3);
    M.r[2] = __vsldoi(V1, V2, 2 << 2);
    M.r[1] = __vrlimi(M.r[1], ZO, 0x1, 3);

    return M;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMMATRIX XMLoadFloat4x3
(
    CONST XMFLOAT4X3* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMMATRIX M;
    XMASSERT(pSource);

    ((UINT *)(&M.r[0].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[0][0]))[0];
    ((UINT *)(&M.r[0].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[0][1]))[0];
    ((UINT *)(&M.r[0].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[0][2]))[0];
    M.r[0].vector4_f32[3] = 0.0f;

    ((UINT *)(&M.r[1].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[1][0]))[0];
    ((UINT *)(&M.r[1].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[1][1]))[0];
    ((UINT *)(&M.r[1].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[1][2]))[0];
    M.r[1].vector4_f32[3] = 0.0f;

    ((UINT *)(&M.r[2].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[2][0]))[0];
    ((UINT *)(&M.r[2].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[2][1]))[0];
    ((UINT *)(&M.r[2].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[2][2]))[0];
    M.r[2].vector4_f32[3] = 0.0f;

    ((UINT *)(&M.r[3].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[3][0]))[0];
    ((UINT *)(&M.r[3].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[3][1]))[0];
    ((UINT *)(&M.r[3].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[3][2]))[0];
    M.r[3].vector4_f32[3] = 1.0f;

    return M;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V0, V1, V2, V3, V4;
    XMVECTOR P0, P1, P2, P3;
    XMVECTOR ZO;
    XMMATRIX M;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    V1 = __lvx(pSource, 12);
    V2 = __lvx(pSource, 24);
    V3 = __lvx(pSource, 36);
    V4 = __lvx(pSource, 47);

    P0 = __lvsl(pSource, 0);
    P1 = __lvsl(pSource, 12);
    P2 = __lvsl(pSource, 24);
    P3 = __lvsl(pSource, 36);

    XMDUMMY_INITIALIZE_VECTOR(ZO);

    M.r[0] = __vperm(V0, V1, P0);
    M.r[1] = __vperm(V1, V2, P1);
    M.r[2] = __vperm(V2, V3, P2);
    M.r[3] = __vperm(V3, V4, P3);

    ZO = __vupkd3d(ZO, VPACK_NORMSHORT2);

    M.r[0] = __vrlimi(M.r[0], M.r[3], 0x1, 3);
    M.r[1] = __vrlimi(M.r[1], M.r[3], 0x1, 3);
    M.r[2] = __vrlimi(M.r[2], M.r[3], 0x1, 3);
    M.r[3] = __vrlimi(M.r[3], ZO, 0x1, 0);

    return M;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR V0L, V0R;
    XMVECTOR V1L, V1R, V1;
    XMVECTOR V2L, V2R, V2;
    XMMATRIX M;

    XMASSERT(pSource);

    XMDUMMY_INITIALIZE_VECTOR(M.r[3]);

    V0L = __lvlx(pSource, 0);
    V0R = __lvrx(pSource, 16);
    V1L = __lvlx(pSource, 16);
    V1R = __lvrx(pSource, 32);
    V2L = __lvlx(pSource, 32);
    V2R = __lvrx(pSource, 48);

    M.r[3] = __vupkd3d(M.r[3], VPACK_NORMSHORT2);

    M.r[0] = __vor(V0L, V0R);
    V1 = __vor(V1L, V1R);
    V2 = __vor(V2L, V2R);

    M.r[1] = __vsldoi(M.r[0], V1, 3 << 2);
    M.r[2] = __vsldoi(V1, V2, 2 << 2);

    M.r[0] = __vrlimi(M.r[0], M.r[3], 0x1, 3);
    M.r[1] = __vrlimi(M.r[1], M.r[3], 0x1, 3);
    M.r[2] = __vrlimi(M.r[2], M.r[3], 0x1, 3);
    M.r[3] = __vrlimi(M.r[3], V2, 0xE, 1);

    return M;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMMATRIX XMLoadFloat4x3A
(
    CONST XMFLOAT4X3A* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX M;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    M.r[0].vector4_f32[0] = pSource->m[0][0];
    M.r[0].vector4_f32[1] = pSource->m[0][1];
    M.r[0].vector4_f32[2] = pSource->m[0][2];
    M.r[0].vector4_f32[3] = 0.0f;

    M.r[1].vector4_f32[0] = pSource->m[1][0];
    M.r[1].vector4_f32[1] = pSource->m[1][1];
    M.r[1].vector4_f32[2] = pSource->m[1][2];
    M.r[1].vector4_f32[3] = 0.0f;

    M.r[2].vector4_f32[0] = pSource->m[2][0];
    M.r[2].vector4_f32[1] = pSource->m[2][1];
    M.r[2].vector4_f32[2] = pSource->m[2][2];
    M.r[2].vector4_f32[3] = 0.0f;

    M.r[3].vector4_f32[0] = pSource->m[3][0];
    M.r[3].vector4_f32[1] = pSource->m[3][1];
    M.r[3].vector4_f32[2] = pSource->m[3][2];
    M.r[3].vector4_f32[3] = 1.0f;

    return M;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1, V2;
    XMMATRIX M;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    XMDUMMY_INITIALIZE_VECTOR(M.r[3]);

    M.r[0] = __lvx(pSource, 0);
    V1 = __lvx(pSource, 16);
    V2 = __lvx(pSource, 32);

    M.r[3] = __vupkd3d(M.r[3], VPACK_NORMSHORT2);

    M.r[1] = __vsldoi(M.r[0], V1, 3 << 2);
    M.r[2] = __vsldoi(V1, V2, 2 << 2);

    M.r[0] = __vrlimi(M.r[0], M.r[3], 0x1, 3);
    M.r[1] = __vrlimi(M.r[1], M.r[3], 0x1, 3);
    M.r[2] = __vrlimi(M.r[2], M.r[3], 0x1, 3);
    M.r[3] = __vrlimi(M.r[3], V2, 0xE, 1);

    return M;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMMATRIX XMLoadFloat4x4
(
    CONST XMFLOAT4X4* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMMATRIX M;
    XMASSERT(pSource);

    ((UINT *)(&M.r[0].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[0][0]))[0];
    ((UINT *)(&M.r[0].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[0][1]))[0];
    ((UINT *)(&M.r[0].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[0][2]))[0];
    ((UINT *)(&M.r[0].vector4_f32[3]))[0] = ((const UINT *)(&pSource->m[0][3]))[0];

    ((UINT *)(&M.r[1].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[1][0]))[0];
    ((UINT *)(&M.r[1].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[1][1]))[0];
    ((UINT *)(&M.r[1].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[1][2]))[0];
    ((UINT *)(&M.r[1].vector4_f32[3]))[0] = ((const UINT *)(&pSource->m[1][3]))[0];

    ((UINT *)(&M.r[2].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[2][0]))[0];
    ((UINT *)(&M.r[2].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[2][1]))[0];
    ((UINT *)(&M.r[2].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[2][2]))[0];
    ((UINT *)(&M.r[2].vector4_f32[3]))[0] = ((const UINT *)(&pSource->m[2][3]))[0];

    ((UINT *)(&M.r[3].vector4_f32[0]))[0] = ((const UINT *)(&pSource->m[3][0]))[0];
    ((UINT *)(&M.r[3].vector4_f32[1]))[0] = ((const UINT *)(&pSource->m[3][1]))[0];
    ((UINT *)(&M.r[3].vector4_f32[2]))[0] = ((const UINT *)(&pSource->m[3][2]))[0];
    ((UINT *)(&M.r[3].vector4_f32[3]))[0] = ((const UINT *)(&pSource->m[3][3]))[0];

    return M;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMMATRIX M;
    XMVECTOR V0, V1, V2, V3, V4;
    XMVECTOR Permute;

    XMASSERT(pSource);

    V0 = __lvx(pSource, 0);
    Permute = __lvsl(pSource, 0);
    V1 = __lvx(pSource, 16);
    V2 = __lvx(pSource, 32);
    V3 = __lvx(pSource, 48);
    V4 = __lvx(pSource, 63);
    M.r[0] = __vperm(V0, V1, Permute);
    M.r[1] = __vperm(V1, V2, Permute);
    M.r[2] = __vperm(V2, V3, Permute);
    M.r[3] = __vperm(V3, V4, Permute);

    return M;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR V0L, V0R;
    XMVECTOR V1L, V1R;
    XMVECTOR V2L, V2R;
    XMVECTOR V3L, V3R;
    XMMATRIX M;

    XMASSERT(pSource);

    V0L = __lvlx(pSource, 0);
    V0R = __lvrx(pSource, 16);
    V1L = __lvlx(pSource, 16);
    V1R = __lvrx(pSource, 32);
    V2L = __lvlx(pSource, 32);
    V2R = __lvrx(pSource, 48);
    V3L = __lvlx(pSource, 48);
    V3R = __lvrx(pSource, 64);

    M.r[0] = __vor(V0L, V0R);
    M.r[1] = __vor(V1L, V1R);
    M.r[2] = __vor(V2L, V2R);
    M.r[3] = __vor(V3L, V3R);

    return M;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMMATRIX XMLoadFloat4x4A
(
    CONST XMFLOAT4X4A* pSource
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX M;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    M.r[0].vector4_f32[0] = pSource->m[0][0];
    M.r[0].vector4_f32[1] = pSource->m[0][1];
    M.r[0].vector4_f32[2] = pSource->m[0][2];
    M.r[0].vector4_f32[3] = pSource->m[0][3];

    M.r[1].vector4_f32[0] = pSource->m[1][0];
    M.r[1].vector4_f32[1] = pSource->m[1][1];
    M.r[1].vector4_f32[2] = pSource->m[1][2];
    M.r[1].vector4_f32[3] = pSource->m[1][3];

    M.r[2].vector4_f32[0] = pSource->m[2][0];
    M.r[2].vector4_f32[1] = pSource->m[2][1];
    M.r[2].vector4_f32[2] = pSource->m[2][2];
    M.r[2].vector4_f32[3] = pSource->m[2][3];

    M.r[3].vector4_f32[0] = pSource->m[3][0];
    M.r[3].vector4_f32[1] = pSource->m[3][1];
    M.r[3].vector4_f32[2] = pSource->m[3][2];
    M.r[3].vector4_f32[3] = pSource->m[3][3];

    return M;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMMATRIX M;

    XMASSERT(pSource);
    XMASSERT(((UINT_PTR)pSource & 0xF) == 0);

    M.r[0] = __lvx(pSource, 0);
    M.r[1] = __lvx(pSource, 16);
    M.r[2] = __lvx(pSource, 32);
    M.r[3] = __lvx(pSource, 48);

    return M;
#endif // _XM_VMX128_INTRINSICS_
}

/****************************************************************************
 *
 * Vector and matrix store operations
 *
 ****************************************************************************/

XMFINLINE VOID XMStoreInt
(
    UINT*    pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    *pDestination = XMVectorGetIntX( V );

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    __stvewx(X, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat
(
    FLOAT*    pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    *pDestination = XMVectorGetX( V );

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    __stvewx(X, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt2
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    Y = __vspltw(V, 1);
    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreSInt2
(
    XMINT2* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = (INT)V.vector4_f32[0];
    pDestination->y = (INT)V.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, V0;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    V0 = __vctsxs(V, 0);
    X = __vspltw(V0, 0);
    Y = __vspltw(V0, 1);
    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUInt2
(
    XMUINT2* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = (UINT)V.vector4_f32[0];
    pDestination->y = (UINT)V.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, V0;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    V0 = __vctuxs(V, 0);
    X = __vspltw(V0, 0);
    Y = __vspltw(V0, 1);
    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt2A
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AlignedV;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    AlignedV = __vsldoi(V, V, 2 << 2);
    __stvrx(AlignedV, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat2
(
    XMFLOAT2* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    Y = __vspltw(V, 1);
    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat2A
(
    XMFLOAT2A*   pDestination, 
    FXMVECTOR     V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AlignedV;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    AlignedV = __vsldoi(V, V, 2 << 2);
    __stvrx(AlignedV, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreHalf2
(
    XMHALF2* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->x = XMConvertFloatToHalf(V.vector4_f32[0]);
    pDestination->y = XMConvertFloatToHalf(V.vector4_f32[1]);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR H;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(H);
    H = __vpkd3d(H, V, VPACK_FLOAT16_2, VPACK_32, 3);
    H = __vspltw(H, 0);

    __stvewx(H, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreShortN2
(
    XMSHORTN2* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR N;
    static CONST XMVECTORF32  Scale = {32767.0f, 32767.0f, 32767.0f, 32767.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = XM_PACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMSHORT2, VPACK_32, 3);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreShort2
(
    XMSHORT2* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-32767.0f, -32767.0f, -32767.0f, -32767.0f};
    static CONST XMVECTOR  Max = {32767.0f, 32767.0f, 32767.0f, 32767.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;

    N = __vctsxs(V, 0);
    N = __vpkswss(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUShortN2
(
    XMUSHORTN2* pDestination, 
    FXMVECTOR    V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {65535.0f, 65535.0f, 65535.0f, 65535.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiplyAdd(N, Scale.v, g_XMOneHalf.v);
    N = XMVectorTruncate(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    XMVECTOR Scale, Offset;
    static CONST XMVECTOR C0 = {65535.0f, 0.5f, 0.0f, 0.0f};

    Scale = __vspltw(C0, 0);
    Offset = __vspltw(C0, 1);
    N = __vmaddfp(V, Scale, Offset);
    N = __vctuxs(N, 0);
    N = __vpkuwus(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUShort2
(
    XMUSHORT2* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {65535.0f, 65535.0f, 65535.0f, 65535.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;

    N = __vctuxs(V, 0);
    N = __vpkuwus(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreByteN2
(
    XMBYTEN2* pDestination, 
    FXMVECTOR   V
)
{
    XMVECTOR N;
    XMFLOAT4A tmp;
    static CONST XMVECTORF32  Scale = {127.0f, 127.0f, 127.0f, 127.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    XMStoreFloat4A( &tmp, N );

    pDestination->x = (CHAR)tmp.x;
    pDestination->y = (CHAR)tmp.y;
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreByte2
(
    XMBYTE2* pDestination, 
    FXMVECTOR  V
)
{
    XMVECTOR               N;
    XMFLOAT4A              tmp;
    static CONST XMVECTOR  Min = {-127.0f, -127.0f, -127.0f, -127.0f};
    static CONST XMVECTOR  Max = {127.0f, 127.0f, 127.0f, 127.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);
    N = XMVectorRound(N);

    XMStoreFloat4A( &tmp, N );

    pDestination->x = (CHAR)tmp.x;
    pDestination->y = (CHAR)tmp.y;
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUByteN2
(
    XMUBYTEN2* pDestination, 
    FXMVECTOR    V
)
{
    XMVECTOR               N;
    XMFLOAT4A              tmp;
    static CONST XMVECTORF32  Scale = {255.0f, 255.0f, 255.0f, 255.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiplyAdd(N, Scale.v, g_XMOneHalf.v);
    N = XMVectorTruncate(N);

    XMStoreFloat4A( &tmp, N );

    pDestination->x = (BYTE)tmp.x;
    pDestination->y = (BYTE)tmp.y;
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUByte2
(
    XMUBYTE2* pDestination, 
    FXMVECTOR   V
)
{
    XMVECTOR               N;
    static CONST XMVECTOR  Max = {255.0f, 255.0f, 255.0f, 255.0f};
    XMFLOAT4A              tmp;

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);
    N = XMVectorRound(N);

    XMStoreFloat4A( &tmp, N );

    pDestination->x = (BYTE)tmp.x;
    pDestination->y = (BYTE)tmp.y;
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt3
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];
    pDestination[2] = V.vector4_u32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, Z;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    Y = __vspltw(V, 1);
    Z = __vspltw(V, 2);

    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
    __stvewx(Z, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreSInt3
(
    XMINT3* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = (INT)V.vector4_f32[0];
    pDestination->y = (INT)V.vector4_f32[1];
    pDestination->z = (INT)V.vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, Z, V0;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    V0 = __vctsxs(V, 0);
    X = __vspltw(V0, 0);
    Y = __vspltw(V0, 1);
    Z = __vspltw(V0, 2);

    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
    __stvewx(Z, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUInt3
(
    XMUINT3* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = (UINT)V.vector4_f32[0];
    pDestination->y = (UINT)V.vector4_f32[1];
    pDestination->z = (UINT)V.vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, Z, V0;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    V0 = __vctuxs(V, 0);
    X = __vspltw(V0, 0);
    Y = __vspltw(V0, 1);
    Z = __vspltw(V0, 2);

    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
    __stvewx(Z, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt3A
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];
    pDestination[2] = V.vector4_u32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AlignedV;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    AlignedV = __vsldoi(V, V, 3 << 2);
    __stvrx(AlignedV, pDestination, 12);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3
(
    XMFLOAT3* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];
    pDestination->z = V.vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X, Y, Z;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    X = __vspltw(V, 0);
    Y = __vspltw(V, 1);
    Z = __vspltw(V, 2);

    __stvewx(X, pDestination, 0);
    __stvewx(Y, pDestination, 4);
    __stvewx(Z, pDestination, 8);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3A
(
    XMFLOAT3A*   pDestination, 
    FXMVECTOR     V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];
    pDestination->z = V.vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AlignedV;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    AlignedV = __vsldoi(V, V, 3 << 2);
    __stvrx(AlignedV, pDestination, 12);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUHenDN3
(
    XMUHENDN3* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {2047.0f, 2047.0f, 1023.0f, 0.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = (((UINT)N.vector4_f32[2] & 0x3FF) << 22) |
                      (((UINT)N.vector4_f32[1] & 0x7FF) << 11) |
                      (((UINT)N.vector4_f32[0] & 0x7FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               Scale, OneHalf;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x44FFE000, 0x0007FF0B, 0x3FFFFF16, 0x447FC000}; // (FLOAT)((1 << 11) - 1), ..., (FLOAT)((1 << 10) - 1)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    OneHalf = __vspltisw(1);
    OneHalf = __vcfsx(OneHalf, 1);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0xC);

    Convert = __vmaddfp(V, Scale, OneHalf);
    Select = __vspltisw(8);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);
    Convert = __vctuxs(Convert, 0);
    XY = __vspltw(Select, 1);
    S = __vspltw(Select, 2);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(Z, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUHenD3
(
    XMUHEND3* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {2047.0f, 2047.0f, 1023.0f, 0.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);

    pDestination->v = (((UINT)N.vector4_f32[2] & 0x3FF) << 22) |
                      (((UINT)N.vector4_f32[1] & 0x7FF) << 11) |
                      (((UINT)N.vector4_f32[0] & 0x7FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0xFFC00000, 0x0000000B, 0x00000016, 0x000007FF};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Convert = __vctuxs(V, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 3);
    S = __vspltw(*(XMVECTOR*)&C0, 0);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(XY, Z, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreHenDN3
(
    XMHENDN3* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {1023.0f, 1023.0f, 511.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = (((INT)N.vector4_f32[2] & 0x3FF) << 22) |
                      (((INT)N.vector4_f32[1] & 0x7FF) << 11) |
                      (((INT)N.vector4_f32[0] & 0x7FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               Scale;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x447FC000, 0x0007FF0B, 0x3FFFFF16, 0x43FF8000}; // (FLOAT)((1 << (11 - 1)) - 1), ..., (FLOAT)((1 << (10 - 1)) - 1)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Scale = __vpermwi(*(XMVECTOR*)&C0, 0xC);

    Convert = __vmulfp(V, Scale);
    Select = __vspltisw(8);
    Convert = __vrfin(Convert);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);
    Convert = __vctsxs(Convert, 0);
    XY = __vspltw(Select, 1);
    S = __vspltw(Select, 2);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(Z, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreHenD3
(
    XMHEND3* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-1023.0f, -1023.0f, -511.0f, -1.0f};
    static CONST XMVECTOR  Max = {1023.0f, 1023.0f, 511.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);

    pDestination->v = (((INT)N.vector4_f32[2] & 0x3FF) << 22) |
                      (((INT)N.vector4_f32[1] & 0x7FF) << 11) |
                      (((INT)N.vector4_f32[0] & 0x7FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0xFFC00000, 0x0000000B, 0x00000016, 0x000007FF};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Convert = __vctsxs(V, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 3);
    S = __vspltw(*(XMVECTOR*)&C0, 0);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(XY, Z, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUDHenN3
(
    XMUDHENN3* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {1023.0f, 2047.0f, 2047.0f, 0.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = (((UINT)N.vector4_f32[2] & 0x7FF) << 21) |
                      (((UINT)N.vector4_f32[1] & 0x7FF) << 10) |
                      (((UINT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               Scale, OneHalf;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x44FFE000, 0x0003FF0A, 0x1FFFFF15, 0x447FC000}; // (FLOAT)((1 << 11) - 1), ..., (FLOAT)((1 << 10) - 1)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    OneHalf = __vspltisw(1);
    OneHalf = __vcfsx(OneHalf, 1);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0xC0);

    Convert = __vmaddfp(V, Scale, OneHalf);
    Select = __vspltisw(8);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);
    Convert = __vctuxs(Convert, 0);
    XY = __vspltw(Select, 1);
    S = __vspltw(Select, 2);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(Z, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUDHen3
(
    XMUDHEN3* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {1023.0f, 2047.0f, 2047.0f, 0.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);

    pDestination->v = (((UINT)N.vector4_f32[2] & 0x7FF) << 21) |
                      (((UINT)N.vector4_f32[1] & 0x7FF) << 10) |
                      (((UINT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0xFFE00000, 0x0000000A, 0x00000015, 0x000003FF};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Convert = __vctuxs(V, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 3);
    S = __vspltw(*(XMVECTOR*)&C0, 0);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(XY, Z, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreDHenN3
(
    XMDHENN3* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {511.0f, 1023.0f, 1023.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = (((INT)N.vector4_f32[2] & 0x7FF) << 21) |
                      (((INT)N.vector4_f32[1] & 0x7FF) << 10) |
                      (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               Scale;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x447FC000, 0x0003FF0A, 0x1FFFFF15, 0x43FF8000}; // (FLOAT)((1 << (11 - 1)) - 1), ..., (FLOAT)((1 << (10 - 1)) - 1)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Scale = __vpermwi(*(XMVECTOR*)&C0, 0xC0);

    Convert = __vmulfp(V, Scale);
    Select = __vspltisw(8);
    Convert = __vrfin(Convert);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);
    Convert = __vctsxs(Convert, 0);
    XY = __vspltw(Select, 1);
    S = __vspltw(Select, 2);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(Z, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreDHen3
(
    XMDHEN3* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-511.0f, -1023.0f, -1023.0f, -1.0f};
    static CONST XMVECTOR  Max = {511.0f, 1023.0f, 1023.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);

    pDestination->v = (((INT)N.vector4_f32[2] & 0x7FF) << 21) |
                      (((INT)N.vector4_f32[1] & 0x7FF) << 10) |
                      (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               X, Y, Z, XY;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0xFFE00000, 0x0000000A, 0x00000015, 0x000003FF};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Convert = __vctsxs(V, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 3);
    S = __vspltw(*(XMVECTOR*)&C0, 0);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    XY = __vsel(Y, X, XY);
    S = __vsel(XY, Z, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreU565
(
    XMU565* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR               N;
    static CONST XMVECTORF32  Max = {31.0f, 63.0f, 31.0f, 0.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max.v);
    N = XMVectorRound(N);

    pDestination->v = (((USHORT)N.vector4_f32[2] & 0x1F) << 11) |
                      (((USHORT)N.vector4_f32[1] & 0x3F) << 5) |
                      (((USHORT)N.vector4_f32[0] & 0x1F));
#endif !_XM_SSE_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3PK
(
    XMFLOAT3PK* pDestination,
    FXMVECTOR V
)
{
    _DECLSPEC_ALIGN_16_ UINT IValue[4];
    UINT I, Sign, j;
    UINT Result[3];

    XMASSERT(pDestination);

    XMStoreFloat3A( (XMFLOAT3A*)&IValue, V );

    // X & Y Channels (5-bit exponent, 6-bit mantissa)
    for(j=0; j < 2; ++j)
    {
        Sign = IValue[j] & 0x80000000;
        I = IValue[j] & 0x7FFFFFFF;

        if ((I & 0x7F800000) == 0x7F800000)
        {
            // INF or NAN
            Result[j] = 0x7c0;
            if (( I & 0x7FFFFF ) != 0)
            {
                Result[j] = 0x7c0 | (((I>>17)|(I>11)|(I>>6)|(I))&0x3f);
            }
            else if ( Sign )
            {
                // -INF is clamped to 0 since 3PK is positive only
                Result[j] = 0;
            }
        }
        else if ( Sign )
        {
            // 3PK is positive only, so clamp to zero
            Result[j] = 0;
        }
        else if (I > 0x477E0000U)
        {
            // The number is too large to be represented as a float11, set to max
            Result[j] = 0x7BF;
        }
        else
        {
            if (I < 0x38800000U)
            {
                // The number is too small to be represented as a normalized float11
                // Convert it to a denormalized value.
                UINT Shift = 113U - (I >> 23U);
                I = (0x800000U | (I & 0x7FFFFFU)) >> Shift;
            }
            else
            {
                // Rebias the exponent to represent the value as a normalized float11
                I += 0xC8000000U;
            }
     
            Result[j] = ((I + 0xFFFFU + ((I >> 17U) & 1U)) >> 17U)&0x7ffU;
        }
    }

    // Z Channel (5-bit exponent, 5-bit mantissa)
    Sign = IValue[2] & 0x80000000;
    I = IValue[2] & 0x7FFFFFFF;

    if ((I & 0x7F800000) == 0x7F800000)
    {
        // INF or NAN
        Result[2] = 0x3e0;
        if ( I & 0x7FFFFF )
        {
            Result[2] = 0x3e0 | (((I>>18)|(I>13)|(I>>3)|(I))&0x1f);
        }
        else if ( Sign )
        {
            // -INF is clamped to 0 since 3PK is positive only
            Result[2] = 0;
        }
    }
    else if ( Sign )
    {
        // 3PK is positive only, so clamp to zero
        Result[2] = 0;
    }
    else if (I > 0x477C0000U)
    {
        // The number is too large to be represented as a float10, set to max
        Result[2] = 0x3df;
    }
    else
    {
        if (I < 0x38800000U)
        {
            // The number is too small to be represented as a normalized float10
            // Convert it to a denormalized value.
            UINT Shift = 113U - (I >> 23U);
            I = (0x800000U | (I & 0x7FFFFFU)) >> Shift;
        }
        else
        {
            // Rebias the exponent to represent the value as a normalized float10
            I += 0xC8000000U;
        }
     
        Result[2] = ((I + 0x1FFFFU + ((I >> 18U) & 1U)) >> 18U)&0x3ffU;
    }

    // Pack Result into memory
    pDestination->v = (Result[0] & 0x7ff)
                      | ( (Result[1] & 0x7ff) << 11 )
                      | ( (Result[2] & 0x3ff) << 22 );
}


//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3SE
(
    XMFLOAT3SE* pDestination,
    FXMVECTOR V
)
{
    _DECLSPEC_ALIGN_16_ UINT IValue[4];
    UINT I, Sign, j, T;
    UINT Frac[3];
    UINT Exp[3];
    

    XMASSERT(pDestination);

    XMStoreFloat3A( (XMFLOAT3A*)&IValue, V );

    // X, Y, Z Channels (5-bit exponent, 9-bit mantissa)
    for(j=0; j < 3; ++j)
    {
        Sign = IValue[j] & 0x80000000;
        I = IValue[j] & 0x7FFFFFFF;

        if ((I & 0x7F800000) == 0x7F800000)
        {
            // INF or NAN
            Exp[j] = 0x1f;
            if (( I & 0x7FFFFF ) != 0)
            {
                Frac[j] = ((I>>14)|(I>5)|(I))&0x1ff;
            }
            else if ( Sign )
            {
                // -INF is clamped to 0 since 3SE is positive only
                Exp[j] = Frac[j] = 0;
            }
        }
        else if ( Sign )
        {
            // 3SE is positive only, so clamp to zero
            Exp[j] = Frac[j] = 0;
        }
        else if (I > 0x477FC000U)
        {
            // The number is too large, set to max
            Exp[j] = 0x1e;
            Frac[j] = 0x1ff;
        }
        else
        {
            if (I < 0x38800000U)
            {
                // The number is too small to be represented as a normalized float11
                // Convert it to a denormalized value.
                UINT Shift = 113U - (I >> 23U);
                I = (0x800000U | (I & 0x7FFFFFU)) >> Shift;
            }
            else
            {
                // Rebias the exponent to represent the value as a normalized float11
                I += 0xC8000000U;
            }
     
            T = ((I + 0x1FFFU + ((I >> 14U) & 1U)) >> 14U)&0x3fffU;

            Exp[j] = (T & 0x3E00) >> 9;
            Frac[j] = T & 0x1ff;
        }
    }

    // Adjust to a shared exponent
    T = XMMax( Exp[0], XMMax( Exp[1], Exp[2] ) );

    Frac[0] = Frac[0] >> (T - Exp[0]);
    Frac[1] = Frac[1] >> (T - Exp[1]);
    Frac[2] = Frac[2] >> (T - Exp[2]);

    // Store packed into memory
    pDestination->xm = Frac[0];
    pDestination->ym = Frac[1];
    pDestination->zm = Frac[2];
    pDestination->e = T;
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt4
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];
    pDestination[2] = V.vector4_u32[2];
    pDestination[3] = V.vector4_u32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    
    __stvlx(V, pDestination, 0);
    __stvrx(V, pDestination, 16);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt4A
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];
    pDestination[2] = V.vector4_u32[2];
    pDestination[3] = V.vector4_u32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    __stvx(V, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreSInt4
(
    XMINT4* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->x = (INT)V.vector4_f32[0];
    pDestination->y = (INT)V.vector4_f32[1];
    pDestination->z = (INT)V.vector4_f32[2];
    pDestination->w = (INT)V.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V0;

    XMASSERT(pDestination);

    V0 = __vctsxs(V, 0);
    __stvlx(V0, pDestination, 0);
    __stvrx(V0, pDestination, 16);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUInt4
(
    XMUINT4* pDestination,
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->x = (UINT)V.vector4_f32[0];
    pDestination->y = (UINT)V.vector4_f32[1];
    pDestination->z = (UINT)V.vector4_f32[2];
    pDestination->w = (UINT)V.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V0;

    XMASSERT(pDestination);

    V0 = __vctuxs(V, 0);
    __stvlx(V0, pDestination, 0);
    __stvrx(V0, pDestination, 16);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreInt4NC
(
    UINT*    pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);
    
    pDestination[0] = V.vector4_u32[0];
    pDestination[1] = V.vector4_u32[1];
    pDestination[2] = V.vector4_u32[2];
    pDestination[3] = V.vector4_u32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S;
    XMVECTOR Permute;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Permute = __lvsr(pDestination, 0);                 // Compute the permute control vector for shift right
    S = __vperm(V, V, Permute);                        // Right rotate the vector

    __stvewx(S, pDestination, 0);
    __stvewx(S, pDestination, 4);
    __stvewx(S, pDestination, 8);
    __stvewx(S, pDestination, 12);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4
(
    XMFLOAT4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    
    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];
    pDestination->z = V.vector4_f32[2];
    pDestination->w = V.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    
    __stvlx(V, pDestination, 0);
    __stvrx(V, pDestination, 16);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4A
(
    XMFLOAT4A*   pDestination, 
    FXMVECTOR     V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];
    pDestination->z = V.vector4_f32[2];
    pDestination->w = V.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    __stvx(V, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4NC
(
    XMFLOAT4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);
    
    pDestination->x = V.vector4_f32[0];
    pDestination->y = V.vector4_f32[1];
    pDestination->z = V.vector4_f32[2];
    pDestination->w = V.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S;
    XMVECTOR Permute;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Permute = __lvsr(pDestination, 0);                 // Compute the permute control vector for shift right
    S = __vperm(V, V, Permute);                        // Right rotate the vector

    __stvewx(S, pDestination, 0);
    __stvewx(S, pDestination, 4);
    __stvewx(S, pDestination, 8);
    __stvewx(S, pDestination, 12);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreHalf4
(
    XMHALF4* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_) 

    XMASSERT(pDestination);

    pDestination->x = XMConvertFloatToHalf(V.vector4_f32[0]);
    pDestination->y = XMConvertFloatToHalf(V.vector4_f32[1]);
    pDestination->z = XMConvertFloatToHalf(V.vector4_f32[2]);
    pDestination->w = XMConvertFloatToHalf(V.vector4_f32[3]);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR H, XY, ZW;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(H);
    H = __vpkd3d(H, V, VPACK_FLOAT16_4, VPACK_64LO, 2);
    XY = __vspltw(H, 0);
    ZW = __vspltw(H, 1);

    __stvewx(XY, pDestination, 0);
    __stvewx(ZW, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreShortN4
(
    XMSHORTN4* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {32767.0f, 32767.0f, 32767.0f, 32767.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];
    pDestination->z = (SHORT)N.vector4_f32[2];
    pDestination->w = (SHORT)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N, XY, ZW;
    static CONST XMVECTOR PackScale = XM_PACK_SIGNEDN_SCALE(16, 16, 16, 16);

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMSHORT4, VPACK_64LO, 2);
    XY = __vspltw(N, 0);
    ZW = __vspltw(N, 1);

    __stvewx(XY, pDestination, 0);
    __stvewx(ZW, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreShort4
(
    XMSHORT4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-32767.0f, -32767.0f, -32767.0f, -32767.0f};
    static CONST XMVECTOR  Max = {32767.0f, 32767.0f, 32767.0f, 32767.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];
    pDestination->z = (SHORT)N.vector4_f32[2];
    pDestination->w = (SHORT)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N, XY, ZW;

    N = __vctsxs(V, 0);
    N = __vpkswss(N, N);
    XY = __vspltw(N, 0);
    ZW = __vspltw(N, 1);

    __stvewx(XY, pDestination, 0);
    __stvewx(ZW, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUShortN4
(
    XMUSHORTN4* pDestination, 
    FXMVECTOR    V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {65535.0f, 65535.0f, 65535.0f, 65535.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiplyAdd(N, Scale.v, g_XMOneHalf.v);
    N = XMVectorTruncate(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];
    pDestination->z = (SHORT)N.vector4_f32[2];
    pDestination->w = (SHORT)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N, XY, ZW;
    XMVECTOR Scale, Offset;
    static CONST XMVECTOR C0 = {65535.0f, 0.5f, 0.0f, 0.0f};

    Scale = __vspltw(C0, 0);
    Offset = __vspltw(C0, 1);
    N = __vmaddfp(V, Scale, Offset);
    N = __vctuxs(N, 0);
    N = __vpkuwus(N, N);
    XY = __vspltw(N, 0);
    ZW = __vspltw(N, 1);

    __stvewx(XY, pDestination, 0);
    __stvewx(ZW, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUShort4
(
    XMUSHORT4* pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {65535.0f, 65535.0f, 65535.0f, 65535.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);
    N = XMVectorRound(N);

    pDestination->x = (SHORT)N.vector4_f32[0];
    pDestination->y = (SHORT)N.vector4_f32[1];
    pDestination->z = (SHORT)N.vector4_f32[2];
    pDestination->w = (SHORT)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N, XY, ZW;

    N = __vctuxs(V, 0);
    N = __vpkuwus(N, N);
    XY = __vspltw(N, 0);
    ZW = __vspltw(N, 1);

    __stvewx(XY, pDestination, 0);
    __stvewx(ZW, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreXIcoN4
(
    XMXICON4*  pDestination, 
    FXMVECTOR   V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Min = {-1.0f, -1.0f, -1.0f, 0.0f};
    static CONST XMVECTORF32  Scale = {524287.0f, 524287.0f, 524287.0f, 15.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->v = ((UINT64)N.vector4_f32[3] << 60) |
                       (((INT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((INT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((INT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = {-(FLOAT)((1 << ((20) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << ((20) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << ((20) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << (4)) - 1) / XM_PACK_FACTOR};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMPACKED64, VPACK_64LO, 2);
    N = __vpermwi(N, 0x11);

    __stvewx(N, pDestination, 0);
    __stvewx(N, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreXIco4
(
    XMXICO4*  pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR N;
    static CONST XMVECTORF32 Min = {-524287.0f, -524287.0f, -524287.0f, 0.0f};
    static CONST XMVECTORF32 Max = {524287.0f, 524287.0f, 524287.0f, 15.0f};

    XMASSERT(pDestination);
    N = XMVectorClamp(V, Min.v, Max.v);
    pDestination->v = ((UINT64)N.vector4_f32[3] << 60) |
                       (((INT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((INT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((INT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = {-1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMPACKED64, VPACK_64LO, 2);
    N = __vpermwi(N, 0x11);

    __stvewx(N, pDestination, 0);
    __stvewx(N, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUIcoN4
(
    XMUICON4*  pDestination, 
    FXMVECTOR   V
)
{
    #define XM_URange       ((FLOAT)(1 << 20))
    #define XM_URangeDiv2   ((FLOAT)(1 << 19))
    #define XM_UMaxXYZ      ((FLOAT)((1 << 20) - 1))
    #define XM_UMaxW        ((FLOAT)((1 << 4) - 1))
    #define XM_ScaleXYZ     (-(FLOAT)((1 << 20) - 1) / XM_PACK_FACTOR)
    #define XM_ScaleW       (-(FLOAT)((1 << 4) - 1) / XM_PACK_FACTOR)
    #define XM_Scale        (-1.0f / XM_PACK_FACTOR)
    #define XM_Offset       (3.0f)

#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32 Scale = {1048575.0f, 1048575.0f, 1048575.0f, 15.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiplyAdd(N, Scale.v, g_XMOneHalf.v);

    pDestination->v = ((UINT64)N.vector4_f32[3] << 60) |
                       (((UINT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((UINT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((UINT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Scale1, Scale2, Convert, Mod;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_UMaxXYZ / XM_URange,
                                XM_UMaxW / XM_URange,
                                XM_ScaleXYZ,
                                XM_ScaleW};
    static CONST XMVECTOR C1 = {XM_URangeDiv2 / XM_URange,
                                XM_Offset,
                                XM_URange * XM_Scale,
                                0.0f};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    Scale1 = __vpermwi(C0, 0x1);
    Convert = __vspltw(C1, 0);
    Scale2 = __vpermwi(C0, 0xAB);
    Result = __vspltw(C1, 1);
    Mod = __vpermwi(C1, 0xAB);

    Convert = __vmaddfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Convert = __vrfiz(Convert);
    Result = __vmaddfp(Convert, Mod, Result);
    Result = __vpkd3d(Result, Result, VPACK_NORMPACKED64, VPACK_64LO, 2);
    Result = __vpermwi(Result, 0x11);

    __stvewx(Result, pDestination, 0);
    __stvewx(Result, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_

    #undef XM_URange
    #undef XM_URangeDiv2
    #undef XM_UMaxXYZ
    #undef XM_UMaxW
    #undef XM_ScaleXYZ
    #undef XM_ScaleW
    #undef XM_Scale
    #undef XM_Offset
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUIco4
(
    XMUICO4*  pDestination, 
    FXMVECTOR  V
)
{
    #define XM_Scale        (-1.0f / XM_PACK_FACTOR)
    #define XM_URange       ((FLOAT)(1 << 20))
    #define XM_URangeDiv2   ((FLOAT)(1 << 19))

#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {1048575.0f, 1048575.0f, 1048575.0f, 15.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);
    N = XMVectorRound(N);

    pDestination->v = ((UINT64)N.vector4_f32[3] << 60) |
                       (((UINT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((UINT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((UINT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Scale1, Scale2, Convert, Mod, Zero;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {1.0f / XM_URange,
                                XM_URangeDiv2 / XM_URange,
                                XM_Scale,
                                XM_URange * XM_Scale};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    Result = __vspltisw(3);
    Result = __vcfsx(Result, 0); // Offset
    Scale1 = __vspltw(C0, 0);
    Convert = __vspltw(C0, 1);
    Scale2 = __vspltw(C0, 2);
    Mod = __vspltw(C0, 3);
    Zero = __vspltisw(0);

    Convert = __vmaddfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vsldoi(Mod, Zero, 1 << 2);
    Convert = __vrfiz(Convert);
    Result = __vmaddfp(Convert, Mod, Result);
    Result = __vpkd3d(Result, Result, VPACK_NORMPACKED64, VPACK_64LO, 2);
    Result = __vpermwi(Result, 0x11);

    __stvewx(Result, pDestination, 0);
    __stvewx(Result, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_

    #undef XM_Scale
    #undef XM_URange
    #undef XM_URangeDiv2
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreIcoN4
(
    XMICON4*  pDestination, 
    FXMVECTOR  V
)
{
    #define XM_Scale    (-1.0f / XM_PACK_FACTOR)
    #define XM_URange   ((FLOAT)(1 << 4))
    #define XM_Offset   (3.0f)
    #define XM_UMaxXYZ  ((FLOAT)((1 << (20 - 1)) - 1))
    #define XM_UMaxW    ((FLOAT)((1 << (4 - 1)) - 1))

#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {524287.0f, 524287.0f, 524287.0f, 7.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiplyAdd(N, Scale.v, g_XMNegativeZero.v);
    N = XMVectorRound(N);

    pDestination->v = ((UINT64)N.vector4_f32[3] << 60) |
                       (((UINT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((UINT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((UINT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Scale1, Scale2, Convert, Mod;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {XM_UMaxXYZ / XM_URange,
                                XM_UMaxW / XM_URange,
                                XM_Scale * XM_UMaxXYZ,
                                XM_Scale * XM_UMaxW};

    static CONST XMVECTOR C1 = {3.0f,
                                -XM_URange * XM_Scale + XM_Offset,
                                XM_URange * XM_Scale,
                                1.0f};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    Scale1 = __vpermwi(C0, 0x1);
    Convert = __vspltw(C1, 3);
    Scale2 = __vpermwi(C0, 0xAB);
    Result = __vpermwi(C1, 0x1);

    Convert = __vmaddfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vspltisw(0);
    Mod = __vrlimi(Mod, C1, 0x1, 3);
    Convert = __vrfiz(Convert);
    Result = __vmaddfp(Convert, Mod, Result);

    Result = __vpkd3d(Result, Result, VPACK_NORMPACKED64, VPACK_64LO, 2);
    Result = __vpermwi(Result, 0x11);

    __stvewx(Result, pDestination, 0);
    __stvewx(Result, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_

    #undef XM_Scale
    #undef XM_URange
    #undef XM_Offset
    #undef XM_UMaxXYZ
    #undef XM_UMaxW
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreIco4
(
    XMICO4*  pDestination, 
    FXMVECTOR V
)
{
    #define XM_Scale    (-1.0f / XM_PACK_FACTOR)
    #define XM_URange   ((FLOAT)(1 << 4))
    #define XM_Offset   (3.0f)

#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-524287.0f, -524287.0f, -524287.0f, -7.0f};
    static CONST XMVECTOR  Max = {524287.0f, 524287.0f, 524287.0f, 7.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);
    N = XMVectorRound(N);

    pDestination->v = ((INT64)N.vector4_f32[3] << 60) |
                       (((INT64)N.vector4_f32[2] & 0xFFFFF) << 40) |
                       (((INT64)N.vector4_f32[1] & 0xFFFFF) << 20) |
                       (((INT64)N.vector4_f32[0] & 0xFFFFF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Scale1, Scale2, Convert, Mod;
    XMVECTOR Result;

    static CONST XMVECTOR C0 = {1.0f / XM_URange,
                                XM_Scale,
                                -XM_URange * XM_Scale + XM_Offset,
                                XM_URange * XM_Scale};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 7) == 0);

    XMDUMMY_INITIALIZE_VECTOR(Convert);

    Convert = __vupkd3d(Convert, VPACK_NORMSHORT2);
    Result = __vspltisw(3);
    Convert = __vspltw(Convert, 3);
    Result = __vcfsx(Result, 0);
    Scale1 = __vspltw(C0, 0);
    Scale2 = __vspltw(C0, 1);
    Result = __vrlimi(Result, C0, 0x1, 3);

    Convert = __vmaddfp(V, Scale1, Convert);
    Result = __vnmsubfp(V, Scale2, Result);
    Mod = __vspltisw(0);
    Mod = __vrlimi(Mod, C0, 0x1, 0);
    Convert = __vrfiz(Convert);
    Result = __vmaddfp(Convert, Mod, Result);

    Result = __vpkd3d(Result, Result, VPACK_NORMPACKED64, VPACK_64LO, 2);
    Result = __vpermwi(Result, 0x11);

    __stvewx(Result, pDestination, 0);
    __stvewx(Result, pDestination, 4);
#endif // _XM_VMX128_INTRINSICS_

    #undef XM_Scale
    #undef XM_URange
    #undef XM_Offset
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreXDecN4
(
    XMXDECN4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Min = {-1.0f, -1.0f, -1.0f, 0.0f};
    static CONST XMVECTORF32  Scale = {511.0f, 511.0f, 511.0f, 3.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->v = ((UINT)N.vector4_f32[3] << 30) |
                       (((INT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((INT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = {-(FLOAT)((1 << ((10) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << ((10) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << ((10) - 1)) - 1) / XM_PACK_FACTOR,
                                       -(FLOAT)((1 << (2)) - 1) / XM_PACK_FACTOR};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMPACKED32, VPACK_32, 3);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreXDec4
(
    XMXDEC4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-511.0f, -511.0f, -511.0f, 0.0f};
    static CONST XMVECTOR  Max = {511.0f, 511.0f, 511.0f, 3.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);

    pDestination->v = ((UINT)N.vector4_f32[3] << 30) |
                       (((INT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((INT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = {-1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR,
                                       -1.0f / XM_PACK_FACTOR};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_NORMPACKED32, VPACK_32, 3);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUDecN4
(
    XMUDECN4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {1023.0f, 1023.0f, 1023.0f, 3.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = ((UINT)N.vector4_f32[3] << 30) |
                       (((UINT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((UINT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((UINT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Scale, OneHalf;
    XMVECTOR               X, Y, Z, W, XY, ZW;
    XMVECTOR               S;

    static CONST __vector4i C0 = {0x447FC000, 0x40400000, 0x3FF, 0xFFFFF}; // (FLOAT)((1 << 10) - 1), (FLOAT)((1 << 2) - 1), ...
    static CONST __vector4i C1 = {0x3FF00000, 0x0000000A, 0x00000014, 0x0000001E};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    OneHalf = __vspltisw(1);
    OneHalf = __vcfsx(OneHalf, 1);
    Scale = __vpermwi(*(XMVECTOR*)&C0, 0x1);

    Convert = __vmaddfp(V, Scale, OneHalf);
    Convert = __vctuxs(Convert, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 2);
    ZW = __vspltw(*(XMVECTOR*)&C1, 0);
    S = __vspltw(*(XMVECTOR*)&C0, 3);
    Convert = __vslw(Convert, *(XMVECTOR*)&C1);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    W = __vspltw(Convert, 3);
    XY = __vsel(Y, X, XY);
    ZW = __vsel(W, Z, ZW);
    S = __vsel(ZW, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUDec4
(
    XMUDEC4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {1023.0f, 1023.0f, 1023.0f, 3.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);

    pDestination->v = ((UINT)N.vector4_f32[3] << 30) |
                       (((UINT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((UINT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((UINT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               X, Y, Z, W, XY, ZW;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x3FF00000, 0x0000000A, 0x003FF014, 0xFFFFF01E};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Select = __vspltisw(12);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);

    Convert = __vctuxs(V, 0);
    ZW = __vspltw(*(XMVECTOR*)&C0, 0);
    XY = __vspltw(Select, 2);
    S = __vspltw(Select, 3);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    Z = __vspltw(Convert, 2);
    W = __vspltw(Convert, 3);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    ZW = __vsel(W, Z, ZW);
    XY = __vsel(Y, X, XY);
    S = __vsel(ZW, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreDecN4
(
    XMDECN4* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {511.0f, 511.0f, 511.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(N, Scale.v);

    pDestination->v = ((INT)N.vector4_f32[3] << 30) |
                       (((INT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((INT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Scale;
    XMVECTOR               X, Y, Z, W, XY, ZW;
    XMVECTOR               S;

    static CONST __vector4i C0 = {0x43FF8000, 0x3F800000, 0x3FF, 0xFFFFF}; // (FLOAT)((1 << (10 - 1)) - 1), (FLOAT)((1 << (2 - 1)) - 1), ...
    static CONST __vector4i C1 = {0x3FF00000, 0x0000000A, 0x00000014, 0x0000001E};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Scale = __vpermwi(*(XMVECTOR*)&C0, 0x1);

    Convert = __vmulfp(V, Scale);
    Convert = __vrfin(Convert);
    Convert = __vctsxs(Convert, 0);
    XY = __vspltw(*(XMVECTOR*)&C0, 2);
    ZW = __vspltw(*(XMVECTOR*)&C1, 0);
    S = __vspltw(*(XMVECTOR*)&C0, 3);
    Convert = __vslw(Convert, *(XMVECTOR*)&C1);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    Z = __vspltw(Convert, 2);
    W = __vspltw(Convert, 3);
    XY = __vsel(Y, X, XY);
    ZW = __vsel(W, Z, ZW);
    S = __vsel(ZW, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreDec4
(
    XMDEC4*  pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-511.0f, -511.0f, -511.0f, -1.0f};
    static CONST XMVECTOR  Max = {511.0f, 511.0f, 511.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);

    pDestination->v = ((INT)N.vector4_f32[3] << 30) |
                       (((INT)N.vector4_f32[2] & 0x3FF) << 20) |
                       (((INT)N.vector4_f32[1] & 0x3FF) << 10) |
                       (((INT)N.vector4_f32[0] & 0x3FF));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               Convert;
    XMVECTOR               Select;
    XMVECTOR               X, Y, Z, W, XY, ZW;
    XMVECTOR               S;
    static CONST __vector4i C0 = {0x3FF00000, 0x0000000A, 0x003FF014, 0xFFFFF01E};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Select = __vspltisw(12);
    Select = __vsrw(*(XMVECTOR*)&C0, Select);

    Convert = __vctsxs(V, 0);
    ZW = __vspltw(*(XMVECTOR*)&C0, 0);
    XY = __vspltw(Select, 2);
    S = __vspltw(Select, 3);
    Convert = __vslw(Convert, *(XMVECTOR*)&C0);
    Z = __vspltw(Convert, 2);
    W = __vspltw(Convert, 3);
    X = __vspltw(Convert, 0);
    Y = __vspltw(Convert, 1);
    ZW = __vsel(W, Z, ZW);
    XY = __vsel(Y, X, XY);
    S = __vsel(ZW, XY, S);

    __stvewx(S, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUByteN4
(
    XMUBYTEN4* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {255.0f, 255.0f, 255.0f, 255.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->x = (BYTE)N.vector4_f32[0];
    pDestination->y = (BYTE)N.vector4_f32[1];
    pDestination->z = (BYTE)N.vector4_f32[2];
    pDestination->w = (BYTE)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S, N;
    static CONST XMVECTOR PackScale = XM_PACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    S = __vpermwi(V, 0x6C); // 1, 2, 3, 0
    N = __vnmsubfp(S, PackScale, N);
    N = __vpkd3d(N, N, VPACK_D3DCOLOR, VPACK_32, 3);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUByte4
(
    XMUBYTE4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Max = {255.0f, 255.0f, 255.0f, 255.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max);
    N = XMVectorRound(N);

    pDestination->x = (BYTE)N.vector4_f32[0];
    pDestination->y = (BYTE)N.vector4_f32[1];
    pDestination->z = (BYTE)N.vector4_f32[2];
    pDestination->w = (BYTE)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;

//    N = __vpermwi(V, 0xE4); // 3, 2, 1, 0
    N = __vctuxs(V, 0);
    N = __vpkuwus(N, N);
    N = __vpkuhus(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreByteN4
(
    XMBYTEN4* pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {127.0f, 127.0f, 127.0f, 127.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, g_XMNegativeOne.v, g_XMOne.v);
    N = XMVectorMultiply(V, Scale.v);
    N = XMVectorRound(N);

    pDestination->x = (CHAR)N.vector4_f32[0];
    pDestination->y = (CHAR)N.vector4_f32[1];
    pDestination->z = (CHAR)N.vector4_f32[2];
    pDestination->w = (CHAR)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTORF32 Scale = {127.0f, 127.0f, 127.0f, 127.0f};

//    N = __vpermwi(V, 0xE4); // 3, 2, 1, 0
    N = __vmulfp(V, Scale.v);
    N = __vrfin(N);
    N = __vctsxs(N, 0);
    N = __vpkswss(N, N);
    N = __vpkshss(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreByte4
(
    XMBYTE4*  pDestination, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTOR  Min = {-127.0f, -127.0f, -127.0f, -127.0f};
    static CONST XMVECTOR  Max = {127.0f, 127.0f, 127.0f, 127.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, Min, Max);
    N = XMVectorRound(N);

    pDestination->x = (CHAR)N.vector4_f32[0];
    pDestination->y = (CHAR)N.vector4_f32[1];
    pDestination->z = (CHAR)N.vector4_f32[2];
    pDestination->w = (CHAR)N.vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;

//    N = __vpermwi(V, 0xE4); // 3, 2, 1, 0
    N = __vctsxs(V, 0);
    N = __vpkswss(N, N);
    N = __vpkshss(N, N);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreUNibble4
(
     XMUNIBBLE4* pDestination,
     FXMVECTOR V
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR               N;
    static CONST XMVECTORF32  Max = {15.0f,15.0f,15.0f,15.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max.v);
    N = XMVectorRound(N);

    pDestination->v = (((USHORT)N.vector4_f32[3] & 0xF) << 12) |
                      (((USHORT)N.vector4_f32[2] & 0xF) << 8) |
                      (((USHORT)N.vector4_f32[1] & 0xF) << 4) |
                      (((USHORT)N.vector4_f32[0] & 0xF));
#endif !_XM_SSE_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreU555(
     XMU555* pDestination,
     FXMVECTOR V
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR               N;
    static CONST XMVECTORF32  Max = {31.0f, 31.0f, 31.0f, 1.0f};

    XMASSERT(pDestination);

    N = XMVectorClamp(V, XMVectorZero(), Max.v);
    N = XMVectorRound(N);

    pDestination->v = ((N.vector4_f32[3] > 0.f) ? 0x8000 : 0) |
                      (((USHORT)N.vector4_f32[2] & 0x1F) << 10) |
                      (((USHORT)N.vector4_f32[1] & 0x1F) << 5) |
                      (((USHORT)N.vector4_f32[0] & 0x1F));
#endif !_XM_SSE_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreColor
(
    XMCOLOR* pDestination, 
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               N;
    static CONST XMVECTORF32  Scale = {255.0f, 255.0f, 255.0f, 255.0f};

    XMASSERT(pDestination);

    N = XMVectorSaturate(V);
    N = XMVectorMultiply(N, Scale.v);
    N = XMVectorRound(N);

    pDestination->c = ((UINT)N.vector4_f32[3] << 24) |
                      ((UINT)N.vector4_f32[0] << 16) |
                      ((UINT)N.vector4_f32[1] <<  8) |
                      ((UINT)N.vector4_f32[2]);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR N;
    static CONST XMVECTOR PackScale = XM_PACK_UNSIGNEDN_SCALE(8, 8, 8, 8);

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    N = XM_PACK_OFFSET;
    N = __vnmsubfp(V, PackScale, N);
    N = __vpkd3d(N, N, VPACK_D3DCOLOR, VPACK_32, 3);
    N = __vspltw(N, 0);

    __stvewx(N, pDestination, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3x3
(
    XMFLOAT3X3*	pDestination, 
    CXMMATRIX	M
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS) || defined(_XM_SSE_INTRINSICS_)

    XMStoreFloat3x3NC(pDestination, M);

#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S0, S1, S2;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    S0 = M.r[0];
    S1 = __vsldoi(M.r[1], M.r[1], 1 << 2);
    S2 = __vspltw(M.r[2], 2);
    S0 = __vrlimi(S0, M.r[1], 0x1, 1);
    S1 = __vrlimi(S1, M.r[2], 0x3, 2);

    __stvlx(S0, pDestination, 0);
    __stvrx(S0, pDestination, 16);
    __stvlx(S1, pDestination, 16);
    __stvrx(S1, pDestination, 32);
    __stvewx(S2, pDestination, 32);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat3x3NC
(
    XMFLOAT3X3* pDestination, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->m[0][0] = M.r[0].vector4_f32[0];
    pDestination->m[0][1] = M.r[0].vector4_f32[1];
    pDestination->m[0][2] = M.r[0].vector4_f32[2];

    pDestination->m[1][0] = M.r[1].vector4_f32[0];
    pDestination->m[1][1] = M.r[1].vector4_f32[1];
    pDestination->m[1][2] = M.r[1].vector4_f32[2];

    pDestination->m[2][0] = M.r[2].vector4_f32[0];
    pDestination->m[2][1] = M.r[2].vector4_f32[1];
    pDestination->m[2][2] = M.r[2].vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Permute;
    XMVECTOR S0, S1, S2;
    static CONST __vector4i Permute0X0Y0Z1X = {XM_PERMUTE_0X, XM_PERMUTE_0Y, XM_PERMUTE_0Z, XM_PERMUTE_1X};
    static CONST __vector4i Permute0Y0Z1X1Y = {XM_PERMUTE_0Y, XM_PERMUTE_0Z, XM_PERMUTE_1X, XM_PERMUTE_1Y};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Permute = __lvsr(pDestination, 0);

    S0 = __vperm(M.r[0], M.r[1], *(XMVECTOR*)&Permute0X0Y0Z1X);
    S1 = __vperm(M.r[1], M.r[2], *(XMVECTOR*)&Permute0Y0Z1X1Y);
    S2 = __vspltw(M.r[2], 2);

    S0 = __vperm(S0, S0, Permute);
    S1 = __vperm(S1, S1, Permute);

    __stvewx(S0, pDestination, 0);
    __stvewx(S0, pDestination, 4);
    __stvewx(S0, pDestination, 8);
    __stvewx(S0, pDestination, 12);

    __stvewx(S1, pDestination, 16);
    __stvewx(S1, pDestination, 20);
    __stvewx(S1, pDestination, 24);
    __stvewx(S1, pDestination, 28);

    __stvewx(S2, pDestination, 32);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x3
(
    XMFLOAT4X3* pDestination, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS) || defined(_XM_SSE_INTRINSICS_)

    XMStoreFloat4x3NC(pDestination, M);

#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S0, S1, S2;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    S0 = M.r[0];
    S1 = __vsldoi(M.r[1], M.r[1], 1 << 2);
    S2 = __vsldoi(M.r[2], M.r[2], 2 << 2);
    S0 = __vrlimi(S0, M.r[1], 0x1, 1);
    S1 = __vrlimi(S1, M.r[2], 0x3, 2);
    S2 = __vrlimi(S2, M.r[3], 0x7, 3);

    __stvlx(S0, pDestination, 0);
    __stvrx(S0, pDestination, 16);
    __stvlx(S1, pDestination, 16);
    __stvrx(S1, pDestination, 32);
    __stvlx(S2, pDestination, 32);
    __stvrx(S2, pDestination, 48);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x3A
(
    XMFLOAT4X3A*	pDestination, 
    CXMMATRIX		M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination->m[0][0] = M.r[0].vector4_f32[0];
    pDestination->m[0][1] = M.r[0].vector4_f32[1];
    pDestination->m[0][2] = M.r[0].vector4_f32[2];

    pDestination->m[1][0] = M.r[1].vector4_f32[0];
    pDestination->m[1][1] = M.r[1].vector4_f32[1];
    pDestination->m[1][2] = M.r[1].vector4_f32[2];

    pDestination->m[2][0] = M.r[2].vector4_f32[0];
    pDestination->m[2][1] = M.r[2].vector4_f32[1];
    pDestination->m[2][2] = M.r[2].vector4_f32[2];

    pDestination->m[3][0] = M.r[3].vector4_f32[0];
    pDestination->m[3][1] = M.r[3].vector4_f32[1];
    pDestination->m[3][2] = M.r[3].vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S0, S1, S2;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    S0 = M.r[0];
    S1 = __vsldoi(M.r[1], M.r[1], 1 << 2);
    S2 = __vsldoi(M.r[2], M.r[2], 2 << 2);
    S0 = __vrlimi(S0, M.r[1], 0x1, 1);
    S1 = __vrlimi(S1, M.r[2], 0x3, 2);
    S2 = __vrlimi(S2, M.r[3], 0x7, 3);

    __stvx(S0, pDestination, 0);
    __stvx(S1, pDestination, 16);
    __stvx(S2, pDestination, 32);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x3NC
(
    XMFLOAT4X3* pDestination, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->m[0][0] = M.r[0].vector4_f32[0];
    pDestination->m[0][1] = M.r[0].vector4_f32[1];
    pDestination->m[0][2] = M.r[0].vector4_f32[2];

    pDestination->m[1][0] = M.r[1].vector4_f32[0];
    pDestination->m[1][1] = M.r[1].vector4_f32[1];
    pDestination->m[1][2] = M.r[1].vector4_f32[2];

    pDestination->m[2][0] = M.r[2].vector4_f32[0];
    pDestination->m[2][1] = M.r[2].vector4_f32[1];
    pDestination->m[2][2] = M.r[2].vector4_f32[2];

    pDestination->m[3][0] = M.r[3].vector4_f32[0];
    pDestination->m[3][1] = M.r[3].vector4_f32[1];
    pDestination->m[3][2] = M.r[3].vector4_f32[2];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Permute;
    XMVECTOR S0, S1, S2;
    static CONST __vector4i Permute0X0Y0Z1X = {XM_PERMUTE_0X, XM_PERMUTE_0Y, XM_PERMUTE_0Z, XM_PERMUTE_1X};
    static CONST __vector4i Permute0Y0Z1X1Y = {XM_PERMUTE_0Y, XM_PERMUTE_0Z, XM_PERMUTE_1X, XM_PERMUTE_1Y};
    static CONST __vector4i Permute0Z1X1Y1Z = {XM_PERMUTE_0Z, XM_PERMUTE_1X, XM_PERMUTE_1Y, XM_PERMUTE_1Z};

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Permute = __lvsr(pDestination, 0);

    S0 = __vperm(M.r[0], M.r[1], *(XMVECTOR*)&Permute0X0Y0Z1X);
    S1 = __vperm(M.r[1], M.r[2], *(XMVECTOR*)&Permute0Y0Z1X1Y);
    S2 = __vperm(M.r[2], M.r[3], *(XMVECTOR*)&Permute0Z1X1Y1Z);

    S0 = __vperm(S0, S0, Permute);
    S1 = __vperm(S1, S1, Permute);
    S2 = __vperm(S2, S2, Permute);

    __stvewx(S0, pDestination, 0);
    __stvewx(S0, pDestination, 4);
    __stvewx(S0, pDestination, 8);
    __stvewx(S0, pDestination, 12);

    __stvewx(S1, pDestination, 16);
    __stvewx(S1, pDestination, 20);
    __stvewx(S1, pDestination, 24);
    __stvewx(S1, pDestination, 28);

    __stvewx(S2, pDestination, 32);
    __stvewx(S2, pDestination, 36);
    __stvewx(S2, pDestination, 40);
    __stvewx(S2, pDestination, 44);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x4
(
    XMFLOAT4X4* pDestination, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS)

    XMStoreFloat4x4NC(pDestination, M);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    __stvlx(M.r[0], pDestination, 0);
    __stvrx(M.r[0], pDestination, 16);
    __stvlx(M.r[1], pDestination, 16);
    __stvrx(M.r[1], pDestination, 32);
    __stvlx(M.r[2], pDestination, 32);
    __stvrx(M.r[2], pDestination, 48);
    __stvlx(M.r[3], pDestination, 48);
    __stvrx(M.r[3], pDestination, 64);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x4A
(
    XMFLOAT4X4A*	pDestination, 
    CXMMATRIX		M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    pDestination->m[0][0] = M.r[0].vector4_f32[0];
    pDestination->m[0][1] = M.r[0].vector4_f32[1];
    pDestination->m[0][2] = M.r[0].vector4_f32[2];
    pDestination->m[0][3] = M.r[0].vector4_f32[3];

    pDestination->m[1][0] = M.r[1].vector4_f32[0];
    pDestination->m[1][1] = M.r[1].vector4_f32[1];
    pDestination->m[1][2] = M.r[1].vector4_f32[2];
    pDestination->m[1][3] = M.r[1].vector4_f32[3];

    pDestination->m[2][0] = M.r[2].vector4_f32[0];
    pDestination->m[2][1] = M.r[2].vector4_f32[1];
    pDestination->m[2][2] = M.r[2].vector4_f32[2];
    pDestination->m[2][3] = M.r[2].vector4_f32[3];

    pDestination->m[3][0] = M.r[3].vector4_f32[0];
    pDestination->m[3][1] = M.r[3].vector4_f32[1];
    pDestination->m[3][2] = M.r[3].vector4_f32[2];
    pDestination->m[3][3] = M.r[3].vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 0xF) == 0);

    __stvx(M.r[0], pDestination, 0);
    __stvx(M.r[1], pDestination, 16);
    __stvx(M.r[2], pDestination, 32);
    __stvx(M.r[3], pDestination, 48);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMStoreFloat4x4NC
(
    XMFLOAT4X4* pDestination, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMASSERT(pDestination);

    pDestination->m[0][0] = M.r[0].vector4_f32[0];
    pDestination->m[0][1] = M.r[0].vector4_f32[1];
    pDestination->m[0][2] = M.r[0].vector4_f32[2];
    pDestination->m[0][3] = M.r[0].vector4_f32[3];

    pDestination->m[1][0] = M.r[1].vector4_f32[0];
    pDestination->m[1][1] = M.r[1].vector4_f32[1];
    pDestination->m[1][2] = M.r[1].vector4_f32[2];
    pDestination->m[1][3] = M.r[1].vector4_f32[3];

    pDestination->m[2][0] = M.r[2].vector4_f32[0];
    pDestination->m[2][1] = M.r[2].vector4_f32[1];
    pDestination->m[2][2] = M.r[2].vector4_f32[2];
    pDestination->m[2][3] = M.r[2].vector4_f32[3];

    pDestination->m[3][0] = M.r[3].vector4_f32[0];
    pDestination->m[3][1] = M.r[3].vector4_f32[1];
    pDestination->m[3][2] = M.r[3].vector4_f32[2];
    pDestination->m[3][3] = M.r[3].vector4_f32[3];

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR S0, S1, S2, S3;
    XMVECTOR Permute;

    XMASSERT(pDestination);
    XMASSERT(((UINT_PTR)pDestination & 3) == 0);

    Permute = __lvsr(pDestination, 0);                  // Compute the permute control vector for shift right

    S0 = __vperm(M.r[0], M.r[0], Permute);
    S1 = __vperm(M.r[1], M.r[1], Permute);
    S2 = __vperm(M.r[2], M.r[2], Permute);
    S3 = __vperm(M.r[3], M.r[3], Permute);

    __stvewx(S0, pDestination, 0);
    __stvewx(S0, pDestination, 4);
    __stvewx(S0, pDestination, 8);
    __stvewx(S0, pDestination, 12);

    __stvewx(S1, pDestination, 16);
    __stvewx(S1, pDestination, 20);
    __stvewx(S1, pDestination, 24);
    __stvewx(S1, pDestination, 28);

    __stvewx(S2, pDestination, 32);
    __stvewx(S2, pDestination, 36);
    __stvewx(S2, pDestination, 40);
    __stvewx(S2, pDestination, 44);

    __stvewx(S3, pDestination, 48);
    __stvewx(S3, pDestination, 52);
    __stvewx(S3, pDestination, 56);
    __stvewx(S3, pDestination, 60);
#endif // _XM_VMX128_INTRINSICS_
}

#endif // __XNAMATHCONVERT_INL__

