/************************************************************************
*                                                                       *
* xnamathvector.inl -- SIMD C++ Math library for Windows and Xbox 360   *
*                      Vector functions                                 *
*                                                                       *
* Copyright (c) Microsoft Corp. All rights reserved.                    *
*                                                                       *
************************************************************************/

#if defined(_MSC_VER) && (_MSC_VER > 1000)
#pragma once
#endif

#ifndef __XNAMATHVECTOR_INL__
#define __XNAMATHVECTOR_INL__

#if defined(_XM_NO_INTRINSICS_)
#define XMISNAN(x)  ((*(UINT*)&(x) & 0x7F800000) == 0x7F800000 && (*(UINT*)&(x) & 0x7FFFFF) != 0)
#define XMISINF(x)  ((*(UINT*)&(x) & 0x7FFFFFFF) == 0x7F800000)
#endif

/****************************************************************************
 *
 * General Vector
 *
 ****************************************************************************/

//------------------------------------------------------------------------------
// Assignment operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Return a vector with all elements equaling zero
XMFINLINE XMVECTOR XMVectorZero()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {0.0f,0.0f,0.0f,0.0f};
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltisw(0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with four floating point values
XMFINLINE XMVECTOR XMVectorSet
(
    FLOAT x, 
    FLOAT y, 
    FLOAT z, 
    FLOAT w
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTORF32 vResult = {x,y,z,w};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // There is no compiler intrinsic and no way for direct
    // fp register to vmx register transfer, so use memory to
    // move from the FPU to VMX unit.
    XMVECTORF32 vTemp;
    vTemp.f[0] = x;
    vTemp.f[1] = y;
    vTemp.f[2] = z;
    vTemp.f[3] = w;
    return vTemp.v;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with four integer values
XMFINLINE XMVECTOR XMVectorSetInt
(
    UINT x, 
    UINT y, 
    UINT z, 
    UINT w
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTORU32 vResult = {x,y,z,w};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // There is no compiler intrinsic and no way for direct
    // integer register to vmx register transfer, so use memory to
    // move from the integer unit to VMX unit.
    XMVECTORU32 vResult = {x,y,z,w};
    return vResult.v;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with a replicated floating point value
XMFINLINE XMVECTOR XMVectorReplicate
(
    FLOAT Value
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTORF32 vResult = {Value,Value,Value,Value};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __lvlx(&Value,0);
    vResult = __vspltw(vResult,0);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with a replicated floating point value passed by pointer
XMFINLINE XMVECTOR XMVectorReplicatePtr
(
    CONST FLOAT *pValue
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    FLOAT Value = pValue[0];
    XMVECTORF32 vResult = {Value,Value,Value,Value};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __lvlx(pValue,0);
    vResult = __vspltw(vResult,0);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with a replicated integer value
XMFINLINE XMVECTOR XMVectorReplicateInt
(
    UINT Value
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTORU32 vResult = {Value,Value,Value,Value};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __lvlx(&Value,0);
    vResult = __vspltw(vResult,0);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with a replicated integer value passed by pointer
XMFINLINE XMVECTOR XMVectorReplicateIntPtr
(
    CONST UINT *pValue
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    UINT Value = pValue[0];
    XMVECTORU32 vResult = {Value,Value,Value,Value};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __lvlx(pValue,0);
    vResult = __vspltw(vResult,0);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with all bits set (true mask)
XMFINLINE XMVECTOR XMVectorTrueInt()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTORU32 vResult = {0xFFFFFFFFU,0xFFFFFFFFU,0xFFFFFFFFU,0xFFFFFFFFU};
    return vResult.v;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltisw(-1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Initialize a vector with all bits clear (false mask)
XMFINLINE XMVECTOR XMVectorFalseInt()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {0.0f,0.0f,0.0f,0.0f};
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltisw(0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Replicate the x component of the vector
XMFINLINE XMVECTOR XMVectorSplatX
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_f32[0] = 
    vResult.vector4_f32[1] = 
    vResult.vector4_f32[2] = 
    vResult.vector4_f32[3] = V.vector4_f32[0];
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltw(V, 0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Replicate the y component of the vector
XMFINLINE XMVECTOR XMVectorSplatY
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_f32[0] = 
    vResult.vector4_f32[1] = 
    vResult.vector4_f32[2] = 
    vResult.vector4_f32[3] = V.vector4_f32[1];
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltw(V, 1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Replicate the z component of the vector
XMFINLINE XMVECTOR XMVectorSplatZ
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_f32[0] = 
    vResult.vector4_f32[1] = 
    vResult.vector4_f32[2] = 
    vResult.vector4_f32[3] = V.vector4_f32[2];
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltw(V, 2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Replicate the w component of the vector
XMFINLINE XMVECTOR XMVectorSplatW
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_f32[0] = 
    vResult.vector4_f32[1] = 
    vResult.vector4_f32[2] = 
    vResult.vector4_f32[3] = V.vector4_f32[3];
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vspltw(V, 3);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a vector of 1.0f,1.0f,1.0f,1.0f
XMFINLINE XMVECTOR XMVectorSplatOne()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_f32[0] = 
    vResult.vector4_f32[1] = 
    vResult.vector4_f32[2] = 
    vResult.vector4_f32[3] = 1.0f;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else //  _XM_VMX128_INTRINSICS_
    // Leverage the unpack instruction to generate a 1.0 with half the
    // latency required by XMVectorSplatConstant.
    XMVECTOR vResult = __vspltisw(0);
    // vResult = 1.0f,1.0f,1.0f,1.0f
    vResult = __vupkd3d(vResult, VPACK_D3DCOLOR);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a vector of INF,INF,INF,INF
XMFINLINE XMVECTOR XMVectorSplatInfinity()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_u32[0] = 
    vResult.vector4_u32[1] = 
    vResult.vector4_u32[2] = 
    vResult.vector4_u32[3] = 0x7F800000;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __vspltisw(-1);
    XMVECTOR vShiftL = __vspltisw(-8); // Shift 24
    XMVECTOR vShiftR = __vspltisw(1);
    // 0xFF800000 = -1<<24
    vResult = __vslw(vResult,vShiftL);
    // 0x7F800000 = 0xFF800000>>1
    vResult = __vsrw(vResult,vShiftR);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a vector of Q_NAN,Q_NAN,Q_NAN,Q_NAN
XMFINLINE XMVECTOR XMVectorSplatQNaN()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_u32[0] = 
    vResult.vector4_u32[1] = 
    vResult.vector4_u32[2] = 
    vResult.vector4_u32[3] = 0x7FC00000;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __vspltisw(-1);
    XMVECTOR vShiftL = __vspltisw(-9); // Shift 23
    XMVECTOR vShiftR = __vspltisw(1);
    // 0xFF800000 = -1<<23
    vResult = __vslw(vResult,vShiftL);
    // 0x7FC00000 = 0xFF800000>>1
    vResult = __vsrw(vResult,vShiftR);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a vector of 1.192092896e-7f,1.192092896e-7f,1.192092896e-7f,1.192092896e-7f
XMFINLINE XMVECTOR XMVectorSplatEpsilon()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_u32[0] = 
    vResult.vector4_u32[1] = 
    vResult.vector4_u32[2] = 
    vResult.vector4_u32[3] = 0x34000000;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult = __vspltisw(13);
    XMVECTOR vShiftL = __vspltisw(-6); // Shift 26
    // 0x34000000 = 0x0D<<26
    vResult = __vslw(vResult,vShiftL);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a vector of -0.0f (0x80000000),-0.0f,-0.0f,-0.0f
XMFINLINE XMVECTOR XMVectorSplatSignMask()
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult;
    vResult.vector4_u32[0] = 
    vResult.vector4_u32[1] = 
    vResult.vector4_u32[2] = 
    vResult.vector4_u32[3] = 0x80000000U;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vResult;
    // 0xFFFFFFFF
    vResult = __vspltisw(-1);
    // 0x80000000 = 0xFFFFFFFF<<31
    vResult = __vslw(vResult,vResult);
    return vResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return a floating point value via an index. This is not a recommended
// function to use due to performance loss.
XMFINLINE FLOAT XMVectorGetByIndex(FXMVECTOR V,UINT i)
{
    XMASSERT( i <= 3 );
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_f32[i];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_f32[i];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return the X component in an FPU register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE FLOAT XMVectorGetX(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_f32[0];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_f32[0];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the Y component in an FPU register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE FLOAT XMVectorGetY(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_f32[1];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_f32[1];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the Z component in an FPU register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE FLOAT XMVectorGetZ(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_f32[2];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_f32[2];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the W component in an FPU register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE FLOAT XMVectorGetW(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_f32[3];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_f32[3];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Store a component indexed by i into a 32 bit float location in memory.
// This causes Load/Hit/Store on VMX targets
XMFINLINE VOID XMVectorGetByIndexPtr(FLOAT *f,FXMVECTOR V,UINT i)
{
    XMASSERT( f != 0 );
    XMASSERT( i <  4 );
#if defined(_XM_NO_INTRINSICS_)
    *f = V.vector4_f32[i];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    *f = V.vector4_f32[i];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Store the X component into a 32 bit float location in memory.
XMFINLINE VOID XMVectorGetXPtr(FLOAT *x,FXMVECTOR V)
{
    XMASSERT( x != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *x = V.vector4_f32[0];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,0),x,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the Y component into a 32 bit float location in memory.
XMFINLINE VOID XMVectorGetYPtr(FLOAT *y,FXMVECTOR V)
{
    XMASSERT( y != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *y = V.vector4_f32[1];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,1),y,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the Z component into a 32 bit float location in memory.
XMFINLINE VOID XMVectorGetZPtr(FLOAT *z,FXMVECTOR V)
{
    XMASSERT( z != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *z = V.vector4_f32[2];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,2),z,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the W component into a 32 bit float location in memory.
XMFINLINE VOID XMVectorGetWPtr(FLOAT *w,FXMVECTOR V)
{
    XMASSERT( w != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *w = V.vector4_f32[3];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,3),w,0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Return an integer value via an index. This is not a recommended
// function to use due to performance loss.
XMFINLINE UINT XMVectorGetIntByIndex(FXMVECTOR V, UINT i)
{
    XMASSERT( i < 4 );
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_u32[i];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_u32[i];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Return the X component in an integer register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE UINT XMVectorGetIntX(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_u32[0];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_u32[0];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the Y component in an integer register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE UINT XMVectorGetIntY(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_u32[1];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_u32[1];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the Z component in an integer register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE UINT XMVectorGetIntZ(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_u32[2];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_u32[2];
#endif // _XM_VMX128_INTRINSICS_
}

// Return the W component in an integer register. 
// This causes Load/Hit/Store on VMX targets
XMFINLINE UINT XMVectorGetIntW(FXMVECTOR V)
{
#if defined(_XM_NO_INTRINSICS_)
    return V.vector4_u32[3];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return V.vector4_u32[3];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Store a component indexed by i into a 32 bit integer location in memory.
// This causes Load/Hit/Store on VMX targets
XMFINLINE VOID XMVectorGetIntByIndexPtr(UINT *x,FXMVECTOR V,UINT i)
{
    XMASSERT( x != 0 );
    XMASSERT( i <  4 );
#if defined(_XM_NO_INTRINSICS_)
    *x = V.vector4_u32[i];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    *x = V.vector4_u32[i];
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Store the X component into a 32 bit integer location in memory.
XMFINLINE VOID XMVectorGetIntXPtr(UINT *x,FXMVECTOR V)
{
    XMASSERT( x != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *x = V.vector4_u32[0];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,0),x,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the Y component into a 32 bit integer location in memory.
XMFINLINE VOID XMVectorGetIntYPtr(UINT *y,FXMVECTOR V)
{
    XMASSERT( y != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *y = V.vector4_u32[1];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,1),y,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the Z component into a 32 bit integer locaCantion in memory.
XMFINLINE VOID XMVectorGetIntZPtr(UINT *z,FXMVECTOR V)
{
    XMASSERT( z != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *z = V.vector4_u32[2];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,2),z,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Store the W component into a 32 bit integer location in memory.
XMFINLINE VOID XMVectorGetIntWPtr(UINT *w,FXMVECTOR V)
{
    XMASSERT( w != 0 );
#if defined(_XM_NO_INTRINSICS_)
    *w = V.vector4_u32[3];
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    __stvewx(__vspltw(V,3),w,0);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Set a single indexed floating point component
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetByIndex(FXMVECTOR V, FLOAT f,UINT i)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_f32[i] = f;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR U;
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_f32[i] = f;
    return U;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets the X component of a vector to a passed floating point value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetX(FXMVECTOR V, FLOAT x)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_f32[0] = x;
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&x,0);
    return __vrlimi(V,vTemp,0x8,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Y component of a vector to a passed floating point value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetY(FXMVECTOR V, FLOAT y)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = y;
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&y,0);
    return __vrlimi(V,vTemp,0x4,3);
#endif // _XM_VMX128_INTRINSICS_
}
// Sets the Z component of a vector to a passed floating point value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetZ(FXMVECTOR V, FLOAT z)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = z;
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&z,0);
    return __vrlimi(V,vTemp,0x2,2);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the W component of a vector to a passed floating point value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetW(FXMVECTOR V, FLOAT w)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = w;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&w,0);
    return __vrlimi(V,vTemp,0x1,1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets a component of a vector to a floating point value passed by pointer
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetByIndexPtr(FXMVECTOR V,CONST FLOAT *f,UINT i)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( f != 0 );
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_f32[i] = *f;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR U;
    XMASSERT( f != 0 );
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_f32[i] = *f;
    return U;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets the X component of a vector to a floating point value passed by pointer
XMFINLINE XMVECTOR XMVectorSetXPtr(FXMVECTOR V,CONST FLOAT *x)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( x != 0 );
    U.vector4_f32[0] = *x;
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( x != 0 );
    vTemp = __lvlx(x,0);
    return __vrlimi(V,vTemp,0x8,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Y component of a vector to a floating point value passed by pointer
XMFINLINE XMVECTOR XMVectorSetYPtr(FXMVECTOR V,CONST FLOAT *y)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( y != 0 );
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = *y;
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( y != 0 );
    vTemp = __lvlx(y,0);
    return __vrlimi(V,vTemp,0x4,3);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Z component of a vector to a floating point value passed by pointer
XMFINLINE XMVECTOR XMVectorSetZPtr(FXMVECTOR V,CONST FLOAT *z)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( z != 0 );
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = *z;
    U.vector4_f32[3] = V.vector4_f32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( z != 0 );
    vTemp = __lvlx(z,0);
    return __vrlimi(V,vTemp,0x2,2);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the W component of a vector to a floating point value passed by pointer
XMFINLINE XMVECTOR XMVectorSetWPtr(FXMVECTOR V,CONST FLOAT *w)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( w != 0 );
    U.vector4_f32[0] = V.vector4_f32[0];
    U.vector4_f32[1] = V.vector4_f32[1];
    U.vector4_f32[2] = V.vector4_f32[2];
    U.vector4_f32[3] = *w;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( w != 0 );
    vTemp = __lvlx(w,0);
    return __vrlimi(V,vTemp,0x1,1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets a component of a vector to an integer passed by value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntByIndex(FXMVECTOR V, UINT x, UINT i)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_u32[i] = x;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR U;
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_u32[i] = x;
    return U;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets the X component of a vector to an integer passed by value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntX(FXMVECTOR V, UINT x)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_u32[0] = x;
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&x,0);
    return __vrlimi(V,vTemp,0x8,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Y component of a vector to an integer passed by value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntY(FXMVECTOR V, UINT y)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = y;
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&y,0);
    return __vrlimi(V,vTemp,0x4,3);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Z component of a vector to an integer passed by value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntZ(FXMVECTOR V, UINT z)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = z;
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&z,0);
    return __vrlimi(V,vTemp,0x2,2);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the W component of a vector to an integer passed by value
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntW(FXMVECTOR V, UINT w)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = w;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp = __lvlx(&w,0);
    return __vrlimi(V,vTemp,0x1,1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets a component of a vector to an integer value passed by pointer
// This causes Load/Hit/Store on VMX targets
XMFINLINE XMVECTOR XMVectorSetIntByIndexPtr(FXMVECTOR V, CONST UINT *x,UINT i)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( x != 0 );
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_u32[i] = *x;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR U;
    XMASSERT( x != 0 );
    XMASSERT( i <= 3 );
    U = V;
    U.vector4_u32[i] = *x;
    return U;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Sets the X component of a vector to an integer value passed by pointer
XMFINLINE XMVECTOR XMVectorSetIntXPtr(FXMVECTOR V,CONST UINT *x)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( x != 0 );
    U.vector4_u32[0] = *x;
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( x != 0 );
    vTemp = __lvlx(x,0);
    return __vrlimi(V,vTemp,0x8,0);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Y component of a vector to an integer value passed by pointer
XMFINLINE XMVECTOR XMVectorSetIntYPtr(FXMVECTOR V,CONST UINT *y)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( y != 0 );
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = *y;
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( y != 0 );
    vTemp = __lvlx(y,0);
    return __vrlimi(V,vTemp,0x4,3);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the Z component of a vector to an integer value passed by pointer
XMFINLINE XMVECTOR XMVectorSetIntZPtr(FXMVECTOR V,CONST UINT *z)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( z != 0 );
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = *z;
    U.vector4_u32[3] = V.vector4_u32[3];
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( z != 0 );
    vTemp = __lvlx(z,0);
    return __vrlimi(V,vTemp,0x2,2);
#endif // _XM_VMX128_INTRINSICS_
}

// Sets the W component of a vector to an integer value passed by pointer
XMFINLINE XMVECTOR XMVectorSetIntWPtr(FXMVECTOR V,CONST UINT *w)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR U;
    XMASSERT( w != 0 );
    U.vector4_u32[0] = V.vector4_u32[0];
    U.vector4_u32[1] = V.vector4_u32[1];
    U.vector4_u32[2] = V.vector4_u32[2];
    U.vector4_u32[3] = *w;
    return U;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR vTemp;
    XMASSERT( w != 0 );
    vTemp = __lvlx(w,0);
    return __vrlimi(V,vTemp,0x1,1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Define a control vector to be used in XMVectorPermute
// operations.  Visualize the two vectors V1 and V2 given
// in a permute as arranged back to back in a linear fashion,
// such that they form an array of 8 floating point values.
// The four integers specified in XMVectorPermuteControl
// will serve as indices into the array to select components
// from the two vectors.  ElementIndex0 is used to select
// an element from the vectors to be placed in the first
// component of the resulting vector, ElementIndex1 is used
// to select an element for the second component, etc.

XMFINLINE XMVECTOR XMVectorPermuteControl
(
    UINT     ElementIndex0, 
    UINT     ElementIndex1, 
    UINT     ElementIndex2, 
    UINT     ElementIndex3
)
{
#if defined(_XM_SSE_INTRINSICS_) || defined(_XM_NO_INTRINSICS_)
    XMVECTORU32 vControl;
    static CONST UINT ControlElement[] = {
                    XM_PERMUTE_0X,
                    XM_PERMUTE_0Y,
                    XM_PERMUTE_0Z,
                    XM_PERMUTE_0W,
                    XM_PERMUTE_1X,
                    XM_PERMUTE_1Y,
                    XM_PERMUTE_1Z,
                    XM_PERMUTE_1W
                };
    XMASSERT(ElementIndex0 < 8);
    XMASSERT(ElementIndex1 < 8);
    XMASSERT(ElementIndex2 < 8);
    XMASSERT(ElementIndex3 < 8);

    vControl.u[0] = ControlElement[ElementIndex0];
    vControl.u[1] = ControlElement[ElementIndex1];
    vControl.u[2] = ControlElement[ElementIndex2];
    vControl.u[3] = ControlElement[ElementIndex3];
    return vControl.v;
#else
    XMVECTOR vControl1;
    XMVECTOR vControl2;
    XMVECTOR vControl3;
    XMVECTOR vControl4;
    static CONST UINT ControlElement[] = {
        XM_PERMUTE_0X,
        XM_PERMUTE_0Y,
        XM_PERMUTE_0Z,
        XM_PERMUTE_0W,
        XM_PERMUTE_1X,
        XM_PERMUTE_1Y,
        XM_PERMUTE_1Z,
        XM_PERMUTE_1W
    };

    XMASSERT(ElementIndex0 < 8);
    XMASSERT(ElementIndex1 < 8);
    XMASSERT(ElementIndex2 < 8);
    XMASSERT(ElementIndex3 < 8);
    vControl1 = __lvlx(&ControlElement[ElementIndex0],0);
    vControl2 = __lvlx(&ControlElement[ElementIndex1],0);
    vControl3 = __lvlx(&ControlElement[ElementIndex2],0);
    vControl4 = __lvlx(&ControlElement[ElementIndex3],0);
    vControl1 = __vrlimi(vControl1,vControl2,0x4,3);
    vControl1 = __vrlimi(vControl1,vControl3,0x2,2);
    vControl1 = __vrlimi(vControl1,vControl4,0x1,1);
    return vControl1;
#endif
}

//------------------------------------------------------------------------------

// Using a control vector made up of 16 bytes from 0-31, remap V1 and V2's byte
// entries into a single 16 byte vector and return it. Index 0-15 = V1,
// 16-31 = V2
XMFINLINE XMVECTOR XMVectorPermute
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Control
)
{
#if defined(_XM_NO_INTRINSICS_)
    const BYTE *aByte[2];
    XMVECTOR Result;
    UINT i, uIndex, VectorIndex;
    const BYTE *pControl;
    BYTE *pWork;

    // Indices must be in range from 0 to 31
    XMASSERT((Control.vector4_u32[0] & 0xE0E0E0E0) == 0);
    XMASSERT((Control.vector4_u32[1] & 0xE0E0E0E0) == 0);
    XMASSERT((Control.vector4_u32[2] & 0xE0E0E0E0) == 0);
    XMASSERT((Control.vector4_u32[3] & 0xE0E0E0E0) == 0);

    // 0-15 = V1, 16-31 = V2
    aByte[0] = (const BYTE*)(&V1);
    aByte[1] = (const BYTE*)(&V2);
    i = 16;
    pControl = (const BYTE *)(&Control);
    pWork = (BYTE *)(&Result);
    do {
        // Get the byte to map from
        uIndex = pControl[0];
        ++pControl;
        VectorIndex = (uIndex>>4)&1;
        uIndex &= 0x0F;
#if defined(_XM_LITTLEENDIAN_)
        uIndex ^= 3; // Swap byte ordering on little endian machines
#endif
        pWork[0] = aByte[VectorIndex][uIndex];
        ++pWork;
    } while (--i);
    return Result;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_SSE_INTRINSICS_
    // XMVectorPermute should be used conservatively, since Control is only
    // able to be assigned to a possible 8 out of the 128 available registers.
#if defined(_PREFAST_) || defined(XMDEBUG)
    // Indices must be in range from 0 to 31
    static const XMVECTORI32 PremuteTest = {0xE0E0E0E0,0xE0E0E0E0,0xE0E0E0E0,0xE0E0E0E0};
    XMVECTOR vAssert = __vand(Control,PremuteTest.v);
    UINT CR;
    __vcmpequwR(vAssert,__vspltisw(0),&CR);
    XMASSERT((CR & XM_CRMASK_CR6TRUE)!=0);
#endif
    return __vperm(V1,V2,Control);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Define a control vector to be used in XMVectorSelect 
// operations.  The four integers specified in XMVectorSelectControl
// serve as indices to select between components in two vectors.
// The first index controls selection for the first component of 
// the vectors involved in a select operation, the second index 
// controls selection for the second component etc.  A value of
// zero for an index causes the corresponding component from the first 
// vector to be selected whereas a one causes the component from the
// second vector to be selected instead.

XMFINLINE XMVECTOR XMVectorSelectControl
(
    UINT VectorIndex0, 
    UINT VectorIndex1, 
    UINT VectorIndex2, 
    UINT VectorIndex3
)
{
#if defined(_XM_SSE_INTRINSICS_) && !defined(_XM_NO_INTRINSICS_)
#else
    XMVECTOR    ControlVector;
    CONST UINT  ControlElement[] =
                {
                    XM_SELECT_0,
                    XM_SELECT_1
                };

    XMASSERT(VectorIndex0 < 2);
    XMASSERT(VectorIndex1 < 2);
    XMASSERT(VectorIndex2 < 2);
    XMASSERT(VectorIndex3 < 2);

    ControlVector.vector4_u32[0] = ControlElement[VectorIndex0];
    ControlVector.vector4_u32[1] = ControlElement[VectorIndex1];
    ControlVector.vector4_u32[2] = ControlElement[VectorIndex2];
    ControlVector.vector4_u32[3] = ControlElement[VectorIndex3];

    return ControlVector;

#endif
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSelect
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Control
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = (V1.vector4_u32[0] & ~Control.vector4_u32[0]) | (V2.vector4_u32[0] & Control.vector4_u32[0]);
    Result.vector4_u32[1] = (V1.vector4_u32[1] & ~Control.vector4_u32[1]) | (V2.vector4_u32[1] & Control.vector4_u32[1]);
    Result.vector4_u32[2] = (V1.vector4_u32[2] & ~Control.vector4_u32[2]) | (V2.vector4_u32[2] & Control.vector4_u32[2]);
    Result.vector4_u32[3] = (V1.vector4_u32[3] & ~Control.vector4_u32[3]) | (V2.vector4_u32[3] & Control.vector4_u32[3]);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // The result should be assigned to Control to avoid an implicit 
    // copy of Control if it is later referenced as an input (i.e.
    // Control = XMVectorSelect(V1, V2, Control);).

    return __vsel(V1, V2, Control);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMergeXY
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[0];
    Result.vector4_u32[1] = V2.vector4_u32[0];
    Result.vector4_u32[2] = V1.vector4_u32[1];
    Result.vector4_u32[3] = V2.vector4_u32[1];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vmrghw(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMergeZW
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[2];
    Result.vector4_u32[1] = V2.vector4_u32[2];
    Result.vector4_u32[2] = V1.vector4_u32[3];
    Result.vector4_u32[3] = V2.vector4_u32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vmrglw(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Comparison operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;

    Control.vector4_u32[0] = (V1.vector4_f32[0] == V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] == V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] == V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] == V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;

    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpeqfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorEqualR
(
    UINT*    pCR,
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ux, uy, uz, uw, CR;
    XMVECTOR Control;

    XMASSERT( pCR );

    ux = (V1.vector4_f32[0] == V2.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    uy = (V1.vector4_f32[1] == V2.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    uz = (V1.vector4_f32[2] == V2.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    uw = (V1.vector4_f32[3] == V2.vector4_f32[3]) ? 0xFFFFFFFFU : 0;
    CR = 0;
    if (ux&uy&uz&uw)
    {
        // All elements are greater
        CR = XM_CRMASK_CR6TRUE;
    }
    else if (!(ux|uy|uz|uw))
    {
        // All elements are not greater
        CR = XM_CRMASK_CR6FALSE;
    }
    *pCR = CR;
    Control.vector4_u32[0] = ux;
    Control.vector4_u32[1] = uy;
    Control.vector4_u32[2] = uz;
    Control.vector4_u32[3] = uw;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT( pCR );
    return __vcmpeqfpR(V1, V2, pCR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Treat the components of the vectors as unsigned integers and
// compare individual bits between the two.  This is useful for
// comparing control vectors and result vectors returned from
// other comparison operations.

XMFINLINE XMVECTOR XMVectorEqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;

    Control.vector4_u32[0] = (V1.vector4_u32[0] == V2.vector4_u32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_u32[1] == V2.vector4_u32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_u32[2] == V2.vector4_u32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_u32[3] == V2.vector4_u32[3]) ? 0xFFFFFFFF : 0;

    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpequw(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorEqualIntR
(
    UINT*    pCR,
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;

    XMASSERT(pCR);

    Control = XMVectorEqualInt(V1, V2);

    *pCR = 0;

    if (XMVector4EqualInt(Control, XMVectorTrueInt()))
    {
        // All elements are equal
        *pCR |= XM_CRMASK_CR6TRUE;
    }
    else if (XMVector4EqualInt(Control, XMVectorFalseInt()))
    {
        // All elements are not equal
        *pCR |= XM_CRMASK_CR6FALSE;
    }

    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT(pCR);
    return __vcmpequwR(V1, V2, pCR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNearEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Epsilon
)
{
#if defined(_XM_NO_INTRINSICS_)

    FLOAT fDeltax, fDeltay, fDeltaz, fDeltaw;
    XMVECTOR Control;

    fDeltax = V1.vector4_f32[0]-V2.vector4_f32[0];
    fDeltay = V1.vector4_f32[1]-V2.vector4_f32[1];
    fDeltaz = V1.vector4_f32[2]-V2.vector4_f32[2];
    fDeltaw = V1.vector4_f32[3]-V2.vector4_f32[3];

    fDeltax = fabsf(fDeltax);
    fDeltay = fabsf(fDeltay);
    fDeltaz = fabsf(fDeltaz);
    fDeltaw = fabsf(fDeltaw);

    Control.vector4_u32[0] = (fDeltax <= Epsilon.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[1] = (fDeltay <= Epsilon.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[2] = (fDeltaz <= Epsilon.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[3] = (fDeltaw <= Epsilon.vector4_f32[3]) ? 0xFFFFFFFFU : 0;

    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Delta;
    XMVECTOR Zero;
    XMVECTOR Control;

    Delta = __vsubfp(V1, V2);
    Zero = __vspltisw(0);
    Control = __vcmpbfp(Delta, Epsilon);
    Control = __vcmpequw(Control, Zero);

    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNotEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_f32[0] != V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] != V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] != V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] != V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Control;

    Control = __vcmpeqfp(V1, V2);
    Control = __vnor(Control, Control);

    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNotEqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_u32[0] != V2.vector4_u32[0]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[1] = (V1.vector4_u32[1] != V2.vector4_u32[1]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[2] = (V1.vector4_u32[2] != V2.vector4_u32[2]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[3] = (V1.vector4_u32[3] != V2.vector4_u32[3]) ? 0xFFFFFFFFU : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Control;
    Control = __vcmpequw(V1, V2);
    Control = __vnor(Control, Control);
    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorGreater
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_f32[0] > V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] > V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] > V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] > V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpgtfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorGreaterR
(
    UINT*    pCR,
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ux, uy, uz, uw, CR;
    XMVECTOR Control;

    XMASSERT( pCR );

    ux = (V1.vector4_f32[0] > V2.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    uy = (V1.vector4_f32[1] > V2.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    uz = (V1.vector4_f32[2] > V2.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    uw = (V1.vector4_f32[3] > V2.vector4_f32[3]) ? 0xFFFFFFFFU : 0;
    CR = 0;
    if (ux&uy&uz&uw)
    {
        // All elements are greater
        CR = XM_CRMASK_CR6TRUE;
    }
    else if (!(ux|uy|uz|uw))
    {
        // All elements are not greater
        CR = XM_CRMASK_CR6FALSE;
    }
    *pCR = CR;
    Control.vector4_u32[0] = ux;
    Control.vector4_u32[1] = uy;
    Control.vector4_u32[2] = uz;
    Control.vector4_u32[3] = uw;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT( pCR );
    return __vcmpgtfpR(V1, V2, pCR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorGreaterOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_f32[0] >= V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] >= V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] >= V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] >= V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpgefp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorGreaterOrEqualR
(
    UINT*    pCR,
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ux, uy, uz, uw, CR;
    XMVECTOR Control;

    XMASSERT( pCR );

    ux = (V1.vector4_f32[0] >= V2.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    uy = (V1.vector4_f32[1] >= V2.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    uz = (V1.vector4_f32[2] >= V2.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    uw = (V1.vector4_f32[3] >= V2.vector4_f32[3]) ? 0xFFFFFFFFU : 0;
    CR = 0;
    if (ux&uy&uz&uw)
    {
        // All elements are greater
        CR = XM_CRMASK_CR6TRUE;
    }
    else if (!(ux|uy|uz|uw))
    {
        // All elements are not greater
        CR = XM_CRMASK_CR6FALSE;
    }
    *pCR = CR;
    Control.vector4_u32[0] = ux;
    Control.vector4_u32[1] = uy;
    Control.vector4_u32[2] = uz;
    Control.vector4_u32[3] = uw;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMASSERT( pCR );
    return __vcmpgefpR(V1, V2, pCR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorLess
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_f32[0] < V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] < V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] < V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] < V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpgtfp(V2, V1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorLessOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V1.vector4_f32[0] <= V2.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V1.vector4_f32[1] <= V2.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V1.vector4_f32[2] <= V2.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V1.vector4_f32[3] <= V2.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vcmpgefp(V2, V1);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorInBounds
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = (V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[1] = (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[2] = (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2]) ? 0xFFFFFFFF : 0;
    Control.vector4_u32[3] = (V.vector4_f32[3] <= Bounds.vector4_f32[3] && V.vector4_f32[3] >= -Bounds.vector4_f32[3]) ? 0xFFFFFFFF : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Control;
    XMVECTOR Zero;

    Zero = __vspltisw(0);
    Control = __vcmpbfp(V, Bounds);
    Control = __vcmpequw(Control, Zero);

    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorInBoundsR
(
    UINT*    pCR,
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT ux, uy, uz, uw, CR;
    XMVECTOR Control;

    XMASSERT( pCR != 0 );

    ux = (V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    uy = (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    uz = (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    uw = (V.vector4_f32[3] <= Bounds.vector4_f32[3] && V.vector4_f32[3] >= -Bounds.vector4_f32[3]) ? 0xFFFFFFFFU : 0;

    CR = 0;

    if (ux&uy&uz&uw)
    {
        // All elements are in bounds
        CR = XM_CRMASK_CR6BOUNDS;
    }
    *pCR = CR;
    Control.vector4_u32[0] = ux;
    Control.vector4_u32[1] = uy;
    Control.vector4_u32[2] = uz;
    Control.vector4_u32[3] = uw;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Control;
    XMVECTOR Zero;

    XMASSERT( pCR != 0 );

    Zero = __vspltisw(0);
    Control = __vcmpbfpR(V, Bounds, pCR);
    Control = __vcmpequw(Control, Zero);

    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorIsNaN
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = XMISNAN(V.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[1] = XMISNAN(V.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[2] = XMISNAN(V.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[3] = XMISNAN(V.vector4_f32[3]) ? 0xFFFFFFFFU : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Control;

    Control = __vcmpeqfp(V, V);
    Control = __vnor(Control, Control);

    return Control;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorIsInfinite
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Control;
    Control.vector4_u32[0] = XMISINF(V.vector4_f32[0]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[1] = XMISINF(V.vector4_f32[1]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[2] = XMISINF(V.vector4_f32[2]) ? 0xFFFFFFFFU : 0;
    Control.vector4_u32[3] = XMISINF(V.vector4_f32[3]) ? 0xFFFFFFFFU : 0;
    return Control;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR OneInt, TrueInt;
    XMVECTOR VShifted, InfinityShifted;
    XMVECTOR VEqualsInfinity;
    XMVECTOR Shift;

    OneInt = __vspltisw(1);
    TrueInt = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    VShifted = __vslw(V, OneInt);
    InfinityShifted = __vslw(TrueInt, Shift);
    VEqualsInfinity = __vcmpequw(VShifted, InfinityShifted);

    return VEqualsInfinity;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Rounding and clamping operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMin
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result.vector4_f32[0] = (V1.vector4_f32[0] < V2.vector4_f32[0]) ? V1.vector4_f32[0] : V2.vector4_f32[0];
    Result.vector4_f32[1] = (V1.vector4_f32[1] < V2.vector4_f32[1]) ? V1.vector4_f32[1] : V2.vector4_f32[1];
    Result.vector4_f32[2] = (V1.vector4_f32[2] < V2.vector4_f32[2]) ? V1.vector4_f32[2] : V2.vector4_f32[2];
    Result.vector4_f32[3] = (V1.vector4_f32[3] < V2.vector4_f32[3]) ? V1.vector4_f32[3] : V2.vector4_f32[3];
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vminfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMax
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result.vector4_f32[0] = (V1.vector4_f32[0] > V2.vector4_f32[0]) ? V1.vector4_f32[0] : V2.vector4_f32[0];
    Result.vector4_f32[1] = (V1.vector4_f32[1] > V2.vector4_f32[1]) ? V1.vector4_f32[1] : V2.vector4_f32[1];
    Result.vector4_f32[2] = (V1.vector4_f32[2] > V2.vector4_f32[2]) ? V1.vector4_f32[2] : V2.vector4_f32[2];
    Result.vector4_f32[3] = (V1.vector4_f32[3] > V2.vector4_f32[3]) ? V1.vector4_f32[3] : V2.vector4_f32[3];
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vmaxfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorRound
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR       Result;
    XMVECTOR       Bias;
    CONST XMVECTOR Zero = XMVectorZero();
    CONST XMVECTOR BiasPos = XMVectorReplicate(0.5f);
    CONST XMVECTOR BiasNeg = XMVectorReplicate(-0.5f);

    Bias = XMVectorLess(V, Zero);
    Bias = XMVectorSelect(BiasPos, BiasNeg, Bias);
    Result = XMVectorAdd(V, Bias);
    Result = XMVectorTruncate(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrfin(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorTruncate
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result;
    UINT     i;

    // Avoid C4701
    Result.vector4_f32[0] = 0.0f;

    for (i = 0; i < 4; i++)
    {
        if (XMISNAN(V.vector4_f32[i]))
        {
            Result.vector4_u32[i] = 0x7FC00000;
        }
        else if (fabsf(V.vector4_f32[i]) < 8388608.0f)
        {
            Result.vector4_f32[i] = (FLOAT)((INT)V.vector4_f32[i]);
        }
        else
        {
            Result.vector4_f32[i] = V.vector4_f32[i];
        }
    }
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrfiz(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorFloor
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR vResult = {
        floorf(V.vector4_f32[0]),
        floorf(V.vector4_f32[1]),
        floorf(V.vector4_f32[2]),
        floorf(V.vector4_f32[3])
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrfim(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorCeiling
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {
        ceilf(V.vector4_f32[0]),
        ceilf(V.vector4_f32[1]),
        ceilf(V.vector4_f32[2]),
        ceilf(V.vector4_f32[3])
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrfip(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorClamp
(
    FXMVECTOR V, 
    FXMVECTOR Min, 
    FXMVECTOR Max
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    XMASSERT(XMVector4LessOrEqual(Min, Max));

    Result = XMVectorMax(Min, V);
    Result = XMVectorMin(Max, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;

    XMASSERT(XMVector4LessOrEqual(Min, Max));

    Result = __vmaxfp(Min, V);
    Result = __vminfp(Max, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSaturate
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    CONST XMVECTOR Zero = XMVectorZero();

    return XMVectorClamp(V, Zero, g_XMOne.v);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Zero;
    XMVECTOR One;
    XMVECTOR Result;

    Zero = __vspltisw(0);
    One = __vupkd3d( Zero, VPACK_D3DCOLOR );

    Result = __vmaxfp(Zero, V);
    Result = __vminfp(One, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Bitwise logical operations
//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorAndInt
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[0] & V2.vector4_u32[0];
    Result.vector4_u32[1] = V1.vector4_u32[1] & V2.vector4_u32[1];
    Result.vector4_u32[2] = V1.vector4_u32[2] & V2.vector4_u32[2];
    Result.vector4_u32[3] = V1.vector4_u32[3] & V2.vector4_u32[3];
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vand(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorAndCInt
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[0] & ~V2.vector4_u32[0];
    Result.vector4_u32[1] = V1.vector4_u32[1] & ~V2.vector4_u32[1];
    Result.vector4_u32[2] = V1.vector4_u32[2] & ~V2.vector4_u32[2];
    Result.vector4_u32[3] = V1.vector4_u32[3] & ~V2.vector4_u32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vandc(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorOrInt
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[0] | V2.vector4_u32[0];
    Result.vector4_u32[1] = V1.vector4_u32[1] | V2.vector4_u32[1];
    Result.vector4_u32[2] = V1.vector4_u32[2] | V2.vector4_u32[2];
    Result.vector4_u32[3] = V1.vector4_u32[3] | V2.vector4_u32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vor(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNorInt
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = ~(V1.vector4_u32[0] | V2.vector4_u32[0]);
    Result.vector4_u32[1] = ~(V1.vector4_u32[1] | V2.vector4_u32[1]);
    Result.vector4_u32[2] = ~(V1.vector4_u32[2] | V2.vector4_u32[2]);
    Result.vector4_u32[3] = ~(V1.vector4_u32[3] | V2.vector4_u32[3]);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vnor(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorXorInt
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_u32[0] = V1.vector4_u32[0] ^ V2.vector4_u32[0];
    Result.vector4_u32[1] = V1.vector4_u32[1] ^ V2.vector4_u32[1];
    Result.vector4_u32[2] = V1.vector4_u32[2] ^ V2.vector4_u32[2];
    Result.vector4_u32[3] = V1.vector4_u32[3] ^ V2.vector4_u32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vxor(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Computation operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNegate
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] = -V.vector4_f32[0];
    Result.vector4_f32[1] = -V.vector4_f32[1];
    Result.vector4_f32[2] = -V.vector4_f32[2];
    Result.vector4_f32[3] = -V.vector4_f32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR SignMask;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    Result = __vxor(V, SignMask);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorAdd
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] = V1.vector4_f32[0] + V2.vector4_f32[0];
    Result.vector4_f32[1] = V1.vector4_f32[1] + V2.vector4_f32[1];
    Result.vector4_f32[2] = V1.vector4_f32[2] + V2.vector4_f32[2];
    Result.vector4_f32[3] = V1.vector4_f32[3] + V2.vector4_f32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vaddfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorAddAngles
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR       Mask;
    XMVECTOR       Offset;
    XMVECTOR       Result;
    CONST XMVECTOR Zero = XMVectorZero();

    // Add the given angles together.  If the range of V1 is such
    // that -Pi <= V1 < Pi and the range of V2 is such that
    // -2Pi <= V2 <= 2Pi, then the range of the resulting angle
    // will be -Pi <= Result < Pi.
    Result = XMVectorAdd(V1, V2);

    Mask = XMVectorLess(Result, g_XMNegativePi.v);
    Offset = XMVectorSelect(Zero, g_XMTwoPi.v, Mask);

    Mask = XMVectorGreaterOrEqual(Result, g_XMPi.v);
    Offset = XMVectorSelect(Offset, g_XMNegativeTwoPi.v, Mask);

    Result = XMVectorAdd(Result, Offset);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Offset1;
    XMVECTOR Offset2;
    XMVECTOR NegativePiConstants0, SignMask;
    XMVECTOR Pi, NegativePi, TwoPi, NegativeTwoPi;
    XMVECTOR Zero;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    Result = __vaddfp(V1, V2);
    NegativePiConstants0 = __vor(g_XMPiConstants0.v, SignMask);
    Pi = __vspltw(g_XMPiConstants0.v, 0);
    Zero = __vspltisw(0);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    NegativePi = __vspltw(NegativePiConstants0, 0);
    NegativeTwoPi = __vspltw(NegativePiConstants0, 1);
    Offset1 = __vcmpgtfp(NegativePi, Result);
    Offset2 = __vcmpgefp(Result, Pi);
    Offset1 = __vsel(Zero, TwoPi, Offset1);
    Offset2 = __vsel(Offset1, NegativeTwoPi, Offset2);
    Result = __vaddfp(Result, Offset2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSubtract
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] = V1.vector4_f32[0] - V2.vector4_f32[0];
    Result.vector4_f32[1] = V1.vector4_f32[1] - V2.vector4_f32[1];
    Result.vector4_f32[2] = V1.vector4_f32[2] - V2.vector4_f32[2];
    Result.vector4_f32[3] = V1.vector4_f32[3] - V2.vector4_f32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vsubfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSubtractAngles
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR       Mask;
    XMVECTOR       Offset;
    XMVECTOR       Result;
    CONST XMVECTOR Zero = XMVectorZero();

    // Subtract the given angles.  If the range of V1 is such
    // that -Pi <= V1 < Pi and the range of V2 is such that
    // -2Pi <= V2 <= 2Pi, then the range of the resulting angle
    // will be -Pi <= Result < Pi.
    Result = XMVectorSubtract(V1, V2);

    Mask = XMVectorLess(Result, g_XMNegativePi.v);
    Offset = XMVectorSelect(Zero, g_XMTwoPi.v, Mask);

    Mask = XMVectorGreaterOrEqual(Result, g_XMPi.v);
    Offset = XMVectorSelect(Offset, g_XMNegativeTwoPi.v, Mask);

    Result = XMVectorAdd(Result, Offset);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Offset1;
    XMVECTOR Offset2;
    XMVECTOR NegativePiConstants0, SignMask;
    XMVECTOR Pi, NegativePi, TwoPi, NegativeTwoPi;
    XMVECTOR Zero;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    Result = __vsubfp(V1, V2);
    NegativePiConstants0 = __vor(g_XMPiConstants0.v, SignMask);
    Pi = __vspltw(g_XMPiConstants0.v, 0);
    Zero = __vspltisw(0);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    NegativePi = __vspltw(NegativePiConstants0, 0);
    NegativeTwoPi = __vspltw(NegativePiConstants0, 1);
    Offset1 = __vcmpgtfp(NegativePi, Result);
    Offset2 = __vcmpgefp(Result, Pi);
    Offset1 = __vsel(Zero, TwoPi, Offset1);
    Offset2 = __vsel(Offset1, NegativeTwoPi, Offset2);
    Result = __vaddfp(Result, Offset2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMultiply
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result = {
        V1.vector4_f32[0] * V2.vector4_f32[0],
        V1.vector4_f32[1] * V2.vector4_f32[1],
        V1.vector4_f32[2] * V2.vector4_f32[2],
        V1.vector4_f32[3] * V2.vector4_f32[3]
    };
    return Result;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vmulfp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMultiplyAdd
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR V3
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {
        (V1.vector4_f32[0] * V2.vector4_f32[0]) + V3.vector4_f32[0],
        (V1.vector4_f32[1] * V2.vector4_f32[1]) + V3.vector4_f32[1],
        (V1.vector4_f32[2] * V2.vector4_f32[2]) + V3.vector4_f32[2],
        (V1.vector4_f32[3] * V2.vector4_f32[3]) + V3.vector4_f32[3]
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // For best performance, the result should be assigned back to either V2 or V3.

    return __vmaddfp(V1, V2, V3);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorDivide
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result;
    Result.vector4_f32[0] = V1.vector4_f32[0] / V2.vector4_f32[0];
    Result.vector4_f32[1] = V1.vector4_f32[1] / V2.vector4_f32[1];
    Result.vector4_f32[2] = V1.vector4_f32[2] / V2.vector4_f32[2];
    Result.vector4_f32[3] = V1.vector4_f32[3] / V2.vector4_f32[3];
    return Result;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR Refine;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Reciprocal = __vrefp(V2);

    // First refinement iteration (Newton-Raphson) for 1.0 / x
    //     y0 = reciprocal_estimate(x)
    //     y1 = y0 + y0 * (1.0 - x * y0)
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(V2, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);

    // Second refinement iteration
    //     y2 = y1 + y1 * (1.0 - x * y1)
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(V2, S0, S1);

    Refine = __vcmpeqfp(S0, S0);

    S0 = __vmaddfp(S0, S1, S0);

    Refine = __vsel(Reciprocal, S0, Refine);

    return __vmulfp(V1, Refine);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorNegativeMultiplySubtract
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR V3
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR vResult = {
        V3.vector4_f32[0] - (V1.vector4_f32[0] * V2.vector4_f32[0]),
        V3.vector4_f32[1] - (V1.vector4_f32[1] * V2.vector4_f32[1]),
        V3.vector4_f32[2] - (V1.vector4_f32[2] * V2.vector4_f32[2]),
        V3.vector4_f32[3] - (V1.vector4_f32[3] * V2.vector4_f32[3])
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // For best performance, the result should be assigned back to V3.

    return __vnmsubfp(V1, V2, V3);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorScale
(
    FXMVECTOR V, 
    FLOAT    ScaleFactor
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {
        V.vector4_f32[0] * ScaleFactor,
        V.vector4_f32[1] * ScaleFactor,
        V.vector4_f32[2] * ScaleFactor,
        V.vector4_f32[3] * ScaleFactor
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR Scale;
    XMVECTOR Result;

    Scale.vector4_f32[0] = 
    Scale.vector4_f32[1] = 
    Scale.vector4_f32[2] = 
    Scale.vector4_f32[3] = ScaleFactor;

    Result = __vmulfp(V, Scale);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Scale;
    XMVECTOR Result;
    XMASSERT(((UINT_PTR)&ScaleFactor & 3) == 0);

    Scale = __lvlx(&ScaleFactor, 0);
    Scale = __vspltw(Scale, 0);

    Result = __vmulfp(V, Scale);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorReciprocalEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result;
    UINT     i;

    // Avoid C4701
    Result.vector4_f32[0] = 0.0f;

    for (i = 0; i < 4; i++)
    {
        if (XMISNAN(V.vector4_f32[i]))
        {
            Result.vector4_u32[i] = 0x7FC00000;
        }
        else if (V.vector4_f32[i] == 0.0f || V.vector4_f32[i] == -0.0f)
        {
            Result.vector4_u32[i] = 0x7F800000 | (V.vector4_u32[i] & 0x80000000);
        }
        else
        {
            Result.vector4_f32[i] = 1.f / V.vector4_f32[i];
        }
    }
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrefp(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorReciprocal
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    return XMVectorReciprocalEst(V);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR Refine;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Reciprocal = __vrefp(V);

    // First refinement iteration (Newton-Raphson) for 1.0 / x
    //     y0 = reciprocal_estimate(x)
    //     y1 = y0 + y0 * (1.0 - x * y0)
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(V, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);

    // Second refinement iteration
    //     y2 = y1 + y1 * (1.0 - x * y1)
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(V, S0, S1);

    Refine = __vcmpeqfp(S0, S0);

    S0 = __vmaddfp(S0, S1, S0);

    Refine = __vsel(Reciprocal, S0, Refine);

    return Refine;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Return an estimated square root
XMFINLINE XMVECTOR XMVectorSqrtEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Select;

    // if (x == +Infinity)  sqrt(x) = +Infinity
    // if (x == +0.0f)      sqrt(x) = +0.0f
    // if (x == -0.0f)      sqrt(x) = -0.0f
    // if (x < 0.0f)        sqrt(x) = QNaN

    XMVECTOR Result = XMVectorReciprocalSqrtEst(V);
    XMVECTOR Zero = XMVectorZero();
    XMVECTOR VEqualsInfinity = XMVectorEqualInt(V, g_XMInfinity.v);
    XMVECTOR VEqualsZero = XMVectorEqual(V, Zero);
    Result = XMVectorMultiply(V, Result);
    Select = XMVectorEqualInt(VEqualsInfinity, VEqualsZero);
    Result = XMVectorSelect(V, Result, Select);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;
    XMVECTOR OneInt, TrueInt, Zero;
    XMVECTOR VShifted, InfinityShifted;
    XMVECTOR VEqualsInfinity, VEqualsZero;
    XMVECTOR Shift;
    XMVECTOR Rsq;

    Rsq = __vrsqrtefp(V);
    OneInt = __vspltisw(1);
    TrueInt = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    Zero = __vspltisw(0);
    VShifted = __vslw(V, OneInt);
    InfinityShifted = __vslw(TrueInt, Shift);
    VEqualsZero = __vcmpeqfp(V, Zero);
    VEqualsInfinity = __vcmpequw(VShifted, InfinityShifted);
    Rsq = __vmulfp(V, Rsq);
    Result = __vcmpequw(VEqualsInfinity, VEqualsZero);
    Result = __vsel(V, Rsq, Result);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSqrt
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Zero;
    XMVECTOR VEqualsInfinity, VEqualsZero;
    XMVECTOR Select;
    XMVECTOR Result;

    // if (x == +Infinity)  sqrt(x) = +Infinity
    // if (x == +0.0f)      sqrt(x) = +0.0f
    // if (x == -0.0f)      sqrt(x) = -0.0f
    // if (x < 0.0f)        sqrt(x) = QNaN

    Result = XMVectorReciprocalSqrt(V);
    Zero = XMVectorZero();
    VEqualsInfinity = XMVectorEqualInt(V, g_XMInfinity.v);
    VEqualsZero = XMVectorEqual(V, Zero);
    Result = XMVectorMultiply(V, Result);
    Select = XMVectorEqualInt(VEqualsInfinity, VEqualsZero);
    Result = XMVectorSelect(V, Result, Select);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR OneHalfV;
    XMVECTOR Reciprocal;
    XMVECTOR Scale;
    XMVECTOR RsqNotNaN;
    XMVECTOR ScaleNotNaN;
    XMVECTOR Select;
    XMVECTOR Result;

    // Refinement (Newton-Raphson) for 1.0 / sqrt(x)
    //     y0 = reciprocal_sqrt_estimate(x)
    //     y1 = y0 + 0.5 * y0 * (1.0 - x * y0 * y0) 
    //        = y0 + y0 * (0.5 - 0.5 * x * y0 * y0)

    Scale = __vspltisw(1);
    Result = __vrsqrtefp(V);
    Scale = __vcfsx(Scale, 1);
    OneHalfV = __vmulfp(V, Scale);
    Reciprocal = __vmulfp(Result, Result);
    RsqNotNaN = __vcmpeqfp(Result, Result); // TRUE if V >= -0.0f
    Scale = __vnmsubfp(OneHalfV, Reciprocal, Scale);
    Result = __vmaddfp(Result, Scale, Result);
    ScaleNotNaN = __vcmpeqfp(Scale, Scale); // TRUE if V > 0.0f and V < +Infinity
    Select = __vxor(ScaleNotNaN, RsqNotNaN); // Select result if Scale is not NaN or V < -0.0f
    Result = __vmulfp(V, Result);
    Select = __vsel(Result, V, Select);
    return Select;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorReciprocalSqrtEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    // if (x == +Infinity)  rsqrt(x) = 0
    // if (x == +0.0f)      rsqrt(x) = +Infinity
    // if (x == -0.0f)      rsqrt(x) = -Infinity
    // if (x < 0.0f)        rsqrt(x) = QNaN

    XMVECTOR Result;
    UINT     i;

    // Avoid C4701
    Result.vector4_f32[0] = 0.0f;

    for (i = 0; i < 4; i++)
    {
        if (XMISNAN(V.vector4_f32[i]))
        {
            Result.vector4_u32[i] = 0x7FC00000;
        }
        else if (V.vector4_f32[i] == 0.0f || V.vector4_f32[i] == -0.0f)
        {
            Result.vector4_u32[i] = 0x7F800000 | (V.vector4_u32[i] & 0x80000000);
        }
        else if (V.vector4_f32[i] < 0.0f)
        {
            Result.vector4_u32[i] = 0x7FFFFFFF;
        }
        else if (XMISINF(V.vector4_f32[i]))
        {
            Result.vector4_f32[i] = 0.0f;
        }
        else
        {
            Result.vector4_f32[i] = 1.0f / sqrtf(V.vector4_f32[i]);
        }
    }

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vrsqrtefp(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorReciprocalSqrt
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    return XMVectorReciprocalSqrtEst(V);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR ReciprocalSqrt;
    XMVECTOR OneHalfV;
    XMVECTOR Reciprocal;
    XMVECTOR Scale;

    // Refinement (Newton-Raphson) for 1.0 / sqrt(x)
    //     y0 = reciprocal_sqrt_estimate(x)
    //     y1 = y0 + 0.5 * y0 * (1.0 - x * y0 * y0) 
    //        = y0 + y0 * (0.5 - 0.5 * x * y0 * y0)
    Scale = __vspltisw(1);
    ReciprocalSqrt = __vrsqrtefp(V);
    Scale = __vcfsx(Scale, 1);
    OneHalfV = __vmulfp(V, Scale);
    Reciprocal = __vmulfp(ReciprocalSqrt, ReciprocalSqrt);
    Scale = __vnmsubfp(OneHalfV, Reciprocal, Scale);
    ReciprocalSqrt = __vmaddfp(ReciprocalSqrt, Scale, ReciprocalSqrt);

    return ReciprocalSqrt;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorExpEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result.vector4_f32[0] = powf(2.0f, V.vector4_f32[0]);
    Result.vector4_f32[1] = powf(2.0f, V.vector4_f32[1]);
    Result.vector4_f32[2] = powf(2.0f, V.vector4_f32[2]);
    Result.vector4_f32[3] = powf(2.0f, V.vector4_f32[3]);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vexptefp(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorExp
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR               E, S;
    XMVECTOR               R, R2, R3, R4;
    XMVECTOR               V0, V1;
    XMVECTOR               C0X, C0Y, C0Z, C0W;
    XMVECTOR               C1X, C1Y, C1Z, C1W;
    XMVECTOR               Result;
    static CONST XMVECTOR  C0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTOR  C1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};

    R = XMVectorFloor(V);
    E = XMVectorExpEst(R);
    R = XMVectorSubtract(V, R);
    R2 = XMVectorMultiply(R, R);
    R3 = XMVectorMultiply(R, R2);
    R4 = XMVectorMultiply(R2, R2);

    C0X = XMVectorSplatX(C0);
    C0Y = XMVectorSplatY(C0);
    C0Z = XMVectorSplatZ(C0);
    C0W = XMVectorSplatW(C0);

    C1X = XMVectorSplatX(C1);
    C1Y = XMVectorSplatY(C1);
    C1Z = XMVectorSplatZ(C1);
    C1W = XMVectorSplatW(C1);

    V0 = XMVectorMultiplyAdd(R, C0Y, C0X);
    V0 = XMVectorMultiplyAdd(R2, C0Z, V0);
    V0 = XMVectorMultiplyAdd(R3, C0W, V0);

    V1 = XMVectorMultiplyAdd(R, C1Y, C1X);
    V1 = XMVectorMultiplyAdd(R2, C1Z, V1);
    V1 = XMVectorMultiplyAdd(R3, C1W, V1);

    S = XMVectorMultiplyAdd(R4, V1, V0);

    S = XMVectorReciprocal(S);
    Result = XMVectorMultiply(E, S);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               E;
    XMVECTOR               R, R2, R3, R4;
    XMVECTOR               V0, V1;
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               Reciprocal, Scale1, Scale2, Refine;
    XMVECTOR               Result;
    static CONST XMVECTOR  C0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTOR  C1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};

    R = __vrfim(V);
    E = __vexptefp(R);

    V0 = __vspltw(C0, 0);
    C0Y = __vspltw(C0, 1);
    V1 = __vspltw(C1, 0);
    C1Y = __vspltw(C1, 1);

    R = __vsubfp(V, R);

    C0Z = __vspltw(C0, 2);
    C0W = __vspltw(C0, 3);
    C1Z = __vspltw(C1, 2);
    C1W = __vspltw(C1, 3);

    R2 = __vmulfp(R, R);

    V0 = __vmaddfp(R, C0Y, V0);
    V1 = __vmaddfp(R, C1Y, V1);

    R3 = __vmulfp(R, R2);

    V0 = __vmaddfp(R2, C0Z, V0);
    V1 = __vmaddfp(R2, C1Z, V1);

    V0 = __vmaddfp(R3, C0W, V0);

    R4 = __vmulfp(R2, R2);

    V1 = __vmaddfp(R3, C1W, V1);

    V0 = __vmaddfp(R4, V1, V0);

    Reciprocal = __vrefp(V0);
    Scale1 = __vspltw(C0, 0);
    Scale1 = __vnmsubfp(V0, Reciprocal, Scale1);
    Scale1 = __vmaddcfp(Reciprocal, Scale1, Reciprocal);
    Scale2 = __vspltw(C0, 0);
    Scale2 = __vnmsubfp(V0, Scale1, Scale2);
    Refine = __vcmpeqfp(Scale1, Scale1);
    Scale1 = __vmaddfp(Scale1, Scale2, Scale1);
    Refine = __vsel(Reciprocal, Scale1, Refine);

    Result = __vmulfp(E, Refine);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorLogEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    FLOAT fScale = (1.0f / logf(2.0f));
    XMVECTOR Result;

    Result.vector4_f32[0] = logf(V.vector4_f32[0])*fScale;
    Result.vector4_f32[1] = logf(V.vector4_f32[1])*fScale;
    Result.vector4_f32[2] = logf(V.vector4_f32[2])*fScale;
    Result.vector4_f32[3] = logf(V.vector4_f32[3])*fScale;
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return __vlogefp(V);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorLog
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fScale = (1.0f / logf(2.0f));
    XMVECTOR Result;

    Result.vector4_f32[0] = logf(V.vector4_f32[0])*fScale;
    Result.vector4_f32[1] = logf(V.vector4_f32[1])*fScale;
    Result.vector4_f32[2] = logf(V.vector4_f32[2])*fScale;
    Result.vector4_f32[3] = logf(V.vector4_f32[3])*fScale;
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               L;
    XMVECTOR               R, R2, R3, R4;
    XMVECTOR               V0, V1;
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               One, Shift;
    static CONST XMVECTOR  C0 = {1.44268966f, -7.21165776e-1f, 4.78684813e-1f, -3.47305417e-1f};
    static CONST XMVECTOR  C1 = {2.41873696e-1f, -1.37531206e-1f, 5.20646796e-2f, -9.31049418e-3f};

    XMDUMMY_INITIALIZE_VECTOR(One);

    One = __vupkd3d(One, VPACK_NORMSHORT2);
    One = __vspltw(One, 3);

    R = __vspltisw(-1);
    Shift = __vspltisw(-9); // Shift 23
    R = __vslw(R, Shift);

    R = __vsel(V, One, R);

    V0 = __vspltw(C0, 0);
    C0Y = __vspltw(C0, 1);
    V1 = __vspltw(C1, 0);
    C1Y = __vspltw(C1, 1);

    R = __vsubfp(R, One);

    C0Z = __vspltw(C0, 2);
    C0W = __vspltw(C0, 3);
    C1Z = __vspltw(C1, 2);
    C1W = __vspltw(C1, 3);

    R2 = __vmulfp(R, R);

    V0 = __vmaddfp(R, C0Y, V0);
    V1 = __vmaddfp(R, C1Y, V1);

    R3 = __vmulfp(R, R2);

    V0 = __vmaddfp(R2, C0Z, V0);
    V1 = __vmaddfp(R2, C1Z, V1);

    L = __vlogefp(V);

    V0 = __vmaddfp(R3, C0W, V0);

    R4 = __vmulfp(R2, R2);

    V1 = __vmaddfp(R3, C1W, V1);

    L = __vrfim(L);

    V0 = __vmaddfp(R4, V1, V0);

    L = __vmaddfp(R, V0, L);

    return L;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorPowEst
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] = powf(V1.vector4_f32[0], V2.vector4_f32[0]);
    Result.vector4_f32[1] = powf(V1.vector4_f32[1], V2.vector4_f32[1]);
    Result.vector4_f32[2] = powf(V1.vector4_f32[2], V2.vector4_f32[2]);
    Result.vector4_f32[3] = powf(V1.vector4_f32[3], V2.vector4_f32[3]);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AbsV1, PowEst;
    XMVECTOR V1IsNegative, V1IsZero, V1Sign, V2IsNegative, V1OrV2IsZero, V2Int, V2IsInt, ResultIsAlt, Alt, ResultAlt;
    XMVECTOR Shift, SignMask, Sign;
    XMVECTOR Zero, SignedZero, One, OneInt, QNaN;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Shift = __vspltisw(-1); // Shift 31
    SignMask = __vslw(Shift, Shift);

    AbsV1 = __vandc(V1, SignMask);

    PowEst = __vlogefp(AbsV1);

    V2IsInt = __vrfiz(V2);

    OneInt = __vspltisw(1);

    Zero = __vspltisw(0);

    QNaN = __vspltisw(-9); // Shift 23

    V2Int = __vctsxs(V2, 0);

    PowEst = __vmulfp(PowEst, V2);

    Sign = __vand(V2Int, OneInt);
    V1Sign = __vand(V1, SignMask);
    QNaN = __vslw(Shift, QNaN);

    Sign = __vslw(Sign, Shift);
    V2IsNegative = __vcmpgtfp(Zero, V2);
    V1IsZero = __vcmpeqfp(V1, Zero);
    ResultAlt = __vcmpeqfp(V2, Zero); // V2IsZero
    QNaN = __vsrw(QNaN, OneInt);
    Sign = __vand(V1Sign, Sign);
    V1IsNegative = __vcmpgtfp(Zero, V1);

    PowEst = __vexptefp(PowEst);

    SignedZero = __vor(Zero, Sign);
    Alt = __vandc(V1IsZero, V2IsNegative);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    V2IsInt = __vcmpeqfp(V2, V2IsInt);
    V1OrV2IsZero = __vor(V1IsZero, ResultAlt);

    Alt = __vsel(QNaN, SignedZero, Alt);
    One = __vspltw(One, 3);
    ResultIsAlt = __vandc(V1IsNegative, V2IsInt);

    ResultAlt = __vsel(Alt, One, ResultAlt);
    ResultIsAlt = __vor(ResultIsAlt, V1OrV2IsZero);

    PowEst = __vor(PowEst, Sign);
    ResultIsAlt = __vsel(PowEst, ResultAlt, ResultIsAlt);

    return ResultIsAlt;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorPow
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(_XM_SSE_INTRINSICS_)

    return XMVectorPowEst(V1, V2);

#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               L, E;
    XMVECTOR               R, R2, R3, R4;
    XMVECTOR               L0, L1;
    XMVECTOR               E0, E1;
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               One, Shift;
    XMVECTOR               Reciprocal, Scale1, Scale2, Refine;
    XMVECTOR               AbsV1, V1IsNegative, V1IsZero, V1Sign, V2IsNegative, V1OrV2IsZero, V2Int, V2IsInt;
    XMVECTOR               Alt, ResultIsAlt, ResultAlt;
    XMVECTOR               SignMask, Sign;
    XMVECTOR               Zero, SignedZero, OneInt, QNaN;
    XMVECTOR               Result;
    static CONST XMVECTOR  LC0 = {1.44268966f, -7.21165776e-1f, 4.78684813e-1f, -3.47305417e-1f};
    static CONST XMVECTOR  LC1 = {2.41873696e-1f, -1.37531206e-1f, 5.20646796e-2f, -9.31049418e-3f};
    static CONST XMVECTOR  EC0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTOR  EC1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};

    XMDUMMY_INITIALIZE_VECTOR(One);

    Shift = __vspltisw(-1); // Shift 31
    SignMask = __vslw(Shift, Shift);

    AbsV1 = __vandc(V1, SignMask);

    One = __vupkd3d(One, VPACK_NORMSHORT2);
    One = __vspltw(One, 3);

    R = __vspltisw(-1);
    Shift = __vspltisw(-9); // Shift 23
    R = __vslw(R, Shift);

    R = __vsel(AbsV1, One, R);

    L0 = __vspltw(LC0, 0);
    C0Y = __vspltw(LC0, 1);
    L1 = __vspltw(LC1, 0);
    C1Y = __vspltw(LC1, 1);

    R = __vsubfp(R, One);

    C0Z = __vspltw(LC0, 2);
    C0W = __vspltw(LC0, 3);
    C1Z = __vspltw(LC1, 2);
    C1W = __vspltw(LC1, 3);

    R2 = __vmulfp(R, R);

    L0 = __vmaddfp(R, C0Y, L0);
    L1 = __vmaddfp(R, C1Y, L1);

    R3 = __vmulfp(R, R2);

    L0 = __vmaddfp(R2, C0Z, L0);
    L1 = __vmaddfp(R2, C1Z, L1);

    L = __vlogefp(AbsV1);

    L0 = __vmaddfp(R3, C0W, L0);

    R4 = __vmulfp(R2, R2);

    L1 = __vmaddfp(R3, C1W, L1);

    L = __vrfim(L);

    R = __vmulfp(R, V2);
    L = __vmulfp(L, V2);

    L0 = __vmaddfp(R4, L1, L0);

    L = __vmaddfp(R, L0, L);

    R = __vrfim(L);
    E = __vexptefp(R);

    E0 = __vspltw(EC0, 0);
    C0Y = __vspltw(EC0, 1);
    E1 = __vspltw(EC1, 0);
    C1Y = __vspltw(EC1, 1);

    R = __vsubfp(L, R);

    C0Z = __vspltw(EC0, 2);
    C0W = __vspltw(EC0, 3);
    C1Z = __vspltw(EC1, 2);
    C1W = __vspltw(EC1, 3);

    R2 = __vmulfp(R, R);

    E0 = __vmaddfp(R, C0Y, E0);
    E1 = __vmaddfp(R, C1Y, E1);

    R3 = __vmulfp(R, R2);

    E0 = __vmaddfp(R2, C0Z, E0);
    E1 = __vmaddfp(R2, C1Z, E1);

    E0 = __vmaddfp(R3, C0W, E0);

    R4 = __vmulfp(R2, R2);

    E1 = __vmaddfp(R3, C1W, E1);

    E0 = __vmaddfp(R4, E1, E0);

    Reciprocal = __vrefp(E0);
    Scale1 = __vspltw(EC0, 0);
    Scale1 = __vnmsubfp(E0, Reciprocal, Scale1);
    Scale1 = __vmaddcfp(Reciprocal, Scale1, Reciprocal);
    Scale2 = __vspltw(EC0, 0);
    Scale2 = __vnmsubfp(E0, Scale1, Scale2);
    Refine = __vcmpeqfp(Scale1, Scale1);
    Scale1 = __vmaddfp(Scale1, Scale2, Scale1);

    OneInt = __vspltisw(1);
    V2Int = __vctsxs(V2, 0);
    Zero = __vspltisw(0);
    QNaN = __vspltisw(-9); // Shift 23
    Shift = __vspltisw(-1); // Regenerate shift 31
    Sign = __vand(V2Int, OneInt);
    V2IsInt = __vrfiz(V2);
    V1IsNegative = __vcmpgtfp(Zero, V1);
    V2IsNegative = __vcmpgtfp(Zero, V2);
    QNaN = __vslw(Shift, QNaN);
    V1Sign = __vand(V1, SignMask);
    Sign = __vslw(Sign, Shift);
    ResultAlt = __vcmpeqfp(V2, Zero);
    QNaN = __vsrw(QNaN, OneInt);

    Refine = __vsel(Reciprocal, Scale1, Refine);

    Sign = __vand(V1Sign, Sign);
    V1IsZero = __vcmpeqfp(V1, Zero);

    Refine = __vmulfp(E, Refine);

    SignedZero = __vor(Zero, Sign);
    Alt = __vandc(V1IsZero, V2IsNegative);
    V2IsInt = __vcmpeqfp(V2, V2IsInt);
    V1OrV2IsZero = __vor(V1IsZero, ResultAlt);

    Alt = __vsel(QNaN, SignedZero, Alt);
    ResultIsAlt = __vandc(V1IsNegative, V2IsInt);

    ResultAlt = __vsel(Alt, One, ResultAlt);
    ResultIsAlt = __vor(ResultIsAlt, V1OrV2IsZero);

    Refine = __vor(Refine, Sign);
    Result = __vsel(Refine, ResultAlt, ResultIsAlt);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorAbs
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {
        fabsf(V.vector4_f32[0]),
        fabsf(V.vector4_f32[1]),
        fabsf(V.vector4_f32[2]),
        fabsf(V.vector4_f32[3])
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR SignMask;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    Result = __vandc(V, SignMask);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorMod
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Reciprocal;
    XMVECTOR Quotient;
    XMVECTOR Result;

    // V1 % V2 = V1 - V2 * truncate(V1 / V2)
    Reciprocal = XMVectorReciprocal(V2);
    Quotient = XMVectorMultiply(V1, Reciprocal);
    Quotient = XMVectorTruncate(Quotient);
    Result = XMVectorNegativeMultiplySubtract(V2, Quotient, V1);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Reciprocal;
    XMVECTOR Rcp;
    XMVECTOR Scale;
    XMVECTOR Refine;
    XMVECTOR Quotient;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(Rcp);

    Reciprocal = __vrefp(V2);
    Rcp = __vupkd3d(Rcp, VPACK_NORMSHORT2);
    Scale = __vspltw(Rcp, 3);
    Rcp = __vspltw(Rcp, 3);
    Rcp = __vnmsubfp(V2, Reciprocal, Rcp);
    Rcp = __vmaddcfp(Reciprocal, Rcp, Reciprocal);
    Scale = __vnmsubfp(V2, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);
    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Refine = __vsel(Reciprocal, Rcp, Refine);
    Quotient = __vmulfp(V1, Refine);
    Result = V1;
    Quotient = __vrfiz(Quotient);
    Result = __vnmsubfp(V2, Quotient, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorModAngles
(
    FXMVECTOR Angles
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR Result;

    // Modulo the range of the given angles such that -XM_PI <= Angles < XM_PI
    V = XMVectorMultiply(Angles, g_XMReciprocalTwoPi.v);
    V = XMVectorRound(V);
    Result = XMVectorNegativeMultiplySubtract(g_XMTwoPi.v, V, Angles);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V;
    XMVECTOR ReciprocalTwoPi, TwoPi;
    XMVECTOR Result;

    ReciprocalTwoPi = __vspltw(g_XMPiConstants0.v, 3);
    V = __vmulfp(Angles, ReciprocalTwoPi);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    Result = Angles;
    V = __vrfin(V);
    Result = __vnmsubfp(TwoPi, V, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorSin
(
    FXMVECTOR V
)
{

#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2, V3, V5, V7, V9, V11, V13, V15, V17, V19, V21, V23;
    XMVECTOR S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11;
    XMVECTOR Result;

    V1 = XMVectorModAngles(V);

    // sin(V) ~= V - V^3 / 3! + V^5 / 5! - V^7 / 7! + V^9 / 9! - V^11 / 11! + V^13 / 13! - 
    //           V^15 / 15! + V^17 / 17! - V^19 / 19! + V^21 / 21! - V^23 / 23! (for -PI <= V < PI)
    V2  = XMVectorMultiply(V1, V1);
    V3  = XMVectorMultiply(V2, V1);
    V5  = XMVectorMultiply(V3, V2);
    V7  = XMVectorMultiply(V5, V2);
    V9  = XMVectorMultiply(V7, V2);
    V11 = XMVectorMultiply(V9, V2);
    V13 = XMVectorMultiply(V11, V2);
    V15 = XMVectorMultiply(V13, V2);
    V17 = XMVectorMultiply(V15, V2);
    V19 = XMVectorMultiply(V17, V2);
    V21 = XMVectorMultiply(V19, V2);
    V23 = XMVectorMultiply(V21, V2);

    S1  = XMVectorSplatY(g_XMSinCoefficients0.v);
    S2  = XMVectorSplatZ(g_XMSinCoefficients0.v);
    S3  = XMVectorSplatW(g_XMSinCoefficients0.v);
    S4  = XMVectorSplatX(g_XMSinCoefficients1.v);
    S5  = XMVectorSplatY(g_XMSinCoefficients1.v);
    S6  = XMVectorSplatZ(g_XMSinCoefficients1.v);
    S7  = XMVectorSplatW(g_XMSinCoefficients1.v);
    S8  = XMVectorSplatX(g_XMSinCoefficients2.v);
    S9  = XMVectorSplatY(g_XMSinCoefficients2.v);
    S10 = XMVectorSplatZ(g_XMSinCoefficients2.v);
    S11 = XMVectorSplatW(g_XMSinCoefficients2.v);

    Result = XMVectorMultiplyAdd(S1, V3, V1);
    Result = XMVectorMultiplyAdd(S2, V5, Result);
    Result = XMVectorMultiplyAdd(S3, V7, Result);
    Result = XMVectorMultiplyAdd(S4, V9, Result);
    Result = XMVectorMultiplyAdd(S5, V11, Result);
    Result = XMVectorMultiplyAdd(S6, V13, Result);
    Result = XMVectorMultiplyAdd(S7, V15, Result);
    Result = XMVectorMultiplyAdd(S8, V17, Result);
    Result = XMVectorMultiplyAdd(S9, V19, Result);
    Result = XMVectorMultiplyAdd(S10, V21, Result);
    Result = XMVectorMultiplyAdd(S11, V23, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1, V2, V3, V5, V7, V9, V11, V13, V15, V17, V19, V21, V23;
    XMVECTOR S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11;
    XMVECTOR ReciprocalTwoPi, TwoPi;
    XMVECTOR Result;

    ReciprocalTwoPi = __vspltw(g_XMPiConstants0.v, 3);
    V1 = __vmulfp(V, ReciprocalTwoPi);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    Result = V;
    V1 = __vrfin(V1);
    Result = __vnmsubfp(TwoPi, V1, Result);

    V2 = __vmulfp(Result, Result);
    S1 = __vspltw(g_XMSinCoefficients0.v, 1);
    S2 = __vspltw(g_XMSinCoefficients0.v, 2);
    S3 = __vspltw(g_XMSinCoefficients0.v, 3);
    S4 = __vspltw(g_XMSinCoefficients1.v, 0);
    S5 = __vspltw(g_XMSinCoefficients1.v, 1);
    S6 = __vspltw(g_XMSinCoefficients1.v, 2);
    S7 = __vspltw(g_XMSinCoefficients1.v, 3);
    S8 = __vspltw(g_XMSinCoefficients2.v, 0);
    S9 = __vspltw(g_XMSinCoefficients2.v, 1);
    S10 = __vspltw(g_XMSinCoefficients2.v, 2);
    S11 = __vspltw(g_XMSinCoefficients2.v, 3);
    V3 = __vmulfp(V2, Result);
    V5 = __vmulfp(V3, V2);
    Result = __vmaddfp(S1, V3, Result);
    V7 = __vmulfp(V5, V2);
    Result = __vmaddfp(S2, V5, Result);
    V9 = __vmulfp(V7, V2);
    Result = __vmaddfp(S3, V7, Result);
    V11 = __vmulfp(V9, V2);
    Result = __vmaddfp(S4, V9, Result);
    V13 = __vmulfp(V11, V2);
    Result = __vmaddfp(S5, V11, Result);
    V15 = __vmulfp(V13, V2);
    Result = __vmaddfp(S6, V13, Result);
    V17 = __vmulfp(V15, V2);
    Result = __vmaddfp(S7, V15, Result);
    V19 = __vmulfp(V17, V2);
    Result = __vmaddfp(S8, V17, Result);
    V21 = __vmulfp(V19, V2);
    Result = __vmaddfp(S9, V19, Result);
    V23 = __vmulfp(V21, V2);
    Result = __vmaddfp(S10, V21, Result);
    Result = __vmaddfp(S11, V23, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorCos
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2, V4, V6, V8, V10, V12, V14, V16, V18, V20, V22;
    XMVECTOR C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR Result;

    V1 = XMVectorModAngles(V);

    // cos(V) ~= 1 - V^2 / 2! + V^4 / 4! - V^6 / 6! + V^8 / 8! - V^10 / 10! + V^12 / 12! - 
    //           V^14 / 14! + V^16 / 16! - V^18 / 18! + V^20 / 20! - V^22 / 22! (for -PI <= V < PI)
    V2 = XMVectorMultiply(V1, V1);
    V4 = XMVectorMultiply(V2, V2);
    V6 = XMVectorMultiply(V4, V2);
    V8 = XMVectorMultiply(V4, V4);
    V10 = XMVectorMultiply(V6, V4);
    V12 = XMVectorMultiply(V6, V6);
    V14 = XMVectorMultiply(V8, V6);
    V16 = XMVectorMultiply(V8, V8);
    V18 = XMVectorMultiply(V10, V8);
    V20 = XMVectorMultiply(V10, V10);
    V22 = XMVectorMultiply(V12, V10);

    C1 = XMVectorSplatY(g_XMCosCoefficients0.v);
    C2 = XMVectorSplatZ(g_XMCosCoefficients0.v);
    C3 = XMVectorSplatW(g_XMCosCoefficients0.v);
    C4 = XMVectorSplatX(g_XMCosCoefficients1.v);
    C5 = XMVectorSplatY(g_XMCosCoefficients1.v);
    C6 = XMVectorSplatZ(g_XMCosCoefficients1.v);
    C7 = XMVectorSplatW(g_XMCosCoefficients1.v);
    C8 = XMVectorSplatX(g_XMCosCoefficients2.v);
    C9 = XMVectorSplatY(g_XMCosCoefficients2.v);
    C10 = XMVectorSplatZ(g_XMCosCoefficients2.v);
    C11 = XMVectorSplatW(g_XMCosCoefficients2.v);

    Result = XMVectorMultiplyAdd(C1, V2, g_XMOne.v);
    Result = XMVectorMultiplyAdd(C2, V4, Result);
    Result = XMVectorMultiplyAdd(C3, V6, Result);
    Result = XMVectorMultiplyAdd(C4, V8, Result);
    Result = XMVectorMultiplyAdd(C5, V10, Result);
    Result = XMVectorMultiplyAdd(C6, V12, Result);
    Result = XMVectorMultiplyAdd(C7, V14, Result);
    Result = XMVectorMultiplyAdd(C8, V16, Result);
    Result = XMVectorMultiplyAdd(C9, V18, Result);
    Result = XMVectorMultiplyAdd(C10, V20, Result);
    Result = XMVectorMultiplyAdd(C11, V22, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR VN, V1, V2, V4, V6, V8, V10, V12, V14, V16, V18, V20, V22;
    XMVECTOR C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR ReciprocalTwoPi, TwoPi;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(Result);

    ReciprocalTwoPi = __vspltw(g_XMPiConstants0.v, 3);
    VN = __vmulfp(V, ReciprocalTwoPi);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    V1 = V;
    VN = __vrfin(VN);
    V1 = __vnmsubfp(TwoPi, VN, V1);

    V2 = __vmulfp(V1, V1);
    Result = __vupkd3d(Result, VPACK_NORMSHORT2);
    C1 = __vspltw(g_XMCosCoefficients0.v, 1);
    C2 = __vspltw(g_XMCosCoefficients0.v, 2);
    C3 = __vspltw(g_XMCosCoefficients0.v, 3);
    C4 = __vspltw(g_XMCosCoefficients1.v, 0);
    C5 = __vspltw(g_XMCosCoefficients1.v, 1);
    Result = __vspltw(Result, 3);
    V4 = __vmulfp(V2, V2);
    Result = __vmaddfp(C1, V2, Result);
    C6 = __vspltw(g_XMCosCoefficients1.v, 2);
    C7 = __vspltw(g_XMCosCoefficients1.v, 3);
    C8 = __vspltw(g_XMCosCoefficients2.v, 0);
    C9 = __vspltw(g_XMCosCoefficients2.v, 1);
    C10 = __vspltw(g_XMCosCoefficients2.v, 2);
    C11 = __vspltw(g_XMCosCoefficients2.v, 3);
    V6 = __vmulfp(V4, V2);
    Result = __vmaddfp(C2, V4, Result);
    V8 = __vmulfp(V4, V4);
    Result = __vmaddfp(C3, V6, Result);
    V10 = __vmulfp(V6, V4);
    Result = __vmaddfp(C4, V8, Result);
    V12 = __vmulfp(V6, V6);
    Result = __vmaddfp(C5, V10, Result);
    V14 = __vmulfp(V8, V6);
    Result = __vmaddfp(C6, V12, Result);
    V16 = __vmulfp(V8, V8);
    Result = __vmaddfp(C7, V14, Result);
    V18 = __vmulfp(V10, V8);
    Result = __vmaddfp(C8, V16, Result);
    V20 = __vmulfp(V10, V10);
    Result = __vmaddfp(C9, V18, Result);
    V22 = __vmulfp(V12, V10);
    Result = __vmaddfp(C10, V20, Result);
    Result = __vmaddfp(C11, V22, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE VOID XMVectorSinCos
(
    XMVECTOR* pSin, 
    XMVECTOR* pCos, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2, V3, V4, V5, V6, V7, V8, V9, V10, V11, V12, V13;
    XMVECTOR V14, V15, V16, V17, V18, V19, V20, V21, V22, V23;
    XMVECTOR S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11;
    XMVECTOR C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR Sin, Cos;

    XMASSERT(pSin);
    XMASSERT(pCos);

    V1 = XMVectorModAngles(V);

    // sin(V) ~= V - V^3 / 3! + V^5 / 5! - V^7 / 7! + V^9 / 9! - V^11 / 11! + V^13 / 13! - 
    //           V^15 / 15! + V^17 / 17! - V^19 / 19! + V^21 / 21! - V^23 / 23! (for -PI <= V < PI)
    // cos(V) ~= 1 - V^2 / 2! + V^4 / 4! - V^6 / 6! + V^8 / 8! - V^10 / 10! + V^12 / 12! - 
    //           V^14 / 14! + V^16 / 16! - V^18 / 18! + V^20 / 20! - V^22 / 22! (for -PI <= V < PI)

    V2 = XMVectorMultiply(V1, V1);
    V3 = XMVectorMultiply(V2, V1);
    V4 = XMVectorMultiply(V2, V2);
    V5 = XMVectorMultiply(V3, V2);
    V6 = XMVectorMultiply(V3, V3);
    V7 = XMVectorMultiply(V4, V3);
    V8 = XMVectorMultiply(V4, V4);
    V9 = XMVectorMultiply(V5, V4);
    V10 = XMVectorMultiply(V5, V5);
    V11 = XMVectorMultiply(V6, V5);
    V12 = XMVectorMultiply(V6, V6);
    V13 = XMVectorMultiply(V7, V6);
    V14 = XMVectorMultiply(V7, V7);
    V15 = XMVectorMultiply(V8, V7);
    V16 = XMVectorMultiply(V8, V8);
    V17 = XMVectorMultiply(V9, V8);
    V18 = XMVectorMultiply(V9, V9);
    V19 = XMVectorMultiply(V10, V9);
    V20 = XMVectorMultiply(V10, V10);
    V21 = XMVectorMultiply(V11, V10);
    V22 = XMVectorMultiply(V11, V11);
    V23 = XMVectorMultiply(V12, V11);

    S1  = XMVectorSplatY(g_XMSinCoefficients0.v);
    S2  = XMVectorSplatZ(g_XMSinCoefficients0.v);
    S3  = XMVectorSplatW(g_XMSinCoefficients0.v);
    S4  = XMVectorSplatX(g_XMSinCoefficients1.v);
    S5  = XMVectorSplatY(g_XMSinCoefficients1.v);
    S6  = XMVectorSplatZ(g_XMSinCoefficients1.v);
    S7  = XMVectorSplatW(g_XMSinCoefficients1.v);
    S8  = XMVectorSplatX(g_XMSinCoefficients2.v);
    S9  = XMVectorSplatY(g_XMSinCoefficients2.v);
    S10  = XMVectorSplatZ(g_XMSinCoefficients2.v);
    S11  = XMVectorSplatW(g_XMSinCoefficients2.v);

    C1 = XMVectorSplatY(g_XMCosCoefficients0.v);
    C2 = XMVectorSplatZ(g_XMCosCoefficients0.v);
    C3 = XMVectorSplatW(g_XMCosCoefficients0.v);
    C4 = XMVectorSplatX(g_XMCosCoefficients1.v);
    C5 = XMVectorSplatY(g_XMCosCoefficients1.v);
    C6 = XMVectorSplatZ(g_XMCosCoefficients1.v);
    C7 = XMVectorSplatW(g_XMCosCoefficients1.v);
    C8 = XMVectorSplatX(g_XMCosCoefficients2.v);
    C9 = XMVectorSplatY(g_XMCosCoefficients2.v);
    C10 = XMVectorSplatZ(g_XMCosCoefficients2.v);
    C11 = XMVectorSplatW(g_XMCosCoefficients2.v);

    Sin = XMVectorMultiplyAdd(S1, V3, V1);
    Sin = XMVectorMultiplyAdd(S2, V5, Sin);
    Sin = XMVectorMultiplyAdd(S3, V7, Sin);
    Sin = XMVectorMultiplyAdd(S4, V9, Sin);
    Sin = XMVectorMultiplyAdd(S5, V11, Sin);
    Sin = XMVectorMultiplyAdd(S6, V13, Sin);
    Sin = XMVectorMultiplyAdd(S7, V15, Sin);
    Sin = XMVectorMultiplyAdd(S8, V17, Sin);
    Sin = XMVectorMultiplyAdd(S9, V19, Sin);
    Sin = XMVectorMultiplyAdd(S10, V21, Sin);
    Sin = XMVectorMultiplyAdd(S11, V23, Sin);

    Cos = XMVectorMultiplyAdd(C1, V2, g_XMOne.v);
    Cos = XMVectorMultiplyAdd(C2, V4, Cos);
    Cos = XMVectorMultiplyAdd(C3, V6, Cos);
    Cos = XMVectorMultiplyAdd(C4, V8, Cos);
    Cos = XMVectorMultiplyAdd(C5, V10, Cos);
    Cos = XMVectorMultiplyAdd(C6, V12, Cos);
    Cos = XMVectorMultiplyAdd(C7, V14, Cos);
    Cos = XMVectorMultiplyAdd(C8, V16, Cos);
    Cos = XMVectorMultiplyAdd(C9, V18, Cos);
    Cos = XMVectorMultiplyAdd(C10, V20, Cos);
    Cos = XMVectorMultiplyAdd(C11, V22, Cos);

    *pSin = Sin;
    *pCos = Cos;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR VN, V1, V2, V3, V4, V5, V6, V7, V8, V9, V10, V11, V12, V13;
    XMVECTOR V14, V15, V16, V17, V18, V19, V20, V21, V22, V23;
    XMVECTOR S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11;
    XMVECTOR C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR ReciprocalTwoPi, TwoPi;
    XMVECTOR Sin, Cos;

    XMASSERT(pSin);
    XMASSERT(pCos);

    XMDUMMY_INITIALIZE_VECTOR(Cos);

    ReciprocalTwoPi = __vspltw(g_XMPiConstants0.v, 3);
    VN = __vmulfp(V, ReciprocalTwoPi);
    TwoPi = __vspltw(g_XMPiConstants0.v, 1);
    V1 = V;
    VN = __vrfin(VN);
    V1 = __vnmsubfp(TwoPi, VN, V1);

    Cos = __vupkd3d(Cos, VPACK_NORMSHORT2);

    V2 = __vmulfp(V1, V1);
    Cos = __vspltw(Cos, 3);
    C1 = __vspltw(g_XMCosCoefficients0.v, 1);
    S1 = __vspltw(g_XMSinCoefficients0.v, 1);
    C2 = __vspltw(g_XMCosCoefficients0.v, 2);
    S2 = __vspltw(g_XMSinCoefficients0.v, 2);
    C3 = __vspltw(g_XMCosCoefficients0.v, 3);
    S3 = __vspltw(g_XMSinCoefficients0.v, 3);
    C4 = __vspltw(g_XMCosCoefficients1.v, 0);
    S4 = __vspltw(g_XMSinCoefficients1.v, 0);
    V3 = __vmulfp(V2, V1);
    V4 = __vmulfp(V2, V2);
    Cos = __vmaddfp(C1, V2, Cos);
    C5 = __vspltw(g_XMCosCoefficients1.v, 1);
    S5 = __vspltw(g_XMSinCoefficients1.v, 1);
    C6 = __vspltw(g_XMCosCoefficients1.v, 2);
    S6 = __vspltw(g_XMSinCoefficients1.v, 2);
    C7 = __vspltw(g_XMCosCoefficients1.v, 3);
    S7 = __vspltw(g_XMSinCoefficients1.v, 3);
    C8 = __vspltw(g_XMCosCoefficients2.v, 0);
    S8 = __vspltw(g_XMSinCoefficients2.v, 0);
    V5 = __vmulfp(V3, V2);
    Sin = __vmaddfp(S1, V3, V1);
    V6 = __vmulfp(V3, V3);
    V7 = __vmulfp(V4, V3);
    Cos = __vmaddfp(C2, V4, Cos);
    C9 = __vspltw(g_XMCosCoefficients2.v, 1);
    S9 = __vspltw(g_XMSinCoefficients2.v, 1);
    C10 = __vspltw(g_XMCosCoefficients2.v, 2);
    S10 = __vspltw(g_XMSinCoefficients2.v, 2);
    C11 = __vspltw(g_XMCosCoefficients2.v, 3);
    S11 = __vspltw(g_XMSinCoefficients2.v, 3);
    Sin = __vmaddfp(S2, V5, Sin);
    V8 = __vmulfp(V4, V4);
    V9 = __vmulfp(V5, V4);
    Cos = __vmaddfp(C3, V6, Cos);
    Sin = __vmaddfp(S3, V7, Sin);
    V10 = __vmulfp(V5, V5);
    V11 = __vmulfp(V6, V5);
    Cos = __vmaddfp(C4, V8, Cos);
    Sin = __vmaddfp(S4, V9, Sin);
    V12 = __vmulfp(V6, V6);
    V13 = __vmulfp(V7, V6);
    Cos = __vmaddfp(C5, V10, Cos);
    Sin = __vmaddfp(S5, V11, Sin);
    V14 = __vmulfp(V7, V7);
    V15 = __vmulfp(V8, V7);
    Cos = __vmaddfp(C6, V12, Cos);
    Sin = __vmaddfp(S6, V13, Sin);
    V16 = __vmulfp(V8, V8);
    V17 = __vmulfp(V9, V8);
    Cos = __vmaddfp(C7, V14, Cos);
    Sin = __vmaddfp(S7, V15, Sin);
    V18 = __vmulfp(V9, V9);
    V19 = __vmulfp(V10, V9);
    Cos = __vmaddfp(C8, V16, Cos);
    Sin = __vmaddfp(S8, V17, Sin);
    V20 = __vmulfp(V10, V10);
    V21 = __vmulfp(V11, V10);
    Cos = __vmaddfp(C9, V18, Cos);
    Sin = __vmaddfp(S9, V19, Sin);
    V22 = __vmulfp(V11, V11);
    V23 = __vmulfp(V12, V11);
    Cos = __vmaddfp(C10, V20, Cos);
    Sin = __vmaddfp(S10, V21, Sin);
    Cos = __vmaddfp(C11, V22, Cos);
    Sin = __vmaddfp(S11, V23, Sin);

    *pCos = Cos;
    *pSin = Sin;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorTan
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    // Cody and Waite algorithm to compute tangent.

    XMVECTOR VA, VB, VC, VC2;
    XMVECTOR T0, T1, T2, T3, T4, T5, T6, T7;
    XMVECTOR C0, C1, TwoDivPi, Epsilon;
    XMVECTOR N, D;
    XMVECTOR R0, R1;
    XMVECTOR VIsZero, VCNearZero, VBIsEven;
    XMVECTOR Zero;
    XMVECTOR Result;
    UINT     i;
    static CONST XMVECTOR TanCoefficients0 = {1.0f, -4.667168334e-1f, 2.566383229e-2f, -3.118153191e-4f};
    static CONST XMVECTOR TanCoefficients1 = {4.981943399e-7f, -1.333835001e-1f, 3.424887824e-3f, -1.786170734e-5f};
    static CONST XMVECTOR TanConstants = {1.570796371f, 6.077100628e-11f, 0.000244140625f, 2.0f / XM_PI};
    static CONST XMVECTORU32 Mask = {0x1, 0x1, 0x1, 0x1};

    TwoDivPi = XMVectorSplatW(TanConstants);

    Zero = XMVectorZero();

    C0 = XMVectorSplatX(TanConstants);
    C1 = XMVectorSplatY(TanConstants);
    Epsilon = XMVectorSplatZ(TanConstants);

    VA = XMVectorMultiply(V, TwoDivPi);

    VA = XMVectorRound(VA);

    VC = XMVectorNegativeMultiplySubtract(VA, C0, V);

    VB = XMVectorAbs(VA);

    VC = XMVectorNegativeMultiplySubtract(VA, C1, VC);

    for (i = 0; i < 4; i++)
    {
        VB.vector4_u32[i] = (UINT)VB.vector4_f32[i];
    }

    VC2 = XMVectorMultiply(VC, VC);

    T7 = XMVectorSplatW(TanCoefficients1);
    T6 = XMVectorSplatZ(TanCoefficients1);
    T4 = XMVectorSplatX(TanCoefficients1);
    T3 = XMVectorSplatW(TanCoefficients0);
    T5 = XMVectorSplatY(TanCoefficients1);
    T2 = XMVectorSplatZ(TanCoefficients0);
    T1 = XMVectorSplatY(TanCoefficients0);
    T0 = XMVectorSplatX(TanCoefficients0);

    VBIsEven = XMVectorAndInt(VB, Mask.v);
    VBIsEven = XMVectorEqualInt(VBIsEven, Zero);

    N = XMVectorMultiplyAdd(VC2, T7, T6);
    D = XMVectorMultiplyAdd(VC2, T4, T3);
    N = XMVectorMultiplyAdd(VC2, N, T5);
    D = XMVectorMultiplyAdd(VC2, D, T2);
    N = XMVectorMultiply(VC2, N);
    D = XMVectorMultiplyAdd(VC2, D, T1);
    N = XMVectorMultiplyAdd(VC, N, VC);
    VCNearZero = XMVectorInBounds(VC, Epsilon);
    D = XMVectorMultiplyAdd(VC2, D, T0);

    N = XMVectorSelect(N, VC, VCNearZero);
    D = XMVectorSelect(D, g_XMOne.v, VCNearZero);

    R0 = XMVectorNegate(N);
    R1 = XMVectorReciprocal(D);
    R0 = XMVectorReciprocal(R0);
    R1 = XMVectorMultiply(N, R1);
    R0 = XMVectorMultiply(D, R0);

    VIsZero = XMVectorEqual(V, Zero);

    Result = XMVectorSelect(R0, R1, VBIsEven);

    Result = XMVectorSelect(Result, Zero, VIsZero);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR VA, VB, VC, VC2;
    XMVECTOR T0, T1, T2, T3, T5, T6;
    XMVECTOR C0, C1, TwoDivPi, Epsilon;
    XMVECTOR N, D, NS, DS;
    XMVECTOR R0, R1;
    XMVECTOR Select;
    XMVECTOR Reciprocal0, Reciprocal1, Rcp0, Rcp1, Scale0, Scale1;
    XMVECTOR Mask;
    XMVECTOR SignMask;
    XMVECTOR One;
    XMVECTOR Zero;
    XMVECTOR Result;
    static CONST XMVECTOR TanCoefficients0 = {1.0f, -4.667168334e-1f, 2.566383229e-2f, -3.118153191e-4f};
    static CONST XMVECTOR TanCoefficients1 = {4.981943399e-7f, -1.333835001e-1f, 3.424887824e-3f, -1.786170734e-5f};
    static CONST XMVECTOR TanConstants = {1.570796371f, 6.077100628e-11f, 0.000244140625f, 2.0f / XM_PI};

    TwoDivPi = __vspltw(TanConstants, 3);

    SignMask = __vspltisw(-1);

    C0 = __vspltw(TanConstants, 0);
    C1 = __vspltw(TanConstants, 1);

    VA = __vmulfp(V, TwoDivPi);

    SignMask = __vslw(SignMask, SignMask);

    VC = V;

    VA = __vrfin(VA);

    VC = __vnmsubfp(VA, C0, VC);

    VB = __vandc(VA, SignMask);

    VC = __vnmsubfp(VA, C1, VC);

    VB = __vctsxs(VB, 0);

    Mask = __vspltisw(1);
    Zero = __vspltisw(0);

    VC2 = __vmulfp(VC, VC);

    N = __vspltw(TanCoefficients1, 3);
    T6 = __vspltw(TanCoefficients1, 2);
    D = __vspltw(TanCoefficients1, 0);
    T3 = __vspltw(TanCoefficients0, 3);
    T5 = __vspltw(TanCoefficients1, 1);
    T2 = __vspltw(TanCoefficients0, 2);
    T1 = __vspltw(TanCoefficients0, 1);
    T0 = __vspltw(TanCoefficients0, 0);

    Select = __vand(VB, Mask);
    Select = __vcmpequw(Select, Zero); // VBIsEven

    N = __vmaddcfp(VC2, N, T6);
    D = __vmaddcfp(VC2, D, T3);
    N = __vmaddcfp(VC2, N, T5);
    D = __vmaddcfp(VC2, D, T2);
    N = __vmulfp(VC2, N);
    D = __vmaddcfp(VC2, D, T1);
    Epsilon = __vspltw(TanConstants, 2);
    N = __vmaddcfp(VC, N, VC);
    One = __vcfsx(Mask, 0);
    D = __vmaddcfp(VC2, D, T0);
    NS = __vcmpbfp(VC, Epsilon); // VCNearZero
    DS = __vcmpbfp(VC, Epsilon); // VCNearZero

    NS = __vsel(N, VC, NS);
    DS = __vsel(D, One, DS);

    R0 = __vxor(NS, SignMask);

    Reciprocal0 = __vrefp(DS);
    Rcp0 = One;
    Reciprocal1 = __vrefp(R0);
    Rcp1 = One;
    Rcp0 = __vnmsubfp(DS, Reciprocal0, Rcp0);
    Rcp1 = __vnmsubfp(R0, Reciprocal1, Rcp1);
    Rcp0 = __vmaddcfp(Reciprocal0, Rcp0, Reciprocal0);
    Scale0 = One;
    Rcp1 = __vmaddcfp(Reciprocal1, Rcp1, Reciprocal1);
    Scale1 = One;
    Scale0 = __vnmsubfp(DS, Rcp0, Scale0);
    Scale1 = __vnmsubfp(R0, Rcp1, Scale1);
    R1 = __vcmpeqfp(Rcp0, Rcp0);
    R0 = __vcmpeqfp(Rcp1, Rcp1);
    Rcp0 = __vmaddfp(Rcp0, Scale0, Rcp0);
    Rcp1 = __vmaddfp(Rcp1, Scale1, Rcp1);
    R1 = __vsel(Reciprocal0, Rcp0, R1);
    R0 = __vsel(Reciprocal1, Rcp1, R0);

    R1 = __vmulfp(NS, R1);
    R0 = __vmulfp(DS, R0);

    Result = __vcmpeqfp(V, Zero); // VIsZero

    Select = __vsel(R0, R1, Select);

    Result = __vsel(Select, Zero, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorSinH
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTORF32 Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f}; // 1.0f / ln(2.0f)

    V1 = XMVectorMultiplyAdd(V, Scale.v, g_XMNegativeOne.v);
    V2 = XMVectorNegativeMultiplySubtract(V, Scale.v, g_XMNegativeOne.v);

    E1 = XMVectorExp(V1);
    E2 = XMVectorExp(V2);

    Result = XMVectorSubtract(E1, E2);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               E1, E2;
    XMVECTOR               E[2];
    XMVECTOR               R[2], R2[2], R3[2], R4[2];
    XMVECTOR               V0[2], V1[2];
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               Reciprocal[2], Rcp[2], Scale[2], Refine[2];
    XMVECTOR               Result;
    static CONST XMVECTORF32  S0 = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f};
    static CONST XMVECTORF32  C0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTORF32  C1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};

    XMDUMMY_INITIALIZE_VECTOR(Scale[1]);

    E2 = __vspltisw(-1);
    E1 = __vcfsx(E2, 0);
    E2 = __vcfsx(E2, 0);

    E1 = __vmaddfp(V, S0.v, E1);
    E2 = __vnmsubfp(V, S0.v, E2);

    R[0] = __vrfim(E1);
    R[1] = __vrfim(E2);
    E[0] = __vexptefp(R[0]);
    E[1] = __vexptefp(R[1]);

    V0[0] = __vspltw(C0.v, 0); // C0X
    V0[1] = __vspltw(C0.v, 0); // C0X
    C0Y = __vspltw(C0.v, 1);
    V1[0] = __vspltw(C1.v, 0); // C1X
    V1[1] = __vspltw(C1.v, 0); // C1X
    C1Y = __vspltw(C1.v, 1);

    R[0] = __vsubfp(E1, R[0]);
    R[1] = __vsubfp(E2, R[1]);

    C0Z = __vspltw(C0.v, 2);
    C0W = __vspltw(C0.v, 3);
    C1Z = __vspltw(C1.v, 2);
    C1W = __vspltw(C1.v, 3);

    R2[0] = __vmulfp(R[0], R[0]);
    R2[1] = __vmulfp(R[1], R[1]);

    V0[0] = __vmaddfp(R[0], C0Y, V0[0]);
    V0[1] = __vmaddfp(R[1], C0Y, V0[1]);
    V1[0] = __vmaddfp(R[0], C1Y, V1[0]);
    V1[1] = __vmaddfp(R[1], C1Y, V1[1]);

    R3[0] = __vmulfp(R[0], R2[0]);
    R3[1] = __vmulfp(R[1], R2[1]);

    V0[0] = __vmaddfp(R2[0], C0Z, V0[0]);
    V0[1] = __vmaddfp(R2[1], C0Z, V0[1]);
    V1[0] = __vmaddfp(R2[0], C1Z, V1[0]);
    V1[1] = __vmaddfp(R2[1], C1Z, V1[1]);

    V0[0] = __vmaddfp(R3[0], C0W, V0[0]);
    V0[1] = __vmaddfp(R3[1], C0W, V0[1]);

    R4[0] = __vmulfp(R2[0], R2[0]);
    R4[1] = __vmulfp(R2[1], R2[1]);

    V1[0] = __vmaddfp(R3[0], C1W, V1[0]);
    V1[1] = __vmaddfp(R3[1], C1W, V1[1]);

    V0[0] = __vmaddfp(R4[0], V1[0], V0[0]);
    V0[1] = __vmaddfp(R4[1], V1[1], V0[1]);

    Reciprocal[0] = __vrefp(V0[0]);
    Reciprocal[1] = __vrefp(V0[1]);
    Scale[1] = __vupkd3d(Scale[1], VPACK_NORMSHORT2);
    Rcp[0] = __vspltw(Scale[1], 3);
    Rcp[1] = __vspltw(Scale[1], 3);
    Rcp[0] = __vnmsubfp(V0[0], Reciprocal[0], Rcp[0]);
    Rcp[1] = __vnmsubfp(V0[1], Reciprocal[1], Rcp[1]);
    Rcp[0] = __vmaddcfp(Reciprocal[0], Rcp[0], Reciprocal[0]);
    Rcp[1] = __vmaddcfp(Reciprocal[1], Rcp[1], Reciprocal[1]);
    Scale[0] = __vspltw(Scale[1], 3);
    Scale[1] = __vspltw(Scale[1], 3);
    Scale[0] = __vnmsubfp(V0[0], Rcp[0], Scale[0]);
    Scale[1] = __vnmsubfp(V0[1], Rcp[1], Scale[1]);
    Refine[0] = __vcmpeqfp(Rcp[0], Rcp[0]);
    Refine[1] = __vcmpeqfp(Rcp[1], Rcp[1]);
    Rcp[0] = __vmaddfp(Rcp[0], Scale[0], Rcp[0]);
    Rcp[1] = __vmaddfp(Rcp[1], Scale[1], Rcp[1]);
    Result = __vspltisw(0);
    Refine[0] = __vsel(Reciprocal[0], Rcp[0], Refine[0]);
    Refine[1] = __vsel(Reciprocal[1], Rcp[1], Refine[1]);

    Result = __vnmsubfp(E[1], Refine[1], Result);
    Result = __vmaddfp(E[0], Refine[0], Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorCosH
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTOR Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f}; // 1.0f / ln(2.0f)

    V1 = XMVectorMultiplyAdd(V, Scale, g_XMNegativeOne.v);
    V2 = XMVectorNegativeMultiplySubtract(V, Scale, g_XMNegativeOne.v);

    E1 = XMVectorExp(V1);
    E2 = XMVectorExp(V2);

    Result = XMVectorAdd(E1, E2);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               E1, E2;
    XMVECTOR               E[2];
    XMVECTOR               R[2], R2[2], R3[2], R4[2];
    XMVECTOR               V0[2], V1[2];
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               Reciprocal[2], Rcp[2], Scale[2], Refine[2];
    static CONST XMVECTOR  S0 = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f};
    static CONST XMVECTOR  C0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTOR  C1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};

    XMDUMMY_INITIALIZE_VECTOR(Scale[1]);

    E2 = __vspltisw(-1);
    E1 = __vcfsx(E2, 0);
    E2 = __vcfsx(E2, 0);

    E1 = __vmaddfp(V, S0, E1);
    E2 = __vnmsubfp(V, S0, E2);

    R[0] = __vrfim(E1);
    R[1] = __vrfim(E2);
    E[0] = __vexptefp(R[0]);
    E[1] = __vexptefp(R[1]);

    V0[0] = __vspltw(C0, 0);
    V0[1] = __vspltw(C0, 0);
    C0Y = __vspltw(C0, 1);
    V1[0] = __vspltw(C1, 0);
    V1[1] = __vspltw(C1, 0);
    C1Y = __vspltw(C1, 1);

    R[0] = __vsubfp(E1, R[0]);
    R[1] = __vsubfp(E2, R[1]);

    C0Z = __vspltw(C0, 2);
    C0W = __vspltw(C0, 3);
    C1Z = __vspltw(C1, 2);
    C1W = __vspltw(C1, 3);

    R2[0] = __vmulfp(R[0], R[0]);
    R2[1] = __vmulfp(R[1], R[1]);

    V0[0] = __vmaddfp(R[0], C0Y, V0[0]);
    V0[1] = __vmaddfp(R[1], C0Y, V0[1]);
    V1[0] = __vmaddfp(R[0], C1Y, V1[0]);
    V1[1] = __vmaddfp(R[1], C1Y, V1[1]);

    R3[0] = __vmulfp(R[0], R2[0]);
    R3[1] = __vmulfp(R[1], R2[1]);

    V0[0] = __vmaddfp(R2[0], C0Z, V0[0]);
    V0[1] = __vmaddfp(R2[1], C0Z, V0[1]);
    V1[0] = __vmaddfp(R2[0], C1Z, V1[0]);
    V1[1] = __vmaddfp(R2[1], C1Z, V1[1]);

    V0[0] = __vmaddfp(R3[0], C0W, V0[0]);
    V0[1] = __vmaddfp(R3[1], C0W, V0[1]);

    R4[0] = __vmulfp(R2[0], R2[0]);
    R4[1] = __vmulfp(R2[1], R2[1]);

    V1[0] = __vmaddfp(R3[0], C1W, V1[0]);
    V1[1] = __vmaddfp(R3[1], C1W, V1[1]);

    V0[0] = __vmaddfp(R4[0], V1[0], V0[0]);
    V0[1] = __vmaddfp(R4[1], V1[1], V0[1]);

    Reciprocal[0] = __vrefp(V0[0]);
    Reciprocal[1] = __vrefp(V0[1]);
    Scale[1] = __vupkd3d(Scale[1], VPACK_NORMSHORT2);
    Rcp[0] = __vspltw(Scale[1], 3);
    Rcp[1] = __vspltw(Scale[1], 3);
    Rcp[0] = __vnmsubfp(V0[0], Reciprocal[0], Rcp[0]);
    Rcp[1] = __vnmsubfp(V0[1], Reciprocal[1], Rcp[1]);
    Rcp[0] = __vmaddcfp(Reciprocal[0], Rcp[0], Reciprocal[0]);
    Rcp[1] = __vmaddcfp(Reciprocal[1], Rcp[1], Reciprocal[1]);
    Scale[0] = __vspltw(Scale[1], 3);
    Scale[1] = __vspltw(Scale[1], 3);
    Scale[0] = __vnmsubfp(V0[0], Rcp[0], Scale[0]);
    Scale[1] = __vnmsubfp(V0[1], Rcp[1], Scale[1]);
    Refine[0] = __vcmpeqfp(Rcp[0], Rcp[0]);
    Refine[1] = __vcmpeqfp(Rcp[1], Rcp[1]);
    Rcp[0] = __vmaddfp(Rcp[0], Scale[0], Rcp[0]);
    Rcp[1] = __vmaddfp(Rcp[1], Scale[1], Rcp[1]);
    Refine[0] = __vsel(Reciprocal[0], Rcp[0], Refine[0]);
    Refine[1] = __vsel(Reciprocal[1], Rcp[1], Refine[1]);

    E1 = __vmulfp(E[0], Refine[0]);
    E1 = __vmaddfp(E[1], Refine[1], E1);

    return E1;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorTanH
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR E;
    XMVECTOR Result;
    static CONST XMVECTORF32 Scale = {2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f}; // 2.0f / ln(2.0f)

    E = XMVectorMultiply(V, Scale.v);
    E = XMVectorExp(E);
    E = XMVectorMultiplyAdd(E, g_XMOneHalf.v, g_XMOneHalf.v);
    E = XMVectorReciprocal(E);

    Result = XMVectorSubtract(g_XMOne.v, E);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               VE, VR;
    XMVECTOR               E;
    XMVECTOR               R, R2, R3, R4;
    XMVECTOR               V0, V1;
    XMVECTOR               C0Y, C0Z, C0W;
    XMVECTOR               C1Y, C1Z, C1W;
    XMVECTOR               Reciprocal, Scale1, Scale2, Refine;
    XMVECTOR               S0, S1;
    XMVECTOR               One, OneInt;
    XMVECTOR               Result;
    static CONST XMVECTORF32  C0 = {1.0f, -6.93147182e-1f, 2.40226462e-1f, -5.55036440e-2f};
    static CONST XMVECTORF32  C1 = {9.61597636e-3f, -1.32823968e-3f, 1.47491097e-4f, -1.08635004e-5f};
    static CONST XMVECTORF32  Scale = {2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f}; // 2.0f / ln(2.0f)

    XMDUMMY_INITIALIZE_VECTOR(One);

    VE = __vmulfp(V, Scale.v);

    R = __vrfim(VE);
    E = __vexptefp(R);

    V0 = __vspltw(C0.v, 0);
    C0Y = __vspltw(C0.v, 1);
    V1 = __vspltw(C1.v, 0);
    C1Y = __vspltw(C1.v, 1);

    R = __vsubfp(VE, R);

    C0Z = __vspltw(C0.v, 2);
    C0W = __vspltw(C0.v, 3);
    C1Z = __vspltw(C1.v, 2);
    C1W = __vspltw(C1.v, 3);

    R2 = __vmulfp(R, R);

    V0 = __vmaddfp(R, C0Y, V0);
    V1 = __vmaddfp(R, C1Y, V1);

    R3 = __vmulfp(R, R2);

    V0 = __vmaddfp(R2, C0Z, V0);
    V1 = __vmaddfp(R2, C1Z, V1);

    V0 = __vmaddfp(R3, C0W, V0);

    R4 = __vmulfp(R2, R2);

    V1 = __vmaddfp(R3, C1W, V1);

    V0 = __vmaddfp(R4, V1, V0);

    Reciprocal = __vrefp(V0);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    Scale1 = __vspltw(One, 3);

    Scale1 = __vnmsubfp(V0, Reciprocal, Scale1);
    Scale2 = __vspltw(One, 3);
    Scale1 = __vmaddcfp(Reciprocal, Scale1, Reciprocal);
    Scale2 = __vnmsubfp(V0, Scale1, Scale2);
    Refine = __vcmpeqfp(Scale1, Scale1);
    Scale1 = __vmaddfp(Scale1, Scale2, Scale1);
    OneInt = __vspltisw(1);
    Refine = __vsel(Reciprocal, Scale1, Refine);

    VR = __vcfsx(OneInt, 1);

    VE = __vmulfp(E, Refine);

    VR = __vmaddfp(VE, VR, VR);

    Reciprocal = __vrefp(VR);

    S0 = __vspltw(One, 3);
    S0 = __vnmsubfp(VR, Reciprocal, S0);
    S1 = __vspltw(One, 3);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);

    S1 = __vnmsubfp(VR, S0, S1);

    VR = __vcmpeqfp(S0, S0);

    One = __vspltw(One, 3);

    S0 = __vmaddfp(S0, S1, S0);

    VR = __vsel(Reciprocal, S0, VR);

    Result = __vsubfp(One, VR);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorASin
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V2, V3, AbsV;
    XMVECTOR C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR R0, R1, R2, R3, R4;
    XMVECTOR OneMinusAbsV;
    XMVECTOR Rsq;
    XMVECTOR Result;
    static CONST XMVECTOR OnePlusEpsilon = {1.00000011921f, 1.00000011921f, 1.00000011921f, 1.00000011921f};

    // asin(V) = V * (C0 + C1 * V + C2 * V^2 + C3 * V^3 + C4 * V^4 + C5 * V^5) + (1 - V) * rsq(1 - V) * 
    //           V * (C6 + C7 * V + C8 * V^2 + C9 * V^3 + C10 * V^4 + C11 * V^5)

    AbsV = XMVectorAbs(V);

    V2 = XMVectorMultiply(V, V);
    V3 = XMVectorMultiply(V2, AbsV);

    R4 = XMVectorNegativeMultiplySubtract(AbsV, V, V);

    OneMinusAbsV = XMVectorSubtract(OnePlusEpsilon, AbsV);
    Rsq = XMVectorReciprocalSqrt(OneMinusAbsV);

    C0 = XMVectorSplatX(g_XMASinCoefficients0.v);
    C1 = XMVectorSplatY(g_XMASinCoefficients0.v);
    C2 = XMVectorSplatZ(g_XMASinCoefficients0.v);
    C3 = XMVectorSplatW(g_XMASinCoefficients0.v);

    C4 = XMVectorSplatX(g_XMASinCoefficients1.v);
    C5 = XMVectorSplatY(g_XMASinCoefficients1.v);
    C6 = XMVectorSplatZ(g_XMASinCoefficients1.v);
    C7 = XMVectorSplatW(g_XMASinCoefficients1.v);

    C8 = XMVectorSplatX(g_XMASinCoefficients2.v);
    C9 = XMVectorSplatY(g_XMASinCoefficients2.v);
    C10 = XMVectorSplatZ(g_XMASinCoefficients2.v);
    C11 = XMVectorSplatW(g_XMASinCoefficients2.v);

    R0 = XMVectorMultiplyAdd(C3, AbsV, C7);
    R1 = XMVectorMultiplyAdd(C1, AbsV, C5);
    R2 = XMVectorMultiplyAdd(C2, AbsV, C6);
    R3 = XMVectorMultiplyAdd(C0, AbsV, C4);

    R0 = XMVectorMultiplyAdd(R0, AbsV, C11);
    R1 = XMVectorMultiplyAdd(R1, AbsV, C9);
    R2 = XMVectorMultiplyAdd(R2, AbsV, C10);
    R3 = XMVectorMultiplyAdd(R3, AbsV, C8);

    R0 = XMVectorMultiplyAdd(R2, V3, R0);
    R1 = XMVectorMultiplyAdd(R3, V3, R1);

    R0 = XMVectorMultiply(V, R0);
    R1 = XMVectorMultiply(R4, R1);

    Result = XMVectorMultiplyAdd(R1, Rsq, R0);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V2, V3, AbsV;
    XMVECTOR C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR R0, R1, R2;
    XMVECTOR OneMinusAbsV, HalfOneMinusAbsV;
    XMVECTOR Rsq, Scale, H;
    XMVECTOR SignMask;
    static CONST XMVECTOR OnePlusEpsilon = {1.00000011921f, 1.00000011921f, 1.00000011921f, 1.00000011921f};

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    V2 = __vmulfp(V, V);
    AbsV = __vandc(V, SignMask);

    C3 = __vspltw(g_XMASinCoefficients0.v, 3);
    C7 = __vspltw(g_XMASinCoefficients1.v, 3);
    C1 = __vspltw(g_XMASinCoefficients0.v, 1);
    C5 = __vspltw(g_XMASinCoefficients1.v, 1);
    C2 = __vspltw(g_XMASinCoefficients0.v, 2);
    C6 = __vspltw(g_XMASinCoefficients1.v, 2);
    C0 = __vspltw(g_XMASinCoefficients0.v, 0);
    C4 = __vspltw(g_XMASinCoefficients1.v, 0);

    OneMinusAbsV = __vsubfp(OnePlusEpsilon, AbsV);

    H = __vspltisw(1);

    C7 = __vmaddfp(C3, AbsV, C7);
    C5 = __vmaddfp(C1, AbsV, C5);
    C6 = __vmaddfp(C2, AbsV, C6);
    C4 = __vmaddfp(C0, AbsV, C4);

    V3 = __vmulfp(V2, AbsV);

    H = __vcfsx(H, 1);

    C11 = __vspltw(g_XMASinCoefficients2.v, 3);
    C9 = __vspltw(g_XMASinCoefficients2.v, 1);
    C10 = __vspltw(g_XMASinCoefficients2.v, 2);
    C8 = __vspltw(g_XMASinCoefficients2.v, 0);

    Rsq = __vrsqrtefp(OneMinusAbsV);
    HalfOneMinusAbsV = __vmulfp(OneMinusAbsV, H);

    R2 = V;

    C11 = __vmaddfp(C7, AbsV, C11);
    C9 = __vmaddfp(C5, AbsV, C9);
    C10 = __vmaddfp(C6, AbsV, C10);
    C8 = __vmaddfp(C4, AbsV, C8);

    Scale = __vmulfp(Rsq, Rsq);

    R2 = __vnmsubfp(AbsV, R2, R2);

    C11 = __vmaddfp(C10, V3, C11);
    C9 = __vmaddfp(C8, V3, C9);

    H = __vnmsubfp(HalfOneMinusAbsV, Scale, H);

    R0 = __vmulfp(V, C11);
    R1 = __vmulfp(R2, C9);

    Rsq = __vmaddfp(Rsq, H, Rsq);

    R0 = __vmaddfp(R1, Rsq, R0);

    return R0;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorACos
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V2, V3, AbsV;
    XMVECTOR C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR R0, R1, R2, R3, R4;
    XMVECTOR OneMinusAbsV;
    XMVECTOR Rsq;
    XMVECTOR Result;
    static CONST XMVECTOR OnePlusEpsilon = {1.00000011921f, 1.00000011921f, 1.00000011921f, 1.00000011921f};

    // acos(V) = PI / 2 - asin(V)

    AbsV = XMVectorAbs(V);

    V2 = XMVectorMultiply(V, V);
    V3 = XMVectorMultiply(V2, AbsV);

    R4 = XMVectorNegativeMultiplySubtract(AbsV, V, V);

    OneMinusAbsV = XMVectorSubtract(OnePlusEpsilon, AbsV);
    Rsq = XMVectorReciprocalSqrt(OneMinusAbsV);

    C0 = XMVectorSplatX(g_XMASinCoefficients0.v);
    C1 = XMVectorSplatY(g_XMASinCoefficients0.v);
    C2 = XMVectorSplatZ(g_XMASinCoefficients0.v);
    C3 = XMVectorSplatW(g_XMASinCoefficients0.v);

    C4 = XMVectorSplatX(g_XMASinCoefficients1.v);
    C5 = XMVectorSplatY(g_XMASinCoefficients1.v);
    C6 = XMVectorSplatZ(g_XMASinCoefficients1.v);
    C7 = XMVectorSplatW(g_XMASinCoefficients1.v);

    C8 = XMVectorSplatX(g_XMASinCoefficients2.v);
    C9 = XMVectorSplatY(g_XMASinCoefficients2.v);
    C10 = XMVectorSplatZ(g_XMASinCoefficients2.v);
    C11 = XMVectorSplatW(g_XMASinCoefficients2.v);

    R0 = XMVectorMultiplyAdd(C3, AbsV, C7);
    R1 = XMVectorMultiplyAdd(C1, AbsV, C5);
    R2 = XMVectorMultiplyAdd(C2, AbsV, C6);
    R3 = XMVectorMultiplyAdd(C0, AbsV, C4);

    R0 = XMVectorMultiplyAdd(R0, AbsV, C11);
    R1 = XMVectorMultiplyAdd(R1, AbsV, C9);
    R2 = XMVectorMultiplyAdd(R2, AbsV, C10);
    R3 = XMVectorMultiplyAdd(R3, AbsV, C8);

    R0 = XMVectorMultiplyAdd(R2, V3, R0);
    R1 = XMVectorMultiplyAdd(R3, V3, R1);

    R0 = XMVectorMultiply(V, R0);
    R1 = XMVectorMultiply(R4, R1);

    Result = XMVectorMultiplyAdd(R1, Rsq, R0);

    Result = XMVectorSubtract(g_XMHalfPi.v, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V2, V3, AbsV;
    XMVECTOR C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11;
    XMVECTOR R0, R1, R2;
    XMVECTOR OneMinusAbsV, HalfOneMinusAbsV;
    XMVECTOR Rsq, Scale, H;
    XMVECTOR HalfPi;
    XMVECTOR SignMask;
    static CONST XMVECTOR OnePlusEpsilon = {1.00000011921f, 1.00000011921f, 1.00000011921f, 1.00000011921f};

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    V2 = __vmulfp(V, V);
    AbsV = __vandc(V, SignMask);

    C3 = __vspltw(g_XMASinCoefficients0.v, 3);
    C7 = __vspltw(g_XMASinCoefficients1.v, 3);
    C1 = __vspltw(g_XMASinCoefficients0.v, 1);
    C5 = __vspltw(g_XMASinCoefficients1.v, 1);
    C2 = __vspltw(g_XMASinCoefficients0.v, 2);
    C6 = __vspltw(g_XMASinCoefficients1.v, 2);
    C0 = __vspltw(g_XMASinCoefficients0.v, 0);
    C4 = __vspltw(g_XMASinCoefficients1.v, 0);

    OneMinusAbsV = __vsubfp(OnePlusEpsilon, AbsV);

    H = __vspltisw(1);

    C7 = __vmaddfp(C3, AbsV, C7);
    C5 = __vmaddfp(C1, AbsV, C5);
    C6 = __vmaddfp(C2, AbsV, C6);
    C4 = __vmaddfp(C0, AbsV, C4);

    V3 = __vmulfp(V2, AbsV);

    H = __vcfsx(H, 1);

    C11 = __vspltw(g_XMASinCoefficients2.v, 3);
    C9 = __vspltw(g_XMASinCoefficients2.v, 1);
    C10 = __vspltw(g_XMASinCoefficients2.v, 2);
    C8 = __vspltw(g_XMASinCoefficients2.v, 0);

    Rsq = __vrsqrtefp(OneMinusAbsV);
    HalfOneMinusAbsV = __vmulfp(OneMinusAbsV, H);

    R2 = V;

    C11 = __vmaddfp(C7, AbsV, C11);
    C9 = __vmaddfp(C5, AbsV, C9);
    C10 = __vmaddfp(C6, AbsV, C10);
    C8 = __vmaddfp(C4, AbsV, C8);

    HalfPi = __vspltw(g_XMPiConstants0.v, 0);

    Scale = __vmulfp(Rsq, Rsq);

    R2 = __vnmsubfp(AbsV, R2, R2);

    C11 = __vmaddfp(C10, V3, C11);
    C9 = __vmaddfp(C8, V3, C9);

    HalfPi = __vmulfp(HalfPi, H);

    H = __vnmsubfp(HalfOneMinusAbsV, Scale, H);

    R0 = __vmulfp(V, C11);
    R1 = __vmulfp(R2, C9);

    Rsq = __vmaddfp(Rsq, H, Rsq);

    R0 = __vmaddfp(R1, Rsq, R0);

    R0 = __vsubfp(HalfPi, R0);

    return R0;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorATan
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    // Cody and Waite algorithm to compute inverse tangent.

    XMVECTOR N, D;
    XMVECTOR VF, G, ReciprocalF, AbsF, FA, FB;
    XMVECTOR Sqrt3, Sqrt3MinusOne, TwoMinusSqrt3;
    XMVECTOR HalfPi, OneThirdPi, OneSixthPi, Epsilon, MinV, MaxV;
    XMVECTOR Zero;
    XMVECTOR NegativeHalfPi;
    XMVECTOR Angle1, Angle2;
    XMVECTOR F_GT_One, F_GT_TwoMinusSqrt3, AbsF_LT_Epsilon, V_LT_Zero, V_GT_MaxV, V_LT_MinV;
    XMVECTOR NegativeResult, Result;
    XMVECTOR P0, P1, P2, P3, Q0, Q1, Q2, Q3;
    static CONST XMVECTOR ATanConstants0 = {-1.3688768894e+1f, -2.0505855195e+1f, -8.4946240351f, -8.3758299368e-1f};
    static CONST XMVECTOR ATanConstants1 = {4.1066306682e+1f, 8.6157349597e+1f, 5.9578436142e+1f, 1.5024001160e+1f};
    static CONST XMVECTOR ATanConstants2 = {1.732050808f, 7.320508076e-1f, 2.679491924e-1f, 0.000244140625f}; // <sqrt(3), sqrt(3) - 1, 2 - sqrt(3), Epsilon>
    static CONST XMVECTOR ATanConstants3 = {XM_PIDIV2, XM_PI / 3.0f, XM_PI / 6.0f, 8.507059173e+37f}; // <Pi / 2, Pi / 3, Pi / 6, MaxV>

    Zero = XMVectorZero();

    P0 = XMVectorSplatX(ATanConstants0);
    P1 = XMVectorSplatY(ATanConstants0);
    P2 = XMVectorSplatZ(ATanConstants0);
    P3 = XMVectorSplatW(ATanConstants0);

    Q0 = XMVectorSplatX(ATanConstants1);
    Q1 = XMVectorSplatY(ATanConstants1);
    Q2 = XMVectorSplatZ(ATanConstants1);
    Q3 = XMVectorSplatW(ATanConstants1);

    Sqrt3 = XMVectorSplatX(ATanConstants2);
    Sqrt3MinusOne = XMVectorSplatY(ATanConstants2);
    TwoMinusSqrt3 = XMVectorSplatZ(ATanConstants2);
    Epsilon = XMVectorSplatW(ATanConstants2);

    HalfPi = XMVectorSplatX(ATanConstants3);
    OneThirdPi = XMVectorSplatY(ATanConstants3);
    OneSixthPi = XMVectorSplatZ(ATanConstants3);
    MaxV = XMVectorSplatW(ATanConstants3);

    VF = XMVectorAbs(V);
    ReciprocalF = XMVectorReciprocal(VF);

    F_GT_One = XMVectorGreater(VF, g_XMOne.v);

    VF = XMVectorSelect(VF, ReciprocalF, F_GT_One);
    Angle1 = XMVectorSelect(Zero, HalfPi, F_GT_One);
    Angle2 = XMVectorSelect(OneSixthPi, OneThirdPi, F_GT_One);

    F_GT_TwoMinusSqrt3 = XMVectorGreater(VF, TwoMinusSqrt3);

    FA = XMVectorMultiplyAdd(Sqrt3MinusOne, VF, VF);
    FA = XMVectorAdd(FA, g_XMNegativeOne.v);
    FB = XMVectorAdd(VF, Sqrt3);
    FB = XMVectorReciprocal(FB);
    FA = XMVectorMultiply(FA, FB);

    VF = XMVectorSelect(VF, FA, F_GT_TwoMinusSqrt3);
    Angle1 = XMVectorSelect(Angle1, Angle2, F_GT_TwoMinusSqrt3);

    AbsF = XMVectorAbs(VF);
    AbsF_LT_Epsilon = XMVectorLess(AbsF, Epsilon);

    G = XMVectorMultiply(VF, VF);

    D = XMVectorAdd(G, Q3);
    D = XMVectorMultiplyAdd(D, G, Q2);
    D = XMVectorMultiplyAdd(D, G, Q1);
    D = XMVectorMultiplyAdd(D, G, Q0);
    D = XMVectorReciprocal(D);

    N = XMVectorMultiplyAdd(P3, G, P2);
    N = XMVectorMultiplyAdd(N, G, P1);
    N = XMVectorMultiplyAdd(N, G, P0);
    N = XMVectorMultiply(N, G);
    Result = XMVectorMultiply(N, D);

    Result = XMVectorMultiplyAdd(Result, VF, VF);

    Result = XMVectorSelect(Result, VF, AbsF_LT_Epsilon);

    NegativeResult = XMVectorNegate(Result);
    Result = XMVectorSelect(Result, NegativeResult, F_GT_One);

    Result = XMVectorAdd(Result, Angle1);

    V_LT_Zero = XMVectorLess(V, Zero);
    NegativeResult = XMVectorNegate(Result);
    Result = XMVectorSelect(Result, NegativeResult, V_LT_Zero);

    MinV = XMVectorNegate(MaxV);
    NegativeHalfPi = XMVectorNegate(HalfPi);
    V_GT_MaxV = XMVectorGreater(V, MaxV);
    V_LT_MinV = XMVectorLess(V, MinV);
    Result = XMVectorSelect(Result, g_XMHalfPi.v, V_GT_MaxV);
    Result = XMVectorSelect(Result, NegativeHalfPi, V_LT_MinV);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR P[4];
    XMVECTOR Q[4];
    XMVECTOR N, D;
    XMVECTOR VF, G, ReciprocalF, AbsF, FA, FB;
    XMVECTOR Sqrt3, Sqrt3MinusOne, TwoMinusSqrt3;
    XMVECTOR HalfPi, OneThirdPi, OneSixthPi, Epsilon, MinV, MaxV;
    XMVECTOR SignMask;
    XMVECTOR OneW, One, Zero, NegativeOne;
    XMVECTOR Scale, RcpE, Rcp, Refine;
    XMVECTOR NegativeHalfPi;
    XMVECTOR Angle1, Angle2;
    XMVECTOR GT1, GT2, F_GT_One, AbsF_LT_Epsilon, V_LT_Zero, V_GT_MaxV, V_LT_MinV;
    XMVECTOR NegativeResult, Result;
    static CONST XMVECTOR ATanConstants0 = {-1.3688768894e+1f, -2.0505855195e+1f, -8.4946240351f, -8.3758299368e-1f};
    static CONST XMVECTOR ATanConstants1 = {4.1066306682e+1f, 8.6157349597e+1f, 5.9578436142e+1f, 1.5024001160e+1f};
    static CONST XMVECTOR ATanConstants2 = {1.732050808f, 7.320508076e-1f, 2.679491924e-1f, 0.000244140625f}; // <sqrt(3), sqrt(3) - 1, 2 - sqrt(3), Epsilon>
    static CONST XMVECTOR ATanConstants3 = {XM_PIDIV2, XM_PI / 3.0f, XM_PI / 6.0f, 8.507059173e+37f}; // <Pi / 2, Pi / 3, Pi / 6, MaxV>

    XMDUMMY_INITIALIZE_VECTOR(OneW);

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    Zero = __vspltisw(0);

    OneW = __vupkd3d(OneW, VPACK_NORMSHORT2);

    VF = __vandc(V, SignMask);
    HalfPi = __vspltw(ATanConstants3, 0);
    OneThirdPi = __vspltw(ATanConstants3, 1);
    OneSixthPi = __vspltw(ATanConstants3, 2);
    MaxV = __vspltw(ATanConstants3, 3);

    One = __vspltw(OneW, 3);

    ReciprocalF = __vrefp(VF);
    Angle1 = __vcmpgtfp(VF, One);
    Angle2 = __vcmpgtfp(VF, One);
    F_GT_One = __vcmpgtfp(VF, One);
    Sqrt3 = __vspltw(ATanConstants2, 0);
    Sqrt3MinusOne = __vspltw(ATanConstants2, 1);
    TwoMinusSqrt3 = __vspltw(ATanConstants2, 2);
    Epsilon = __vspltw(ATanConstants2, 3);
    V_LT_Zero = __vcmpgtfp(Zero, V);
    NegativeHalfPi = __vxor(HalfPi, SignMask);
    Scale = __vspltw(OneW, 3);

    Scale = __vnmsubfp(VF, ReciprocalF, Scale);
    Rcp = ReciprocalF;
    Angle1 = __vsel(Zero, HalfPi, Angle1);
    Angle2 = __vsel(OneSixthPi, OneThirdPi, Angle2);
    MinV = __vxor(MaxV, SignMask);

    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Scale = __vspltw(OneW, 3);
    Q[0] = __vspltw(ATanConstants1, 0);
    Q[1] = __vspltw(ATanConstants1, 1);
    Q[2] = __vspltw(ATanConstants1, 2);
    Q[3] = __vspltw(ATanConstants1, 3);

    Scale = __vnmsubfp(VF, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);
    P[0] = __vspltw(ATanConstants0, 0);
    P[1] = __vspltw(ATanConstants0, 1);
    P[2] = __vspltw(ATanConstants0, 2);
    N = __vspltw(ATanConstants0, 3);
    V_GT_MaxV = __vcmpgtfp(V, MaxV);
    V_LT_MinV = __vcmpgtfp(MinV, V);

    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Refine = __vsel(ReciprocalF, Rcp, Refine);

    VF = __vsel(VF, Refine, F_GT_One);

    FB = __vaddfp(VF, Sqrt3);

    GT1 = __vcmpgtfp(VF, TwoMinusSqrt3); // F_GT_TwoMinusSqrt3
    GT2 = __vcmpgtfp(VF, TwoMinusSqrt3);

    Sqrt3MinusOne = __vmaddcfp(VF, Sqrt3MinusOne, VF);

    RcpE = __vrefp(FB);

    NegativeOne = __vspltisw(-1);
    NegativeOne = __vcfsx(NegativeOne, 0);

    FA = __vaddfp(Sqrt3MinusOne, NegativeOne);

    GT1 = __vsel(Angle1, Angle2, GT1);

    Scale = __vspltw(OneW, 3);

    Rcp = __vnmsubfp(FB, RcpE, Scale);
    Rcp = __vmaddcfp(RcpE, Rcp, RcpE);
    Scale = __vspltw(OneW, 3);
    Scale = __vnmsubfp(FB, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);
    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Refine = __vsel(RcpE, Rcp, Refine);

    FA = __vmulfp(FA, Refine);

    GT2 = __vsel(VF, FA, GT2);

    AbsF = __vandc(GT2, SignMask);
    G = __vmulfp(GT2, GT2);
    AbsF_LT_Epsilon = __vcmpgtfp(Epsilon, AbsF);

    D = __vaddfp(G, Q[3]);
    N = __vmaddcfp(G, N, P[2]);
    D = __vmaddcfp(G, D, Q[2]);
    N = __vmaddcfp(G, N, P[1]);
    D = __vmaddcfp(G, D, Q[1]);
    N = __vmaddcfp(G, N, P[0]);
    D = __vmaddcfp(G, D, Q[0]);
    N = __vmulfp(N, G);

    RcpE = __vrefp(D);
    Scale = __vspltw(OneW, 3);
    Rcp = __vnmsubfp(D, RcpE, Scale);
    Rcp = __vmaddcfp(RcpE, Rcp, RcpE);
    Scale = __vspltw(OneW, 3);
    Scale = __vnmsubfp(D, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);
    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Refine = __vsel(RcpE, Rcp, Refine);

    Result = __vmulfp(N, Refine);

    Result = __vmaddcfp(GT2, Result, GT2);

    AbsF_LT_Epsilon = __vsel(Result, GT2, AbsF_LT_Epsilon);

    NegativeResult = __vxor(AbsF_LT_Epsilon, SignMask);
    F_GT_One = __vsel(AbsF_LT_Epsilon, NegativeResult, F_GT_One);

    Result = __vaddfp(F_GT_One, GT1);

    NegativeResult = __vxor(Result, SignMask);
    V_LT_Zero = __vsel(Result, NegativeResult, V_LT_Zero);

    V_GT_MaxV = __vsel(V_LT_Zero, HalfPi, V_GT_MaxV);
    V_LT_MinV = __vsel(V_GT_MaxV, NegativeHalfPi, V_LT_MinV);

    return V_LT_MinV;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVectorATan2
(
    FXMVECTOR Y, 
    FXMVECTOR X
)
{
#if defined(_XM_NO_INTRINSICS_)

    // Return the inverse tangent of Y / X in the range of -Pi to Pi with the following exceptions:

    //     Y == 0 and X is Negative         -> Pi with the sign of Y
    //     y == 0 and x is positive         -> 0 with the sign of y
    //     Y != 0 and X == 0                -> Pi / 2 with the sign of Y
    //     Y != 0 and X is Negative         -> atan(y/x) + (PI with the sign of Y)
    //     X == -Infinity and Finite Y      -> Pi with the sign of Y
    //     X == +Infinity and Finite Y      -> 0 with the sign of Y
    //     Y == Infinity and X is Finite    -> Pi / 2 with the sign of Y
    //     Y == Infinity and X == -Infinity -> 3Pi / 4 with the sign of Y
    //     Y == Infinity and X == +Infinity -> Pi / 4 with the sign of Y

    XMVECTOR Reciprocal;
    XMVECTOR V;
    XMVECTOR YSign;
    XMVECTOR Pi, PiOverTwo, PiOverFour, ThreePiOverFour;
    XMVECTOR YEqualsZero, XEqualsZero, XIsPositive, YEqualsInfinity, XEqualsInfinity;
    XMVECTOR ATanResultValid;
    XMVECTOR R0, R1, R2, R3, R4, R5;
    XMVECTOR Zero;
    XMVECTOR Result;
    static CONST XMVECTOR ATan2Constants = {XM_PI, XM_PIDIV2, XM_PIDIV4, XM_PI * 3.0f / 4.0f};

    Zero = XMVectorZero();
    ATanResultValid = XMVectorTrueInt();

    Pi = XMVectorSplatX(ATan2Constants);
    PiOverTwo = XMVectorSplatY(ATan2Constants);
    PiOverFour = XMVectorSplatZ(ATan2Constants);
    ThreePiOverFour = XMVectorSplatW(ATan2Constants);

    YEqualsZero = XMVectorEqual(Y, Zero);
    XEqualsZero = XMVectorEqual(X, Zero);
    XIsPositive = XMVectorAndInt(X, g_XMNegativeZero.v);
    XIsPositive = XMVectorEqualInt(XIsPositive, Zero);
    YEqualsInfinity = XMVectorIsInfinite(Y);
    XEqualsInfinity = XMVectorIsInfinite(X);

    YSign = XMVectorAndInt(Y, g_XMNegativeZero.v);
    Pi = XMVectorOrInt(Pi, YSign);
    PiOverTwo = XMVectorOrInt(PiOverTwo, YSign);
    PiOverFour = XMVectorOrInt(PiOverFour, YSign);
    ThreePiOverFour = XMVectorOrInt(ThreePiOverFour, YSign);

    R1 = XMVectorSelect(Pi, YSign, XIsPositive);
    R2 = XMVectorSelect(ATanResultValid, PiOverTwo, XEqualsZero);
    R3 = XMVectorSelect(R2, R1, YEqualsZero);
    R4 = XMVectorSelect(ThreePiOverFour, PiOverFour, XIsPositive);
    R5 = XMVectorSelect(PiOverTwo, R4, XEqualsInfinity);
    Result = XMVectorSelect(R3, R5, YEqualsInfinity);
    ATanResultValid = XMVectorEqualInt(Result, ATanResultValid);

    Reciprocal = XMVectorReciprocal(X);
    V = XMVectorMultiply(Y, Reciprocal);
    R0 = XMVectorATan(V);

    R1 = XMVectorSelect( Pi, Zero, XIsPositive );
    R2 = XMVectorAdd(R0, R1);

    Result = XMVectorSelect(Result, R2, ATanResultValid);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Reciprocal;
    XMVECTOR V;
    XMVECTOR YSign;
    XMVECTOR Pi, PiOverTwo, PiOverFour, ThreePiOverFour;
    XMVECTOR XShifted, YShifted, InfinityShifted, XIsPositive;
    XMVECTOR ATanResultValid;
    XMVECTOR R0, R1, R2, R3, R4, R5, R6, R7, R8;
    XMVECTOR Rcp, Scale, Refine;
    XMVECTOR SignMask;
    XMVECTOR Zero, One, OneInt, TrueInt, Shift;
    XMVECTOR Result;
    static CONST XMVECTOR ATan2Constants = {XM_PI, XM_PIDIV2, XM_PIDIV4, XM_PI * 3.0f / 4.0f};

    XMDUMMY_INITIALIZE_VECTOR(One);

    TrueInt = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    OneInt = __vspltisw(1);
    Shift = __vspltisw(-8); // Shift 24
    SignMask = __vslw(TrueInt, TrueInt);

    Reciprocal = __vrefp(X);
    Scale = __vspltw(One, 3);

    YSign = __vand(Y, SignMask);
    Zero = __vspltisw(0);
    R1 = __vand(X, SignMask); // XIsNegative
    InfinityShifted = __vslw(TrueInt, Shift);
    YShifted = __vslw(Y, OneInt);
    XShifted = __vslw(X, OneInt);

    Rcp = __vnmsubfp(X, Reciprocal, Scale);

    Pi = __vspltw(ATan2Constants, 0);
    PiOverTwo = __vspltw(ATan2Constants, 1);
    ATanResultValid = __vspltisw(-1);
    PiOverFour = __vspltw(ATan2Constants, 2);
    ThreePiOverFour = __vspltw(ATan2Constants, 3);

    Rcp = __vmaddcfp(Reciprocal, Rcp, Reciprocal);

    Scale = __vspltw(One, 3);
    XIsPositive = __vcmpequw(R1, Zero);
    R2 = __vcmpeqfp(X, Zero); // XEqualsZero
    R3 = __vcmpeqfp(Y, Zero); // YEqualsZero
    R6 = __vcmpequw(YShifted, InfinityShifted); // YEqualsInfinity
    R7 = __vcmpequw(YShifted, InfinityShifted);
    R5 = __vcmpequw(XShifted, InfinityShifted); // XEqualsInfinity
    R8 = __vcmpequw(XShifted, InfinityShifted);

    Scale = __vnmsubfp(X, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);

    Pi = __vor(Pi, YSign);
    PiOverTwo = __vor(PiOverTwo, YSign);
    PiOverFour = __vor(PiOverFour, YSign);
    ThreePiOverFour = __vor(ThreePiOverFour, YSign);

    Rcp = __vmaddfp(Rcp, Scale, Rcp);
    Refine = __vsel(Reciprocal, Rcp, Refine);
    
    R1 = __vsel(Pi, YSign, XIsPositive);
    R2 = __vsel(ATanResultValid, PiOverTwo, R2);
    R4 = __vsel(ThreePiOverFour, PiOverFour, XIsPositive);
    R3 = __vsel(R2, R1, R3);
    R5 = __vsel(PiOverTwo, R4, R5);
    R6 = __vsel(R3, R5, R6);
    R7 = __vsel(R1, R6, R7);
    R8 = __vsel(R6, R7, R8);
    ATanResultValid = __vcmpequw(R8, ATanResultValid);

    V = __vmulfp(Y, Refine);
    R0 = XMVectorATan(V);

    R1 = __vsel(Pi, Zero, XIsPositive);
    R0 = __vaddfp(R0, R1);

    Result = __vsel(R8, R0, ATanResultValid);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSinEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V2, V3, V5, V7;
    XMVECTOR S1, S2, S3;
    XMVECTOR Result;

    // sin(V) ~= V - V^3 / 3! + V^5 / 5! - V^7 / 7! (for -PI <= V < PI)
    V2 = XMVectorMultiply(V, V);
    V3 = XMVectorMultiply(V2, V);
    V5 = XMVectorMultiply(V3, V2);
    V7 = XMVectorMultiply(V5, V2);

    S1 = XMVectorSplatY(g_XMSinEstCoefficients.v);
    S2 = XMVectorSplatZ(g_XMSinEstCoefficients.v);
    S3 = XMVectorSplatW(g_XMSinEstCoefficients.v);

    Result = XMVectorMultiplyAdd(S1, V3, V);
    Result = XMVectorMultiplyAdd(S2, V5, Result);
    Result = XMVectorMultiplyAdd(S3, V7, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V2, V5, V7;
    XMVECTOR S1, S2, S3;
    XMVECTOR Result;

    V2 = __vmulfp(V, V);
    S1 = __vspltw(g_XMSinEstCoefficients.v, 1);
    S2 = __vspltw(g_XMSinEstCoefficients.v, 2);
    S3 = __vspltw(g_XMSinEstCoefficients.v, 3);
    Result = __vmulfp(V2, V);
    V5 = __vmulfp(Result, V2);
    Result = __vmaddcfp(S1, Result, V);
    V7 = __vmulfp(V5, V2);
    Result = __vmaddfp(S2, V5, Result);
    Result = __vmaddfp(S3, V7, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorCosEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V2, V4, V6;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR Result;

    V2 = XMVectorMultiply(V, V);
    V4 = XMVectorMultiply(V2, V2);
    V6 = XMVectorMultiply(V4, V2);

    C0 = XMVectorSplatX(g_XMCosEstCoefficients.v);
    C1 = XMVectorSplatY(g_XMCosEstCoefficients.v);
    C2 = XMVectorSplatZ(g_XMCosEstCoefficients.v);
    C3 = XMVectorSplatW(g_XMCosEstCoefficients.v);

    Result = XMVectorMultiplyAdd(C1, V2, C0);
    Result = XMVectorMultiplyAdd(C2, V4, Result);
    Result = XMVectorMultiplyAdd(C3, V6, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V2, V4, V6;
    XMVECTOR C1, C2, C3;
    XMVECTOR Result;

    V2 = __vmulfp(V, V);
    Result = __vspltw(g_XMCosEstCoefficients.v, 0);
    C1 = __vspltw(g_XMCosEstCoefficients.v, 1);
    C2 = __vspltw(g_XMCosEstCoefficients.v, 2);
    C3 = __vspltw(g_XMCosEstCoefficients.v, 3);
    V4 = __vmulfp(V2, V2);
    Result = __vmaddfp(C1, V2, Result);
    V6 = __vmulfp(V4, V2);
    Result = __vmaddfp(C2, V4, Result);
    Result = __vmaddfp(C3, V6, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMVectorSinCosEst
(
    XMVECTOR* pSin, 
    XMVECTOR* pCos, 
    FXMVECTOR  V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V2, V3, V4, V5, V6, V7;
    XMVECTOR S1, S2, S3;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR Sin, Cos;

    XMASSERT(pSin);
    XMASSERT(pCos);

    // sin(V) ~= V - V^3 / 3! + V^5 / 5! - V^7 / 7! (for -PI <= V < PI)
    // cos(V) ~= 1 - V^2 / 2! + V^4 / 4! - V^6 / 6! (for -PI <= V < PI)
    V2 = XMVectorMultiply(V, V);
    V3 = XMVectorMultiply(V2, V);
    V4 = XMVectorMultiply(V2, V2);
    V5 = XMVectorMultiply(V3, V2);
    V6 = XMVectorMultiply(V3, V3);
    V7 = XMVectorMultiply(V4, V3);

    S1 = XMVectorSplatY(g_XMSinEstCoefficients.v);
    S2 = XMVectorSplatZ(g_XMSinEstCoefficients.v);
    S3 = XMVectorSplatW(g_XMSinEstCoefficients.v);

    C0 = XMVectorSplatX(g_XMCosEstCoefficients.v);
    C1 = XMVectorSplatY(g_XMCosEstCoefficients.v);
    C2 = XMVectorSplatZ(g_XMCosEstCoefficients.v);
    C3 = XMVectorSplatW(g_XMCosEstCoefficients.v);

    Sin = XMVectorMultiplyAdd(S1, V3, V);
    Sin = XMVectorMultiplyAdd(S2, V5, Sin);
    Sin = XMVectorMultiplyAdd(S3, V7, Sin);

    Cos = XMVectorMultiplyAdd(C1, V2, C0);
    Cos = XMVectorMultiplyAdd(C2, V4, Cos);
    Cos = XMVectorMultiplyAdd(C3, V6, Cos);

    *pSin = Sin;
    *pCos = Cos;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V2, V3, V4, V5, V6, V7;
    XMVECTOR S2, S3;
    XMVECTOR C0, C2, C3;
    XMVECTOR Sin, Cos;

    XMASSERT(pSin);
    XMASSERT(pCos);

    V2 = __vmulfp(V, V);
    C0 = __vspltw(g_XMCosEstCoefficients.v, 0);
    Cos = __vspltw(g_XMCosEstCoefficients.v, 1);
    Sin = __vspltw(g_XMSinEstCoefficients.v, 1);
    C2 = __vspltw(g_XMCosEstCoefficients.v, 2);
    S2 = __vspltw(g_XMSinEstCoefficients.v, 2);
    C3 = __vspltw(g_XMCosEstCoefficients.v, 3);
    S3 = __vspltw(g_XMSinEstCoefficients.v, 3);
    V3 = __vmulfp(V2, V);
    V4 = __vmulfp(V2, V2);
    Cos = __vmaddcfp(V2, Cos, C0);
    V5 = __vmulfp(V3, V2);
    Sin = __vmaddcfp(V3, Sin, V);
    V6 = __vmulfp(V3, V3);
    V7 = __vmulfp(V4, V3);
    Cos = __vmaddfp(C2, V4, Cos);
    Sin = __vmaddfp(S2, V5, Sin);
    Cos = __vmaddfp(C3, V6, Cos);
    Sin = __vmaddfp(S3, V7, Sin);

    *pCos = Cos;
    *pSin = Sin;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorTanEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2, V1T0, V1T1, V2T2;
    XMVECTOR T0, T1, T2;
    XMVECTOR N, D;
    XMVECTOR OneOverPi;
    XMVECTOR Result;

    OneOverPi = XMVectorSplatW(g_XMTanEstCoefficients.v);

    V1 = XMVectorMultiply(V, OneOverPi);
    V1 = XMVectorRound(V1);

    V1 = XMVectorNegativeMultiplySubtract(g_XMPi.v, V1, V);

    T0 = XMVectorSplatX(g_XMTanEstCoefficients.v);
    T1 = XMVectorSplatY(g_XMTanEstCoefficients.v);
    T2 = XMVectorSplatZ(g_XMTanEstCoefficients.v);

    V2T2 = XMVectorNegativeMultiplySubtract(V1, V1, T2);
    V2 = XMVectorMultiply(V1, V1);
    V1T0 = XMVectorMultiply(V1, T0);
    V1T1 = XMVectorMultiply(V1, T1);

    D = XMVectorReciprocalEst(V2T2);
    N = XMVectorMultiplyAdd(V2, V1T1, V1T0);

    Result = XMVectorMultiply(N, D);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1, V2, V1D, V1T0, V1T1, V2T2;
    XMVECTOR T0, T1, T2;
    XMVECTOR N, D;
    XMVECTOR OneOverPi, Pi;
    XMVECTOR Result;

    OneOverPi = __vspltw(g_XMTanEstCoefficients.v, 3);

    V1 = __vmulfp(V, OneOverPi);
    Pi = __vspltw(g_XMPiConstants0.v, 0);
    V1 = __vrfin(V1);
    V1D = V;

    V1D = __vnmsubfp(Pi, V1, V1D);

    T0 = __vspltw(g_XMTanEstCoefficients.v, 0);
    T1 = __vspltw(g_XMTanEstCoefficients.v, 1);
    T2 = __vspltw(g_XMTanEstCoefficients.v, 2);

    V2T2 = __vnmsubfp(V1D, V1D, T2);
    V2 = __vmulfp(V1D, V1D);
    V1T0 = __vmulfp(V1D, T0);
    V1T1 = __vmulfp(V1D, T1);

    D = __vrefp(V2T2);
    N = __vmaddfp(V2, V1T1, V1T0);

    Result = __vmulfp(N, D);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorSinHEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTORF32 Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f}; // 1.0f / ln(2.0f)

    V1 = XMVectorMultiplyAdd(V, Scale.v, g_XMNegativeOne.v);
    V2 = XMVectorNegativeMultiplySubtract(V, Scale.v, g_XMNegativeOne.v);

    E1 = XMVectorExpEst(V1);
    E2 = XMVectorExpEst(V2);

    Result = XMVectorSubtract(E1, E2);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTORF32 Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f};

    V2 = __vspltisw(-1);
    V1 = __vcfsx(V2, 0);
    V2 = __vcfsx(V2, 0);

    V1 = __vmaddfp(V, Scale.v, V1);
    V2 = __vnmsubfp(V, Scale.v, V2);

    E1 = __vexptefp(V1);
    E2 = __vexptefp(V2);

    Result = __vsubfp(E1, E2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorCosHEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTOR Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f}; // 1.0f / ln(2.0f)

    V1 = XMVectorMultiplyAdd(V, Scale, g_XMNegativeOne.v);
    V2 = XMVectorNegativeMultiplySubtract(V, Scale, g_XMNegativeOne.v);

    E1 = XMVectorExpEst(V1);
    E2 = XMVectorExpEst(V2);

    Result = XMVectorAdd(E1, E2);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1, V2;
    XMVECTOR E1, E2;
    XMVECTOR Result;
    static CONST XMVECTOR Scale = {1.442695040888963f, 1.442695040888963f, 1.442695040888963f, 1.442695040888963f};

    V2 = __vspltisw(-1);
    V1 = __vcfsx(V2, 0);
    V2 = __vcfsx(V2, 0);

    V1 = __vmaddfp(V, Scale, V1);
    V2 = __vnmsubfp(V, Scale, V2);

    E1 = __vexptefp(V1);
    E2 = __vexptefp(V2);

    Result = __vaddfp(E1, E2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorTanHEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR E;
    XMVECTOR Result;
    static CONST XMVECTOR Scale = {2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f}; // 2.0f / ln(2.0f)

    E = XMVectorMultiply(V, Scale);
    E = XMVectorExpEst(E);
    E = XMVectorMultiplyAdd(E, g_XMOneHalf.v, g_XMOneHalf.v);
    E = XMVectorReciprocalEst(E);

    Result = XMVectorSubtract(g_XMOne.v, E);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR E;
    XMVECTOR OneHalf, One;
    XMVECTOR Result;
    static CONST XMVECTOR Scale = {2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f, 2.8853900817779268f};

    E = __vmulfp(V, Scale);
    One = __vspltisw(1);
    E = __vexptefp(E);
    OneHalf = __vcfsx(One, 1);
    E = __vmaddfp(E, OneHalf, OneHalf);
    One = __vcfsx(One, 0);
    E = __vrefp(E);

    Result = __vsubfp(One, E);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorASinEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR AbsV, V2, VD, VC0, V2C3;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR D, Rsq, SqrtD;
    XMVECTOR OnePlusEps;
    XMVECTOR Result;

    AbsV = XMVectorAbs(V);

    OnePlusEps = XMVectorSplatX(g_XMASinEstConstants.v);

    C0 = XMVectorSplatX(g_XMASinEstCoefficients.v);
    C1 = XMVectorSplatY(g_XMASinEstCoefficients.v);
    C2 = XMVectorSplatZ(g_XMASinEstCoefficients.v);
    C3 = XMVectorSplatW(g_XMASinEstCoefficients.v);

    D = XMVectorSubtract(OnePlusEps, AbsV);

    Rsq = XMVectorReciprocalSqrtEst(D);
    SqrtD = XMVectorMultiply(D, Rsq);

    V2 = XMVectorMultiply(V, AbsV);
    V2C3 = XMVectorMultiply(V2, C3);
    VD = XMVectorMultiply(D, AbsV);
    VC0 = XMVectorMultiply(V, C0);

    Result = XMVectorMultiply(V, C1);
    Result = XMVectorMultiplyAdd(V2, C2, Result);
    Result = XMVectorMultiplyAdd(V2C3, VD, Result);
    Result = XMVectorMultiplyAdd(VC0, SqrtD, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AbsV, V2, VD, VC0, V2C3;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR D, Rsq, SqrtD;
    XMVECTOR SignMask;
    XMVECTOR OnePlusEps;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    AbsV = __vandc(V, SignMask);

    OnePlusEps = __vspltw(g_XMASinEstConstants.v, 0);
    C1 = __vspltw(g_XMASinEstCoefficients.v, 1);
    C3 = __vspltw(g_XMASinEstCoefficients.v, 3);

    D = __vsubfp(OnePlusEps, AbsV);

    V2 = __vmulfp(V, AbsV);
    Result = __vmulfp(V, C1);
    C2 = __vspltw(g_XMASinEstCoefficients.v, 2);

    C0 = __vspltw(g_XMASinEstCoefficients.v, 0);

    VD = __vmulfp(D, AbsV);
    V2C3 = __vmulfp(V2, C3);

    Rsq = __vrsqrtefp(D);
    Result = __vmaddfp(V2, C2, Result);
    VC0 = __vmulfp(V, C0);
    SqrtD = __vmulfp(D, Rsq);
    Result = __vmaddfp(V2C3, VD, Result);
    Result = __vmaddfp(VC0, SqrtD, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorACosEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR AbsV, V2, VD, VC0, V2C3;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR D, Rsq, SqrtD;
    XMVECTOR OnePlusEps, HalfPi;
    XMVECTOR Result;

    // acos(V) = PI / 2 - asin(V)

    AbsV = XMVectorAbs(V);

    OnePlusEps = XMVectorSplatX(g_XMASinEstConstants.v);
    HalfPi = XMVectorSplatY(g_XMASinEstConstants.v);

    C0 = XMVectorSplatX(g_XMASinEstCoefficients.v);
    C1 = XMVectorSplatY(g_XMASinEstCoefficients.v);
    C2 = XMVectorSplatZ(g_XMASinEstCoefficients.v);
    C3 = XMVectorSplatW(g_XMASinEstCoefficients.v);

    D = XMVectorSubtract(OnePlusEps, AbsV);

    Rsq = XMVectorReciprocalSqrtEst(D);
    SqrtD = XMVectorMultiply(D, Rsq);

    V2 = XMVectorMultiply(V, AbsV);
    V2C3 = XMVectorMultiply(V2, C3);
    VD = XMVectorMultiply(D, AbsV);
    VC0 = XMVectorMultiply(V, C0);

    Result = XMVectorMultiply(V, C1);
    Result = XMVectorMultiplyAdd(V2, C2, Result);
    Result = XMVectorMultiplyAdd(V2C3, VD, Result);
    Result = XMVectorMultiplyAdd(VC0, SqrtD, Result);
    Result = XMVectorSubtract(HalfPi, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AbsV, V2, VD, VC0, V2C3;
    XMVECTOR C0, C1, C2, C3;
    XMVECTOR D, Rsq, SqrtD;
    XMVECTOR OnePlusEps, HalfPi, SignMask;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    AbsV = __vandc(V, SignMask);

    OnePlusEps = __vspltw(g_XMASinEstConstants.v, 0);
    C1 = __vspltw(g_XMASinEstCoefficients.v, 1);
    C3 = __vspltw(g_XMASinEstCoefficients.v, 3);

    D = __vsubfp(OnePlusEps, AbsV);

    V2 = __vmulfp(V, AbsV);
    Result = __vmulfp(V, C1);
    C2 = __vspltw(g_XMASinEstCoefficients.v, 2);

    C0 = __vspltw(g_XMASinEstCoefficients.v, 0);

    VD = __vmulfp(D, AbsV);
    V2C3 = __vmulfp(V2, C3);

    Rsq = __vrsqrtefp(D);
    Result = __vmaddfp(V2, C2, Result);
    VC0 = __vmulfp(V, C0);
    SqrtD = __vmulfp(D, Rsq);
    Result = __vmaddfp(V2C3, VD, Result);
    HalfPi = __vspltw(g_XMASinEstConstants.v, 1);
    Result = __vmaddfp(VC0, SqrtD, Result);
    Result = __vsubfp(HalfPi, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorATanEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR AbsV, V2S2, N, D;
    XMVECTOR S0, S1, S2;
    XMVECTOR HalfPi;
    XMVECTOR Result;

    S0 = XMVectorSplatX(g_XMATanEstCoefficients.v);
    S1 = XMVectorSplatY(g_XMATanEstCoefficients.v);
    S2 = XMVectorSplatZ(g_XMATanEstCoefficients.v);
    HalfPi = XMVectorSplatW(g_XMATanEstCoefficients.v);

    AbsV = XMVectorAbs(V);

    V2S2 = XMVectorMultiplyAdd(V, V, S2);
    N = XMVectorMultiplyAdd(AbsV, HalfPi, S0);
    D = XMVectorMultiplyAdd(AbsV, S1, V2S2);
    N = XMVectorMultiply(N, V);
    D = XMVectorReciprocalEst(D);

    Result = XMVectorMultiply(N, D);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR AbsV, V2S2, N, D;
    XMVECTOR SignMask;
    XMVECTOR HalfPi;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    V2S2 = __vspltw(g_XMATanEstCoefficients.v, 2);
    AbsV = __vandc(V, SignMask);
    HalfPi = __vspltw(g_XMATanEstCoefficients.v, 3);
    N = __vspltw(g_XMATanEstCoefficients.v, 0);
    D = __vspltw(g_XMATanEstCoefficients.v, 1);

    V2S2 = __vmaddfp(V, V, V2S2);
    N = __vmaddfp(AbsV, HalfPi, N);
    D = __vmaddcfp(AbsV, D, V2S2);
    N = __vmulfp(N, V);
    D = __vrefp(D);

    Result = __vmulfp(N, D);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorATan2Est
(
    FXMVECTOR Y, 
    FXMVECTOR X
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Reciprocal;
    XMVECTOR V;
    XMVECTOR YSign;
    XMVECTOR Pi, PiOverTwo, PiOverFour, ThreePiOverFour;
    XMVECTOR YEqualsZero, XEqualsZero, XIsPositive, YEqualsInfinity, XEqualsInfinity;
    XMVECTOR ATanResultValid;
    XMVECTOR R0, R1, R2, R3, R4, R5;
    XMVECTOR Zero;
    XMVECTOR Result;
    static CONST XMVECTOR ATan2Constants = {XM_PI, XM_PIDIV2, XM_PIDIV4, XM_PI * 3.0f / 4.0f};

    Zero = XMVectorZero();
    ATanResultValid = XMVectorTrueInt();

    Pi = XMVectorSplatX(ATan2Constants);
    PiOverTwo = XMVectorSplatY(ATan2Constants);
    PiOverFour = XMVectorSplatZ(ATan2Constants);
    ThreePiOverFour = XMVectorSplatW(ATan2Constants);

    YEqualsZero = XMVectorEqual(Y, Zero);
    XEqualsZero = XMVectorEqual(X, Zero);
    XIsPositive = XMVectorAndInt(X, g_XMNegativeZero.v);
    XIsPositive = XMVectorEqualInt(XIsPositive, Zero);
    YEqualsInfinity = XMVectorIsInfinite(Y);
    XEqualsInfinity = XMVectorIsInfinite(X);

    YSign = XMVectorAndInt(Y, g_XMNegativeZero.v);
    Pi = XMVectorOrInt(Pi, YSign);
    PiOverTwo = XMVectorOrInt(PiOverTwo, YSign);
    PiOverFour = XMVectorOrInt(PiOverFour, YSign);
    ThreePiOverFour = XMVectorOrInt(ThreePiOverFour, YSign);

    R1 = XMVectorSelect(Pi, YSign, XIsPositive);
    R2 = XMVectorSelect(ATanResultValid, PiOverTwo, XEqualsZero);
    R3 = XMVectorSelect(R2, R1, YEqualsZero);
    R4 = XMVectorSelect(ThreePiOverFour, PiOverFour, XIsPositive);
    R5 = XMVectorSelect(PiOverTwo, R4, XEqualsInfinity);
    Result = XMVectorSelect(R3, R5, YEqualsInfinity);
    ATanResultValid = XMVectorEqualInt(Result, ATanResultValid);

    Reciprocal = XMVectorReciprocalEst(X);
    V = XMVectorMultiply(Y, Reciprocal);
    R0 = XMVectorATanEst(V);

    R1 = XMVectorSelect( Pi, Zero, XIsPositive );
    R2 = XMVectorAdd(R0, R1);

    Result = XMVectorSelect(Result, R2, ATanResultValid);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Reciprocal;
    XMVECTOR V;
    XMVECTOR YSign;
    XMVECTOR OneInt;
    XMVECTOR Shift, InfinityShifted, XIsPositive;
    XMVECTOR Pi, PiOverTwo, PiOverFour, ThreePiOverFour;
    XMVECTOR AbsV, V2S2, N, D;
    XMVECTOR HalfPi;
    XMVECTOR FinalResult;
    XMVECTOR R0, R1, R2, R3, R4, R5, R6, R7, R8;
    XMVECTOR SignMask;
    XMVECTOR Zero;
    static CONST XMVECTOR ATan2Constants = {XM_PI, XM_PIDIV2, XM_PIDIV4, XM_PI * 3.0f / 4.0f};

    SignMask = __vspltisw(-1);

    Reciprocal = __vrefp(X);

    Pi = __vspltw(ATan2Constants, 0);
    PiOverTwo = __vspltw(ATan2Constants, 1);
    PiOverFour = __vspltw(ATan2Constants, 2);
    ThreePiOverFour = __vspltw(ATan2Constants, 3);

    SignMask = __vslw(SignMask, SignMask);

    OneInt = __vspltisw(1);
    FinalResult = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    Zero = __vspltisw(0);

    R1 = __vand(X, SignMask);
    R6 = __vslw(Y, OneInt); // YShifted
    R5 = __vslw(X, OneInt); // XShifted
    InfinityShifted = __vslw(FinalResult, Shift);

    YSign = __vand(Y, SignMask);

    V = __vmulfp(Y, Reciprocal);

    R4 = __vcmpequw(R1, Zero);
    XIsPositive = __vcmpequw(R1, Zero);
    R2 = __vcmpeqfp(X, Zero); // XEqualsZero
    R3 = __vcmpeqfp(Y, Zero); // YEqualsZero

    V2S2 = __vspltw(g_XMATanEstCoefficients.v, 2);
    AbsV = __vandc(V, SignMask);
    HalfPi = __vspltw(g_XMATanEstCoefficients.v, 3);
    N = __vspltw(g_XMATanEstCoefficients.v, 0);
    D = __vspltw(g_XMATanEstCoefficients.v, 1);

    R7 = __vcmpequw(R6, InfinityShifted); // YEqualsInfinity
    R6 = __vcmpequw(R6, InfinityShifted);
    R8 = __vcmpequw(R5, InfinityShifted); // XEqualsInfinity
    R5 = __vcmpequw(R5, InfinityShifted);

    V2S2 = __vmaddfp(V, V, V2S2);
    N = __vmaddfp(AbsV, HalfPi, N);

    Pi = __vor(Pi, YSign);
    PiOverTwo = __vor(PiOverTwo, YSign);
    PiOverFour = __vor(PiOverFour, YSign);
    ThreePiOverFour = __vor(ThreePiOverFour, YSign);

    D = __vmaddcfp(AbsV, D, V2S2);

    R1 = __vsel(Pi, YSign, XIsPositive);
    R2 = __vsel(FinalResult, PiOverTwo, R2);
    R4 = __vsel(ThreePiOverFour, PiOverFour, R4);

    N = __vmulfp(N, V);
    D = __vrefp(D);

    R3 = __vsel(R2, R1, R3);
    R5 = __vsel(PiOverTwo, R4, R5);
    R6 = __vsel(R3, R5, R6);
    R7 = __vsel(R1, R6, R7);

    R0 = __vmulfp(N, D);

    R8 = __vsel(R6, R7, R8);
    FinalResult = __vcmpequw(R8, FinalResult);

    R1 = __vsel(Pi, Zero, XIsPositive);
    R0 = __vaddfp(R0, R1);
    
    FinalResult = __vsel(R8, R0, FinalResult);

    return FinalResult;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorLerp
(
    FXMVECTOR V0, 
    FXMVECTOR V1, 
    FLOAT    t
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Scale;
    XMVECTOR Length;
    XMVECTOR Result;

    // V0 + t * (V1 - V0)
    Scale = XMVectorReplicate(t);
    Length = XMVectorSubtract(V1, V0);
    Result = XMVectorMultiplyAdd(Length, Scale, V0);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR Scale;
    XMVECTOR Length;
    XMVECTOR Result;

    Length = __vsubfp(V1, V0);

    Scale.vector4_f32[0] = 
    Scale.vector4_f32[1] = 
    Scale.vector4_f32[2] = 
    Scale.vector4_f32[3] = t;

    Result = __vmaddcfp(Length, Scale, V0);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Lerp;
    XMVECTOR Length;

    XMASSERT((((UINT_PTR)&t) & 3) == 0);

    Length = __vsubfp(V1, V0);
    Lerp = __lvlx(&t, 0);
    Lerp = __vspltw(Lerp, 0);
    Lerp = __vmaddcfp(Length, Lerp, V0);

    return Lerp;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorLerpV
(
    FXMVECTOR V0, 
    FXMVECTOR V1, 
    FXMVECTOR T
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Length;
    XMVECTOR Result;

    // V0 + T * (V1 - V0)
    Length = XMVectorSubtract(V1, V0);
    Result = XMVectorMultiplyAdd(Length, T, V0);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;

    Result = __vsubfp(V1, V0);
    Result = __vmaddcfp(T, Result, V0);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorHermite
(
    FXMVECTOR Position0, 
    FXMVECTOR Tangent0, 
    FXMVECTOR Position1, 
    CXMVECTOR Tangent1, 
    FLOAT    t
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR P0;
    XMVECTOR T0;
    XMVECTOR P1;
    XMVECTOR T1;
    XMVECTOR Result;
    FLOAT    t2;
    FLOAT    t3;

    // Result = (2 * t^3 - 3 * t^2 + 1) * Position0 +
    //          (t^3 - 2 * t^2 + t) * Tangent0 +
    //          (-2 * t^3 + 3 * t^2) * Position1 +
    //          (t^3 - t^2) * Tangent1
    t2 = t * t;
    t3 = t * t2;

    P0 = XMVectorReplicate(2.0f * t3 - 3.0f * t2 + 1.0f);
    T0 = XMVectorReplicate(t3 - 2.0f * t2 + t);
    P1 = XMVectorReplicate(-2.0f * t3 + 3.0f * t2);
    T1 = XMVectorReplicate(t3 - t2);

    Result = XMVectorMultiply(P0, Position0);
    Result = XMVectorMultiplyAdd(T0, Tangent0, Result);
    Result = XMVectorMultiplyAdd(P1, Position1, Result);
    Result = XMVectorMultiplyAdd(T1, Tangent1, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR P0;
    XMVECTOR T0;
    XMVECTOR P1;
    XMVECTOR T1;
    XMVECTOR Result;
    FLOAT    t2;
    FLOAT    t3;
    FLOAT    t2x3;
    FLOAT    t3x2;

    t2 = t * t;
    t3 = t * t2;
    t2x3 = t2 * 3.0f;
    t3x2 = t3 + t3;

    P0.vector4_f32[0] = 
    P0.vector4_f32[1] = 
    P0.vector4_f32[2] = 
    P0.vector4_f32[3] = t3x2 - t2x3 + 1.0f;

    T0.vector4_f32[0] = 
    T0.vector4_f32[1] = 
    T0.vector4_f32[2] = 
    T0.vector4_f32[3] = t3 - (t2 + t2) + t;

    P1.vector4_f32[0] = 
    P1.vector4_f32[1] = 
    P1.vector4_f32[2] = 
    P1.vector4_f32[3] = -t3x2 + t2x3;

    T1.vector4_f32[0] = 
    T1.vector4_f32[1] = 
    T1.vector4_f32[2] = 
    T1.vector4_f32[3] = t3 - t2;

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(T0, Tangent0, Result);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(T1, Tangent1, Result);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR P0;
    XMVECTOR T0;
    XMVECTOR P1;
    XMVECTOR T1;
    XMVECTOR T;
    XMVECTOR TS, TC, S0, S1, PT;
    XMVECTOR Result;
    static CONST XMVECTOR C0 = {-2.0f, 3.0f, 2.0f, -3.0f};

    XMASSERT((((UINT_PTR)&t) & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(S1);

    PT = __vspltisw(0);

    T = __lvlx(&t, 0);
    PT = __vrlimi(PT, T, 0x4, 3);
    T = __vspltw(T, 0);

    TS = __vmulfp(T, T);

    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S1 = __vspltw(S1, 3);
    PT = __vrlimi(PT, S1, 0x8, 0);
    S0 = __vsldoi(C0, S1, 1 << 2);

    PT = __vnmsubfp(TS, S0, PT);
    TC = __vmulfp(T, TS);
    S1 = __vrlimi(S1, C0, 0xA, 2);

    PT = __vmaddfp(TC, S1, PT);

    P0 = __vspltw(PT, 0);
    T0 = __vspltw(PT, 1);
    P1 = __vspltw(PT, 2);
    T1 = __vspltw(PT, 3);

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(T0, Tangent0, Result);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(T1, Tangent1, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorHermiteV
(
    FXMVECTOR Position0, 
    FXMVECTOR Tangent0, 
    FXMVECTOR Position1, 
    CXMVECTOR Tangent1, 
    CXMVECTOR T
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR P0;
    XMVECTOR T0;
    XMVECTOR P1;
    XMVECTOR T1;
    XMVECTOR Result;
    XMVECTOR T2;
    XMVECTOR T3;

    // Result = (2 * t^3 - 3 * t^2 + 1) * Position0 +
    //          (t^3 - 2 * t^2 + t) * Tangent0 +
    //          (-2 * t^3 + 3 * t^2) * Position1 +
    //          (t^3 - t^2) * Tangent1
    T2 = XMVectorMultiply(T, T);
    T3 = XMVectorMultiply(T , T2);

    P0 = XMVectorReplicate(2.0f * T3.vector4_f32[0] - 3.0f * T2.vector4_f32[0] + 1.0f);
    T0 = XMVectorReplicate(T3.vector4_f32[1] - 2.0f * T2.vector4_f32[1] + T.vector4_f32[1]);
    P1 = XMVectorReplicate(-2.0f * T3.vector4_f32[2] + 3.0f * T2.vector4_f32[2]);
    T1 = XMVectorReplicate(T3.vector4_f32[3] - T2.vector4_f32[3]);

    Result = XMVectorMultiply(P0, Position0);
    Result = XMVectorMultiplyAdd(T0, Tangent0, Result);
    Result = XMVectorMultiplyAdd(P1, Position1, Result);
    Result = XMVectorMultiplyAdd(T1, Tangent1, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR P0;
    XMVECTOR T0;
    XMVECTOR P1;
    XMVECTOR T1;
    XMVECTOR TS, TC, S0, S1, PT;
    XMVECTOR Result;
    static CONST XMVECTOR C0 = {-2.0f, 3.0f, 2.0f, -3.0f};

    XMDUMMY_INITIALIZE_VECTOR(S1);

    PT = __vspltisw(0);

    PT = __vrlimi(PT, T, 0x4, 0);

    TS = __vmulfp(T, T);

    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S1 = __vspltw(S1, 3);
    PT = __vrlimi(PT, S1, 0x8, 0);
    S0 = __vsldoi(C0, S1, 1 << 2);

    PT = __vnmsubfp(TS, S0, PT);
    TC = __vmulfp(T, TS);
    S1 = __vrlimi(S1, C0, 0xA, 2);

    PT = __vmaddfp(TC, S1, PT);

    P0 = __vspltw(PT, 0);
    T0 = __vspltw(PT, 1);
    P1 = __vspltw(PT, 2);
    T1 = __vspltw(PT, 3);

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(T0, Tangent0, Result);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(T1, Tangent1, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorCatmullRom
(
    FXMVECTOR Position0, 
    FXMVECTOR Position1, 
    FXMVECTOR Position2, 
    CXMVECTOR Position3, 
    FLOAT    t
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR P0;
    XMVECTOR P1;
    XMVECTOR P2;
    XMVECTOR P3;
    XMVECTOR Result;
    FLOAT    t2;
    FLOAT    t3;

    // Result = ((-t^3 + 2 * t^2 - t) * Position0 +
    //           (3 * t^3 - 5 * t^2 + 2) * Position1 +
    //           (-3 * t^3 + 4 * t^2 + t) * Position2 +
    //           (t^3 - t^2) * Position3) * 0.5
    t2 = t * t;
    t3 = t * t2;

    P0 = XMVectorReplicate((-t3 + 2.0f * t2 - t) * 0.5f);
    P1 = XMVectorReplicate((3.0f * t3 - 5.0f * t2 + 2.0f) * 0.5f);
    P2 = XMVectorReplicate((-3.0f * t3 + 4.0f * t2 + t) * 0.5f);
    P3 = XMVectorReplicate((t3 - t2) * 0.5f);

    Result = XMVectorMultiply(P0, Position0);
    Result = XMVectorMultiplyAdd(P1, Position1, Result);
    Result = XMVectorMultiplyAdd(P2, Position2, Result);
    Result = XMVectorMultiplyAdd(P3, Position3, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR P0;
    XMVECTOR P1;
    XMVECTOR P2;
    XMVECTOR P3;
    XMVECTOR Result;
    FLOAT    t2;
    FLOAT    t3;

    t2 = t * t;
    t3 = t * t2;

    P0.vector4_f32[0] = 
    P0.vector4_f32[1] = 
    P0.vector4_f32[2] = 
    P0.vector4_f32[3] = (-t3 + 2.0f * t2 - t) * 0.5f;

    P1.vector4_f32[0] = 
    P1.vector4_f32[1] = 
    P1.vector4_f32[2] = 
    P1.vector4_f32[3] = (3.0f * t3 - 5.0f * t2 + 2.0f) * 0.5f;

    P2.vector4_f32[0] = 
    P2.vector4_f32[1] = 
    P2.vector4_f32[2] = 
    P2.vector4_f32[3] = (-3.0f * t3 + 4.0f * t2 + t) * 0.5f;

    P3.vector4_f32[0] = 
    P3.vector4_f32[1] = 
    P3.vector4_f32[2] = 
    P3.vector4_f32[3] = (t3 - t2) * 0.5f;

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(P2, Position2, Result);
    Result = __vmaddfp(P3, Position3, Result);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR P0, P1, P2, P3;
    XMVECTOR S0, S1, S2, S3;
    XMVECTOR PT, T, TS, TC, Zero, One;
    XMVECTOR Result;
    static CONST XMVECTOR C0 = {-2.5f, 1.5f, -2.0f, -0.5f};

    XMASSERT((((UINT_PTR)&t) & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(One);

    T = __lvlx(&t, 0);
    T = __vspltw(T, 0);
    PT = __vspltw(T, 0);

    TS = __vmulfp(T, T);

    One = __vupkd3d(One, VPACK_NORMSHORT2);
    One = __vspltw(One, 3);
    S2 = __vspltisw(-1);
    Zero = __vspltisw(0);

    PT = __vrlimi(PT, One, 0x1, 0);

    S0 = __vspltw(C0, 3);
    S1 = __vpermwi(C0, 0x40);
    S2 = __vslw(S2, S2);
    S3 = __vspltisw(0);

    TC = __vmulfp(T, TS);
    PT = __vrlimi(PT, TS, 0x4, 0);

    S0 = __vsldoi(S0, Zero, 1 << 2);
    S1 = __vrlimi(S1, Zero, 0x2, 0);
    S3 = __vrlimi(S3, C0, 0x4, 2);
    S2 = __vxor(C0, S2);

    S0 = __vrlimi(S0, One, 0x4, 0); // <-0.5, 1.0, -0.5, 0.0>
    S1 = __vrlimi(S1, One, 0x1, 0); // <1.5, -2.5, 0.0, 1.0>
    S3 = __vrlimi(S3, S2, 0x8, 3); // <0.5, -0.5, 0.0, 0.0>
    S2 = __vsldoi(S2, Zero, 1 << 2); // <-1.5, 2.0, 0.5, 0.0>

    PT = __vrlimi(PT, TC, 0x8, 0);

    P0 = __vmsum4fp(PT, S0);
    P1 = __vmsum4fp(PT, S1);
    P2 = __vmsum4fp(PT, S2);
    P3 = __vmsum4fp(PT, S3);

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(P2, Position2, Result);
    Result = __vmaddfp(P3, Position3, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorCatmullRomV
(
    FXMVECTOR Position0, 
    FXMVECTOR Position1, 
    FXMVECTOR Position2, 
    CXMVECTOR Position3, 
    CXMVECTOR T
)
{
#if defined(_XM_NO_INTRINSICS_)
    float fx = T.vector4_f32[0];
    float fy = T.vector4_f32[1];
    float fz = T.vector4_f32[2];
    float fw = T.vector4_f32[3];
    XMVECTOR vResult = {
        0.5f*((-fx*fx*fx+2*fx*fx-fx)*Position0.vector4_f32[0]+
        (3*fx*fx*fx-5*fx*fx+2)*Position1.vector4_f32[0]+
        (-3*fx*fx*fx+4*fx*fx+fx)*Position2.vector4_f32[0]+
        (fx*fx*fx-fx*fx)*Position3.vector4_f32[0]),
        0.5f*((-fy*fy*fy+2*fy*fy-fy)*Position0.vector4_f32[1]+
        (3*fy*fy*fy-5*fy*fy+2)*Position1.vector4_f32[1]+
        (-3*fy*fy*fy+4*fy*fy+fy)*Position2.vector4_f32[1]+
        (fy*fy*fy-fy*fy)*Position3.vector4_f32[1]),
        0.5f*((-fz*fz*fz+2*fz*fz-fz)*Position0.vector4_f32[2]+
        (3*fz*fz*fz-5*fz*fz+2)*Position1.vector4_f32[2]+
        (-3*fz*fz*fz+4*fz*fz+fz)*Position2.vector4_f32[2]+
        (fz*fz*fz-fz*fz)*Position3.vector4_f32[2]),
        0.5f*((-fw*fw*fw+2*fw*fw-fw)*Position0.vector4_f32[3]+
        (3*fw*fw*fw-5*fw*fw+2)*Position1.vector4_f32[3]+
        (-3*fw*fw*fw+4*fw*fw+fw)*Position2.vector4_f32[3]+
        (fw*fw*fw-fw*fw)*Position3.vector4_f32[3])
    };
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR P0;
    XMVECTOR P1;
    XMVECTOR P2;
    XMVECTOR P3;
    XMVECTOR T2;
    XMVECTOR T3;
    XMVECTOR NegT3;
    XMVECTOR Half;
    XMVECTOR Two;
    XMVECTOR NegThree;
    XMVECTOR Five;
    XMVECTOR SignMask;
    XMVECTOR Result;

    Half = __vspltisw(1);
    Two = __vspltisw(2);
    P1 = __vspltisw(3);
    NegThree = __vspltisw(-3);
    P2 = __vspltisw(4);
    Five = __vspltisw(5);
    SignMask = __vspltisw(-1);
    NegT3 = __vspltisw(0);

    T2 = __vmulfp(T, T);

    Half = __vcfsx(Half, 1);
    SignMask = __vslw(SignMask, SignMask);
    Two = __vcfsx(Two, 0);
    P1 = __vcfsx(P1, 0);
    NegThree = __vcfsx(NegThree, 0);
    P2 = __vcfsx(P2, 0);
    P0 = __vxor(T, SignMask);
    Five = __vcfsx(Five, 0);

    NegT3 = __vnmsubfp(T , T2, NegT3);
    T3 = __vmulfp(T , T2);

    P0 = __vmaddfp(T2, Two, P0);
    Two = __vnmsubfp(Five, T2, Two);
    P2 = __vmaddcfp(T2, P2, T);

    P0 = __vaddfp(P0, NegT3);
    P1 = __vmaddcfp(T3, P1, Two);
    P2 = __vmaddfp(T3, NegThree, P2);
    P3 = __vsubfp(T3, T2);

    P0 = __vmulfp(P0, Half);
    P1 = __vmulfp(P1, Half);
    P2 = __vmulfp(P2, Half);
    P3 = __vmulfp(P3, Half);

    Result = __vmulfp(P0, Position0);
    Result = __vmaddfp(P1, Position1, Result);
    Result = __vmaddfp(P2, Position2, Result);
    Result = __vmaddfp(P3, Position3, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorBaryCentric
(
    FXMVECTOR Position0, 
    FXMVECTOR Position1, 
    FXMVECTOR Position2, 
    FLOAT    f, 
    FLOAT    g
)
{
#if defined(_XM_NO_INTRINSICS_)

    // Result = Position0 + f * (Position1 - Position0) + g * (Position2 - Position0)
    XMVECTOR P10;
    XMVECTOR P20;
    XMVECTOR ScaleF;
    XMVECTOR ScaleG;
    XMVECTOR Result;

    P10 = XMVectorSubtract(Position1, Position0);
    ScaleF = XMVectorReplicate(f);

    P20 = XMVectorSubtract(Position2, Position0);
    ScaleG = XMVectorReplicate(g);

    Result = XMVectorMultiplyAdd(P10, ScaleF, Position0);
    Result = XMVectorMultiplyAdd(P20, ScaleG, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR P10;
    XMVECTOR P20;
    XMVECTOR ScaleF;
    XMVECTOR ScaleG;
    XMVECTOR Result;

    P10 = __vsubfp(Position1, Position0);
    P20 = __vsubfp(Position2, Position0);

    ScaleF.vector4_f32[0] = 
    ScaleF.vector4_f32[1] = 
    ScaleF.vector4_f32[2] = 
    ScaleF.vector4_f32[3] = f;

    ScaleG.vector4_f32[0] = 
    ScaleG.vector4_f32[1] = 
    ScaleG.vector4_f32[2] = 
    ScaleG.vector4_f32[3] = g;

    Result = __vmaddfp(P10, ScaleF, Position0);
    Result = __vmaddfp(P20, ScaleG, Result);

    return Result;

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR P10;
    XMVECTOR P20;
    XMVECTOR ScaleF;
    XMVECTOR ScaleG;
    XMVECTOR Result;

    XMASSERT((((UINT_PTR)&f) & 3) == 0);
    XMASSERT((((UINT_PTR)&g) & 3) == 0);

    P10 = __vsubfp(Position1, Position0);
    P20 = __vsubfp(Position2, Position0);

    ScaleG = __lvlx(&g, 0);
    ScaleF = __lvlx(&f, 0);
    ScaleG = __vspltw(ScaleG, 0);
    Result = __vspltw(ScaleF, 0);

    Result = __vmaddcfp(P10, Result, Position0);
    Result = __vmaddfp(P20, ScaleG, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVectorBaryCentricV
(
    FXMVECTOR Position0, 
    FXMVECTOR Position1, 
    FXMVECTOR Position2, 
    CXMVECTOR F, 
    CXMVECTOR G
)
{
#if defined(_XM_NO_INTRINSICS_)

    // Result = Position0 + f * (Position1 - Position0) + g * (Position2 - Position0)
    XMVECTOR P10;
    XMVECTOR P20;
    XMVECTOR Result;

    P10 = XMVectorSubtract(Position1, Position0);
    P20 = XMVectorSubtract(Position2, Position0);

    Result = XMVectorMultiplyAdd(P10, F, Position0);
    Result = XMVectorMultiplyAdd(P20, G, Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR P20;
    XMVECTOR Result;

    Result = __vsubfp(Position1, Position0);
    P20 = __vsubfp(Position2, Position0);

    Result = __vmaddcfp(F, Result, Position0);
    Result = __vmaddfp(G, P20, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

/****************************************************************************
 *
 * 2D Vector
 *
 ****************************************************************************/

//------------------------------------------------------------------------------
// Comparison operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2Equal
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] == V2.vector4_f32[0]) && (V1.vector4_f32[1] == V2.vector4_f32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2EqualR(V1, V2));
#endif
}


//------------------------------------------------------------------------------

XMFINLINE UINT XMVector2EqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;

    if ((V1.vector4_f32[0] == V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] == V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] != V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] != V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x11);
    C2 = __vpermwi(V2, 0x11);

    __vcmpeqfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2EqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] == V2.vector4_u32[0]) && (V1.vector4_u32[1] == V2.vector4_u32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector2EqualIntR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;
    if ((V1.vector4_u32[0] == V2.vector4_u32[0]) && 
        (V1.vector4_u32[1] == V2.vector4_u32[1]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_u32[0] != V2.vector4_u32[0]) && 
        (V1.vector4_u32[1] != V2.vector4_u32[1]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x11);
    C2 = __vpermwi(V2, 0x11);

    __vcmpequwR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2NearEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Epsilon
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT dx, dy;
    dx = fabsf(V1.vector4_f32[0]-V2.vector4_f32[0]);
    dy = fabsf(V1.vector4_f32[1]-V2.vector4_f32[1]);
    return ((dx <= Epsilon.vector4_f32[0]) &&
            (dy <= Epsilon.vector4_f32[1]));
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    Delta;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    Delta = __vsubfp(V1, V2);
    Delta = __vrlimi(Delta, Epsilon, 0x3, 0);
    __vcmpbfpR(Delta, Epsilon, &CR);

    return XMComparisonAllInBounds(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2NotEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] != V2.vector4_f32[0]) || (V1.vector4_f32[1] != V2.vector4_f32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAnyFalse(XMVector2EqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2NotEqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] != V2.vector4_u32[0]) || (V1.vector4_u32[1] != V2.vector4_u32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAnyFalse(XMVector2EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2Greater
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] > V2.vector4_f32[0]) && (V1.vector4_f32[1] > V2.vector4_f32[1])) != 0);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2GreaterR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector2GreaterR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;
    if ((V1.vector4_f32[0] > V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] > V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] <= V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] <= V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x11);
    C2 = __vpermwi(V2, 0x11);

    __vcmpgtfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2GreaterOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] >= V2.vector4_f32[0]) && (V1.vector4_f32[1] >= V2.vector4_f32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2GreaterOrEqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector2GreaterOrEqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V1.vector4_f32[0] >= V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] >= V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] < V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] < V2.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x11);
    C2 = __vpermwi(V2, 0x11);

    __vcmpgefpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2Less
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] < V2.vector4_f32[0]) && (V1.vector4_f32[1] < V2.vector4_f32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2GreaterR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2LessOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] <= V2.vector4_f32[0]) && (V1.vector4_f32[1] <= V2.vector4_f32[1])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector2GreaterOrEqualR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2InBounds
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
 #if defined(_XM_NO_INTRINSICS_)
    return (((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1])) != 0);
 #elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_   
    return XMComparisonAllInBounds(XMVector2InBoundsR(V, Bounds));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector2InBoundsR
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]))
    {
        CR = XM_CRMASK_CR6BOUNDS;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V, 0x11);
    C2 = __vpermwi(Bounds, 0x11);

    __vcmpbfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2IsNaN
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (XMISNAN(V.vector4_f32[0]) ||
            XMISNAN(V.vector4_f32[1]));
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C = __vpermwi(V, 0x11);

    __vcmpeqfpR(C, C, &CR);

    return XMComparisonAnyFalse(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector2IsInfinite
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    return (XMISINF(V.vector4_f32[0]) ||
            XMISINF(V.vector4_f32[1]));
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR OneInt, TrueInt;
    XMVECTOR XYXY, VShifted, InfinityShifted;
    XMVECTOR Shift;
    UINT     CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    XYXY = __vpermwi(V, 0x11);
    OneInt = __vspltisw(1);
    TrueInt = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    VShifted = __vslw(XYXY, OneInt);
    InfinityShifted = __vslw(TrueInt, Shift);

    __vcmpequwR(VShifted, InfinityShifted, &CR);

    return XMComparisonAnyTrue(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Computation operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Dot
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] =
    Result.vector4_f32[1] =
    Result.vector4_f32[2] =
    Result.vector4_f32[3] = V1.vector4_f32[0] * V2.vector4_f32[0] + V1.vector4_f32[1] * V2.vector4_f32[1];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Overflow will result in a QNaN instead of +/- infinity.

    XMVECTOR D1, D2, Zero;
    XMVECTOR Result;

    Zero = __vspltisw(0);
    D1 = __vsldoi(Zero, V1, 2 << 2);
    D2 = __vsldoi(Zero, V2, 2 << 2);

    Result = __vmsum4fp(D1, D2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Cross
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fCross = (V1.vector4_f32[0] * V2.vector4_f32[1]) - (V1.vector4_f32[1] * V2.vector4_f32[0]);
    XMVECTOR vResult = { 
        fCross,
        fCross,
        fCross,
        fCross
    };
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR CrossP;
    XMVECTOR CrossR;
    XMVECTOR Result;

    CrossP = __vpermwi(V2, 0x4B);
    CrossR = __vmulfp(V1, CrossP);
    CrossP = __vpermwi(CrossR, 0x4B);
    CrossR = __vsubfp(CrossR, CrossP);
    Result = __vspltw(CrossR, 0);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2LengthSq
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    return XMVector2Dot(V, V);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMVector2Dot(V, V);
#endif
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2ReciprocalLengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector2LengthSq(V);
    Result = XMVectorReciprocalSqrtEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Zero;
    XMVECTOR Result;

    Zero = __vspltisw(0);
    Result = __vsldoi(Zero, V, 2 << 2);
    Result = __vmsum4fp(Result, Result);
    Result = __vrsqrtefp(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2ReciprocalLength
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector2LengthSq(V);
    Result = XMVectorReciprocalSqrt(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Zero;
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR H;

    Zero = __vspltisw(0);
    D = __vsldoi(Zero, V, 2 << 2);
    D = __vmsum4fp(D, D);
    H = __vspltisw(1);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);

    return Rsq;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2LengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result;
    Result = XMVector2LengthSq(V);
    Result = XMVectorSqrtEst(Result);
    return Result;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Zero;
    XMVECTOR Result;
    // Returns a QNaN on infinite vectors.
    XMVECTOR Length;
    Zero = __vspltisw(0);
    D = __vsldoi(Zero, V, 2 << 2);
    D = __vmsum4fp(D, D);
    Rsq = __vrsqrtefp(D);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Length
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result = XMVector2LengthSq(V);
    Result = XMVectorSqrt(Result);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR Length;
    XMVECTOR H;

    Zero = __vspltisw(0);
    D = __vsldoi(Zero, V, 2 << 2);
    H = __vspltisw(1);
    D = __vmsum4fp(D, D);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// XMVector2NormalizeEst uses a reciprocal estimate and
// returns QNaN on zero and infinite vectors.

XMFINLINE XMVECTOR XMVector2NormalizeEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result = XMVector2ReciprocalLength(V);
    Result = XMVectorMultiply(V, Result);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Zero;
    XMVECTOR Result;
    // Faster, but returns a QNaN on zero and infinite vectors.
    Zero = __vspltisw(0);
    D = __vsldoi(Zero, V, 2 << 2);
    D = __vmsum4fp(D, D);
    Rsq = __vrsqrtefp(D);
    Result = __vmulfp(V, Rsq);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Normalize
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fLength;
    XMVECTOR vResult;

    vResult = XMVector2Length( V );
    fLength = vResult.vector4_f32[0];

    // Prevent divide by zero
    if (fLength > 0) {
        fLength = 1.0f/fLength;
    }
    
    vResult.vector4_f32[0] = V.vector4_f32[0]*fLength;
    vResult.vector4_f32[1] = V.vector4_f32[1]*fLength;
    vResult.vector4_f32[2] = V.vector4_f32[2]*fLength;
    vResult.vector4_f32[3] = V.vector4_f32[3]*fLength;
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR RcpLength;
    XMVECTOR OneHalfLengthSq;
    XMVECTOR Scale;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR H;
    XMVECTOR N;

    Zero = __vspltisw(0);
    D = __vsldoi(Zero, V, 2 << 2);
    H = __vspltisw(1);
    D = __vmsum4fp(D, D);
    H = __vcfsx(H, 1);
    RcpLength = __vrsqrtefp(D);
    OneHalfLengthSq = __vmulfp(D, H);
    Result = __vcmpeqfp(D, Zero);
    Scale = __vmulfp(RcpLength, RcpLength);
    H = __vnmsubfp(OneHalfLengthSq, Scale, H);
    H = __vmaddcfp(RcpLength, H, RcpLength);
    N = __vmulfp(V, H);
    Result = __vsel(N, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2ClampLength
(
    FXMVECTOR V, 
    FLOAT    LengthMin, 
    FLOAT    LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax = XMVectorReplicate(LengthMax);
    ClampMin = XMVectorReplicate(LengthMin);

    return XMVector2ClampLengthV(V, ClampMin, ClampMax);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax.vector4_f32[0] =
    ClampMax.vector4_f32[1] = 
    ClampMax.vector4_f32[2] =
    ClampMax.vector4_f32[3] = LengthMax;

    ClampMin.vector4_f32[0] =
    ClampMin.vector4_f32[1] = 
    ClampMin.vector4_f32[2] =
    ClampMin.vector4_f32[3] = LengthMin;

    return XMVector2ClampLengthV(V, ClampMin, ClampMax);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    XMASSERT(((UINT_PTR)&LengthMin & 3) == 0);
    XMASSERT(((UINT_PTR)&LengthMax & 3) == 0);

    ClampMax = __lvlx(&LengthMax, 0);
    ClampMin = __lvlx(&LengthMin, 0);

    ClampMax = __vspltw(ClampMax, 0);
    ClampMin = __vspltw(ClampMin, 0);

    return XMVector2ClampLengthV(V, ClampMin, ClampMax);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2ClampLengthV
(
    FXMVECTOR V, 
    FXMVECTOR LengthMin, 
    FXMVECTOR LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR RcpLength;
    XMVECTOR Length;
    XMVECTOR Normal;
    XMVECTOR Zero;
    XMVECTOR InfiniteLength;
    XMVECTOR ZeroLength;
    XMVECTOR Select;
    XMVECTOR ControlMax;
    XMVECTOR ControlMin;
    XMVECTOR Control;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector2GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector2GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector2GreaterOrEqual(LengthMax, LengthMin));

    LengthSq = XMVector2LengthSq(V);

    Zero = XMVectorZero();

    RcpLength = XMVectorReciprocalSqrt(LengthSq);

    InfiniteLength = XMVectorEqualInt(LengthSq, g_XMInfinity.v);
    ZeroLength = XMVectorEqual(LengthSq, Zero);

    Length = XMVectorMultiply(LengthSq, RcpLength);

    Normal = XMVectorMultiply(V, RcpLength);

    Select = XMVectorEqualInt(InfiniteLength, ZeroLength);
    Length = XMVectorSelect(LengthSq, Length, Select);
    Normal = XMVectorSelect(LengthSq, Normal, Select);

    ControlMax = XMVectorGreater(Length, LengthMax);
    ControlMin = XMVectorLess(Length, LengthMin);

    ClampLength = XMVectorSelect(Length, LengthMax, ControlMax);
    ClampLength = XMVectorSelect(ClampLength, LengthMin, ControlMin);

    Result = XMVectorMultiply(Normal, ClampLength);

    // Preserve the original vector (with no precision loss) if the length falls within the given range
    Control = XMVectorEqualInt(ControlMax, ControlMin);
    Result = XMVectorSelect(Result, V, Control);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR Length, LengthS;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Normal, NormalS;
    XMVECTOR Zero;
    XMVECTOR Control;
    XMVECTOR H;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector2GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector2GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector2GreaterOrEqual(LengthMax, LengthMin));

    Zero = __vspltisw(0);

    D = __vsldoi(Zero, V, 2 << 2);
    H = __vspltisw(1);
    LengthSq = __vmsum4fp(D, D);

    H = __vcfsx(H, 1);

    Rsq = __vrsqrtefp(LengthSq);
    RT = __vmulfp(LengthSq, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    H = __vmaddcfp(Rsq, H, Rsq);

    Length = __vmulfp(LengthSq, H);

    Normal = __vmulfp(V, H);

    LengthS = __vcmpeqfp(LengthSq, Zero);
    NormalS = __vcmpeqfp(LengthSq, Zero);

    LengthS = __vsel(Length, LengthSq, LengthS);
    NormalS = __vsel(Normal, LengthSq, NormalS);

    Control = __vcmpgtfp(LengthS, LengthMax);
    ClampLength = __vcmpgtfp(LengthMin, LengthS);

    Result = __vcmpequw(Control, ClampLength);

    Control = __vsel(LengthS, LengthMax, Control);
    ClampLength = __vsel(Control, LengthMin, ClampLength);

    RT = __vmulfp(NormalS, ClampLength);

    Result = __vsel(RT, V, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Reflect
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    // Result = Incident - (2 * dot(Incident, Normal)) * Normal
    Result = XMVector2Dot(Incident, Normal);
    Result = XMVectorAdd(Result, Result);
    Result = XMVectorNegativeMultiplySubtract(Result, Normal, Incident);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D, DI, DN;
    XMVECTOR Zero;
    XMVECTOR Result;

    Zero = __vspltisw(0);
    DI = __vsldoi(Zero, Incident, 2 << 2);
    DN = __vsldoi(Zero, Normal, 2 << 2);
    D = __vmsum4fp(DI, DN);
    D = __vaddfp(D, D);
    Result = Incident;
    Result = __vnmsubfp(D, Normal, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Refract
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FLOAT    RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Index;
    Index = XMVectorReplicate(RefractionIndex);
    return XMVector2RefractV(Incident, Normal, Index);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR Index;

    Index.vector4_f32[0] =
    Index.vector4_f32[1] = 
    Index.vector4_f32[2] =
    Index.vector4_f32[3] = RefractionIndex;

    return XMVector2RefractV(Incident, Normal, Index);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Index;

    XMASSERT(((UINT_PTR)&RefractionIndex & 3) == 0);

    Index = __lvlx(&RefractionIndex, 0);
    Index = __vspltw(Index, 0);

    return XMVector2RefractV(Incident, Normal, Index);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

// Return the refraction of a 2D vector
XMFINLINE XMVECTOR XMVector2RefractV
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FXMVECTOR RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)
    float IDotN;
    float RX,RY;
    XMVECTOR vResult;
    // Result = RefractionIndex * Incident - Normal * (RefractionIndex * dot(Incident, Normal) + 
    // sqrt(1 - RefractionIndex * RefractionIndex * (1 - dot(Incident, Normal) * dot(Incident, Normal))))
    IDotN = (Incident.vector4_f32[0]*Normal.vector4_f32[0])+(Incident.vector4_f32[1]*Normal.vector4_f32[1]);
    // R = 1.0f - RefractionIndex * RefractionIndex * (1.0f - IDotN * IDotN)
    RY = 1.0f-(IDotN*IDotN);
    RX = 1.0f-(RY*RefractionIndex.vector4_f32[0]*RefractionIndex.vector4_f32[0]);
    RY = 1.0f-(RY*RefractionIndex.vector4_f32[1]*RefractionIndex.vector4_f32[1]);
    if (RX>=0.0f) {
        RX = (RefractionIndex.vector4_f32[0]*Incident.vector4_f32[0])-(Normal.vector4_f32[0]*((RefractionIndex.vector4_f32[0]*IDotN)+sqrtf(RX)));
    } else {
        RX = 0.0f;
    }
    if (RY>=0.0f) {
        RY = (RefractionIndex.vector4_f32[1]*Incident.vector4_f32[1])-(Normal.vector4_f32[1]*((RefractionIndex.vector4_f32[1]*IDotN)+sqrtf(RY)));
    } else {
        RY = 0.0f;
    }
    vResult.vector4_f32[0] = RX;
    vResult.vector4_f32[1] = RY;
    vResult.vector4_f32[2] = 0.0f;   
    vResult.vector4_f32[3] = 0.0f;
    return vResult;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR DI, DN;
    XMVECTOR IDotN;
    XMVECTOR R;
    XMVECTOR I;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR ZL;
    XMVECTOR D1;
    XMVECTOR H;
    XMVECTOR Zero;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(R);

    Zero = __vspltisw(0);
    DI = __vsldoi(Zero, Incident, 2 << 2);
    DN = __vsldoi(Zero, Normal, 2 << 2);
    IDotN = __vmsum4fp(DI, DN);

    H = __vspltisw(1);
    R = __vupkd3d(R, VPACK_NORMSHORT2);
    H = __vcfsx(H, 1);
    RT = __vspltw(R, 3);
    R = __vspltw(R, 3);

    RT = __vnmsubfp(IDotN, IDotN, RT);
    RT = __vmulfp(RT, RefractionIndex);
    R = __vnmsubfp(RT, RefractionIndex, R);

    Rsq = __vrsqrtefp(R);
    RT = __vmulfp(R, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Zero = __vspltisw(0);
    H = __vmaddcfp(Rsq, H, Rsq);

    ZL = __vcmpeqfp(R, Zero);

    Result = __vcmpgefp(Zero, R);

    D1 = __vmulfp(R, H);

    I = __vmulfp(RefractionIndex, Incident);

    ZL = __vsel(D1, R, ZL);

    ZL = __vmaddfp(RefractionIndex, IDotN, ZL);

    I = __vnmsubfp(Normal, ZL, I);

    Result = __vsel(I, Zero, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Orthogonal
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] = -V.vector4_f32[1];
    Result.vector4_f32[1] = V.vector4_f32[0];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR NegativeV;
    XMVECTOR SignMask;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    Result = __vspltw(V, 0);
    NegativeV = __vxor(V, SignMask);

    Result = __vrlimi(Result, NegativeV, 0x8, 1);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2AngleBetweenNormalsEst
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    Result = XMVector2Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACosEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D1, D2, Zero;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Zero = __vspltisw(0);
    D1 = __vsldoi(Zero, N1, 2 << 2);
    D2 = __vsldoi(Zero, N2, 2 << 2);
    Result = __vmsum4fp(D1, D2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    Result = __vmaxfp(NegativeOne, Result);
    Result = __vminfp(One, Result);

    Result = XMVectorACosEst(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2AngleBetweenNormals
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    Result = XMVector2Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACos(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D1, D2, Zero;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Zero = __vspltisw(0);
    D1 = __vsldoi(Zero, N1, 2 << 2);
    D2 = __vsldoi(Zero, N2, 2 << 2);
    Result = __vmsum4fp(D1, D2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    Result = __vmaxfp(NegativeOne, Result);
    Result = __vminfp(One, Result);

    Result = XMVectorACos(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2AngleBetweenVectors
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR L1;
    XMVECTOR L2;
    XMVECTOR Dot;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    L1 = XMVector2ReciprocalLength(V1);
    L2 = XMVector2ReciprocalLength(V2);

    Dot = XMVector2Dot(V1, V2);

    L1 = XMVectorMultiply(L1, L2);

    CosAngle = XMVectorMultiply(Dot, L1);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    CosAngle = XMVectorClamp(CosAngle, NegativeOne, One);

    Result = XMVectorACos(CosAngle);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Zero;
    XMVECTOR VS1, VS2;
    XMVECTOR D1, D2;
    XMVECTOR Rsq1, Rsq2;
    XMVECTOR Rcp1, Rcp2;
    XMVECTOR RT1, RT2;
    XMVECTOR H1, H2;
    XMVECTOR L;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Zero = __vspltisw(0);
    VS1 = __vsldoi(Zero, V1, 2 << 2);
    VS2 = __vsldoi(Zero, V2, 2 << 2);
    D1 = __vmsum4fp(VS1, VS1);
    D2 = __vmsum4fp(VS2, VS2);
    H2 = __vspltisw(1);
    H1 = __vcfsx(H2, 1);
    H2 = __vcfsx(H2, 1);
    Rsq1 = __vrsqrtefp(D1);
    Rsq2 = __vrsqrtefp(D2);
    RT1 = __vmulfp(D1, H1);
    RT2 = __vmulfp(D2, H2);
    Rcp1 = __vmulfp(Rsq1, Rsq1);
    Rcp2 = __vmulfp(Rsq2, Rsq2);
    H1 = __vnmsubfp(RT1, Rcp1, H1);
    H2 = __vnmsubfp(RT2, Rcp2, H2);
    Result = __vmsum4fp(VS1, VS2);
    Rsq1 = __vmaddfp(Rsq1, H1, Rsq1);
    Rsq2 = __vmaddfp(Rsq2, H2, Rsq2);

    L = __vmulfp(Rsq1, Rsq2);

    CosAngle = __vmulfp(Result, L);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    CosAngle = __vmaxfp(NegativeOne, CosAngle);
    CosAngle = __vminfp(One, CosAngle);

    Result = XMVectorACos(CosAngle);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2LinePointDistance
(
    FXMVECTOR LinePoint1, 
    FXMVECTOR LinePoint2, 
    FXMVECTOR Point
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR PointVector;
    XMVECTOR LineVector;
    XMVECTOR ReciprocalLengthSq;
    XMVECTOR PointProjectionScale;
    XMVECTOR DistanceVector;
    XMVECTOR Result;

    // Given a vector PointVector from LinePoint1 to Point and a vector
    // LineVector from LinePoint1 to LinePoint2, the scaled distance 
    // PointProjectionScale from LinePoint1 to the perpendicular projection
    // of PointVector onto the line is defined as:
    //
    //     PointProjectionScale = dot(PointVector, LineVector) / LengthSq(LineVector)

    PointVector = XMVectorSubtract(Point, LinePoint1);
    LineVector = XMVectorSubtract(LinePoint2, LinePoint1);

    ReciprocalLengthSq = XMVector2LengthSq(LineVector);
    ReciprocalLengthSq = XMVectorReciprocal(ReciprocalLengthSq);

    PointProjectionScale = XMVector2Dot(PointVector, LineVector);
    PointProjectionScale = XMVectorMultiply(PointProjectionScale, ReciprocalLengthSq);

    DistanceVector = XMVectorMultiply(LineVector, PointProjectionScale);
    DistanceVector = XMVectorSubtract(PointVector, DistanceVector);

    Result = XMVector2Length(DistanceVector);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR PointVector;
    XMVECTOR LineVector;
    XMVECTOR PointProjectionScale;
    XMVECTOR DistanceVector;
    XMVECTOR ReciprocalLengthSq;
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Length;
    XMVECTOR H;
    XMVECTOR LVS, PVS, Zero;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    LineVector = __vsubfp(LinePoint2, LinePoint1);

    Zero = __vspltisw(0);
    LVS = __vsldoi(Zero, LineVector, 2 << 2);

    ReciprocalLengthSq = __vmsum4fp(LVS, LVS);

    Reciprocal = __vrefp(ReciprocalLengthSq);

    PointVector = __vsubfp(Point, LinePoint1);

    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(ReciprocalLengthSq, Reciprocal, S0);

    PVS = __vsldoi(Zero, PointVector, 2 << 2);

    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);

    S1 = __vspltw(S1, 3);

    S1 = __vnmsubfp(ReciprocalLengthSq, S0, S1);

    ReciprocalLengthSq = __vcmpeqfp(S0, S0);

    PointProjectionScale = __vmsum4fp(PVS, LVS);

    S0 = __vmaddfp(S0, S1, S0);

    ReciprocalLengthSq = __vsel(Reciprocal, S0, ReciprocalLengthSq);

    PointProjectionScale = __vmulfp(PointProjectionScale, ReciprocalLengthSq);

    DistanceVector = __vmulfp(LineVector, PointProjectionScale);
    DistanceVector = __vsubfp(PointVector, DistanceVector);

    D = __vsldoi(Zero, DistanceVector, 2 << 2);
    H = __vspltisw(1);
    D = __vmsum4fp(D, D);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2IntersectLine
(
    FXMVECTOR Line1Point1, 
    FXMVECTOR Line1Point2, 
    FXMVECTOR Line2Point1, 
    CXMVECTOR Line2Point2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR        V1;
    XMVECTOR        V2;
    XMVECTOR        V3;
    XMVECTOR        C1;
    XMVECTOR        C2;
    XMVECTOR        Result;
    CONST XMVECTOR  Zero = XMVectorZero();

    V1 = XMVectorSubtract(Line1Point2, Line1Point1);
    V2 = XMVectorSubtract(Line2Point2, Line2Point1);
    V3 = XMVectorSubtract(Line1Point1, Line2Point1);

    C1 = XMVector2Cross(V1, V2);
    C2 = XMVector2Cross(V2, V3);

    if (XMVector2NearEqual(C1, Zero, g_XMEpsilon.v))
    {
        if (XMVector2NearEqual(C2, Zero, g_XMEpsilon.v))
        {
            // Coincident
            Result = g_XMInfinity.v;
        }
        else
        {
            // Parallel
            Result = g_XMQNaN.v;
        }
    }
    else
    {
        // Intersection point = Line1Point1 + V1 * (C2 / C1)
        XMVECTOR Scale;
        Scale = XMVectorReciprocal(C1);
        Scale = XMVectorMultiply(C2, Scale);
        Result = XMVectorMultiplyAdd(V1, Scale, Line1Point1);
    }

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR V1;
    XMVECTOR V2;
    XMVECTOR V3;
    XMVECTOR C1;
    XMVECTOR C2;
    XMVECTOR Scale;
    XMVECTOR Rcp;
    XMVECTOR Reciprocal;
    XMVECTOR Refine;
    XMVECTOR ControlC1;
    XMVECTOR ControlC2;
    XMVECTOR Zero;
    XMVECTOR Intersection;
    XMVECTOR Result;

    XMVECTOR CrossP1, CrossP2;
    XMVECTOR CrossR1, CrossR2;
    XMVECTOR TrueInt, ShiftL, ShiftR, ShiftLI, ShiftLQ;
    XMVECTOR One, Epsilon, Infinity, QNaN;

    XMDUMMY_INITIALIZE_VECTOR(One);

    V3 = __vsubfp(Line1Point1, Line2Point1);
    V2 = __vsubfp(Line2Point2, Line2Point1);
    V1 = __vsubfp(Line1Point2, Line1Point1);

    CrossP2 = __vpermwi(V3, 0x4B);
    CrossP1 = __vpermwi(V2, 0x4B);
    CrossR2 = __vmulfp(V2, CrossP2);
    CrossR1 = __vmulfp(V1, CrossP1);
    CrossP2 = __vpermwi(CrossR2, 0x4B);
    CrossP1 = __vpermwi(CrossR1, 0x4B);
    CrossR2 = __vsubfp(CrossR2, CrossP2);
    CrossR1 = __vsubfp(CrossR1, CrossP1);
    C2 = __vspltw(CrossR2, 0);
    C1 = __vspltw(CrossR1, 0);

    Reciprocal = __vrefp(C1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    Rcp = __vspltw(One, 3);
    Rcp = __vnmsubfp(C1, Reciprocal, Rcp);
    Rcp = __vmaddcfp(Reciprocal, Rcp, Reciprocal);
    Scale = __vspltw(One, 3);
    Scale = __vnmsubfp(C1, Rcp, Scale);
    Refine = __vcmpeqfp(Rcp, Rcp);

    Epsilon = __vspltisw(13);

    Rcp = __vmaddfp(Rcp, Scale, Rcp);

    ShiftL = __vspltisw(-6); // Shift 26
    Epsilon = __vslw(Epsilon, ShiftL);
    Zero = __vspltisw(0);

    Refine = __vsel(Reciprocal, Rcp, Refine);

    ControlC2 = __vcmpbfp(C2, Epsilon);
    ControlC1 = __vcmpbfp(C1, Epsilon);

    Intersection = __vmulfp(C2, Refine);

    TrueInt = __vspltisw(-1);
    ShiftLQ = __vspltisw(-9); // Shift 23
    ShiftLI = __vspltisw(-8); // Shift 24
    ShiftR = __vspltisw(1);

    ControlC2 = __vcmpequw(ControlC2, Zero);

    QNaN = __vslw(TrueInt, ShiftLQ);
    Infinity = __vslw(TrueInt, ShiftLI);

    Result = __vcmpequw(ControlC1, Zero);

    Intersection = __vmaddcfp(V1, Intersection, Line1Point1);

    QNaN = __vsrw(QNaN, ShiftR);
    Infinity = __vsrw(Infinity, ShiftR);

    ControlC2 = __vsel(QNaN, Infinity, ControlC2);
    Result = __vsel(Intersection, ControlC2, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2Transform
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;

    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR Result;

    Result = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmaddcfp(M.r[1], Result, M.r[3]);
    Result = __vmaddfp(X, M.r[0], Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT4* XMVector2TransformStream
(
    XMFLOAT4*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT2* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Y = XMVectorReplicate(((XMFLOAT2*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT2*)pInputVector)->x);

        Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat4((XMFLOAT4*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
        Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);
        XMStoreFloat4NC((XMFLOAT4*)pOutputVector, Result);
        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }
    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR R0[8], R1[8], R2[8], R3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT2) == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    if (((InputVector & (sizeof(XMFLOAT2) - 1)) == 0) &&
        (InputStride == sizeof(XMFLOAT2)))
    {
        UINT GroupIndex;
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT2);
        UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            __stvlx(R0[0], (VOID*)OutputVector, 0);
            __stvrx(R0[0], (VOID*)OutputVector, 16);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                R2[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[6] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[7] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R1[0] = __vspltw(R2[0], 0);
                R0[0] = __vspltw(R2[0], 1);
                R1[1] = __vspltw(R2[0], 2);
                R0[1] = __vspltw(R2[0], 3);
                R1[2] = __vspltw(R2[1], 0);
                R0[2] = __vspltw(R2[1], 1);
                R1[3] = __vspltw(R2[1], 2);
                R0[3] = __vspltw(R2[1], 3);
                R1[4] = __vspltw(R2[2], 0);
                R0[4] = __vspltw(R2[2], 1);
                R1[5] = __vspltw(R2[2], 2);
                R0[5] = __vspltw(R2[2], 3);
                R1[6] = __vspltw(R2[3], 0);
                R0[6] = __vspltw(R2[3], 1);
                R1[7] = __vspltw(R2[3], 2);
                R0[7] = __vspltw(R2[3], 3);
                R3[0] = __vspltw(R2[4], 0);
                R2[0] = __vspltw(R2[4], 1);
                R3[1] = __vspltw(R2[4], 2);
                R2[1] = __vspltw(R2[4], 3);
                R3[2] = __vspltw(R2[5], 0);
                R2[2] = __vspltw(R2[5], 1);
                R3[3] = __vspltw(R2[5], 2);
                R2[3] = __vspltw(R2[5], 3);
                R3[4] = __vspltw(R2[6], 0);
                R2[4] = __vspltw(R2[6], 1);
                R3[5] = __vspltw(R2[6], 2);
                R2[5] = __vspltw(R2[6], 3);
                R3[6] = __vspltw(R2[7], 0);
                R2[6] = __vspltw(R2[7], 1);
                R3[7] = __vspltw(R2[7], 2);
                R2[7] = __vspltw(R2[7], 3);

                R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
                R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
                R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
                R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
                R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
                R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
                R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
                R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

                R2[0] = __vmaddcfp(M.r[1], R2[0], M.r[3]);
                R2[1] = __vmaddcfp(M.r[1], R2[1], M.r[3]);
                R2[2] = __vmaddcfp(M.r[1], R2[2], M.r[3]);
                R2[3] = __vmaddcfp(M.r[1], R2[3], M.r[3]);
                R2[4] = __vmaddcfp(M.r[1], R2[4], M.r[3]);
                R2[5] = __vmaddcfp(M.r[1], R2[5], M.r[3]);
                R2[6] = __vmaddcfp(M.r[1], R2[6], M.r[3]);
                R2[7] = __vmaddcfp(M.r[1], R2[7], M.r[3]);

                R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
                R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
                R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
                R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
                R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
                R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
                R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
                R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

                R2[0] = __vmaddfp(R3[0], M.r[0], R2[0]);
                R2[1] = __vmaddfp(R3[1], M.r[0], R2[1]);
                R2[2] = __vmaddfp(R3[2], M.r[0], R2[2]);
                R2[3] = __vmaddfp(R3[3], M.r[0], R2[3]);
                R2[4] = __vmaddfp(R3[4], M.r[0], R2[4]);
                R2[5] = __vmaddfp(R3[5], M.r[0], R2[5]);
                R2[6] = __vmaddfp(R3[6], M.r[0], R2[6]);
                R2[7] = __vmaddfp(R3[7], M.r[0], R2[7]);

                __stvlx(R0[0], (VOID*)OutputVector, 0);
                __stvrx(R0[0], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[1], (VOID*)OutputVector, 0);
                __stvrx(R0[1], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[2], (VOID*)OutputVector, 0);
                __stvrx(R0[2], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[3], (VOID*)OutputVector, 0);
                __stvrx(R0[3], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[4], (VOID*)OutputVector, 0);
                __stvrx(R0[4], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[5], (VOID*)OutputVector, 0);
                __stvrx(R0[5], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[6], (VOID*)OutputVector, 0);
                __stvrx(R0[6], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R0[7], (VOID*)OutputVector, 0);
                __stvrx(R0[7], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;

                __stvlx(R2[0], (VOID*)OutputVector, 0);
                __stvrx(R2[0], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[1], (VOID*)OutputVector, 0);
                __stvrx(R2[1], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[2], (VOID*)OutputVector, 0);
                __stvrx(R2[2], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[3], (VOID*)OutputVector, 0);
                __stvrx(R2[3], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[4], (VOID*)OutputVector, 0);
                __stvrx(R2[4], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[5], (VOID*)OutputVector, 0);
                __stvrx(R2[5], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[6], (VOID*)OutputVector, 0);
                __stvrx(R2[6], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
                __stvlx(R2[7], (VOID*)OutputVector, 0);
                __stvrx(R2[7], (VOID*)OutputVector, 16);
                 OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            __stvlx(R0[0], (VOID*)OutputVector, 0);
            __stvrx(R0[0], (VOID*)OutputVector, 16);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[1] = __lvlx((VOID*)InputVector, 0);
            R0[1] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[2] = __lvlx((VOID*)InputVector, 0);
            R0[2] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[3] = __lvlx((VOID*)InputVector, 0);
            R0[3] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[4] = __lvlx((VOID*)InputVector, 0);
            R0[4] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[5] = __lvlx((VOID*)InputVector, 0);
            R0[5] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[6] = __lvlx((VOID*)InputVector, 0);
            R0[6] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[7] = __lvlx((VOID*)InputVector, 0);
            R0[7] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);
            R1[1] = __vspltw(R1[1], 0);
            R0[1] = __vspltw(R0[1], 0);
            R1[2] = __vspltw(R1[2], 0);
            R0[2] = __vspltw(R0[2], 0);
            R1[3] = __vspltw(R1[3], 0);
            R0[3] = __vspltw(R0[3], 0);
            R1[4] = __vspltw(R1[4], 0);
            R0[4] = __vspltw(R0[4], 0);
            R1[5] = __vspltw(R1[5], 0);
            R0[5] = __vspltw(R0[5], 0);
            R1[6] = __vspltw(R1[6], 0);
            R0[6] = __vspltw(R0[6], 0);
            R1[7] = __vspltw(R1[7], 0);
            R0[7] = __vspltw(R0[7], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
            R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
            R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
            R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
            R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
            R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
            R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
            R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
            R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
            R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
            R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
            R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
            R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
            R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

            __stvlx(R0[0], (VOID*)OutputVector, 0);
            __stvrx(R0[0], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[1], (VOID*)OutputVector, 0);
            __stvrx(R0[1], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[2], (VOID*)OutputVector, 0);
            __stvrx(R0[2], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[3], (VOID*)OutputVector, 0);
            __stvrx(R0[3], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[4], (VOID*)OutputVector, 0);
            __stvrx(R0[4], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[5], (VOID*)OutputVector, 0);
            __stvrx(R0[5], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[6], (VOID*)OutputVector, 0);
            __stvrx(R0[6], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
            __stvlx(R0[7], (VOID*)OutputVector, 0);
            __stvrx(R0[7], (VOID*)OutputVector, 16);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            __stvlx(R0[0], (VOID*)OutputVector, 0);
            __stvrx(R0[0], (VOID*)OutputVector, 16);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT4* XMVector2TransformStreamNC
(
    XMFLOAT4*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT2* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS) || defined(_XM_SSE_INTRINSICS_)
    return XMVector2TransformStream( pOutputStream, OutputStride, pInputStream, InputStride, VectorCount, M );
#else // _XM_VMX128_INTRINSICS_
    CONST UINT UnrollCount = 8;
    XMVECTOR R0[8], R1[8], R2[8], R3[8], R4[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT2) == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    if (((InputVector & (sizeof(XMFLOAT2) - 1)) == 0) &&
        (InputStride == sizeof(XMFLOAT2)))
    {
        UINT GroupIndex;
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT2);
        UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R3[0] = __vspltw(R0[0], 0);
            R2[0] = __vspltw(R0[0], 1);
            R1[0] = __vspltw(R0[0], 2);
            R0[0] = __vspltw(R0[0], 3);

            __stvewx(R3[0], (VOID*)OutputVector, 0);
            __stvewx(R2[0], (VOID*)OutputVector, 4);
            __stvewx(R1[0], (VOID*)OutputVector, 8);
            __stvewx(R0[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                R2[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[6] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[7] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R1[0] = __vspltw(R2[0], 0);
                R0[0] = __vspltw(R2[0], 1);
                R1[1] = __vspltw(R2[0], 2);
                R0[1] = __vspltw(R2[0], 3);
                R1[2] = __vspltw(R2[1], 0);
                R0[2] = __vspltw(R2[1], 1);
                R1[3] = __vspltw(R2[1], 2);
                R0[3] = __vspltw(R2[1], 3);
                R1[4] = __vspltw(R2[2], 0);
                R0[4] = __vspltw(R2[2], 1);
                R1[5] = __vspltw(R2[2], 2);
                R0[5] = __vspltw(R2[2], 3);
                R1[6] = __vspltw(R2[3], 0);
                R0[6] = __vspltw(R2[3], 1);
                R1[7] = __vspltw(R2[3], 2);
                R0[7] = __vspltw(R2[3], 3);
                R3[0] = __vspltw(R2[4], 0);
                R2[0] = __vspltw(R2[4], 1);
                R3[1] = __vspltw(R2[4], 2);
                R2[1] = __vspltw(R2[4], 3);
                R3[2] = __vspltw(R2[5], 0);
                R2[2] = __vspltw(R2[5], 1);
                R3[3] = __vspltw(R2[5], 2);
                R2[3] = __vspltw(R2[5], 3);
                R3[4] = __vspltw(R2[6], 0);
                R2[4] = __vspltw(R2[6], 1);
                R3[5] = __vspltw(R2[6], 2);
                R2[5] = __vspltw(R2[6], 3);
                R3[6] = __vspltw(R2[7], 0);
                R2[6] = __vspltw(R2[7], 1);
                R3[7] = __vspltw(R2[7], 2);
                R2[7] = __vspltw(R2[7], 3);

                R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
                R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
                R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
                R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
                R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
                R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
                R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
                R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

                R2[0] = __vmaddcfp(M.r[1], R2[0], M.r[3]);
                R2[1] = __vmaddcfp(M.r[1], R2[1], M.r[3]);
                R2[2] = __vmaddcfp(M.r[1], R2[2], M.r[3]);
                R2[3] = __vmaddcfp(M.r[1], R2[3], M.r[3]);
                R2[4] = __vmaddcfp(M.r[1], R2[4], M.r[3]);
                R2[5] = __vmaddcfp(M.r[1], R2[5], M.r[3]);
                R2[6] = __vmaddcfp(M.r[1], R2[6], M.r[3]);
                R2[7] = __vmaddcfp(M.r[1], R2[7], M.r[3]);

                R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
                R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
                R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
                R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
                R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
                R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
                R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
                R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

                R2[0] = __vmaddfp(R3[0], M.r[0], R2[0]);
                R2[1] = __vmaddfp(R3[1], M.r[0], R2[1]);
                R2[2] = __vmaddfp(R3[2], M.r[0], R2[2]);
                R2[3] = __vmaddfp(R3[3], M.r[0], R2[3]);
                R2[4] = __vmaddfp(R3[4], M.r[0], R2[4]);
                R2[5] = __vmaddfp(R3[5], M.r[0], R2[5]);
                R2[6] = __vmaddfp(R3[6], M.r[0], R2[6]);
                R2[7] = __vmaddfp(R3[7], M.r[0], R2[7]);

                R4[0] = __vspltw(R0[0], 0);
                R3[0] = __vspltw(R0[0], 1);
                R1[0] = __vspltw(R0[0], 2);
                R0[0] = __vspltw(R0[0], 3);
                                     
                R4[1] = __vspltw(R0[1], 0);
                R3[1] = __vspltw(R0[1], 1);
                R1[1] = __vspltw(R0[1], 2);
                R0[1] = __vspltw(R0[1], 3);
                                     
                R4[2] = __vspltw(R0[2], 0);
                R3[2] = __vspltw(R0[2], 1);
                R1[2] = __vspltw(R0[2], 2);
                R0[2] = __vspltw(R0[2], 3);
                                     
                R4[3] = __vspltw(R0[3], 0);
                R3[3] = __vspltw(R0[3], 1);
                R1[3] = __vspltw(R0[3], 2);
                R0[3] = __vspltw(R0[3], 3);
                                     
                R4[4] = __vspltw(R0[4], 0);
                R3[4] = __vspltw(R0[4], 1);
                R1[4] = __vspltw(R0[4], 2);
                R0[4] = __vspltw(R0[4], 3);
                                     
                R4[5] = __vspltw(R0[5], 0);
                R3[5] = __vspltw(R0[5], 1);
                R1[5] = __vspltw(R0[5], 2);
                R0[5] = __vspltw(R0[5], 3);
                                     
                R4[6] = __vspltw(R0[6], 0);
                R3[6] = __vspltw(R0[6], 1);
                R1[6] = __vspltw(R0[6], 2);
                R0[6] = __vspltw(R0[6], 3);
                                     
                R4[7] = __vspltw(R0[7], 0);
                R3[7] = __vspltw(R0[7], 1);
                R1[7] = __vspltw(R0[7], 2);
                R0[7] = __vspltw(R0[7], 3);

                __stvewx(R4[0], (VOID*)OutputVector, 0);
                __stvewx(R3[0], (VOID*)OutputVector, 4);
                __stvewx(R1[0], (VOID*)OutputVector, 8);
                __stvewx(R0[0], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[1], (VOID*)OutputVector, 0);
                __stvewx(R3[1], (VOID*)OutputVector, 4);
                __stvewx(R1[1], (VOID*)OutputVector, 8);
                __stvewx(R0[1], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[2], (VOID*)OutputVector, 0);
                __stvewx(R3[2], (VOID*)OutputVector, 4);
                __stvewx(R1[2], (VOID*)OutputVector, 8);
                __stvewx(R0[2], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[3], (VOID*)OutputVector, 0);
                __stvewx(R3[3], (VOID*)OutputVector, 4);
                __stvewx(R1[3], (VOID*)OutputVector, 8);
                __stvewx(R0[3], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[4], (VOID*)OutputVector, 0);
                __stvewx(R3[4], (VOID*)OutputVector, 4);
                __stvewx(R1[4], (VOID*)OutputVector, 8);
                __stvewx(R0[4], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[5], (VOID*)OutputVector, 0);
                __stvewx(R3[5], (VOID*)OutputVector, 4);
                __stvewx(R1[5], (VOID*)OutputVector, 8);
                __stvewx(R0[5], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[6], (VOID*)OutputVector, 0);
                __stvewx(R3[6], (VOID*)OutputVector, 4);
                __stvewx(R1[6], (VOID*)OutputVector, 8);
                __stvewx(R0[6], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[7], (VOID*)OutputVector, 0);
                __stvewx(R3[7], (VOID*)OutputVector, 4);
                __stvewx(R1[7], (VOID*)OutputVector, 8);
                __stvewx(R0[7], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;

                R4[0] = __vspltw(R2[0], 0);
                R3[0] = __vspltw(R2[0], 1);
                R1[0] = __vspltw(R2[0], 2);
                R0[0] = __vspltw(R2[0], 3);
                                     
                R4[1] = __vspltw(R2[1], 0);
                R3[1] = __vspltw(R2[1], 1);
                R1[1] = __vspltw(R2[1], 2);
                R0[1] = __vspltw(R2[1], 3);
                                     
                R4[2] = __vspltw(R2[2], 0);
                R3[2] = __vspltw(R2[2], 1);
                R1[2] = __vspltw(R2[2], 2);
                R0[2] = __vspltw(R2[2], 3);
                                     
                R4[3] = __vspltw(R2[3], 0);
                R3[3] = __vspltw(R2[3], 1);
                R1[3] = __vspltw(R2[3], 2);
                R0[3] = __vspltw(R2[3], 3);
                                     
                R4[4] = __vspltw(R2[4], 0);
                R3[4] = __vspltw(R2[4], 1);
                R1[4] = __vspltw(R2[4], 2);
                R0[4] = __vspltw(R2[4], 3);
                                     
                R4[5] = __vspltw(R2[5], 0);
                R3[5] = __vspltw(R2[5], 1);
                R1[5] = __vspltw(R2[5], 2);
                R0[5] = __vspltw(R2[5], 3);
                                     
                R4[6] = __vspltw(R2[6], 0);
                R3[6] = __vspltw(R2[6], 1);
                R1[6] = __vspltw(R2[6], 2);
                R0[6] = __vspltw(R2[6], 3);
                                     
                R4[7] = __vspltw(R2[7], 0);
                R3[7] = __vspltw(R2[7], 1);
                R1[7] = __vspltw(R2[7], 2);
                R0[7] = __vspltw(R2[7], 3);

                __stvewx(R4[0], (VOID*)OutputVector, 0);
                __stvewx(R3[0], (VOID*)OutputVector, 4);
                __stvewx(R1[0], (VOID*)OutputVector, 8);
                __stvewx(R0[0], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[1], (VOID*)OutputVector, 0);
                __stvewx(R3[1], (VOID*)OutputVector, 4);
                __stvewx(R1[1], (VOID*)OutputVector, 8);
                __stvewx(R0[1], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[2], (VOID*)OutputVector, 0);
                __stvewx(R3[2], (VOID*)OutputVector, 4);
                __stvewx(R1[2], (VOID*)OutputVector, 8);
                __stvewx(R0[2], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[3], (VOID*)OutputVector, 0);
                __stvewx(R3[3], (VOID*)OutputVector, 4);
                __stvewx(R1[3], (VOID*)OutputVector, 8);
                __stvewx(R0[3], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[4], (VOID*)OutputVector, 0);
                __stvewx(R3[4], (VOID*)OutputVector, 4);
                __stvewx(R1[4], (VOID*)OutputVector, 8);
                __stvewx(R0[4], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[5], (VOID*)OutputVector, 0);
                __stvewx(R3[5], (VOID*)OutputVector, 4);
                __stvewx(R1[5], (VOID*)OutputVector, 8);
                __stvewx(R0[5], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[6], (VOID*)OutputVector, 0);
                __stvewx(R3[6], (VOID*)OutputVector, 4);
                __stvewx(R1[6], (VOID*)OutputVector, 8);
                __stvewx(R0[6], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                __stvewx(R4[7], (VOID*)OutputVector, 0);
                __stvewx(R3[7], (VOID*)OutputVector, 4);
                __stvewx(R1[7], (VOID*)OutputVector, 8);
                __stvewx(R0[7], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R3[0] = __vspltw(R0[0], 0);
            R2[0] = __vspltw(R0[0], 1);
            R1[0] = __vspltw(R0[0], 2);
            R0[0] = __vspltw(R0[0], 3);

            __stvewx(R3[0], (VOID*)OutputVector, 0);
            __stvewx(R2[0], (VOID*)OutputVector, 4);
            __stvewx(R1[0], (VOID*)OutputVector, 8);
            __stvewx(R0[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[1] = __lvlx((VOID*)InputVector, 0);
            R0[1] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[2] = __lvlx((VOID*)InputVector, 0);
            R0[2] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[3] = __lvlx((VOID*)InputVector, 0);
            R0[3] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[4] = __lvlx((VOID*)InputVector, 0);
            R0[4] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[5] = __lvlx((VOID*)InputVector, 0);
            R0[5] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[6] = __lvlx((VOID*)InputVector, 0);
            R0[6] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[7] = __lvlx((VOID*)InputVector, 0);
            R0[7] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);
            R1[1] = __vspltw(R1[1], 0);
            R0[1] = __vspltw(R0[1], 0);
            R1[2] = __vspltw(R1[2], 0);
            R0[2] = __vspltw(R0[2], 0);
            R1[3] = __vspltw(R1[3], 0);
            R0[3] = __vspltw(R0[3], 0);
            R1[4] = __vspltw(R1[4], 0);
            R0[4] = __vspltw(R0[4], 0);
            R1[5] = __vspltw(R1[5], 0);
            R0[5] = __vspltw(R0[5], 0);
            R1[6] = __vspltw(R1[6], 0);
            R0[6] = __vspltw(R0[6], 0);
            R1[7] = __vspltw(R1[7], 0);
            R0[7] = __vspltw(R0[7], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
            R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
            R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
            R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
            R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
            R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
            R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
            R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
            R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
            R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
            R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
            R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
            R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
            R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

            R3[0] = __vspltw(R0[0], 0);
            R2[0] = __vspltw(R0[0], 1);
            R1[0] = __vspltw(R0[0], 2);
            R0[0] = __vspltw(R0[0], 3);
                                
            R3[1] = __vspltw(R0[1], 0);
            R2[1] = __vspltw(R0[1], 1);
            R1[1] = __vspltw(R0[1], 2);
            R0[1] = __vspltw(R0[1], 3);
                                
            R3[2] = __vspltw(R0[2], 0);
            R2[2] = __vspltw(R0[2], 1);
            R1[2] = __vspltw(R0[2], 2);
            R0[2] = __vspltw(R0[2], 3);
                                
            R3[3] = __vspltw(R0[3], 0);
            R2[3] = __vspltw(R0[3], 1);
            R1[3] = __vspltw(R0[3], 2);
            R0[3] = __vspltw(R0[3], 3);
                                
            R3[4] = __vspltw(R0[4], 0);
            R2[4] = __vspltw(R0[4], 1);
            R1[4] = __vspltw(R0[4], 2);
            R0[4] = __vspltw(R0[4], 3);
                                
            R3[5] = __vspltw(R0[5], 0);
            R2[5] = __vspltw(R0[5], 1);
            R1[5] = __vspltw(R0[5], 2);
            R0[5] = __vspltw(R0[5], 3);
                                
            R3[6] = __vspltw(R0[6], 0);
            R2[6] = __vspltw(R0[6], 1);
            R1[6] = __vspltw(R0[6], 2);
            R0[6] = __vspltw(R0[6], 3);
                                
            R3[7] = __vspltw(R0[7], 0);
            R2[7] = __vspltw(R0[7], 1);
            R1[7] = __vspltw(R0[7], 2);
            R0[7] = __vspltw(R0[7], 3);

            __stvewx(R3[0], (VOID*)OutputVector, 0);
            __stvewx(R2[0], (VOID*)OutputVector, 4);
            __stvewx(R1[0], (VOID*)OutputVector, 8);
            __stvewx(R0[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[1], (VOID*)OutputVector, 0);
            __stvewx(R2[1], (VOID*)OutputVector, 4);
            __stvewx(R1[1], (VOID*)OutputVector, 8);
            __stvewx(R0[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[2], (VOID*)OutputVector, 0);
            __stvewx(R2[2], (VOID*)OutputVector, 4);
            __stvewx(R1[2], (VOID*)OutputVector, 8);
            __stvewx(R0[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[3], (VOID*)OutputVector, 0);
            __stvewx(R2[3], (VOID*)OutputVector, 4);
            __stvewx(R1[3], (VOID*)OutputVector, 8);
            __stvewx(R0[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[4], (VOID*)OutputVector, 0);
            __stvewx(R2[4], (VOID*)OutputVector, 4);
            __stvewx(R1[4], (VOID*)OutputVector, 8);
            __stvewx(R0[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[5], (VOID*)OutputVector, 0);
            __stvewx(R2[5], (VOID*)OutputVector, 4);
            __stvewx(R1[5], (VOID*)OutputVector, 8);
            __stvewx(R0[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[6], (VOID*)OutputVector, 0);
            __stvewx(R2[6], (VOID*)OutputVector, 4);
            __stvewx(R1[6], (VOID*)OutputVector, 8);
            __stvewx(R0[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R3[7], (VOID*)OutputVector, 0);
            __stvewx(R2[7], (VOID*)OutputVector, 4);
            __stvewx(R1[7], (VOID*)OutputVector, 8);
            __stvewx(R0[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R3[0] = __vspltw(R0[0], 0);
            R2[0] = __vspltw(R0[0], 1);
            R1[0] = __vspltw(R0[0], 2);
            R0[0] = __vspltw(R0[0], 3);

            __stvewx(R3[0], (VOID*)OutputVector, 0);
            __stvewx(R2[0], (VOID*)OutputVector, 4);
            __stvewx(R1[0], (VOID*)OutputVector, 8);
            __stvewx(R0[0], (VOID*)OutputVector, 12);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2TransformCoord
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR InverseW;
    XMVECTOR Result;

    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    InverseW = XMVectorSplatW(Result);
    InverseW = XMVectorReciprocal(InverseW);

    Result = XMVectorMultiply(Result, InverseW);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR InverseW;
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Result = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmaddcfp(M.r[1], Result, M.r[3]);
    Result = __vmaddfp(X, M.r[0], Result);

    InverseW = __vspltw(Result, 3);

    Reciprocal = __vrefp(InverseW);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(InverseW, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(InverseW, S0, S1);
    InverseW = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    InverseW = __vsel(Reciprocal, S0, InverseW);

    Result = __vmulfp(Result, InverseW);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT2* XMVector2TransformCoordStream
(
    XMFLOAT2*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT2* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR InverseW;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Y = XMVectorReplicate(((XMFLOAT2*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT2*)pInputVector)->x);

        Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        InverseW = XMVectorSplatW(Result);
        InverseW = XMVectorReciprocal(InverseW);

        Result = XMVectorMultiply(Result, InverseW);

        XMStoreFloat2((XMFLOAT2*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR InverseW;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Y = XMVectorReplicate(((XMFLOAT2*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT2*)pInputVector)->x);

        Result = XMVectorMultiplyAdd(Y, M.r[1], M.r[3]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        InverseW = XMVectorSplatW(Result);
        InverseW = XMVectorReciprocal(InverseW);

        Result = XMVectorMultiply(Result, InverseW);

        XMStoreFloat2((XMFLOAT2*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR R0[8], R1[8], R2[8], R3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT2) == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    if (((InputVector & (sizeof(XMFLOAT2) - 1)) == 0) &&
        (InputStride == sizeof(XMFLOAT2)))
    {
        UINT GroupIndex;
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT2);
        UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 3);

            R1[0] = XMVectorReciprocal(R1[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                R2[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[6] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[7] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R1[0] = __vspltw(R2[0], 0);
                R0[0] = __vspltw(R2[0], 1);
                R1[1] = __vspltw(R2[0], 2);
                R0[1] = __vspltw(R2[0], 3);
                R1[2] = __vspltw(R2[1], 0);
                R0[2] = __vspltw(R2[1], 1);
                R1[3] = __vspltw(R2[1], 2);
                R0[3] = __vspltw(R2[1], 3);
                R1[4] = __vspltw(R2[2], 0);
                R0[4] = __vspltw(R2[2], 1);
                R1[5] = __vspltw(R2[2], 2);
                R0[5] = __vspltw(R2[2], 3);
                R1[6] = __vspltw(R2[3], 0);
                R0[6] = __vspltw(R2[3], 1);
                R1[7] = __vspltw(R2[3], 2);
                R0[7] = __vspltw(R2[3], 3);
                R3[0] = __vspltw(R2[4], 0);
                R2[0] = __vspltw(R2[4], 1);
                R3[1] = __vspltw(R2[4], 2);
                R2[1] = __vspltw(R2[4], 3);
                R3[2] = __vspltw(R2[5], 0);
                R2[2] = __vspltw(R2[5], 1);
                R3[3] = __vspltw(R2[5], 2);
                R2[3] = __vspltw(R2[5], 3);
                R3[4] = __vspltw(R2[6], 0);
                R2[4] = __vspltw(R2[6], 1);
                R3[5] = __vspltw(R2[6], 2);
                R2[5] = __vspltw(R2[6], 3);
                R3[6] = __vspltw(R2[7], 0);
                R2[6] = __vspltw(R2[7], 1);
                R3[7] = __vspltw(R2[7], 2);
                R2[7] = __vspltw(R2[7], 3);

                R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
                R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
                R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
                R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
                R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
                R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
                R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
                R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

                R2[0] = __vmaddcfp(M.r[1], R2[0], M.r[3]);
                R2[1] = __vmaddcfp(M.r[1], R2[1], M.r[3]);
                R2[2] = __vmaddcfp(M.r[1], R2[2], M.r[3]);
                R2[3] = __vmaddcfp(M.r[1], R2[3], M.r[3]);
                R2[4] = __vmaddcfp(M.r[1], R2[4], M.r[3]);
                R2[5] = __vmaddcfp(M.r[1], R2[5], M.r[3]);
                R2[6] = __vmaddcfp(M.r[1], R2[6], M.r[3]);
                R2[7] = __vmaddcfp(M.r[1], R2[7], M.r[3]);

                R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
                R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
                R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
                R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
                R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
                R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
                R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
                R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

                R2[0] = __vmaddfp(R3[0], M.r[0], R2[0]);
                R2[1] = __vmaddfp(R3[1], M.r[0], R2[1]);
                R2[2] = __vmaddfp(R3[2], M.r[0], R2[2]);
                R2[3] = __vmaddfp(R3[3], M.r[0], R2[3]);
                R2[4] = __vmaddfp(R3[4], M.r[0], R2[4]);
                R2[5] = __vmaddfp(R3[5], M.r[0], R2[5]);
                R2[6] = __vmaddfp(R3[6], M.r[0], R2[6]);
                R2[7] = __vmaddfp(R3[7], M.r[0], R2[7]);

                R1[0] = __vspltw(R0[0], 3);
                R1[1] = __vspltw(R0[1], 3);
                R1[2] = __vspltw(R0[2], 3);
                R1[3] = __vspltw(R0[3], 3);
                R1[4] = __vspltw(R0[4], 3);
                R1[5] = __vspltw(R0[5], 3);
                R1[6] = __vspltw(R0[6], 3);
                R1[7] = __vspltw(R0[7], 3);

                R3[0] = __vspltw(R2[0], 3);
                R3[1] = __vspltw(R2[1], 3);
                R3[2] = __vspltw(R2[2], 3);
                R3[3] = __vspltw(R2[3], 3);
                R3[4] = __vspltw(R2[4], 3);
                R3[5] = __vspltw(R2[5], 3);
                R3[6] = __vspltw(R2[6], 3);
                R3[7] = __vspltw(R2[7], 3);

                R1[0] = XMVectorReciprocal(R1[0]);
                R1[1] = XMVectorReciprocal(R1[1]);
                R1[2] = XMVectorReciprocal(R1[2]);
                R1[3] = XMVectorReciprocal(R1[3]);
                R1[4] = XMVectorReciprocal(R1[4]);
                R1[5] = XMVectorReciprocal(R1[5]);
                R1[6] = XMVectorReciprocal(R1[6]);
                R1[7] = XMVectorReciprocal(R1[7]);

                R3[0] = XMVectorReciprocal(R3[0]);
                R3[1] = XMVectorReciprocal(R3[1]);
                R3[2] = XMVectorReciprocal(R3[2]);
                R3[3] = XMVectorReciprocal(R3[3]);
                R3[4] = XMVectorReciprocal(R3[4]);
                R3[5] = XMVectorReciprocal(R3[5]);
                R3[6] = XMVectorReciprocal(R3[6]);
                R3[7] = XMVectorReciprocal(R3[7]);

                R0[0] = __vmulfp(R0[0], R1[0]);
                R0[1] = __vmulfp(R0[1], R1[1]);
                R0[2] = __vmulfp(R0[2], R1[2]);
                R0[3] = __vmulfp(R0[3], R1[3]);
                R0[4] = __vmulfp(R0[4], R1[4]);
                R0[5] = __vmulfp(R0[5], R1[5]);
                R0[6] = __vmulfp(R0[6], R1[6]);
                R0[7] = __vmulfp(R0[7], R1[7]);

                R2[0] = __vmulfp(R2[0], R3[0]);
                R2[1] = __vmulfp(R2[1], R3[1]);
                R2[2] = __vmulfp(R2[2], R3[2]);
                R2[3] = __vmulfp(R2[3], R3[3]);
                R2[4] = __vmulfp(R2[4], R3[4]);
                R2[5] = __vmulfp(R2[5], R3[5]);
                R2[6] = __vmulfp(R2[6], R3[6]);
                R2[7] = __vmulfp(R2[7], R3[7]);

                R1[0] = __vspltw(R0[0], 0);
                R0[0] = __vspltw(R0[0], 1);
                R1[1] = __vspltw(R0[1], 0);
                R0[1] = __vspltw(R0[1], 1);
                R1[2] = __vspltw(R0[2], 0);
                R0[2] = __vspltw(R0[2], 1);
                R1[3] = __vspltw(R0[3], 0);
                R0[3] = __vspltw(R0[3], 1);
                R1[4] = __vspltw(R0[4], 0);
                R0[4] = __vspltw(R0[4], 1);
                R1[5] = __vspltw(R0[5], 0);
                R0[5] = __vspltw(R0[5], 1);
                R1[6] = __vspltw(R0[6], 0);
                R0[6] = __vspltw(R0[6], 1);
                R1[7] = __vspltw(R0[7], 0);
                R0[7] = __vspltw(R0[7], 1);

                R3[0] = __vspltw(R2[0], 0);
                R2[0] = __vspltw(R2[0], 1);
                R3[1] = __vspltw(R2[1], 0);
                R2[1] = __vspltw(R2[1], 1);
                R3[2] = __vspltw(R2[2], 0);
                R2[2] = __vspltw(R2[2], 1);
                R3[3] = __vspltw(R2[3], 0);
                R2[3] = __vspltw(R2[3], 1);
                R3[4] = __vspltw(R2[4], 0);
                R2[4] = __vspltw(R2[4], 1);
                R3[5] = __vspltw(R2[5], 0);
                R2[5] = __vspltw(R2[5], 1);
                R3[6] = __vspltw(R2[6], 0);
                R2[6] = __vspltw(R2[6], 1);
                R3[7] = __vspltw(R2[7], 0);
                R2[7] = __vspltw(R2[7], 1);

                __stvewx(R1[0], (VOID*)OutputVector, 0);
                __stvewx(R0[0], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[1], (VOID*)OutputVector, 0);
                __stvewx(R0[1], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[2], (VOID*)OutputVector, 0);
                __stvewx(R0[2], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[3], (VOID*)OutputVector, 0);
                __stvewx(R0[3], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[4], (VOID*)OutputVector, 0);
                __stvewx(R0[4], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[5], (VOID*)OutputVector, 0);
                __stvewx(R0[5], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[6], (VOID*)OutputVector, 0);
                __stvewx(R0[6], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[7], (VOID*)OutputVector, 0);
                __stvewx(R0[7], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;

                __stvewx(R3[0], (VOID*)OutputVector, 0);
                __stvewx(R2[0], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[1], (VOID*)OutputVector, 0);
                __stvewx(R2[1], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[2], (VOID*)OutputVector, 0);
                __stvewx(R2[2], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[3], (VOID*)OutputVector, 0);
                __stvewx(R2[3], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[4], (VOID*)OutputVector, 0);
                __stvewx(R2[4], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[5], (VOID*)OutputVector, 0);
                __stvewx(R2[5], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[6], (VOID*)OutputVector, 0);
                __stvewx(R2[6], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[7], (VOID*)OutputVector, 0);
                __stvewx(R2[7], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 3);

            R1[0] = XMVectorReciprocal(R1[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[1] = __lvlx((VOID*)InputVector, 0);
            R0[1] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[2] = __lvlx((VOID*)InputVector, 0);
            R0[2] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[3] = __lvlx((VOID*)InputVector, 0);
            R0[3] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[4] = __lvlx((VOID*)InputVector, 0);
            R0[4] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[5] = __lvlx((VOID*)InputVector, 0);
            R0[5] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[6] = __lvlx((VOID*)InputVector, 0);
            R0[6] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[7] = __lvlx((VOID*)InputVector, 0);
            R0[7] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);
            R1[1] = __vspltw(R1[1], 0);
            R0[1] = __vspltw(R0[1], 0);
            R1[2] = __vspltw(R1[2], 0);
            R0[2] = __vspltw(R0[2], 0);
            R1[3] = __vspltw(R1[3], 0);
            R0[3] = __vspltw(R0[3], 0);
            R1[4] = __vspltw(R1[4], 0);
            R0[4] = __vspltw(R0[4], 0);
            R1[5] = __vspltw(R1[5], 0);
            R0[5] = __vspltw(R0[5], 0);
            R1[6] = __vspltw(R1[6], 0);
            R0[6] = __vspltw(R0[6], 0);
            R1[7] = __vspltw(R1[7], 0);
            R0[7] = __vspltw(R0[7], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[1] = __vmaddcfp(M.r[1], R0[1], M.r[3]);
            R0[2] = __vmaddcfp(M.r[1], R0[2], M.r[3]);
            R0[3] = __vmaddcfp(M.r[1], R0[3], M.r[3]);
            R0[4] = __vmaddcfp(M.r[1], R0[4], M.r[3]);
            R0[5] = __vmaddcfp(M.r[1], R0[5], M.r[3]);
            R0[6] = __vmaddcfp(M.r[1], R0[6], M.r[3]);
            R0[7] = __vmaddcfp(M.r[1], R0[7], M.r[3]);

            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
            R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
            R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
            R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
            R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
            R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
            R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
            R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

            R1[0] = __vspltw(R0[0], 3);
            R1[1] = __vspltw(R0[1], 3);
            R1[2] = __vspltw(R0[2], 3);
            R1[3] = __vspltw(R0[3], 3);
            R1[4] = __vspltw(R0[4], 3);
            R1[5] = __vspltw(R0[5], 3);
            R1[6] = __vspltw(R0[6], 3);
            R1[7] = __vspltw(R0[7], 3);

            R1[0] = XMVectorReciprocal(R1[0]);
            R1[1] = XMVectorReciprocal(R1[1]);
            R1[2] = XMVectorReciprocal(R1[2]);
            R1[3] = XMVectorReciprocal(R1[3]);
            R1[4] = XMVectorReciprocal(R1[4]);
            R1[5] = XMVectorReciprocal(R1[5]);
            R1[6] = XMVectorReciprocal(R1[6]);
            R1[7] = XMVectorReciprocal(R1[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);
                                
            R1[1] = __vspltw(R0[1], 0);
            R0[1] = __vspltw(R0[1], 1);
                                
            R1[2] = __vspltw(R0[2], 0);
            R0[2] = __vspltw(R0[2], 1);
                                
            R1[3] = __vspltw(R0[3], 0);
            R0[3] = __vspltw(R0[3], 1);
                                
            R1[4] = __vspltw(R0[4], 0);
            R0[4] = __vspltw(R0[4], 1);
                                
            R1[5] = __vspltw(R0[5], 0);
            R0[5] = __vspltw(R0[5], 1);
                                
            R1[6] = __vspltw(R0[6], 0);
            R0[6] = __vspltw(R0[6], 1);
                                
            R1[7] = __vspltw(R0[7], 0);
            R0[7] = __vspltw(R0[7], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[1], (VOID*)OutputVector, 0);
            __stvewx(R0[1], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[2], (VOID*)OutputVector, 0);
            __stvewx(R0[2], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[3], (VOID*)OutputVector, 0);
            __stvewx(R0[3], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[4], (VOID*)OutputVector, 0);
            __stvewx(R0[4], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[5], (VOID*)OutputVector, 0);
            __stvewx(R0[5], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[6], (VOID*)OutputVector, 0);
            __stvewx(R0[6], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[7], (VOID*)OutputVector, 0);
            __stvewx(R0[7], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmaddcfp(M.r[1], R0[0], M.r[3]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 3);

            R1[0] = XMVectorReciprocal(R1[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector2TransformNormal
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;

    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiply(Y, M.r[1]);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;

    Y = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmulfp(Y, M.r[1]);
    Result = __vmaddfp(X, M.r[0], Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT2* XMVector2TransformNormalStream
(
    XMFLOAT2*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT2* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Y = XMVectorReplicate(((XMFLOAT2*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT2*)pInputVector)->x);

        Result = XMVectorMultiply(Y, M.r[1]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat2((XMFLOAT2*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat2((const XMFLOAT2*)pInputVector);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Y = XMVectorReplicate(((XMFLOAT2*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT2*)pInputVector)->x);

        Result = XMVectorMultiply(Y, M.r[1]);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat2((XMFLOAT2*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR R0[8], R1[8], R2[8], R3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT2) == 8);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    if (((InputVector & (sizeof(XMFLOAT2) - 1)) == 0) &&
        (InputStride == sizeof(XMFLOAT2)))
    {
        UINT GroupIndex;
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT2);
        UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmulfp(M.r[1], R0[0]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                R2[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[6] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                R2[7] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R1[0] = __vspltw(R2[0], 0);
                R0[0] = __vspltw(R2[0], 1);
                R1[1] = __vspltw(R2[0], 2);
                R0[1] = __vspltw(R2[0], 3);
                R1[2] = __vspltw(R2[1], 0);
                R0[2] = __vspltw(R2[1], 1);
                R1[3] = __vspltw(R2[1], 2);
                R0[3] = __vspltw(R2[1], 3);
                R1[4] = __vspltw(R2[2], 0);
                R0[4] = __vspltw(R2[2], 1);
                R1[5] = __vspltw(R2[2], 2);
                R0[5] = __vspltw(R2[2], 3);
                R1[6] = __vspltw(R2[3], 0);
                R0[6] = __vspltw(R2[3], 1);
                R1[7] = __vspltw(R2[3], 2);
                R0[7] = __vspltw(R2[3], 3);
                R3[0] = __vspltw(R2[4], 0);
                R2[0] = __vspltw(R2[4], 1);
                R3[1] = __vspltw(R2[4], 2);
                R2[1] = __vspltw(R2[4], 3);
                R3[2] = __vspltw(R2[5], 0);
                R2[2] = __vspltw(R2[5], 1);
                R3[3] = __vspltw(R2[5], 2);
                R2[3] = __vspltw(R2[5], 3);
                R3[4] = __vspltw(R2[6], 0);
                R2[4] = __vspltw(R2[6], 1);
                R3[5] = __vspltw(R2[6], 2);
                R2[5] = __vspltw(R2[6], 3);
                R3[6] = __vspltw(R2[7], 0);
                R2[6] = __vspltw(R2[7], 1);
                R3[7] = __vspltw(R2[7], 2);
                R2[7] = __vspltw(R2[7], 3);

                R0[0] = __vmulfp(M.r[1], R0[0]);
                R0[1] = __vmulfp(M.r[1], R0[1]);
                R0[2] = __vmulfp(M.r[1], R0[2]);
                R0[3] = __vmulfp(M.r[1], R0[3]);
                R0[4] = __vmulfp(M.r[1], R0[4]);
                R0[5] = __vmulfp(M.r[1], R0[5]);
                R0[6] = __vmulfp(M.r[1], R0[6]);
                R0[7] = __vmulfp(M.r[1], R0[7]);

                R2[0] = __vmulfp(M.r[1], R2[0]);
                R2[1] = __vmulfp(M.r[1], R2[1]);
                R2[2] = __vmulfp(M.r[1], R2[2]);
                R2[3] = __vmulfp(M.r[1], R2[3]);
                R2[4] = __vmulfp(M.r[1], R2[4]);
                R2[5] = __vmulfp(M.r[1], R2[5]);
                R2[6] = __vmulfp(M.r[1], R2[6]);
                R2[7] = __vmulfp(M.r[1], R2[7]);

                R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
                R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
                R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
                R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
                R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
                R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
                R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
                R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

                R2[0] = __vmaddfp(R3[0], M.r[0], R2[0]);
                R2[1] = __vmaddfp(R3[1], M.r[0], R2[1]);
                R2[2] = __vmaddfp(R3[2], M.r[0], R2[2]);
                R2[3] = __vmaddfp(R3[3], M.r[0], R2[3]);
                R2[4] = __vmaddfp(R3[4], M.r[0], R2[4]);
                R2[5] = __vmaddfp(R3[5], M.r[0], R2[5]);
                R2[6] = __vmaddfp(R3[6], M.r[0], R2[6]);
                R2[7] = __vmaddfp(R3[7], M.r[0], R2[7]);

                R1[0] = __vspltw(R0[0], 0);
                R0[0] = __vspltw(R0[0], 1);
                R1[1] = __vspltw(R0[1], 0);
                R0[1] = __vspltw(R0[1], 1);
                R1[2] = __vspltw(R0[2], 0);
                R0[2] = __vspltw(R0[2], 1);
                R1[3] = __vspltw(R0[3], 0);
                R0[3] = __vspltw(R0[3], 1);
                R1[4] = __vspltw(R0[4], 0);
                R0[4] = __vspltw(R0[4], 1);
                R1[5] = __vspltw(R0[5], 0);
                R0[5] = __vspltw(R0[5], 1);
                R1[6] = __vspltw(R0[6], 0);
                R0[6] = __vspltw(R0[6], 1);
                R1[7] = __vspltw(R0[7], 0);
                R0[7] = __vspltw(R0[7], 1);

                R3[0] = __vspltw(R2[0], 0);
                R2[0] = __vspltw(R2[0], 1);
                R3[1] = __vspltw(R2[1], 0);
                R2[1] = __vspltw(R2[1], 1);
                R3[2] = __vspltw(R2[2], 0);
                R2[2] = __vspltw(R2[2], 1);
                R3[3] = __vspltw(R2[3], 0);
                R2[3] = __vspltw(R2[3], 1);
                R3[4] = __vspltw(R2[4], 0);
                R2[4] = __vspltw(R2[4], 1);
                R3[5] = __vspltw(R2[5], 0);
                R2[5] = __vspltw(R2[5], 1);
                R3[6] = __vspltw(R2[6], 0);
                R2[6] = __vspltw(R2[6], 1);
                R3[7] = __vspltw(R2[7], 0);
                R2[7] = __vspltw(R2[7], 1);

                __stvewx(R1[0], (VOID*)OutputVector, 0);
                __stvewx(R0[0], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[1], (VOID*)OutputVector, 0);
                __stvewx(R0[1], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[2], (VOID*)OutputVector, 0);
                __stvewx(R0[2], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[3], (VOID*)OutputVector, 0);
                __stvewx(R0[3], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[4], (VOID*)OutputVector, 0);
                __stvewx(R0[4], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[5], (VOID*)OutputVector, 0);
                __stvewx(R0[5], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[6], (VOID*)OutputVector, 0);
                __stvewx(R0[6], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R1[7], (VOID*)OutputVector, 0);
                __stvewx(R0[7], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;

                __stvewx(R3[0], (VOID*)OutputVector, 0);
                __stvewx(R2[0], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[1], (VOID*)OutputVector, 0);
                __stvewx(R2[1], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[2], (VOID*)OutputVector, 0);
                __stvewx(R2[2], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[3], (VOID*)OutputVector, 0);
                __stvewx(R2[3], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[4], (VOID*)OutputVector, 0);
                __stvewx(R2[4], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[5], (VOID*)OutputVector, 0);
                __stvewx(R2[5], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[6], (VOID*)OutputVector, 0);
                __stvewx(R2[6], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
                __stvewx(R3[7], (VOID*)OutputVector, 0);
                __stvewx(R2[7], (VOID*)OutputVector, 4);
                 OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmulfp(M.r[1], R0[0]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += sizeof(XMFLOAT2); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[1] = __lvlx((VOID*)InputVector, 0);
            R0[1] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[2] = __lvlx((VOID*)InputVector, 0);
            R0[2] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[3] = __lvlx((VOID*)InputVector, 0);
            R0[3] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[4] = __lvlx((VOID*)InputVector, 0);
            R0[4] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[5] = __lvlx((VOID*)InputVector, 0);
            R0[5] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[6] = __lvlx((VOID*)InputVector, 0);
            R0[6] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 
            R1[7] = __lvlx((VOID*)InputVector, 0);
            R0[7] = __lvlx((VOID*)InputVector, 4);
            InputVector += InputStride; 

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);
            R1[1] = __vspltw(R1[1], 0);
            R0[1] = __vspltw(R0[1], 0);
            R1[2] = __vspltw(R1[2], 0);
            R0[2] = __vspltw(R0[2], 0);
            R1[3] = __vspltw(R1[3], 0);
            R0[3] = __vspltw(R0[3], 0);
            R1[4] = __vspltw(R1[4], 0);
            R0[4] = __vspltw(R0[4], 0);
            R1[5] = __vspltw(R1[5], 0);
            R0[5] = __vspltw(R0[5], 0);
            R1[6] = __vspltw(R1[6], 0);
            R0[6] = __vspltw(R0[6], 0);
            R1[7] = __vspltw(R1[7], 0);
            R0[7] = __vspltw(R0[7], 0);

            R0[0] = __vmulfp(M.r[1], R0[0]);
            R0[1] = __vmulfp(M.r[1], R0[1]);
            R0[2] = __vmulfp(M.r[1], R0[2]);
            R0[3] = __vmulfp(M.r[1], R0[3]);
            R0[4] = __vmulfp(M.r[1], R0[4]);
            R0[5] = __vmulfp(M.r[1], R0[5]);
            R0[6] = __vmulfp(M.r[1], R0[6]);
            R0[7] = __vmulfp(M.r[1], R0[7]);

            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);
            R0[1] = __vmaddfp(R1[1], M.r[0], R0[1]);
            R0[2] = __vmaddfp(R1[2], M.r[0], R0[2]);
            R0[3] = __vmaddfp(R1[3], M.r[0], R0[3]);
            R0[4] = __vmaddfp(R1[4], M.r[0], R0[4]);
            R0[5] = __vmaddfp(R1[5], M.r[0], R0[5]);
            R0[6] = __vmaddfp(R1[6], M.r[0], R0[6]);
            R0[7] = __vmaddfp(R1[7], M.r[0], R0[7]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);
                                
            R1[1] = __vspltw(R0[1], 0);
            R0[1] = __vspltw(R0[1], 1);
                                
            R1[2] = __vspltw(R0[2], 0);
            R0[2] = __vspltw(R0[2], 1);
                                
            R1[3] = __vspltw(R0[3], 0);
            R0[3] = __vspltw(R0[3], 1);
                                
            R1[4] = __vspltw(R0[4], 0);
            R0[4] = __vspltw(R0[4], 1);
                                
            R1[5] = __vspltw(R0[5], 0);
            R0[5] = __vspltw(R0[5], 1);
                                
            R1[6] = __vspltw(R0[6], 0);
            R0[6] = __vspltw(R0[6], 1);
                                
            R1[7] = __vspltw(R0[7], 0);
            R0[7] = __vspltw(R0[7], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[1], (VOID*)OutputVector, 0);
            __stvewx(R0[1], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[2], (VOID*)OutputVector, 0);
            __stvewx(R0[2], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[3], (VOID*)OutputVector, 0);
            __stvewx(R0[3], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[4], (VOID*)OutputVector, 0);
            __stvewx(R0[4], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[5], (VOID*)OutputVector, 0);
            __stvewx(R0[5], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[6], (VOID*)OutputVector, 0);
            __stvewx(R0[6], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
            __stvewx(R1[7], (VOID*)OutputVector, 0);
            __stvewx(R0[7], (VOID*)OutputVector, 4);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvlx((VOID*)InputVector, 0);
            R0[0] = __lvlx((VOID*)InputVector, 4);

            R1[0] = __vspltw(R1[0], 0);
            R0[0] = __vspltw(R0[0], 0);

            R0[0] = __vmulfp(M.r[1], R0[0]);
            R0[0] = __vmaddfp(R1[0], M.r[0], R0[0]);

            R1[0] = __vspltw(R0[0], 0);
            R0[0] = __vspltw(R0[0], 1);

            __stvewx(R1[0], (VOID*)OutputVector, 0);
            __stvewx(R0[0], (VOID*)OutputVector, 4);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

/****************************************************************************
 *
 * 3D Vector
 *
 ****************************************************************************/

//------------------------------------------------------------------------------
// Comparison operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3Equal
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] == V2.vector4_f32[0]) && (V1.vector4_f32[1] == V2.vector4_f32[1]) && (V1.vector4_f32[2] == V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3EqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector3EqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V1.vector4_f32[0] == V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] == V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] == V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] != V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] != V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] != V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x18);
    C2 = __vpermwi(V2, 0x18);

    __vcmpeqfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3EqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] == V2.vector4_u32[0]) && (V1.vector4_u32[1] == V2.vector4_u32[1]) && (V1.vector4_u32[2] == V2.vector4_u32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector3EqualIntR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V1.vector4_u32[0] == V2.vector4_u32[0]) && 
        (V1.vector4_u32[1] == V2.vector4_u32[1]) &&
        (V1.vector4_u32[2] == V2.vector4_u32[2]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_u32[0] != V2.vector4_u32[0]) && 
        (V1.vector4_u32[1] != V2.vector4_u32[1]) &&
        (V1.vector4_u32[2] != V2.vector4_u32[2]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x18);
    C2 = __vpermwi(V2, 0x18);

    __vcmpequwR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3NearEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Epsilon
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT dx, dy, dz;

    dx = fabsf(V1.vector4_f32[0]-V2.vector4_f32[0]);
    dy = fabsf(V1.vector4_f32[1]-V2.vector4_f32[1]);
    dz = fabsf(V1.vector4_f32[2]-V2.vector4_f32[2]);
    return (((dx <= Epsilon.vector4_f32[0]) &&
            (dy <= Epsilon.vector4_f32[1]) &&
            (dz <= Epsilon.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    Delta;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    Delta = __vsubfp(V1, V2);
    Delta = __vrlimi(Delta, Epsilon, 0x1, 0);
    __vcmpbfpR(Delta, Epsilon, &CR);

    return XMComparisonAllInBounds(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3NotEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] != V2.vector4_f32[0]) || (V1.vector4_f32[1] != V2.vector4_f32[1]) || (V1.vector4_f32[2] != V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAnyFalse(XMVector3EqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3NotEqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] != V2.vector4_u32[0]) || (V1.vector4_u32[1] != V2.vector4_u32[1]) || (V1.vector4_u32[2] != V2.vector4_u32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAnyFalse(XMVector3EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3Greater
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] > V2.vector4_f32[0]) && (V1.vector4_f32[1] > V2.vector4_f32[1]) && (V1.vector4_f32[2] > V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3GreaterR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector3GreaterR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V1.vector4_f32[0] > V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] > V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] > V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] <= V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] <= V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] <= V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x18);
    C2 = __vpermwi(V2, 0x18);

    __vcmpgtfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3GreaterOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] >= V2.vector4_f32[0]) && (V1.vector4_f32[1] >= V2.vector4_f32[1]) && (V1.vector4_f32[2] >= V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3GreaterOrEqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector3GreaterOrEqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;
    if ((V1.vector4_f32[0] >= V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] >= V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] >= V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] < V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] < V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] < V2.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V1, 0x18);
    C2 = __vpermwi(V2, 0x18);

    __vcmpgefpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3Less
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] < V2.vector4_f32[0]) && (V1.vector4_f32[1] < V2.vector4_f32[1]) && (V1.vector4_f32[2] < V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3GreaterR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3LessOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] <= V2.vector4_f32[0]) && (V1.vector4_f32[1] <= V2.vector4_f32[1]) && (V1.vector4_f32[2] <= V2.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    return XMComparisonAllTrue(XMVector3GreaterOrEqualR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3InBounds
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) &&
        (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllInBounds(XMVector3InBoundsR(V, Bounds));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector3InBoundsR
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) &&
        (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2]))
    {
        CR = XM_CRMASK_CR6BOUNDS;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C1, C2;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C1 = __vpermwi(V, 0x18);
    C2 = __vpermwi(Bounds, 0x18);

    __vcmpbfpR(C1, C2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3IsNaN
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    return (XMISNAN(V.vector4_f32[0]) ||
            XMISNAN(V.vector4_f32[1]) ||
            XMISNAN(V.vector4_f32[2]));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    C;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    C = __vpermwi(V, 0x18);
    __vcmpeqfpR(C, C, &CR);

    return XMComparisonAnyFalse(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector3IsInfinite
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (XMISINF(V.vector4_f32[0]) ||
            XMISINF(V.vector4_f32[1]) ||
            XMISINF(V.vector4_f32[2]));
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR OneInt, TrueInt;
    XMVECTOR XYZX, VShifted, InfinityShifted;
    XMVECTOR Shift;
    UINT     CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    XYZX = __vpermwi(V, 0x18);
    OneInt = __vspltisw(1);
    TrueInt = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    VShifted = __vslw(XYZX, OneInt);
    InfinityShifted = __vslw(TrueInt, Shift);

    __vcmpequwR(VShifted, InfinityShifted, &CR);

    return XMComparisonAnyTrue(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Computation operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Dot
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fValue = V1.vector4_f32[0] * V2.vector4_f32[0] + V1.vector4_f32[1] * V2.vector4_f32[1] + V1.vector4_f32[2] * V2.vector4_f32[2];
    XMVECTOR vResult = {
        fValue,
        fValue,
        fValue,
        fValue
    };            
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Overflow will result in a QNaN instead of +/- infinity.

    return __vmsum3fp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Cross
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR vResult = {
        (V1.vector4_f32[1] * V2.vector4_f32[2]) - (V1.vector4_f32[2] * V2.vector4_f32[1]),
        (V1.vector4_f32[2] * V2.vector4_f32[0]) - (V1.vector4_f32[0] * V2.vector4_f32[2]),
        (V1.vector4_f32[0] * V2.vector4_f32[1]) - (V1.vector4_f32[1] * V2.vector4_f32[0]),
        0.0f
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR CrossV1;
    XMVECTOR CrossV2;
    XMVECTOR CrossR;

    CrossV1 = __vpermwi(V1, 0x63);
    CrossV2 = __vpermwi(V2, 0x87);
    CrossR = __vmulfp(CrossV1, CrossV2);
    CrossV1 = __vpermwi(V1, 0x87);
    CrossV2 = __vpermwi(V2, 0x63);
    CrossR = __vnmsubfp(CrossV1, CrossV2, CrossR);

    return CrossR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3LengthSq
(
    FXMVECTOR V
)
{
    return XMVector3Dot(V, V);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3ReciprocalLengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector3LengthSq(V);
    Result = XMVectorReciprocalSqrtEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;

    Result = __vmsum3fp(V, V);
    Result = __vrsqrtefp(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3ReciprocalLength
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector3LengthSq(V);
    Result = XMVectorReciprocalSqrt(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR H;

    D = __vmsum3fp(V, V);
    H = __vspltisw(1);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);

    return Rsq;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3LengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector3LengthSq(V);
    Result = XMVectorSqrtEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Result;

    // Returns a QNaN on infinite vectors.

    XMVECTOR Zero;
    XMVECTOR Length;

    D = __vmsum3fp(V, V);
    Zero = __vspltisw(0);
    Rsq = __vrsqrtefp(D);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Length
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector3LengthSq(V);
    Result = XMVectorSqrt(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR Length;
    XMVECTOR H;

    H = __vspltisw(1);
    D = __vmsum3fp(V, V);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    Zero = __vspltisw(0);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// XMVector3NormalizeEst uses a reciprocal estimate and
// returns QNaN on zero and infinite vectors.

XMFINLINE XMVECTOR XMVector3NormalizeEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result = XMVector3ReciprocalLength(V);
    Result = XMVectorMultiply(V, Result);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Result;
    // Faster, but returns a QNaN on zero and infinite vectors.
    D = __vmsum3fp(V, V);
    Rsq = __vrsqrtefp(D);
    Result = __vmulfp(V, Rsq);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Normalize
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fLength;
    XMVECTOR vResult;

    vResult = XMVector3Length( V );
    fLength = vResult.vector4_f32[0];

    // Prevent divide by zero
    if (fLength > 0) {
        fLength = 1.0f/fLength;
    }
    
    vResult.vector4_f32[0] = V.vector4_f32[0]*fLength;
    vResult.vector4_f32[1] = V.vector4_f32[1]*fLength;
    vResult.vector4_f32[2] = V.vector4_f32[2]*fLength;
    vResult.vector4_f32[3] = V.vector4_f32[3]*fLength;
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR RcpLength;
    XMVECTOR OneHalfLengthSq;
    XMVECTOR Scale;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR H;
    XMVECTOR N;

    H = __vspltisw(1);
    D = __vmsum3fp(V, V);
    H = __vcfsx(H, 1);
    Zero = __vspltisw(0);
    RcpLength = __vrsqrtefp(D);
    OneHalfLengthSq = __vmulfp(D, H);
    Result = __vcmpeqfp(D, Zero);
    Scale = __vmulfp(RcpLength, RcpLength);
    H = __vnmsubfp(OneHalfLengthSq, Scale, H);
    H = __vmaddcfp(RcpLength, H, RcpLength);
    N = __vmulfp(V, H);
    Result = __vsel(N, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3ClampLength
(
    FXMVECTOR V, 
    FLOAT    LengthMin, 
    FLOAT    LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax = XMVectorReplicate(LengthMax);
    ClampMin = XMVectorReplicate(LengthMin);

    return XMVector3ClampLengthV(V, ClampMin, ClampMax);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax.vector4_f32[0] =
    ClampMax.vector4_f32[1] = 
    ClampMax.vector4_f32[2] =
    ClampMax.vector4_f32[3] = LengthMax;

    ClampMin.vector4_f32[0] =
    ClampMin.vector4_f32[1] = 
    ClampMin.vector4_f32[2] =
    ClampMin.vector4_f32[3] = LengthMin;

    return XMVector3ClampLengthV(V, ClampMin, ClampMax);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    XMASSERT(((UINT_PTR)&LengthMin & 3) == 0);
    XMASSERT(((UINT_PTR)&LengthMax & 3) == 0);

    ClampMax = __lvlx(&LengthMax, 0);
    ClampMin = __lvlx(&LengthMin, 0);

    ClampMax = __vspltw(ClampMax, 0);
    ClampMin = __vspltw(ClampMin, 0);

    return XMVector3ClampLengthV(V, ClampMin, ClampMax);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3ClampLengthV
(
    FXMVECTOR V, 
    FXMVECTOR LengthMin, 
    FXMVECTOR LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR RcpLength;
    XMVECTOR Length;
    XMVECTOR Normal;
    XMVECTOR Zero;
    XMVECTOR InfiniteLength;
    XMVECTOR ZeroLength;
    XMVECTOR Select;
    XMVECTOR ControlMax;
    XMVECTOR ControlMin;
    XMVECTOR Control;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[2] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[2] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector3GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector3GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector3GreaterOrEqual(LengthMax, LengthMin));

    LengthSq = XMVector3LengthSq(V);

    Zero = XMVectorZero();

    RcpLength = XMVectorReciprocalSqrt(LengthSq);

    InfiniteLength = XMVectorEqualInt(LengthSq, g_XMInfinity.v);
    ZeroLength = XMVectorEqual(LengthSq, Zero);

    Normal = XMVectorMultiply(V, RcpLength);

    Length = XMVectorMultiply(LengthSq, RcpLength);

    Select = XMVectorEqualInt(InfiniteLength, ZeroLength);
    Length = XMVectorSelect(LengthSq, Length, Select);
    Normal = XMVectorSelect(LengthSq, Normal, Select);

    ControlMax = XMVectorGreater(Length, LengthMax);
    ControlMin = XMVectorLess(Length, LengthMin);

    ClampLength = XMVectorSelect(Length, LengthMax, ControlMax);
    ClampLength = XMVectorSelect(ClampLength, LengthMin, ControlMin);

    Result = XMVectorMultiply(Normal, ClampLength);

    // Preserve the original vector (with no precision loss) if the length falls within the given range
    Control = XMVectorEqualInt(ControlMax, ControlMin);
    Result = XMVectorSelect(Result, V, Control);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR Length, LengthS;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Normal, NormalS;
    XMVECTOR Zero;
    XMVECTOR Control;
    XMVECTOR H;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[2] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[2] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector3GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector3GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector3GreaterOrEqual(LengthMax, LengthMin));

    H = __vspltisw(1);

    LengthSq = __vmsum3fp(V, V);

    H = __vcfsx(H, 1);

    Rsq = __vrsqrtefp(LengthSq);
    RT = __vmulfp(LengthSq, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    H = __vmaddcfp(Rsq, H, Rsq);

    Zero = __vspltisw(0);

    Length = __vmulfp(LengthSq, H);

    Normal = __vmulfp(V, H);

    LengthS = __vcmpeqfp(LengthSq, Zero);
    NormalS = __vcmpeqfp(LengthSq, Zero);

    LengthS = __vsel(Length, LengthSq, LengthS);
    NormalS = __vsel(Normal, LengthSq, NormalS);

    Control = __vcmpgtfp(LengthS, LengthMax);
    ClampLength = __vcmpgtfp(LengthMin, LengthS);

    Result = __vcmpequw(Control, ClampLength);

    Control = __vsel(LengthS, LengthMax, Control);
    ClampLength = __vsel(Control, LengthMin, ClampLength);

    RT = __vmulfp(NormalS, ClampLength);

    Result = __vsel(RT, V, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Reflect
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    // Result = Incident - (2 * dot(Incident, Normal)) * Normal
    Result = XMVector3Dot(Incident, Normal);
    Result = XMVectorAdd(Result, Result);
    Result = XMVectorNegativeMultiplySubtract(Result, Normal, Incident);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Result;

    D = __vmsum3fp(Incident, Normal);
    D = __vaddfp(D, D);
    Result = Incident;
    Result = __vnmsubfp(D, Normal, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Refract
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FLOAT    RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Index;
    Index = XMVectorReplicate(RefractionIndex);
    return XMVector3RefractV(Incident, Normal, Index);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR Index;

    Index.vector4_f32[0] =
    Index.vector4_f32[1] = 
    Index.vector4_f32[2] =
    Index.vector4_f32[3] = RefractionIndex;

    return XMVector3RefractV(Incident, Normal, Index);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Index;

    XMASSERT(((UINT_PTR)&RefractionIndex & 3) == 0);

    Index = __lvlx(&RefractionIndex, 0);
    Index = __vspltw(Index, 0);

    return XMVector3RefractV(Incident, Normal, Index);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3RefractV
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FXMVECTOR RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR        IDotN;
    XMVECTOR        R;
    CONST XMVECTOR  Zero = XMVectorZero();

    // Result = RefractionIndex * Incident - Normal * (RefractionIndex * dot(Incident, Normal) + 
    // sqrt(1 - RefractionIndex * RefractionIndex * (1 - dot(Incident, Normal) * dot(Incident, Normal))))

    IDotN = XMVector3Dot(Incident, Normal);

    // R = 1.0f - RefractionIndex * RefractionIndex * (1.0f - IDotN * IDotN)
    R = XMVectorNegativeMultiplySubtract(IDotN, IDotN, g_XMOne.v);
    R = XMVectorMultiply(R, RefractionIndex);
    R = XMVectorNegativeMultiplySubtract(R, RefractionIndex, g_XMOne.v);

    if (XMVector4LessOrEqual(R, Zero))
    {
        // Total internal reflection
        return Zero;
    }
    else
    {
        XMVECTOR Result;

        // R = RefractionIndex * IDotN + sqrt(R)
        R = XMVectorSqrt(R);
        R = XMVectorMultiplyAdd(RefractionIndex, IDotN, R);

        // Result = RefractionIndex * Incident - Normal * R
        Result = XMVectorMultiply(RefractionIndex, Incident);
        Result = XMVectorNegativeMultiplySubtract(Normal, R, Result);

        return Result;
    }

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR IDotN;
    XMVECTOR R;
    XMVECTOR I;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR ZL;
    XMVECTOR D1;
    XMVECTOR H;
    XMVECTOR Zero;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(R);

    IDotN = __vmsum3fp(Incident, Normal);

    H = __vspltisw(1);
    R = __vupkd3d(R, VPACK_NORMSHORT2);
    H = __vcfsx(H, 1);
    RT = __vspltw(R, 3);
    R = __vspltw(R, 3);

    RT = __vnmsubfp(IDotN, IDotN, RT);
    RT = __vmulfp(RT, RefractionIndex);
    R = __vnmsubfp(RT, RefractionIndex, R);

    Rsq = __vrsqrtefp(R);
    RT = __vmulfp(R, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Zero = __vspltisw(0);
    H = __vmaddcfp(Rsq, H, Rsq);

    ZL = __vcmpeqfp(R, Zero);

    Result = __vcmpgefp(Zero, R);

    D1 = __vmulfp(R, H);

    I = __vmulfp(RefractionIndex, Incident);

    ZL = __vsel(D1, R, ZL);

    ZL = __vmaddfp(RefractionIndex, IDotN, ZL);

    I = __vnmsubfp(Normal, ZL, I);

    Result = __vsel(I, Zero, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Orthogonal
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR NegativeV;
    XMVECTOR Z, YZYY;
    XMVECTOR ZIsNegative, YZYYIsNegative;
    XMVECTOR S, D;
    XMVECTOR R0, R1;
    XMVECTOR Select;
    XMVECTOR Zero;
    XMVECTOR Result;
    static CONST XMVECTORU32 Permute1X0X0X0X = {XM_PERMUTE_1X, XM_PERMUTE_0X, XM_PERMUTE_0X, XM_PERMUTE_0X};
    static CONST XMVECTORU32 Permute0Y0Z0Y0Y= {XM_PERMUTE_0Y, XM_PERMUTE_0Z, XM_PERMUTE_0Y, XM_PERMUTE_0Y};

    Zero = XMVectorZero();
    Z = XMVectorSplatZ(V);
    YZYY = XMVectorPermute(V, V, Permute0Y0Z0Y0Y.v);

    NegativeV = XMVectorSubtract(Zero, V);

    ZIsNegative = XMVectorLess(Z, Zero);
    YZYYIsNegative = XMVectorLess(YZYY, Zero);

    S = XMVectorAdd(YZYY, Z);
    D = XMVectorSubtract(YZYY, Z);

    Select = XMVectorEqualInt(ZIsNegative, YZYYIsNegative);

    R0 = XMVectorPermute(NegativeV, S, Permute1X0X0X0X.v);
    R1 = XMVectorPermute(V, D, Permute1X0X0X0X.v);

    Result = XMVectorSelect(R1, R0, Select);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR NegativeV;
    XMVECTOR Z, YZYY;
    XMVECTOR ZIsNegative, YZYYIsNegative;
    XMVECTOR S, D;
    XMVECTOR R0, R1;
    XMVECTOR SignMask;
    XMVECTOR Zero;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    Zero = __vspltisw(0);
    Z = __vspltw(V, 2);
    YZYY = __vpermwi(V, 0x65);

    NegativeV = __vxor(V, SignMask);

    ZIsNegative = __vcmpgtfp(Zero, Z);
    YZYYIsNegative = __vcmpgtfp(Zero, YZYY);

    S = __vaddfp(YZYY, Z);
    D = __vsubfp(YZYY, Z);
    R0 = __vspltw(NegativeV, 0);
    R1 = __vspltw(V, 0);

    Result = __vcmpequw(ZIsNegative, YZYYIsNegative);

    R0 = __vrlimi(R0, S, 0x8, 0);
    R1 = __vrlimi(R1, D, 0x8, 0);

    Result = __vsel(R1, R0, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3AngleBetweenNormalsEst
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    XMVECTOR NegativeOne;
    XMVECTOR One;

    Result = XMVector3Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACosEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    CosAngle = __vmsum3fp(N1, N2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    CosAngle = __vmaxfp(NegativeOne, CosAngle);
    CosAngle = __vminfp(One, CosAngle);
    Result = XMVectorACosEst(CosAngle);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3AngleBetweenNormals
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    XMVECTOR NegativeOne;
    XMVECTOR One;

    Result = XMVector3Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACos(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;
    XMVECTOR NegativeOne;
    XMVECTOR One;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Result = __vmsum3fp(N1, N2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    Result = __vmaxfp(NegativeOne, Result);
    Result = __vminfp(One, Result);

    Result = XMVectorACos(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3AngleBetweenVectors
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR L1;
    XMVECTOR L2;
    XMVECTOR Dot;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    L1 = XMVector3ReciprocalLength(V1);
    L2 = XMVector3ReciprocalLength(V2);

    Dot = XMVector3Dot(V1, V2);

    L1 = XMVectorMultiply(L1, L2);

    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();

    CosAngle = XMVectorMultiply(Dot, L1);

    CosAngle = XMVectorClamp(CosAngle, NegativeOne, One);

    Result = XMVectorACos(CosAngle);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D1, D2;
    XMVECTOR Rsq1, Rsq2;
    XMVECTOR Rcp1, Rcp2;
    XMVECTOR RT1, RT2;
    XMVECTOR H1, H2;
    XMVECTOR L;
    XMVECTOR Dot;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    D1 = __vmsum3fp(V1, V1);
    D2 = __vmsum3fp(V2, V2);
    H2 = __vspltisw(1);
    H1 = __vcfsx(H2, 1);
    H2 = __vcfsx(H2, 1);
    Rsq1 = __vrsqrtefp(D1);
    Rsq2 = __vrsqrtefp(D2);
    RT1 = __vmulfp(D1, H1);
    RT2 = __vmulfp(D2, H2);
    Rcp1 = __vmulfp(Rsq1, Rsq1);
    Rcp2 = __vmulfp(Rsq2, Rsq2);
    H1 = __vnmsubfp(RT1, Rcp1, H1);
    H2 = __vnmsubfp(RT2, Rcp2, H2);
    Rsq1 = __vmaddfp(Rsq1, H1, Rsq1);
    Rsq2 = __vmaddfp(Rsq2, H2, Rsq2);
    Dot = __vmsum3fp(V1, V2);
    L = __vmulfp(Rsq1, Rsq2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    CosAngle = __vmulfp(Dot, L);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    CosAngle = __vmaxfp(NegativeOne, CosAngle);
    CosAngle = __vminfp(One, CosAngle);

    Result = XMVectorACos(CosAngle);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3LinePointDistance
(
    FXMVECTOR LinePoint1, 
    FXMVECTOR LinePoint2, 
    FXMVECTOR Point
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR PointVector;
    XMVECTOR LineVector;
    XMVECTOR ReciprocalLengthSq;
    XMVECTOR PointProjectionScale;
    XMVECTOR DistanceVector;
    XMVECTOR Result;

    // Given a vector PointVector from LinePoint1 to Point and a vector
    // LineVector from LinePoint1 to LinePoint2, the scaled distance 
    // PointProjectionScale from LinePoint1 to the perpendicular projection
    // of PointVector onto the line is defined as:
    //
    //     PointProjectionScale = dot(PointVector, LineVector) / LengthSq(LineVector)

    PointVector = XMVectorSubtract(Point, LinePoint1);
    LineVector = XMVectorSubtract(LinePoint2, LinePoint1);

    ReciprocalLengthSq = XMVector3LengthSq(LineVector);
    ReciprocalLengthSq = XMVectorReciprocal(ReciprocalLengthSq);

    PointProjectionScale = XMVector3Dot(PointVector, LineVector);
    PointProjectionScale = XMVectorMultiply(PointProjectionScale, ReciprocalLengthSq);

    DistanceVector = XMVectorMultiply(LineVector, PointProjectionScale);
    DistanceVector = XMVectorSubtract(PointVector, DistanceVector);

    Result = XMVector3Length(DistanceVector);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR PointVector;
    XMVECTOR LineVector;
    XMVECTOR ReciprocalLengthSq;
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR PointProjectionScale;
    XMVECTOR DistanceVector;
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Zero;
    XMVECTOR Length;
    XMVECTOR H;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    LineVector = __vsubfp(LinePoint2, LinePoint1);

    ReciprocalLengthSq = __vmsum3fp(LineVector, LineVector);

    Reciprocal = __vrefp(ReciprocalLengthSq);

    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(ReciprocalLengthSq, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);

    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(ReciprocalLengthSq, S0, S1);

    PointVector = __vsubfp(Point, LinePoint1);

    ReciprocalLengthSq = __vcmpeqfp(S0, S0);

    S0 = __vmaddfp(S0, S1, S0);

    PointProjectionScale = __vmsum3fp(PointVector, LineVector);

    ReciprocalLengthSq = __vsel(Reciprocal, S0, ReciprocalLengthSq);

    PointProjectionScale = __vmulfp(PointProjectionScale, ReciprocalLengthSq);

    DistanceVector = __vmulfp(LineVector, PointProjectionScale);
    DistanceVector = __vsubfp(PointVector, DistanceVector);

    H = __vspltisw(1);
    D = __vmsum3fp(DistanceVector, DistanceVector);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    Zero = __vspltisw(0);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE VOID XMVector3ComponentsFromNormal
(
    XMVECTOR* pParallel, 
    XMVECTOR* pPerpendicular, 
    FXMVECTOR  V, 
    FXMVECTOR  Normal
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Parallel;
    XMVECTOR Scale;

    XMASSERT(pParallel);
    XMASSERT(pPerpendicular);

    Scale = XMVector3Dot(V, Normal);

    Parallel = XMVectorMultiply(Normal, Scale);

    *pParallel = Parallel;
    *pPerpendicular = XMVectorSubtract(V, Parallel);

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Parallel;
    XMVECTOR Scale;

    XMASSERT(pParallel);
    XMASSERT(pPerpendicular);

    Scale = __vmsum3fp(V, Normal);

    Parallel = __vmulfp(Normal, Scale);

    *pParallel = Parallel;
    *pPerpendicular = __vsubfp(V, Parallel);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Transform a vector using a rotation expressed as a unit quaternion

XMFINLINE XMVECTOR XMVector3Rotate
(
    FXMVECTOR V, 
    FXMVECTOR RotationQuaternion
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR A;
    XMVECTOR Q;
    XMVECTOR Result;

    A = XMVectorSelect(g_XMSelect1110.v, V, g_XMSelect1110.v);
    Q = XMQuaternionConjugate(RotationQuaternion);
    Result = XMQuaternionMultiply(Q, A);
    Result = XMQuaternionMultiply(Result, RotationQuaternion);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Q, A, QA;
    XMVECTOR Q10, Q11, Q12, Q13;
    XMVECTOR Q2WZYX, Q2ZWXY, Q2YXWZ;
    XMVECTOR RX, RY, RZ, RW, RXZ, RYW, Result;
    XMVECTOR SignMask;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);
    A = __vspltisw(0);

    Q = __vxor(RotationQuaternion, SignMask);
    A = __vrlimi(A, V, 0xE, 0);
    Q = __vrlimi(Q, RotationQuaternion, 0x1, 0);

    Q2ZWXY = __vpermwi(A, 0xB1);
    Q2YXWZ = __vpermwi(A, 0x4E);
    Q2WZYX = __vpermwi(A, 0xE4);

    Q13 = __vxor(Q, SignMask);
    Q11 = __vxor(Q, SignMask);
    Q12 = __vxor(Q, SignMask);
    Q10 = __vxor(Q, SignMask);

    Q13 = __vrlimi(Q13, Q, 0x1, 0);
    Q11 = __vrlimi(Q11, Q, 0xD, 0);
    Q12 = __vrlimi(Q12, Q, 0x7, 0);
    Q10 = __vrlimi(Q10, Q, 0xB, 0);

    RW = __vmsum4fp(Q13, A);
    RY = __vmsum4fp(Q11, Q2ZWXY);
    RZ = __vmsum4fp(Q12, Q2YXWZ);
    RX = __vmsum4fp(Q10, Q2WZYX);

    RYW = __vmrghw(RY, RW);
    RXZ = __vmrghw(RX, RZ);
    QA = __vmrghw(RXZ, RYW);

    Q2ZWXY = __vpermwi(RotationQuaternion, 0xB1);
    Q2YXWZ = __vpermwi(RotationQuaternion, 0x4E);
    Q2WZYX = __vpermwi(RotationQuaternion, 0xE4);

    Q13 = __vxor(QA, SignMask);
    Q11 = __vxor(QA, SignMask);
    Q12 = __vxor(QA, SignMask);
    Q10 = __vxor(QA, SignMask);

    Q13 = __vrlimi(Q13, QA, 0x1, 0);
    Q11 = __vrlimi(Q11, QA, 0xD, 0);
    Q12 = __vrlimi(Q12, QA, 0x7, 0);
    Q10 = __vrlimi(Q10, QA, 0xB, 0);

    RW = __vmsum4fp(Q13, RotationQuaternion);
    RY = __vmsum4fp(Q11, Q2ZWXY);
    RZ = __vmsum4fp(Q12, Q2YXWZ);
    RX = __vmsum4fp(Q10, Q2WZYX);

    RYW = __vmrghw(RY, RW);
    RXZ = __vmrghw(RX, RZ);
    Result = __vmrghw(RXZ, RYW);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Transform a vector using the inverse of a rotation expressed as a unit quaternion

XMFINLINE XMVECTOR XMVector3InverseRotate
(
    FXMVECTOR V, 
    FXMVECTOR RotationQuaternion
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR A;
    XMVECTOR Q;
    XMVECTOR Result;

    A = XMVectorSelect(g_XMSelect1110.v, V, g_XMSelect1110.v);
    Result = XMQuaternionMultiply(RotationQuaternion, A);
    Q = XMQuaternionConjugate(RotationQuaternion);
    Result = XMQuaternionMultiply(Result, Q);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Q10, Q11, Q12, Q13;
    XMVECTOR Q2WZYX, Q2ZWXY, Q2YXWZ;
    XMVECTOR RX, RY, RZ, RW, RXZ, RYW, Result;
    XMVECTOR SignMask;
    XMVECTOR A, Q, QA;

    A = __vspltisw(0);
    A = __vrlimi(A, V, 0xE, 0);

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    Q2ZWXY = __vpermwi(A, 0xB1);
    Q2YXWZ = __vpermwi(A, 0x4E);
    Q2WZYX = __vpermwi(A, 0xE4);

    Q13 = __vxor(RotationQuaternion, SignMask);
    Q11 = __vxor(RotationQuaternion, SignMask);
    Q12 = __vxor(RotationQuaternion, SignMask);
    Q10 = __vxor(RotationQuaternion, SignMask);

    Q13 = __vrlimi(Q13, RotationQuaternion, 0x1, 0);
    Q11 = __vrlimi(Q11, RotationQuaternion, 0xD, 0);
    Q12 = __vrlimi(Q12, RotationQuaternion, 0x7, 0);
    Q10 = __vrlimi(Q10, RotationQuaternion, 0xB, 0);

    RW = __vmsum4fp(Q13, A);
    RY = __vmsum4fp(Q11, Q2ZWXY);
    RZ = __vmsum4fp(Q12, Q2YXWZ);
    RX = __vmsum4fp(Q10, Q2WZYX);

    Q = __vxor(RotationQuaternion, SignMask);
    RYW = __vmrghw(RY, RW);
    RXZ = __vmrghw(RX, RZ);
    Q = __vrlimi(Q, RotationQuaternion, 0x1, 0);
    QA = __vmrghw(RXZ, RYW);

    Q2ZWXY = __vpermwi(Q, 0xB1);
    Q2YXWZ = __vpermwi(Q, 0x4E);
    Q2WZYX = __vpermwi(Q, 0xE4);

    Q13 = __vxor(QA, SignMask);
    Q11 = __vxor(QA, SignMask);
    Q12 = __vxor(QA, SignMask);
    Q10 = __vxor(QA, SignMask);

    Q13 = __vrlimi(Q13, QA, 0x1, 0);
    Q11 = __vrlimi(Q11, QA, 0xD, 0);
    Q12 = __vrlimi(Q12, QA, 0x7, 0);
    Q10 = __vrlimi(Q10, QA, 0xB, 0);

    RW = __vmsum4fp(Q13, Q);
    RY = __vmsum4fp(Q11, Q2ZWXY);
    RZ = __vmsum4fp(Q12, Q2YXWZ);
    RX = __vmsum4fp(Q10, Q2WZYX);

    RYW = __vmrghw(RY, RW);
    RXZ = __vmrghw(RX, RZ);
    Result = __vmrghw(RXZ, RYW);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Transform
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;

    Z = XMVectorSplatZ(V);
    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
    Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Result;

    Result = __vspltw(V, 2);
    Y = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmaddcfp(M.r[2], Result, M.r[3]);
    Result = __vmaddfp(Y, M.r[1], Result);
    Result = __vmaddfp(X, M.r[0], Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT4* XMVector3TransformStream
(
    XMFLOAT4*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);

        Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat4((XMFLOAT4*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);

        Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat4NC((XMFLOAT4*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMMATRIX P, MT;
    XMVECTOR C1W[8], R0[8], R1[8], R2[8], R3[8];
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(C1W[0]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[1]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[2]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[3]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[4]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[5]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[6]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[7]);

    C1W[0] = __vupkd3d(C1W[0], VPACK_NORMSHORT2);
    C1W[1] = __vupkd3d(C1W[1], VPACK_NORMSHORT2);
    C1W[2] = __vupkd3d(C1W[2], VPACK_NORMSHORT2);
    C1W[3] = __vupkd3d(C1W[3], VPACK_NORMSHORT2);
    C1W[4] = __vupkd3d(C1W[4], VPACK_NORMSHORT2);
    C1W[5] = __vupkd3d(C1W[5], VPACK_NORMSHORT2);
    C1W[6] = __vupkd3d(C1W[6], VPACK_NORMSHORT2);
    C1W[7] = __vupkd3d(C1W[7], VPACK_NORMSHORT2);

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    if (InputStride == sizeof(XMFLOAT3))
    {
//        C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R0[0] = __vspltw(L[0], 2);
                R1[0] = __vspltw(L[0], 1);
                R2[0] = __vspltw(L[0], 0);

                R0[1] = __vspltw(L[1], 1);
                R1[1] = __vspltw(L[1], 0);
                R2[1] = __vspltw(L[0], 3);

                R0[2] = __vspltw(L[2], 0);
                R1[2] = __vspltw(L[1], 3);
                R2[2] = __vspltw(L[1], 2);

                R0[3] = __vspltw(L[2], 3);
                R1[3] = __vspltw(L[2], 2);
                R2[3] = __vspltw(L[2], 1);

                R0[4] = __vspltw(L[3], 2);
                R1[4] = __vspltw(L[3], 1);
                R2[4] = __vspltw(L[3], 0);

                R0[5] = __vspltw(L[4], 1);
                R1[5] = __vspltw(L[4], 0);
                R2[5] = __vspltw(L[3], 3);

                R0[6] = __vspltw(L[5], 0);
                R1[6] = __vspltw(L[4], 3);
                R2[6] = __vspltw(L[4], 2);

                R0[7] = __vspltw(L[5], 3);
                R1[7] = __vspltw(L[5], 2);
                R2[7] = __vspltw(L[5], 1);

                R0[0] = __vmaddcfp(R0[0], M.r[2], M.r[3]);
                R0[1] = __vmaddcfp(R0[1], M.r[2], M.r[3]);
                R0[2] = __vmaddcfp(R0[2], M.r[2], M.r[3]);
                R0[3] = __vmaddcfp(R0[3], M.r[2], M.r[3]);
                R0[4] = __vmaddcfp(R0[4], M.r[2], M.r[3]);
                R0[5] = __vmaddcfp(R0[5], M.r[2], M.r[3]);
                R0[6] = __vmaddcfp(R0[6], M.r[2], M.r[3]);
                R0[7] = __vmaddcfp(R0[7], M.r[2], M.r[3]);

                R0[0] = __vmaddcfp(R1[0], M.r[1], R0[0]);
                R0[1] = __vmaddcfp(R1[1], M.r[1], R0[1]);
                R0[2] = __vmaddcfp(R1[2], M.r[1], R0[2]);
                R0[3] = __vmaddcfp(R1[3], M.r[1], R0[3]);
                R0[4] = __vmaddcfp(R1[4], M.r[1], R0[4]);
                R0[5] = __vmaddcfp(R1[5], M.r[1], R0[5]);
                R0[6] = __vmaddcfp(R1[6], M.r[1], R0[6]);
                R0[7] = __vmaddcfp(R1[7], M.r[1], R0[7]);

                R0[0] = __vmaddcfp(R2[0], M.r[0], R0[0]);
                R0[1] = __vmaddcfp(R2[1], M.r[0], R0[1]);
                R0[2] = __vmaddcfp(R2[2], M.r[0], R0[2]);
                R0[3] = __vmaddcfp(R2[3], M.r[0], R0[3]);
                R0[4] = __vmaddcfp(R2[4], M.r[0], R0[4]);
                R0[5] = __vmaddcfp(R2[5], M.r[0], R0[5]);
                R0[6] = __vmaddcfp(R2[6], M.r[0], R0[6]);
                R0[7] = __vmaddcfp(R2[7], M.r[0], R0[7]);

                __stvlx(R0[0], (VOID*)OutputVector, 0);
                __stvrx(R0[0], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[1], (VOID*)OutputVector, 0);
                __stvrx(R0[1], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[2], (VOID*)OutputVector, 0);
                __stvrx(R0[2], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[3], (VOID*)OutputVector, 0);
                __stvrx(R0[3], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[4], (VOID*)OutputVector, 0);
                __stvrx(R0[4], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[5], (VOID*)OutputVector, 0);
                __stvrx(R0[5], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[6], (VOID*)OutputVector, 0);
                __stvrx(R0[6], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
                __stvlx(R0[7], (VOID*)OutputVector, 0);
                __stvrx(R0[7], (VOID*)OutputVector, 16);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT4* XMVector3TransformStreamNC
(
    XMFLOAT4*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX     M
)
{
#if defined(_XM_NO_INTRINSICS_) || defined(XM_NO_MISALIGNED_VECTOR_ACCESS) || defined(_XM_SSE_INTRINSICS_)
    return XMVector3TransformStream( pOutputStream, OutputStride, pInputStream, InputStride, VectorCount, M );
#else // _XM_VMX128_INTRINSICS_
    CONST UINT UnrollCount = 8;
    XMMATRIX P, MT;
    XMVECTOR C1W[8], R0[8], R1[8], R2[8], R3[8];
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(C1W[0]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[1]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[2]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[3]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[4]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[5]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[6]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[7]);

    C1W[0] = __vupkd3d(C1W[0], VPACK_NORMSHORT2);
    C1W[1] = __vupkd3d(C1W[1], VPACK_NORMSHORT2);
    C1W[2] = __vupkd3d(C1W[2], VPACK_NORMSHORT2);
    C1W[3] = __vupkd3d(C1W[3], VPACK_NORMSHORT2);
    C1W[4] = __vupkd3d(C1W[4], VPACK_NORMSHORT2);
    C1W[5] = __vupkd3d(C1W[5], VPACK_NORMSHORT2);
    C1W[6] = __vupkd3d(C1W[6], VPACK_NORMSHORT2);
    C1W[7] = __vupkd3d(C1W[7], VPACK_NORMSHORT2);

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    if (InputStride == sizeof(XMFLOAT3))
    {
    //    C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                C1W[0] = __vrlimi(C1W[0], L[0], 0xE, 0);
                L[0] = __vrlimi(L[0], L[1], 0xC, 0);
                L[1] = __vrlimi(L[1], L[2], 0xC, 0);
                C1W[3] = __vrlimi(C1W[3], L[2], 0xE, 1);
                C1W[4] = __vrlimi(C1W[4], L[3], 0xE, 0);
                L[3] = __vrlimi(L[3], L[4], 0xC, 0);
                L[4] = __vrlimi(L[4], L[5], 0xC, 0);
                C1W[7] = __vrlimi(C1W[7], L[5], 0xE, 1);
                C1W[1] = __vrlimi(C1W[1], L[0], 0xE, 3);
                C1W[2] = __vrlimi(C1W[2], L[1], 0xE, 2);
                C1W[5] = __vrlimi(C1W[5], L[3], 0xE, 3);
                C1W[6] = __vrlimi(C1W[6], L[4], 0xE, 2);

                R0[0] = __vmsum4fp(C1W[0], MT.r[0]);
                R1[0] = __vmsum4fp(C1W[0], MT.r[1]);
                R2[0] = __vmsum4fp(C1W[0], MT.r[2]);
                R3[0] = __vmsum4fp(C1W[0], MT.r[3]);
                                      
                R0[1] = __vmsum4fp(C1W[1], MT.r[0]);
                R1[1] = __vmsum4fp(C1W[1], MT.r[1]);
                R2[1] = __vmsum4fp(C1W[1], MT.r[2]);
                R3[1] = __vmsum4fp(C1W[1], MT.r[3]);
                                      
                R0[2] = __vmsum4fp(C1W[2], MT.r[0]);
                R1[2] = __vmsum4fp(C1W[2], MT.r[1]);
                R2[2] = __vmsum4fp(C1W[2], MT.r[2]);
                R3[2] = __vmsum4fp(C1W[2], MT.r[3]);
                                      
                R0[3] = __vmsum4fp(C1W[3], MT.r[0]);
                R1[3] = __vmsum4fp(C1W[3], MT.r[1]);
                R2[3] = __vmsum4fp(C1W[3], MT.r[2]);
                R3[3] = __vmsum4fp(C1W[3], MT.r[3]);
                                      
                R0[4] = __vmsum4fp(C1W[4], MT.r[0]);
                R1[4] = __vmsum4fp(C1W[4], MT.r[1]);
                R2[4] = __vmsum4fp(C1W[4], MT.r[2]);
                R3[4] = __vmsum4fp(C1W[4], MT.r[3]);
                                      
                R0[5] = __vmsum4fp(C1W[5], MT.r[0]);
                R1[5] = __vmsum4fp(C1W[5], MT.r[1]);
                R2[5] = __vmsum4fp(C1W[5], MT.r[2]);
                R3[5] = __vmsum4fp(C1W[5], MT.r[3]);
                                      
                R0[6] = __vmsum4fp(C1W[6], MT.r[0]);
                R1[6] = __vmsum4fp(C1W[6], MT.r[1]);
                R2[6] = __vmsum4fp(C1W[6], MT.r[2]);
                R3[6] = __vmsum4fp(C1W[6], MT.r[3]);
                                      
                R0[7] = __vmsum4fp(C1W[7], MT.r[0]);
                R1[7] = __vmsum4fp(C1W[7], MT.r[1]);
                R2[7] = __vmsum4fp(C1W[7], MT.r[2]);
                R3[7] = __vmsum4fp(C1W[7], MT.r[3]);

                __stvewx(R0[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R2[0], (VOID*)OutputVector, 8);
                __stvewx(R3[0], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R2[1], (VOID*)OutputVector, 8);
                __stvewx(R3[1], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R2[2], (VOID*)OutputVector, 8);
                __stvewx(R3[2], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R2[3], (VOID*)OutputVector, 8);
                __stvewx(R3[3], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R2[4], (VOID*)OutputVector, 8);
                __stvewx(R3[4], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R2[5], (VOID*)OutputVector, 8);
                __stvewx(R3[5], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R2[6], (VOID*)OutputVector, 8);
                __stvewx(R3[6], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                __stvewx(R0[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R2[7], (VOID*)OutputVector, 8);
                __stvewx(R3[7], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3TransformCoord
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR InverseW;
    XMVECTOR Result;

    Z = XMVectorSplatZ(V);
    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
    Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    InverseW = XMVectorSplatW(Result);
    InverseW = XMVectorReciprocal(InverseW);

    Result = XMVectorMultiply(Result, InverseW);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR InverseW;
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Result = __vspltw(V, 2);
    Y = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmaddcfp(M.r[2], Result, M.r[3]);
    Result = __vmaddfp(Y, M.r[1], Result);
    Result = __vmaddfp(X, M.r[0], Result);

    InverseW = __vspltw(Result, 3);

    Reciprocal = __vrefp(InverseW);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(InverseW, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(InverseW, S0, S1);
    InverseW = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    InverseW = __vsel(Reciprocal, S0, InverseW);

    Result = __vmulfp(Result, InverseW);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT3* XMVector3TransformCoordStream
(
    XMFLOAT3*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR InverseW;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Z = XMVectorReplicate(((XMFLOAT3*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT3*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT3*)pInputVector)->x);

        Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        InverseW = XMVectorSplatW(Result);
        InverseW = XMVectorReciprocal(InverseW);

        Result = XMVectorMultiply(Result, InverseW);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR InverseW;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Z = XMVectorReplicate(((XMFLOAT3*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT3*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT3*)pInputVector)->x);

        Result = XMVectorMultiplyAdd(Z, M.r[2], M.r[3]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        InverseW = XMVectorSplatW(Result);
        InverseW = XMVectorReciprocal(InverseW);

        Result = XMVectorMultiply(Result, InverseW);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMMATRIX P, MT;
    XMVECTOR C1W[8], R0[8], R1[8], R2[8], R3[8];
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(C1W[0]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[1]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[2]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[3]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[4]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[5]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[6]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[7]);

    C1W[0] = __vupkd3d(C1W[0], VPACK_NORMSHORT2);
    C1W[1] = __vupkd3d(C1W[1], VPACK_NORMSHORT2);
    C1W[2] = __vupkd3d(C1W[2], VPACK_NORMSHORT2);
    C1W[3] = __vupkd3d(C1W[3], VPACK_NORMSHORT2);
    C1W[4] = __vupkd3d(C1W[4], VPACK_NORMSHORT2);
    C1W[5] = __vupkd3d(C1W[5], VPACK_NORMSHORT2);
    C1W[6] = __vupkd3d(C1W[6], VPACK_NORMSHORT2);
    C1W[7] = __vupkd3d(C1W[7], VPACK_NORMSHORT2);

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    if (InputStride == sizeof(XMFLOAT3))
    {
//        C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        CONST UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R0[0] = __vspltw(L[0], 2);
                R1[0] = __vspltw(L[0], 1);
                R2[0] = __vspltw(L[0], 0);

                R0[1] = __vspltw(L[1], 1);
                R1[1] = __vspltw(L[1], 0);
                R2[1] = __vspltw(L[0], 3);

                R0[2] = __vspltw(L[2], 0);
                R1[2] = __vspltw(L[1], 3);
                R2[2] = __vspltw(L[1], 2);

                R0[3] = __vspltw(L[2], 3);
                R1[3] = __vspltw(L[2], 2);
                R2[3] = __vspltw(L[2], 1);

                R0[4] = __vspltw(L[3], 2);
                R1[4] = __vspltw(L[3], 1);
                R2[4] = __vspltw(L[3], 0);

                R0[5] = __vspltw(L[4], 1);
                R1[5] = __vspltw(L[4], 0);
                R2[5] = __vspltw(L[3], 3);

                R0[6] = __vspltw(L[5], 0);
                R1[6] = __vspltw(L[4], 3);
                R2[6] = __vspltw(L[4], 2);

                R0[7] = __vspltw(L[5], 3);
                R1[7] = __vspltw(L[5], 2);
                R2[7] = __vspltw(L[5], 1);

                R0[0] = __vmaddcfp(R0[0], M.r[2], M.r[3]);
                R0[1] = __vmaddcfp(R0[1], M.r[2], M.r[3]);
                R0[2] = __vmaddcfp(R0[2], M.r[2], M.r[3]);
                R0[3] = __vmaddcfp(R0[3], M.r[2], M.r[3]);
                R0[4] = __vmaddcfp(R0[4], M.r[2], M.r[3]);
                R0[5] = __vmaddcfp(R0[5], M.r[2], M.r[3]);
                R0[6] = __vmaddcfp(R0[6], M.r[2], M.r[3]);
                R0[7] = __vmaddcfp(R0[7], M.r[2], M.r[3]);

                R0[0] = __vmaddcfp(R1[0], M.r[1], R0[0]);
                R0[1] = __vmaddcfp(R1[1], M.r[1], R0[1]);
                R0[2] = __vmaddcfp(R1[2], M.r[1], R0[2]);
                R0[3] = __vmaddcfp(R1[3], M.r[1], R0[3]);
                R0[4] = __vmaddcfp(R1[4], M.r[1], R0[4]);
                R0[5] = __vmaddcfp(R1[5], M.r[1], R0[5]);
                R0[6] = __vmaddcfp(R1[6], M.r[1], R0[6]);
                R0[7] = __vmaddcfp(R1[7], M.r[1], R0[7]);

                R0[0] = __vmaddcfp(R2[0], M.r[0], R0[0]);
                R0[1] = __vmaddcfp(R2[1], M.r[0], R0[1]);
                R0[2] = __vmaddcfp(R2[2], M.r[0], R0[2]);
                R0[3] = __vmaddcfp(R2[3], M.r[0], R0[3]);
                R0[4] = __vmaddcfp(R2[4], M.r[0], R0[4]);
                R0[5] = __vmaddcfp(R2[5], M.r[0], R0[5]);
                R0[6] = __vmaddcfp(R2[6], M.r[0], R0[6]);
                R0[7] = __vmaddcfp(R2[7], M.r[0], R0[7]);

                R1[0] = __vspltw(R0[0], 3);
                R1[1] = __vspltw(R0[1], 3);
                R1[2] = __vspltw(R0[2], 3);
                R1[3] = __vspltw(R0[3], 3);
                R1[4] = __vspltw(R0[4], 3);
                R1[5] = __vspltw(R0[5], 3);
                R1[6] = __vspltw(R0[6], 3);
                R1[7] = __vspltw(R0[7], 3);

                R1[0] = XMVectorReciprocal(R1[0]);
                R1[1] = XMVectorReciprocal(R1[1]);
                R1[2] = XMVectorReciprocal(R1[2]);
                R1[3] = XMVectorReciprocal(R1[3]);
                R1[4] = XMVectorReciprocal(R1[4]);
                R1[5] = XMVectorReciprocal(R1[5]);
                R1[6] = XMVectorReciprocal(R1[6]);
                R1[7] = XMVectorReciprocal(R1[7]);

                R0[0] = __vmulfp(R0[0], R1[0]);
                R0[1] = __vmulfp(R0[1], R1[1]);
                R0[2] = __vmulfp(R0[2], R1[2]);
                R0[3] = __vmulfp(R0[3], R1[3]);
                R0[4] = __vmulfp(R0[4], R1[4]);
                R0[5] = __vmulfp(R0[5], R1[5]);
                R0[6] = __vmulfp(R0[6], R1[6]);
                R0[7] = __vmulfp(R0[7], R1[7]);

                R2[0] = __vspltw(R0[0], 0);
                R1[0] = __vspltw(R0[0], 1);
                R0[0] = __vspltw(R0[0], 2);
                                    
                R2[1] = __vspltw(R0[1], 0);
                R1[1] = __vspltw(R0[1], 1);
                R0[1] = __vspltw(R0[1], 2);
                                    
                R2[2] = __vspltw(R0[2], 0);
                R1[2] = __vspltw(R0[2], 1);
                R0[2] = __vspltw(R0[2], 2);
                                    
                R2[3] = __vspltw(R0[3], 0);
                R1[3] = __vspltw(R0[3], 1);
                R0[3] = __vspltw(R0[3], 2);
                                    
                R2[4] = __vspltw(R0[4], 0);
                R1[4] = __vspltw(R0[4], 1);
                R0[4] = __vspltw(R0[4], 2);
                                    
                R2[5] = __vspltw(R0[5], 0);
                R1[5] = __vspltw(R0[5], 1);
                R0[5] = __vspltw(R0[5], 2);
                                    
                R2[6] = __vspltw(R0[6], 0);
                R1[6] = __vspltw(R0[6], 1);
                R0[6] = __vspltw(R0[6], 2);
                                    
                R2[7] = __vspltw(R0[7], 0);
                R1[7] = __vspltw(R0[7], 1);
                R0[7] = __vspltw(R0[7], 2);

                __stvewx(R2[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R0[0], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R0[1], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R0[2], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R0[3], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R0[4], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R0[5], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R0[6], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R0[7], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3TransformNormal
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;

    Z = XMVectorSplatZ(V);
    Y = XMVectorSplatY(V);
    X = XMVectorSplatX(V);

    Result = XMVectorMultiply(Z, M.r[2]);
    Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
    Result = XMVectorMultiplyAdd(X, M.r[0], Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;

    Z = __vspltw(V, 2);
    Y = __vspltw(V, 1);
    X = __vspltw(V, 0);

    Result = __vmulfp(Z, M.r[2]);
    Result = __vmaddfp(Y, M.r[1], Result);
    Result = __vmaddfp(X, M.r[0], Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT3* XMVector3TransformNormalStream
(
    XMFLOAT3*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX       M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Z = XMVectorReplicate(((XMFLOAT3*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT3*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT3*)pInputVector)->x);

        Result = XMVectorMultiply(Z, M.r[2]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        Z = XMVectorReplicate(((XMFLOAT3*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT3*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT3*)pInputVector)->x);

        Result = XMVectorMultiply(Z, M.r[2]);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMMATRIX P, MT;
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    XMVECTOR R0[8], R1[8], R2[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);

    if (InputStride == sizeof(XMFLOAT3))
    {
//        C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R2[0] = __vor(R0[0], R1[0]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R2[0] = __vor(R0[0], R1[0]);
            R2[1] = __vor(R0[1], R1[1]);
            R2[2] = __vor(R0[2], R1[2]);
            R2[3] = __vor(R0[3], R1[3]);
            R2[4] = __vor(R0[4], R1[4]);
            R2[5] = __vor(R0[5], R1[5]);
            R2[6] = __vor(R0[6], R1[6]);
            R2[7] = __vor(R0[7], R1[7]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            R0[1] = __vmsum3fp(R2[1], MT.r[0]);
            R1[1] = __vmsum3fp(R2[1], MT.r[1]);
            R2[1] = __vmsum3fp(R2[1], MT.r[2]);

            R0[2] = __vmsum3fp(R2[2], MT.r[0]);
            R1[2] = __vmsum3fp(R2[2], MT.r[1]);
            R2[2] = __vmsum3fp(R2[2], MT.r[2]);

            R0[3] = __vmsum3fp(R2[3], MT.r[0]);
            R1[3] = __vmsum3fp(R2[3], MT.r[1]);
            R2[3] = __vmsum3fp(R2[3], MT.r[2]);

            R0[4] = __vmsum3fp(R2[4], MT.r[0]);
            R1[4] = __vmsum3fp(R2[4], MT.r[1]);
            R2[4] = __vmsum3fp(R2[4], MT.r[2]);

            R0[5] = __vmsum3fp(R2[5], MT.r[0]);
            R1[5] = __vmsum3fp(R2[5], MT.r[1]);
            R2[5] = __vmsum3fp(R2[5], MT.r[2]);

            R0[6] = __vmsum3fp(R2[6], MT.r[0]);
            R1[6] = __vmsum3fp(R2[6], MT.r[1]);
            R2[6] = __vmsum3fp(R2[6], MT.r[2]);

            R0[7] = __vmsum3fp(R2[7], MT.r[0]);
            R1[7] = __vmsum3fp(R2[7], MT.r[1]);
            R2[7] = __vmsum3fp(R2[7], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R0[0] = __vspltw(L[0], 2);
                R1[0] = __vspltw(L[0], 1);
                R2[0] = __vspltw(L[0], 0);

                R0[1] = __vspltw(L[1], 1);
                R1[1] = __vspltw(L[1], 0);
                R2[1] = __vspltw(L[0], 3);

                R0[2] = __vspltw(L[2], 0);
                R1[2] = __vspltw(L[1], 3);
                R2[2] = __vspltw(L[1], 2);

                R0[3] = __vspltw(L[2], 3);
                R1[3] = __vspltw(L[2], 2);
                R2[3] = __vspltw(L[2], 1);

                R0[4] = __vspltw(L[3], 2);
                R1[4] = __vspltw(L[3], 1);
                R2[4] = __vspltw(L[3], 0);

                R0[5] = __vspltw(L[4], 1);
                R1[5] = __vspltw(L[4], 0);
                R2[5] = __vspltw(L[3], 3);

                R0[6] = __vspltw(L[5], 0);
                R1[6] = __vspltw(L[4], 3);
                R2[6] = __vspltw(L[4], 2);

                R0[7] = __vspltw(L[5], 3);
                R1[7] = __vspltw(L[5], 2);
                R2[7] = __vspltw(L[5], 1);

                R0[0] = __vmulfp(R0[0], M.r[2]);
                R0[1] = __vmulfp(R0[1], M.r[2]);
                R0[2] = __vmulfp(R0[2], M.r[2]);
                R0[3] = __vmulfp(R0[3], M.r[2]);
                R0[4] = __vmulfp(R0[4], M.r[2]);
                R0[5] = __vmulfp(R0[5], M.r[2]);
                R0[6] = __vmulfp(R0[6], M.r[2]);
                R0[7] = __vmulfp(R0[7], M.r[2]);

                R0[0] = __vmaddcfp(R1[0], M.r[1], R0[0]);
                R0[1] = __vmaddcfp(R1[1], M.r[1], R0[1]);
                R0[2] = __vmaddcfp(R1[2], M.r[1], R0[2]);
                R0[3] = __vmaddcfp(R1[3], M.r[1], R0[3]);
                R0[4] = __vmaddcfp(R1[4], M.r[1], R0[4]);
                R0[5] = __vmaddcfp(R1[5], M.r[1], R0[5]);
                R0[6] = __vmaddcfp(R1[6], M.r[1], R0[6]);
                R0[7] = __vmaddcfp(R1[7], M.r[1], R0[7]);

                R0[0] = __vmaddcfp(R2[0], M.r[0], R0[0]);
                R0[1] = __vmaddcfp(R2[1], M.r[0], R0[1]);
                R0[2] = __vmaddcfp(R2[2], M.r[0], R0[2]);
                R0[3] = __vmaddcfp(R2[3], M.r[0], R0[3]);
                R0[4] = __vmaddcfp(R2[4], M.r[0], R0[4]);
                R0[5] = __vmaddcfp(R2[5], M.r[0], R0[5]);
                R0[6] = __vmaddcfp(R2[6], M.r[0], R0[6]);
                R0[7] = __vmaddcfp(R2[7], M.r[0], R0[7]);

                R2[0] = __vspltw(R0[0], 0);
                R1[0] = __vspltw(R0[0], 1);
                R0[0] = __vspltw(R0[0], 2);

                R2[1] = __vspltw(R0[1], 0);
                R1[1] = __vspltw(R0[1], 1);
                R0[1] = __vspltw(R0[1], 2);

                R2[2] = __vspltw(R0[2], 0);
                R1[2] = __vspltw(R0[2], 1);
                R0[2] = __vspltw(R0[2], 2);

                R2[3] = __vspltw(R0[3], 0);
                R1[3] = __vspltw(R0[3], 1);
                R0[3] = __vspltw(R0[3], 2);

                R2[4] = __vspltw(R0[4], 0);
                R1[4] = __vspltw(R0[4], 1);
                R0[4] = __vspltw(R0[4], 2);

                R2[5] = __vspltw(R0[5], 0);
                R1[5] = __vspltw(R0[5], 1);
                R0[5] = __vspltw(R0[5], 2);

                R2[6] = __vspltw(R0[6], 0);
                R1[6] = __vspltw(R0[6], 1);
                R0[6] = __vspltw(R0[6], 2);

                R2[7] = __vspltw(R0[7], 0);
                R1[7] = __vspltw(R0[7], 1);
                R0[7] = __vspltw(R0[7], 2);

                __stvewx(R2[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R0[0], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R0[1], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R0[2], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R0[3], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R0[4], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R0[5], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R0[6], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R0[7], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R2[0] = __vor(R0[0], R1[0]);
            R2[1] = __vor(R0[1], R1[1]);
            R2[2] = __vor(R0[2], R1[2]);
            R2[3] = __vor(R0[3], R1[3]);
            R2[4] = __vor(R0[4], R1[4]);
            R2[5] = __vor(R0[5], R1[5]);
            R2[6] = __vor(R0[6], R1[6]);
            R2[7] = __vor(R0[7], R1[7]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            R0[1] = __vmsum3fp(R2[1], MT.r[0]);
            R1[1] = __vmsum3fp(R2[1], MT.r[1]);
            R2[1] = __vmsum3fp(R2[1], MT.r[2]);

            R0[2] = __vmsum3fp(R2[2], MT.r[0]);
            R1[2] = __vmsum3fp(R2[2], MT.r[1]);
            R2[2] = __vmsum3fp(R2[2], MT.r[2]);

            R0[3] = __vmsum3fp(R2[3], MT.r[0]);
            R1[3] = __vmsum3fp(R2[3], MT.r[1]);
            R2[3] = __vmsum3fp(R2[3], MT.r[2]);

            R0[4] = __vmsum3fp(R2[4], MT.r[0]);
            R1[4] = __vmsum3fp(R2[4], MT.r[1]);
            R2[4] = __vmsum3fp(R2[4], MT.r[2]);

            R0[5] = __vmsum3fp(R2[5], MT.r[0]);
            R1[5] = __vmsum3fp(R2[5], MT.r[1]);
            R2[5] = __vmsum3fp(R2[5], MT.r[2]);

            R0[6] = __vmsum3fp(R2[6], MT.r[0]);
            R1[6] = __vmsum3fp(R2[6], MT.r[1]);
            R2[6] = __vmsum3fp(R2[6], MT.r[2]);

            R0[7] = __vmsum3fp(R2[7], MT.r[0]);
            R1[7] = __vmsum3fp(R2[7], MT.r[1]);
            R2[7] = __vmsum3fp(R2[7], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R2[0] = __vor(R0[0], R1[0]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3);
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R2[0] = __vor(R0[0], R1[0]);
            R2[1] = __vor(R0[1], R1[1]);
            R2[2] = __vor(R0[2], R1[2]);
            R2[3] = __vor(R0[3], R1[3]);
            R2[4] = __vor(R0[4], R1[4]);
            R2[5] = __vor(R0[5], R1[5]);
            R2[6] = __vor(R0[6], R1[6]);
            R2[7] = __vor(R0[7], R1[7]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            R0[1] = __vmsum3fp(R2[1], MT.r[0]);
            R1[1] = __vmsum3fp(R2[1], MT.r[1]);
            R2[1] = __vmsum3fp(R2[1], MT.r[2]);

            R0[2] = __vmsum3fp(R2[2], MT.r[0]);
            R1[2] = __vmsum3fp(R2[2], MT.r[1]);
            R2[2] = __vmsum3fp(R2[2], MT.r[2]);

            R0[3] = __vmsum3fp(R2[3], MT.r[0]);
            R1[3] = __vmsum3fp(R2[3], MT.r[1]);
            R2[3] = __vmsum3fp(R2[3], MT.r[2]);

            R0[4] = __vmsum3fp(R2[4], MT.r[0]);
            R1[4] = __vmsum3fp(R2[4], MT.r[1]);
            R2[4] = __vmsum3fp(R2[4], MT.r[2]);

            R0[5] = __vmsum3fp(R2[5], MT.r[0]);
            R1[5] = __vmsum3fp(R2[5], MT.r[1]);
            R2[5] = __vmsum3fp(R2[5], MT.r[2]);

            R0[6] = __vmsum3fp(R2[6], MT.r[0]);
            R1[6] = __vmsum3fp(R2[6], MT.r[1]);
            R2[6] = __vmsum3fp(R2[6], MT.r[2]);

            R0[7] = __vmsum3fp(R2[7], MT.r[0]);
            R1[7] = __vmsum3fp(R2[7], MT.r[1]);
            R2[7] = __vmsum3fp(R2[7], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R2[0] = __vor(R0[0], R1[0]);

            R0[0] = __vmsum3fp(R2[0], MT.r[0]);
            R1[0] = __vmsum3fp(R2[0], MT.r[1]);
            R2[0] = __vmsum3fp(R2[0], MT.r[2]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);

            InputVector += InputStride;
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMVECTOR XMVector3Project
(
    FXMVECTOR V, 
    FLOAT    ViewportX, 
    FLOAT    ViewportY, 
    FLOAT    ViewportWidth, 
    FLOAT    ViewportHeight, 
    FLOAT    ViewportMinZ, 
    FLOAT    ViewportMaxZ, 
    CXMMATRIX Projection, 
    CXMMATRIX View, 
    CXMMATRIX World
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX Transform;
    XMVECTOR Scale;
    XMVECTOR Offset;
    XMVECTOR Result;
    FLOAT    HalfViewportWidth = ViewportWidth * 0.5f;
    FLOAT    HalfViewportHeight = ViewportHeight * 0.5f;

    Scale = XMVectorSet(HalfViewportWidth, 
                        -HalfViewportHeight,
                        ViewportMaxZ - ViewportMinZ,
                        0.0f);

    Offset = XMVectorSet(ViewportX + HalfViewportWidth,
                        ViewportY + HalfViewportHeight,
                        ViewportMinZ,
                        0.0f);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);

    Result = XMVector3TransformCoord(V, Transform);

    Result = XMVectorMultiplyAdd(Result, Scale, Offset);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMMATRIX Transform;
    XMMATRIX M2T, P;
    XMVECTOR M00, M01, M02, M03,
             M10, M11, M12, M13,
             M20, M21, M22, M23,
             M30, M31, M32, M33;
    XMVECTOR M00M02, M01M03, M10M12, M11M13,
             M20M22, M21M23, M30M32, M31M33;
    XMVECTOR M00M20, M10M30, M01M21, M11M31,
             M02M22, M12M32, M03M23, M13M33;
    XMVECTOR R0, R1, R2, R3, R0R2;
    XMVECTOR Scale;
    XMVECTOR Offset;
    XMVECTOR InverseW;
    XMVECTOR Reciprocal;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR I;
    XMVECTOR Result;
    FLOAT    HalfViewportWidth = ViewportWidth * 0.5f;
    FLOAT    HalfViewportHeight = ViewportHeight * 0.5f;

    XMDUMMY_INITIALIZE_VECTOR(I);

    Scale.vector4_f32[0] = HalfViewportWidth;
    Scale.vector4_f32[1] = -HalfViewportHeight;
    Scale.vector4_f32[2] = ViewportMaxZ - ViewportMinZ;
    Scale.vector4_f32[3] = 0.0f;

    Offset.vector4_f32[0] = ViewportX + HalfViewportWidth;
    Offset.vector4_f32[1] = ViewportY + HalfViewportHeight;
    Offset.vector4_f32[2] = ViewportMinZ;
    Offset.vector4_f32[3] = 0.0f;

    P.r[0] = __vmrghw(View.r[0], View.r[2]);
    P.r[1] = __vmrghw(View.r[1], View.r[3]);
    P.r[2] = __vmrglw(View.r[0], View.r[2]);
    P.r[3] = __vmrglw(View.r[1], View.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00 = __vmsum4fp(World.r[0], M2T.r[0]);
    M02 = __vmsum4fp(World.r[0], M2T.r[2]);
    M01 = __vmsum4fp(World.r[0], M2T.r[1]);
    M03 = __vmsum4fp(World.r[0], M2T.r[3]);

    M10 = __vmsum4fp(World.r[1], M2T.r[0]);
    M12 = __vmsum4fp(World.r[1], M2T.r[2]);
    M11 = __vmsum4fp(World.r[1], M2T.r[1]);
    M13 = __vmsum4fp(World.r[1], M2T.r[3]);

    M20 = __vmsum4fp(World.r[2], M2T.r[0]);
    M22 = __vmsum4fp(World.r[2], M2T.r[2]);
    M21 = __vmsum4fp(World.r[2], M2T.r[1]);
    M23 = __vmsum4fp(World.r[2], M2T.r[3]);

    M30 = __vmsum4fp(World.r[3], M2T.r[0]);
    M32 = __vmsum4fp(World.r[3], M2T.r[2]);
    M31 = __vmsum4fp(World.r[3], M2T.r[1]);
    M33 = __vmsum4fp(World.r[3], M2T.r[3]);

    P.r[0] = __vmrghw(Projection.r[0], Projection.r[2]);
    P.r[1] = __vmrghw(Projection.r[1], Projection.r[3]);
    P.r[2] = __vmrglw(Projection.r[0], Projection.r[2]);
    P.r[3] = __vmrglw(Projection.r[1], Projection.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    M00 = __vmsum4fp(Transform.r[0], M2T.r[0]);
    M02 = __vmsum4fp(Transform.r[0], M2T.r[2]);
    M01 = __vmsum4fp(Transform.r[0], M2T.r[1]);
    M03 = __vmsum4fp(Transform.r[0], M2T.r[3]);

    M10 = __vmsum4fp(Transform.r[1], M2T.r[0]);
    M12 = __vmsum4fp(Transform.r[1], M2T.r[2]);
    M11 = __vmsum4fp(Transform.r[1], M2T.r[1]);
    M13 = __vmsum4fp(Transform.r[1], M2T.r[3]);

    M20 = __vmsum4fp(Transform.r[2], M2T.r[0]);
    M22 = __vmsum4fp(Transform.r[2], M2T.r[2]);
    M21 = __vmsum4fp(Transform.r[2], M2T.r[1]);
    M23 = __vmsum4fp(Transform.r[2], M2T.r[3]);

    M30 = __vmsum4fp(Transform.r[3], M2T.r[0]);
    M32 = __vmsum4fp(Transform.r[3], M2T.r[2]);
    M31 = __vmsum4fp(Transform.r[3], M2T.r[1]);
    M33 = __vmsum4fp(Transform.r[3], M2T.r[3]);

    I = __vupkd3d(I, VPACK_NORMSHORT2);

    M00M20 = __vmrghw(M00, M20);
    M10M30 = __vmrghw(M10, M30);
    M01M21 = __vmrghw(M01, M21);
    M11M31 = __vmrghw(M11, M31);
    M02M22 = __vmrghw(M02, M22);
    M12M32 = __vmrghw(M12, M32);
    M03M23 = __vmrghw(M03, M23);
    M13M33 = __vmrghw(M13, M33);

    I = __vrlimi(I, V, 0xE, 0);

    Transform.r[0] = __vmrghw(M00M20, M10M30);
    Transform.r[1] = __vmrghw(M01M21, M11M31);
    Transform.r[2] = __vmrghw(M02M22, M12M32);
    Transform.r[3] = __vmrghw(M03M23, M13M33);

    R0 = __vmsum4fp(I, Transform.r[0]);
    R1 = __vmsum4fp(I, Transform.r[1]);
    R2 = __vmsum4fp(I, Transform.r[2]);
    R3 = __vmsum4fp(I, Transform.r[3]);

    R0R2 = __vmrghw(R0, R2);

    Result = __vmrghw(R0R2, R1);

    Reciprocal = __vrefp(R3);
    S0 = __vspltw(I, 3);
    S0 = __vnmsubfp(R3, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(I, 3);
    S1 = __vnmsubfp(R3, S0, S1);
    InverseW = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    InverseW = __vsel(Reciprocal, S0, InverseW);

    Result = __vmulfp(Result, InverseW);

    Result = __vmaddcfp(Scale, Result, Offset);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT3* XMVector3ProjectStream
(
    XMFLOAT3*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    FLOAT           ViewportX, 
    FLOAT           ViewportY, 
    FLOAT           ViewportWidth, 
    FLOAT           ViewportHeight, 
    FLOAT           ViewportMinZ, 
    FLOAT           ViewportMaxZ, 
    CXMMATRIX       Projection, 
    CXMMATRIX       View, 
    CXMMATRIX       World
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX Transform;
    XMVECTOR V;
    XMVECTOR Scale;
    XMVECTOR Offset;
    XMVECTOR Result;
    size_t   i;
    FLOAT    HalfViewportWidth = ViewportWidth * 0.5f;
    FLOAT    HalfViewportHeight = ViewportHeight * 0.5f;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    Scale = XMVectorSet(HalfViewportWidth, 
                        -HalfViewportHeight,
                        ViewportMaxZ - ViewportMinZ,
                        1.0f);

    Offset = XMVectorSet(ViewportX + HalfViewportWidth,
                        ViewportY + HalfViewportHeight,
                        ViewportMinZ,
                        0.0f);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);

        Result = XMVector3TransformCoord(V, Transform);

        Result = XMVectorMultiplyAdd(Result, Scale, Offset);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMMATRIX Transform;
    XMVECTOR V;
    XMVECTOR Scale;
    XMVECTOR Offset;
    XMVECTOR Result;
    size_t   i;
    FLOAT    HalfViewportWidth = ViewportWidth * 0.5f;
    FLOAT    HalfViewportHeight = ViewportHeight * 0.5f;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    Scale = XMVectorSet(HalfViewportWidth, 
                        -HalfViewportHeight,
                        ViewportMaxZ - ViewportMinZ,
                        1.0f);

    Offset = XMVectorSet(ViewportX + HalfViewportWidth,
                        ViewportY + HalfViewportHeight,
                        ViewportMinZ,
                        0.0f);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);

        Result = XMVector3TransformCoord(V, Transform);

        Result = XMVectorMultiplyAdd(Result, Scale, Offset);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMMATRIX Transform, TransformT;
    XMVECTOR M00, M01, M02, M03,
             M10, M11, M12, M13,
             M20, M21, M22, M23,
             M30, M31, M32, M33;
    XMVECTOR M00M02, M01M03, M10M12, M11M13,
             M20M22, M21M23, M30M32, M31M33;
    XMVECTOR M00M20, M10M30, M01M21, M11M31,
             M02M22, M12M32, M03M23, M13M33;
    XMVECTOR C1W[8], R0[8], R1[8], R2[8], R3[8];
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    XMMATRIX P, MT, M2T;
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;
    XMVECTOR Scale;
    XMVECTOR Offset;
    FLOAT    HalfViewportWidth = ViewportWidth * 0.5f;
    FLOAT    HalfViewportHeight = ViewportHeight * 0.5f;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    Scale.vector4_f32[0] = HalfViewportWidth;
    Scale.vector4_f32[1] = -HalfViewportHeight;
    Scale.vector4_f32[2] = ViewportMaxZ - ViewportMinZ;
    Scale.vector4_f32[3] = 0.0f;

    Offset.vector4_f32[0] = ViewportX + HalfViewportWidth;
    Offset.vector4_f32[1] = ViewportY + HalfViewportHeight;
    Offset.vector4_f32[2] = ViewportMinZ;
    Offset.vector4_f32[3] = 0.0f;

    P.r[0] = __vmrghw(View.r[0], View.r[2]);
    P.r[1] = __vmrghw(View.r[1], View.r[3]);
    P.r[2] = __vmrglw(View.r[0], View.r[2]);
    P.r[3] = __vmrglw(View.r[1], View.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00 = __vmsum4fp(World.r[0], M2T.r[0]);
    M02 = __vmsum4fp(World.r[0], M2T.r[2]);
    M01 = __vmsum4fp(World.r[0], M2T.r[1]);
    M03 = __vmsum4fp(World.r[0], M2T.r[3]);

    M10 = __vmsum4fp(World.r[1], M2T.r[0]);
    M12 = __vmsum4fp(World.r[1], M2T.r[2]);
    M11 = __vmsum4fp(World.r[1], M2T.r[1]);
    M13 = __vmsum4fp(World.r[1], M2T.r[3]);

    M20 = __vmsum4fp(World.r[2], M2T.r[0]);
    M22 = __vmsum4fp(World.r[2], M2T.r[2]);
    M21 = __vmsum4fp(World.r[2], M2T.r[1]);
    M23 = __vmsum4fp(World.r[2], M2T.r[3]);

    M30 = __vmsum4fp(World.r[3], M2T.r[0]);
    M32 = __vmsum4fp(World.r[3], M2T.r[2]);
    M31 = __vmsum4fp(World.r[3], M2T.r[1]);
    M33 = __vmsum4fp(World.r[3], M2T.r[3]);

    P.r[0] = __vmrghw(Projection.r[0], Projection.r[2]);
    P.r[1] = __vmrghw(Projection.r[1], Projection.r[3]);
    P.r[2] = __vmrglw(Projection.r[0], Projection.r[2]);
    P.r[3] = __vmrglw(Projection.r[1], Projection.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    M00 = __vmsum4fp(Transform.r[0], M2T.r[0]);
    M02 = __vmsum4fp(Transform.r[0], M2T.r[2]);
    M01 = __vmsum4fp(Transform.r[0], M2T.r[1]);
    M03 = __vmsum4fp(Transform.r[0], M2T.r[3]);

    M10 = __vmsum4fp(Transform.r[1], M2T.r[0]);
    M12 = __vmsum4fp(Transform.r[1], M2T.r[2]);
    M11 = __vmsum4fp(Transform.r[1], M2T.r[1]);
    M13 = __vmsum4fp(Transform.r[1], M2T.r[3]);

    M20 = __vmsum4fp(Transform.r[2], M2T.r[0]);
    M22 = __vmsum4fp(Transform.r[2], M2T.r[2]);
    M21 = __vmsum4fp(Transform.r[2], M2T.r[1]);
    M23 = __vmsum4fp(Transform.r[2], M2T.r[3]);

    M30 = __vmsum4fp(Transform.r[3], M2T.r[0]);
    M32 = __vmsum4fp(Transform.r[3], M2T.r[2]);
    M31 = __vmsum4fp(Transform.r[3], M2T.r[1]);
    M33 = __vmsum4fp(Transform.r[3], M2T.r[3]);

    M00M20 = __vmrghw(M00, M20);
    M10M30 = __vmrghw(M10, M30);
    M01M21 = __vmrghw(M01, M21);
    M11M31 = __vmrghw(M11, M31);
    M02M22 = __vmrghw(M02, M22);
    M12M32 = __vmrghw(M12, M32);
    M03M23 = __vmrghw(M03, M23);
    M13M33 = __vmrghw(M13, M33);

    TransformT.r[0] = __vmrghw(M00M20, M10M30);
    TransformT.r[1] = __vmrghw(M01M21, M11M31);
    TransformT.r[2] = __vmrghw(M02M22, M12M32);
    TransformT.r[3] = __vmrghw(M03M23, M13M33);

    P.r[0] = __vmrghw(TransformT.r[0], TransformT.r[2]);
    P.r[1] = __vmrghw(TransformT.r[1], TransformT.r[3]);
    P.r[2] = __vmrglw(TransformT.r[0], TransformT.r[2]);
    P.r[3] = __vmrglw(TransformT.r[1], TransformT.r[3]);

    Transform.r[0] = __vmrghw(P.r[0], P.r[1]);
    Transform.r[1] = __vmrglw(P.r[0], P.r[1]);
    Transform.r[2] = __vmrghw(P.r[2], P.r[3]);
    Transform.r[3] = __vmrglw(P.r[2], P.r[3]);

    XMDUMMY_INITIALIZE_VECTOR(C1W[0]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[1]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[2]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[3]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[4]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[5]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[6]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[7]);

    C1W[0] = __vupkd3d(C1W[0], VPACK_NORMSHORT2);
    C1W[1] = __vupkd3d(C1W[1], VPACK_NORMSHORT2);
    C1W[2] = __vupkd3d(C1W[2], VPACK_NORMSHORT2);
    C1W[3] = __vupkd3d(C1W[3], VPACK_NORMSHORT2);
    C1W[4] = __vupkd3d(C1W[4], VPACK_NORMSHORT2);
    C1W[5] = __vupkd3d(C1W[5], VPACK_NORMSHORT2);
    C1W[6] = __vupkd3d(C1W[6], VPACK_NORMSHORT2);
    C1W[7] = __vupkd3d(C1W[7], VPACK_NORMSHORT2);

    if (InputStride == sizeof(XMFLOAT3))
    {
//        C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], TransformT.r[0]);
            R1[0] = __vmsum4fp(R3[0], TransformT.r[1]);
            R2[0] = __vmsum4fp(R3[0], TransformT.r[2]);
            R3[0] = __vmsum4fp(R3[0], TransformT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], TransformT.r[0]);
            R1[0] = __vmsum4fp(R3[0], TransformT.r[1]);
            R2[0] = __vmsum4fp(R3[0], TransformT.r[2]);
            R3[0] = __vmsum4fp(R3[0], TransformT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], TransformT.r[0]);
            R1[1] = __vmsum4fp(R3[1], TransformT.r[1]);
            R2[1] = __vmsum4fp(R3[1], TransformT.r[2]);
            R3[1] = __vmsum4fp(R3[1], TransformT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], TransformT.r[0]);
            R1[2] = __vmsum4fp(R3[2], TransformT.r[1]);
            R2[2] = __vmsum4fp(R3[2], TransformT.r[2]);
            R3[2] = __vmsum4fp(R3[2], TransformT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], TransformT.r[0]);
            R1[3] = __vmsum4fp(R3[3], TransformT.r[1]);
            R2[3] = __vmsum4fp(R3[3], TransformT.r[2]);
            R3[3] = __vmsum4fp(R3[3], TransformT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], TransformT.r[0]);
            R1[4] = __vmsum4fp(R3[4], TransformT.r[1]);
            R2[4] = __vmsum4fp(R3[4], TransformT.r[2]);
            R3[4] = __vmsum4fp(R3[4], TransformT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], TransformT.r[0]);
            R1[5] = __vmsum4fp(R3[5], TransformT.r[1]);
            R2[5] = __vmsum4fp(R3[5], TransformT.r[2]);
            R3[5] = __vmsum4fp(R3[5], TransformT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], TransformT.r[0]);
            R1[6] = __vmsum4fp(R3[6], TransformT.r[1]);
            R2[6] = __vmsum4fp(R3[6], TransformT.r[2]);
            R3[6] = __vmsum4fp(R3[6], TransformT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], TransformT.r[0]);
            R1[7] = __vmsum4fp(R3[7], TransformT.r[1]);
            R2[7] = __vmsum4fp(R3[7], TransformT.r[2]);
            R3[7] = __vmsum4fp(R3[7], TransformT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);
            R0[1] = __vmaddcfp(Scale, R0[1], Offset);
            R0[2] = __vmaddcfp(Scale, R0[2], Offset);
            R0[3] = __vmaddcfp(Scale, R0[3], Offset);
            R0[4] = __vmaddcfp(Scale, R0[4], Offset);
            R0[5] = __vmaddcfp(Scale, R0[5], Offset);
            R0[6] = __vmaddcfp(Scale, R0[6], Offset);
            R0[7] = __vmaddcfp(Scale, R0[7], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                R0[0] = __vspltw(L[0], 2);
                R1[0] = __vspltw(L[0], 1);
                R2[0] = __vspltw(L[0], 0);

                R0[1] = __vspltw(L[1], 1);
                R1[1] = __vspltw(L[1], 0);
                R2[1] = __vspltw(L[0], 3);

                R0[2] = __vspltw(L[2], 0);
                R1[2] = __vspltw(L[1], 3);
                R2[2] = __vspltw(L[1], 2);

                R0[3] = __vspltw(L[2], 3);
                R1[3] = __vspltw(L[2], 2);
                R2[3] = __vspltw(L[2], 1);

                R0[4] = __vspltw(L[3], 2);
                R1[4] = __vspltw(L[3], 1);
                R2[4] = __vspltw(L[3], 0);

                R0[5] = __vspltw(L[4], 1);
                R1[5] = __vspltw(L[4], 0);
                R2[5] = __vspltw(L[3], 3);

                R0[6] = __vspltw(L[5], 0);
                R1[6] = __vspltw(L[4], 3);
                R2[6] = __vspltw(L[4], 2);

                R0[7] = __vspltw(L[5], 3);
                R1[7] = __vspltw(L[5], 2);
                R2[7] = __vspltw(L[5], 1);

                R0[0] = __vmaddcfp(R0[0], Transform.r[2], Transform.r[3]);
                R0[1] = __vmaddcfp(R0[1], Transform.r[2], Transform.r[3]);
                R0[2] = __vmaddcfp(R0[2], Transform.r[2], Transform.r[3]);
                R0[3] = __vmaddcfp(R0[3], Transform.r[2], Transform.r[3]);
                R0[4] = __vmaddcfp(R0[4], Transform.r[2], Transform.r[3]);
                R0[5] = __vmaddcfp(R0[5], Transform.r[2], Transform.r[3]);
                R0[6] = __vmaddcfp(R0[6], Transform.r[2], Transform.r[3]);
                R0[7] = __vmaddcfp(R0[7], Transform.r[2], Transform.r[3]);

                R0[0] = __vmaddcfp(R1[0], Transform.r[1], R0[0]);
                R0[1] = __vmaddcfp(R1[1], Transform.r[1], R0[1]);
                R0[2] = __vmaddcfp(R1[2], Transform.r[1], R0[2]);
                R0[3] = __vmaddcfp(R1[3], Transform.r[1], R0[3]);
                R0[4] = __vmaddcfp(R1[4], Transform.r[1], R0[4]);
                R0[5] = __vmaddcfp(R1[5], Transform.r[1], R0[5]);
                R0[6] = __vmaddcfp(R1[6], Transform.r[1], R0[6]);
                R0[7] = __vmaddcfp(R1[7], Transform.r[1], R0[7]);

                R0[0] = __vmaddcfp(R2[0], Transform.r[0], R0[0]);
                R0[1] = __vmaddcfp(R2[1], Transform.r[0], R0[1]);
                R0[2] = __vmaddcfp(R2[2], Transform.r[0], R0[2]);
                R0[3] = __vmaddcfp(R2[3], Transform.r[0], R0[3]);
                R0[4] = __vmaddcfp(R2[4], Transform.r[0], R0[4]);
                R0[5] = __vmaddcfp(R2[5], Transform.r[0], R0[5]);
                R0[6] = __vmaddcfp(R2[6], Transform.r[0], R0[6]);
                R0[7] = __vmaddcfp(R2[7], Transform.r[0], R0[7]);

                R1[0] = __vspltw(R0[0], 3);
                R1[1] = __vspltw(R0[1], 3);
                R1[2] = __vspltw(R0[2], 3);
                R1[3] = __vspltw(R0[3], 3);
                R1[4] = __vspltw(R0[4], 3);
                R1[5] = __vspltw(R0[5], 3);
                R1[6] = __vspltw(R0[6], 3);
                R1[7] = __vspltw(R0[7], 3);

                R1[0] = XMVectorReciprocal(R1[0]);
                R1[1] = XMVectorReciprocal(R1[1]);
                R1[2] = XMVectorReciprocal(R1[2]);
                R1[3] = XMVectorReciprocal(R1[3]);
                R1[4] = XMVectorReciprocal(R1[4]);
                R1[5] = XMVectorReciprocal(R1[5]);
                R1[6] = XMVectorReciprocal(R1[6]);
                R1[7] = XMVectorReciprocal(R1[7]);

                R0[0] = __vmulfp(R0[0], R1[0]);
                R0[1] = __vmulfp(R0[1], R1[1]);
                R0[2] = __vmulfp(R0[2], R1[2]);
                R0[3] = __vmulfp(R0[3], R1[3]);
                R0[4] = __vmulfp(R0[4], R1[4]);
                R0[5] = __vmulfp(R0[5], R1[5]);
                R0[6] = __vmulfp(R0[6], R1[6]);
                R0[7] = __vmulfp(R0[7], R1[7]);

                R0[0] = __vmaddcfp(Scale, R0[0], Offset);
                R0[1] = __vmaddcfp(Scale, R0[1], Offset);
                R0[2] = __vmaddcfp(Scale, R0[2], Offset);
                R0[3] = __vmaddcfp(Scale, R0[3], Offset);
                R0[4] = __vmaddcfp(Scale, R0[4], Offset);
                R0[5] = __vmaddcfp(Scale, R0[5], Offset);
                R0[6] = __vmaddcfp(Scale, R0[6], Offset);
                R0[7] = __vmaddcfp(Scale, R0[7], Offset);

                R2[0] = __vspltw(R0[0], 0);
                R1[0] = __vspltw(R0[0], 1);
                R0[0] = __vspltw(R0[0], 2);
                                    
                R2[1] = __vspltw(R0[1], 0);
                R1[1] = __vspltw(R0[1], 1);
                R0[1] = __vspltw(R0[1], 2);
                                    
                R2[2] = __vspltw(R0[2], 0);
                R1[2] = __vspltw(R0[2], 1);
                R0[2] = __vspltw(R0[2], 2);
                                    
                R2[3] = __vspltw(R0[3], 0);
                R1[3] = __vspltw(R0[3], 1);
                R0[3] = __vspltw(R0[3], 2);
                                    
                R2[4] = __vspltw(R0[4], 0);
                R1[4] = __vspltw(R0[4], 1);
                R0[4] = __vspltw(R0[4], 2);
                                    
                R2[5] = __vspltw(R0[5], 0);
                R1[5] = __vspltw(R0[5], 1);
                R0[5] = __vspltw(R0[5], 2);
                                    
                R2[6] = __vspltw(R0[6], 0);
                R1[6] = __vspltw(R0[6], 1);
                R0[6] = __vspltw(R0[6], 2);
                                    
                R2[7] = __vspltw(R0[7], 0);
                R1[7] = __vspltw(R0[7], 1);
                R0[7] = __vspltw(R0[7], 2);

                __stvewx(R2[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R0[0], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R0[1], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R0[2], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R0[3], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R0[4], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R0[5], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R0[6], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R0[7], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], TransformT.r[0]);
            R1[0] = __vmsum4fp(R3[0], TransformT.r[1]);
            R2[0] = __vmsum4fp(R3[0], TransformT.r[2]);
            R3[0] = __vmsum4fp(R3[0], TransformT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], TransformT.r[0]);
            R1[1] = __vmsum4fp(R3[1], TransformT.r[1]);
            R2[1] = __vmsum4fp(R3[1], TransformT.r[2]);
            R3[1] = __vmsum4fp(R3[1], TransformT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], TransformT.r[0]);
            R1[2] = __vmsum4fp(R3[2], TransformT.r[1]);
            R2[2] = __vmsum4fp(R3[2], TransformT.r[2]);
            R3[2] = __vmsum4fp(R3[2], TransformT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], TransformT.r[0]);
            R1[3] = __vmsum4fp(R3[3], TransformT.r[1]);
            R2[3] = __vmsum4fp(R3[3], TransformT.r[2]);
            R3[3] = __vmsum4fp(R3[3], TransformT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], TransformT.r[0]);
            R1[4] = __vmsum4fp(R3[4], TransformT.r[1]);
            R2[4] = __vmsum4fp(R3[4], TransformT.r[2]);
            R3[4] = __vmsum4fp(R3[4], TransformT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], TransformT.r[0]);
            R1[5] = __vmsum4fp(R3[5], TransformT.r[1]);
            R2[5] = __vmsum4fp(R3[5], TransformT.r[2]);
            R3[5] = __vmsum4fp(R3[5], TransformT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], TransformT.r[0]);
            R1[6] = __vmsum4fp(R3[6], TransformT.r[1]);
            R2[6] = __vmsum4fp(R3[6], TransformT.r[2]);
            R3[6] = __vmsum4fp(R3[6], TransformT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], TransformT.r[0]);
            R1[7] = __vmsum4fp(R3[7], TransformT.r[1]);
            R2[7] = __vmsum4fp(R3[7], TransformT.r[2]);
            R3[7] = __vmsum4fp(R3[7], TransformT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);
            R0[1] = __vmaddcfp(Scale, R0[1], Offset);
            R0[2] = __vmaddcfp(Scale, R0[2], Offset);
            R0[3] = __vmaddcfp(Scale, R0[3], Offset);
            R0[4] = __vmaddcfp(Scale, R0[4], Offset);
            R0[5] = __vmaddcfp(Scale, R0[5], Offset);
            R0[6] = __vmaddcfp(Scale, R0[6], Offset);
            R0[7] = __vmaddcfp(Scale, R0[7], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], TransformT.r[0]);
            R1[0] = __vmsum4fp(R3[0], TransformT.r[1]);
            R2[0] = __vmsum4fp(R3[0], TransformT.r[2]);
            R3[0] = __vmsum4fp(R3[0], TransformT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], TransformT.r[0]);
            R1[1] = __vmsum4fp(R3[1], TransformT.r[1]);
            R2[1] = __vmsum4fp(R3[1], TransformT.r[2]);
            R3[1] = __vmsum4fp(R3[1], TransformT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], TransformT.r[0]);
            R1[2] = __vmsum4fp(R3[2], TransformT.r[1]);
            R2[2] = __vmsum4fp(R3[2], TransformT.r[2]);
            R3[2] = __vmsum4fp(R3[2], TransformT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], TransformT.r[0]);
            R1[3] = __vmsum4fp(R3[3], TransformT.r[1]);
            R2[3] = __vmsum4fp(R3[3], TransformT.r[2]);
            R3[3] = __vmsum4fp(R3[3], TransformT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], TransformT.r[0]);
            R1[4] = __vmsum4fp(R3[4], TransformT.r[1]);
            R2[4] = __vmsum4fp(R3[4], TransformT.r[2]);
            R3[4] = __vmsum4fp(R3[4], TransformT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], TransformT.r[0]);
            R1[5] = __vmsum4fp(R3[5], TransformT.r[1]);
            R2[5] = __vmsum4fp(R3[5], TransformT.r[2]);
            R3[5] = __vmsum4fp(R3[5], TransformT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], TransformT.r[0]);
            R1[6] = __vmsum4fp(R3[6], TransformT.r[1]);
            R2[6] = __vmsum4fp(R3[6], TransformT.r[2]);
            R3[6] = __vmsum4fp(R3[6], TransformT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], TransformT.r[0]);
            R1[7] = __vmsum4fp(R3[7], TransformT.r[1]);
            R2[7] = __vmsum4fp(R3[7], TransformT.r[2]);
            R3[7] = __vmsum4fp(R3[7], TransformT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);
            R0[1] = __vmaddcfp(Scale, R0[1], Offset);
            R0[2] = __vmaddcfp(Scale, R0[2], Offset);
            R0[3] = __vmaddcfp(Scale, R0[3], Offset);
            R0[4] = __vmaddcfp(Scale, R0[4], Offset);
            R0[5] = __vmaddcfp(Scale, R0[5], Offset);
            R0[6] = __vmaddcfp(Scale, R0[6], Offset);
            R0[7] = __vmaddcfp(Scale, R0[7], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], TransformT.r[0]);
            R1[0] = __vmsum4fp(R3[0], TransformT.r[1]);
            R2[0] = __vmsum4fp(R3[0], TransformT.r[2]);
            R3[0] = __vmsum4fp(R3[0], TransformT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R0[0] = __vmaddcfp(Scale, R0[0], Offset);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector3Unproject
(
    FXMVECTOR V, 
    FLOAT    ViewportX, 
    FLOAT    ViewportY, 
    FLOAT    ViewportWidth, 
    FLOAT    ViewportHeight, 
    FLOAT    ViewportMinZ, 
    FLOAT    ViewportMaxZ, 
    CXMMATRIX Projection, 
    CXMMATRIX View, 
    CXMMATRIX World
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX        Transform;
    XMVECTOR        Scale;
    XMVECTOR        Offset;
    XMVECTOR        Determinant;
    XMVECTOR        Result;
    CONST XMVECTOR  D = XMVectorSet(-1.0f, 1.0f, 0.0f, 0.0f);

    Scale = XMVectorSet(ViewportWidth * 0.5f,
                        -ViewportHeight * 0.5f,
                        ViewportMaxZ - ViewportMinZ,
                        1.0f);
    Scale = XMVectorReciprocal(Scale);

    Offset = XMVectorSet(-ViewportX,
                        -ViewportY,
                        -ViewportMinZ,
                        0.0f);
    Offset = XMVectorMultiplyAdd(Scale, Offset, D);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);
    Transform = XMMatrixInverse(&Determinant, Transform);

    Result = XMVectorMultiplyAdd(V, Scale, Offset);

    Result = XMVector3TransformCoord(Result, Transform);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR M00, M01, M02, M03,
             M10, M11, M12, M13,
             M20, M21, M22, M23,
             M30, M31, M32, M33;
    XMVECTOR M00M02, M01M03, M10M12, M11M13,
             M20M22, M21M23, M30M32, M31M33;
    XMMATRIX M2T, P, MT;
    XMMATRIX Transform;
    XMVECTOR Scale;
    XMVECTOR Result;
    XMVECTOR Determinant;
    XMVECTOR Reciprocal;
    XMVECTOR PR1, PR2;
    XMVECTOR D0, D1, D2;
    XMVECTOR Refine;
    XMVECTOR V0[4], V1[4];
    XMVECTOR Zero, SignMask, Sign0, Sign1;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR D;
    XMMATRIX R;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR RZ;
    XMVECTOR InverseW;
    XMVECTOR NegativeOne;

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Scale.vector4_f32[0] = ViewportWidth * 0.5f;
    Scale.vector4_f32[1] = -ViewportHeight * 0.5f;
    Scale.vector4_f32[2] = ViewportMaxZ - ViewportMinZ;
    Scale.vector4_f32[3] = 1.0f;

    Result.vector4_f32[0] = -ViewportX;
    Result.vector4_f32[1] = -ViewportY;
    Result.vector4_f32[2] = -ViewportMinZ;
    Result.vector4_f32[3] = 0.0f;

    P.r[0] = __vmrghw(View.r[0], View.r[2]);
    P.r[1] = __vmrghw(View.r[1], View.r[3]);
    P.r[2] = __vmrglw(View.r[0], View.r[2]);
    P.r[3] = __vmrglw(View.r[1], View.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00 = __vmsum4fp(World.r[0], M2T.r[0]);
    M02 = __vmsum4fp(World.r[0], M2T.r[2]);
    M01 = __vmsum4fp(World.r[0], M2T.r[1]);
    M03 = __vmsum4fp(World.r[0], M2T.r[3]);

    M10 = __vmsum4fp(World.r[1], M2T.r[0]);
    M12 = __vmsum4fp(World.r[1], M2T.r[2]);
    M11 = __vmsum4fp(World.r[1], M2T.r[1]);
    M13 = __vmsum4fp(World.r[1], M2T.r[3]);

    M20 = __vmsum4fp(World.r[2], M2T.r[0]);
    M22 = __vmsum4fp(World.r[2], M2T.r[2]);
    M21 = __vmsum4fp(World.r[2], M2T.r[1]);
    M23 = __vmsum4fp(World.r[2], M2T.r[3]);

    M30 = __vmsum4fp(World.r[3], M2T.r[0]);
    M32 = __vmsum4fp(World.r[3], M2T.r[2]);
    M31 = __vmsum4fp(World.r[3], M2T.r[1]);
    M33 = __vmsum4fp(World.r[3], M2T.r[3]);

    P.r[0] = __vmrghw(Projection.r[0], Projection.r[2]);
    P.r[1] = __vmrghw(Projection.r[1], Projection.r[3]);
    P.r[2] = __vmrglw(Projection.r[0], Projection.r[2]);
    P.r[3] = __vmrglw(Projection.r[1], Projection.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    M00 = __vmsum4fp(Transform.r[0], M2T.r[0]);
    M02 = __vmsum4fp(Transform.r[0], M2T.r[2]);
    M01 = __vmsum4fp(Transform.r[0], M2T.r[1]);
    M03 = __vmsum4fp(Transform.r[0], M2T.r[3]);

    M10 = __vmsum4fp(Transform.r[1], M2T.r[0]);
    M12 = __vmsum4fp(Transform.r[1], M2T.r[2]);
    M11 = __vmsum4fp(Transform.r[1], M2T.r[1]);
    M13 = __vmsum4fp(Transform.r[1], M2T.r[3]);

    M20 = __vmsum4fp(Transform.r[2], M2T.r[0]);
    M22 = __vmsum4fp(Transform.r[2], M2T.r[2]);
    M21 = __vmsum4fp(Transform.r[2], M2T.r[1]);
    M23 = __vmsum4fp(Transform.r[2], M2T.r[3]);

    M30 = __vmsum4fp(Transform.r[3], M2T.r[0]);
    M32 = __vmsum4fp(Transform.r[3], M2T.r[2]);
    M31 = __vmsum4fp(Transform.r[3], M2T.r[1]);
    M33 = __vmsum4fp(Transform.r[3], M2T.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    P.r[0] = __vmrghw(Transform.r[0], Transform.r[2]);
    P.r[1] = __vmrghw(Transform.r[1], Transform.r[3]);
    P.r[2] = __vmrglw(Transform.r[0], Transform.r[2]);
    P.r[3] = __vmrglw(Transform.r[1], Transform.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    PR2 = P.r[2];
    PR1 = P.r[1];

    V0[0] = __vpermwi(MT.r[2], 0x5);
    V1[0] = __vpermwi(MT.r[3], 0xBB);
    V0[1] = __vpermwi(MT.r[0], 0x5);
    V1[1] = __vpermwi(MT.r[1], 0xBB);
    PR2 = __vrlimi(PR2, P.r[0], 0x3, 2);
    PR1 = __vrlimi(PR1, P.r[3], 0xC, 2);

    D0 = __vmulfp(V0[0], V1[0]);
    D1 = __vmulfp(V0[1], V1[1]);
    D2 = __vmulfp(PR2, PR1);

    V0[0] = __vpermwi(MT.r[2], 0xBB);
    V1[0] = __vpermwi(MT.r[3], 0x5);
    V0[1] = __vpermwi(MT.r[0], 0xBB);
    V1[1] = __vpermwi(MT.r[1], 0x5);
    P.r[3] = __vrlimi(P.r[3], P.r[1], 0x3, 2);
    P.r[0] = __vrlimi(P.r[0], P.r[2], 0xC, 2);

    D0 = __vnmsubfp(V0[0], V1[0], D0);
    D1 = __vnmsubfp(V0[1], V1[1], D1);
    D2 = __vnmsubfp(P.r[3], P.r[0], D2);

    SignMask = __vspltisw(-1);
    Zero = __vspltisw(0);
    SignMask = __vslw(SignMask, SignMask);
    Sign0 = __vmrghw(Zero, SignMask);
    Sign1 = __vmrghw(SignMask, Zero);

    V1[0] = __vpermwi(D0, 0x9C);
    V1[1] = __vpermwi(D0, 0xC6);
    V1[2] = __vpermwi(D1, 0x9C);
    V1[3] = __vpermwi(D1, 0xC6);

    V0[0] = __vpermwi(MT.r[1], 0x61);
    V1[0] = __vrlimi(V1[0], D2, 0x8, 1);
    V0[1] = __vpermwi(MT.r[0], 0x84);
    V1[1] = __vrlimi(V1[1], D2, 0x4, 0);
    V0[2] = __vpermwi(MT.r[3], 0x61);
    V1[2] = __vrlimi(V1[2], D2, 0x8, 3);
    V0[3] = __vpermwi(MT.r[2], 0x84);
    V1[3] = __vrlimi(V1[3], D2, 0x4, 2);

    R.r[0] = __vmulfp(V0[0], V1[0]);
    R.r[1] = __vmulfp(V0[1], V1[1]);
    R.r[2] = __vmulfp(V0[2], V1[2]);
    R.r[3] = __vmulfp(V0[3], V1[3]);

    V1[0] = __vpermwi(D0, 0xC6);
    V1[1] = __vpermwi(D0, 0x9C);
    V1[2] = __vpermwi(D1, 0xC6);
    V1[3] = __vpermwi(D1, 0x9C);

    V0[0] = __vpermwi(MT.r[1], 0xB6);
    V1[0] = __vrlimi(V1[0], D2, 0x1, 1);
    V0[1] = __vpermwi(MT.r[0], 0xED);
    V1[1] = __vrlimi(V1[1], D2, 0x2, 2);
    V0[2] = __vpermwi(MT.r[3], 0xB6);
    V1[2] = __vrlimi(V1[2], D2, 0x1, 3);
    V0[3] = __vpermwi(MT.r[2], 0xED);
    V1[3] = __vrlimi(V1[3], D2, 0x2, 0);

    R.r[0] = __vnmsubfp(V0[0], V1[0], R.r[0]);
    R.r[1] = __vnmsubfp(V0[1], V1[1], R.r[1]);
    R.r[2] = __vnmsubfp(V0[2], V1[2], R.r[2]);
    R.r[3] = __vnmsubfp(V0[3], V1[3], R.r[3]);

    V1[0] = __vsldoi(D0, D2, 2 << 2);
    V1[1] = __vpermwi(D0, 0x4E);
    V1[2] = __vmrglw(D1, D2);
    V1[3] = __vpermwi(D1, 0x4E);

    V0[0] = __vpermwi(MT.r[1], 0xCC);
    V1[0] = __vpermwi(V1[0], 0x38);
    V0[1] = __vpermwi(MT.r[0], 0x72);
    V1[1] = __vrlimi(V1[1], D2, 0x9, 1);
    V0[2] = __vpermwi(MT.r[3], 0xCC);
    V1[2] = __vpermwi(V1[2], 0x34);
    V0[3] = __vpermwi(MT.r[2], 0x72);
    V1[3] = __vrlimi(V1[3], D2, 0x9, 3);

    V0[0] = __vxor(V0[0], Sign0);
    V0[1] = __vxor(V0[1], Sign1);
    V0[2] = __vxor(V0[2], Sign0);
    V0[3] = __vxor(V0[3], Sign1);

    R.r[0] = __vmaddfp(V0[0], V1[0], R.r[0]);
    R.r[1] = __vmaddfp(V0[1], V1[1], R.r[1]);
    R.r[2] = __vmaddfp(V0[2], V1[2], R.r[2]);
    R.r[3] = __vmaddfp(V0[3], V1[3], R.r[3]);

    Determinant = __vmsum4fp(R.r[0], MT.r[0]);

    Reciprocal = __vrefp(Determinant);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(Determinant, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(Determinant, S0, S1);
    Refine = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    Refine = __vsel(Reciprocal, S0, Refine);

    Transform.r[0] = __vmulfp(R.r[0], Refine);
    Transform.r[1] = __vmulfp(R.r[1], Refine);
    Transform.r[2] = __vmulfp(R.r[2], Refine);
    Transform.r[3] = __vmulfp(R.r[3], Refine);

    Reciprocal = __vrefp(Scale);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    D = __vpermwi(S1, 0xEA);
    S0 = __vnmsubfp(Scale, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    NegativeOne = __vspltisw(-1);
    S1 = __vnmsubfp(Scale, S0, S1);
    Scale = __vcmpeqfp(S0, S0);
    NegativeOne = __vcfsx(NegativeOne, 0);
    S0 = __vmaddfp(S0, S1, S0);
    D = __vsldoi(NegativeOne, D, 3 << 2);
    Scale = __vsel(Reciprocal, S0, Scale);

    Result = __vmaddcfp(Scale, Result, D);

    Result = __vmaddfp(V, Scale, Result);

    RZ = __vspltw(Result, 2);
    Y = __vspltw(Result, 1);
    X = __vspltw(Result, 0);

    RZ = __vmaddcfp(Transform.r[2], RZ, Transform.r[3]);
    RZ = __vmaddfp(Y, Transform.r[1], RZ);
    RZ = __vmaddfp(X, Transform.r[0], RZ);

    InverseW = __vspltw(RZ, 3);

    Reciprocal = __vrefp(InverseW);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(InverseW, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(InverseW, S0, S1);
    InverseW = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    InverseW = __vsel(Reciprocal, S0, InverseW);

    Result = __vmulfp(RZ, InverseW);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT3* XMVector3UnprojectStream
(
    XMFLOAT3*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT3* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    FLOAT           ViewportX, 
    FLOAT           ViewportY, 
    FLOAT           ViewportWidth, 
    FLOAT           ViewportHeight, 
    FLOAT           ViewportMinZ, 
    FLOAT           ViewportMaxZ, 
    CXMMATRIX       Projection, 
    CXMMATRIX       View, 
    CXMMATRIX       World)
{
#if defined(_XM_NO_INTRINSICS_)

    XMMATRIX        Transform;
    XMVECTOR        Scale;
    XMVECTOR        Offset;
    XMVECTOR        V;
    XMVECTOR        Determinant;
    XMVECTOR        Result;
    size_t          i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*           pOutputVector = (BYTE*)pOutputStream;
    CONST XMVECTOR  D = XMVectorSet(-1.0f, 1.0f, 0.0f, 0.0f);

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    Scale = XMVectorSet(ViewportWidth * 0.5f,
                        -ViewportHeight * 0.5f,
                        ViewportMaxZ - ViewportMinZ,
                        1.0f);
    Scale = XMVectorReciprocal(Scale);

    Offset = XMVectorSet(-ViewportX,
                        -ViewportY,
                        -ViewportMinZ,
                        0.0f);
    Offset = XMVectorMultiplyAdd(Scale, Offset, D);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);
    Transform = XMMatrixInverse(&Determinant, Transform);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);

        Result = XMVectorMultiplyAdd(V, Scale, Offset);

        Result = XMVector3TransformCoord(Result, Transform);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMMATRIX              Transform;
    XMVECTOR              Scale;
    XMVECTOR              Offset;
    XMVECTOR              V;
    XMVECTOR              Determinant;
    XMVECTOR              Result;
    size_t                i;
    CONST BYTE*           pInputVector = (CONST BYTE*)pInputStream;
    BYTE*                 pOutputVector = (BYTE*)pOutputStream;
    static CONST XMVECTOR D = {-1.0f, 1.0f, 0.0f, 0.0f};

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    Scale = XMVectorSet(ViewportWidth * 0.5f,
                        -ViewportHeight * 0.5f,
                        ViewportMaxZ - ViewportMinZ,
                        1.0f);
    Scale = XMVectorReciprocal(Scale);

    Offset = XMVectorSet(-ViewportX,
                        -ViewportY,
                        -ViewportMinZ,
                        0.0f);
    Offset = XMVectorMultiplyAdd(Scale, Offset, D);

    Transform = XMMatrixMultiply(World, View);
    Transform = XMMatrixMultiply(Transform, Projection);
    Transform = XMMatrixInverse(&Determinant, Transform);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat3((const XMFLOAT3*)pInputVector);

        Result = XMVectorMultiplyAdd(V, Scale, Offset);

        Result = XMVector3TransformCoord(Result, Transform);

        XMStoreFloat3((XMFLOAT3*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMVECTOR M00, M01, M02, M03,
             M10, M11, M12, M13,
             M20, M21, M22, M23,
             M30, M31, M32, M33;
    XMVECTOR M00M02, M01M03, M10M12, M11M13,
             M20M22, M21M23, M30M32, M31M33;
    XMMATRIX M2T, P, MT;
    XMMATRIX Transform;
    XMVECTOR Scale;
    XMVECTOR Offset;
    XMVECTOR ScaleXYZX, ScaleYZXY, ScaleZXYZ;
    XMVECTOR OffsetXYZX, OffsetYZXY, OffsetZXYZ;
    XMVECTOR Determinant;
    XMVECTOR Reciprocal;
    XMVECTOR PR1, PR2;
    XMVECTOR D0, D1, D2;
    XMVECTOR Refine;
    XMVECTOR V0[4], V1[4];
    XMVECTOR C1W[8], R0[8], R1[8], R2[8], R3[8];
    XMVECTOR L[6];//L[UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR)];
    XMVECTOR Zero, SignMask, Sign0, Sign1;
    XMVECTOR S0;
    XMVECTOR S1;
    XMVECTOR D;
    XMMATRIX R;
    XMVECTOR NegativeOne;
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT3) == 12);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    XMDUMMY_INITIALIZE_VECTOR(S1);

    Scale.vector4_f32[0] = ViewportWidth * 0.5f;
    Scale.vector4_f32[1] = -ViewportHeight * 0.5f;
    Scale.vector4_f32[2] = ViewportMaxZ - ViewportMinZ;
    Scale.vector4_f32[3] = 1.0f;

    Offset.vector4_f32[0] = -ViewportX;
    Offset.vector4_f32[1] = -ViewportY;
    Offset.vector4_f32[2] = -ViewportMinZ;
    Offset.vector4_f32[3] = 0.0f;

    P.r[0] = __vmrghw(View.r[0], View.r[2]);
    P.r[1] = __vmrghw(View.r[1], View.r[3]);
    P.r[2] = __vmrglw(View.r[0], View.r[2]);
    P.r[3] = __vmrglw(View.r[1], View.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00 = __vmsum4fp(World.r[0], M2T.r[0]);
    M02 = __vmsum4fp(World.r[0], M2T.r[2]);
    M01 = __vmsum4fp(World.r[0], M2T.r[1]);
    M03 = __vmsum4fp(World.r[0], M2T.r[3]);

    M10 = __vmsum4fp(World.r[1], M2T.r[0]);
    M12 = __vmsum4fp(World.r[1], M2T.r[2]);
    M11 = __vmsum4fp(World.r[1], M2T.r[1]);
    M13 = __vmsum4fp(World.r[1], M2T.r[3]);

    M20 = __vmsum4fp(World.r[2], M2T.r[0]);
    M22 = __vmsum4fp(World.r[2], M2T.r[2]);
    M21 = __vmsum4fp(World.r[2], M2T.r[1]);
    M23 = __vmsum4fp(World.r[2], M2T.r[3]);

    M30 = __vmsum4fp(World.r[3], M2T.r[0]);
    M32 = __vmsum4fp(World.r[3], M2T.r[2]);
    M31 = __vmsum4fp(World.r[3], M2T.r[1]);
    M33 = __vmsum4fp(World.r[3], M2T.r[3]);

    P.r[0] = __vmrghw(Projection.r[0], Projection.r[2]);
    P.r[1] = __vmrghw(Projection.r[1], Projection.r[3]);
    P.r[2] = __vmrglw(Projection.r[0], Projection.r[2]);
    P.r[3] = __vmrglw(Projection.r[1], Projection.r[3]);

    M2T.r[0] = __vmrghw(P.r[0], P.r[1]);
    M2T.r[2] = __vmrghw(P.r[2], P.r[3]);
    M2T.r[1] = __vmrglw(P.r[0], P.r[1]);
    M2T.r[3] = __vmrglw(P.r[2], P.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    M00 = __vmsum4fp(Transform.r[0], M2T.r[0]);
    M02 = __vmsum4fp(Transform.r[0], M2T.r[2]);
    M01 = __vmsum4fp(Transform.r[0], M2T.r[1]);
    M03 = __vmsum4fp(Transform.r[0], M2T.r[3]);

    M10 = __vmsum4fp(Transform.r[1], M2T.r[0]);
    M12 = __vmsum4fp(Transform.r[1], M2T.r[2]);
    M11 = __vmsum4fp(Transform.r[1], M2T.r[1]);
    M13 = __vmsum4fp(Transform.r[1], M2T.r[3]);

    M20 = __vmsum4fp(Transform.r[2], M2T.r[0]);
    M22 = __vmsum4fp(Transform.r[2], M2T.r[2]);
    M21 = __vmsum4fp(Transform.r[2], M2T.r[1]);
    M23 = __vmsum4fp(Transform.r[2], M2T.r[3]);

    M30 = __vmsum4fp(Transform.r[3], M2T.r[0]);
    M32 = __vmsum4fp(Transform.r[3], M2T.r[2]);
    M31 = __vmsum4fp(Transform.r[3], M2T.r[1]);
    M33 = __vmsum4fp(Transform.r[3], M2T.r[3]);

    M00M02 = __vmrghw(M00, M02);
    M01M03 = __vmrghw(M01, M03);
    M10M12 = __vmrghw(M10, M12);
    M11M13 = __vmrghw(M11, M13);
    M20M22 = __vmrghw(M20, M22);
    M21M23 = __vmrghw(M21, M23);
    M30M32 = __vmrghw(M30, M32);
    M31M33 = __vmrghw(M31, M33);

    Transform.r[0] = __vmrghw(M00M02, M01M03);
    Transform.r[1] = __vmrghw(M10M12, M11M13);
    Transform.r[2] = __vmrghw(M20M22, M21M23);
    Transform.r[3] = __vmrghw(M30M32, M31M33);

    P.r[0] = __vmrghw(Transform.r[0], Transform.r[2]);
    P.r[1] = __vmrghw(Transform.r[1], Transform.r[3]);
    P.r[2] = __vmrglw(Transform.r[0], Transform.r[2]);
    P.r[3] = __vmrglw(Transform.r[1], Transform.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    PR2 = P.r[2];
    PR1 = P.r[1];

    V0[0] = __vpermwi(MT.r[2], 0x5);
    V1[0] = __vpermwi(MT.r[3], 0xBB);
    V0[1] = __vpermwi(MT.r[0], 0x5);
    V1[1] = __vpermwi(MT.r[1], 0xBB);
    PR2 = __vrlimi(PR2, P.r[0], 0x3, 2);
    PR1 = __vrlimi(PR1, P.r[3], 0xC, 2);

    D0 = __vmulfp(V0[0], V1[0]);
    D1 = __vmulfp(V0[1], V1[1]);
    D2 = __vmulfp(PR2, PR1);

    V0[0] = __vpermwi(MT.r[2], 0xBB);
    V1[0] = __vpermwi(MT.r[3], 0x5);
    V0[1] = __vpermwi(MT.r[0], 0xBB);
    V1[1] = __vpermwi(MT.r[1], 0x5);
    P.r[3] = __vrlimi(P.r[3], P.r[1], 0x3, 2);
    P.r[0] = __vrlimi(P.r[0], P.r[2], 0xC, 2);

    D0 = __vnmsubfp(V0[0], V1[0], D0);
    D1 = __vnmsubfp(V0[1], V1[1], D1);
    D2 = __vnmsubfp(P.r[3], P.r[0], D2);

    SignMask = __vspltisw(-1);
    Zero = __vspltisw(0);
    SignMask = __vslw(SignMask, SignMask);
    Sign0 = __vmrghw(Zero, SignMask);
    Sign1 = __vmrghw(SignMask, Zero);

    V1[0] = __vpermwi(D0, 0x9C);
    V1[1] = __vpermwi(D0, 0xC6);
    V1[2] = __vpermwi(D1, 0x9C);
    V1[3] = __vpermwi(D1, 0xC6);

    V0[0] = __vpermwi(MT.r[1], 0x61);
    V1[0] = __vrlimi(V1[0], D2, 0x8, 1);
    V0[1] = __vpermwi(MT.r[0], 0x84);
    V1[1] = __vrlimi(V1[1], D2, 0x4, 0);
    V0[2] = __vpermwi(MT.r[3], 0x61);
    V1[2] = __vrlimi(V1[2], D2, 0x8, 3);
    V0[3] = __vpermwi(MT.r[2], 0x84);
    V1[3] = __vrlimi(V1[3], D2, 0x4, 2);

    R.r[0] = __vmulfp(V0[0], V1[0]);
    R.r[1] = __vmulfp(V0[1], V1[1]);
    R.r[2] = __vmulfp(V0[2], V1[2]);
    R.r[3] = __vmulfp(V0[3], V1[3]);

    V1[0] = __vpermwi(D0, 0xC6);
    V1[1] = __vpermwi(D0, 0x9C);
    V1[2] = __vpermwi(D1, 0xC6);
    V1[3] = __vpermwi(D1, 0x9C);

    V0[0] = __vpermwi(MT.r[1], 0xB6);
    V1[0] = __vrlimi(V1[0], D2, 0x1, 1);
    V0[1] = __vpermwi(MT.r[0], 0xED);
    V1[1] = __vrlimi(V1[1], D2, 0x2, 2);
    V0[2] = __vpermwi(MT.r[3], 0xB6);
    V1[2] = __vrlimi(V1[2], D2, 0x1, 3);
    V0[3] = __vpermwi(MT.r[2], 0xED);
    V1[3] = __vrlimi(V1[3], D2, 0x2, 0);

    R.r[0] = __vnmsubfp(V0[0], V1[0], R.r[0]);
    R.r[1] = __vnmsubfp(V0[1], V1[1], R.r[1]);
    R.r[2] = __vnmsubfp(V0[2], V1[2], R.r[2]);
    R.r[3] = __vnmsubfp(V0[3], V1[3], R.r[3]);

    V1[0] = __vsldoi(D0, D2, 2 << 2);
    V1[1] = __vpermwi(D0, 0x4E);
    V1[2] = __vmrglw(D1, D2);
    V1[3] = __vpermwi(D1, 0x4E);

    V0[0] = __vpermwi(MT.r[1], 0xCC);
    V1[0] = __vpermwi(V1[0], 0x38);
    V0[1] = __vpermwi(MT.r[0], 0x72);
    V1[1] = __vrlimi(V1[1], D2, 0x9, 1);
    V0[2] = __vpermwi(MT.r[3], 0xCC);
    V1[2] = __vpermwi(V1[2], 0x34);
    V0[3] = __vpermwi(MT.r[2], 0x72);
    V1[3] = __vrlimi(V1[3], D2, 0x9, 3);

    V0[0] = __vxor(V0[0], Sign0);
    V0[1] = __vxor(V0[1], Sign1);
    V0[2] = __vxor(V0[2], Sign0);
    V0[3] = __vxor(V0[3], Sign1);

    R.r[0] = __vmaddfp(V0[0], V1[0], R.r[0]);
    R.r[1] = __vmaddfp(V0[1], V1[1], R.r[1]);
    R.r[2] = __vmaddfp(V0[2], V1[2], R.r[2]);
    R.r[3] = __vmaddfp(V0[3], V1[3], R.r[3]);

    Determinant = __vmsum4fp(R.r[0], MT.r[0]);

    Reciprocal = __vrefp(Determinant);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    S0 = __vnmsubfp(Determinant, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    S1 = __vnmsubfp(Determinant, S0, S1);
    Refine = __vcmpeqfp(S0, S0);
    S0 = __vmaddfp(S0, S1, S0);
    Refine = __vsel(Reciprocal, S0, Refine);

    Transform.r[0] = __vmulfp(R.r[0], Refine);
    Transform.r[1] = __vmulfp(R.r[1], Refine);
    Transform.r[2] = __vmulfp(R.r[2], Refine);
    Transform.r[3] = __vmulfp(R.r[3], Refine);

    Reciprocal = __vrefp(Scale);
    S1 = __vupkd3d(S1, VPACK_NORMSHORT2);
    S0 = __vspltw(S1, 3);
    D = __vpermwi(S1, 0xEA);
    S0 = __vnmsubfp(Scale, Reciprocal, S0);
    S0 = __vmaddcfp(Reciprocal, S0, Reciprocal);
    S1 = __vspltw(S1, 3);
    NegativeOne = __vspltisw(-1);
    S1 = __vnmsubfp(Scale, S0, S1);
    Scale = __vcmpeqfp(S0, S0);
    NegativeOne = __vcfsx(NegativeOne, 0);
    S0 = __vmaddfp(S0, S1, S0);
    D = __vsldoi(NegativeOne, D, 3 << 2);
    Scale = __vsel(Reciprocal, S0, Scale);

    Offset = __vmaddcfp(Scale, Offset, D);

    P.r[0] = __vmrghw(Transform.r[0], Transform.r[2]);
    P.r[1] = __vmrghw(Transform.r[1], Transform.r[3]);
    P.r[2] = __vmrglw(Transform.r[0], Transform.r[2]);
    P.r[3] = __vmrglw(Transform.r[1], Transform.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    XMDUMMY_INITIALIZE_VECTOR(C1W[0]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[1]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[2]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[3]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[4]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[5]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[6]);
    XMDUMMY_INITIALIZE_VECTOR(C1W[7]);

    C1W[0] = __vupkd3d(C1W[0], VPACK_NORMSHORT2);
    C1W[1] = __vupkd3d(C1W[1], VPACK_NORMSHORT2);
    C1W[2] = __vupkd3d(C1W[2], VPACK_NORMSHORT2);
    C1W[3] = __vupkd3d(C1W[3], VPACK_NORMSHORT2);
    C1W[4] = __vupkd3d(C1W[4], VPACK_NORMSHORT2);
    C1W[5] = __vupkd3d(C1W[5], VPACK_NORMSHORT2);
    C1W[6] = __vupkd3d(C1W[6], VPACK_NORMSHORT2);
    C1W[7] = __vupkd3d(C1W[7], VPACK_NORMSHORT2);

    ScaleXYZX = XMVectorSwizzle(Scale, 0, 1, 2, 0);
    ScaleYZXY = XMVectorSwizzle(Scale, 1, 2, 0, 1);
    ScaleZXYZ = XMVectorSwizzle(Scale, 2, 0, 1, 2);
    OffsetXYZX = XMVectorSwizzle(Offset, 0, 1, 2, 0);
    OffsetYZXY = XMVectorSwizzle(Offset, 1, 2, 0, 1);
    OffsetZXYZ = XMVectorSwizzle(Offset, 2, 0, 1, 2);

    if (InputStride == sizeof(XMFLOAT3))
    {
//        C_ASSERT(UnrollCount * sizeof(XMFLOAT3) / sizeof(XMVECTOR) == 6);

        UINT GroupIndex;
        UINT VGroupSize = UnrollCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT3);
        UINT_PTR InputStreamCacheAligned = (InputVector + XM_CACHE_LINE_SIZE - 1) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupBase;
        UINT_PTR InputStreamCGroupEnd;
        UINT_PTR InputStreamVGroupBase;
        UINT_PTR InputStreamVGroupEnd;
        InputStreamCGroupBase = InputStreamCacheAligned + (((InputStreamCacheAligned - InputVector) % 12) >> 2) * XM_CACHE_LINE_SIZE;
        if (InputStreamCGroupBase >= InputStreamEnd) InputStreamCGroupBase = InputVector;
        InputStreamCGroupEnd = InputStreamCGroupBase + (InputStreamEnd - InputStreamCGroupBase) / (3 * XM_CACHE_LINE_SIZE) * (3 * XM_CACHE_LINE_SIZE);
        InputStreamVGroupBase = InputVector + (InputStreamCGroupBase - InputVector) % VGroupSize;
        InputStreamVGroupEnd = InputStreamCGroupEnd + (InputStreamEnd - InputStreamCGroupEnd) / VGroupSize * VGroupSize;

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamVGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupBase)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);
            R3[1] = __vmaddcfp(ScaleXYZX, R3[1], OffsetXYZX);
            R3[2] = __vmaddcfp(ScaleXYZX, R3[2], OffsetXYZX);
            R3[3] = __vmaddcfp(ScaleXYZX, R3[3], OffsetXYZX);
            R3[4] = __vmaddcfp(ScaleXYZX, R3[4], OffsetXYZX);
            R3[5] = __vmaddcfp(ScaleXYZX, R3[5], OffsetXYZX);
            R3[6] = __vmaddcfp(ScaleXYZX, R3[6], OffsetXYZX);
            R3[7] = __vmaddcfp(ScaleXYZX, R3[7], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                L[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);
                L[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMVECTOR);

                L[0] = __vmaddcfp(ScaleXYZX, L[0], OffsetXYZX);
                L[1] = __vmaddcfp(ScaleYZXY, L[1], OffsetYZXY);
                L[2] = __vmaddcfp(ScaleZXYZ, L[2], OffsetZXYZ);
                L[3] = __vmaddcfp(ScaleXYZX, L[3], OffsetXYZX);
                L[4] = __vmaddcfp(ScaleYZXY, L[4], OffsetYZXY);
                L[5] = __vmaddcfp(ScaleZXYZ, L[5], OffsetZXYZ);

                R0[0] = __vspltw(L[0], 2);
                R1[0] = __vspltw(L[0], 1);
                R2[0] = __vspltw(L[0], 0);

                R0[1] = __vspltw(L[1], 1);
                R1[1] = __vspltw(L[1], 0);
                R2[1] = __vspltw(L[0], 3);

                R0[2] = __vspltw(L[2], 0);
                R1[2] = __vspltw(L[1], 3);
                R2[2] = __vspltw(L[1], 2);

                R0[3] = __vspltw(L[2], 3);
                R1[3] = __vspltw(L[2], 2);
                R2[3] = __vspltw(L[2], 1);

                R0[4] = __vspltw(L[3], 2);
                R1[4] = __vspltw(L[3], 1);
                R2[4] = __vspltw(L[3], 0);

                R0[5] = __vspltw(L[4], 1);
                R1[5] = __vspltw(L[4], 0);
                R2[5] = __vspltw(L[3], 3);

                R0[6] = __vspltw(L[5], 0);
                R1[6] = __vspltw(L[4], 3);
                R2[6] = __vspltw(L[4], 2);

                R0[7] = __vspltw(L[5], 3);
                R1[7] = __vspltw(L[5], 2);
                R2[7] = __vspltw(L[5], 1);

                R0[0] = __vmaddcfp(R0[0], Transform.r[2], Transform.r[3]);
                R0[1] = __vmaddcfp(R0[1], Transform.r[2], Transform.r[3]);
                R0[2] = __vmaddcfp(R0[2], Transform.r[2], Transform.r[3]);
                R0[3] = __vmaddcfp(R0[3], Transform.r[2], Transform.r[3]);
                R0[4] = __vmaddcfp(R0[4], Transform.r[2], Transform.r[3]);
                R0[5] = __vmaddcfp(R0[5], Transform.r[2], Transform.r[3]);
                R0[6] = __vmaddcfp(R0[6], Transform.r[2], Transform.r[3]);
                R0[7] = __vmaddcfp(R0[7], Transform.r[2], Transform.r[3]);

                R0[0] = __vmaddcfp(R1[0], Transform.r[1], R0[0]);
                R0[1] = __vmaddcfp(R1[1], Transform.r[1], R0[1]);
                R0[2] = __vmaddcfp(R1[2], Transform.r[1], R0[2]);
                R0[3] = __vmaddcfp(R1[3], Transform.r[1], R0[3]);
                R0[4] = __vmaddcfp(R1[4], Transform.r[1], R0[4]);
                R0[5] = __vmaddcfp(R1[5], Transform.r[1], R0[5]);
                R0[6] = __vmaddcfp(R1[6], Transform.r[1], R0[6]);
                R0[7] = __vmaddcfp(R1[7], Transform.r[1], R0[7]);

                R0[0] = __vmaddcfp(R2[0], Transform.r[0], R0[0]);
                R0[1] = __vmaddcfp(R2[1], Transform.r[0], R0[1]);
                R0[2] = __vmaddcfp(R2[2], Transform.r[0], R0[2]);
                R0[3] = __vmaddcfp(R2[3], Transform.r[0], R0[3]);
                R0[4] = __vmaddcfp(R2[4], Transform.r[0], R0[4]);
                R0[5] = __vmaddcfp(R2[5], Transform.r[0], R0[5]);
                R0[6] = __vmaddcfp(R2[6], Transform.r[0], R0[6]);
                R0[7] = __vmaddcfp(R2[7], Transform.r[0], R0[7]);

                R1[0] = __vspltw(R0[0], 3);
                R1[1] = __vspltw(R0[1], 3);
                R1[2] = __vspltw(R0[2], 3);
                R1[3] = __vspltw(R0[3], 3);
                R1[4] = __vspltw(R0[4], 3);
                R1[5] = __vspltw(R0[5], 3);
                R1[6] = __vspltw(R0[6], 3);
                R1[7] = __vspltw(R0[7], 3);

                R1[0] = XMVectorReciprocal(R1[0]);
                R1[1] = XMVectorReciprocal(R1[1]);
                R1[2] = XMVectorReciprocal(R1[2]);
                R1[3] = XMVectorReciprocal(R1[3]);
                R1[4] = XMVectorReciprocal(R1[4]);
                R1[5] = XMVectorReciprocal(R1[5]);
                R1[6] = XMVectorReciprocal(R1[6]);
                R1[7] = XMVectorReciprocal(R1[7]);

                R0[0] = __vmulfp(R0[0], R1[0]);
                R0[1] = __vmulfp(R0[1], R1[1]);
                R0[2] = __vmulfp(R0[2], R1[2]);
                R0[3] = __vmulfp(R0[3], R1[3]);
                R0[4] = __vmulfp(R0[4], R1[4]);
                R0[5] = __vmulfp(R0[5], R1[5]);
                R0[6] = __vmulfp(R0[6], R1[6]);
                R0[7] = __vmulfp(R0[7], R1[7]);

                R2[0] = __vspltw(R0[0], 0);
                R1[0] = __vspltw(R0[0], 1);
                R0[0] = __vspltw(R0[0], 2);
                                    
                R2[1] = __vspltw(R0[1], 0);
                R1[1] = __vspltw(R0[1], 1);
                R0[1] = __vspltw(R0[1], 2);
                                    
                R2[2] = __vspltw(R0[2], 0);
                R1[2] = __vspltw(R0[2], 1);
                R0[2] = __vspltw(R0[2], 2);
                                    
                R2[3] = __vspltw(R0[3], 0);
                R1[3] = __vspltw(R0[3], 1);
                R0[3] = __vspltw(R0[3], 2);
                                    
                R2[4] = __vspltw(R0[4], 0);
                R1[4] = __vspltw(R0[4], 1);
                R0[4] = __vspltw(R0[4], 2);
                                    
                R2[5] = __vspltw(R0[5], 0);
                R1[5] = __vspltw(R0[5], 1);
                R0[5] = __vspltw(R0[5], 2);
                                    
                R2[6] = __vspltw(R0[6], 0);
                R1[6] = __vspltw(R0[6], 1);
                R0[6] = __vspltw(R0[6], 2);
                                    
                R2[7] = __vspltw(R0[7], 0);
                R1[7] = __vspltw(R0[7], 1);
                R0[7] = __vspltw(R0[7], 2);

                __stvewx(R2[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R0[0], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R0[1], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R0[2], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R0[3], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R0[4], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R0[5], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R0[6], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
                __stvewx(R2[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R0[7], (VOID*)OutputVector, 8);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += sizeof(XMFLOAT3);

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);
            R3[1] = __vmaddcfp(ScaleXYZX, R3[1], OffsetXYZX);
            R3[2] = __vmaddcfp(ScaleXYZX, R3[2], OffsetXYZX);
            R3[3] = __vmaddcfp(ScaleXYZX, R3[3], OffsetXYZX);
            R3[4] = __vmaddcfp(ScaleXYZX, R3[4], OffsetXYZX);
            R3[5] = __vmaddcfp(ScaleXYZX, R3[5], OffsetXYZX);
            R3[6] = __vmaddcfp(ScaleXYZX, R3[6], OffsetXYZX);
            R3[7] = __vmaddcfp(ScaleXYZX, R3[7], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += sizeof(XMFLOAT3); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[1] = __lvrx((VOID*)InputVector, 12);
            R0[1] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[2] = __lvrx((VOID*)InputVector, 12);
            R0[2] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[3] = __lvrx((VOID*)InputVector, 12);
            R0[3] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[4] = __lvrx((VOID*)InputVector, 12);
            R0[4] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[5] = __lvrx((VOID*)InputVector, 12);
            R0[5] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[6] = __lvrx((VOID*)InputVector, 12);
            R0[6] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;
            R1[7] = __lvrx((VOID*)InputVector, 12);
            R0[7] = __lvlx((VOID*)InputVector, 0);
            InputVector += InputStride;

            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R1[1] = __vsldoi(R1[1], R1[1], 1 << 2);
            R1[2] = __vsldoi(R1[2], R1[2], 1 << 2);
            R1[3] = __vsldoi(R1[3], R1[3], 1 << 2);
            R1[4] = __vsldoi(R1[4], R1[4], 1 << 2);
            R1[5] = __vsldoi(R1[5], R1[5], 1 << 2);
            R1[6] = __vsldoi(R1[6], R1[6], 1 << 2);
            R1[7] = __vsldoi(R1[7], R1[7], 1 << 2);

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);
            R3[1] = __vmaddcfp(ScaleXYZX, R3[1], OffsetXYZX);
            R3[2] = __vmaddcfp(ScaleXYZX, R3[2], OffsetXYZX);
            R3[3] = __vmaddcfp(ScaleXYZX, R3[3], OffsetXYZX);
            R3[4] = __vmaddcfp(ScaleXYZX, R3[4], OffsetXYZX);
            R3[5] = __vmaddcfp(ScaleXYZX, R3[5], OffsetXYZX);
            R3[6] = __vmaddcfp(ScaleXYZX, R3[6], OffsetXYZX);
            R3[7] = __vmaddcfp(ScaleXYZX, R3[7], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);
            R3[1] = __vrlimi(R3[1], C1W[0], 0x1, 0);
            R3[2] = __vrlimi(R3[2], C1W[0], 0x1, 0);
            R3[3] = __vrlimi(R3[3], C1W[0], 0x1, 0);
            R3[4] = __vrlimi(R3[4], C1W[0], 0x1, 0);
            R3[5] = __vrlimi(R3[5], C1W[0], 0x1, 0);
            R3[6] = __vrlimi(R3[6], C1W[0], 0x1, 0);
            R3[7] = __vrlimi(R3[7], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[1] = __vmrghw(R0[1], R2[1]);
            R0[2] = __vmrghw(R0[2], R2[2]);
            R0[3] = __vmrghw(R0[3], R2[3]);
            R0[4] = __vmrghw(R0[4], R2[4]);
            R0[5] = __vmrghw(R0[5], R2[5]);
            R0[6] = __vmrghw(R0[6], R2[6]);
            R0[7] = __vmrghw(R0[7], R2[7]);

            R0[0] = __vmrghw(R0[0], R1[0]);
            R0[1] = __vmrghw(R0[1], R1[1]);
            R0[2] = __vmrghw(R0[2], R1[2]);
            R0[3] = __vmrghw(R0[3], R1[3]);
            R0[4] = __vmrghw(R0[4], R1[4]);
            R0[5] = __vmrghw(R0[5], R1[5]);
            R0[6] = __vmrghw(R0[6], R1[6]);
            R0[7] = __vmrghw(R0[7], R1[7]);

            R1[0] = XMVectorReciprocal(R3[0]);
            R1[1] = XMVectorReciprocal(R3[1]);
            R1[2] = XMVectorReciprocal(R3[2]);
            R1[3] = XMVectorReciprocal(R3[3]);
            R1[4] = XMVectorReciprocal(R3[4]);
            R1[5] = XMVectorReciprocal(R3[5]);
            R1[6] = XMVectorReciprocal(R3[6]);
            R1[7] = XMVectorReciprocal(R3[7]);

            R0[0] = __vmulfp(R0[0], R1[0]);
            R0[1] = __vmulfp(R0[1], R1[1]);
            R0[2] = __vmulfp(R0[2], R1[2]);
            R0[3] = __vmulfp(R0[3], R1[3]);
            R0[4] = __vmulfp(R0[4], R1[4]);
            R0[5] = __vmulfp(R0[5], R1[5]);
            R0[6] = __vmulfp(R0[6], R1[6]);
            R0[7] = __vmulfp(R0[7], R1[7]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);
                                
            R2[1] = __vspltw(R0[1], 0);
            R1[1] = __vspltw(R0[1], 1);
            R0[1] = __vspltw(R0[1], 2);
                                
            R2[2] = __vspltw(R0[2], 0);
            R1[2] = __vspltw(R0[2], 1);
            R0[2] = __vspltw(R0[2], 2);
                                
            R2[3] = __vspltw(R0[3], 0);
            R1[3] = __vspltw(R0[3], 1);
            R0[3] = __vspltw(R0[3], 2);
                                
            R2[4] = __vspltw(R0[4], 0);
            R1[4] = __vspltw(R0[4], 1);
            R0[4] = __vspltw(R0[4], 2);
                                
            R2[5] = __vspltw(R0[5], 0);
            R1[5] = __vspltw(R0[5], 1);
            R0[5] = __vspltw(R0[5], 2);
                                
            R2[6] = __vspltw(R0[6], 0);
            R1[6] = __vspltw(R0[6], 1);
            R0[6] = __vspltw(R0[6], 2);
                                
            R2[7] = __vspltw(R0[7], 0);
            R1[7] = __vspltw(R0[7], 1);
            R0[7] = __vspltw(R0[7], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R0[1], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R0[2], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R0[3], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R0[4], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R0[5], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R0[6], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
            __stvewx(R2[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R0[7], (VOID*)OutputVector, 8);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R1[0] = __lvrx((VOID*)InputVector, 12);
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __vsldoi(R1[0], R1[0], 1 << 2);
            R3[0] = __vor(R0[0], R1[0]);

            R3[0] = __vmaddcfp(ScaleXYZX, R3[0], OffsetXYZX);

            R3[0] = __vrlimi(R3[0], C1W[0], 0x1, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            R0[0] = __vmrghw(R0[0], R2[0]);
            R0[0] = __vmrghw(R0[0], R1[0]);

            R1[0] = XMVectorReciprocal(R3[0]);

            R0[0] = __vmulfp(R0[0], R1[0]);

            R2[0] = __vspltw(R0[0], 0);
            R1[0] = __vspltw(R0[0], 1);
            R0[0] = __vspltw(R0[0], 2);

            __stvewx(R2[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R0[0], (VOID*)OutputVector, 8);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

/****************************************************************************
 *
 * 4D Vector
 *
 ****************************************************************************/

//------------------------------------------------------------------------------
// Comparison operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4Equal
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] == V2.vector4_f32[0]) && (V1.vector4_f32[1] == V2.vector4_f32[1]) && (V1.vector4_f32[2] == V2.vector4_f32[2]) && (V1.vector4_f32[3] == V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4EqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector4EqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;

    if ((V1.vector4_f32[0] == V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] == V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] == V2.vector4_f32[2]) &&
        (V1.vector4_f32[3] == V2.vector4_f32[3]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] != V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] != V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] != V2.vector4_f32[2]) &&
        (V1.vector4_f32[3] != V2.vector4_f32[3]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    __vcmpeqfpR(V1, V2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4EqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] == V2.vector4_u32[0]) && (V1.vector4_u32[1] == V2.vector4_u32[1]) && (V1.vector4_u32[2] == V2.vector4_u32[2]) && (V1.vector4_u32[3] == V2.vector4_u32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector4EqualIntR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if (V1.vector4_u32[0] == V2.vector4_u32[0] && 
        V1.vector4_u32[1] == V2.vector4_u32[1] &&
        V1.vector4_u32[2] == V2.vector4_u32[2] &&
        V1.vector4_u32[3] == V2.vector4_u32[3])
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if (V1.vector4_u32[0] != V2.vector4_u32[0] && 
        V1.vector4_u32[1] != V2.vector4_u32[1] &&
        V1.vector4_u32[2] != V2.vector4_u32[2] &&
        V1.vector4_u32[3] != V2.vector4_u32[3])
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    __vcmpequwR(V1, V2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

XMFINLINE BOOL XMVector4NearEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR Epsilon
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT dx, dy, dz, dw;

    dx = fabsf(V1.vector4_f32[0]-V2.vector4_f32[0]);
    dy = fabsf(V1.vector4_f32[1]-V2.vector4_f32[1]);
    dz = fabsf(V1.vector4_f32[2]-V2.vector4_f32[2]);
    dw = fabsf(V1.vector4_f32[3]-V2.vector4_f32[3]);
    return (((dx <= Epsilon.vector4_f32[0]) &&
            (dy <= Epsilon.vector4_f32[1]) &&
            (dz <= Epsilon.vector4_f32[2]) &&
            (dw <= Epsilon.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR    Delta;
    UINT        CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    Delta = __vsubfp(V1, V2);
    __vcmpbfpR(Delta, Epsilon, &CR);

    return XMComparisonAllInBounds(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4NotEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] != V2.vector4_f32[0]) || (V1.vector4_f32[1] != V2.vector4_f32[1]) || (V1.vector4_f32[2] != V2.vector4_f32[2]) || (V1.vector4_f32[3] != V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAnyFalse(XMVector4EqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4NotEqualInt
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_u32[0] != V2.vector4_u32[0]) || (V1.vector4_u32[1] != V2.vector4_u32[1]) || (V1.vector4_u32[2] != V2.vector4_u32[2]) || (V1.vector4_u32[3] != V2.vector4_u32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAnyFalse(XMVector4EqualIntR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4Greater
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] > V2.vector4_f32[0]) && (V1.vector4_f32[1] > V2.vector4_f32[1]) && (V1.vector4_f32[2] > V2.vector4_f32[2]) && (V1.vector4_f32[3] > V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4GreaterR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector4GreaterR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if (V1.vector4_f32[0] > V2.vector4_f32[0] && 
        V1.vector4_f32[1] > V2.vector4_f32[1] &&
        V1.vector4_f32[2] > V2.vector4_f32[2] &&
        V1.vector4_f32[3] > V2.vector4_f32[3])
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if (V1.vector4_f32[0] <= V2.vector4_f32[0] && 
        V1.vector4_f32[1] <= V2.vector4_f32[1] &&
        V1.vector4_f32[2] <= V2.vector4_f32[2] &&
        V1.vector4_f32[3] <= V2.vector4_f32[3])
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    __vcmpgtfpR(V1, V2, &CR);

    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4GreaterOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] >= V2.vector4_f32[0]) && (V1.vector4_f32[1] >= V2.vector4_f32[1]) && (V1.vector4_f32[2] >= V2.vector4_f32[2]) && (V1.vector4_f32[3] >= V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4GreaterOrEqualR(V1, V2));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector4GreaterOrEqualR
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    UINT CR = 0;
    if ((V1.vector4_f32[0] >= V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] >= V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] >= V2.vector4_f32[2]) &&
        (V1.vector4_f32[3] >= V2.vector4_f32[3]))
    {
        CR = XM_CRMASK_CR6TRUE;
    }
    else if ((V1.vector4_f32[0] < V2.vector4_f32[0]) && 
        (V1.vector4_f32[1] < V2.vector4_f32[1]) &&
        (V1.vector4_f32[2] < V2.vector4_f32[2]) &&
        (V1.vector4_f32[3] < V2.vector4_f32[3]))
    {
        CR = XM_CRMASK_CR6FALSE;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;
    XMDUMMY_INITIALIZE_UINT(CR);
    __vcmpgefpR(V1, V2, &CR);
    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4Less
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] < V2.vector4_f32[0]) && (V1.vector4_f32[1] < V2.vector4_f32[1]) && (V1.vector4_f32[2] < V2.vector4_f32[2]) && (V1.vector4_f32[3] < V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4GreaterR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4LessOrEqual
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V1.vector4_f32[0] <= V2.vector4_f32[0]) && (V1.vector4_f32[1] <= V2.vector4_f32[1]) && (V1.vector4_f32[2] <= V2.vector4_f32[2]) && (V1.vector4_f32[3] <= V2.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllTrue(XMVector4GreaterOrEqualR(V2, V1));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4InBounds
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) &&
        (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2]) &&
        (V.vector4_f32[3] <= Bounds.vector4_f32[3] && V.vector4_f32[3] >= -Bounds.vector4_f32[3])) != 0);
#elif defined(_XM_SSE_INTRINSICS_)
#else
    return XMComparisonAllInBounds(XMVector4InBoundsR(V, Bounds));
#endif
}

//------------------------------------------------------------------------------

XMFINLINE UINT XMVector4InBoundsR
(
    FXMVECTOR V, 
    FXMVECTOR Bounds
)
{
#if defined(_XM_NO_INTRINSICS_)

    UINT CR = 0;
    if ((V.vector4_f32[0] <= Bounds.vector4_f32[0] && V.vector4_f32[0] >= -Bounds.vector4_f32[0]) && 
        (V.vector4_f32[1] <= Bounds.vector4_f32[1] && V.vector4_f32[1] >= -Bounds.vector4_f32[1]) &&
        (V.vector4_f32[2] <= Bounds.vector4_f32[2] && V.vector4_f32[2] >= -Bounds.vector4_f32[2]) &&
        (V.vector4_f32[3] <= Bounds.vector4_f32[3] && V.vector4_f32[3] >= -Bounds.vector4_f32[3]))
    {
        CR = XM_CRMASK_CR6BOUNDS;
    }
    return CR;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;
    XMDUMMY_INITIALIZE_UINT(CR);
    __vcmpbfpR(V, Bounds, &CR);
    return CR;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4IsNaN
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    return (XMISNAN(V.vector4_f32[0]) ||
            XMISNAN(V.vector4_f32[1]) ||
            XMISNAN(V.vector4_f32[2]) ||
            XMISNAN(V.vector4_f32[3]));
#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    UINT CR;
    XMDUMMY_INITIALIZE_UINT(CR);
    __vcmpeqfpR(V, V, &CR);
    return XMComparisonAnyFalse(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE BOOL XMVector4IsInfinite
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    return (XMISINF(V.vector4_f32[0]) ||
            XMISINF(V.vector4_f32[1]) ||
            XMISINF(V.vector4_f32[2]) ||
            XMISINF(V.vector4_f32[3]));

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR OneInt, TrueInt;
    XMVECTOR VShifted, InfinityShifted;
    XMVECTOR Shift;
    UINT     CR;

    XMDUMMY_INITIALIZE_UINT(CR);

    OneInt = __vspltisw(1);
    TrueInt = __vspltisw(-1);
    Shift = __vspltisw(-8); // Shift 24
    VShifted = __vslw(V, OneInt);
    InfinityShifted = __vslw(TrueInt, Shift);
    __vcmpequwR(VShifted, InfinityShifted, &CR);

    return XMComparisonAnyTrue(CR);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// Computation operations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Dot
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result.vector4_f32[0] =
    Result.vector4_f32[1] =
    Result.vector4_f32[2] =
    Result.vector4_f32[3] = V1.vector4_f32[0] * V2.vector4_f32[0] + V1.vector4_f32[1] * V2.vector4_f32[1] + V1.vector4_f32[2] * V2.vector4_f32[2] + V1.vector4_f32[3] * V2.vector4_f32[3];

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Overflow will result in a QNaN instead of +/- infinity.

    return __vmsum4fp(V1, V2);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Cross
(
    FXMVECTOR V1, 
    FXMVECTOR V2, 
    FXMVECTOR V3
)
{
#if defined(_XM_NO_INTRINSICS_)
    XMVECTOR Result;   

    Result.vector4_f32[0] = (((V2.vector4_f32[2]*V3.vector4_f32[3])-(V2.vector4_f32[3]*V3.vector4_f32[2]))*V1.vector4_f32[1])-(((V2.vector4_f32[1]*V3.vector4_f32[3])-(V2.vector4_f32[3]*V3.vector4_f32[1]))*V1.vector4_f32[2])+(((V2.vector4_f32[1]*V3.vector4_f32[2])-(V2.vector4_f32[2]*V3.vector4_f32[1]))*V1.vector4_f32[3]);
    Result.vector4_f32[1] = (((V2.vector4_f32[3]*V3.vector4_f32[2])-(V2.vector4_f32[2]*V3.vector4_f32[3]))*V1.vector4_f32[0])-(((V2.vector4_f32[3]*V3.vector4_f32[0])-(V2.vector4_f32[0]*V3.vector4_f32[3]))*V1.vector4_f32[2])+(((V2.vector4_f32[2]*V3.vector4_f32[0])-(V2.vector4_f32[0]*V3.vector4_f32[2]))*V1.vector4_f32[3]);
    Result.vector4_f32[2] = (((V2.vector4_f32[1]*V3.vector4_f32[3])-(V2.vector4_f32[3]*V3.vector4_f32[1]))*V1.vector4_f32[0])-(((V2.vector4_f32[0]*V3.vector4_f32[3])-(V2.vector4_f32[3]*V3.vector4_f32[0]))*V1.vector4_f32[1])+(((V2.vector4_f32[0]*V3.vector4_f32[1])-(V2.vector4_f32[1]*V3.vector4_f32[0]))*V1.vector4_f32[3]);
    Result.vector4_f32[3] = (((V2.vector4_f32[2]*V3.vector4_f32[1])-(V2.vector4_f32[1]*V3.vector4_f32[2]))*V1.vector4_f32[0])-(((V2.vector4_f32[2]*V3.vector4_f32[0])-(V2.vector4_f32[0]*V3.vector4_f32[2]))*V1.vector4_f32[1])+(((V2.vector4_f32[1]*V3.vector4_f32[0])-(V2.vector4_f32[0]*V3.vector4_f32[1]))*V1.vector4_f32[2]);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR               V1A, V1B, V1C, V2A, V2B, V2C, V3A, V3B, V3C;
    XMVECTOR               R1, R2, R3;

    V2A = __vpermwi(V2, 0xB6);
    V3A = __vpermwi(V3, 0xED);
    V2B = __vpermwi(V2, 0x72);
    V3B = __vpermwi(V3, 0xCC);
    V2C = __vpermwi(V2, 0x61);
    V3C = __vpermwi(V3, 0x84);

    R1 = __vmulfp(V2A, V3A);
    R2 = __vmulfp(V2B, V3B);
    R3 = __vmulfp(V2C, V3C);

    V2A = __vpermwi(V2, 0xED);
    V3A = __vpermwi(V3, 0xB6);
    V2B = __vpermwi(V2, 0xCC);
    V3B = __vpermwi(V3, 0x72);
    V2C = __vpermwi(V2, 0x84);
    V3C = __vpermwi(V3, 0x61);

    R1 = __vnmsubfp(V2A, V3A, R1);
    R2 = __vnmsubfp(V2B, V3B, R2);
    R3 = __vnmsubfp(V2C, V3C, R3);

    V1A = __vpermwi(V1, 0x40);
    V1B = __vpermwi(V1, 0xA5);
    V1C = __vpermwi(V1, 0xFE);

    R1 = __vmulfp(V1A, R1);
    R1 = __vnmsubfp(V1B, R2, R1);
    R1 = __vmaddfp(V1C, R3, R1);

    return R1;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4LengthSq
(
    FXMVECTOR V
)
{
    return XMVector4Dot(V, V);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4ReciprocalLengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector4LengthSq(V);
    Result = XMVectorReciprocalSqrtEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR Result;

    Result = __vmsum4fp(V, V);
    Result = __vrsqrtefp(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4ReciprocalLength
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector4LengthSq(V);
    Result = XMVectorReciprocalSqrt(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR H;

    D = __vmsum4fp(V, V);
    H = __vspltisw(1);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);

    return Rsq;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4LengthEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;

    Result = XMVector4LengthSq(V);
    Result = XMVectorSqrtEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Result;
    // Returns a QNaN on infinite vectors.
    XMVECTOR Zero;
    XMVECTOR Length;

    D = __vmsum4fp(V, V);
    Zero = __vspltisw(0);
    Rsq = __vrsqrtefp(D);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Length
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_) 

    XMVECTOR Result;

    Result = XMVector4LengthSq(V);
    Result = XMVectorSqrt(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR Length;
    XMVECTOR H;

    H = __vspltisw(1);
    D = __vmsum4fp(V, V);
    H = __vcfsx(H, 1);
    Rsq = __vrsqrtefp(D);
    RT = __vmulfp(D, H);
    Rcp = __vmulfp(Rsq, Rsq);
    Zero = __vspltisw(0);
    H = __vnmsubfp(RT, Rcp, H);
    Rsq = __vmaddfp(Rsq, H, Rsq);
    Result = __vcmpeqfp(D, Zero);
    Length = __vmulfp(D, Rsq);
    Result = __vsel(Length, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------
// XMVector4NormalizeEst uses a reciprocal estimate and
// returns QNaN on zero and infinite vectors.

XMFINLINE XMVECTOR XMVector4NormalizeEst
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result = XMVector4ReciprocalLength(V);
    Result = XMVectorMultiply(V, Result);
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Rsq;
    XMVECTOR Result;
    // Faster, but returns a QNaN on zero and infinite vectors.
    D = __vmsum4fp(V, V);
    Rsq = __vrsqrtefp(D);
    Result = __vmulfp(V, Rsq);
    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Normalize
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fLength;
    XMVECTOR vResult;

    vResult = XMVector4Length( V );
    fLength = vResult.vector4_f32[0];

    // Prevent divide by zero
    if (fLength > 0) {
        fLength = 1.0f/fLength;
    }
    
    vResult.vector4_f32[0] = V.vector4_f32[0]*fLength;
    vResult.vector4_f32[1] = V.vector4_f32[1]*fLength;
    vResult.vector4_f32[2] = V.vector4_f32[2]*fLength;
    vResult.vector4_f32[3] = V.vector4_f32[3]*fLength;
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    // Returns a QNaN on infinite vectors.

    XMVECTOR D;
    XMVECTOR RcpLength;
    XMVECTOR OneHalfLengthSq;
    XMVECTOR Scale;
    XMVECTOR Result;
    XMVECTOR Zero;
    XMVECTOR H;
    XMVECTOR N;

    H = __vspltisw(1);
    D = __vmsum4fp(V, V);
    H = __vcfsx(H, 1);
    Zero = __vspltisw(0);
    RcpLength = __vrsqrtefp(D);
    OneHalfLengthSq = __vmulfp(D, H);
    Result = __vcmpeqfp(D, Zero);
    Scale = __vmulfp(RcpLength, RcpLength);
    H = __vnmsubfp(OneHalfLengthSq, Scale, H);
    H = __vmaddcfp(RcpLength, H, RcpLength);
    N = __vmulfp(V, H);
    Result = __vsel(N, D, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4ClampLength
(
    FXMVECTOR V, 
    FLOAT    LengthMin, 
    FLOAT    LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax = XMVectorReplicate(LengthMax);
    ClampMin = XMVectorReplicate(LengthMin);

    return XMVector4ClampLengthV(V, ClampMin, ClampMax);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    ClampMax.vector4_f32[0] =
    ClampMax.vector4_f32[1] = 
    ClampMax.vector4_f32[2] =
    ClampMax.vector4_f32[3] = LengthMax;

    ClampMin.vector4_f32[0] =
    ClampMin.vector4_f32[1] = 
    ClampMin.vector4_f32[2] =
    ClampMin.vector4_f32[3] = LengthMin;

    return XMVector4ClampLengthV(V, ClampMin, ClampMax);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR ClampMax;
    XMVECTOR ClampMin;

    XMASSERT(((UINT_PTR)&LengthMin & 3) == 0);
    XMASSERT(((UINT_PTR)&LengthMax & 3) == 0);

    ClampMax = __lvlx(&LengthMax, 0);
    ClampMin = __lvlx(&LengthMin, 0);

    ClampMax = __vspltw(ClampMax, 0);
    ClampMin = __vspltw(ClampMin, 0);

    return XMVector4ClampLengthV(V, ClampMin, ClampMax);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4ClampLengthV
(
    FXMVECTOR V, 
    FXMVECTOR LengthMin, 
    FXMVECTOR LengthMax
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR RcpLength;
    XMVECTOR Length;
    XMVECTOR Normal;
    XMVECTOR Zero;
    XMVECTOR InfiniteLength;
    XMVECTOR ZeroLength;
    XMVECTOR Select;
    XMVECTOR ControlMax;
    XMVECTOR ControlMin;
    XMVECTOR Control;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[2] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[3] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[2] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[3] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector4GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector4GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector4GreaterOrEqual(LengthMax, LengthMin));

    LengthSq = XMVector4LengthSq(V);

    Zero = XMVectorZero();

    RcpLength = XMVectorReciprocalSqrt(LengthSq);

    InfiniteLength = XMVectorEqualInt(LengthSq, g_XMInfinity.v);
    ZeroLength = XMVectorEqual(LengthSq, Zero);

    Normal = XMVectorMultiply(V, RcpLength);

    Length = XMVectorMultiply(LengthSq, RcpLength);

    Select = XMVectorEqualInt(InfiniteLength, ZeroLength);
    Length = XMVectorSelect(LengthSq, Length, Select);
    Normal = XMVectorSelect(LengthSq, Normal, Select);

    ControlMax = XMVectorGreater(Length, LengthMax);
    ControlMin = XMVectorLess(Length, LengthMin);

    ClampLength = XMVectorSelect(Length, LengthMax, ControlMax);
    ClampLength = XMVectorSelect(ClampLength, LengthMin, ControlMin);

    Result = XMVectorMultiply(Normal, ClampLength);

    // Preserve the original vector (with no precision loss) if the length falls within the given range
    Control = XMVectorEqualInt(ControlMax, ControlMin);
    Result = XMVectorSelect(Result, V, Control);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR ClampLength;
    XMVECTOR LengthSq;
    XMVECTOR Length, LengthS;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR Normal, NormalS;
    XMVECTOR Zero;
    XMVECTOR Control;
    XMVECTOR H;
    XMVECTOR Result;

    XMASSERT((LengthMin.vector4_f32[1] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[2] == LengthMin.vector4_f32[0]) && (LengthMin.vector4_f32[3] == LengthMin.vector4_f32[0]));
    XMASSERT((LengthMax.vector4_f32[1] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[2] == LengthMax.vector4_f32[0]) && (LengthMax.vector4_f32[3] == LengthMax.vector4_f32[0]));
    XMASSERT(XMVector4GreaterOrEqual(LengthMin, XMVectorZero()));
    XMASSERT(XMVector4GreaterOrEqual(LengthMax, XMVectorZero()));
    XMASSERT(XMVector4GreaterOrEqual(LengthMax, LengthMin));

    H = __vspltisw(1);

    LengthSq = __vmsum4fp(V, V);

    H = __vcfsx(H, 1);

    Rsq = __vrsqrtefp(LengthSq);
    RT = __vmulfp(LengthSq, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    H = __vmaddcfp(Rsq, H, Rsq);

    Zero = __vspltisw(0);

    Length = __vmulfp(LengthSq, H);

    Normal = __vmulfp(V, H);

    LengthS = __vcmpeqfp(LengthSq, Zero);
    NormalS = __vcmpeqfp(LengthSq, Zero);

    LengthS = __vsel(Length, LengthSq, LengthS);
    NormalS = __vsel(Normal, LengthSq, NormalS);

    Control = __vcmpgtfp(LengthS, LengthMax);
    ClampLength = __vcmpgtfp(LengthMin, LengthS);

    Result = __vcmpequw(Control, ClampLength);

    Control = __vsel(LengthS, LengthMax, Control);
    ClampLength = __vsel(Control, LengthMin, ClampLength);

    RT = __vmulfp(NormalS, ClampLength);

    Result = __vsel(RT, V, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Reflect
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal
)
{
#if defined(_XM_NO_INTRINSICS_) 

    XMVECTOR Result;

    // Result = Incident - (2 * dot(Incident, Normal)) * Normal
    Result = XMVector4Dot(Incident, Normal);
    Result = XMVectorAdd(Result, Result);
    Result = XMVectorNegativeMultiplySubtract(Result, Normal, Incident);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D;
    XMVECTOR Result;

    D = __vmsum4fp(Incident, Normal);
    D = __vaddfp(D, D);
    Result = Incident;
    Result = __vnmsubfp(D, Normal, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Refract
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FLOAT    RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Index;
    Index = XMVectorReplicate(RefractionIndex);
    return XMVector4RefractV(Incident, Normal, Index);

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR Index;

    Index.vector4_f32[0] =
    Index.vector4_f32[1] = 
    Index.vector4_f32[2] =
    Index.vector4_f32[3] = RefractionIndex;

    return XMVector4RefractV(Incident, Normal, Index);

#else // _XM_VMX128_INTRINSICS_

    XMVECTOR Index;

    XMASSERT(((UINT_PTR)&RefractionIndex & 3) == 0);

    Index = __lvlx(&RefractionIndex, 0);
    Index = __vspltw(Index, 0);

    return XMVector4RefractV(Incident, Normal, Index);
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4RefractV
(
    FXMVECTOR Incident, 
    FXMVECTOR Normal, 
    FXMVECTOR RefractionIndex
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR        IDotN;
    XMVECTOR        R;
    CONST XMVECTOR  Zero = XMVectorZero();

    // Result = RefractionIndex * Incident - Normal * (RefractionIndex * dot(Incident, Normal) + 
    // sqrt(1 - RefractionIndex * RefractionIndex * (1 - dot(Incident, Normal) * dot(Incident, Normal))))

    IDotN = XMVector4Dot(Incident, Normal);

    // R = 1.0f - RefractionIndex * RefractionIndex * (1.0f - IDotN * IDotN)
    R = XMVectorNegativeMultiplySubtract(IDotN, IDotN, g_XMOne.v);
    R = XMVectorMultiply(R, RefractionIndex);
    R = XMVectorNegativeMultiplySubtract(R, RefractionIndex, g_XMOne.v);

    if (XMVector4LessOrEqual(R, Zero))
    {
        // Total internal reflection
        return Zero;
    }
    else
    {
        XMVECTOR Result;

        // R = RefractionIndex * IDotN + sqrt(R)
        R = XMVectorSqrt(R);
        R = XMVectorMultiplyAdd(RefractionIndex, IDotN, R);

        // Result = RefractionIndex * Incident - Normal * R
        Result = XMVectorMultiply(RefractionIndex, Incident);
        Result = XMVectorNegativeMultiplySubtract(Normal, R, Result);

        return Result;
    }

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR IDotN;
    XMVECTOR R;
    XMVECTOR I;
    XMVECTOR Rsq;
    XMVECTOR Rcp;
    XMVECTOR RT;
    XMVECTOR ZL;
    XMVECTOR D1;
    XMVECTOR H;
    XMVECTOR Zero;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(R);

    IDotN = __vmsum4fp(Incident, Normal);

    H = __vspltisw(1);
    R = __vupkd3d(R, VPACK_NORMSHORT2);
    H = __vcfsx(H, 1);
    RT = __vspltw(R, 3);
    R = __vspltw(R, 3);

    RT = __vnmsubfp(IDotN, IDotN, RT);
    RT = __vmulfp(RT, RefractionIndex);
    R = __vnmsubfp(RT, RefractionIndex, R);

    Rsq = __vrsqrtefp(R);
    RT = __vmulfp(R, H);
    Rcp = __vmulfp(Rsq, Rsq);
    H = __vnmsubfp(RT, Rcp, H);
    Zero = __vspltisw(0);
    H = __vmaddcfp(Rsq, H, Rsq);

    ZL = __vcmpeqfp(R, Zero);

    Result = __vcmpgefp(Zero, R);

    D1 = __vmulfp(R, H);

    I = __vmulfp(RefractionIndex, Incident);

    ZL = __vsel(D1, R, ZL);

    ZL = __vmaddfp(RefractionIndex, IDotN, ZL);

    I = __vnmsubfp(Normal, ZL, I);

    Result = __vsel(I, Zero, Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Orthogonal
(
    FXMVECTOR V
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR Result;
    Result.vector4_f32[0] = V.vector4_f32[2];
    Result.vector4_f32[1] = V.vector4_f32[3];
    Result.vector4_f32[2] = -V.vector4_f32[0];
    Result.vector4_f32[3] = -V.vector4_f32[1];
    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR NegativeV;
    XMVECTOR SignMask;
    XMVECTOR Result;

    SignMask = __vspltisw(-1);
    SignMask = __vslw(SignMask, SignMask);

    NegativeV = __vxor(V, SignMask);

    Result = __vsldoi(V, NegativeV, 2 << 2);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4AngleBetweenNormalsEst
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    Result = XMVector4Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACosEst(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    CosAngle = __vmsum4fp(N1, N2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    CosAngle = __vmaxfp(NegativeOne, CosAngle);
    CosAngle = __vminfp(One, CosAngle);
    Result = XMVectorACosEst(CosAngle);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4AngleBetweenNormals
(
    FXMVECTOR N1, 
    FXMVECTOR N2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    Result = XMVector4Dot(N1, N2);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    Result = XMVectorClamp(Result, NegativeOne, One);
    Result = XMVectorACos(Result);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    Result = __vmsum4fp(N1, N2);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    Result = __vmaxfp(NegativeOne, Result);
    Result = __vminfp(One, Result);
    Result = XMVectorACos(Result);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4AngleBetweenVectors
(
    FXMVECTOR V1, 
    FXMVECTOR V2
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR L1;
    XMVECTOR L2;
    XMVECTOR Dot;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    L1 = XMVector4ReciprocalLength(V1);
    L2 = XMVector4ReciprocalLength(V2);

    Dot = XMVector4Dot(V1, V2);

    L1 = XMVectorMultiply(L1, L2);

    CosAngle = XMVectorMultiply(Dot, L1);
    NegativeOne = XMVectorSplatConstant(-1, 0);
    One = XMVectorSplatOne();
    CosAngle = XMVectorClamp(CosAngle, NegativeOne, One);

    Result = XMVectorACos(CosAngle);

    return Result;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMVECTOR D1, D2;
    XMVECTOR Rsq1, Rsq2;
    XMVECTOR Rcp1, Rcp2;
    XMVECTOR RT1, RT2;
    XMVECTOR H1, H2;
    XMVECTOR L;
    XMVECTOR Dot;
    XMVECTOR CosAngle;
    XMVECTOR NegativeOne;
    XMVECTOR One;
    XMVECTOR Result;

    XMDUMMY_INITIALIZE_VECTOR(One);

    D1 = __vmsum4fp(V1, V1);
    D2 = __vmsum4fp(V2, V2);
    H2 = __vspltisw(1);
    H1 = __vcfsx(H2, 1);
    H2 = __vcfsx(H2, 1);
    Rsq1 = __vrsqrtefp(D1);
    Rsq2 = __vrsqrtefp(D2);
    RT1 = __vmulfp(D1, H1);
    RT2 = __vmulfp(D2, H2);
    Rcp1 = __vmulfp(Rsq1, Rsq1);
    Rcp2 = __vmulfp(Rsq2, Rsq2);
    H1 = __vnmsubfp(RT1, Rcp1, H1);
    H2 = __vnmsubfp(RT2, Rcp2, H2);
    Rsq1 = __vmaddfp(Rsq1, H1, Rsq1);
    Rsq2 = __vmaddfp(Rsq2, H2, Rsq2);
    Dot = __vmsum4fp(V1, V2);
    L = __vmulfp(Rsq1, Rsq2);
    CosAngle = __vmulfp(Dot, L);
    NegativeOne = __vspltisw(-1);
    One = __vupkd3d(One, VPACK_NORMSHORT2);
    NegativeOne = __vcfsx(NegativeOne, 0);
    One = __vspltw(One, 3);
    CosAngle = __vmaxfp(NegativeOne, CosAngle);
    CosAngle = __vminfp(One, CosAngle);

    Result = XMVectorACos(CosAngle);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR XMVector4Transform
(
    FXMVECTOR V, 
    CXMMATRIX M
)
{
#if defined(_XM_NO_INTRINSICS_)
    FLOAT fX = (M.m[0][0]*V.vector4_f32[0])+(M.m[1][0]*V.vector4_f32[1])+(M.m[2][0]*V.vector4_f32[2])+(M.m[3][0]*V.vector4_f32[3]);
    FLOAT fY = (M.m[0][1]*V.vector4_f32[0])+(M.m[1][1]*V.vector4_f32[1])+(M.m[2][1]*V.vector4_f32[2])+(M.m[3][1]*V.vector4_f32[3]);
    FLOAT fZ = (M.m[0][2]*V.vector4_f32[0])+(M.m[1][2]*V.vector4_f32[1])+(M.m[2][2]*V.vector4_f32[2])+(M.m[3][2]*V.vector4_f32[3]);
    FLOAT fW = (M.m[0][3]*V.vector4_f32[0])+(M.m[1][3]*V.vector4_f32[1])+(M.m[2][3]*V.vector4_f32[2])+(M.m[3][3]*V.vector4_f32[3]);
    XMVECTOR vResult = {
        fX,
        fY,
        fZ,
        fW
    };
    return vResult;

#elif defined(_XM_SSE_INTRINSICS_)
#else // _XM_VMX128_INTRINSICS_
    XMMATRIX P, MT;
    XMVECTOR R0, R1, R2, R3, R0R2, R1R3;
    XMVECTOR Result;

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    R0 = __vmsum4fp(V, MT.r[0]);
    R1 = __vmsum4fp(V, MT.r[1]);
    R2 = __vmsum4fp(V, MT.r[2]);
    R3 = __vmsum4fp(V, MT.r[3]);

    R0R2 = __vmrghw(R0, R2);
    R1R3 = __vmrghw(R1, R3);

    Result = __vmrghw(R0R2, R1R3);

    return Result;
#endif // _XM_VMX128_INTRINSICS_
}

//------------------------------------------------------------------------------

XMINLINE XMFLOAT4* XMVector4TransformStream
(
    XMFLOAT4*       pOutputStream, 
    size_t          OutputStride, 
    CONST XMFLOAT4* pInputStream, 
    size_t          InputStride, 
    size_t          VectorCount, 
    CXMMATRIX     M
)
{
#if defined(_XM_NO_INTRINSICS_)

    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR W;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat4((const XMFLOAT4*)pInputVector);
        W = XMVectorSplatW(V);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        W = XMVectorReplicate(((XMFLOAT4*)pInputVector)->w);
//        Z = XMVectorReplicate(((XMFLOAT4*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT4*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT4*)pInputVector)->x);

        Result = XMVectorMultiply(W, M.r[3]);
        Result = XMVectorMultiplyAdd(Z, M.r[2], Result);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat4((XMFLOAT4*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#elif defined(_XM_SSE_INTRINSICS_)
#elif defined(XM_NO_MISALIGNED_VECTOR_ACCESS)
    XMVECTOR V;
    XMVECTOR X;
    XMVECTOR Y;
    XMVECTOR Z;
    XMVECTOR W;
    XMVECTOR Result;
    size_t   i;
    CONST BYTE* pInputVector = (CONST BYTE*)pInputStream;
    BYTE*    pOutputVector = (BYTE*)pOutputStream;

    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);

    for (i = 0; i < VectorCount; i++)
    {
        V = XMLoadFloat4((const XMFLOAT4*)pInputVector);
        W = XMVectorSplatW(V);
        Z = XMVectorSplatZ(V);
        Y = XMVectorSplatY(V);
        X = XMVectorSplatX(V);
//        W = XMVectorReplicate(((XMFLOAT4*)pInputVector)->w);
//        Z = XMVectorReplicate(((XMFLOAT4*)pInputVector)->z);
//        Y = XMVectorReplicate(((XMFLOAT4*)pInputVector)->y);
//        X = XMVectorReplicate(((XMFLOAT4*)pInputVector)->x);

        Result = XMVectorMultiply(W, M.r[3]);
        Result = XMVectorMultiplyAdd(Z, M.r[2], Result);
        Result = XMVectorMultiplyAdd(Y, M.r[1], Result);
        Result = XMVectorMultiplyAdd(X, M.r[0], Result);

        XMStoreFloat4NC((XMFLOAT4*)pOutputVector, Result);

        pInputVector += InputStride; 
        pOutputVector += OutputStride;
    }

    return pOutputStream;

#else // _XM_VMX128_INTRINSICS_

    CONST UINT UnrollCount = 8;
    XMMATRIX P, MT;
    XMVECTOR R0[8], R1[8], R2[8], R3[8];
    UINT_PTR InputVector = (UINT_PTR)pInputStream;
    UINT_PTR OutputVector = (UINT_PTR)pOutputStream;

//    C_ASSERT(UnrollCount == 8);
//    C_ASSERT(sizeof(XMFLOAT4) == 16);
    XMASSERT(pOutputStream);
    XMASSERT(pInputStream);
    XMASSERT(((UINT_PTR)pInputStream & 3) == 0);
    XMASSERT(((UINT_PTR)pOutputStream & 3) == 0);

    P.r[0] = __vmrghw(M.r[0], M.r[2]);
    P.r[1] = __vmrghw(M.r[1], M.r[3]);
    P.r[2] = __vmrglw(M.r[0], M.r[2]);
    P.r[3] = __vmrglw(M.r[1], M.r[3]);

    MT.r[0] = __vmrghw(P.r[0], P.r[1]);
    MT.r[1] = __vmrglw(P.r[0], P.r[1]);
    MT.r[2] = __vmrghw(P.r[2], P.r[3]);
    MT.r[3] = __vmrglw(P.r[2], P.r[3]);

    if (((InputVector & (sizeof(XMFLOAT4) - 1)) == 0) &&
        (InputStride == sizeof(XMFLOAT4)))
    {
        UINT GroupIndex;
        UINT_PTR InputStreamEnd = InputVector + VectorCount * sizeof(XMFLOAT4);
        UINT_PTR InputStreamCGroupBase = XMMin(InputVector + (XM_CACHE_LINE_SIZE - 1), InputStreamEnd) & ~(XM_CACHE_LINE_SIZE - 1);
        UINT_PTR InputStreamCGroupEnd = InputStreamCGroupBase + ((InputStreamEnd - InputStreamCGroupBase) & ~(4 * XM_CACHE_LINE_SIZE - 1));

        __dcbt(0,                      (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE,     (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 2, (VOID*)InputStreamCGroupBase);
        __dcbt(XM_CACHE_LINE_SIZE * 3, (VOID*)InputStreamCGroupBase);

        while (InputVector < InputStreamCGroupBase)
        {
            R3[0] = __lvx((VOID*)InputVector, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT4); 
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamCGroupEnd)
        {
            __dcbt(XM_CACHE_LINE_SIZE * 4, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 5, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 6, (VOID*)InputVector);
            __dcbt(XM_CACHE_LINE_SIZE * 7, (VOID*)InputVector);

            for (GroupIndex = 0; GroupIndex < 4; GroupIndex++)
            {
                R3[0] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[1] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[2] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[3] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[4] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[5] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[6] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);
                R3[7] = __lvx((VOID*)InputVector, 0);
                InputVector += sizeof(XMFLOAT4);

                R0[0] = __vmsum4fp(R3[0], MT.r[0]);
                R1[0] = __vmsum4fp(R3[0], MT.r[1]);
                R2[0] = __vmsum4fp(R3[0], MT.r[2]);
                R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                      
                R0[1] = __vmsum4fp(R3[1], MT.r[0]);
                R1[1] = __vmsum4fp(R3[1], MT.r[1]);
                R2[1] = __vmsum4fp(R3[1], MT.r[2]);
                R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                      
                R0[2] = __vmsum4fp(R3[2], MT.r[0]);
                R1[2] = __vmsum4fp(R3[2], MT.r[1]);
                R2[2] = __vmsum4fp(R3[2], MT.r[2]);
                R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                      
                R0[3] = __vmsum4fp(R3[3], MT.r[0]);
                R1[3] = __vmsum4fp(R3[3], MT.r[1]);
                R2[3] = __vmsum4fp(R3[3], MT.r[2]);
                R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                      
                R0[4] = __vmsum4fp(R3[4], MT.r[0]);
                R1[4] = __vmsum4fp(R3[4], MT.r[1]);
                R2[4] = __vmsum4fp(R3[4], MT.r[2]);
                R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                      
                R0[5] = __vmsum4fp(R3[5], MT.r[0]);
                R1[5] = __vmsum4fp(R3[5], MT.r[1]);
                R2[5] = __vmsum4fp(R3[5], MT.r[2]);
                R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                      
                R0[6] = __vmsum4fp(R3[6], MT.r[0]);
                R1[6] = __vmsum4fp(R3[6], MT.r[1]);
                R2[6] = __vmsum4fp(R3[6], MT.r[2]);
                R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                      
                R0[7] = __vmsum4fp(R3[7], MT.r[0]);
                R1[7] = __vmsum4fp(R3[7], MT.r[1]);
                R2[7] = __vmsum4fp(R3[7], MT.r[2]);
                R3[7] = __vmsum4fp(R3[7], MT.r[3]);

                __stvewx(R0[0], (VOID*)OutputVector, 0);
                __stvewx(R1[0], (VOID*)OutputVector, 4);
                __stvewx(R2[0], (VOID*)OutputVector, 8);
                __stvewx(R3[0], (VOID*)OutputVector, 12);
                 OutputVector += OutputStride;
                            
                __stvewx(R0[1], (VOID*)OutputVector, 0);
                __stvewx(R1[1], (VOID*)OutputVector, 4);
                __stvewx(R2[1], (VOID*)OutputVector, 8);
                __stvewx(R3[1], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[2], (VOID*)OutputVector, 0);
                __stvewx(R1[2], (VOID*)OutputVector, 4);
                __stvewx(R2[2], (VOID*)OutputVector, 8);
                __stvewx(R3[2], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[3], (VOID*)OutputVector, 0);
                __stvewx(R1[3], (VOID*)OutputVector, 4);
                __stvewx(R2[3], (VOID*)OutputVector, 8);
                __stvewx(R3[3], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[4], (VOID*)OutputVector, 0);
                __stvewx(R1[4], (VOID*)OutputVector, 4);
                __stvewx(R2[4], (VOID*)OutputVector, 8);
                __stvewx(R3[4], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[5], (VOID*)OutputVector, 0);
                __stvewx(R1[5], (VOID*)OutputVector, 4);
                __stvewx(R2[5], (VOID*)OutputVector, 8);
                __stvewx(R3[5], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[6], (VOID*)OutputVector, 0);
                __stvewx(R1[6], (VOID*)OutputVector, 4);
                __stvewx(R2[6], (VOID*)OutputVector, 8);
                __stvewx(R3[6], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
                             
                __stvewx(R0[7], (VOID*)OutputVector, 0);
                __stvewx(R1[7], (VOID*)OutputVector, 4);
                __stvewx(R2[7], (VOID*)OutputVector, 8);
                __stvewx(R3[7], (VOID*)OutputVector, 12);
                OutputVector += OutputStride;
            }
        }

        while (InputVector < InputStreamEnd)
        {
            R3[0] = __lvx((VOID*)InputVector, 0);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += sizeof(XMFLOAT4); 
            OutputVector += OutputStride;
        }
    }
    else
    {
        UINT_PTR InputStreamEnd = InputVector + VectorCount * InputStride;
        UINT_PTR InputStreamVGroupEnd = InputVector + (VectorCount & ~(UnrollCount - 1)) * InputStride;

        while (InputVector < InputStreamVGroupEnd)
        {
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[1] = __lvlx((VOID*)InputVector, 0);
            R1[1] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[2] = __lvlx((VOID*)InputVector, 0);
            R1[2] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[3] = __lvlx((VOID*)InputVector, 0);
            R1[3] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[4] = __lvlx((VOID*)InputVector, 0);
            R1[4] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[5] = __lvlx((VOID*)InputVector, 0);
            R1[5] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[6] = __lvlx((VOID*)InputVector, 0);
            R1[6] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;
            R0[7] = __lvlx((VOID*)InputVector, 0);
            R1[7] = __lvrx((VOID*)InputVector, 16);
            InputVector += InputStride;

            R3[0] = __vor(R0[0], R1[0]);
            R3[1] = __vor(R0[1], R1[1]);
            R3[2] = __vor(R0[2], R1[2]);
            R3[3] = __vor(R0[3], R1[3]);
            R3[4] = __vor(R0[4], R1[4]);
            R3[5] = __vor(R0[5], R1[5]);
            R3[6] = __vor(R0[6], R1[6]);
            R3[7] = __vor(R0[7], R1[7]);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);
                                  
            R0[1] = __vmsum4fp(R3[1], MT.r[0]);
            R1[1] = __vmsum4fp(R3[1], MT.r[1]);
            R2[1] = __vmsum4fp(R3[1], MT.r[2]);
            R3[1] = __vmsum4fp(R3[1], MT.r[3]);
                                  
            R0[2] = __vmsum4fp(R3[2], MT.r[0]);
            R1[2] = __vmsum4fp(R3[2], MT.r[1]);
            R2[2] = __vmsum4fp(R3[2], MT.r[2]);
            R3[2] = __vmsum4fp(R3[2], MT.r[3]);
                                  
            R0[3] = __vmsum4fp(R3[3], MT.r[0]);
            R1[3] = __vmsum4fp(R3[3], MT.r[1]);
            R2[3] = __vmsum4fp(R3[3], MT.r[2]);
            R3[3] = __vmsum4fp(R3[3], MT.r[3]);
                                  
            R0[4] = __vmsum4fp(R3[4], MT.r[0]);
            R1[4] = __vmsum4fp(R3[4], MT.r[1]);
            R2[4] = __vmsum4fp(R3[4], MT.r[2]);
            R3[4] = __vmsum4fp(R3[4], MT.r[3]);
                                  
            R0[5] = __vmsum4fp(R3[5], MT.r[0]);
            R1[5] = __vmsum4fp(R3[5], MT.r[1]);
            R2[5] = __vmsum4fp(R3[5], MT.r[2]);
            R3[5] = __vmsum4fp(R3[5], MT.r[3]);
                                  
            R0[6] = __vmsum4fp(R3[6], MT.r[0]);
            R1[6] = __vmsum4fp(R3[6], MT.r[1]);
            R2[6] = __vmsum4fp(R3[6], MT.r[2]);
            R3[6] = __vmsum4fp(R3[6], MT.r[3]);
                                  
            R0[7] = __vmsum4fp(R3[7], MT.r[0]);
            R1[7] = __vmsum4fp(R3[7], MT.r[1]);
            R2[7] = __vmsum4fp(R3[7], MT.r[2]);
            R3[7] = __vmsum4fp(R3[7], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[1], (VOID*)OutputVector, 0);
            __stvewx(R1[1], (VOID*)OutputVector, 4);
            __stvewx(R2[1], (VOID*)OutputVector, 8);
            __stvewx(R3[1], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[2], (VOID*)OutputVector, 0);
            __stvewx(R1[2], (VOID*)OutputVector, 4);
            __stvewx(R2[2], (VOID*)OutputVector, 8);
            __stvewx(R3[2], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[3], (VOID*)OutputVector, 0);
            __stvewx(R1[3], (VOID*)OutputVector, 4);
            __stvewx(R2[3], (VOID*)OutputVector, 8);
            __stvewx(R3[3], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[4], (VOID*)OutputVector, 0);
            __stvewx(R1[4], (VOID*)OutputVector, 4);
            __stvewx(R2[4], (VOID*)OutputVector, 8);
            __stvewx(R3[4], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[5], (VOID*)OutputVector, 0);
            __stvewx(R1[5], (VOID*)OutputVector, 4);
            __stvewx(R2[5], (VOID*)OutputVector, 8);
            __stvewx(R3[5], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[6], (VOID*)OutputVector, 0);
            __stvewx(R1[6], (VOID*)OutputVector, 4);
            __stvewx(R2[6], (VOID*)OutputVector, 8);
            __stvewx(R3[6], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
            __stvewx(R0[7], (VOID*)OutputVector, 0);
            __stvewx(R1[7], (VOID*)OutputVector, 4);
            __stvewx(R2[7], (VOID*)OutputVector, 8);
            __stvewx(R3[7], (VOID*)OutputVector, 12);
            OutputVector += OutputStride;
        }

        while (InputVector < InputStreamEnd)
        {
            R0[0] = __lvlx((VOID*)InputVector, 0);
            R1[0] = __lvrx((VOID*)InputVector, 16);
            R3[0] = __vor(R0[0], R1[0]);

            R0[0] = __vmsum4fp(R3[0], MT.r[0]);
            R1[0] = __vmsum4fp(R3[0], MT.r[1]);
            R2[0] = __vmsum4fp(R3[0], MT.r[2]);
            R3[0] = __vmsum4fp(R3[0], MT.r[3]);

            __stvewx(R0[0], (VOID*)OutputVector, 0);
            __stvewx(R1[0], (VOID*)OutputVector, 4);
            __stvewx(R2[0], (VOID*)OutputVector, 8);
            __stvewx(R3[0], (VOID*)OutputVector, 12);

            InputVector += InputStride; 
            OutputVector += OutputStride;
        }
    }

    return pOutputStream;
#endif // _XM_VMX128_INTRINSICS_
}

#ifdef __cplusplus

/****************************************************************************
 *
 * XMVECTOR operators
 *
 ****************************************************************************/

#ifndef XM_NO_OPERATOR_OVERLOADS

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator+ (FXMVECTOR V)
{
    return V;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator- (FXMVECTOR V)
{
    return XMVectorNegate(V);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator+=
(
    XMVECTOR&       V1,
    FXMVECTOR       V2
)
{
    V1 = XMVectorAdd(V1, V2);
    return V1;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator-=
(
    XMVECTOR&       V1,
    FXMVECTOR       V2
)
{
    V1 = XMVectorSubtract(V1, V2);
    return V1;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator*=
(
    XMVECTOR&       V1,
    FXMVECTOR       V2
)
{
    V1 = XMVectorMultiply(V1, V2);
    return V1;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator/=
(
    XMVECTOR&       V1,
    FXMVECTOR       V2
)
{
    V1 = XMVectorDivide(V1,V2);
    return V1;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator*=
(
    XMVECTOR&   V,
    CONST FLOAT S
)
{
    V = XMVectorScale(V, S);
    return V;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR& operator/=
(
    XMVECTOR&   V,
    CONST FLOAT S
)
{
    V = XMVectorScale(V, 1.0f / S);
    return V;
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator+
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
    return XMVectorAdd(V1, V2);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator-
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
    return XMVectorSubtract(V1, V2);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator*
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
    return XMVectorMultiply(V1, V2);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator/
(
    FXMVECTOR V1,
    FXMVECTOR V2
)
{
    return XMVectorDivide(V1,V2);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator*
(
    FXMVECTOR      V,
    CONST FLOAT    S
)
{
    return XMVectorScale(V, S);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator/
(
    FXMVECTOR      V,
    CONST FLOAT    S
)
{
    return XMVectorScale(V, 1.0f / S);
}

//------------------------------------------------------------------------------

XMFINLINE XMVECTOR operator*
(
    FLOAT           S,
    FXMVECTOR  	    V
)
{
    return XMVectorScale(V, S);
}

#endif // !XM_NO_OPERATOR_OVERLOADS

/****************************************************************************
 *
 * XMFLOAT2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT2::_XMFLOAT2
(
    CONST FLOAT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT2& _XMFLOAT2::operator=
(
    CONST _XMFLOAT2& Float2
)
{
    x = Float2.x;
    y = Float2.y;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE XMFLOAT2A& XMFLOAT2A::operator=
(
    CONST XMFLOAT2A& Float2
)
{
    x = Float2.x;
    y = Float2.y;
    return *this;
}

/****************************************************************************
 *
 * XMINT2 operators
 *
 ****************************************************************************/

XMFINLINE _XMINT2::_XMINT2
(
    CONST INT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE XMINT2& _XMINT2::operator=
(
    CONST _XMINT2& Int2
)
{
    x = Int2.x;
    y = Int2.y;
    return *this;
}

/****************************************************************************
 *
 * XMUINT2 operators
 *
 ****************************************************************************/

XMFINLINE _XMUINT2::_XMUINT2
(
    CONST UINT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE XMUINT2& _XMUINT2::operator=
(
    CONST _XMUINT2& UInt2
)
{
    x = UInt2.x;
    y = UInt2.y;
    return *this;
}

/****************************************************************************
 *
 * XMHALF2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMHALF2::_XMHALF2
(
    CONST HALF* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF2::_XMHALF2
(
    FLOAT _x,
    FLOAT _y
)
{
    x = XMConvertFloatToHalf(_x);
    y = XMConvertFloatToHalf(_y);
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF2::_XMHALF2
(
    CONST FLOAT* pArray
)
{
    x = XMConvertFloatToHalf(pArray[0]);
    y = XMConvertFloatToHalf(pArray[1]);
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF2& _XMHALF2::operator=
(
    CONST _XMHALF2& Half2
)
{
    x = Half2.x;
    y = Half2.y;
    return *this;
}

/****************************************************************************
 *
 * XMSHORTN2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN2::_XMSHORTN2
(
    CONST SHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN2::_XMSHORTN2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreShortN2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN2::_XMSHORTN2
(
    CONST FLOAT* pArray
)
{
    XMStoreShortN2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN2& _XMSHORTN2::operator=
(
    CONST _XMSHORTN2& ShortN2
)
{
    x = ShortN2.x;
    y = ShortN2.y;
    return *this;
}

/****************************************************************************
 *
 * XMSHORT2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT2::_XMSHORT2
(
    CONST SHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT2::_XMSHORT2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreShort2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT2::_XMSHORT2
(
    CONST FLOAT* pArray
)
{
    XMStoreShort2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT2& _XMSHORT2::operator=
(
    CONST _XMSHORT2& Short2
)
{
    x = Short2.x;
    y = Short2.y;
    return *this;
}

/****************************************************************************
 *
 * XMUSHORTN2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN2::_XMUSHORTN2
(
    CONST USHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN2::_XMUSHORTN2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreUShortN2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN2::_XMUSHORTN2
(
    CONST FLOAT* pArray
)
{
    XMStoreUShortN2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN2& _XMUSHORTN2::operator=
(
    CONST _XMUSHORTN2& UShortN2
)
{
    x = UShortN2.x;
    y = UShortN2.y;
    return *this;
}

/****************************************************************************
 *
 * XMUSHORT2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT2::_XMUSHORT2
(
    CONST USHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT2::_XMUSHORT2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreUShort2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT2::_XMUSHORT2
(
    CONST FLOAT* pArray
)
{
    XMStoreUShort2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT2& _XMUSHORT2::operator=
(
    CONST _XMUSHORT2& UShort2
)
{
    x = UShort2.x;
    y = UShort2.y;
    return *this;
}

/****************************************************************************
 *
 * XMBYTEN2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN2::_XMBYTEN2
(
    CONST CHAR* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN2::_XMBYTEN2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreByteN2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN2::_XMBYTEN2
(
    CONST FLOAT* pArray
)
{
    XMStoreByteN2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN2& _XMBYTEN2::operator=
(
    CONST _XMBYTEN2& ByteN2
)
{
    x = ByteN2.x;
    y = ByteN2.y;
    return *this;
}

/****************************************************************************
 *
 * XMBYTE2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE2::_XMBYTE2
(
    CONST CHAR* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE2::_XMBYTE2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreByte2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE2::_XMBYTE2
(
    CONST FLOAT* pArray
)
{
    XMStoreByte2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE2& _XMBYTE2::operator=
(
    CONST _XMBYTE2& Byte2
)
{
    x = Byte2.x;
    y = Byte2.y;
    return *this;
}

/****************************************************************************
 *
 * XMUBYTEN2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN2::_XMUBYTEN2
(
    CONST BYTE* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN2::_XMUBYTEN2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreUByteN2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN2::_XMUBYTEN2
(
    CONST FLOAT* pArray
)
{
    XMStoreUByteN2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN2& _XMUBYTEN2::operator=
(
    CONST _XMUBYTEN2& UByteN2
)
{
    x = UByteN2.x;
    y = UByteN2.y;
    return *this;
}

/****************************************************************************
 *
 * XMUBYTE2 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE2::_XMUBYTE2
(
    CONST BYTE* pArray
)
{
    x = pArray[0];
    y = pArray[1];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE2::_XMUBYTE2
(
    FLOAT _x,
    FLOAT _y
)
{
    XMStoreUByte2(this, XMVectorSet(_x, _y, 0.0f, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE2::_XMUBYTE2
(
    CONST FLOAT* pArray
)
{
    XMStoreUByte2(this, XMLoadFloat2((const XMFLOAT2*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE2& _XMUBYTE2::operator=
(
    CONST _XMUBYTE2& UByte2
)
{
    x = UByte2.x;
    y = UByte2.y;
    return *this;
}

/****************************************************************************
 *
 * XMFLOAT3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT3::_XMFLOAT3
(
    CONST FLOAT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
}

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT3& _XMFLOAT3::operator=
(
    CONST _XMFLOAT3& Float3
)
{
    x = Float3.x;
    y = Float3.y;
    z = Float3.z;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE XMFLOAT3A& XMFLOAT3A::operator=
(
    CONST XMFLOAT3A& Float3
)
{
    x = Float3.x;
    y = Float3.y;
    z = Float3.z;
    return *this;
}

/****************************************************************************
 *
 * XMINT3 operators
 *
 ****************************************************************************/

XMFINLINE _XMINT3::_XMINT3
(
    CONST INT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
}

//------------------------------------------------------------------------------

XMFINLINE XMINT3& _XMINT3::operator=
(
    CONST _XMINT3& Int3
)
{
    x = Int3.x;
    y = Int3.y;
    z = Int3.z;
    return *this;
}

/****************************************************************************
 *
 * XMUINT3 operators
 *
 ****************************************************************************/

XMFINLINE _XMUINT3::_XMUINT3
(
    CONST UINT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
}

//------------------------------------------------------------------------------

XMFINLINE XMUINT3& _XMUINT3::operator=
(
    CONST _XMUINT3& UInt3
)
{
    x = UInt3.x;
    y = UInt3.y;
    z = UInt3.z;
    return *this;
}

/****************************************************************************
 *
 * XMHENDN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMHENDN3::_XMHENDN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreHenDN3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMHENDN3::_XMHENDN3
(
    CONST FLOAT* pArray
)
{
    XMStoreHenDN3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMHENDN3& _XMHENDN3::operator=
(
    CONST _XMHENDN3& HenDN3
)
{
    v = HenDN3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMHENDN3& _XMHENDN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMHEND3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMHEND3::_XMHEND3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreHenD3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMHEND3::_XMHEND3
(
    CONST FLOAT* pArray
)
{
    XMStoreHenD3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMHEND3& _XMHEND3::operator=
(
    CONST _XMHEND3& HenD3
)
{
    v = HenD3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMHEND3& _XMHEND3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUHENDN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUHENDN3::_XMUHENDN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreUHenDN3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHENDN3::_XMUHENDN3
(
    CONST FLOAT* pArray
)
{
    XMStoreUHenDN3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHENDN3& _XMUHENDN3::operator=
(
    CONST _XMUHENDN3& UHenDN3
)
{
    v = UHenDN3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHENDN3& _XMUHENDN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUHEND3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUHEND3::_XMUHEND3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreUHenD3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHEND3::_XMUHEND3
(
    CONST FLOAT* pArray
)
{
    XMStoreUHenD3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHEND3& _XMUHEND3::operator=
(
    CONST _XMUHEND3& UHenD3
)
{
    v = UHenD3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUHEND3& _XMUHEND3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMDHENN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMDHENN3::_XMDHENN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreDHenN3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHENN3::_XMDHENN3
(
    CONST FLOAT* pArray
)
{
    XMStoreDHenN3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHENN3& _XMDHENN3::operator=
(
    CONST _XMDHENN3& DHenN3
)
{
    v = DHenN3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHENN3& _XMDHENN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMDHEN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMDHEN3::_XMDHEN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreDHen3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHEN3::_XMDHEN3
(
    CONST FLOAT* pArray
)
{
    XMStoreDHen3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHEN3& _XMDHEN3::operator=
(
    CONST _XMDHEN3& DHen3
)
{
    v = DHen3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMDHEN3& _XMDHEN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUDHENN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUDHENN3::_XMUDHENN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreUDHenN3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHENN3::_XMUDHENN3
(
    CONST FLOAT* pArray
)
{
    XMStoreUDHenN3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHENN3& _XMUDHENN3::operator=
(
    CONST _XMUDHENN3& UDHenN3
)
{
    v = UDHenN3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHENN3& _XMUDHENN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUDHEN3 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUDHEN3::_XMUDHEN3
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreUDHen3(this, XMVectorSet(_x, _y, _z, 0.0f));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHEN3::_XMUDHEN3
(
    CONST FLOAT* pArray
)
{
    XMStoreUDHen3(this, XMLoadFloat3((const XMFLOAT3*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHEN3& _XMUDHEN3::operator=
(
    CONST _XMUDHEN3& UDHen3
)
{
    v = UDHen3.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDHEN3& _XMUDHEN3::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMU565 operators
 *
 ****************************************************************************/

XMFINLINE _XMU565::_XMU565
(
    CONST CHAR *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
}

XMFINLINE _XMU565::_XMU565
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreU565(this, XMVectorSet( _x, _y, _z, 0.0f ));
}

XMFINLINE _XMU565::_XMU565
(
    CONST FLOAT *pArray
)
{
    XMStoreU565(this, XMLoadFloat3((const XMFLOAT3*)pArray ));
}

XMFINLINE _XMU565& _XMU565::operator=
(
    CONST _XMU565& U565
)
{
    v = U565.v;
    return *this;
}

XMFINLINE _XMU565& _XMU565::operator=
(
    CONST USHORT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMFLOAT3PK operators
 *
 ****************************************************************************/

XMFINLINE _XMFLOAT3PK::_XMFLOAT3PK
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreFloat3PK(this, XMVectorSet( _x, _y, _z, 0.0f ));
}

XMFINLINE _XMFLOAT3PK::_XMFLOAT3PK
(
    CONST FLOAT *pArray
)
{
    XMStoreFloat3PK(this, XMLoadFloat3((const XMFLOAT3*)pArray ));
}

XMFINLINE _XMFLOAT3PK& _XMFLOAT3PK::operator=
(
    CONST _XMFLOAT3PK& float3pk
)
{
    v = float3pk.v;
    return *this;
}

XMFINLINE _XMFLOAT3PK& _XMFLOAT3PK::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMFLOAT3SE operators
 *
 ****************************************************************************/

XMFINLINE _XMFLOAT3SE::_XMFLOAT3SE
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z
)
{
    XMStoreFloat3SE(this, XMVectorSet( _x, _y, _z, 0.0f ));
}

XMFINLINE _XMFLOAT3SE::_XMFLOAT3SE
(
    CONST FLOAT *pArray
)
{
    XMStoreFloat3SE(this, XMLoadFloat3((const XMFLOAT3*)pArray ));
}

XMFINLINE _XMFLOAT3SE& _XMFLOAT3SE::operator=
(
    CONST _XMFLOAT3SE& float3se
)
{
    v = float3se.v;
    return *this;
}

XMFINLINE _XMFLOAT3SE& _XMFLOAT3SE::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMFLOAT4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT4::_XMFLOAT4
(
    CONST FLOAT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMFLOAT4& _XMFLOAT4::operator=
(
    CONST _XMFLOAT4& Float4
)
{
    x = Float4.x;
    y = Float4.y;
    z = Float4.z;
    w = Float4.w;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE XMFLOAT4A& XMFLOAT4A::operator=
(
    CONST XMFLOAT4A& Float4
)
{
    x = Float4.x;
    y = Float4.y;
    z = Float4.z;
    w = Float4.w;
    return *this;
}

/****************************************************************************
 *
 * XMINT4 operators
 *
 ****************************************************************************/

XMFINLINE _XMINT4::_XMINT4
(
    CONST INT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE XMINT4& _XMINT4::operator=
(
    CONST _XMINT4& Int4
)
{
    x = Int4.x;
    y = Int4.y;
    z = Int4.z;
    w = Int4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUINT4 operators
 *
 ****************************************************************************/

XMFINLINE _XMUINT4::_XMUINT4
(
    CONST UINT *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE XMUINT4& _XMUINT4::operator=
(
    CONST _XMUINT4& UInt4
)
{
    x = UInt4.x;
    y = UInt4.y;
    z = UInt4.z;
    w = UInt4.w;
    return *this;
}

/****************************************************************************
 *
 * XMHALF4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMHALF4::_XMHALF4
(
    CONST HALF* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF4::_XMHALF4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    x = XMConvertFloatToHalf(_x);
    y = XMConvertFloatToHalf(_y);
    z = XMConvertFloatToHalf(_z);
    w = XMConvertFloatToHalf(_w);
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF4::_XMHALF4
(
    CONST FLOAT* pArray
)
{
    XMConvertFloatToHalfStream(&x, sizeof(HALF), pArray, sizeof(FLOAT), 4);
}

//------------------------------------------------------------------------------

XMFINLINE _XMHALF4& _XMHALF4::operator=
(
    CONST _XMHALF4& Half4
)
{
    x = Half4.x;
    y = Half4.y;
    z = Half4.z;
    w = Half4.w;
    return *this;
}

/****************************************************************************
 *
 * XMSHORTN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN4::_XMSHORTN4
(
    CONST SHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN4::_XMSHORTN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreShortN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN4::_XMSHORTN4
(
    CONST FLOAT* pArray
)
{
    XMStoreShortN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORTN4& _XMSHORTN4::operator=
(
    CONST _XMSHORTN4& ShortN4
)
{
    x = ShortN4.x;
    y = ShortN4.y;
    z = ShortN4.z;
    w = ShortN4.w;
    return *this;
}

/****************************************************************************
 *
 * XMSHORT4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT4::_XMSHORT4
(
    CONST SHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT4::_XMSHORT4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreShort4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT4::_XMSHORT4
(
    CONST FLOAT* pArray
)
{
    XMStoreShort4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMSHORT4& _XMSHORT4::operator=
(
    CONST _XMSHORT4& Short4
)
{
    x = Short4.x;
    y = Short4.y;
    z = Short4.z;
    w = Short4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUSHORTN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN4::_XMUSHORTN4
(
    CONST USHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN4::_XMUSHORTN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUShortN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN4::_XMUSHORTN4
(
    CONST FLOAT* pArray
)
{
    XMStoreUShortN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORTN4& _XMUSHORTN4::operator=
(
    CONST _XMUSHORTN4& UShortN4
)
{
    x = UShortN4.x;
    y = UShortN4.y;
    z = UShortN4.z;
    w = UShortN4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUSHORT4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT4::_XMUSHORT4
(
    CONST USHORT* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT4::_XMUSHORT4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUShort4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT4::_XMUSHORT4
(
    CONST FLOAT* pArray
)
{
    XMStoreUShort4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUSHORT4& _XMUSHORT4::operator=
(
    CONST _XMUSHORT4& UShort4
)
{
    x = UShort4.x;
    y = UShort4.y;
    z = UShort4.z;
    w = UShort4.w;
    return *this;
}

/****************************************************************************
 *
 * XMXDECN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMXDECN4::_XMXDECN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreXDecN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDECN4::_XMXDECN4
(
    CONST FLOAT* pArray
)
{
    XMStoreXDecN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDECN4& _XMXDECN4::operator=
(
    CONST _XMXDECN4& XDecN4
)
{
    v = XDecN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDECN4& _XMXDECN4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMXDEC4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMXDEC4::_XMXDEC4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreXDec4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDEC4::_XMXDEC4
(
    CONST FLOAT* pArray
)
{
    XMStoreXDec4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDEC4& _XMXDEC4::operator=
(
    CONST _XMXDEC4& XDec4
)
{
    v = XDec4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMXDEC4& _XMXDEC4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMDECN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMDECN4::_XMDECN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreDecN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDECN4::_XMDECN4
(
    CONST FLOAT* pArray
)
{
    XMStoreDecN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDECN4& _XMDECN4::operator=
(
    CONST _XMDECN4& DecN4
)
{
    v = DecN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMDECN4& _XMDECN4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMDEC4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMDEC4::_XMDEC4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreDec4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDEC4::_XMDEC4
(
    CONST FLOAT* pArray
)
{
    XMStoreDec4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMDEC4& _XMDEC4::operator=
(
    CONST _XMDEC4& Dec4
)
{
    v = Dec4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMDEC4& _XMDEC4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUDECN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUDECN4::_XMUDECN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUDecN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDECN4::_XMUDECN4
(
    CONST FLOAT* pArray
)
{
    XMStoreUDecN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDECN4& _XMUDECN4::operator=
(
    CONST _XMUDECN4& UDecN4
)
{
    v = UDecN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDECN4& _XMUDECN4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUDEC4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUDEC4::_XMUDEC4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUDec4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDEC4::_XMUDEC4
(
    CONST FLOAT* pArray
)
{
    XMStoreUDec4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDEC4& _XMUDEC4::operator=
(
    CONST _XMUDEC4& UDec4
)
{
    v = UDec4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUDEC4& _XMUDEC4::operator=
(
    CONST UINT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMXICON4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMXICON4::_XMXICON4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreXIcoN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICON4::_XMXICON4
(
    CONST FLOAT* pArray
)
{
    XMStoreXIcoN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICON4& _XMXICON4::operator=
(
    CONST _XMXICON4& XIcoN4
)
{
    v = XIcoN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICON4& _XMXICON4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMXICO4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMXICO4::_XMXICO4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreXIco4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICO4::_XMXICO4
(
    CONST FLOAT* pArray
)
{
    XMStoreXIco4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICO4& _XMXICO4::operator=
(
    CONST _XMXICO4& XIco4
)
{
    v = XIco4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMXICO4& _XMXICO4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMICON4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMICON4::_XMICON4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreIcoN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMICON4::_XMICON4
(
    CONST FLOAT* pArray
)
{
    XMStoreIcoN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMICON4& _XMICON4::operator=
(
    CONST _XMICON4& IcoN4
)
{
    v = IcoN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMICON4& _XMICON4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMICO4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMICO4::_XMICO4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreIco4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMICO4::_XMICO4
(
    CONST FLOAT* pArray
)
{
    XMStoreIco4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMICO4& _XMICO4::operator=
(
    CONST _XMICO4& Ico4
)
{
    v = Ico4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMICO4& _XMICO4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUICON4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUICON4::_XMUICON4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUIcoN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICON4::_XMUICON4
(
    CONST FLOAT* pArray
)
{
    XMStoreUIcoN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICON4& _XMUICON4::operator=
(
    CONST _XMUICON4& UIcoN4
)
{
    v = UIcoN4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICON4& _XMUICON4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMUICO4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUICO4::_XMUICO4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUIco4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICO4::_XMUICO4
(
    CONST FLOAT* pArray
)
{
    XMStoreUIco4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICO4& _XMUICO4::operator=
(
    CONST _XMUICO4& UIco4
)
{
    v = UIco4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUICO4& _XMUICO4::operator=
(
    CONST UINT64 Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMCOLOR4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMCOLOR::_XMCOLOR
(
    FLOAT _r,
    FLOAT _g,
    FLOAT _b,
    FLOAT _a
)
{
    XMStoreColor(this, XMVectorSet(_r, _g, _b, _a));
}

//------------------------------------------------------------------------------

XMFINLINE _XMCOLOR::_XMCOLOR
(
    CONST FLOAT* pArray
)
{
    XMStoreColor(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMCOLOR& _XMCOLOR::operator=
(
    CONST _XMCOLOR& Color
)
{
    c = Color.c;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMCOLOR& _XMCOLOR::operator=
(
    CONST UINT Color
)
{
    c = Color;
    return *this;
}

/****************************************************************************
 *
 * XMBYTEN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN4::_XMBYTEN4
(
    CONST CHAR* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN4::_XMBYTEN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreByteN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN4::_XMBYTEN4
(
    CONST FLOAT* pArray
)
{
    XMStoreByteN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTEN4& _XMBYTEN4::operator=
(
    CONST _XMBYTEN4& ByteN4
)
{
    x = ByteN4.x;
    y = ByteN4.y;
    z = ByteN4.z;
    w = ByteN4.w;
    return *this;
}

/****************************************************************************
 *
 * XMBYTE4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE4::_XMBYTE4
(
    CONST CHAR* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE4::_XMBYTE4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreByte4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE4::_XMBYTE4
(
    CONST FLOAT* pArray
)
{
    XMStoreByte4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMBYTE4& _XMBYTE4::operator=
(
    CONST _XMBYTE4& Byte4
)
{
    x = Byte4.x;
    y = Byte4.y;
    z = Byte4.z;
    w = Byte4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUBYTEN4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN4::_XMUBYTEN4
(
    CONST BYTE* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN4::_XMUBYTEN4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUByteN4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN4::_XMUBYTEN4
(
    CONST FLOAT* pArray
)
{
    XMStoreUByteN4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTEN4& _XMUBYTEN4::operator=
(
    CONST _XMUBYTEN4& UByteN4
)
{
    x = UByteN4.x;
    y = UByteN4.y;
    z = UByteN4.z;
    w = UByteN4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUBYTE4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE4::_XMUBYTE4
(
    CONST BYTE* pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE4::_XMUBYTE4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUByte4(this, XMVectorSet(_x, _y, _z, _w));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE4::_XMUBYTE4
(
    CONST FLOAT* pArray
)
{
    XMStoreUByte4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUBYTE4& _XMUBYTE4::operator=
(
    CONST _XMUBYTE4& UByte4
)
{
    x = UByte4.x;
    y = UByte4.y;
    z = UByte4.z;
    w = UByte4.w;
    return *this;
}

/****************************************************************************
 *
 * XMUNIBBLE4 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMUNIBBLE4::_XMUNIBBLE4
(
    CONST CHAR *pArray
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = pArray[3];
}

//------------------------------------------------------------------------------

XMFINLINE _XMUNIBBLE4::_XMUNIBBLE4
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    FLOAT _w
)
{
    XMStoreUNibble4(this, XMVectorSet( _x, _y, _z, _w ));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUNIBBLE4::_XMUNIBBLE4
(
    CONST FLOAT *pArray
)
{
    XMStoreUNibble4(this, XMLoadFloat4((const XMFLOAT4*)pArray));
}

//------------------------------------------------------------------------------

XMFINLINE _XMUNIBBLE4& _XMUNIBBLE4::operator=
(
    CONST _XMUNIBBLE4& UNibble4
)
{
    v = UNibble4.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMUNIBBLE4& _XMUNIBBLE4::operator=
(
    CONST USHORT Packed
)
{
    v = Packed;
    return *this;
}

/****************************************************************************
 *
 * XMU555 operators
 *
 ****************************************************************************/

//------------------------------------------------------------------------------

XMFINLINE _XMU555::_XMU555
(
    CONST CHAR *pArray,
    BOOL _w
)
{
    x = pArray[0];
    y = pArray[1];
    z = pArray[2];
    w = _w;
}

//------------------------------------------------------------------------------

XMFINLINE _XMU555::_XMU555
(
    FLOAT _x,
    FLOAT _y,
    FLOAT _z,
    BOOL _w
)
{
    XMStoreU555(this, XMVectorSet(_x, _y, _z, ((_w) ? 1.0f : 0.0f) ));
}

//------------------------------------------------------------------------------

XMFINLINE _XMU555::_XMU555
(
    CONST FLOAT *pArray,
    BOOL _w
)
{
    XMVECTOR V = XMLoadFloat3((const XMFLOAT3*)pArray);
    XMStoreU555(this, XMVectorSetW(V, ((_w) ? 1.0f : 0.0f) ));
}

//------------------------------------------------------------------------------

XMFINLINE _XMU555& _XMU555::operator=
(
    CONST _XMU555& U555
)
{
    v = U555.v;
    return *this;
}

//------------------------------------------------------------------------------

XMFINLINE _XMU555& _XMU555::operator=
(
    CONST USHORT Packed
)
{
    v = Packed;
    return *this;
}

#endif // __cplusplus

#if defined(_XM_NO_INTRINSICS_)
#undef XMISNAN
#undef XMISINF
#endif

#endif // __XNAMATHVECTOR_INL__

